#pragma once

////////////////////////////////////////////////////////////////////////////////
// The MIT License (MIT)
//
// Copyright (c) 2019 Nicholas Frechette & Animation Compression Library contributors
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
////////////////////////////////////////////////////////////////////////////////

#include "acl/version.h"
#include "acl/core/impl/compiler_utils.h"

#include <cstdint>

ACL_IMPL_FILE_PRAGMA_PUSH

// This file is generated with the python script located at: ./tools/calc_local_bit_rates.py

namespace acl
{
	ACL_IMPL_VERSION_NAMESPACE_BEGIN

	namespace acl_impl
	{
		constexpr uint8_t k_local_bit_rate_permutations_no_scale[625][2] =
		{
			{ 0, 0 },		// 0 bits per transform
			{ 0, 1 },		// 3 bits per transform
			{ 1, 0 },		// 3 bits per transform
			{ 0, 2 },		// 6 bits per transform
			{ 1, 1 },		// 6 bits per transform
			{ 2, 0 },		// 6 bits per transform
			{ 0, 3 },		// 9 bits per transform
			{ 1, 2 },		// 9 bits per transform
			{ 2, 1 },		// 9 bits per transform
			{ 3, 0 },		// 9 bits per transform
			{ 0, 4 },		// 12 bits per transform
			{ 1, 3 },		// 12 bits per transform
			{ 2, 2 },		// 12 bits per transform
			{ 3, 1 },		// 12 bits per transform
			{ 4, 0 },		// 12 bits per transform
			{ 0, 5 },		// 15 bits per transform
			{ 1, 4 },		// 15 bits per transform
			{ 2, 3 },		// 15 bits per transform
			{ 3, 2 },		// 15 bits per transform
			{ 4, 1 },		// 15 bits per transform
			{ 5, 0 },		// 15 bits per transform
			{ 0, 6 },		// 18 bits per transform
			{ 1, 5 },		// 18 bits per transform
			{ 2, 4 },		// 18 bits per transform
			{ 3, 3 },		// 18 bits per transform
			{ 4, 2 },		// 18 bits per transform
			{ 5, 1 },		// 18 bits per transform
			{ 6, 0 },		// 18 bits per transform
			{ 0, 7 },		// 21 bits per transform
			{ 1, 6 },		// 21 bits per transform
			{ 2, 5 },		// 21 bits per transform
			{ 3, 4 },		// 21 bits per transform
			{ 4, 3 },		// 21 bits per transform
			{ 5, 2 },		// 21 bits per transform
			{ 6, 1 },		// 21 bits per transform
			{ 7, 0 },		// 21 bits per transform
			{ 0, 8 },		// 24 bits per transform
			{ 1, 7 },		// 24 bits per transform
			{ 2, 6 },		// 24 bits per transform
			{ 3, 5 },		// 24 bits per transform
			{ 4, 4 },		// 24 bits per transform
			{ 5, 3 },		// 24 bits per transform
			{ 6, 2 },		// 24 bits per transform
			{ 7, 1 },		// 24 bits per transform
			{ 8, 0 },		// 24 bits per transform
			{ 0, 9 },		// 27 bits per transform
			{ 1, 8 },		// 27 bits per transform
			{ 2, 7 },		// 27 bits per transform
			{ 3, 6 },		// 27 bits per transform
			{ 4, 5 },		// 27 bits per transform
			{ 5, 4 },		// 27 bits per transform
			{ 6, 3 },		// 27 bits per transform
			{ 7, 2 },		// 27 bits per transform
			{ 8, 1 },		// 27 bits per transform
			{ 9, 0 },		// 27 bits per transform
			{ 0, 10 },		// 30 bits per transform
			{ 1, 9 },		// 30 bits per transform
			{ 2, 8 },		// 30 bits per transform
			{ 3, 7 },		// 30 bits per transform
			{ 4, 6 },		// 30 bits per transform
			{ 5, 5 },		// 30 bits per transform
			{ 6, 4 },		// 30 bits per transform
			{ 7, 3 },		// 30 bits per transform
			{ 8, 2 },		// 30 bits per transform
			{ 9, 1 },		// 30 bits per transform
			{ 10, 0 },		// 30 bits per transform
			{ 0, 11 },		// 33 bits per transform
			{ 1, 10 },		// 33 bits per transform
			{ 2, 9 },		// 33 bits per transform
			{ 3, 8 },		// 33 bits per transform
			{ 4, 7 },		// 33 bits per transform
			{ 5, 6 },		// 33 bits per transform
			{ 6, 5 },		// 33 bits per transform
			{ 7, 4 },		// 33 bits per transform
			{ 8, 3 },		// 33 bits per transform
			{ 9, 2 },		// 33 bits per transform
			{ 10, 1 },		// 33 bits per transform
			{ 11, 0 },		// 33 bits per transform
			{ 0, 12 },		// 36 bits per transform
			{ 1, 11 },		// 36 bits per transform
			{ 2, 10 },		// 36 bits per transform
			{ 3, 9 },		// 36 bits per transform
			{ 4, 8 },		// 36 bits per transform
			{ 5, 7 },		// 36 bits per transform
			{ 6, 6 },		// 36 bits per transform
			{ 7, 5 },		// 36 bits per transform
			{ 8, 4 },		// 36 bits per transform
			{ 9, 3 },		// 36 bits per transform
			{ 10, 2 },		// 36 bits per transform
			{ 11, 1 },		// 36 bits per transform
			{ 12, 0 },		// 36 bits per transform
			{ 0, 13 },		// 39 bits per transform
			{ 1, 12 },		// 39 bits per transform
			{ 2, 11 },		// 39 bits per transform
			{ 3, 10 },		// 39 bits per transform
			{ 4, 9 },		// 39 bits per transform
			{ 5, 8 },		// 39 bits per transform
			{ 6, 7 },		// 39 bits per transform
			{ 7, 6 },		// 39 bits per transform
			{ 8, 5 },		// 39 bits per transform
			{ 9, 4 },		// 39 bits per transform
			{ 10, 3 },		// 39 bits per transform
			{ 11, 2 },		// 39 bits per transform
			{ 12, 1 },		// 39 bits per transform
			{ 13, 0 },		// 39 bits per transform
			{ 0, 14 },		// 42 bits per transform
			{ 1, 13 },		// 42 bits per transform
			{ 2, 12 },		// 42 bits per transform
			{ 3, 11 },		// 42 bits per transform
			{ 4, 10 },		// 42 bits per transform
			{ 5, 9 },		// 42 bits per transform
			{ 6, 8 },		// 42 bits per transform
			{ 7, 7 },		// 42 bits per transform
			{ 8, 6 },		// 42 bits per transform
			{ 9, 5 },		// 42 bits per transform
			{ 10, 4 },		// 42 bits per transform
			{ 11, 3 },		// 42 bits per transform
			{ 12, 2 },		// 42 bits per transform
			{ 13, 1 },		// 42 bits per transform
			{ 14, 0 },		// 42 bits per transform
			{ 0, 15 },		// 45 bits per transform
			{ 1, 14 },		// 45 bits per transform
			{ 2, 13 },		// 45 bits per transform
			{ 3, 12 },		// 45 bits per transform
			{ 4, 11 },		// 45 bits per transform
			{ 5, 10 },		// 45 bits per transform
			{ 6, 9 },		// 45 bits per transform
			{ 7, 8 },		// 45 bits per transform
			{ 8, 7 },		// 45 bits per transform
			{ 9, 6 },		// 45 bits per transform
			{ 10, 5 },		// 45 bits per transform
			{ 11, 4 },		// 45 bits per transform
			{ 12, 3 },		// 45 bits per transform
			{ 13, 2 },		// 45 bits per transform
			{ 14, 1 },		// 45 bits per transform
			{ 15, 0 },		// 45 bits per transform
			{ 0, 16 },		// 48 bits per transform
			{ 1, 15 },		// 48 bits per transform
			{ 2, 14 },		// 48 bits per transform
			{ 3, 13 },		// 48 bits per transform
			{ 4, 12 },		// 48 bits per transform
			{ 5, 11 },		// 48 bits per transform
			{ 6, 10 },		// 48 bits per transform
			{ 7, 9 },		// 48 bits per transform
			{ 8, 8 },		// 48 bits per transform
			{ 9, 7 },		// 48 bits per transform
			{ 10, 6 },		// 48 bits per transform
			{ 11, 5 },		// 48 bits per transform
			{ 12, 4 },		// 48 bits per transform
			{ 13, 3 },		// 48 bits per transform
			{ 14, 2 },		// 48 bits per transform
			{ 15, 1 },		// 48 bits per transform
			{ 16, 0 },		// 48 bits per transform
			{ 0, 17 },		// 51 bits per transform
			{ 1, 16 },		// 51 bits per transform
			{ 2, 15 },		// 51 bits per transform
			{ 3, 14 },		// 51 bits per transform
			{ 4, 13 },		// 51 bits per transform
			{ 5, 12 },		// 51 bits per transform
			{ 6, 11 },		// 51 bits per transform
			{ 7, 10 },		// 51 bits per transform
			{ 8, 9 },		// 51 bits per transform
			{ 9, 8 },		// 51 bits per transform
			{ 10, 7 },		// 51 bits per transform
			{ 11, 6 },		// 51 bits per transform
			{ 12, 5 },		// 51 bits per transform
			{ 13, 4 },		// 51 bits per transform
			{ 14, 3 },		// 51 bits per transform
			{ 15, 2 },		// 51 bits per transform
			{ 16, 1 },		// 51 bits per transform
			{ 17, 0 },		// 51 bits per transform
			{ 0, 18 },		// 54 bits per transform
			{ 1, 17 },		// 54 bits per transform
			{ 2, 16 },		// 54 bits per transform
			{ 3, 15 },		// 54 bits per transform
			{ 4, 14 },		// 54 bits per transform
			{ 5, 13 },		// 54 bits per transform
			{ 6, 12 },		// 54 bits per transform
			{ 7, 11 },		// 54 bits per transform
			{ 8, 10 },		// 54 bits per transform
			{ 9, 9 },		// 54 bits per transform
			{ 10, 8 },		// 54 bits per transform
			{ 11, 7 },		// 54 bits per transform
			{ 12, 6 },		// 54 bits per transform
			{ 13, 5 },		// 54 bits per transform
			{ 14, 4 },		// 54 bits per transform
			{ 15, 3 },		// 54 bits per transform
			{ 16, 2 },		// 54 bits per transform
			{ 17, 1 },		// 54 bits per transform
			{ 18, 0 },		// 54 bits per transform
			{ 0, 19 },		// 57 bits per transform
			{ 1, 18 },		// 57 bits per transform
			{ 2, 17 },		// 57 bits per transform
			{ 3, 16 },		// 57 bits per transform
			{ 4, 15 },		// 57 bits per transform
			{ 5, 14 },		// 57 bits per transform
			{ 6, 13 },		// 57 bits per transform
			{ 7, 12 },		// 57 bits per transform
			{ 8, 11 },		// 57 bits per transform
			{ 9, 10 },		// 57 bits per transform
			{ 10, 9 },		// 57 bits per transform
			{ 11, 8 },		// 57 bits per transform
			{ 12, 7 },		// 57 bits per transform
			{ 13, 6 },		// 57 bits per transform
			{ 14, 5 },		// 57 bits per transform
			{ 15, 4 },		// 57 bits per transform
			{ 16, 3 },		// 57 bits per transform
			{ 17, 2 },		// 57 bits per transform
			{ 18, 1 },		// 57 bits per transform
			{ 19, 0 },		// 57 bits per transform
			{ 0, 20 },		// 60 bits per transform
			{ 1, 19 },		// 60 bits per transform
			{ 2, 18 },		// 60 bits per transform
			{ 3, 17 },		// 60 bits per transform
			{ 4, 16 },		// 60 bits per transform
			{ 5, 15 },		// 60 bits per transform
			{ 6, 14 },		// 60 bits per transform
			{ 7, 13 },		// 60 bits per transform
			{ 8, 12 },		// 60 bits per transform
			{ 9, 11 },		// 60 bits per transform
			{ 10, 10 },		// 60 bits per transform
			{ 11, 9 },		// 60 bits per transform
			{ 12, 8 },		// 60 bits per transform
			{ 13, 7 },		// 60 bits per transform
			{ 14, 6 },		// 60 bits per transform
			{ 15, 5 },		// 60 bits per transform
			{ 16, 4 },		// 60 bits per transform
			{ 17, 3 },		// 60 bits per transform
			{ 18, 2 },		// 60 bits per transform
			{ 19, 1 },		// 60 bits per transform
			{ 20, 0 },		// 60 bits per transform
			{ 0, 21 },		// 63 bits per transform
			{ 1, 20 },		// 63 bits per transform
			{ 2, 19 },		// 63 bits per transform
			{ 3, 18 },		// 63 bits per transform
			{ 4, 17 },		// 63 bits per transform
			{ 5, 16 },		// 63 bits per transform
			{ 6, 15 },		// 63 bits per transform
			{ 7, 14 },		// 63 bits per transform
			{ 8, 13 },		// 63 bits per transform
			{ 9, 12 },		// 63 bits per transform
			{ 10, 11 },		// 63 bits per transform
			{ 11, 10 },		// 63 bits per transform
			{ 12, 9 },		// 63 bits per transform
			{ 13, 8 },		// 63 bits per transform
			{ 14, 7 },		// 63 bits per transform
			{ 15, 6 },		// 63 bits per transform
			{ 16, 5 },		// 63 bits per transform
			{ 17, 4 },		// 63 bits per transform
			{ 18, 3 },		// 63 bits per transform
			{ 19, 2 },		// 63 bits per transform
			{ 20, 1 },		// 63 bits per transform
			{ 21, 0 },		// 63 bits per transform
			{ 0, 22 },		// 66 bits per transform
			{ 1, 21 },		// 66 bits per transform
			{ 2, 20 },		// 66 bits per transform
			{ 3, 19 },		// 66 bits per transform
			{ 4, 18 },		// 66 bits per transform
			{ 5, 17 },		// 66 bits per transform
			{ 6, 16 },		// 66 bits per transform
			{ 7, 15 },		// 66 bits per transform
			{ 8, 14 },		// 66 bits per transform
			{ 9, 13 },		// 66 bits per transform
			{ 10, 12 },		// 66 bits per transform
			{ 11, 11 },		// 66 bits per transform
			{ 12, 10 },		// 66 bits per transform
			{ 13, 9 },		// 66 bits per transform
			{ 14, 8 },		// 66 bits per transform
			{ 15, 7 },		// 66 bits per transform
			{ 16, 6 },		// 66 bits per transform
			{ 17, 5 },		// 66 bits per transform
			{ 18, 4 },		// 66 bits per transform
			{ 19, 3 },		// 66 bits per transform
			{ 20, 2 },		// 66 bits per transform
			{ 21, 1 },		// 66 bits per transform
			{ 22, 0 },		// 66 bits per transform
			{ 0, 23 },		// 69 bits per transform
			{ 1, 22 },		// 69 bits per transform
			{ 2, 21 },		// 69 bits per transform
			{ 3, 20 },		// 69 bits per transform
			{ 4, 19 },		// 69 bits per transform
			{ 5, 18 },		// 69 bits per transform
			{ 6, 17 },		// 69 bits per transform
			{ 7, 16 },		// 69 bits per transform
			{ 8, 15 },		// 69 bits per transform
			{ 9, 14 },		// 69 bits per transform
			{ 10, 13 },		// 69 bits per transform
			{ 11, 12 },		// 69 bits per transform
			{ 12, 11 },		// 69 bits per transform
			{ 13, 10 },		// 69 bits per transform
			{ 14, 9 },		// 69 bits per transform
			{ 15, 8 },		// 69 bits per transform
			{ 16, 7 },		// 69 bits per transform
			{ 17, 6 },		// 69 bits per transform
			{ 18, 5 },		// 69 bits per transform
			{ 19, 4 },		// 69 bits per transform
			{ 20, 3 },		// 69 bits per transform
			{ 21, 2 },		// 69 bits per transform
			{ 22, 1 },		// 69 bits per transform
			{ 23, 0 },		// 69 bits per transform
			{ 1, 23 },		// 72 bits per transform
			{ 2, 22 },		// 72 bits per transform
			{ 3, 21 },		// 72 bits per transform
			{ 4, 20 },		// 72 bits per transform
			{ 5, 19 },		// 72 bits per transform
			{ 6, 18 },		// 72 bits per transform
			{ 7, 17 },		// 72 bits per transform
			{ 8, 16 },		// 72 bits per transform
			{ 9, 15 },		// 72 bits per transform
			{ 10, 14 },		// 72 bits per transform
			{ 11, 13 },		// 72 bits per transform
			{ 12, 12 },		// 72 bits per transform
			{ 13, 11 },		// 72 bits per transform
			{ 14, 10 },		// 72 bits per transform
			{ 15, 9 },		// 72 bits per transform
			{ 16, 8 },		// 72 bits per transform
			{ 17, 7 },		// 72 bits per transform
			{ 18, 6 },		// 72 bits per transform
			{ 19, 5 },		// 72 bits per transform
			{ 20, 4 },		// 72 bits per transform
			{ 21, 3 },		// 72 bits per transform
			{ 22, 2 },		// 72 bits per transform
			{ 23, 1 },		// 72 bits per transform
			{ 2, 23 },		// 75 bits per transform
			{ 3, 22 },		// 75 bits per transform
			{ 4, 21 },		// 75 bits per transform
			{ 5, 20 },		// 75 bits per transform
			{ 6, 19 },		// 75 bits per transform
			{ 7, 18 },		// 75 bits per transform
			{ 8, 17 },		// 75 bits per transform
			{ 9, 16 },		// 75 bits per transform
			{ 10, 15 },		// 75 bits per transform
			{ 11, 14 },		// 75 bits per transform
			{ 12, 13 },		// 75 bits per transform
			{ 13, 12 },		// 75 bits per transform
			{ 14, 11 },		// 75 bits per transform
			{ 15, 10 },		// 75 bits per transform
			{ 16, 9 },		// 75 bits per transform
			{ 17, 8 },		// 75 bits per transform
			{ 18, 7 },		// 75 bits per transform
			{ 19, 6 },		// 75 bits per transform
			{ 20, 5 },		// 75 bits per transform
			{ 21, 4 },		// 75 bits per transform
			{ 22, 3 },		// 75 bits per transform
			{ 23, 2 },		// 75 bits per transform
			{ 3, 23 },		// 78 bits per transform
			{ 4, 22 },		// 78 bits per transform
			{ 5, 21 },		// 78 bits per transform
			{ 6, 20 },		// 78 bits per transform
			{ 7, 19 },		// 78 bits per transform
			{ 8, 18 },		// 78 bits per transform
			{ 9, 17 },		// 78 bits per transform
			{ 10, 16 },		// 78 bits per transform
			{ 11, 15 },		// 78 bits per transform
			{ 12, 14 },		// 78 bits per transform
			{ 13, 13 },		// 78 bits per transform
			{ 14, 12 },		// 78 bits per transform
			{ 15, 11 },		// 78 bits per transform
			{ 16, 10 },		// 78 bits per transform
			{ 17, 9 },		// 78 bits per transform
			{ 18, 8 },		// 78 bits per transform
			{ 19, 7 },		// 78 bits per transform
			{ 20, 6 },		// 78 bits per transform
			{ 21, 5 },		// 78 bits per transform
			{ 22, 4 },		// 78 bits per transform
			{ 23, 3 },		// 78 bits per transform
			{ 4, 23 },		// 81 bits per transform
			{ 5, 22 },		// 81 bits per transform
			{ 6, 21 },		// 81 bits per transform
			{ 7, 20 },		// 81 bits per transform
			{ 8, 19 },		// 81 bits per transform
			{ 9, 18 },		// 81 bits per transform
			{ 10, 17 },		// 81 bits per transform
			{ 11, 16 },		// 81 bits per transform
			{ 12, 15 },		// 81 bits per transform
			{ 13, 14 },		// 81 bits per transform
			{ 14, 13 },		// 81 bits per transform
			{ 15, 12 },		// 81 bits per transform
			{ 16, 11 },		// 81 bits per transform
			{ 17, 10 },		// 81 bits per transform
			{ 18, 9 },		// 81 bits per transform
			{ 19, 8 },		// 81 bits per transform
			{ 20, 7 },		// 81 bits per transform
			{ 21, 6 },		// 81 bits per transform
			{ 22, 5 },		// 81 bits per transform
			{ 23, 4 },		// 81 bits per transform
			{ 5, 23 },		// 84 bits per transform
			{ 6, 22 },		// 84 bits per transform
			{ 7, 21 },		// 84 bits per transform
			{ 8, 20 },		// 84 bits per transform
			{ 9, 19 },		// 84 bits per transform
			{ 10, 18 },		// 84 bits per transform
			{ 11, 17 },		// 84 bits per transform
			{ 12, 16 },		// 84 bits per transform
			{ 13, 15 },		// 84 bits per transform
			{ 14, 14 },		// 84 bits per transform
			{ 15, 13 },		// 84 bits per transform
			{ 16, 12 },		// 84 bits per transform
			{ 17, 11 },		// 84 bits per transform
			{ 18, 10 },		// 84 bits per transform
			{ 19, 9 },		// 84 bits per transform
			{ 20, 8 },		// 84 bits per transform
			{ 21, 7 },		// 84 bits per transform
			{ 22, 6 },		// 84 bits per transform
			{ 23, 5 },		// 84 bits per transform
			{ 6, 23 },		// 87 bits per transform
			{ 7, 22 },		// 87 bits per transform
			{ 8, 21 },		// 87 bits per transform
			{ 9, 20 },		// 87 bits per transform
			{ 10, 19 },		// 87 bits per transform
			{ 11, 18 },		// 87 bits per transform
			{ 12, 17 },		// 87 bits per transform
			{ 13, 16 },		// 87 bits per transform
			{ 14, 15 },		// 87 bits per transform
			{ 15, 14 },		// 87 bits per transform
			{ 16, 13 },		// 87 bits per transform
			{ 17, 12 },		// 87 bits per transform
			{ 18, 11 },		// 87 bits per transform
			{ 19, 10 },		// 87 bits per transform
			{ 20, 9 },		// 87 bits per transform
			{ 21, 8 },		// 87 bits per transform
			{ 22, 7 },		// 87 bits per transform
			{ 23, 6 },		// 87 bits per transform
			{ 7, 23 },		// 90 bits per transform
			{ 8, 22 },		// 90 bits per transform
			{ 9, 21 },		// 90 bits per transform
			{ 10, 20 },		// 90 bits per transform
			{ 11, 19 },		// 90 bits per transform
			{ 12, 18 },		// 90 bits per transform
			{ 13, 17 },		// 90 bits per transform
			{ 14, 16 },		// 90 bits per transform
			{ 15, 15 },		// 90 bits per transform
			{ 16, 14 },		// 90 bits per transform
			{ 17, 13 },		// 90 bits per transform
			{ 18, 12 },		// 90 bits per transform
			{ 19, 11 },		// 90 bits per transform
			{ 20, 10 },		// 90 bits per transform
			{ 21, 9 },		// 90 bits per transform
			{ 22, 8 },		// 90 bits per transform
			{ 23, 7 },		// 90 bits per transform
			{ 8, 23 },		// 93 bits per transform
			{ 9, 22 },		// 93 bits per transform
			{ 10, 21 },		// 93 bits per transform
			{ 11, 20 },		// 93 bits per transform
			{ 12, 19 },		// 93 bits per transform
			{ 13, 18 },		// 93 bits per transform
			{ 14, 17 },		// 93 bits per transform
			{ 15, 16 },		// 93 bits per transform
			{ 16, 15 },		// 93 bits per transform
			{ 17, 14 },		// 93 bits per transform
			{ 18, 13 },		// 93 bits per transform
			{ 19, 12 },		// 93 bits per transform
			{ 20, 11 },		// 93 bits per transform
			{ 21, 10 },		// 93 bits per transform
			{ 22, 9 },		// 93 bits per transform
			{ 23, 8 },		// 93 bits per transform
			{ 0, 24 },		// 96 bits per transform
			{ 9, 23 },		// 96 bits per transform
			{ 10, 22 },		// 96 bits per transform
			{ 11, 21 },		// 96 bits per transform
			{ 12, 20 },		// 96 bits per transform
			{ 13, 19 },		// 96 bits per transform
			{ 14, 18 },		// 96 bits per transform
			{ 15, 17 },		// 96 bits per transform
			{ 16, 16 },		// 96 bits per transform
			{ 17, 15 },		// 96 bits per transform
			{ 18, 14 },		// 96 bits per transform
			{ 19, 13 },		// 96 bits per transform
			{ 20, 12 },		// 96 bits per transform
			{ 21, 11 },		// 96 bits per transform
			{ 22, 10 },		// 96 bits per transform
			{ 23, 9 },		// 96 bits per transform
			{ 24, 0 },		// 96 bits per transform
			{ 1, 24 },		// 99 bits per transform
			{ 10, 23 },		// 99 bits per transform
			{ 11, 22 },		// 99 bits per transform
			{ 12, 21 },		// 99 bits per transform
			{ 13, 20 },		// 99 bits per transform
			{ 14, 19 },		// 99 bits per transform
			{ 15, 18 },		// 99 bits per transform
			{ 16, 17 },		// 99 bits per transform
			{ 17, 16 },		// 99 bits per transform
			{ 18, 15 },		// 99 bits per transform
			{ 19, 14 },		// 99 bits per transform
			{ 20, 13 },		// 99 bits per transform
			{ 21, 12 },		// 99 bits per transform
			{ 22, 11 },		// 99 bits per transform
			{ 23, 10 },		// 99 bits per transform
			{ 24, 1 },		// 99 bits per transform
			{ 2, 24 },		// 102 bits per transform
			{ 11, 23 },		// 102 bits per transform
			{ 12, 22 },		// 102 bits per transform
			{ 13, 21 },		// 102 bits per transform
			{ 14, 20 },		// 102 bits per transform
			{ 15, 19 },		// 102 bits per transform
			{ 16, 18 },		// 102 bits per transform
			{ 17, 17 },		// 102 bits per transform
			{ 18, 16 },		// 102 bits per transform
			{ 19, 15 },		// 102 bits per transform
			{ 20, 14 },		// 102 bits per transform
			{ 21, 13 },		// 102 bits per transform
			{ 22, 12 },		// 102 bits per transform
			{ 23, 11 },		// 102 bits per transform
			{ 24, 2 },		// 102 bits per transform
			{ 3, 24 },		// 105 bits per transform
			{ 12, 23 },		// 105 bits per transform
			{ 13, 22 },		// 105 bits per transform
			{ 14, 21 },		// 105 bits per transform
			{ 15, 20 },		// 105 bits per transform
			{ 16, 19 },		// 105 bits per transform
			{ 17, 18 },		// 105 bits per transform
			{ 18, 17 },		// 105 bits per transform
			{ 19, 16 },		// 105 bits per transform
			{ 20, 15 },		// 105 bits per transform
			{ 21, 14 },		// 105 bits per transform
			{ 22, 13 },		// 105 bits per transform
			{ 23, 12 },		// 105 bits per transform
			{ 24, 3 },		// 105 bits per transform
			{ 4, 24 },		// 108 bits per transform
			{ 13, 23 },		// 108 bits per transform
			{ 14, 22 },		// 108 bits per transform
			{ 15, 21 },		// 108 bits per transform
			{ 16, 20 },		// 108 bits per transform
			{ 17, 19 },		// 108 bits per transform
			{ 18, 18 },		// 108 bits per transform
			{ 19, 17 },		// 108 bits per transform
			{ 20, 16 },		// 108 bits per transform
			{ 21, 15 },		// 108 bits per transform
			{ 22, 14 },		// 108 bits per transform
			{ 23, 13 },		// 108 bits per transform
			{ 24, 4 },		// 108 bits per transform
			{ 5, 24 },		// 111 bits per transform
			{ 14, 23 },		// 111 bits per transform
			{ 15, 22 },		// 111 bits per transform
			{ 16, 21 },		// 111 bits per transform
			{ 17, 20 },		// 111 bits per transform
			{ 18, 19 },		// 111 bits per transform
			{ 19, 18 },		// 111 bits per transform
			{ 20, 17 },		// 111 bits per transform
			{ 21, 16 },		// 111 bits per transform
			{ 22, 15 },		// 111 bits per transform
			{ 23, 14 },		// 111 bits per transform
			{ 24, 5 },		// 111 bits per transform
			{ 6, 24 },		// 114 bits per transform
			{ 15, 23 },		// 114 bits per transform
			{ 16, 22 },		// 114 bits per transform
			{ 17, 21 },		// 114 bits per transform
			{ 18, 20 },		// 114 bits per transform
			{ 19, 19 },		// 114 bits per transform
			{ 20, 18 },		// 114 bits per transform
			{ 21, 17 },		// 114 bits per transform
			{ 22, 16 },		// 114 bits per transform
			{ 23, 15 },		// 114 bits per transform
			{ 24, 6 },		// 114 bits per transform
			{ 7, 24 },		// 117 bits per transform
			{ 16, 23 },		// 117 bits per transform
			{ 17, 22 },		// 117 bits per transform
			{ 18, 21 },		// 117 bits per transform
			{ 19, 20 },		// 117 bits per transform
			{ 20, 19 },		// 117 bits per transform
			{ 21, 18 },		// 117 bits per transform
			{ 22, 17 },		// 117 bits per transform
			{ 23, 16 },		// 117 bits per transform
			{ 24, 7 },		// 117 bits per transform
			{ 8, 24 },		// 120 bits per transform
			{ 17, 23 },		// 120 bits per transform
			{ 18, 22 },		// 120 bits per transform
			{ 19, 21 },		// 120 bits per transform
			{ 20, 20 },		// 120 bits per transform
			{ 21, 19 },		// 120 bits per transform
			{ 22, 18 },		// 120 bits per transform
			{ 23, 17 },		// 120 bits per transform
			{ 24, 8 },		// 120 bits per transform
			{ 9, 24 },		// 123 bits per transform
			{ 18, 23 },		// 123 bits per transform
			{ 19, 22 },		// 123 bits per transform
			{ 20, 21 },		// 123 bits per transform
			{ 21, 20 },		// 123 bits per transform
			{ 22, 19 },		// 123 bits per transform
			{ 23, 18 },		// 123 bits per transform
			{ 24, 9 },		// 123 bits per transform
			{ 10, 24 },		// 126 bits per transform
			{ 19, 23 },		// 126 bits per transform
			{ 20, 22 },		// 126 bits per transform
			{ 21, 21 },		// 126 bits per transform
			{ 22, 20 },		// 126 bits per transform
			{ 23, 19 },		// 126 bits per transform
			{ 24, 10 },		// 126 bits per transform
			{ 11, 24 },		// 129 bits per transform
			{ 20, 23 },		// 129 bits per transform
			{ 21, 22 },		// 129 bits per transform
			{ 22, 21 },		// 129 bits per transform
			{ 23, 20 },		// 129 bits per transform
			{ 24, 11 },		// 129 bits per transform
			{ 12, 24 },		// 132 bits per transform
			{ 21, 23 },		// 132 bits per transform
			{ 22, 22 },		// 132 bits per transform
			{ 23, 21 },		// 132 bits per transform
			{ 24, 12 },		// 132 bits per transform
			{ 13, 24 },		// 135 bits per transform
			{ 22, 23 },		// 135 bits per transform
			{ 23, 22 },		// 135 bits per transform
			{ 24, 13 },		// 135 bits per transform
			{ 14, 24 },		// 138 bits per transform
			{ 23, 23 },		// 138 bits per transform
			{ 24, 14 },		// 138 bits per transform
			{ 15, 24 },		// 141 bits per transform
			{ 24, 15 },		// 141 bits per transform
			{ 16, 24 },		// 144 bits per transform
			{ 24, 16 },		// 144 bits per transform
			{ 17, 24 },		// 147 bits per transform
			{ 24, 17 },		// 147 bits per transform
			{ 18, 24 },		// 150 bits per transform
			{ 24, 18 },		// 150 bits per transform
			{ 19, 24 },		// 153 bits per transform
			{ 24, 19 },		// 153 bits per transform
			{ 20, 24 },		// 156 bits per transform
			{ 24, 20 },		// 156 bits per transform
			{ 21, 24 },		// 159 bits per transform
			{ 24, 21 },		// 159 bits per transform
			{ 22, 24 },		// 162 bits per transform
			{ 24, 22 },		// 162 bits per transform
			{ 23, 24 },		// 165 bits per transform
			{ 24, 23 },		// 165 bits per transform
			{ 24, 24 },		// 192 bits per transform
		};

		constexpr uint8_t k_local_bit_rate_permutations[15625][3] =
		{
			{ 0, 0, 0 },		// 0 bits per transform
			{ 0, 0, 1 },		// 3 bits per transform
			{ 0, 1, 0 },		// 3 bits per transform
			{ 1, 0, 0 },		// 3 bits per transform
			{ 0, 0, 2 },		// 6 bits per transform
			{ 0, 1, 1 },		// 6 bits per transform
			{ 0, 2, 0 },		// 6 bits per transform
			{ 1, 0, 1 },		// 6 bits per transform
			{ 1, 1, 0 },		// 6 bits per transform
			{ 2, 0, 0 },		// 6 bits per transform
			{ 0, 0, 3 },		// 9 bits per transform
			{ 0, 1, 2 },		// 9 bits per transform
			{ 0, 2, 1 },		// 9 bits per transform
			{ 0, 3, 0 },		// 9 bits per transform
			{ 1, 0, 2 },		// 9 bits per transform
			{ 1, 1, 1 },		// 9 bits per transform
			{ 1, 2, 0 },		// 9 bits per transform
			{ 2, 0, 1 },		// 9 bits per transform
			{ 2, 1, 0 },		// 9 bits per transform
			{ 3, 0, 0 },		// 9 bits per transform
			{ 0, 0, 4 },		// 12 bits per transform
			{ 0, 1, 3 },		// 12 bits per transform
			{ 0, 2, 2 },		// 12 bits per transform
			{ 0, 3, 1 },		// 12 bits per transform
			{ 0, 4, 0 },		// 12 bits per transform
			{ 1, 0, 3 },		// 12 bits per transform
			{ 1, 1, 2 },		// 12 bits per transform
			{ 1, 2, 1 },		// 12 bits per transform
			{ 1, 3, 0 },		// 12 bits per transform
			{ 2, 0, 2 },		// 12 bits per transform
			{ 2, 1, 1 },		// 12 bits per transform
			{ 2, 2, 0 },		// 12 bits per transform
			{ 3, 0, 1 },		// 12 bits per transform
			{ 3, 1, 0 },		// 12 bits per transform
			{ 4, 0, 0 },		// 12 bits per transform
			{ 0, 0, 5 },		// 15 bits per transform
			{ 0, 1, 4 },		// 15 bits per transform
			{ 0, 2, 3 },		// 15 bits per transform
			{ 0, 3, 2 },		// 15 bits per transform
			{ 0, 4, 1 },		// 15 bits per transform
			{ 0, 5, 0 },		// 15 bits per transform
			{ 1, 0, 4 },		// 15 bits per transform
			{ 1, 1, 3 },		// 15 bits per transform
			{ 1, 2, 2 },		// 15 bits per transform
			{ 1, 3, 1 },		// 15 bits per transform
			{ 1, 4, 0 },		// 15 bits per transform
			{ 2, 0, 3 },		// 15 bits per transform
			{ 2, 1, 2 },		// 15 bits per transform
			{ 2, 2, 1 },		// 15 bits per transform
			{ 2, 3, 0 },		// 15 bits per transform
			{ 3, 0, 2 },		// 15 bits per transform
			{ 3, 1, 1 },		// 15 bits per transform
			{ 3, 2, 0 },		// 15 bits per transform
			{ 4, 0, 1 },		// 15 bits per transform
			{ 4, 1, 0 },		// 15 bits per transform
			{ 5, 0, 0 },		// 15 bits per transform
			{ 0, 0, 6 },		// 18 bits per transform
			{ 0, 1, 5 },		// 18 bits per transform
			{ 0, 2, 4 },		// 18 bits per transform
			{ 0, 3, 3 },		// 18 bits per transform
			{ 0, 4, 2 },		// 18 bits per transform
			{ 0, 5, 1 },		// 18 bits per transform
			{ 0, 6, 0 },		// 18 bits per transform
			{ 1, 0, 5 },		// 18 bits per transform
			{ 1, 1, 4 },		// 18 bits per transform
			{ 1, 2, 3 },		// 18 bits per transform
			{ 1, 3, 2 },		// 18 bits per transform
			{ 1, 4, 1 },		// 18 bits per transform
			{ 1, 5, 0 },		// 18 bits per transform
			{ 2, 0, 4 },		// 18 bits per transform
			{ 2, 1, 3 },		// 18 bits per transform
			{ 2, 2, 2 },		// 18 bits per transform
			{ 2, 3, 1 },		// 18 bits per transform
			{ 2, 4, 0 },		// 18 bits per transform
			{ 3, 0, 3 },		// 18 bits per transform
			{ 3, 1, 2 },		// 18 bits per transform
			{ 3, 2, 1 },		// 18 bits per transform
			{ 3, 3, 0 },		// 18 bits per transform
			{ 4, 0, 2 },		// 18 bits per transform
			{ 4, 1, 1 },		// 18 bits per transform
			{ 4, 2, 0 },		// 18 bits per transform
			{ 5, 0, 1 },		// 18 bits per transform
			{ 5, 1, 0 },		// 18 bits per transform
			{ 6, 0, 0 },		// 18 bits per transform
			{ 0, 0, 7 },		// 21 bits per transform
			{ 0, 1, 6 },		// 21 bits per transform
			{ 0, 2, 5 },		// 21 bits per transform
			{ 0, 3, 4 },		// 21 bits per transform
			{ 0, 4, 3 },		// 21 bits per transform
			{ 0, 5, 2 },		// 21 bits per transform
			{ 0, 6, 1 },		// 21 bits per transform
			{ 0, 7, 0 },		// 21 bits per transform
			{ 1, 0, 6 },		// 21 bits per transform
			{ 1, 1, 5 },		// 21 bits per transform
			{ 1, 2, 4 },		// 21 bits per transform
			{ 1, 3, 3 },		// 21 bits per transform
			{ 1, 4, 2 },		// 21 bits per transform
			{ 1, 5, 1 },		// 21 bits per transform
			{ 1, 6, 0 },		// 21 bits per transform
			{ 2, 0, 5 },		// 21 bits per transform
			{ 2, 1, 4 },		// 21 bits per transform
			{ 2, 2, 3 },		// 21 bits per transform
			{ 2, 3, 2 },		// 21 bits per transform
			{ 2, 4, 1 },		// 21 bits per transform
			{ 2, 5, 0 },		// 21 bits per transform
			{ 3, 0, 4 },		// 21 bits per transform
			{ 3, 1, 3 },		// 21 bits per transform
			{ 3, 2, 2 },		// 21 bits per transform
			{ 3, 3, 1 },		// 21 bits per transform
			{ 3, 4, 0 },		// 21 bits per transform
			{ 4, 0, 3 },		// 21 bits per transform
			{ 4, 1, 2 },		// 21 bits per transform
			{ 4, 2, 1 },		// 21 bits per transform
			{ 4, 3, 0 },		// 21 bits per transform
			{ 5, 0, 2 },		// 21 bits per transform
			{ 5, 1, 1 },		// 21 bits per transform
			{ 5, 2, 0 },		// 21 bits per transform
			{ 6, 0, 1 },		// 21 bits per transform
			{ 6, 1, 0 },		// 21 bits per transform
			{ 7, 0, 0 },		// 21 bits per transform
			{ 0, 0, 8 },		// 24 bits per transform
			{ 0, 1, 7 },		// 24 bits per transform
			{ 0, 2, 6 },		// 24 bits per transform
			{ 0, 3, 5 },		// 24 bits per transform
			{ 0, 4, 4 },		// 24 bits per transform
			{ 0, 5, 3 },		// 24 bits per transform
			{ 0, 6, 2 },		// 24 bits per transform
			{ 0, 7, 1 },		// 24 bits per transform
			{ 0, 8, 0 },		// 24 bits per transform
			{ 1, 0, 7 },		// 24 bits per transform
			{ 1, 1, 6 },		// 24 bits per transform
			{ 1, 2, 5 },		// 24 bits per transform
			{ 1, 3, 4 },		// 24 bits per transform
			{ 1, 4, 3 },		// 24 bits per transform
			{ 1, 5, 2 },		// 24 bits per transform
			{ 1, 6, 1 },		// 24 bits per transform
			{ 1, 7, 0 },		// 24 bits per transform
			{ 2, 0, 6 },		// 24 bits per transform
			{ 2, 1, 5 },		// 24 bits per transform
			{ 2, 2, 4 },		// 24 bits per transform
			{ 2, 3, 3 },		// 24 bits per transform
			{ 2, 4, 2 },		// 24 bits per transform
			{ 2, 5, 1 },		// 24 bits per transform
			{ 2, 6, 0 },		// 24 bits per transform
			{ 3, 0, 5 },		// 24 bits per transform
			{ 3, 1, 4 },		// 24 bits per transform
			{ 3, 2, 3 },		// 24 bits per transform
			{ 3, 3, 2 },		// 24 bits per transform
			{ 3, 4, 1 },		// 24 bits per transform
			{ 3, 5, 0 },		// 24 bits per transform
			{ 4, 0, 4 },		// 24 bits per transform
			{ 4, 1, 3 },		// 24 bits per transform
			{ 4, 2, 2 },		// 24 bits per transform
			{ 4, 3, 1 },		// 24 bits per transform
			{ 4, 4, 0 },		// 24 bits per transform
			{ 5, 0, 3 },		// 24 bits per transform
			{ 5, 1, 2 },		// 24 bits per transform
			{ 5, 2, 1 },		// 24 bits per transform
			{ 5, 3, 0 },		// 24 bits per transform
			{ 6, 0, 2 },		// 24 bits per transform
			{ 6, 1, 1 },		// 24 bits per transform
			{ 6, 2, 0 },		// 24 bits per transform
			{ 7, 0, 1 },		// 24 bits per transform
			{ 7, 1, 0 },		// 24 bits per transform
			{ 8, 0, 0 },		// 24 bits per transform
			{ 0, 0, 9 },		// 27 bits per transform
			{ 0, 1, 8 },		// 27 bits per transform
			{ 0, 2, 7 },		// 27 bits per transform
			{ 0, 3, 6 },		// 27 bits per transform
			{ 0, 4, 5 },		// 27 bits per transform
			{ 0, 5, 4 },		// 27 bits per transform
			{ 0, 6, 3 },		// 27 bits per transform
			{ 0, 7, 2 },		// 27 bits per transform
			{ 0, 8, 1 },		// 27 bits per transform
			{ 0, 9, 0 },		// 27 bits per transform
			{ 1, 0, 8 },		// 27 bits per transform
			{ 1, 1, 7 },		// 27 bits per transform
			{ 1, 2, 6 },		// 27 bits per transform
			{ 1, 3, 5 },		// 27 bits per transform
			{ 1, 4, 4 },		// 27 bits per transform
			{ 1, 5, 3 },		// 27 bits per transform
			{ 1, 6, 2 },		// 27 bits per transform
			{ 1, 7, 1 },		// 27 bits per transform
			{ 1, 8, 0 },		// 27 bits per transform
			{ 2, 0, 7 },		// 27 bits per transform
			{ 2, 1, 6 },		// 27 bits per transform
			{ 2, 2, 5 },		// 27 bits per transform
			{ 2, 3, 4 },		// 27 bits per transform
			{ 2, 4, 3 },		// 27 bits per transform
			{ 2, 5, 2 },		// 27 bits per transform
			{ 2, 6, 1 },		// 27 bits per transform
			{ 2, 7, 0 },		// 27 bits per transform
			{ 3, 0, 6 },		// 27 bits per transform
			{ 3, 1, 5 },		// 27 bits per transform
			{ 3, 2, 4 },		// 27 bits per transform
			{ 3, 3, 3 },		// 27 bits per transform
			{ 3, 4, 2 },		// 27 bits per transform
			{ 3, 5, 1 },		// 27 bits per transform
			{ 3, 6, 0 },		// 27 bits per transform
			{ 4, 0, 5 },		// 27 bits per transform
			{ 4, 1, 4 },		// 27 bits per transform
			{ 4, 2, 3 },		// 27 bits per transform
			{ 4, 3, 2 },		// 27 bits per transform
			{ 4, 4, 1 },		// 27 bits per transform
			{ 4, 5, 0 },		// 27 bits per transform
			{ 5, 0, 4 },		// 27 bits per transform
			{ 5, 1, 3 },		// 27 bits per transform
			{ 5, 2, 2 },		// 27 bits per transform
			{ 5, 3, 1 },		// 27 bits per transform
			{ 5, 4, 0 },		// 27 bits per transform
			{ 6, 0, 3 },		// 27 bits per transform
			{ 6, 1, 2 },		// 27 bits per transform
			{ 6, 2, 1 },		// 27 bits per transform
			{ 6, 3, 0 },		// 27 bits per transform
			{ 7, 0, 2 },		// 27 bits per transform
			{ 7, 1, 1 },		// 27 bits per transform
			{ 7, 2, 0 },		// 27 bits per transform
			{ 8, 0, 1 },		// 27 bits per transform
			{ 8, 1, 0 },		// 27 bits per transform
			{ 9, 0, 0 },		// 27 bits per transform
			{ 0, 0, 10 },		// 30 bits per transform
			{ 0, 1, 9 },		// 30 bits per transform
			{ 0, 2, 8 },		// 30 bits per transform
			{ 0, 3, 7 },		// 30 bits per transform
			{ 0, 4, 6 },		// 30 bits per transform
			{ 0, 5, 5 },		// 30 bits per transform
			{ 0, 6, 4 },		// 30 bits per transform
			{ 0, 7, 3 },		// 30 bits per transform
			{ 0, 8, 2 },		// 30 bits per transform
			{ 0, 9, 1 },		// 30 bits per transform
			{ 0, 10, 0 },		// 30 bits per transform
			{ 1, 0, 9 },		// 30 bits per transform
			{ 1, 1, 8 },		// 30 bits per transform
			{ 1, 2, 7 },		// 30 bits per transform
			{ 1, 3, 6 },		// 30 bits per transform
			{ 1, 4, 5 },		// 30 bits per transform
			{ 1, 5, 4 },		// 30 bits per transform
			{ 1, 6, 3 },		// 30 bits per transform
			{ 1, 7, 2 },		// 30 bits per transform
			{ 1, 8, 1 },		// 30 bits per transform
			{ 1, 9, 0 },		// 30 bits per transform
			{ 2, 0, 8 },		// 30 bits per transform
			{ 2, 1, 7 },		// 30 bits per transform
			{ 2, 2, 6 },		// 30 bits per transform
			{ 2, 3, 5 },		// 30 bits per transform
			{ 2, 4, 4 },		// 30 bits per transform
			{ 2, 5, 3 },		// 30 bits per transform
			{ 2, 6, 2 },		// 30 bits per transform
			{ 2, 7, 1 },		// 30 bits per transform
			{ 2, 8, 0 },		// 30 bits per transform
			{ 3, 0, 7 },		// 30 bits per transform
			{ 3, 1, 6 },		// 30 bits per transform
			{ 3, 2, 5 },		// 30 bits per transform
			{ 3, 3, 4 },		// 30 bits per transform
			{ 3, 4, 3 },		// 30 bits per transform
			{ 3, 5, 2 },		// 30 bits per transform
			{ 3, 6, 1 },		// 30 bits per transform
			{ 3, 7, 0 },		// 30 bits per transform
			{ 4, 0, 6 },		// 30 bits per transform
			{ 4, 1, 5 },		// 30 bits per transform
			{ 4, 2, 4 },		// 30 bits per transform
			{ 4, 3, 3 },		// 30 bits per transform
			{ 4, 4, 2 },		// 30 bits per transform
			{ 4, 5, 1 },		// 30 bits per transform
			{ 4, 6, 0 },		// 30 bits per transform
			{ 5, 0, 5 },		// 30 bits per transform
			{ 5, 1, 4 },		// 30 bits per transform
			{ 5, 2, 3 },		// 30 bits per transform
			{ 5, 3, 2 },		// 30 bits per transform
			{ 5, 4, 1 },		// 30 bits per transform
			{ 5, 5, 0 },		// 30 bits per transform
			{ 6, 0, 4 },		// 30 bits per transform
			{ 6, 1, 3 },		// 30 bits per transform
			{ 6, 2, 2 },		// 30 bits per transform
			{ 6, 3, 1 },		// 30 bits per transform
			{ 6, 4, 0 },		// 30 bits per transform
			{ 7, 0, 3 },		// 30 bits per transform
			{ 7, 1, 2 },		// 30 bits per transform
			{ 7, 2, 1 },		// 30 bits per transform
			{ 7, 3, 0 },		// 30 bits per transform
			{ 8, 0, 2 },		// 30 bits per transform
			{ 8, 1, 1 },		// 30 bits per transform
			{ 8, 2, 0 },		// 30 bits per transform
			{ 9, 0, 1 },		// 30 bits per transform
			{ 9, 1, 0 },		// 30 bits per transform
			{ 10, 0, 0 },		// 30 bits per transform
			{ 0, 0, 11 },		// 33 bits per transform
			{ 0, 1, 10 },		// 33 bits per transform
			{ 0, 2, 9 },		// 33 bits per transform
			{ 0, 3, 8 },		// 33 bits per transform
			{ 0, 4, 7 },		// 33 bits per transform
			{ 0, 5, 6 },		// 33 bits per transform
			{ 0, 6, 5 },		// 33 bits per transform
			{ 0, 7, 4 },		// 33 bits per transform
			{ 0, 8, 3 },		// 33 bits per transform
			{ 0, 9, 2 },		// 33 bits per transform
			{ 0, 10, 1 },		// 33 bits per transform
			{ 0, 11, 0 },		// 33 bits per transform
			{ 1, 0, 10 },		// 33 bits per transform
			{ 1, 1, 9 },		// 33 bits per transform
			{ 1, 2, 8 },		// 33 bits per transform
			{ 1, 3, 7 },		// 33 bits per transform
			{ 1, 4, 6 },		// 33 bits per transform
			{ 1, 5, 5 },		// 33 bits per transform
			{ 1, 6, 4 },		// 33 bits per transform
			{ 1, 7, 3 },		// 33 bits per transform
			{ 1, 8, 2 },		// 33 bits per transform
			{ 1, 9, 1 },		// 33 bits per transform
			{ 1, 10, 0 },		// 33 bits per transform
			{ 2, 0, 9 },		// 33 bits per transform
			{ 2, 1, 8 },		// 33 bits per transform
			{ 2, 2, 7 },		// 33 bits per transform
			{ 2, 3, 6 },		// 33 bits per transform
			{ 2, 4, 5 },		// 33 bits per transform
			{ 2, 5, 4 },		// 33 bits per transform
			{ 2, 6, 3 },		// 33 bits per transform
			{ 2, 7, 2 },		// 33 bits per transform
			{ 2, 8, 1 },		// 33 bits per transform
			{ 2, 9, 0 },		// 33 bits per transform
			{ 3, 0, 8 },		// 33 bits per transform
			{ 3, 1, 7 },		// 33 bits per transform
			{ 3, 2, 6 },		// 33 bits per transform
			{ 3, 3, 5 },		// 33 bits per transform
			{ 3, 4, 4 },		// 33 bits per transform
			{ 3, 5, 3 },		// 33 bits per transform
			{ 3, 6, 2 },		// 33 bits per transform
			{ 3, 7, 1 },		// 33 bits per transform
			{ 3, 8, 0 },		// 33 bits per transform
			{ 4, 0, 7 },		// 33 bits per transform
			{ 4, 1, 6 },		// 33 bits per transform
			{ 4, 2, 5 },		// 33 bits per transform
			{ 4, 3, 4 },		// 33 bits per transform
			{ 4, 4, 3 },		// 33 bits per transform
			{ 4, 5, 2 },		// 33 bits per transform
			{ 4, 6, 1 },		// 33 bits per transform
			{ 4, 7, 0 },		// 33 bits per transform
			{ 5, 0, 6 },		// 33 bits per transform
			{ 5, 1, 5 },		// 33 bits per transform
			{ 5, 2, 4 },		// 33 bits per transform
			{ 5, 3, 3 },		// 33 bits per transform
			{ 5, 4, 2 },		// 33 bits per transform
			{ 5, 5, 1 },		// 33 bits per transform
			{ 5, 6, 0 },		// 33 bits per transform
			{ 6, 0, 5 },		// 33 bits per transform
			{ 6, 1, 4 },		// 33 bits per transform
			{ 6, 2, 3 },		// 33 bits per transform
			{ 6, 3, 2 },		// 33 bits per transform
			{ 6, 4, 1 },		// 33 bits per transform
			{ 6, 5, 0 },		// 33 bits per transform
			{ 7, 0, 4 },		// 33 bits per transform
			{ 7, 1, 3 },		// 33 bits per transform
			{ 7, 2, 2 },		// 33 bits per transform
			{ 7, 3, 1 },		// 33 bits per transform
			{ 7, 4, 0 },		// 33 bits per transform
			{ 8, 0, 3 },		// 33 bits per transform
			{ 8, 1, 2 },		// 33 bits per transform
			{ 8, 2, 1 },		// 33 bits per transform
			{ 8, 3, 0 },		// 33 bits per transform
			{ 9, 0, 2 },		// 33 bits per transform
			{ 9, 1, 1 },		// 33 bits per transform
			{ 9, 2, 0 },		// 33 bits per transform
			{ 10, 0, 1 },		// 33 bits per transform
			{ 10, 1, 0 },		// 33 bits per transform
			{ 11, 0, 0 },		// 33 bits per transform
			{ 0, 0, 12 },		// 36 bits per transform
			{ 0, 1, 11 },		// 36 bits per transform
			{ 0, 2, 10 },		// 36 bits per transform
			{ 0, 3, 9 },		// 36 bits per transform
			{ 0, 4, 8 },		// 36 bits per transform
			{ 0, 5, 7 },		// 36 bits per transform
			{ 0, 6, 6 },		// 36 bits per transform
			{ 0, 7, 5 },		// 36 bits per transform
			{ 0, 8, 4 },		// 36 bits per transform
			{ 0, 9, 3 },		// 36 bits per transform
			{ 0, 10, 2 },		// 36 bits per transform
			{ 0, 11, 1 },		// 36 bits per transform
			{ 0, 12, 0 },		// 36 bits per transform
			{ 1, 0, 11 },		// 36 bits per transform
			{ 1, 1, 10 },		// 36 bits per transform
			{ 1, 2, 9 },		// 36 bits per transform
			{ 1, 3, 8 },		// 36 bits per transform
			{ 1, 4, 7 },		// 36 bits per transform
			{ 1, 5, 6 },		// 36 bits per transform
			{ 1, 6, 5 },		// 36 bits per transform
			{ 1, 7, 4 },		// 36 bits per transform
			{ 1, 8, 3 },		// 36 bits per transform
			{ 1, 9, 2 },		// 36 bits per transform
			{ 1, 10, 1 },		// 36 bits per transform
			{ 1, 11, 0 },		// 36 bits per transform
			{ 2, 0, 10 },		// 36 bits per transform
			{ 2, 1, 9 },		// 36 bits per transform
			{ 2, 2, 8 },		// 36 bits per transform
			{ 2, 3, 7 },		// 36 bits per transform
			{ 2, 4, 6 },		// 36 bits per transform
			{ 2, 5, 5 },		// 36 bits per transform
			{ 2, 6, 4 },		// 36 bits per transform
			{ 2, 7, 3 },		// 36 bits per transform
			{ 2, 8, 2 },		// 36 bits per transform
			{ 2, 9, 1 },		// 36 bits per transform
			{ 2, 10, 0 },		// 36 bits per transform
			{ 3, 0, 9 },		// 36 bits per transform
			{ 3, 1, 8 },		// 36 bits per transform
			{ 3, 2, 7 },		// 36 bits per transform
			{ 3, 3, 6 },		// 36 bits per transform
			{ 3, 4, 5 },		// 36 bits per transform
			{ 3, 5, 4 },		// 36 bits per transform
			{ 3, 6, 3 },		// 36 bits per transform
			{ 3, 7, 2 },		// 36 bits per transform
			{ 3, 8, 1 },		// 36 bits per transform
			{ 3, 9, 0 },		// 36 bits per transform
			{ 4, 0, 8 },		// 36 bits per transform
			{ 4, 1, 7 },		// 36 bits per transform
			{ 4, 2, 6 },		// 36 bits per transform
			{ 4, 3, 5 },		// 36 bits per transform
			{ 4, 4, 4 },		// 36 bits per transform
			{ 4, 5, 3 },		// 36 bits per transform
			{ 4, 6, 2 },		// 36 bits per transform
			{ 4, 7, 1 },		// 36 bits per transform
			{ 4, 8, 0 },		// 36 bits per transform
			{ 5, 0, 7 },		// 36 bits per transform
			{ 5, 1, 6 },		// 36 bits per transform
			{ 5, 2, 5 },		// 36 bits per transform
			{ 5, 3, 4 },		// 36 bits per transform
			{ 5, 4, 3 },		// 36 bits per transform
			{ 5, 5, 2 },		// 36 bits per transform
			{ 5, 6, 1 },		// 36 bits per transform
			{ 5, 7, 0 },		// 36 bits per transform
			{ 6, 0, 6 },		// 36 bits per transform
			{ 6, 1, 5 },		// 36 bits per transform
			{ 6, 2, 4 },		// 36 bits per transform
			{ 6, 3, 3 },		// 36 bits per transform
			{ 6, 4, 2 },		// 36 bits per transform
			{ 6, 5, 1 },		// 36 bits per transform
			{ 6, 6, 0 },		// 36 bits per transform
			{ 7, 0, 5 },		// 36 bits per transform
			{ 7, 1, 4 },		// 36 bits per transform
			{ 7, 2, 3 },		// 36 bits per transform
			{ 7, 3, 2 },		// 36 bits per transform
			{ 7, 4, 1 },		// 36 bits per transform
			{ 7, 5, 0 },		// 36 bits per transform
			{ 8, 0, 4 },		// 36 bits per transform
			{ 8, 1, 3 },		// 36 bits per transform
			{ 8, 2, 2 },		// 36 bits per transform
			{ 8, 3, 1 },		// 36 bits per transform
			{ 8, 4, 0 },		// 36 bits per transform
			{ 9, 0, 3 },		// 36 bits per transform
			{ 9, 1, 2 },		// 36 bits per transform
			{ 9, 2, 1 },		// 36 bits per transform
			{ 9, 3, 0 },		// 36 bits per transform
			{ 10, 0, 2 },		// 36 bits per transform
			{ 10, 1, 1 },		// 36 bits per transform
			{ 10, 2, 0 },		// 36 bits per transform
			{ 11, 0, 1 },		// 36 bits per transform
			{ 11, 1, 0 },		// 36 bits per transform
			{ 12, 0, 0 },		// 36 bits per transform
			{ 0, 0, 13 },		// 39 bits per transform
			{ 0, 1, 12 },		// 39 bits per transform
			{ 0, 2, 11 },		// 39 bits per transform
			{ 0, 3, 10 },		// 39 bits per transform
			{ 0, 4, 9 },		// 39 bits per transform
			{ 0, 5, 8 },		// 39 bits per transform
			{ 0, 6, 7 },		// 39 bits per transform
			{ 0, 7, 6 },		// 39 bits per transform
			{ 0, 8, 5 },		// 39 bits per transform
			{ 0, 9, 4 },		// 39 bits per transform
			{ 0, 10, 3 },		// 39 bits per transform
			{ 0, 11, 2 },		// 39 bits per transform
			{ 0, 12, 1 },		// 39 bits per transform
			{ 0, 13, 0 },		// 39 bits per transform
			{ 1, 0, 12 },		// 39 bits per transform
			{ 1, 1, 11 },		// 39 bits per transform
			{ 1, 2, 10 },		// 39 bits per transform
			{ 1, 3, 9 },		// 39 bits per transform
			{ 1, 4, 8 },		// 39 bits per transform
			{ 1, 5, 7 },		// 39 bits per transform
			{ 1, 6, 6 },		// 39 bits per transform
			{ 1, 7, 5 },		// 39 bits per transform
			{ 1, 8, 4 },		// 39 bits per transform
			{ 1, 9, 3 },		// 39 bits per transform
			{ 1, 10, 2 },		// 39 bits per transform
			{ 1, 11, 1 },		// 39 bits per transform
			{ 1, 12, 0 },		// 39 bits per transform
			{ 2, 0, 11 },		// 39 bits per transform
			{ 2, 1, 10 },		// 39 bits per transform
			{ 2, 2, 9 },		// 39 bits per transform
			{ 2, 3, 8 },		// 39 bits per transform
			{ 2, 4, 7 },		// 39 bits per transform
			{ 2, 5, 6 },		// 39 bits per transform
			{ 2, 6, 5 },		// 39 bits per transform
			{ 2, 7, 4 },		// 39 bits per transform
			{ 2, 8, 3 },		// 39 bits per transform
			{ 2, 9, 2 },		// 39 bits per transform
			{ 2, 10, 1 },		// 39 bits per transform
			{ 2, 11, 0 },		// 39 bits per transform
			{ 3, 0, 10 },		// 39 bits per transform
			{ 3, 1, 9 },		// 39 bits per transform
			{ 3, 2, 8 },		// 39 bits per transform
			{ 3, 3, 7 },		// 39 bits per transform
			{ 3, 4, 6 },		// 39 bits per transform
			{ 3, 5, 5 },		// 39 bits per transform
			{ 3, 6, 4 },		// 39 bits per transform
			{ 3, 7, 3 },		// 39 bits per transform
			{ 3, 8, 2 },		// 39 bits per transform
			{ 3, 9, 1 },		// 39 bits per transform
			{ 3, 10, 0 },		// 39 bits per transform
			{ 4, 0, 9 },		// 39 bits per transform
			{ 4, 1, 8 },		// 39 bits per transform
			{ 4, 2, 7 },		// 39 bits per transform
			{ 4, 3, 6 },		// 39 bits per transform
			{ 4, 4, 5 },		// 39 bits per transform
			{ 4, 5, 4 },		// 39 bits per transform
			{ 4, 6, 3 },		// 39 bits per transform
			{ 4, 7, 2 },		// 39 bits per transform
			{ 4, 8, 1 },		// 39 bits per transform
			{ 4, 9, 0 },		// 39 bits per transform
			{ 5, 0, 8 },		// 39 bits per transform
			{ 5, 1, 7 },		// 39 bits per transform
			{ 5, 2, 6 },		// 39 bits per transform
			{ 5, 3, 5 },		// 39 bits per transform
			{ 5, 4, 4 },		// 39 bits per transform
			{ 5, 5, 3 },		// 39 bits per transform
			{ 5, 6, 2 },		// 39 bits per transform
			{ 5, 7, 1 },		// 39 bits per transform
			{ 5, 8, 0 },		// 39 bits per transform
			{ 6, 0, 7 },		// 39 bits per transform
			{ 6, 1, 6 },		// 39 bits per transform
			{ 6, 2, 5 },		// 39 bits per transform
			{ 6, 3, 4 },		// 39 bits per transform
			{ 6, 4, 3 },		// 39 bits per transform
			{ 6, 5, 2 },		// 39 bits per transform
			{ 6, 6, 1 },		// 39 bits per transform
			{ 6, 7, 0 },		// 39 bits per transform
			{ 7, 0, 6 },		// 39 bits per transform
			{ 7, 1, 5 },		// 39 bits per transform
			{ 7, 2, 4 },		// 39 bits per transform
			{ 7, 3, 3 },		// 39 bits per transform
			{ 7, 4, 2 },		// 39 bits per transform
			{ 7, 5, 1 },		// 39 bits per transform
			{ 7, 6, 0 },		// 39 bits per transform
			{ 8, 0, 5 },		// 39 bits per transform
			{ 8, 1, 4 },		// 39 bits per transform
			{ 8, 2, 3 },		// 39 bits per transform
			{ 8, 3, 2 },		// 39 bits per transform
			{ 8, 4, 1 },		// 39 bits per transform
			{ 8, 5, 0 },		// 39 bits per transform
			{ 9, 0, 4 },		// 39 bits per transform
			{ 9, 1, 3 },		// 39 bits per transform
			{ 9, 2, 2 },		// 39 bits per transform
			{ 9, 3, 1 },		// 39 bits per transform
			{ 9, 4, 0 },		// 39 bits per transform
			{ 10, 0, 3 },		// 39 bits per transform
			{ 10, 1, 2 },		// 39 bits per transform
			{ 10, 2, 1 },		// 39 bits per transform
			{ 10, 3, 0 },		// 39 bits per transform
			{ 11, 0, 2 },		// 39 bits per transform
			{ 11, 1, 1 },		// 39 bits per transform
			{ 11, 2, 0 },		// 39 bits per transform
			{ 12, 0, 1 },		// 39 bits per transform
			{ 12, 1, 0 },		// 39 bits per transform
			{ 13, 0, 0 },		// 39 bits per transform
			{ 0, 0, 14 },		// 42 bits per transform
			{ 0, 1, 13 },		// 42 bits per transform
			{ 0, 2, 12 },		// 42 bits per transform
			{ 0, 3, 11 },		// 42 bits per transform
			{ 0, 4, 10 },		// 42 bits per transform
			{ 0, 5, 9 },		// 42 bits per transform
			{ 0, 6, 8 },		// 42 bits per transform
			{ 0, 7, 7 },		// 42 bits per transform
			{ 0, 8, 6 },		// 42 bits per transform
			{ 0, 9, 5 },		// 42 bits per transform
			{ 0, 10, 4 },		// 42 bits per transform
			{ 0, 11, 3 },		// 42 bits per transform
			{ 0, 12, 2 },		// 42 bits per transform
			{ 0, 13, 1 },		// 42 bits per transform
			{ 0, 14, 0 },		// 42 bits per transform
			{ 1, 0, 13 },		// 42 bits per transform
			{ 1, 1, 12 },		// 42 bits per transform
			{ 1, 2, 11 },		// 42 bits per transform
			{ 1, 3, 10 },		// 42 bits per transform
			{ 1, 4, 9 },		// 42 bits per transform
			{ 1, 5, 8 },		// 42 bits per transform
			{ 1, 6, 7 },		// 42 bits per transform
			{ 1, 7, 6 },		// 42 bits per transform
			{ 1, 8, 5 },		// 42 bits per transform
			{ 1, 9, 4 },		// 42 bits per transform
			{ 1, 10, 3 },		// 42 bits per transform
			{ 1, 11, 2 },		// 42 bits per transform
			{ 1, 12, 1 },		// 42 bits per transform
			{ 1, 13, 0 },		// 42 bits per transform
			{ 2, 0, 12 },		// 42 bits per transform
			{ 2, 1, 11 },		// 42 bits per transform
			{ 2, 2, 10 },		// 42 bits per transform
			{ 2, 3, 9 },		// 42 bits per transform
			{ 2, 4, 8 },		// 42 bits per transform
			{ 2, 5, 7 },		// 42 bits per transform
			{ 2, 6, 6 },		// 42 bits per transform
			{ 2, 7, 5 },		// 42 bits per transform
			{ 2, 8, 4 },		// 42 bits per transform
			{ 2, 9, 3 },		// 42 bits per transform
			{ 2, 10, 2 },		// 42 bits per transform
			{ 2, 11, 1 },		// 42 bits per transform
			{ 2, 12, 0 },		// 42 bits per transform
			{ 3, 0, 11 },		// 42 bits per transform
			{ 3, 1, 10 },		// 42 bits per transform
			{ 3, 2, 9 },		// 42 bits per transform
			{ 3, 3, 8 },		// 42 bits per transform
			{ 3, 4, 7 },		// 42 bits per transform
			{ 3, 5, 6 },		// 42 bits per transform
			{ 3, 6, 5 },		// 42 bits per transform
			{ 3, 7, 4 },		// 42 bits per transform
			{ 3, 8, 3 },		// 42 bits per transform
			{ 3, 9, 2 },		// 42 bits per transform
			{ 3, 10, 1 },		// 42 bits per transform
			{ 3, 11, 0 },		// 42 bits per transform
			{ 4, 0, 10 },		// 42 bits per transform
			{ 4, 1, 9 },		// 42 bits per transform
			{ 4, 2, 8 },		// 42 bits per transform
			{ 4, 3, 7 },		// 42 bits per transform
			{ 4, 4, 6 },		// 42 bits per transform
			{ 4, 5, 5 },		// 42 bits per transform
			{ 4, 6, 4 },		// 42 bits per transform
			{ 4, 7, 3 },		// 42 bits per transform
			{ 4, 8, 2 },		// 42 bits per transform
			{ 4, 9, 1 },		// 42 bits per transform
			{ 4, 10, 0 },		// 42 bits per transform
			{ 5, 0, 9 },		// 42 bits per transform
			{ 5, 1, 8 },		// 42 bits per transform
			{ 5, 2, 7 },		// 42 bits per transform
			{ 5, 3, 6 },		// 42 bits per transform
			{ 5, 4, 5 },		// 42 bits per transform
			{ 5, 5, 4 },		// 42 bits per transform
			{ 5, 6, 3 },		// 42 bits per transform
			{ 5, 7, 2 },		// 42 bits per transform
			{ 5, 8, 1 },		// 42 bits per transform
			{ 5, 9, 0 },		// 42 bits per transform
			{ 6, 0, 8 },		// 42 bits per transform
			{ 6, 1, 7 },		// 42 bits per transform
			{ 6, 2, 6 },		// 42 bits per transform
			{ 6, 3, 5 },		// 42 bits per transform
			{ 6, 4, 4 },		// 42 bits per transform
			{ 6, 5, 3 },		// 42 bits per transform
			{ 6, 6, 2 },		// 42 bits per transform
			{ 6, 7, 1 },		// 42 bits per transform
			{ 6, 8, 0 },		// 42 bits per transform
			{ 7, 0, 7 },		// 42 bits per transform
			{ 7, 1, 6 },		// 42 bits per transform
			{ 7, 2, 5 },		// 42 bits per transform
			{ 7, 3, 4 },		// 42 bits per transform
			{ 7, 4, 3 },		// 42 bits per transform
			{ 7, 5, 2 },		// 42 bits per transform
			{ 7, 6, 1 },		// 42 bits per transform
			{ 7, 7, 0 },		// 42 bits per transform
			{ 8, 0, 6 },		// 42 bits per transform
			{ 8, 1, 5 },		// 42 bits per transform
			{ 8, 2, 4 },		// 42 bits per transform
			{ 8, 3, 3 },		// 42 bits per transform
			{ 8, 4, 2 },		// 42 bits per transform
			{ 8, 5, 1 },		// 42 bits per transform
			{ 8, 6, 0 },		// 42 bits per transform
			{ 9, 0, 5 },		// 42 bits per transform
			{ 9, 1, 4 },		// 42 bits per transform
			{ 9, 2, 3 },		// 42 bits per transform
			{ 9, 3, 2 },		// 42 bits per transform
			{ 9, 4, 1 },		// 42 bits per transform
			{ 9, 5, 0 },		// 42 bits per transform
			{ 10, 0, 4 },		// 42 bits per transform
			{ 10, 1, 3 },		// 42 bits per transform
			{ 10, 2, 2 },		// 42 bits per transform
			{ 10, 3, 1 },		// 42 bits per transform
			{ 10, 4, 0 },		// 42 bits per transform
			{ 11, 0, 3 },		// 42 bits per transform
			{ 11, 1, 2 },		// 42 bits per transform
			{ 11, 2, 1 },		// 42 bits per transform
			{ 11, 3, 0 },		// 42 bits per transform
			{ 12, 0, 2 },		// 42 bits per transform
			{ 12, 1, 1 },		// 42 bits per transform
			{ 12, 2, 0 },		// 42 bits per transform
			{ 13, 0, 1 },		// 42 bits per transform
			{ 13, 1, 0 },		// 42 bits per transform
			{ 14, 0, 0 },		// 42 bits per transform
			{ 0, 0, 15 },		// 45 bits per transform
			{ 0, 1, 14 },		// 45 bits per transform
			{ 0, 2, 13 },		// 45 bits per transform
			{ 0, 3, 12 },		// 45 bits per transform
			{ 0, 4, 11 },		// 45 bits per transform
			{ 0, 5, 10 },		// 45 bits per transform
			{ 0, 6, 9 },		// 45 bits per transform
			{ 0, 7, 8 },		// 45 bits per transform
			{ 0, 8, 7 },		// 45 bits per transform
			{ 0, 9, 6 },		// 45 bits per transform
			{ 0, 10, 5 },		// 45 bits per transform
			{ 0, 11, 4 },		// 45 bits per transform
			{ 0, 12, 3 },		// 45 bits per transform
			{ 0, 13, 2 },		// 45 bits per transform
			{ 0, 14, 1 },		// 45 bits per transform
			{ 0, 15, 0 },		// 45 bits per transform
			{ 1, 0, 14 },		// 45 bits per transform
			{ 1, 1, 13 },		// 45 bits per transform
			{ 1, 2, 12 },		// 45 bits per transform
			{ 1, 3, 11 },		// 45 bits per transform
			{ 1, 4, 10 },		// 45 bits per transform
			{ 1, 5, 9 },		// 45 bits per transform
			{ 1, 6, 8 },		// 45 bits per transform
			{ 1, 7, 7 },		// 45 bits per transform
			{ 1, 8, 6 },		// 45 bits per transform
			{ 1, 9, 5 },		// 45 bits per transform
			{ 1, 10, 4 },		// 45 bits per transform
			{ 1, 11, 3 },		// 45 bits per transform
			{ 1, 12, 2 },		// 45 bits per transform
			{ 1, 13, 1 },		// 45 bits per transform
			{ 1, 14, 0 },		// 45 bits per transform
			{ 2, 0, 13 },		// 45 bits per transform
			{ 2, 1, 12 },		// 45 bits per transform
			{ 2, 2, 11 },		// 45 bits per transform
			{ 2, 3, 10 },		// 45 bits per transform
			{ 2, 4, 9 },		// 45 bits per transform
			{ 2, 5, 8 },		// 45 bits per transform
			{ 2, 6, 7 },		// 45 bits per transform
			{ 2, 7, 6 },		// 45 bits per transform
			{ 2, 8, 5 },		// 45 bits per transform
			{ 2, 9, 4 },		// 45 bits per transform
			{ 2, 10, 3 },		// 45 bits per transform
			{ 2, 11, 2 },		// 45 bits per transform
			{ 2, 12, 1 },		// 45 bits per transform
			{ 2, 13, 0 },		// 45 bits per transform
			{ 3, 0, 12 },		// 45 bits per transform
			{ 3, 1, 11 },		// 45 bits per transform
			{ 3, 2, 10 },		// 45 bits per transform
			{ 3, 3, 9 },		// 45 bits per transform
			{ 3, 4, 8 },		// 45 bits per transform
			{ 3, 5, 7 },		// 45 bits per transform
			{ 3, 6, 6 },		// 45 bits per transform
			{ 3, 7, 5 },		// 45 bits per transform
			{ 3, 8, 4 },		// 45 bits per transform
			{ 3, 9, 3 },		// 45 bits per transform
			{ 3, 10, 2 },		// 45 bits per transform
			{ 3, 11, 1 },		// 45 bits per transform
			{ 3, 12, 0 },		// 45 bits per transform
			{ 4, 0, 11 },		// 45 bits per transform
			{ 4, 1, 10 },		// 45 bits per transform
			{ 4, 2, 9 },		// 45 bits per transform
			{ 4, 3, 8 },		// 45 bits per transform
			{ 4, 4, 7 },		// 45 bits per transform
			{ 4, 5, 6 },		// 45 bits per transform
			{ 4, 6, 5 },		// 45 bits per transform
			{ 4, 7, 4 },		// 45 bits per transform
			{ 4, 8, 3 },		// 45 bits per transform
			{ 4, 9, 2 },		// 45 bits per transform
			{ 4, 10, 1 },		// 45 bits per transform
			{ 4, 11, 0 },		// 45 bits per transform
			{ 5, 0, 10 },		// 45 bits per transform
			{ 5, 1, 9 },		// 45 bits per transform
			{ 5, 2, 8 },		// 45 bits per transform
			{ 5, 3, 7 },		// 45 bits per transform
			{ 5, 4, 6 },		// 45 bits per transform
			{ 5, 5, 5 },		// 45 bits per transform
			{ 5, 6, 4 },		// 45 bits per transform
			{ 5, 7, 3 },		// 45 bits per transform
			{ 5, 8, 2 },		// 45 bits per transform
			{ 5, 9, 1 },		// 45 bits per transform
			{ 5, 10, 0 },		// 45 bits per transform
			{ 6, 0, 9 },		// 45 bits per transform
			{ 6, 1, 8 },		// 45 bits per transform
			{ 6, 2, 7 },		// 45 bits per transform
			{ 6, 3, 6 },		// 45 bits per transform
			{ 6, 4, 5 },		// 45 bits per transform
			{ 6, 5, 4 },		// 45 bits per transform
			{ 6, 6, 3 },		// 45 bits per transform
			{ 6, 7, 2 },		// 45 bits per transform
			{ 6, 8, 1 },		// 45 bits per transform
			{ 6, 9, 0 },		// 45 bits per transform
			{ 7, 0, 8 },		// 45 bits per transform
			{ 7, 1, 7 },		// 45 bits per transform
			{ 7, 2, 6 },		// 45 bits per transform
			{ 7, 3, 5 },		// 45 bits per transform
			{ 7, 4, 4 },		// 45 bits per transform
			{ 7, 5, 3 },		// 45 bits per transform
			{ 7, 6, 2 },		// 45 bits per transform
			{ 7, 7, 1 },		// 45 bits per transform
			{ 7, 8, 0 },		// 45 bits per transform
			{ 8, 0, 7 },		// 45 bits per transform
			{ 8, 1, 6 },		// 45 bits per transform
			{ 8, 2, 5 },		// 45 bits per transform
			{ 8, 3, 4 },		// 45 bits per transform
			{ 8, 4, 3 },		// 45 bits per transform
			{ 8, 5, 2 },		// 45 bits per transform
			{ 8, 6, 1 },		// 45 bits per transform
			{ 8, 7, 0 },		// 45 bits per transform
			{ 9, 0, 6 },		// 45 bits per transform
			{ 9, 1, 5 },		// 45 bits per transform
			{ 9, 2, 4 },		// 45 bits per transform
			{ 9, 3, 3 },		// 45 bits per transform
			{ 9, 4, 2 },		// 45 bits per transform
			{ 9, 5, 1 },		// 45 bits per transform
			{ 9, 6, 0 },		// 45 bits per transform
			{ 10, 0, 5 },		// 45 bits per transform
			{ 10, 1, 4 },		// 45 bits per transform
			{ 10, 2, 3 },		// 45 bits per transform
			{ 10, 3, 2 },		// 45 bits per transform
			{ 10, 4, 1 },		// 45 bits per transform
			{ 10, 5, 0 },		// 45 bits per transform
			{ 11, 0, 4 },		// 45 bits per transform
			{ 11, 1, 3 },		// 45 bits per transform
			{ 11, 2, 2 },		// 45 bits per transform
			{ 11, 3, 1 },		// 45 bits per transform
			{ 11, 4, 0 },		// 45 bits per transform
			{ 12, 0, 3 },		// 45 bits per transform
			{ 12, 1, 2 },		// 45 bits per transform
			{ 12, 2, 1 },		// 45 bits per transform
			{ 12, 3, 0 },		// 45 bits per transform
			{ 13, 0, 2 },		// 45 bits per transform
			{ 13, 1, 1 },		// 45 bits per transform
			{ 13, 2, 0 },		// 45 bits per transform
			{ 14, 0, 1 },		// 45 bits per transform
			{ 14, 1, 0 },		// 45 bits per transform
			{ 15, 0, 0 },		// 45 bits per transform
			{ 0, 0, 16 },		// 48 bits per transform
			{ 0, 1, 15 },		// 48 bits per transform
			{ 0, 2, 14 },		// 48 bits per transform
			{ 0, 3, 13 },		// 48 bits per transform
			{ 0, 4, 12 },		// 48 bits per transform
			{ 0, 5, 11 },		// 48 bits per transform
			{ 0, 6, 10 },		// 48 bits per transform
			{ 0, 7, 9 },		// 48 bits per transform
			{ 0, 8, 8 },		// 48 bits per transform
			{ 0, 9, 7 },		// 48 bits per transform
			{ 0, 10, 6 },		// 48 bits per transform
			{ 0, 11, 5 },		// 48 bits per transform
			{ 0, 12, 4 },		// 48 bits per transform
			{ 0, 13, 3 },		// 48 bits per transform
			{ 0, 14, 2 },		// 48 bits per transform
			{ 0, 15, 1 },		// 48 bits per transform
			{ 0, 16, 0 },		// 48 bits per transform
			{ 1, 0, 15 },		// 48 bits per transform
			{ 1, 1, 14 },		// 48 bits per transform
			{ 1, 2, 13 },		// 48 bits per transform
			{ 1, 3, 12 },		// 48 bits per transform
			{ 1, 4, 11 },		// 48 bits per transform
			{ 1, 5, 10 },		// 48 bits per transform
			{ 1, 6, 9 },		// 48 bits per transform
			{ 1, 7, 8 },		// 48 bits per transform
			{ 1, 8, 7 },		// 48 bits per transform
			{ 1, 9, 6 },		// 48 bits per transform
			{ 1, 10, 5 },		// 48 bits per transform
			{ 1, 11, 4 },		// 48 bits per transform
			{ 1, 12, 3 },		// 48 bits per transform
			{ 1, 13, 2 },		// 48 bits per transform
			{ 1, 14, 1 },		// 48 bits per transform
			{ 1, 15, 0 },		// 48 bits per transform
			{ 2, 0, 14 },		// 48 bits per transform
			{ 2, 1, 13 },		// 48 bits per transform
			{ 2, 2, 12 },		// 48 bits per transform
			{ 2, 3, 11 },		// 48 bits per transform
			{ 2, 4, 10 },		// 48 bits per transform
			{ 2, 5, 9 },		// 48 bits per transform
			{ 2, 6, 8 },		// 48 bits per transform
			{ 2, 7, 7 },		// 48 bits per transform
			{ 2, 8, 6 },		// 48 bits per transform
			{ 2, 9, 5 },		// 48 bits per transform
			{ 2, 10, 4 },		// 48 bits per transform
			{ 2, 11, 3 },		// 48 bits per transform
			{ 2, 12, 2 },		// 48 bits per transform
			{ 2, 13, 1 },		// 48 bits per transform
			{ 2, 14, 0 },		// 48 bits per transform
			{ 3, 0, 13 },		// 48 bits per transform
			{ 3, 1, 12 },		// 48 bits per transform
			{ 3, 2, 11 },		// 48 bits per transform
			{ 3, 3, 10 },		// 48 bits per transform
			{ 3, 4, 9 },		// 48 bits per transform
			{ 3, 5, 8 },		// 48 bits per transform
			{ 3, 6, 7 },		// 48 bits per transform
			{ 3, 7, 6 },		// 48 bits per transform
			{ 3, 8, 5 },		// 48 bits per transform
			{ 3, 9, 4 },		// 48 bits per transform
			{ 3, 10, 3 },		// 48 bits per transform
			{ 3, 11, 2 },		// 48 bits per transform
			{ 3, 12, 1 },		// 48 bits per transform
			{ 3, 13, 0 },		// 48 bits per transform
			{ 4, 0, 12 },		// 48 bits per transform
			{ 4, 1, 11 },		// 48 bits per transform
			{ 4, 2, 10 },		// 48 bits per transform
			{ 4, 3, 9 },		// 48 bits per transform
			{ 4, 4, 8 },		// 48 bits per transform
			{ 4, 5, 7 },		// 48 bits per transform
			{ 4, 6, 6 },		// 48 bits per transform
			{ 4, 7, 5 },		// 48 bits per transform
			{ 4, 8, 4 },		// 48 bits per transform
			{ 4, 9, 3 },		// 48 bits per transform
			{ 4, 10, 2 },		// 48 bits per transform
			{ 4, 11, 1 },		// 48 bits per transform
			{ 4, 12, 0 },		// 48 bits per transform
			{ 5, 0, 11 },		// 48 bits per transform
			{ 5, 1, 10 },		// 48 bits per transform
			{ 5, 2, 9 },		// 48 bits per transform
			{ 5, 3, 8 },		// 48 bits per transform
			{ 5, 4, 7 },		// 48 bits per transform
			{ 5, 5, 6 },		// 48 bits per transform
			{ 5, 6, 5 },		// 48 bits per transform
			{ 5, 7, 4 },		// 48 bits per transform
			{ 5, 8, 3 },		// 48 bits per transform
			{ 5, 9, 2 },		// 48 bits per transform
			{ 5, 10, 1 },		// 48 bits per transform
			{ 5, 11, 0 },		// 48 bits per transform
			{ 6, 0, 10 },		// 48 bits per transform
			{ 6, 1, 9 },		// 48 bits per transform
			{ 6, 2, 8 },		// 48 bits per transform
			{ 6, 3, 7 },		// 48 bits per transform
			{ 6, 4, 6 },		// 48 bits per transform
			{ 6, 5, 5 },		// 48 bits per transform
			{ 6, 6, 4 },		// 48 bits per transform
			{ 6, 7, 3 },		// 48 bits per transform
			{ 6, 8, 2 },		// 48 bits per transform
			{ 6, 9, 1 },		// 48 bits per transform
			{ 6, 10, 0 },		// 48 bits per transform
			{ 7, 0, 9 },		// 48 bits per transform
			{ 7, 1, 8 },		// 48 bits per transform
			{ 7, 2, 7 },		// 48 bits per transform
			{ 7, 3, 6 },		// 48 bits per transform
			{ 7, 4, 5 },		// 48 bits per transform
			{ 7, 5, 4 },		// 48 bits per transform
			{ 7, 6, 3 },		// 48 bits per transform
			{ 7, 7, 2 },		// 48 bits per transform
			{ 7, 8, 1 },		// 48 bits per transform
			{ 7, 9, 0 },		// 48 bits per transform
			{ 8, 0, 8 },		// 48 bits per transform
			{ 8, 1, 7 },		// 48 bits per transform
			{ 8, 2, 6 },		// 48 bits per transform
			{ 8, 3, 5 },		// 48 bits per transform
			{ 8, 4, 4 },		// 48 bits per transform
			{ 8, 5, 3 },		// 48 bits per transform
			{ 8, 6, 2 },		// 48 bits per transform
			{ 8, 7, 1 },		// 48 bits per transform
			{ 8, 8, 0 },		// 48 bits per transform
			{ 9, 0, 7 },		// 48 bits per transform
			{ 9, 1, 6 },		// 48 bits per transform
			{ 9, 2, 5 },		// 48 bits per transform
			{ 9, 3, 4 },		// 48 bits per transform
			{ 9, 4, 3 },		// 48 bits per transform
			{ 9, 5, 2 },		// 48 bits per transform
			{ 9, 6, 1 },		// 48 bits per transform
			{ 9, 7, 0 },		// 48 bits per transform
			{ 10, 0, 6 },		// 48 bits per transform
			{ 10, 1, 5 },		// 48 bits per transform
			{ 10, 2, 4 },		// 48 bits per transform
			{ 10, 3, 3 },		// 48 bits per transform
			{ 10, 4, 2 },		// 48 bits per transform
			{ 10, 5, 1 },		// 48 bits per transform
			{ 10, 6, 0 },		// 48 bits per transform
			{ 11, 0, 5 },		// 48 bits per transform
			{ 11, 1, 4 },		// 48 bits per transform
			{ 11, 2, 3 },		// 48 bits per transform
			{ 11, 3, 2 },		// 48 bits per transform
			{ 11, 4, 1 },		// 48 bits per transform
			{ 11, 5, 0 },		// 48 bits per transform
			{ 12, 0, 4 },		// 48 bits per transform
			{ 12, 1, 3 },		// 48 bits per transform
			{ 12, 2, 2 },		// 48 bits per transform
			{ 12, 3, 1 },		// 48 bits per transform
			{ 12, 4, 0 },		// 48 bits per transform
			{ 13, 0, 3 },		// 48 bits per transform
			{ 13, 1, 2 },		// 48 bits per transform
			{ 13, 2, 1 },		// 48 bits per transform
			{ 13, 3, 0 },		// 48 bits per transform
			{ 14, 0, 2 },		// 48 bits per transform
			{ 14, 1, 1 },		// 48 bits per transform
			{ 14, 2, 0 },		// 48 bits per transform
			{ 15, 0, 1 },		// 48 bits per transform
			{ 15, 1, 0 },		// 48 bits per transform
			{ 16, 0, 0 },		// 48 bits per transform
			{ 0, 0, 17 },		// 51 bits per transform
			{ 0, 1, 16 },		// 51 bits per transform
			{ 0, 2, 15 },		// 51 bits per transform
			{ 0, 3, 14 },		// 51 bits per transform
			{ 0, 4, 13 },		// 51 bits per transform
			{ 0, 5, 12 },		// 51 bits per transform
			{ 0, 6, 11 },		// 51 bits per transform
			{ 0, 7, 10 },		// 51 bits per transform
			{ 0, 8, 9 },		// 51 bits per transform
			{ 0, 9, 8 },		// 51 bits per transform
			{ 0, 10, 7 },		// 51 bits per transform
			{ 0, 11, 6 },		// 51 bits per transform
			{ 0, 12, 5 },		// 51 bits per transform
			{ 0, 13, 4 },		// 51 bits per transform
			{ 0, 14, 3 },		// 51 bits per transform
			{ 0, 15, 2 },		// 51 bits per transform
			{ 0, 16, 1 },		// 51 bits per transform
			{ 0, 17, 0 },		// 51 bits per transform
			{ 1, 0, 16 },		// 51 bits per transform
			{ 1, 1, 15 },		// 51 bits per transform
			{ 1, 2, 14 },		// 51 bits per transform
			{ 1, 3, 13 },		// 51 bits per transform
			{ 1, 4, 12 },		// 51 bits per transform
			{ 1, 5, 11 },		// 51 bits per transform
			{ 1, 6, 10 },		// 51 bits per transform
			{ 1, 7, 9 },		// 51 bits per transform
			{ 1, 8, 8 },		// 51 bits per transform
			{ 1, 9, 7 },		// 51 bits per transform
			{ 1, 10, 6 },		// 51 bits per transform
			{ 1, 11, 5 },		// 51 bits per transform
			{ 1, 12, 4 },		// 51 bits per transform
			{ 1, 13, 3 },		// 51 bits per transform
			{ 1, 14, 2 },		// 51 bits per transform
			{ 1, 15, 1 },		// 51 bits per transform
			{ 1, 16, 0 },		// 51 bits per transform
			{ 2, 0, 15 },		// 51 bits per transform
			{ 2, 1, 14 },		// 51 bits per transform
			{ 2, 2, 13 },		// 51 bits per transform
			{ 2, 3, 12 },		// 51 bits per transform
			{ 2, 4, 11 },		// 51 bits per transform
			{ 2, 5, 10 },		// 51 bits per transform
			{ 2, 6, 9 },		// 51 bits per transform
			{ 2, 7, 8 },		// 51 bits per transform
			{ 2, 8, 7 },		// 51 bits per transform
			{ 2, 9, 6 },		// 51 bits per transform
			{ 2, 10, 5 },		// 51 bits per transform
			{ 2, 11, 4 },		// 51 bits per transform
			{ 2, 12, 3 },		// 51 bits per transform
			{ 2, 13, 2 },		// 51 bits per transform
			{ 2, 14, 1 },		// 51 bits per transform
			{ 2, 15, 0 },		// 51 bits per transform
			{ 3, 0, 14 },		// 51 bits per transform
			{ 3, 1, 13 },		// 51 bits per transform
			{ 3, 2, 12 },		// 51 bits per transform
			{ 3, 3, 11 },		// 51 bits per transform
			{ 3, 4, 10 },		// 51 bits per transform
			{ 3, 5, 9 },		// 51 bits per transform
			{ 3, 6, 8 },		// 51 bits per transform
			{ 3, 7, 7 },		// 51 bits per transform
			{ 3, 8, 6 },		// 51 bits per transform
			{ 3, 9, 5 },		// 51 bits per transform
			{ 3, 10, 4 },		// 51 bits per transform
			{ 3, 11, 3 },		// 51 bits per transform
			{ 3, 12, 2 },		// 51 bits per transform
			{ 3, 13, 1 },		// 51 bits per transform
			{ 3, 14, 0 },		// 51 bits per transform
			{ 4, 0, 13 },		// 51 bits per transform
			{ 4, 1, 12 },		// 51 bits per transform
			{ 4, 2, 11 },		// 51 bits per transform
			{ 4, 3, 10 },		// 51 bits per transform
			{ 4, 4, 9 },		// 51 bits per transform
			{ 4, 5, 8 },		// 51 bits per transform
			{ 4, 6, 7 },		// 51 bits per transform
			{ 4, 7, 6 },		// 51 bits per transform
			{ 4, 8, 5 },		// 51 bits per transform
			{ 4, 9, 4 },		// 51 bits per transform
			{ 4, 10, 3 },		// 51 bits per transform
			{ 4, 11, 2 },		// 51 bits per transform
			{ 4, 12, 1 },		// 51 bits per transform
			{ 4, 13, 0 },		// 51 bits per transform
			{ 5, 0, 12 },		// 51 bits per transform
			{ 5, 1, 11 },		// 51 bits per transform
			{ 5, 2, 10 },		// 51 bits per transform
			{ 5, 3, 9 },		// 51 bits per transform
			{ 5, 4, 8 },		// 51 bits per transform
			{ 5, 5, 7 },		// 51 bits per transform
			{ 5, 6, 6 },		// 51 bits per transform
			{ 5, 7, 5 },		// 51 bits per transform
			{ 5, 8, 4 },		// 51 bits per transform
			{ 5, 9, 3 },		// 51 bits per transform
			{ 5, 10, 2 },		// 51 bits per transform
			{ 5, 11, 1 },		// 51 bits per transform
			{ 5, 12, 0 },		// 51 bits per transform
			{ 6, 0, 11 },		// 51 bits per transform
			{ 6, 1, 10 },		// 51 bits per transform
			{ 6, 2, 9 },		// 51 bits per transform
			{ 6, 3, 8 },		// 51 bits per transform
			{ 6, 4, 7 },		// 51 bits per transform
			{ 6, 5, 6 },		// 51 bits per transform
			{ 6, 6, 5 },		// 51 bits per transform
			{ 6, 7, 4 },		// 51 bits per transform
			{ 6, 8, 3 },		// 51 bits per transform
			{ 6, 9, 2 },		// 51 bits per transform
			{ 6, 10, 1 },		// 51 bits per transform
			{ 6, 11, 0 },		// 51 bits per transform
			{ 7, 0, 10 },		// 51 bits per transform
			{ 7, 1, 9 },		// 51 bits per transform
			{ 7, 2, 8 },		// 51 bits per transform
			{ 7, 3, 7 },		// 51 bits per transform
			{ 7, 4, 6 },		// 51 bits per transform
			{ 7, 5, 5 },		// 51 bits per transform
			{ 7, 6, 4 },		// 51 bits per transform
			{ 7, 7, 3 },		// 51 bits per transform
			{ 7, 8, 2 },		// 51 bits per transform
			{ 7, 9, 1 },		// 51 bits per transform
			{ 7, 10, 0 },		// 51 bits per transform
			{ 8, 0, 9 },		// 51 bits per transform
			{ 8, 1, 8 },		// 51 bits per transform
			{ 8, 2, 7 },		// 51 bits per transform
			{ 8, 3, 6 },		// 51 bits per transform
			{ 8, 4, 5 },		// 51 bits per transform
			{ 8, 5, 4 },		// 51 bits per transform
			{ 8, 6, 3 },		// 51 bits per transform
			{ 8, 7, 2 },		// 51 bits per transform
			{ 8, 8, 1 },		// 51 bits per transform
			{ 8, 9, 0 },		// 51 bits per transform
			{ 9, 0, 8 },		// 51 bits per transform
			{ 9, 1, 7 },		// 51 bits per transform
			{ 9, 2, 6 },		// 51 bits per transform
			{ 9, 3, 5 },		// 51 bits per transform
			{ 9, 4, 4 },		// 51 bits per transform
			{ 9, 5, 3 },		// 51 bits per transform
			{ 9, 6, 2 },		// 51 bits per transform
			{ 9, 7, 1 },		// 51 bits per transform
			{ 9, 8, 0 },		// 51 bits per transform
			{ 10, 0, 7 },		// 51 bits per transform
			{ 10, 1, 6 },		// 51 bits per transform
			{ 10, 2, 5 },		// 51 bits per transform
			{ 10, 3, 4 },		// 51 bits per transform
			{ 10, 4, 3 },		// 51 bits per transform
			{ 10, 5, 2 },		// 51 bits per transform
			{ 10, 6, 1 },		// 51 bits per transform
			{ 10, 7, 0 },		// 51 bits per transform
			{ 11, 0, 6 },		// 51 bits per transform
			{ 11, 1, 5 },		// 51 bits per transform
			{ 11, 2, 4 },		// 51 bits per transform
			{ 11, 3, 3 },		// 51 bits per transform
			{ 11, 4, 2 },		// 51 bits per transform
			{ 11, 5, 1 },		// 51 bits per transform
			{ 11, 6, 0 },		// 51 bits per transform
			{ 12, 0, 5 },		// 51 bits per transform
			{ 12, 1, 4 },		// 51 bits per transform
			{ 12, 2, 3 },		// 51 bits per transform
			{ 12, 3, 2 },		// 51 bits per transform
			{ 12, 4, 1 },		// 51 bits per transform
			{ 12, 5, 0 },		// 51 bits per transform
			{ 13, 0, 4 },		// 51 bits per transform
			{ 13, 1, 3 },		// 51 bits per transform
			{ 13, 2, 2 },		// 51 bits per transform
			{ 13, 3, 1 },		// 51 bits per transform
			{ 13, 4, 0 },		// 51 bits per transform
			{ 14, 0, 3 },		// 51 bits per transform
			{ 14, 1, 2 },		// 51 bits per transform
			{ 14, 2, 1 },		// 51 bits per transform
			{ 14, 3, 0 },		// 51 bits per transform
			{ 15, 0, 2 },		// 51 bits per transform
			{ 15, 1, 1 },		// 51 bits per transform
			{ 15, 2, 0 },		// 51 bits per transform
			{ 16, 0, 1 },		// 51 bits per transform
			{ 16, 1, 0 },		// 51 bits per transform
			{ 17, 0, 0 },		// 51 bits per transform
			{ 0, 0, 18 },		// 54 bits per transform
			{ 0, 1, 17 },		// 54 bits per transform
			{ 0, 2, 16 },		// 54 bits per transform
			{ 0, 3, 15 },		// 54 bits per transform
			{ 0, 4, 14 },		// 54 bits per transform
			{ 0, 5, 13 },		// 54 bits per transform
			{ 0, 6, 12 },		// 54 bits per transform
			{ 0, 7, 11 },		// 54 bits per transform
			{ 0, 8, 10 },		// 54 bits per transform
			{ 0, 9, 9 },		// 54 bits per transform
			{ 0, 10, 8 },		// 54 bits per transform
			{ 0, 11, 7 },		// 54 bits per transform
			{ 0, 12, 6 },		// 54 bits per transform
			{ 0, 13, 5 },		// 54 bits per transform
			{ 0, 14, 4 },		// 54 bits per transform
			{ 0, 15, 3 },		// 54 bits per transform
			{ 0, 16, 2 },		// 54 bits per transform
			{ 0, 17, 1 },		// 54 bits per transform
			{ 0, 18, 0 },		// 54 bits per transform
			{ 1, 0, 17 },		// 54 bits per transform
			{ 1, 1, 16 },		// 54 bits per transform
			{ 1, 2, 15 },		// 54 bits per transform
			{ 1, 3, 14 },		// 54 bits per transform
			{ 1, 4, 13 },		// 54 bits per transform
			{ 1, 5, 12 },		// 54 bits per transform
			{ 1, 6, 11 },		// 54 bits per transform
			{ 1, 7, 10 },		// 54 bits per transform
			{ 1, 8, 9 },		// 54 bits per transform
			{ 1, 9, 8 },		// 54 bits per transform
			{ 1, 10, 7 },		// 54 bits per transform
			{ 1, 11, 6 },		// 54 bits per transform
			{ 1, 12, 5 },		// 54 bits per transform
			{ 1, 13, 4 },		// 54 bits per transform
			{ 1, 14, 3 },		// 54 bits per transform
			{ 1, 15, 2 },		// 54 bits per transform
			{ 1, 16, 1 },		// 54 bits per transform
			{ 1, 17, 0 },		// 54 bits per transform
			{ 2, 0, 16 },		// 54 bits per transform
			{ 2, 1, 15 },		// 54 bits per transform
			{ 2, 2, 14 },		// 54 bits per transform
			{ 2, 3, 13 },		// 54 bits per transform
			{ 2, 4, 12 },		// 54 bits per transform
			{ 2, 5, 11 },		// 54 bits per transform
			{ 2, 6, 10 },		// 54 bits per transform
			{ 2, 7, 9 },		// 54 bits per transform
			{ 2, 8, 8 },		// 54 bits per transform
			{ 2, 9, 7 },		// 54 bits per transform
			{ 2, 10, 6 },		// 54 bits per transform
			{ 2, 11, 5 },		// 54 bits per transform
			{ 2, 12, 4 },		// 54 bits per transform
			{ 2, 13, 3 },		// 54 bits per transform
			{ 2, 14, 2 },		// 54 bits per transform
			{ 2, 15, 1 },		// 54 bits per transform
			{ 2, 16, 0 },		// 54 bits per transform
			{ 3, 0, 15 },		// 54 bits per transform
			{ 3, 1, 14 },		// 54 bits per transform
			{ 3, 2, 13 },		// 54 bits per transform
			{ 3, 3, 12 },		// 54 bits per transform
			{ 3, 4, 11 },		// 54 bits per transform
			{ 3, 5, 10 },		// 54 bits per transform
			{ 3, 6, 9 },		// 54 bits per transform
			{ 3, 7, 8 },		// 54 bits per transform
			{ 3, 8, 7 },		// 54 bits per transform
			{ 3, 9, 6 },		// 54 bits per transform
			{ 3, 10, 5 },		// 54 bits per transform
			{ 3, 11, 4 },		// 54 bits per transform
			{ 3, 12, 3 },		// 54 bits per transform
			{ 3, 13, 2 },		// 54 bits per transform
			{ 3, 14, 1 },		// 54 bits per transform
			{ 3, 15, 0 },		// 54 bits per transform
			{ 4, 0, 14 },		// 54 bits per transform
			{ 4, 1, 13 },		// 54 bits per transform
			{ 4, 2, 12 },		// 54 bits per transform
			{ 4, 3, 11 },		// 54 bits per transform
			{ 4, 4, 10 },		// 54 bits per transform
			{ 4, 5, 9 },		// 54 bits per transform
			{ 4, 6, 8 },		// 54 bits per transform
			{ 4, 7, 7 },		// 54 bits per transform
			{ 4, 8, 6 },		// 54 bits per transform
			{ 4, 9, 5 },		// 54 bits per transform
			{ 4, 10, 4 },		// 54 bits per transform
			{ 4, 11, 3 },		// 54 bits per transform
			{ 4, 12, 2 },		// 54 bits per transform
			{ 4, 13, 1 },		// 54 bits per transform
			{ 4, 14, 0 },		// 54 bits per transform
			{ 5, 0, 13 },		// 54 bits per transform
			{ 5, 1, 12 },		// 54 bits per transform
			{ 5, 2, 11 },		// 54 bits per transform
			{ 5, 3, 10 },		// 54 bits per transform
			{ 5, 4, 9 },		// 54 bits per transform
			{ 5, 5, 8 },		// 54 bits per transform
			{ 5, 6, 7 },		// 54 bits per transform
			{ 5, 7, 6 },		// 54 bits per transform
			{ 5, 8, 5 },		// 54 bits per transform
			{ 5, 9, 4 },		// 54 bits per transform
			{ 5, 10, 3 },		// 54 bits per transform
			{ 5, 11, 2 },		// 54 bits per transform
			{ 5, 12, 1 },		// 54 bits per transform
			{ 5, 13, 0 },		// 54 bits per transform
			{ 6, 0, 12 },		// 54 bits per transform
			{ 6, 1, 11 },		// 54 bits per transform
			{ 6, 2, 10 },		// 54 bits per transform
			{ 6, 3, 9 },		// 54 bits per transform
			{ 6, 4, 8 },		// 54 bits per transform
			{ 6, 5, 7 },		// 54 bits per transform
			{ 6, 6, 6 },		// 54 bits per transform
			{ 6, 7, 5 },		// 54 bits per transform
			{ 6, 8, 4 },		// 54 bits per transform
			{ 6, 9, 3 },		// 54 bits per transform
			{ 6, 10, 2 },		// 54 bits per transform
			{ 6, 11, 1 },		// 54 bits per transform
			{ 6, 12, 0 },		// 54 bits per transform
			{ 7, 0, 11 },		// 54 bits per transform
			{ 7, 1, 10 },		// 54 bits per transform
			{ 7, 2, 9 },		// 54 bits per transform
			{ 7, 3, 8 },		// 54 bits per transform
			{ 7, 4, 7 },		// 54 bits per transform
			{ 7, 5, 6 },		// 54 bits per transform
			{ 7, 6, 5 },		// 54 bits per transform
			{ 7, 7, 4 },		// 54 bits per transform
			{ 7, 8, 3 },		// 54 bits per transform
			{ 7, 9, 2 },		// 54 bits per transform
			{ 7, 10, 1 },		// 54 bits per transform
			{ 7, 11, 0 },		// 54 bits per transform
			{ 8, 0, 10 },		// 54 bits per transform
			{ 8, 1, 9 },		// 54 bits per transform
			{ 8, 2, 8 },		// 54 bits per transform
			{ 8, 3, 7 },		// 54 bits per transform
			{ 8, 4, 6 },		// 54 bits per transform
			{ 8, 5, 5 },		// 54 bits per transform
			{ 8, 6, 4 },		// 54 bits per transform
			{ 8, 7, 3 },		// 54 bits per transform
			{ 8, 8, 2 },		// 54 bits per transform
			{ 8, 9, 1 },		// 54 bits per transform
			{ 8, 10, 0 },		// 54 bits per transform
			{ 9, 0, 9 },		// 54 bits per transform
			{ 9, 1, 8 },		// 54 bits per transform
			{ 9, 2, 7 },		// 54 bits per transform
			{ 9, 3, 6 },		// 54 bits per transform
			{ 9, 4, 5 },		// 54 bits per transform
			{ 9, 5, 4 },		// 54 bits per transform
			{ 9, 6, 3 },		// 54 bits per transform
			{ 9, 7, 2 },		// 54 bits per transform
			{ 9, 8, 1 },		// 54 bits per transform
			{ 9, 9, 0 },		// 54 bits per transform
			{ 10, 0, 8 },		// 54 bits per transform
			{ 10, 1, 7 },		// 54 bits per transform
			{ 10, 2, 6 },		// 54 bits per transform
			{ 10, 3, 5 },		// 54 bits per transform
			{ 10, 4, 4 },		// 54 bits per transform
			{ 10, 5, 3 },		// 54 bits per transform
			{ 10, 6, 2 },		// 54 bits per transform
			{ 10, 7, 1 },		// 54 bits per transform
			{ 10, 8, 0 },		// 54 bits per transform
			{ 11, 0, 7 },		// 54 bits per transform
			{ 11, 1, 6 },		// 54 bits per transform
			{ 11, 2, 5 },		// 54 bits per transform
			{ 11, 3, 4 },		// 54 bits per transform
			{ 11, 4, 3 },		// 54 bits per transform
			{ 11, 5, 2 },		// 54 bits per transform
			{ 11, 6, 1 },		// 54 bits per transform
			{ 11, 7, 0 },		// 54 bits per transform
			{ 12, 0, 6 },		// 54 bits per transform
			{ 12, 1, 5 },		// 54 bits per transform
			{ 12, 2, 4 },		// 54 bits per transform
			{ 12, 3, 3 },		// 54 bits per transform
			{ 12, 4, 2 },		// 54 bits per transform
			{ 12, 5, 1 },		// 54 bits per transform
			{ 12, 6, 0 },		// 54 bits per transform
			{ 13, 0, 5 },		// 54 bits per transform
			{ 13, 1, 4 },		// 54 bits per transform
			{ 13, 2, 3 },		// 54 bits per transform
			{ 13, 3, 2 },		// 54 bits per transform
			{ 13, 4, 1 },		// 54 bits per transform
			{ 13, 5, 0 },		// 54 bits per transform
			{ 14, 0, 4 },		// 54 bits per transform
			{ 14, 1, 3 },		// 54 bits per transform
			{ 14, 2, 2 },		// 54 bits per transform
			{ 14, 3, 1 },		// 54 bits per transform
			{ 14, 4, 0 },		// 54 bits per transform
			{ 15, 0, 3 },		// 54 bits per transform
			{ 15, 1, 2 },		// 54 bits per transform
			{ 15, 2, 1 },		// 54 bits per transform
			{ 15, 3, 0 },		// 54 bits per transform
			{ 16, 0, 2 },		// 54 bits per transform
			{ 16, 1, 1 },		// 54 bits per transform
			{ 16, 2, 0 },		// 54 bits per transform
			{ 17, 0, 1 },		// 54 bits per transform
			{ 17, 1, 0 },		// 54 bits per transform
			{ 18, 0, 0 },		// 54 bits per transform
			{ 0, 0, 19 },		// 57 bits per transform
			{ 0, 1, 18 },		// 57 bits per transform
			{ 0, 2, 17 },		// 57 bits per transform
			{ 0, 3, 16 },		// 57 bits per transform
			{ 0, 4, 15 },		// 57 bits per transform
			{ 0, 5, 14 },		// 57 bits per transform
			{ 0, 6, 13 },		// 57 bits per transform
			{ 0, 7, 12 },		// 57 bits per transform
			{ 0, 8, 11 },		// 57 bits per transform
			{ 0, 9, 10 },		// 57 bits per transform
			{ 0, 10, 9 },		// 57 bits per transform
			{ 0, 11, 8 },		// 57 bits per transform
			{ 0, 12, 7 },		// 57 bits per transform
			{ 0, 13, 6 },		// 57 bits per transform
			{ 0, 14, 5 },		// 57 bits per transform
			{ 0, 15, 4 },		// 57 bits per transform
			{ 0, 16, 3 },		// 57 bits per transform
			{ 0, 17, 2 },		// 57 bits per transform
			{ 0, 18, 1 },		// 57 bits per transform
			{ 0, 19, 0 },		// 57 bits per transform
			{ 1, 0, 18 },		// 57 bits per transform
			{ 1, 1, 17 },		// 57 bits per transform
			{ 1, 2, 16 },		// 57 bits per transform
			{ 1, 3, 15 },		// 57 bits per transform
			{ 1, 4, 14 },		// 57 bits per transform
			{ 1, 5, 13 },		// 57 bits per transform
			{ 1, 6, 12 },		// 57 bits per transform
			{ 1, 7, 11 },		// 57 bits per transform
			{ 1, 8, 10 },		// 57 bits per transform
			{ 1, 9, 9 },		// 57 bits per transform
			{ 1, 10, 8 },		// 57 bits per transform
			{ 1, 11, 7 },		// 57 bits per transform
			{ 1, 12, 6 },		// 57 bits per transform
			{ 1, 13, 5 },		// 57 bits per transform
			{ 1, 14, 4 },		// 57 bits per transform
			{ 1, 15, 3 },		// 57 bits per transform
			{ 1, 16, 2 },		// 57 bits per transform
			{ 1, 17, 1 },		// 57 bits per transform
			{ 1, 18, 0 },		// 57 bits per transform
			{ 2, 0, 17 },		// 57 bits per transform
			{ 2, 1, 16 },		// 57 bits per transform
			{ 2, 2, 15 },		// 57 bits per transform
			{ 2, 3, 14 },		// 57 bits per transform
			{ 2, 4, 13 },		// 57 bits per transform
			{ 2, 5, 12 },		// 57 bits per transform
			{ 2, 6, 11 },		// 57 bits per transform
			{ 2, 7, 10 },		// 57 bits per transform
			{ 2, 8, 9 },		// 57 bits per transform
			{ 2, 9, 8 },		// 57 bits per transform
			{ 2, 10, 7 },		// 57 bits per transform
			{ 2, 11, 6 },		// 57 bits per transform
			{ 2, 12, 5 },		// 57 bits per transform
			{ 2, 13, 4 },		// 57 bits per transform
			{ 2, 14, 3 },		// 57 bits per transform
			{ 2, 15, 2 },		// 57 bits per transform
			{ 2, 16, 1 },		// 57 bits per transform
			{ 2, 17, 0 },		// 57 bits per transform
			{ 3, 0, 16 },		// 57 bits per transform
			{ 3, 1, 15 },		// 57 bits per transform
			{ 3, 2, 14 },		// 57 bits per transform
			{ 3, 3, 13 },		// 57 bits per transform
			{ 3, 4, 12 },		// 57 bits per transform
			{ 3, 5, 11 },		// 57 bits per transform
			{ 3, 6, 10 },		// 57 bits per transform
			{ 3, 7, 9 },		// 57 bits per transform
			{ 3, 8, 8 },		// 57 bits per transform
			{ 3, 9, 7 },		// 57 bits per transform
			{ 3, 10, 6 },		// 57 bits per transform
			{ 3, 11, 5 },		// 57 bits per transform
			{ 3, 12, 4 },		// 57 bits per transform
			{ 3, 13, 3 },		// 57 bits per transform
			{ 3, 14, 2 },		// 57 bits per transform
			{ 3, 15, 1 },		// 57 bits per transform
			{ 3, 16, 0 },		// 57 bits per transform
			{ 4, 0, 15 },		// 57 bits per transform
			{ 4, 1, 14 },		// 57 bits per transform
			{ 4, 2, 13 },		// 57 bits per transform
			{ 4, 3, 12 },		// 57 bits per transform
			{ 4, 4, 11 },		// 57 bits per transform
			{ 4, 5, 10 },		// 57 bits per transform
			{ 4, 6, 9 },		// 57 bits per transform
			{ 4, 7, 8 },		// 57 bits per transform
			{ 4, 8, 7 },		// 57 bits per transform
			{ 4, 9, 6 },		// 57 bits per transform
			{ 4, 10, 5 },		// 57 bits per transform
			{ 4, 11, 4 },		// 57 bits per transform
			{ 4, 12, 3 },		// 57 bits per transform
			{ 4, 13, 2 },		// 57 bits per transform
			{ 4, 14, 1 },		// 57 bits per transform
			{ 4, 15, 0 },		// 57 bits per transform
			{ 5, 0, 14 },		// 57 bits per transform
			{ 5, 1, 13 },		// 57 bits per transform
			{ 5, 2, 12 },		// 57 bits per transform
			{ 5, 3, 11 },		// 57 bits per transform
			{ 5, 4, 10 },		// 57 bits per transform
			{ 5, 5, 9 },		// 57 bits per transform
			{ 5, 6, 8 },		// 57 bits per transform
			{ 5, 7, 7 },		// 57 bits per transform
			{ 5, 8, 6 },		// 57 bits per transform
			{ 5, 9, 5 },		// 57 bits per transform
			{ 5, 10, 4 },		// 57 bits per transform
			{ 5, 11, 3 },		// 57 bits per transform
			{ 5, 12, 2 },		// 57 bits per transform
			{ 5, 13, 1 },		// 57 bits per transform
			{ 5, 14, 0 },		// 57 bits per transform
			{ 6, 0, 13 },		// 57 bits per transform
			{ 6, 1, 12 },		// 57 bits per transform
			{ 6, 2, 11 },		// 57 bits per transform
			{ 6, 3, 10 },		// 57 bits per transform
			{ 6, 4, 9 },		// 57 bits per transform
			{ 6, 5, 8 },		// 57 bits per transform
			{ 6, 6, 7 },		// 57 bits per transform
			{ 6, 7, 6 },		// 57 bits per transform
			{ 6, 8, 5 },		// 57 bits per transform
			{ 6, 9, 4 },		// 57 bits per transform
			{ 6, 10, 3 },		// 57 bits per transform
			{ 6, 11, 2 },		// 57 bits per transform
			{ 6, 12, 1 },		// 57 bits per transform
			{ 6, 13, 0 },		// 57 bits per transform
			{ 7, 0, 12 },		// 57 bits per transform
			{ 7, 1, 11 },		// 57 bits per transform
			{ 7, 2, 10 },		// 57 bits per transform
			{ 7, 3, 9 },		// 57 bits per transform
			{ 7, 4, 8 },		// 57 bits per transform
			{ 7, 5, 7 },		// 57 bits per transform
			{ 7, 6, 6 },		// 57 bits per transform
			{ 7, 7, 5 },		// 57 bits per transform
			{ 7, 8, 4 },		// 57 bits per transform
			{ 7, 9, 3 },		// 57 bits per transform
			{ 7, 10, 2 },		// 57 bits per transform
			{ 7, 11, 1 },		// 57 bits per transform
			{ 7, 12, 0 },		// 57 bits per transform
			{ 8, 0, 11 },		// 57 bits per transform
			{ 8, 1, 10 },		// 57 bits per transform
			{ 8, 2, 9 },		// 57 bits per transform
			{ 8, 3, 8 },		// 57 bits per transform
			{ 8, 4, 7 },		// 57 bits per transform
			{ 8, 5, 6 },		// 57 bits per transform
			{ 8, 6, 5 },		// 57 bits per transform
			{ 8, 7, 4 },		// 57 bits per transform
			{ 8, 8, 3 },		// 57 bits per transform
			{ 8, 9, 2 },		// 57 bits per transform
			{ 8, 10, 1 },		// 57 bits per transform
			{ 8, 11, 0 },		// 57 bits per transform
			{ 9, 0, 10 },		// 57 bits per transform
			{ 9, 1, 9 },		// 57 bits per transform
			{ 9, 2, 8 },		// 57 bits per transform
			{ 9, 3, 7 },		// 57 bits per transform
			{ 9, 4, 6 },		// 57 bits per transform
			{ 9, 5, 5 },		// 57 bits per transform
			{ 9, 6, 4 },		// 57 bits per transform
			{ 9, 7, 3 },		// 57 bits per transform
			{ 9, 8, 2 },		// 57 bits per transform
			{ 9, 9, 1 },		// 57 bits per transform
			{ 9, 10, 0 },		// 57 bits per transform
			{ 10, 0, 9 },		// 57 bits per transform
			{ 10, 1, 8 },		// 57 bits per transform
			{ 10, 2, 7 },		// 57 bits per transform
			{ 10, 3, 6 },		// 57 bits per transform
			{ 10, 4, 5 },		// 57 bits per transform
			{ 10, 5, 4 },		// 57 bits per transform
			{ 10, 6, 3 },		// 57 bits per transform
			{ 10, 7, 2 },		// 57 bits per transform
			{ 10, 8, 1 },		// 57 bits per transform
			{ 10, 9, 0 },		// 57 bits per transform
			{ 11, 0, 8 },		// 57 bits per transform
			{ 11, 1, 7 },		// 57 bits per transform
			{ 11, 2, 6 },		// 57 bits per transform
			{ 11, 3, 5 },		// 57 bits per transform
			{ 11, 4, 4 },		// 57 bits per transform
			{ 11, 5, 3 },		// 57 bits per transform
			{ 11, 6, 2 },		// 57 bits per transform
			{ 11, 7, 1 },		// 57 bits per transform
			{ 11, 8, 0 },		// 57 bits per transform
			{ 12, 0, 7 },		// 57 bits per transform
			{ 12, 1, 6 },		// 57 bits per transform
			{ 12, 2, 5 },		// 57 bits per transform
			{ 12, 3, 4 },		// 57 bits per transform
			{ 12, 4, 3 },		// 57 bits per transform
			{ 12, 5, 2 },		// 57 bits per transform
			{ 12, 6, 1 },		// 57 bits per transform
			{ 12, 7, 0 },		// 57 bits per transform
			{ 13, 0, 6 },		// 57 bits per transform
			{ 13, 1, 5 },		// 57 bits per transform
			{ 13, 2, 4 },		// 57 bits per transform
			{ 13, 3, 3 },		// 57 bits per transform
			{ 13, 4, 2 },		// 57 bits per transform
			{ 13, 5, 1 },		// 57 bits per transform
			{ 13, 6, 0 },		// 57 bits per transform
			{ 14, 0, 5 },		// 57 bits per transform
			{ 14, 1, 4 },		// 57 bits per transform
			{ 14, 2, 3 },		// 57 bits per transform
			{ 14, 3, 2 },		// 57 bits per transform
			{ 14, 4, 1 },		// 57 bits per transform
			{ 14, 5, 0 },		// 57 bits per transform
			{ 15, 0, 4 },		// 57 bits per transform
			{ 15, 1, 3 },		// 57 bits per transform
			{ 15, 2, 2 },		// 57 bits per transform
			{ 15, 3, 1 },		// 57 bits per transform
			{ 15, 4, 0 },		// 57 bits per transform
			{ 16, 0, 3 },		// 57 bits per transform
			{ 16, 1, 2 },		// 57 bits per transform
			{ 16, 2, 1 },		// 57 bits per transform
			{ 16, 3, 0 },		// 57 bits per transform
			{ 17, 0, 2 },		// 57 bits per transform
			{ 17, 1, 1 },		// 57 bits per transform
			{ 17, 2, 0 },		// 57 bits per transform
			{ 18, 0, 1 },		// 57 bits per transform
			{ 18, 1, 0 },		// 57 bits per transform
			{ 19, 0, 0 },		// 57 bits per transform
			{ 0, 0, 20 },		// 60 bits per transform
			{ 0, 1, 19 },		// 60 bits per transform
			{ 0, 2, 18 },		// 60 bits per transform
			{ 0, 3, 17 },		// 60 bits per transform
			{ 0, 4, 16 },		// 60 bits per transform
			{ 0, 5, 15 },		// 60 bits per transform
			{ 0, 6, 14 },		// 60 bits per transform
			{ 0, 7, 13 },		// 60 bits per transform
			{ 0, 8, 12 },		// 60 bits per transform
			{ 0, 9, 11 },		// 60 bits per transform
			{ 0, 10, 10 },		// 60 bits per transform
			{ 0, 11, 9 },		// 60 bits per transform
			{ 0, 12, 8 },		// 60 bits per transform
			{ 0, 13, 7 },		// 60 bits per transform
			{ 0, 14, 6 },		// 60 bits per transform
			{ 0, 15, 5 },		// 60 bits per transform
			{ 0, 16, 4 },		// 60 bits per transform
			{ 0, 17, 3 },		// 60 bits per transform
			{ 0, 18, 2 },		// 60 bits per transform
			{ 0, 19, 1 },		// 60 bits per transform
			{ 0, 20, 0 },		// 60 bits per transform
			{ 1, 0, 19 },		// 60 bits per transform
			{ 1, 1, 18 },		// 60 bits per transform
			{ 1, 2, 17 },		// 60 bits per transform
			{ 1, 3, 16 },		// 60 bits per transform
			{ 1, 4, 15 },		// 60 bits per transform
			{ 1, 5, 14 },		// 60 bits per transform
			{ 1, 6, 13 },		// 60 bits per transform
			{ 1, 7, 12 },		// 60 bits per transform
			{ 1, 8, 11 },		// 60 bits per transform
			{ 1, 9, 10 },		// 60 bits per transform
			{ 1, 10, 9 },		// 60 bits per transform
			{ 1, 11, 8 },		// 60 bits per transform
			{ 1, 12, 7 },		// 60 bits per transform
			{ 1, 13, 6 },		// 60 bits per transform
			{ 1, 14, 5 },		// 60 bits per transform
			{ 1, 15, 4 },		// 60 bits per transform
			{ 1, 16, 3 },		// 60 bits per transform
			{ 1, 17, 2 },		// 60 bits per transform
			{ 1, 18, 1 },		// 60 bits per transform
			{ 1, 19, 0 },		// 60 bits per transform
			{ 2, 0, 18 },		// 60 bits per transform
			{ 2, 1, 17 },		// 60 bits per transform
			{ 2, 2, 16 },		// 60 bits per transform
			{ 2, 3, 15 },		// 60 bits per transform
			{ 2, 4, 14 },		// 60 bits per transform
			{ 2, 5, 13 },		// 60 bits per transform
			{ 2, 6, 12 },		// 60 bits per transform
			{ 2, 7, 11 },		// 60 bits per transform
			{ 2, 8, 10 },		// 60 bits per transform
			{ 2, 9, 9 },		// 60 bits per transform
			{ 2, 10, 8 },		// 60 bits per transform
			{ 2, 11, 7 },		// 60 bits per transform
			{ 2, 12, 6 },		// 60 bits per transform
			{ 2, 13, 5 },		// 60 bits per transform
			{ 2, 14, 4 },		// 60 bits per transform
			{ 2, 15, 3 },		// 60 bits per transform
			{ 2, 16, 2 },		// 60 bits per transform
			{ 2, 17, 1 },		// 60 bits per transform
			{ 2, 18, 0 },		// 60 bits per transform
			{ 3, 0, 17 },		// 60 bits per transform
			{ 3, 1, 16 },		// 60 bits per transform
			{ 3, 2, 15 },		// 60 bits per transform
			{ 3, 3, 14 },		// 60 bits per transform
			{ 3, 4, 13 },		// 60 bits per transform
			{ 3, 5, 12 },		// 60 bits per transform
			{ 3, 6, 11 },		// 60 bits per transform
			{ 3, 7, 10 },		// 60 bits per transform
			{ 3, 8, 9 },		// 60 bits per transform
			{ 3, 9, 8 },		// 60 bits per transform
			{ 3, 10, 7 },		// 60 bits per transform
			{ 3, 11, 6 },		// 60 bits per transform
			{ 3, 12, 5 },		// 60 bits per transform
			{ 3, 13, 4 },		// 60 bits per transform
			{ 3, 14, 3 },		// 60 bits per transform
			{ 3, 15, 2 },		// 60 bits per transform
			{ 3, 16, 1 },		// 60 bits per transform
			{ 3, 17, 0 },		// 60 bits per transform
			{ 4, 0, 16 },		// 60 bits per transform
			{ 4, 1, 15 },		// 60 bits per transform
			{ 4, 2, 14 },		// 60 bits per transform
			{ 4, 3, 13 },		// 60 bits per transform
			{ 4, 4, 12 },		// 60 bits per transform
			{ 4, 5, 11 },		// 60 bits per transform
			{ 4, 6, 10 },		// 60 bits per transform
			{ 4, 7, 9 },		// 60 bits per transform
			{ 4, 8, 8 },		// 60 bits per transform
			{ 4, 9, 7 },		// 60 bits per transform
			{ 4, 10, 6 },		// 60 bits per transform
			{ 4, 11, 5 },		// 60 bits per transform
			{ 4, 12, 4 },		// 60 bits per transform
			{ 4, 13, 3 },		// 60 bits per transform
			{ 4, 14, 2 },		// 60 bits per transform
			{ 4, 15, 1 },		// 60 bits per transform
			{ 4, 16, 0 },		// 60 bits per transform
			{ 5, 0, 15 },		// 60 bits per transform
			{ 5, 1, 14 },		// 60 bits per transform
			{ 5, 2, 13 },		// 60 bits per transform
			{ 5, 3, 12 },		// 60 bits per transform
			{ 5, 4, 11 },		// 60 bits per transform
			{ 5, 5, 10 },		// 60 bits per transform
			{ 5, 6, 9 },		// 60 bits per transform
			{ 5, 7, 8 },		// 60 bits per transform
			{ 5, 8, 7 },		// 60 bits per transform
			{ 5, 9, 6 },		// 60 bits per transform
			{ 5, 10, 5 },		// 60 bits per transform
			{ 5, 11, 4 },		// 60 bits per transform
			{ 5, 12, 3 },		// 60 bits per transform
			{ 5, 13, 2 },		// 60 bits per transform
			{ 5, 14, 1 },		// 60 bits per transform
			{ 5, 15, 0 },		// 60 bits per transform
			{ 6, 0, 14 },		// 60 bits per transform
			{ 6, 1, 13 },		// 60 bits per transform
			{ 6, 2, 12 },		// 60 bits per transform
			{ 6, 3, 11 },		// 60 bits per transform
			{ 6, 4, 10 },		// 60 bits per transform
			{ 6, 5, 9 },		// 60 bits per transform
			{ 6, 6, 8 },		// 60 bits per transform
			{ 6, 7, 7 },		// 60 bits per transform
			{ 6, 8, 6 },		// 60 bits per transform
			{ 6, 9, 5 },		// 60 bits per transform
			{ 6, 10, 4 },		// 60 bits per transform
			{ 6, 11, 3 },		// 60 bits per transform
			{ 6, 12, 2 },		// 60 bits per transform
			{ 6, 13, 1 },		// 60 bits per transform
			{ 6, 14, 0 },		// 60 bits per transform
			{ 7, 0, 13 },		// 60 bits per transform
			{ 7, 1, 12 },		// 60 bits per transform
			{ 7, 2, 11 },		// 60 bits per transform
			{ 7, 3, 10 },		// 60 bits per transform
			{ 7, 4, 9 },		// 60 bits per transform
			{ 7, 5, 8 },		// 60 bits per transform
			{ 7, 6, 7 },		// 60 bits per transform
			{ 7, 7, 6 },		// 60 bits per transform
			{ 7, 8, 5 },		// 60 bits per transform
			{ 7, 9, 4 },		// 60 bits per transform
			{ 7, 10, 3 },		// 60 bits per transform
			{ 7, 11, 2 },		// 60 bits per transform
			{ 7, 12, 1 },		// 60 bits per transform
			{ 7, 13, 0 },		// 60 bits per transform
			{ 8, 0, 12 },		// 60 bits per transform
			{ 8, 1, 11 },		// 60 bits per transform
			{ 8, 2, 10 },		// 60 bits per transform
			{ 8, 3, 9 },		// 60 bits per transform
			{ 8, 4, 8 },		// 60 bits per transform
			{ 8, 5, 7 },		// 60 bits per transform
			{ 8, 6, 6 },		// 60 bits per transform
			{ 8, 7, 5 },		// 60 bits per transform
			{ 8, 8, 4 },		// 60 bits per transform
			{ 8, 9, 3 },		// 60 bits per transform
			{ 8, 10, 2 },		// 60 bits per transform
			{ 8, 11, 1 },		// 60 bits per transform
			{ 8, 12, 0 },		// 60 bits per transform
			{ 9, 0, 11 },		// 60 bits per transform
			{ 9, 1, 10 },		// 60 bits per transform
			{ 9, 2, 9 },		// 60 bits per transform
			{ 9, 3, 8 },		// 60 bits per transform
			{ 9, 4, 7 },		// 60 bits per transform
			{ 9, 5, 6 },		// 60 bits per transform
			{ 9, 6, 5 },		// 60 bits per transform
			{ 9, 7, 4 },		// 60 bits per transform
			{ 9, 8, 3 },		// 60 bits per transform
			{ 9, 9, 2 },		// 60 bits per transform
			{ 9, 10, 1 },		// 60 bits per transform
			{ 9, 11, 0 },		// 60 bits per transform
			{ 10, 0, 10 },		// 60 bits per transform
			{ 10, 1, 9 },		// 60 bits per transform
			{ 10, 2, 8 },		// 60 bits per transform
			{ 10, 3, 7 },		// 60 bits per transform
			{ 10, 4, 6 },		// 60 bits per transform
			{ 10, 5, 5 },		// 60 bits per transform
			{ 10, 6, 4 },		// 60 bits per transform
			{ 10, 7, 3 },		// 60 bits per transform
			{ 10, 8, 2 },		// 60 bits per transform
			{ 10, 9, 1 },		// 60 bits per transform
			{ 10, 10, 0 },		// 60 bits per transform
			{ 11, 0, 9 },		// 60 bits per transform
			{ 11, 1, 8 },		// 60 bits per transform
			{ 11, 2, 7 },		// 60 bits per transform
			{ 11, 3, 6 },		// 60 bits per transform
			{ 11, 4, 5 },		// 60 bits per transform
			{ 11, 5, 4 },		// 60 bits per transform
			{ 11, 6, 3 },		// 60 bits per transform
			{ 11, 7, 2 },		// 60 bits per transform
			{ 11, 8, 1 },		// 60 bits per transform
			{ 11, 9, 0 },		// 60 bits per transform
			{ 12, 0, 8 },		// 60 bits per transform
			{ 12, 1, 7 },		// 60 bits per transform
			{ 12, 2, 6 },		// 60 bits per transform
			{ 12, 3, 5 },		// 60 bits per transform
			{ 12, 4, 4 },		// 60 bits per transform
			{ 12, 5, 3 },		// 60 bits per transform
			{ 12, 6, 2 },		// 60 bits per transform
			{ 12, 7, 1 },		// 60 bits per transform
			{ 12, 8, 0 },		// 60 bits per transform
			{ 13, 0, 7 },		// 60 bits per transform
			{ 13, 1, 6 },		// 60 bits per transform
			{ 13, 2, 5 },		// 60 bits per transform
			{ 13, 3, 4 },		// 60 bits per transform
			{ 13, 4, 3 },		// 60 bits per transform
			{ 13, 5, 2 },		// 60 bits per transform
			{ 13, 6, 1 },		// 60 bits per transform
			{ 13, 7, 0 },		// 60 bits per transform
			{ 14, 0, 6 },		// 60 bits per transform
			{ 14, 1, 5 },		// 60 bits per transform
			{ 14, 2, 4 },		// 60 bits per transform
			{ 14, 3, 3 },		// 60 bits per transform
			{ 14, 4, 2 },		// 60 bits per transform
			{ 14, 5, 1 },		// 60 bits per transform
			{ 14, 6, 0 },		// 60 bits per transform
			{ 15, 0, 5 },		// 60 bits per transform
			{ 15, 1, 4 },		// 60 bits per transform
			{ 15, 2, 3 },		// 60 bits per transform
			{ 15, 3, 2 },		// 60 bits per transform
			{ 15, 4, 1 },		// 60 bits per transform
			{ 15, 5, 0 },		// 60 bits per transform
			{ 16, 0, 4 },		// 60 bits per transform
			{ 16, 1, 3 },		// 60 bits per transform
			{ 16, 2, 2 },		// 60 bits per transform
			{ 16, 3, 1 },		// 60 bits per transform
			{ 16, 4, 0 },		// 60 bits per transform
			{ 17, 0, 3 },		// 60 bits per transform
			{ 17, 1, 2 },		// 60 bits per transform
			{ 17, 2, 1 },		// 60 bits per transform
			{ 17, 3, 0 },		// 60 bits per transform
			{ 18, 0, 2 },		// 60 bits per transform
			{ 18, 1, 1 },		// 60 bits per transform
			{ 18, 2, 0 },		// 60 bits per transform
			{ 19, 0, 1 },		// 60 bits per transform
			{ 19, 1, 0 },		// 60 bits per transform
			{ 20, 0, 0 },		// 60 bits per transform
			{ 0, 0, 21 },		// 63 bits per transform
			{ 0, 1, 20 },		// 63 bits per transform
			{ 0, 2, 19 },		// 63 bits per transform
			{ 0, 3, 18 },		// 63 bits per transform
			{ 0, 4, 17 },		// 63 bits per transform
			{ 0, 5, 16 },		// 63 bits per transform
			{ 0, 6, 15 },		// 63 bits per transform
			{ 0, 7, 14 },		// 63 bits per transform
			{ 0, 8, 13 },		// 63 bits per transform
			{ 0, 9, 12 },		// 63 bits per transform
			{ 0, 10, 11 },		// 63 bits per transform
			{ 0, 11, 10 },		// 63 bits per transform
			{ 0, 12, 9 },		// 63 bits per transform
			{ 0, 13, 8 },		// 63 bits per transform
			{ 0, 14, 7 },		// 63 bits per transform
			{ 0, 15, 6 },		// 63 bits per transform
			{ 0, 16, 5 },		// 63 bits per transform
			{ 0, 17, 4 },		// 63 bits per transform
			{ 0, 18, 3 },		// 63 bits per transform
			{ 0, 19, 2 },		// 63 bits per transform
			{ 0, 20, 1 },		// 63 bits per transform
			{ 0, 21, 0 },		// 63 bits per transform
			{ 1, 0, 20 },		// 63 bits per transform
			{ 1, 1, 19 },		// 63 bits per transform
			{ 1, 2, 18 },		// 63 bits per transform
			{ 1, 3, 17 },		// 63 bits per transform
			{ 1, 4, 16 },		// 63 bits per transform
			{ 1, 5, 15 },		// 63 bits per transform
			{ 1, 6, 14 },		// 63 bits per transform
			{ 1, 7, 13 },		// 63 bits per transform
			{ 1, 8, 12 },		// 63 bits per transform
			{ 1, 9, 11 },		// 63 bits per transform
			{ 1, 10, 10 },		// 63 bits per transform
			{ 1, 11, 9 },		// 63 bits per transform
			{ 1, 12, 8 },		// 63 bits per transform
			{ 1, 13, 7 },		// 63 bits per transform
			{ 1, 14, 6 },		// 63 bits per transform
			{ 1, 15, 5 },		// 63 bits per transform
			{ 1, 16, 4 },		// 63 bits per transform
			{ 1, 17, 3 },		// 63 bits per transform
			{ 1, 18, 2 },		// 63 bits per transform
			{ 1, 19, 1 },		// 63 bits per transform
			{ 1, 20, 0 },		// 63 bits per transform
			{ 2, 0, 19 },		// 63 bits per transform
			{ 2, 1, 18 },		// 63 bits per transform
			{ 2, 2, 17 },		// 63 bits per transform
			{ 2, 3, 16 },		// 63 bits per transform
			{ 2, 4, 15 },		// 63 bits per transform
			{ 2, 5, 14 },		// 63 bits per transform
			{ 2, 6, 13 },		// 63 bits per transform
			{ 2, 7, 12 },		// 63 bits per transform
			{ 2, 8, 11 },		// 63 bits per transform
			{ 2, 9, 10 },		// 63 bits per transform
			{ 2, 10, 9 },		// 63 bits per transform
			{ 2, 11, 8 },		// 63 bits per transform
			{ 2, 12, 7 },		// 63 bits per transform
			{ 2, 13, 6 },		// 63 bits per transform
			{ 2, 14, 5 },		// 63 bits per transform
			{ 2, 15, 4 },		// 63 bits per transform
			{ 2, 16, 3 },		// 63 bits per transform
			{ 2, 17, 2 },		// 63 bits per transform
			{ 2, 18, 1 },		// 63 bits per transform
			{ 2, 19, 0 },		// 63 bits per transform
			{ 3, 0, 18 },		// 63 bits per transform
			{ 3, 1, 17 },		// 63 bits per transform
			{ 3, 2, 16 },		// 63 bits per transform
			{ 3, 3, 15 },		// 63 bits per transform
			{ 3, 4, 14 },		// 63 bits per transform
			{ 3, 5, 13 },		// 63 bits per transform
			{ 3, 6, 12 },		// 63 bits per transform
			{ 3, 7, 11 },		// 63 bits per transform
			{ 3, 8, 10 },		// 63 bits per transform
			{ 3, 9, 9 },		// 63 bits per transform
			{ 3, 10, 8 },		// 63 bits per transform
			{ 3, 11, 7 },		// 63 bits per transform
			{ 3, 12, 6 },		// 63 bits per transform
			{ 3, 13, 5 },		// 63 bits per transform
			{ 3, 14, 4 },		// 63 bits per transform
			{ 3, 15, 3 },		// 63 bits per transform
			{ 3, 16, 2 },		// 63 bits per transform
			{ 3, 17, 1 },		// 63 bits per transform
			{ 3, 18, 0 },		// 63 bits per transform
			{ 4, 0, 17 },		// 63 bits per transform
			{ 4, 1, 16 },		// 63 bits per transform
			{ 4, 2, 15 },		// 63 bits per transform
			{ 4, 3, 14 },		// 63 bits per transform
			{ 4, 4, 13 },		// 63 bits per transform
			{ 4, 5, 12 },		// 63 bits per transform
			{ 4, 6, 11 },		// 63 bits per transform
			{ 4, 7, 10 },		// 63 bits per transform
			{ 4, 8, 9 },		// 63 bits per transform
			{ 4, 9, 8 },		// 63 bits per transform
			{ 4, 10, 7 },		// 63 bits per transform
			{ 4, 11, 6 },		// 63 bits per transform
			{ 4, 12, 5 },		// 63 bits per transform
			{ 4, 13, 4 },		// 63 bits per transform
			{ 4, 14, 3 },		// 63 bits per transform
			{ 4, 15, 2 },		// 63 bits per transform
			{ 4, 16, 1 },		// 63 bits per transform
			{ 4, 17, 0 },		// 63 bits per transform
			{ 5, 0, 16 },		// 63 bits per transform
			{ 5, 1, 15 },		// 63 bits per transform
			{ 5, 2, 14 },		// 63 bits per transform
			{ 5, 3, 13 },		// 63 bits per transform
			{ 5, 4, 12 },		// 63 bits per transform
			{ 5, 5, 11 },		// 63 bits per transform
			{ 5, 6, 10 },		// 63 bits per transform
			{ 5, 7, 9 },		// 63 bits per transform
			{ 5, 8, 8 },		// 63 bits per transform
			{ 5, 9, 7 },		// 63 bits per transform
			{ 5, 10, 6 },		// 63 bits per transform
			{ 5, 11, 5 },		// 63 bits per transform
			{ 5, 12, 4 },		// 63 bits per transform
			{ 5, 13, 3 },		// 63 bits per transform
			{ 5, 14, 2 },		// 63 bits per transform
			{ 5, 15, 1 },		// 63 bits per transform
			{ 5, 16, 0 },		// 63 bits per transform
			{ 6, 0, 15 },		// 63 bits per transform
			{ 6, 1, 14 },		// 63 bits per transform
			{ 6, 2, 13 },		// 63 bits per transform
			{ 6, 3, 12 },		// 63 bits per transform
			{ 6, 4, 11 },		// 63 bits per transform
			{ 6, 5, 10 },		// 63 bits per transform
			{ 6, 6, 9 },		// 63 bits per transform
			{ 6, 7, 8 },		// 63 bits per transform
			{ 6, 8, 7 },		// 63 bits per transform
			{ 6, 9, 6 },		// 63 bits per transform
			{ 6, 10, 5 },		// 63 bits per transform
			{ 6, 11, 4 },		// 63 bits per transform
			{ 6, 12, 3 },		// 63 bits per transform
			{ 6, 13, 2 },		// 63 bits per transform
			{ 6, 14, 1 },		// 63 bits per transform
			{ 6, 15, 0 },		// 63 bits per transform
			{ 7, 0, 14 },		// 63 bits per transform
			{ 7, 1, 13 },		// 63 bits per transform
			{ 7, 2, 12 },		// 63 bits per transform
			{ 7, 3, 11 },		// 63 bits per transform
			{ 7, 4, 10 },		// 63 bits per transform
			{ 7, 5, 9 },		// 63 bits per transform
			{ 7, 6, 8 },		// 63 bits per transform
			{ 7, 7, 7 },		// 63 bits per transform
			{ 7, 8, 6 },		// 63 bits per transform
			{ 7, 9, 5 },		// 63 bits per transform
			{ 7, 10, 4 },		// 63 bits per transform
			{ 7, 11, 3 },		// 63 bits per transform
			{ 7, 12, 2 },		// 63 bits per transform
			{ 7, 13, 1 },		// 63 bits per transform
			{ 7, 14, 0 },		// 63 bits per transform
			{ 8, 0, 13 },		// 63 bits per transform
			{ 8, 1, 12 },		// 63 bits per transform
			{ 8, 2, 11 },		// 63 bits per transform
			{ 8, 3, 10 },		// 63 bits per transform
			{ 8, 4, 9 },		// 63 bits per transform
			{ 8, 5, 8 },		// 63 bits per transform
			{ 8, 6, 7 },		// 63 bits per transform
			{ 8, 7, 6 },		// 63 bits per transform
			{ 8, 8, 5 },		// 63 bits per transform
			{ 8, 9, 4 },		// 63 bits per transform
			{ 8, 10, 3 },		// 63 bits per transform
			{ 8, 11, 2 },		// 63 bits per transform
			{ 8, 12, 1 },		// 63 bits per transform
			{ 8, 13, 0 },		// 63 bits per transform
			{ 9, 0, 12 },		// 63 bits per transform
			{ 9, 1, 11 },		// 63 bits per transform
			{ 9, 2, 10 },		// 63 bits per transform
			{ 9, 3, 9 },		// 63 bits per transform
			{ 9, 4, 8 },		// 63 bits per transform
			{ 9, 5, 7 },		// 63 bits per transform
			{ 9, 6, 6 },		// 63 bits per transform
			{ 9, 7, 5 },		// 63 bits per transform
			{ 9, 8, 4 },		// 63 bits per transform
			{ 9, 9, 3 },		// 63 bits per transform
			{ 9, 10, 2 },		// 63 bits per transform
			{ 9, 11, 1 },		// 63 bits per transform
			{ 9, 12, 0 },		// 63 bits per transform
			{ 10, 0, 11 },		// 63 bits per transform
			{ 10, 1, 10 },		// 63 bits per transform
			{ 10, 2, 9 },		// 63 bits per transform
			{ 10, 3, 8 },		// 63 bits per transform
			{ 10, 4, 7 },		// 63 bits per transform
			{ 10, 5, 6 },		// 63 bits per transform
			{ 10, 6, 5 },		// 63 bits per transform
			{ 10, 7, 4 },		// 63 bits per transform
			{ 10, 8, 3 },		// 63 bits per transform
			{ 10, 9, 2 },		// 63 bits per transform
			{ 10, 10, 1 },		// 63 bits per transform
			{ 10, 11, 0 },		// 63 bits per transform
			{ 11, 0, 10 },		// 63 bits per transform
			{ 11, 1, 9 },		// 63 bits per transform
			{ 11, 2, 8 },		// 63 bits per transform
			{ 11, 3, 7 },		// 63 bits per transform
			{ 11, 4, 6 },		// 63 bits per transform
			{ 11, 5, 5 },		// 63 bits per transform
			{ 11, 6, 4 },		// 63 bits per transform
			{ 11, 7, 3 },		// 63 bits per transform
			{ 11, 8, 2 },		// 63 bits per transform
			{ 11, 9, 1 },		// 63 bits per transform
			{ 11, 10, 0 },		// 63 bits per transform
			{ 12, 0, 9 },		// 63 bits per transform
			{ 12, 1, 8 },		// 63 bits per transform
			{ 12, 2, 7 },		// 63 bits per transform
			{ 12, 3, 6 },		// 63 bits per transform
			{ 12, 4, 5 },		// 63 bits per transform
			{ 12, 5, 4 },		// 63 bits per transform
			{ 12, 6, 3 },		// 63 bits per transform
			{ 12, 7, 2 },		// 63 bits per transform
			{ 12, 8, 1 },		// 63 bits per transform
			{ 12, 9, 0 },		// 63 bits per transform
			{ 13, 0, 8 },		// 63 bits per transform
			{ 13, 1, 7 },		// 63 bits per transform
			{ 13, 2, 6 },		// 63 bits per transform
			{ 13, 3, 5 },		// 63 bits per transform
			{ 13, 4, 4 },		// 63 bits per transform
			{ 13, 5, 3 },		// 63 bits per transform
			{ 13, 6, 2 },		// 63 bits per transform
			{ 13, 7, 1 },		// 63 bits per transform
			{ 13, 8, 0 },		// 63 bits per transform
			{ 14, 0, 7 },		// 63 bits per transform
			{ 14, 1, 6 },		// 63 bits per transform
			{ 14, 2, 5 },		// 63 bits per transform
			{ 14, 3, 4 },		// 63 bits per transform
			{ 14, 4, 3 },		// 63 bits per transform
			{ 14, 5, 2 },		// 63 bits per transform
			{ 14, 6, 1 },		// 63 bits per transform
			{ 14, 7, 0 },		// 63 bits per transform
			{ 15, 0, 6 },		// 63 bits per transform
			{ 15, 1, 5 },		// 63 bits per transform
			{ 15, 2, 4 },		// 63 bits per transform
			{ 15, 3, 3 },		// 63 bits per transform
			{ 15, 4, 2 },		// 63 bits per transform
			{ 15, 5, 1 },		// 63 bits per transform
			{ 15, 6, 0 },		// 63 bits per transform
			{ 16, 0, 5 },		// 63 bits per transform
			{ 16, 1, 4 },		// 63 bits per transform
			{ 16, 2, 3 },		// 63 bits per transform
			{ 16, 3, 2 },		// 63 bits per transform
			{ 16, 4, 1 },		// 63 bits per transform
			{ 16, 5, 0 },		// 63 bits per transform
			{ 17, 0, 4 },		// 63 bits per transform
			{ 17, 1, 3 },		// 63 bits per transform
			{ 17, 2, 2 },		// 63 bits per transform
			{ 17, 3, 1 },		// 63 bits per transform
			{ 17, 4, 0 },		// 63 bits per transform
			{ 18, 0, 3 },		// 63 bits per transform
			{ 18, 1, 2 },		// 63 bits per transform
			{ 18, 2, 1 },		// 63 bits per transform
			{ 18, 3, 0 },		// 63 bits per transform
			{ 19, 0, 2 },		// 63 bits per transform
			{ 19, 1, 1 },		// 63 bits per transform
			{ 19, 2, 0 },		// 63 bits per transform
			{ 20, 0, 1 },		// 63 bits per transform
			{ 20, 1, 0 },		// 63 bits per transform
			{ 21, 0, 0 },		// 63 bits per transform
			{ 0, 0, 22 },		// 66 bits per transform
			{ 0, 1, 21 },		// 66 bits per transform
			{ 0, 2, 20 },		// 66 bits per transform
			{ 0, 3, 19 },		// 66 bits per transform
			{ 0, 4, 18 },		// 66 bits per transform
			{ 0, 5, 17 },		// 66 bits per transform
			{ 0, 6, 16 },		// 66 bits per transform
			{ 0, 7, 15 },		// 66 bits per transform
			{ 0, 8, 14 },		// 66 bits per transform
			{ 0, 9, 13 },		// 66 bits per transform
			{ 0, 10, 12 },		// 66 bits per transform
			{ 0, 11, 11 },		// 66 bits per transform
			{ 0, 12, 10 },		// 66 bits per transform
			{ 0, 13, 9 },		// 66 bits per transform
			{ 0, 14, 8 },		// 66 bits per transform
			{ 0, 15, 7 },		// 66 bits per transform
			{ 0, 16, 6 },		// 66 bits per transform
			{ 0, 17, 5 },		// 66 bits per transform
			{ 0, 18, 4 },		// 66 bits per transform
			{ 0, 19, 3 },		// 66 bits per transform
			{ 0, 20, 2 },		// 66 bits per transform
			{ 0, 21, 1 },		// 66 bits per transform
			{ 0, 22, 0 },		// 66 bits per transform
			{ 1, 0, 21 },		// 66 bits per transform
			{ 1, 1, 20 },		// 66 bits per transform
			{ 1, 2, 19 },		// 66 bits per transform
			{ 1, 3, 18 },		// 66 bits per transform
			{ 1, 4, 17 },		// 66 bits per transform
			{ 1, 5, 16 },		// 66 bits per transform
			{ 1, 6, 15 },		// 66 bits per transform
			{ 1, 7, 14 },		// 66 bits per transform
			{ 1, 8, 13 },		// 66 bits per transform
			{ 1, 9, 12 },		// 66 bits per transform
			{ 1, 10, 11 },		// 66 bits per transform
			{ 1, 11, 10 },		// 66 bits per transform
			{ 1, 12, 9 },		// 66 bits per transform
			{ 1, 13, 8 },		// 66 bits per transform
			{ 1, 14, 7 },		// 66 bits per transform
			{ 1, 15, 6 },		// 66 bits per transform
			{ 1, 16, 5 },		// 66 bits per transform
			{ 1, 17, 4 },		// 66 bits per transform
			{ 1, 18, 3 },		// 66 bits per transform
			{ 1, 19, 2 },		// 66 bits per transform
			{ 1, 20, 1 },		// 66 bits per transform
			{ 1, 21, 0 },		// 66 bits per transform
			{ 2, 0, 20 },		// 66 bits per transform
			{ 2, 1, 19 },		// 66 bits per transform
			{ 2, 2, 18 },		// 66 bits per transform
			{ 2, 3, 17 },		// 66 bits per transform
			{ 2, 4, 16 },		// 66 bits per transform
			{ 2, 5, 15 },		// 66 bits per transform
			{ 2, 6, 14 },		// 66 bits per transform
			{ 2, 7, 13 },		// 66 bits per transform
			{ 2, 8, 12 },		// 66 bits per transform
			{ 2, 9, 11 },		// 66 bits per transform
			{ 2, 10, 10 },		// 66 bits per transform
			{ 2, 11, 9 },		// 66 bits per transform
			{ 2, 12, 8 },		// 66 bits per transform
			{ 2, 13, 7 },		// 66 bits per transform
			{ 2, 14, 6 },		// 66 bits per transform
			{ 2, 15, 5 },		// 66 bits per transform
			{ 2, 16, 4 },		// 66 bits per transform
			{ 2, 17, 3 },		// 66 bits per transform
			{ 2, 18, 2 },		// 66 bits per transform
			{ 2, 19, 1 },		// 66 bits per transform
			{ 2, 20, 0 },		// 66 bits per transform
			{ 3, 0, 19 },		// 66 bits per transform
			{ 3, 1, 18 },		// 66 bits per transform
			{ 3, 2, 17 },		// 66 bits per transform
			{ 3, 3, 16 },		// 66 bits per transform
			{ 3, 4, 15 },		// 66 bits per transform
			{ 3, 5, 14 },		// 66 bits per transform
			{ 3, 6, 13 },		// 66 bits per transform
			{ 3, 7, 12 },		// 66 bits per transform
			{ 3, 8, 11 },		// 66 bits per transform
			{ 3, 9, 10 },		// 66 bits per transform
			{ 3, 10, 9 },		// 66 bits per transform
			{ 3, 11, 8 },		// 66 bits per transform
			{ 3, 12, 7 },		// 66 bits per transform
			{ 3, 13, 6 },		// 66 bits per transform
			{ 3, 14, 5 },		// 66 bits per transform
			{ 3, 15, 4 },		// 66 bits per transform
			{ 3, 16, 3 },		// 66 bits per transform
			{ 3, 17, 2 },		// 66 bits per transform
			{ 3, 18, 1 },		// 66 bits per transform
			{ 3, 19, 0 },		// 66 bits per transform
			{ 4, 0, 18 },		// 66 bits per transform
			{ 4, 1, 17 },		// 66 bits per transform
			{ 4, 2, 16 },		// 66 bits per transform
			{ 4, 3, 15 },		// 66 bits per transform
			{ 4, 4, 14 },		// 66 bits per transform
			{ 4, 5, 13 },		// 66 bits per transform
			{ 4, 6, 12 },		// 66 bits per transform
			{ 4, 7, 11 },		// 66 bits per transform
			{ 4, 8, 10 },		// 66 bits per transform
			{ 4, 9, 9 },		// 66 bits per transform
			{ 4, 10, 8 },		// 66 bits per transform
			{ 4, 11, 7 },		// 66 bits per transform
			{ 4, 12, 6 },		// 66 bits per transform
			{ 4, 13, 5 },		// 66 bits per transform
			{ 4, 14, 4 },		// 66 bits per transform
			{ 4, 15, 3 },		// 66 bits per transform
			{ 4, 16, 2 },		// 66 bits per transform
			{ 4, 17, 1 },		// 66 bits per transform
			{ 4, 18, 0 },		// 66 bits per transform
			{ 5, 0, 17 },		// 66 bits per transform
			{ 5, 1, 16 },		// 66 bits per transform
			{ 5, 2, 15 },		// 66 bits per transform
			{ 5, 3, 14 },		// 66 bits per transform
			{ 5, 4, 13 },		// 66 bits per transform
			{ 5, 5, 12 },		// 66 bits per transform
			{ 5, 6, 11 },		// 66 bits per transform
			{ 5, 7, 10 },		// 66 bits per transform
			{ 5, 8, 9 },		// 66 bits per transform
			{ 5, 9, 8 },		// 66 bits per transform
			{ 5, 10, 7 },		// 66 bits per transform
			{ 5, 11, 6 },		// 66 bits per transform
			{ 5, 12, 5 },		// 66 bits per transform
			{ 5, 13, 4 },		// 66 bits per transform
			{ 5, 14, 3 },		// 66 bits per transform
			{ 5, 15, 2 },		// 66 bits per transform
			{ 5, 16, 1 },		// 66 bits per transform
			{ 5, 17, 0 },		// 66 bits per transform
			{ 6, 0, 16 },		// 66 bits per transform
			{ 6, 1, 15 },		// 66 bits per transform
			{ 6, 2, 14 },		// 66 bits per transform
			{ 6, 3, 13 },		// 66 bits per transform
			{ 6, 4, 12 },		// 66 bits per transform
			{ 6, 5, 11 },		// 66 bits per transform
			{ 6, 6, 10 },		// 66 bits per transform
			{ 6, 7, 9 },		// 66 bits per transform
			{ 6, 8, 8 },		// 66 bits per transform
			{ 6, 9, 7 },		// 66 bits per transform
			{ 6, 10, 6 },		// 66 bits per transform
			{ 6, 11, 5 },		// 66 bits per transform
			{ 6, 12, 4 },		// 66 bits per transform
			{ 6, 13, 3 },		// 66 bits per transform
			{ 6, 14, 2 },		// 66 bits per transform
			{ 6, 15, 1 },		// 66 bits per transform
			{ 6, 16, 0 },		// 66 bits per transform
			{ 7, 0, 15 },		// 66 bits per transform
			{ 7, 1, 14 },		// 66 bits per transform
			{ 7, 2, 13 },		// 66 bits per transform
			{ 7, 3, 12 },		// 66 bits per transform
			{ 7, 4, 11 },		// 66 bits per transform
			{ 7, 5, 10 },		// 66 bits per transform
			{ 7, 6, 9 },		// 66 bits per transform
			{ 7, 7, 8 },		// 66 bits per transform
			{ 7, 8, 7 },		// 66 bits per transform
			{ 7, 9, 6 },		// 66 bits per transform
			{ 7, 10, 5 },		// 66 bits per transform
			{ 7, 11, 4 },		// 66 bits per transform
			{ 7, 12, 3 },		// 66 bits per transform
			{ 7, 13, 2 },		// 66 bits per transform
			{ 7, 14, 1 },		// 66 bits per transform
			{ 7, 15, 0 },		// 66 bits per transform
			{ 8, 0, 14 },		// 66 bits per transform
			{ 8, 1, 13 },		// 66 bits per transform
			{ 8, 2, 12 },		// 66 bits per transform
			{ 8, 3, 11 },		// 66 bits per transform
			{ 8, 4, 10 },		// 66 bits per transform
			{ 8, 5, 9 },		// 66 bits per transform
			{ 8, 6, 8 },		// 66 bits per transform
			{ 8, 7, 7 },		// 66 bits per transform
			{ 8, 8, 6 },		// 66 bits per transform
			{ 8, 9, 5 },		// 66 bits per transform
			{ 8, 10, 4 },		// 66 bits per transform
			{ 8, 11, 3 },		// 66 bits per transform
			{ 8, 12, 2 },		// 66 bits per transform
			{ 8, 13, 1 },		// 66 bits per transform
			{ 8, 14, 0 },		// 66 bits per transform
			{ 9, 0, 13 },		// 66 bits per transform
			{ 9, 1, 12 },		// 66 bits per transform
			{ 9, 2, 11 },		// 66 bits per transform
			{ 9, 3, 10 },		// 66 bits per transform
			{ 9, 4, 9 },		// 66 bits per transform
			{ 9, 5, 8 },		// 66 bits per transform
			{ 9, 6, 7 },		// 66 bits per transform
			{ 9, 7, 6 },		// 66 bits per transform
			{ 9, 8, 5 },		// 66 bits per transform
			{ 9, 9, 4 },		// 66 bits per transform
			{ 9, 10, 3 },		// 66 bits per transform
			{ 9, 11, 2 },		// 66 bits per transform
			{ 9, 12, 1 },		// 66 bits per transform
			{ 9, 13, 0 },		// 66 bits per transform
			{ 10, 0, 12 },		// 66 bits per transform
			{ 10, 1, 11 },		// 66 bits per transform
			{ 10, 2, 10 },		// 66 bits per transform
			{ 10, 3, 9 },		// 66 bits per transform
			{ 10, 4, 8 },		// 66 bits per transform
			{ 10, 5, 7 },		// 66 bits per transform
			{ 10, 6, 6 },		// 66 bits per transform
			{ 10, 7, 5 },		// 66 bits per transform
			{ 10, 8, 4 },		// 66 bits per transform
			{ 10, 9, 3 },		// 66 bits per transform
			{ 10, 10, 2 },		// 66 bits per transform
			{ 10, 11, 1 },		// 66 bits per transform
			{ 10, 12, 0 },		// 66 bits per transform
			{ 11, 0, 11 },		// 66 bits per transform
			{ 11, 1, 10 },		// 66 bits per transform
			{ 11, 2, 9 },		// 66 bits per transform
			{ 11, 3, 8 },		// 66 bits per transform
			{ 11, 4, 7 },		// 66 bits per transform
			{ 11, 5, 6 },		// 66 bits per transform
			{ 11, 6, 5 },		// 66 bits per transform
			{ 11, 7, 4 },		// 66 bits per transform
			{ 11, 8, 3 },		// 66 bits per transform
			{ 11, 9, 2 },		// 66 bits per transform
			{ 11, 10, 1 },		// 66 bits per transform
			{ 11, 11, 0 },		// 66 bits per transform
			{ 12, 0, 10 },		// 66 bits per transform
			{ 12, 1, 9 },		// 66 bits per transform
			{ 12, 2, 8 },		// 66 bits per transform
			{ 12, 3, 7 },		// 66 bits per transform
			{ 12, 4, 6 },		// 66 bits per transform
			{ 12, 5, 5 },		// 66 bits per transform
			{ 12, 6, 4 },		// 66 bits per transform
			{ 12, 7, 3 },		// 66 bits per transform
			{ 12, 8, 2 },		// 66 bits per transform
			{ 12, 9, 1 },		// 66 bits per transform
			{ 12, 10, 0 },		// 66 bits per transform
			{ 13, 0, 9 },		// 66 bits per transform
			{ 13, 1, 8 },		// 66 bits per transform
			{ 13, 2, 7 },		// 66 bits per transform
			{ 13, 3, 6 },		// 66 bits per transform
			{ 13, 4, 5 },		// 66 bits per transform
			{ 13, 5, 4 },		// 66 bits per transform
			{ 13, 6, 3 },		// 66 bits per transform
			{ 13, 7, 2 },		// 66 bits per transform
			{ 13, 8, 1 },		// 66 bits per transform
			{ 13, 9, 0 },		// 66 bits per transform
			{ 14, 0, 8 },		// 66 bits per transform
			{ 14, 1, 7 },		// 66 bits per transform
			{ 14, 2, 6 },		// 66 bits per transform
			{ 14, 3, 5 },		// 66 bits per transform
			{ 14, 4, 4 },		// 66 bits per transform
			{ 14, 5, 3 },		// 66 bits per transform
			{ 14, 6, 2 },		// 66 bits per transform
			{ 14, 7, 1 },		// 66 bits per transform
			{ 14, 8, 0 },		// 66 bits per transform
			{ 15, 0, 7 },		// 66 bits per transform
			{ 15, 1, 6 },		// 66 bits per transform
			{ 15, 2, 5 },		// 66 bits per transform
			{ 15, 3, 4 },		// 66 bits per transform
			{ 15, 4, 3 },		// 66 bits per transform
			{ 15, 5, 2 },		// 66 bits per transform
			{ 15, 6, 1 },		// 66 bits per transform
			{ 15, 7, 0 },		// 66 bits per transform
			{ 16, 0, 6 },		// 66 bits per transform
			{ 16, 1, 5 },		// 66 bits per transform
			{ 16, 2, 4 },		// 66 bits per transform
			{ 16, 3, 3 },		// 66 bits per transform
			{ 16, 4, 2 },		// 66 bits per transform
			{ 16, 5, 1 },		// 66 bits per transform
			{ 16, 6, 0 },		// 66 bits per transform
			{ 17, 0, 5 },		// 66 bits per transform
			{ 17, 1, 4 },		// 66 bits per transform
			{ 17, 2, 3 },		// 66 bits per transform
			{ 17, 3, 2 },		// 66 bits per transform
			{ 17, 4, 1 },		// 66 bits per transform
			{ 17, 5, 0 },		// 66 bits per transform
			{ 18, 0, 4 },		// 66 bits per transform
			{ 18, 1, 3 },		// 66 bits per transform
			{ 18, 2, 2 },		// 66 bits per transform
			{ 18, 3, 1 },		// 66 bits per transform
			{ 18, 4, 0 },		// 66 bits per transform
			{ 19, 0, 3 },		// 66 bits per transform
			{ 19, 1, 2 },		// 66 bits per transform
			{ 19, 2, 1 },		// 66 bits per transform
			{ 19, 3, 0 },		// 66 bits per transform
			{ 20, 0, 2 },		// 66 bits per transform
			{ 20, 1, 1 },		// 66 bits per transform
			{ 20, 2, 0 },		// 66 bits per transform
			{ 21, 0, 1 },		// 66 bits per transform
			{ 21, 1, 0 },		// 66 bits per transform
			{ 22, 0, 0 },		// 66 bits per transform
			{ 0, 0, 23 },		// 69 bits per transform
			{ 0, 1, 22 },		// 69 bits per transform
			{ 0, 2, 21 },		// 69 bits per transform
			{ 0, 3, 20 },		// 69 bits per transform
			{ 0, 4, 19 },		// 69 bits per transform
			{ 0, 5, 18 },		// 69 bits per transform
			{ 0, 6, 17 },		// 69 bits per transform
			{ 0, 7, 16 },		// 69 bits per transform
			{ 0, 8, 15 },		// 69 bits per transform
			{ 0, 9, 14 },		// 69 bits per transform
			{ 0, 10, 13 },		// 69 bits per transform
			{ 0, 11, 12 },		// 69 bits per transform
			{ 0, 12, 11 },		// 69 bits per transform
			{ 0, 13, 10 },		// 69 bits per transform
			{ 0, 14, 9 },		// 69 bits per transform
			{ 0, 15, 8 },		// 69 bits per transform
			{ 0, 16, 7 },		// 69 bits per transform
			{ 0, 17, 6 },		// 69 bits per transform
			{ 0, 18, 5 },		// 69 bits per transform
			{ 0, 19, 4 },		// 69 bits per transform
			{ 0, 20, 3 },		// 69 bits per transform
			{ 0, 21, 2 },		// 69 bits per transform
			{ 0, 22, 1 },		// 69 bits per transform
			{ 0, 23, 0 },		// 69 bits per transform
			{ 1, 0, 22 },		// 69 bits per transform
			{ 1, 1, 21 },		// 69 bits per transform
			{ 1, 2, 20 },		// 69 bits per transform
			{ 1, 3, 19 },		// 69 bits per transform
			{ 1, 4, 18 },		// 69 bits per transform
			{ 1, 5, 17 },		// 69 bits per transform
			{ 1, 6, 16 },		// 69 bits per transform
			{ 1, 7, 15 },		// 69 bits per transform
			{ 1, 8, 14 },		// 69 bits per transform
			{ 1, 9, 13 },		// 69 bits per transform
			{ 1, 10, 12 },		// 69 bits per transform
			{ 1, 11, 11 },		// 69 bits per transform
			{ 1, 12, 10 },		// 69 bits per transform
			{ 1, 13, 9 },		// 69 bits per transform
			{ 1, 14, 8 },		// 69 bits per transform
			{ 1, 15, 7 },		// 69 bits per transform
			{ 1, 16, 6 },		// 69 bits per transform
			{ 1, 17, 5 },		// 69 bits per transform
			{ 1, 18, 4 },		// 69 bits per transform
			{ 1, 19, 3 },		// 69 bits per transform
			{ 1, 20, 2 },		// 69 bits per transform
			{ 1, 21, 1 },		// 69 bits per transform
			{ 1, 22, 0 },		// 69 bits per transform
			{ 2, 0, 21 },		// 69 bits per transform
			{ 2, 1, 20 },		// 69 bits per transform
			{ 2, 2, 19 },		// 69 bits per transform
			{ 2, 3, 18 },		// 69 bits per transform
			{ 2, 4, 17 },		// 69 bits per transform
			{ 2, 5, 16 },		// 69 bits per transform
			{ 2, 6, 15 },		// 69 bits per transform
			{ 2, 7, 14 },		// 69 bits per transform
			{ 2, 8, 13 },		// 69 bits per transform
			{ 2, 9, 12 },		// 69 bits per transform
			{ 2, 10, 11 },		// 69 bits per transform
			{ 2, 11, 10 },		// 69 bits per transform
			{ 2, 12, 9 },		// 69 bits per transform
			{ 2, 13, 8 },		// 69 bits per transform
			{ 2, 14, 7 },		// 69 bits per transform
			{ 2, 15, 6 },		// 69 bits per transform
			{ 2, 16, 5 },		// 69 bits per transform
			{ 2, 17, 4 },		// 69 bits per transform
			{ 2, 18, 3 },		// 69 bits per transform
			{ 2, 19, 2 },		// 69 bits per transform
			{ 2, 20, 1 },		// 69 bits per transform
			{ 2, 21, 0 },		// 69 bits per transform
			{ 3, 0, 20 },		// 69 bits per transform
			{ 3, 1, 19 },		// 69 bits per transform
			{ 3, 2, 18 },		// 69 bits per transform
			{ 3, 3, 17 },		// 69 bits per transform
			{ 3, 4, 16 },		// 69 bits per transform
			{ 3, 5, 15 },		// 69 bits per transform
			{ 3, 6, 14 },		// 69 bits per transform
			{ 3, 7, 13 },		// 69 bits per transform
			{ 3, 8, 12 },		// 69 bits per transform
			{ 3, 9, 11 },		// 69 bits per transform
			{ 3, 10, 10 },		// 69 bits per transform
			{ 3, 11, 9 },		// 69 bits per transform
			{ 3, 12, 8 },		// 69 bits per transform
			{ 3, 13, 7 },		// 69 bits per transform
			{ 3, 14, 6 },		// 69 bits per transform
			{ 3, 15, 5 },		// 69 bits per transform
			{ 3, 16, 4 },		// 69 bits per transform
			{ 3, 17, 3 },		// 69 bits per transform
			{ 3, 18, 2 },		// 69 bits per transform
			{ 3, 19, 1 },		// 69 bits per transform
			{ 3, 20, 0 },		// 69 bits per transform
			{ 4, 0, 19 },		// 69 bits per transform
			{ 4, 1, 18 },		// 69 bits per transform
			{ 4, 2, 17 },		// 69 bits per transform
			{ 4, 3, 16 },		// 69 bits per transform
			{ 4, 4, 15 },		// 69 bits per transform
			{ 4, 5, 14 },		// 69 bits per transform
			{ 4, 6, 13 },		// 69 bits per transform
			{ 4, 7, 12 },		// 69 bits per transform
			{ 4, 8, 11 },		// 69 bits per transform
			{ 4, 9, 10 },		// 69 bits per transform
			{ 4, 10, 9 },		// 69 bits per transform
			{ 4, 11, 8 },		// 69 bits per transform
			{ 4, 12, 7 },		// 69 bits per transform
			{ 4, 13, 6 },		// 69 bits per transform
			{ 4, 14, 5 },		// 69 bits per transform
			{ 4, 15, 4 },		// 69 bits per transform
			{ 4, 16, 3 },		// 69 bits per transform
			{ 4, 17, 2 },		// 69 bits per transform
			{ 4, 18, 1 },		// 69 bits per transform
			{ 4, 19, 0 },		// 69 bits per transform
			{ 5, 0, 18 },		// 69 bits per transform
			{ 5, 1, 17 },		// 69 bits per transform
			{ 5, 2, 16 },		// 69 bits per transform
			{ 5, 3, 15 },		// 69 bits per transform
			{ 5, 4, 14 },		// 69 bits per transform
			{ 5, 5, 13 },		// 69 bits per transform
			{ 5, 6, 12 },		// 69 bits per transform
			{ 5, 7, 11 },		// 69 bits per transform
			{ 5, 8, 10 },		// 69 bits per transform
			{ 5, 9, 9 },		// 69 bits per transform
			{ 5, 10, 8 },		// 69 bits per transform
			{ 5, 11, 7 },		// 69 bits per transform
			{ 5, 12, 6 },		// 69 bits per transform
			{ 5, 13, 5 },		// 69 bits per transform
			{ 5, 14, 4 },		// 69 bits per transform
			{ 5, 15, 3 },		// 69 bits per transform
			{ 5, 16, 2 },		// 69 bits per transform
			{ 5, 17, 1 },		// 69 bits per transform
			{ 5, 18, 0 },		// 69 bits per transform
			{ 6, 0, 17 },		// 69 bits per transform
			{ 6, 1, 16 },		// 69 bits per transform
			{ 6, 2, 15 },		// 69 bits per transform
			{ 6, 3, 14 },		// 69 bits per transform
			{ 6, 4, 13 },		// 69 bits per transform
			{ 6, 5, 12 },		// 69 bits per transform
			{ 6, 6, 11 },		// 69 bits per transform
			{ 6, 7, 10 },		// 69 bits per transform
			{ 6, 8, 9 },		// 69 bits per transform
			{ 6, 9, 8 },		// 69 bits per transform
			{ 6, 10, 7 },		// 69 bits per transform
			{ 6, 11, 6 },		// 69 bits per transform
			{ 6, 12, 5 },		// 69 bits per transform
			{ 6, 13, 4 },		// 69 bits per transform
			{ 6, 14, 3 },		// 69 bits per transform
			{ 6, 15, 2 },		// 69 bits per transform
			{ 6, 16, 1 },		// 69 bits per transform
			{ 6, 17, 0 },		// 69 bits per transform
			{ 7, 0, 16 },		// 69 bits per transform
			{ 7, 1, 15 },		// 69 bits per transform
			{ 7, 2, 14 },		// 69 bits per transform
			{ 7, 3, 13 },		// 69 bits per transform
			{ 7, 4, 12 },		// 69 bits per transform
			{ 7, 5, 11 },		// 69 bits per transform
			{ 7, 6, 10 },		// 69 bits per transform
			{ 7, 7, 9 },		// 69 bits per transform
			{ 7, 8, 8 },		// 69 bits per transform
			{ 7, 9, 7 },		// 69 bits per transform
			{ 7, 10, 6 },		// 69 bits per transform
			{ 7, 11, 5 },		// 69 bits per transform
			{ 7, 12, 4 },		// 69 bits per transform
			{ 7, 13, 3 },		// 69 bits per transform
			{ 7, 14, 2 },		// 69 bits per transform
			{ 7, 15, 1 },		// 69 bits per transform
			{ 7, 16, 0 },		// 69 bits per transform
			{ 8, 0, 15 },		// 69 bits per transform
			{ 8, 1, 14 },		// 69 bits per transform
			{ 8, 2, 13 },		// 69 bits per transform
			{ 8, 3, 12 },		// 69 bits per transform
			{ 8, 4, 11 },		// 69 bits per transform
			{ 8, 5, 10 },		// 69 bits per transform
			{ 8, 6, 9 },		// 69 bits per transform
			{ 8, 7, 8 },		// 69 bits per transform
			{ 8, 8, 7 },		// 69 bits per transform
			{ 8, 9, 6 },		// 69 bits per transform
			{ 8, 10, 5 },		// 69 bits per transform
			{ 8, 11, 4 },		// 69 bits per transform
			{ 8, 12, 3 },		// 69 bits per transform
			{ 8, 13, 2 },		// 69 bits per transform
			{ 8, 14, 1 },		// 69 bits per transform
			{ 8, 15, 0 },		// 69 bits per transform
			{ 9, 0, 14 },		// 69 bits per transform
			{ 9, 1, 13 },		// 69 bits per transform
			{ 9, 2, 12 },		// 69 bits per transform
			{ 9, 3, 11 },		// 69 bits per transform
			{ 9, 4, 10 },		// 69 bits per transform
			{ 9, 5, 9 },		// 69 bits per transform
			{ 9, 6, 8 },		// 69 bits per transform
			{ 9, 7, 7 },		// 69 bits per transform
			{ 9, 8, 6 },		// 69 bits per transform
			{ 9, 9, 5 },		// 69 bits per transform
			{ 9, 10, 4 },		// 69 bits per transform
			{ 9, 11, 3 },		// 69 bits per transform
			{ 9, 12, 2 },		// 69 bits per transform
			{ 9, 13, 1 },		// 69 bits per transform
			{ 9, 14, 0 },		// 69 bits per transform
			{ 10, 0, 13 },		// 69 bits per transform
			{ 10, 1, 12 },		// 69 bits per transform
			{ 10, 2, 11 },		// 69 bits per transform
			{ 10, 3, 10 },		// 69 bits per transform
			{ 10, 4, 9 },		// 69 bits per transform
			{ 10, 5, 8 },		// 69 bits per transform
			{ 10, 6, 7 },		// 69 bits per transform
			{ 10, 7, 6 },		// 69 bits per transform
			{ 10, 8, 5 },		// 69 bits per transform
			{ 10, 9, 4 },		// 69 bits per transform
			{ 10, 10, 3 },		// 69 bits per transform
			{ 10, 11, 2 },		// 69 bits per transform
			{ 10, 12, 1 },		// 69 bits per transform
			{ 10, 13, 0 },		// 69 bits per transform
			{ 11, 0, 12 },		// 69 bits per transform
			{ 11, 1, 11 },		// 69 bits per transform
			{ 11, 2, 10 },		// 69 bits per transform
			{ 11, 3, 9 },		// 69 bits per transform
			{ 11, 4, 8 },		// 69 bits per transform
			{ 11, 5, 7 },		// 69 bits per transform
			{ 11, 6, 6 },		// 69 bits per transform
			{ 11, 7, 5 },		// 69 bits per transform
			{ 11, 8, 4 },		// 69 bits per transform
			{ 11, 9, 3 },		// 69 bits per transform
			{ 11, 10, 2 },		// 69 bits per transform
			{ 11, 11, 1 },		// 69 bits per transform
			{ 11, 12, 0 },		// 69 bits per transform
			{ 12, 0, 11 },		// 69 bits per transform
			{ 12, 1, 10 },		// 69 bits per transform
			{ 12, 2, 9 },		// 69 bits per transform
			{ 12, 3, 8 },		// 69 bits per transform
			{ 12, 4, 7 },		// 69 bits per transform
			{ 12, 5, 6 },		// 69 bits per transform
			{ 12, 6, 5 },		// 69 bits per transform
			{ 12, 7, 4 },		// 69 bits per transform
			{ 12, 8, 3 },		// 69 bits per transform
			{ 12, 9, 2 },		// 69 bits per transform
			{ 12, 10, 1 },		// 69 bits per transform
			{ 12, 11, 0 },		// 69 bits per transform
			{ 13, 0, 10 },		// 69 bits per transform
			{ 13, 1, 9 },		// 69 bits per transform
			{ 13, 2, 8 },		// 69 bits per transform
			{ 13, 3, 7 },		// 69 bits per transform
			{ 13, 4, 6 },		// 69 bits per transform
			{ 13, 5, 5 },		// 69 bits per transform
			{ 13, 6, 4 },		// 69 bits per transform
			{ 13, 7, 3 },		// 69 bits per transform
			{ 13, 8, 2 },		// 69 bits per transform
			{ 13, 9, 1 },		// 69 bits per transform
			{ 13, 10, 0 },		// 69 bits per transform
			{ 14, 0, 9 },		// 69 bits per transform
			{ 14, 1, 8 },		// 69 bits per transform
			{ 14, 2, 7 },		// 69 bits per transform
			{ 14, 3, 6 },		// 69 bits per transform
			{ 14, 4, 5 },		// 69 bits per transform
			{ 14, 5, 4 },		// 69 bits per transform
			{ 14, 6, 3 },		// 69 bits per transform
			{ 14, 7, 2 },		// 69 bits per transform
			{ 14, 8, 1 },		// 69 bits per transform
			{ 14, 9, 0 },		// 69 bits per transform
			{ 15, 0, 8 },		// 69 bits per transform
			{ 15, 1, 7 },		// 69 bits per transform
			{ 15, 2, 6 },		// 69 bits per transform
			{ 15, 3, 5 },		// 69 bits per transform
			{ 15, 4, 4 },		// 69 bits per transform
			{ 15, 5, 3 },		// 69 bits per transform
			{ 15, 6, 2 },		// 69 bits per transform
			{ 15, 7, 1 },		// 69 bits per transform
			{ 15, 8, 0 },		// 69 bits per transform
			{ 16, 0, 7 },		// 69 bits per transform
			{ 16, 1, 6 },		// 69 bits per transform
			{ 16, 2, 5 },		// 69 bits per transform
			{ 16, 3, 4 },		// 69 bits per transform
			{ 16, 4, 3 },		// 69 bits per transform
			{ 16, 5, 2 },		// 69 bits per transform
			{ 16, 6, 1 },		// 69 bits per transform
			{ 16, 7, 0 },		// 69 bits per transform
			{ 17, 0, 6 },		// 69 bits per transform
			{ 17, 1, 5 },		// 69 bits per transform
			{ 17, 2, 4 },		// 69 bits per transform
			{ 17, 3, 3 },		// 69 bits per transform
			{ 17, 4, 2 },		// 69 bits per transform
			{ 17, 5, 1 },		// 69 bits per transform
			{ 17, 6, 0 },		// 69 bits per transform
			{ 18, 0, 5 },		// 69 bits per transform
			{ 18, 1, 4 },		// 69 bits per transform
			{ 18, 2, 3 },		// 69 bits per transform
			{ 18, 3, 2 },		// 69 bits per transform
			{ 18, 4, 1 },		// 69 bits per transform
			{ 18, 5, 0 },		// 69 bits per transform
			{ 19, 0, 4 },		// 69 bits per transform
			{ 19, 1, 3 },		// 69 bits per transform
			{ 19, 2, 2 },		// 69 bits per transform
			{ 19, 3, 1 },		// 69 bits per transform
			{ 19, 4, 0 },		// 69 bits per transform
			{ 20, 0, 3 },		// 69 bits per transform
			{ 20, 1, 2 },		// 69 bits per transform
			{ 20, 2, 1 },		// 69 bits per transform
			{ 20, 3, 0 },		// 69 bits per transform
			{ 21, 0, 2 },		// 69 bits per transform
			{ 21, 1, 1 },		// 69 bits per transform
			{ 21, 2, 0 },		// 69 bits per transform
			{ 22, 0, 1 },		// 69 bits per transform
			{ 22, 1, 0 },		// 69 bits per transform
			{ 23, 0, 0 },		// 69 bits per transform
			{ 0, 1, 23 },		// 72 bits per transform
			{ 0, 2, 22 },		// 72 bits per transform
			{ 0, 3, 21 },		// 72 bits per transform
			{ 0, 4, 20 },		// 72 bits per transform
			{ 0, 5, 19 },		// 72 bits per transform
			{ 0, 6, 18 },		// 72 bits per transform
			{ 0, 7, 17 },		// 72 bits per transform
			{ 0, 8, 16 },		// 72 bits per transform
			{ 0, 9, 15 },		// 72 bits per transform
			{ 0, 10, 14 },		// 72 bits per transform
			{ 0, 11, 13 },		// 72 bits per transform
			{ 0, 12, 12 },		// 72 bits per transform
			{ 0, 13, 11 },		// 72 bits per transform
			{ 0, 14, 10 },		// 72 bits per transform
			{ 0, 15, 9 },		// 72 bits per transform
			{ 0, 16, 8 },		// 72 bits per transform
			{ 0, 17, 7 },		// 72 bits per transform
			{ 0, 18, 6 },		// 72 bits per transform
			{ 0, 19, 5 },		// 72 bits per transform
			{ 0, 20, 4 },		// 72 bits per transform
			{ 0, 21, 3 },		// 72 bits per transform
			{ 0, 22, 2 },		// 72 bits per transform
			{ 0, 23, 1 },		// 72 bits per transform
			{ 1, 0, 23 },		// 72 bits per transform
			{ 1, 1, 22 },		// 72 bits per transform
			{ 1, 2, 21 },		// 72 bits per transform
			{ 1, 3, 20 },		// 72 bits per transform
			{ 1, 4, 19 },		// 72 bits per transform
			{ 1, 5, 18 },		// 72 bits per transform
			{ 1, 6, 17 },		// 72 bits per transform
			{ 1, 7, 16 },		// 72 bits per transform
			{ 1, 8, 15 },		// 72 bits per transform
			{ 1, 9, 14 },		// 72 bits per transform
			{ 1, 10, 13 },		// 72 bits per transform
			{ 1, 11, 12 },		// 72 bits per transform
			{ 1, 12, 11 },		// 72 bits per transform
			{ 1, 13, 10 },		// 72 bits per transform
			{ 1, 14, 9 },		// 72 bits per transform
			{ 1, 15, 8 },		// 72 bits per transform
			{ 1, 16, 7 },		// 72 bits per transform
			{ 1, 17, 6 },		// 72 bits per transform
			{ 1, 18, 5 },		// 72 bits per transform
			{ 1, 19, 4 },		// 72 bits per transform
			{ 1, 20, 3 },		// 72 bits per transform
			{ 1, 21, 2 },		// 72 bits per transform
			{ 1, 22, 1 },		// 72 bits per transform
			{ 1, 23, 0 },		// 72 bits per transform
			{ 2, 0, 22 },		// 72 bits per transform
			{ 2, 1, 21 },		// 72 bits per transform
			{ 2, 2, 20 },		// 72 bits per transform
			{ 2, 3, 19 },		// 72 bits per transform
			{ 2, 4, 18 },		// 72 bits per transform
			{ 2, 5, 17 },		// 72 bits per transform
			{ 2, 6, 16 },		// 72 bits per transform
			{ 2, 7, 15 },		// 72 bits per transform
			{ 2, 8, 14 },		// 72 bits per transform
			{ 2, 9, 13 },		// 72 bits per transform
			{ 2, 10, 12 },		// 72 bits per transform
			{ 2, 11, 11 },		// 72 bits per transform
			{ 2, 12, 10 },		// 72 bits per transform
			{ 2, 13, 9 },		// 72 bits per transform
			{ 2, 14, 8 },		// 72 bits per transform
			{ 2, 15, 7 },		// 72 bits per transform
			{ 2, 16, 6 },		// 72 bits per transform
			{ 2, 17, 5 },		// 72 bits per transform
			{ 2, 18, 4 },		// 72 bits per transform
			{ 2, 19, 3 },		// 72 bits per transform
			{ 2, 20, 2 },		// 72 bits per transform
			{ 2, 21, 1 },		// 72 bits per transform
			{ 2, 22, 0 },		// 72 bits per transform
			{ 3, 0, 21 },		// 72 bits per transform
			{ 3, 1, 20 },		// 72 bits per transform
			{ 3, 2, 19 },		// 72 bits per transform
			{ 3, 3, 18 },		// 72 bits per transform
			{ 3, 4, 17 },		// 72 bits per transform
			{ 3, 5, 16 },		// 72 bits per transform
			{ 3, 6, 15 },		// 72 bits per transform
			{ 3, 7, 14 },		// 72 bits per transform
			{ 3, 8, 13 },		// 72 bits per transform
			{ 3, 9, 12 },		// 72 bits per transform
			{ 3, 10, 11 },		// 72 bits per transform
			{ 3, 11, 10 },		// 72 bits per transform
			{ 3, 12, 9 },		// 72 bits per transform
			{ 3, 13, 8 },		// 72 bits per transform
			{ 3, 14, 7 },		// 72 bits per transform
			{ 3, 15, 6 },		// 72 bits per transform
			{ 3, 16, 5 },		// 72 bits per transform
			{ 3, 17, 4 },		// 72 bits per transform
			{ 3, 18, 3 },		// 72 bits per transform
			{ 3, 19, 2 },		// 72 bits per transform
			{ 3, 20, 1 },		// 72 bits per transform
			{ 3, 21, 0 },		// 72 bits per transform
			{ 4, 0, 20 },		// 72 bits per transform
			{ 4, 1, 19 },		// 72 bits per transform
			{ 4, 2, 18 },		// 72 bits per transform
			{ 4, 3, 17 },		// 72 bits per transform
			{ 4, 4, 16 },		// 72 bits per transform
			{ 4, 5, 15 },		// 72 bits per transform
			{ 4, 6, 14 },		// 72 bits per transform
			{ 4, 7, 13 },		// 72 bits per transform
			{ 4, 8, 12 },		// 72 bits per transform
			{ 4, 9, 11 },		// 72 bits per transform
			{ 4, 10, 10 },		// 72 bits per transform
			{ 4, 11, 9 },		// 72 bits per transform
			{ 4, 12, 8 },		// 72 bits per transform
			{ 4, 13, 7 },		// 72 bits per transform
			{ 4, 14, 6 },		// 72 bits per transform
			{ 4, 15, 5 },		// 72 bits per transform
			{ 4, 16, 4 },		// 72 bits per transform
			{ 4, 17, 3 },		// 72 bits per transform
			{ 4, 18, 2 },		// 72 bits per transform
			{ 4, 19, 1 },		// 72 bits per transform
			{ 4, 20, 0 },		// 72 bits per transform
			{ 5, 0, 19 },		// 72 bits per transform
			{ 5, 1, 18 },		// 72 bits per transform
			{ 5, 2, 17 },		// 72 bits per transform
			{ 5, 3, 16 },		// 72 bits per transform
			{ 5, 4, 15 },		// 72 bits per transform
			{ 5, 5, 14 },		// 72 bits per transform
			{ 5, 6, 13 },		// 72 bits per transform
			{ 5, 7, 12 },		// 72 bits per transform
			{ 5, 8, 11 },		// 72 bits per transform
			{ 5, 9, 10 },		// 72 bits per transform
			{ 5, 10, 9 },		// 72 bits per transform
			{ 5, 11, 8 },		// 72 bits per transform
			{ 5, 12, 7 },		// 72 bits per transform
			{ 5, 13, 6 },		// 72 bits per transform
			{ 5, 14, 5 },		// 72 bits per transform
			{ 5, 15, 4 },		// 72 bits per transform
			{ 5, 16, 3 },		// 72 bits per transform
			{ 5, 17, 2 },		// 72 bits per transform
			{ 5, 18, 1 },		// 72 bits per transform
			{ 5, 19, 0 },		// 72 bits per transform
			{ 6, 0, 18 },		// 72 bits per transform
			{ 6, 1, 17 },		// 72 bits per transform
			{ 6, 2, 16 },		// 72 bits per transform
			{ 6, 3, 15 },		// 72 bits per transform
			{ 6, 4, 14 },		// 72 bits per transform
			{ 6, 5, 13 },		// 72 bits per transform
			{ 6, 6, 12 },		// 72 bits per transform
			{ 6, 7, 11 },		// 72 bits per transform
			{ 6, 8, 10 },		// 72 bits per transform
			{ 6, 9, 9 },		// 72 bits per transform
			{ 6, 10, 8 },		// 72 bits per transform
			{ 6, 11, 7 },		// 72 bits per transform
			{ 6, 12, 6 },		// 72 bits per transform
			{ 6, 13, 5 },		// 72 bits per transform
			{ 6, 14, 4 },		// 72 bits per transform
			{ 6, 15, 3 },		// 72 bits per transform
			{ 6, 16, 2 },		// 72 bits per transform
			{ 6, 17, 1 },		// 72 bits per transform
			{ 6, 18, 0 },		// 72 bits per transform
			{ 7, 0, 17 },		// 72 bits per transform
			{ 7, 1, 16 },		// 72 bits per transform
			{ 7, 2, 15 },		// 72 bits per transform
			{ 7, 3, 14 },		// 72 bits per transform
			{ 7, 4, 13 },		// 72 bits per transform
			{ 7, 5, 12 },		// 72 bits per transform
			{ 7, 6, 11 },		// 72 bits per transform
			{ 7, 7, 10 },		// 72 bits per transform
			{ 7, 8, 9 },		// 72 bits per transform
			{ 7, 9, 8 },		// 72 bits per transform
			{ 7, 10, 7 },		// 72 bits per transform
			{ 7, 11, 6 },		// 72 bits per transform
			{ 7, 12, 5 },		// 72 bits per transform
			{ 7, 13, 4 },		// 72 bits per transform
			{ 7, 14, 3 },		// 72 bits per transform
			{ 7, 15, 2 },		// 72 bits per transform
			{ 7, 16, 1 },		// 72 bits per transform
			{ 7, 17, 0 },		// 72 bits per transform
			{ 8, 0, 16 },		// 72 bits per transform
			{ 8, 1, 15 },		// 72 bits per transform
			{ 8, 2, 14 },		// 72 bits per transform
			{ 8, 3, 13 },		// 72 bits per transform
			{ 8, 4, 12 },		// 72 bits per transform
			{ 8, 5, 11 },		// 72 bits per transform
			{ 8, 6, 10 },		// 72 bits per transform
			{ 8, 7, 9 },		// 72 bits per transform
			{ 8, 8, 8 },		// 72 bits per transform
			{ 8, 9, 7 },		// 72 bits per transform
			{ 8, 10, 6 },		// 72 bits per transform
			{ 8, 11, 5 },		// 72 bits per transform
			{ 8, 12, 4 },		// 72 bits per transform
			{ 8, 13, 3 },		// 72 bits per transform
			{ 8, 14, 2 },		// 72 bits per transform
			{ 8, 15, 1 },		// 72 bits per transform
			{ 8, 16, 0 },		// 72 bits per transform
			{ 9, 0, 15 },		// 72 bits per transform
			{ 9, 1, 14 },		// 72 bits per transform
			{ 9, 2, 13 },		// 72 bits per transform
			{ 9, 3, 12 },		// 72 bits per transform
			{ 9, 4, 11 },		// 72 bits per transform
			{ 9, 5, 10 },		// 72 bits per transform
			{ 9, 6, 9 },		// 72 bits per transform
			{ 9, 7, 8 },		// 72 bits per transform
			{ 9, 8, 7 },		// 72 bits per transform
			{ 9, 9, 6 },		// 72 bits per transform
			{ 9, 10, 5 },		// 72 bits per transform
			{ 9, 11, 4 },		// 72 bits per transform
			{ 9, 12, 3 },		// 72 bits per transform
			{ 9, 13, 2 },		// 72 bits per transform
			{ 9, 14, 1 },		// 72 bits per transform
			{ 9, 15, 0 },		// 72 bits per transform
			{ 10, 0, 14 },		// 72 bits per transform
			{ 10, 1, 13 },		// 72 bits per transform
			{ 10, 2, 12 },		// 72 bits per transform
			{ 10, 3, 11 },		// 72 bits per transform
			{ 10, 4, 10 },		// 72 bits per transform
			{ 10, 5, 9 },		// 72 bits per transform
			{ 10, 6, 8 },		// 72 bits per transform
			{ 10, 7, 7 },		// 72 bits per transform
			{ 10, 8, 6 },		// 72 bits per transform
			{ 10, 9, 5 },		// 72 bits per transform
			{ 10, 10, 4 },		// 72 bits per transform
			{ 10, 11, 3 },		// 72 bits per transform
			{ 10, 12, 2 },		// 72 bits per transform
			{ 10, 13, 1 },		// 72 bits per transform
			{ 10, 14, 0 },		// 72 bits per transform
			{ 11, 0, 13 },		// 72 bits per transform
			{ 11, 1, 12 },		// 72 bits per transform
			{ 11, 2, 11 },		// 72 bits per transform
			{ 11, 3, 10 },		// 72 bits per transform
			{ 11, 4, 9 },		// 72 bits per transform
			{ 11, 5, 8 },		// 72 bits per transform
			{ 11, 6, 7 },		// 72 bits per transform
			{ 11, 7, 6 },		// 72 bits per transform
			{ 11, 8, 5 },		// 72 bits per transform
			{ 11, 9, 4 },		// 72 bits per transform
			{ 11, 10, 3 },		// 72 bits per transform
			{ 11, 11, 2 },		// 72 bits per transform
			{ 11, 12, 1 },		// 72 bits per transform
			{ 11, 13, 0 },		// 72 bits per transform
			{ 12, 0, 12 },		// 72 bits per transform
			{ 12, 1, 11 },		// 72 bits per transform
			{ 12, 2, 10 },		// 72 bits per transform
			{ 12, 3, 9 },		// 72 bits per transform
			{ 12, 4, 8 },		// 72 bits per transform
			{ 12, 5, 7 },		// 72 bits per transform
			{ 12, 6, 6 },		// 72 bits per transform
			{ 12, 7, 5 },		// 72 bits per transform
			{ 12, 8, 4 },		// 72 bits per transform
			{ 12, 9, 3 },		// 72 bits per transform
			{ 12, 10, 2 },		// 72 bits per transform
			{ 12, 11, 1 },		// 72 bits per transform
			{ 12, 12, 0 },		// 72 bits per transform
			{ 13, 0, 11 },		// 72 bits per transform
			{ 13, 1, 10 },		// 72 bits per transform
			{ 13, 2, 9 },		// 72 bits per transform
			{ 13, 3, 8 },		// 72 bits per transform
			{ 13, 4, 7 },		// 72 bits per transform
			{ 13, 5, 6 },		// 72 bits per transform
			{ 13, 6, 5 },		// 72 bits per transform
			{ 13, 7, 4 },		// 72 bits per transform
			{ 13, 8, 3 },		// 72 bits per transform
			{ 13, 9, 2 },		// 72 bits per transform
			{ 13, 10, 1 },		// 72 bits per transform
			{ 13, 11, 0 },		// 72 bits per transform
			{ 14, 0, 10 },		// 72 bits per transform
			{ 14, 1, 9 },		// 72 bits per transform
			{ 14, 2, 8 },		// 72 bits per transform
			{ 14, 3, 7 },		// 72 bits per transform
			{ 14, 4, 6 },		// 72 bits per transform
			{ 14, 5, 5 },		// 72 bits per transform
			{ 14, 6, 4 },		// 72 bits per transform
			{ 14, 7, 3 },		// 72 bits per transform
			{ 14, 8, 2 },		// 72 bits per transform
			{ 14, 9, 1 },		// 72 bits per transform
			{ 14, 10, 0 },		// 72 bits per transform
			{ 15, 0, 9 },		// 72 bits per transform
			{ 15, 1, 8 },		// 72 bits per transform
			{ 15, 2, 7 },		// 72 bits per transform
			{ 15, 3, 6 },		// 72 bits per transform
			{ 15, 4, 5 },		// 72 bits per transform
			{ 15, 5, 4 },		// 72 bits per transform
			{ 15, 6, 3 },		// 72 bits per transform
			{ 15, 7, 2 },		// 72 bits per transform
			{ 15, 8, 1 },		// 72 bits per transform
			{ 15, 9, 0 },		// 72 bits per transform
			{ 16, 0, 8 },		// 72 bits per transform
			{ 16, 1, 7 },		// 72 bits per transform
			{ 16, 2, 6 },		// 72 bits per transform
			{ 16, 3, 5 },		// 72 bits per transform
			{ 16, 4, 4 },		// 72 bits per transform
			{ 16, 5, 3 },		// 72 bits per transform
			{ 16, 6, 2 },		// 72 bits per transform
			{ 16, 7, 1 },		// 72 bits per transform
			{ 16, 8, 0 },		// 72 bits per transform
			{ 17, 0, 7 },		// 72 bits per transform
			{ 17, 1, 6 },		// 72 bits per transform
			{ 17, 2, 5 },		// 72 bits per transform
			{ 17, 3, 4 },		// 72 bits per transform
			{ 17, 4, 3 },		// 72 bits per transform
			{ 17, 5, 2 },		// 72 bits per transform
			{ 17, 6, 1 },		// 72 bits per transform
			{ 17, 7, 0 },		// 72 bits per transform
			{ 18, 0, 6 },		// 72 bits per transform
			{ 18, 1, 5 },		// 72 bits per transform
			{ 18, 2, 4 },		// 72 bits per transform
			{ 18, 3, 3 },		// 72 bits per transform
			{ 18, 4, 2 },		// 72 bits per transform
			{ 18, 5, 1 },		// 72 bits per transform
			{ 18, 6, 0 },		// 72 bits per transform
			{ 19, 0, 5 },		// 72 bits per transform
			{ 19, 1, 4 },		// 72 bits per transform
			{ 19, 2, 3 },		// 72 bits per transform
			{ 19, 3, 2 },		// 72 bits per transform
			{ 19, 4, 1 },		// 72 bits per transform
			{ 19, 5, 0 },		// 72 bits per transform
			{ 20, 0, 4 },		// 72 bits per transform
			{ 20, 1, 3 },		// 72 bits per transform
			{ 20, 2, 2 },		// 72 bits per transform
			{ 20, 3, 1 },		// 72 bits per transform
			{ 20, 4, 0 },		// 72 bits per transform
			{ 21, 0, 3 },		// 72 bits per transform
			{ 21, 1, 2 },		// 72 bits per transform
			{ 21, 2, 1 },		// 72 bits per transform
			{ 21, 3, 0 },		// 72 bits per transform
			{ 22, 0, 2 },		// 72 bits per transform
			{ 22, 1, 1 },		// 72 bits per transform
			{ 22, 2, 0 },		// 72 bits per transform
			{ 23, 0, 1 },		// 72 bits per transform
			{ 23, 1, 0 },		// 72 bits per transform
			{ 0, 2, 23 },		// 75 bits per transform
			{ 0, 3, 22 },		// 75 bits per transform
			{ 0, 4, 21 },		// 75 bits per transform
			{ 0, 5, 20 },		// 75 bits per transform
			{ 0, 6, 19 },		// 75 bits per transform
			{ 0, 7, 18 },		// 75 bits per transform
			{ 0, 8, 17 },		// 75 bits per transform
			{ 0, 9, 16 },		// 75 bits per transform
			{ 0, 10, 15 },		// 75 bits per transform
			{ 0, 11, 14 },		// 75 bits per transform
			{ 0, 12, 13 },		// 75 bits per transform
			{ 0, 13, 12 },		// 75 bits per transform
			{ 0, 14, 11 },		// 75 bits per transform
			{ 0, 15, 10 },		// 75 bits per transform
			{ 0, 16, 9 },		// 75 bits per transform
			{ 0, 17, 8 },		// 75 bits per transform
			{ 0, 18, 7 },		// 75 bits per transform
			{ 0, 19, 6 },		// 75 bits per transform
			{ 0, 20, 5 },		// 75 bits per transform
			{ 0, 21, 4 },		// 75 bits per transform
			{ 0, 22, 3 },		// 75 bits per transform
			{ 0, 23, 2 },		// 75 bits per transform
			{ 1, 1, 23 },		// 75 bits per transform
			{ 1, 2, 22 },		// 75 bits per transform
			{ 1, 3, 21 },		// 75 bits per transform
			{ 1, 4, 20 },		// 75 bits per transform
			{ 1, 5, 19 },		// 75 bits per transform
			{ 1, 6, 18 },		// 75 bits per transform
			{ 1, 7, 17 },		// 75 bits per transform
			{ 1, 8, 16 },		// 75 bits per transform
			{ 1, 9, 15 },		// 75 bits per transform
			{ 1, 10, 14 },		// 75 bits per transform
			{ 1, 11, 13 },		// 75 bits per transform
			{ 1, 12, 12 },		// 75 bits per transform
			{ 1, 13, 11 },		// 75 bits per transform
			{ 1, 14, 10 },		// 75 bits per transform
			{ 1, 15, 9 },		// 75 bits per transform
			{ 1, 16, 8 },		// 75 bits per transform
			{ 1, 17, 7 },		// 75 bits per transform
			{ 1, 18, 6 },		// 75 bits per transform
			{ 1, 19, 5 },		// 75 bits per transform
			{ 1, 20, 4 },		// 75 bits per transform
			{ 1, 21, 3 },		// 75 bits per transform
			{ 1, 22, 2 },		// 75 bits per transform
			{ 1, 23, 1 },		// 75 bits per transform
			{ 2, 0, 23 },		// 75 bits per transform
			{ 2, 1, 22 },		// 75 bits per transform
			{ 2, 2, 21 },		// 75 bits per transform
			{ 2, 3, 20 },		// 75 bits per transform
			{ 2, 4, 19 },		// 75 bits per transform
			{ 2, 5, 18 },		// 75 bits per transform
			{ 2, 6, 17 },		// 75 bits per transform
			{ 2, 7, 16 },		// 75 bits per transform
			{ 2, 8, 15 },		// 75 bits per transform
			{ 2, 9, 14 },		// 75 bits per transform
			{ 2, 10, 13 },		// 75 bits per transform
			{ 2, 11, 12 },		// 75 bits per transform
			{ 2, 12, 11 },		// 75 bits per transform
			{ 2, 13, 10 },		// 75 bits per transform
			{ 2, 14, 9 },		// 75 bits per transform
			{ 2, 15, 8 },		// 75 bits per transform
			{ 2, 16, 7 },		// 75 bits per transform
			{ 2, 17, 6 },		// 75 bits per transform
			{ 2, 18, 5 },		// 75 bits per transform
			{ 2, 19, 4 },		// 75 bits per transform
			{ 2, 20, 3 },		// 75 bits per transform
			{ 2, 21, 2 },		// 75 bits per transform
			{ 2, 22, 1 },		// 75 bits per transform
			{ 2, 23, 0 },		// 75 bits per transform
			{ 3, 0, 22 },		// 75 bits per transform
			{ 3, 1, 21 },		// 75 bits per transform
			{ 3, 2, 20 },		// 75 bits per transform
			{ 3, 3, 19 },		// 75 bits per transform
			{ 3, 4, 18 },		// 75 bits per transform
			{ 3, 5, 17 },		// 75 bits per transform
			{ 3, 6, 16 },		// 75 bits per transform
			{ 3, 7, 15 },		// 75 bits per transform
			{ 3, 8, 14 },		// 75 bits per transform
			{ 3, 9, 13 },		// 75 bits per transform
			{ 3, 10, 12 },		// 75 bits per transform
			{ 3, 11, 11 },		// 75 bits per transform
			{ 3, 12, 10 },		// 75 bits per transform
			{ 3, 13, 9 },		// 75 bits per transform
			{ 3, 14, 8 },		// 75 bits per transform
			{ 3, 15, 7 },		// 75 bits per transform
			{ 3, 16, 6 },		// 75 bits per transform
			{ 3, 17, 5 },		// 75 bits per transform
			{ 3, 18, 4 },		// 75 bits per transform
			{ 3, 19, 3 },		// 75 bits per transform
			{ 3, 20, 2 },		// 75 bits per transform
			{ 3, 21, 1 },		// 75 bits per transform
			{ 3, 22, 0 },		// 75 bits per transform
			{ 4, 0, 21 },		// 75 bits per transform
			{ 4, 1, 20 },		// 75 bits per transform
			{ 4, 2, 19 },		// 75 bits per transform
			{ 4, 3, 18 },		// 75 bits per transform
			{ 4, 4, 17 },		// 75 bits per transform
			{ 4, 5, 16 },		// 75 bits per transform
			{ 4, 6, 15 },		// 75 bits per transform
			{ 4, 7, 14 },		// 75 bits per transform
			{ 4, 8, 13 },		// 75 bits per transform
			{ 4, 9, 12 },		// 75 bits per transform
			{ 4, 10, 11 },		// 75 bits per transform
			{ 4, 11, 10 },		// 75 bits per transform
			{ 4, 12, 9 },		// 75 bits per transform
			{ 4, 13, 8 },		// 75 bits per transform
			{ 4, 14, 7 },		// 75 bits per transform
			{ 4, 15, 6 },		// 75 bits per transform
			{ 4, 16, 5 },		// 75 bits per transform
			{ 4, 17, 4 },		// 75 bits per transform
			{ 4, 18, 3 },		// 75 bits per transform
			{ 4, 19, 2 },		// 75 bits per transform
			{ 4, 20, 1 },		// 75 bits per transform
			{ 4, 21, 0 },		// 75 bits per transform
			{ 5, 0, 20 },		// 75 bits per transform
			{ 5, 1, 19 },		// 75 bits per transform
			{ 5, 2, 18 },		// 75 bits per transform
			{ 5, 3, 17 },		// 75 bits per transform
			{ 5, 4, 16 },		// 75 bits per transform
			{ 5, 5, 15 },		// 75 bits per transform
			{ 5, 6, 14 },		// 75 bits per transform
			{ 5, 7, 13 },		// 75 bits per transform
			{ 5, 8, 12 },		// 75 bits per transform
			{ 5, 9, 11 },		// 75 bits per transform
			{ 5, 10, 10 },		// 75 bits per transform
			{ 5, 11, 9 },		// 75 bits per transform
			{ 5, 12, 8 },		// 75 bits per transform
			{ 5, 13, 7 },		// 75 bits per transform
			{ 5, 14, 6 },		// 75 bits per transform
			{ 5, 15, 5 },		// 75 bits per transform
			{ 5, 16, 4 },		// 75 bits per transform
			{ 5, 17, 3 },		// 75 bits per transform
			{ 5, 18, 2 },		// 75 bits per transform
			{ 5, 19, 1 },		// 75 bits per transform
			{ 5, 20, 0 },		// 75 bits per transform
			{ 6, 0, 19 },		// 75 bits per transform
			{ 6, 1, 18 },		// 75 bits per transform
			{ 6, 2, 17 },		// 75 bits per transform
			{ 6, 3, 16 },		// 75 bits per transform
			{ 6, 4, 15 },		// 75 bits per transform
			{ 6, 5, 14 },		// 75 bits per transform
			{ 6, 6, 13 },		// 75 bits per transform
			{ 6, 7, 12 },		// 75 bits per transform
			{ 6, 8, 11 },		// 75 bits per transform
			{ 6, 9, 10 },		// 75 bits per transform
			{ 6, 10, 9 },		// 75 bits per transform
			{ 6, 11, 8 },		// 75 bits per transform
			{ 6, 12, 7 },		// 75 bits per transform
			{ 6, 13, 6 },		// 75 bits per transform
			{ 6, 14, 5 },		// 75 bits per transform
			{ 6, 15, 4 },		// 75 bits per transform
			{ 6, 16, 3 },		// 75 bits per transform
			{ 6, 17, 2 },		// 75 bits per transform
			{ 6, 18, 1 },		// 75 bits per transform
			{ 6, 19, 0 },		// 75 bits per transform
			{ 7, 0, 18 },		// 75 bits per transform
			{ 7, 1, 17 },		// 75 bits per transform
			{ 7, 2, 16 },		// 75 bits per transform
			{ 7, 3, 15 },		// 75 bits per transform
			{ 7, 4, 14 },		// 75 bits per transform
			{ 7, 5, 13 },		// 75 bits per transform
			{ 7, 6, 12 },		// 75 bits per transform
			{ 7, 7, 11 },		// 75 bits per transform
			{ 7, 8, 10 },		// 75 bits per transform
			{ 7, 9, 9 },		// 75 bits per transform
			{ 7, 10, 8 },		// 75 bits per transform
			{ 7, 11, 7 },		// 75 bits per transform
			{ 7, 12, 6 },		// 75 bits per transform
			{ 7, 13, 5 },		// 75 bits per transform
			{ 7, 14, 4 },		// 75 bits per transform
			{ 7, 15, 3 },		// 75 bits per transform
			{ 7, 16, 2 },		// 75 bits per transform
			{ 7, 17, 1 },		// 75 bits per transform
			{ 7, 18, 0 },		// 75 bits per transform
			{ 8, 0, 17 },		// 75 bits per transform
			{ 8, 1, 16 },		// 75 bits per transform
			{ 8, 2, 15 },		// 75 bits per transform
			{ 8, 3, 14 },		// 75 bits per transform
			{ 8, 4, 13 },		// 75 bits per transform
			{ 8, 5, 12 },		// 75 bits per transform
			{ 8, 6, 11 },		// 75 bits per transform
			{ 8, 7, 10 },		// 75 bits per transform
			{ 8, 8, 9 },		// 75 bits per transform
			{ 8, 9, 8 },		// 75 bits per transform
			{ 8, 10, 7 },		// 75 bits per transform
			{ 8, 11, 6 },		// 75 bits per transform
			{ 8, 12, 5 },		// 75 bits per transform
			{ 8, 13, 4 },		// 75 bits per transform
			{ 8, 14, 3 },		// 75 bits per transform
			{ 8, 15, 2 },		// 75 bits per transform
			{ 8, 16, 1 },		// 75 bits per transform
			{ 8, 17, 0 },		// 75 bits per transform
			{ 9, 0, 16 },		// 75 bits per transform
			{ 9, 1, 15 },		// 75 bits per transform
			{ 9, 2, 14 },		// 75 bits per transform
			{ 9, 3, 13 },		// 75 bits per transform
			{ 9, 4, 12 },		// 75 bits per transform
			{ 9, 5, 11 },		// 75 bits per transform
			{ 9, 6, 10 },		// 75 bits per transform
			{ 9, 7, 9 },		// 75 bits per transform
			{ 9, 8, 8 },		// 75 bits per transform
			{ 9, 9, 7 },		// 75 bits per transform
			{ 9, 10, 6 },		// 75 bits per transform
			{ 9, 11, 5 },		// 75 bits per transform
			{ 9, 12, 4 },		// 75 bits per transform
			{ 9, 13, 3 },		// 75 bits per transform
			{ 9, 14, 2 },		// 75 bits per transform
			{ 9, 15, 1 },		// 75 bits per transform
			{ 9, 16, 0 },		// 75 bits per transform
			{ 10, 0, 15 },		// 75 bits per transform
			{ 10, 1, 14 },		// 75 bits per transform
			{ 10, 2, 13 },		// 75 bits per transform
			{ 10, 3, 12 },		// 75 bits per transform
			{ 10, 4, 11 },		// 75 bits per transform
			{ 10, 5, 10 },		// 75 bits per transform
			{ 10, 6, 9 },		// 75 bits per transform
			{ 10, 7, 8 },		// 75 bits per transform
			{ 10, 8, 7 },		// 75 bits per transform
			{ 10, 9, 6 },		// 75 bits per transform
			{ 10, 10, 5 },		// 75 bits per transform
			{ 10, 11, 4 },		// 75 bits per transform
			{ 10, 12, 3 },		// 75 bits per transform
			{ 10, 13, 2 },		// 75 bits per transform
			{ 10, 14, 1 },		// 75 bits per transform
			{ 10, 15, 0 },		// 75 bits per transform
			{ 11, 0, 14 },		// 75 bits per transform
			{ 11, 1, 13 },		// 75 bits per transform
			{ 11, 2, 12 },		// 75 bits per transform
			{ 11, 3, 11 },		// 75 bits per transform
			{ 11, 4, 10 },		// 75 bits per transform
			{ 11, 5, 9 },		// 75 bits per transform
			{ 11, 6, 8 },		// 75 bits per transform
			{ 11, 7, 7 },		// 75 bits per transform
			{ 11, 8, 6 },		// 75 bits per transform
			{ 11, 9, 5 },		// 75 bits per transform
			{ 11, 10, 4 },		// 75 bits per transform
			{ 11, 11, 3 },		// 75 bits per transform
			{ 11, 12, 2 },		// 75 bits per transform
			{ 11, 13, 1 },		// 75 bits per transform
			{ 11, 14, 0 },		// 75 bits per transform
			{ 12, 0, 13 },		// 75 bits per transform
			{ 12, 1, 12 },		// 75 bits per transform
			{ 12, 2, 11 },		// 75 bits per transform
			{ 12, 3, 10 },		// 75 bits per transform
			{ 12, 4, 9 },		// 75 bits per transform
			{ 12, 5, 8 },		// 75 bits per transform
			{ 12, 6, 7 },		// 75 bits per transform
			{ 12, 7, 6 },		// 75 bits per transform
			{ 12, 8, 5 },		// 75 bits per transform
			{ 12, 9, 4 },		// 75 bits per transform
			{ 12, 10, 3 },		// 75 bits per transform
			{ 12, 11, 2 },		// 75 bits per transform
			{ 12, 12, 1 },		// 75 bits per transform
			{ 12, 13, 0 },		// 75 bits per transform
			{ 13, 0, 12 },		// 75 bits per transform
			{ 13, 1, 11 },		// 75 bits per transform
			{ 13, 2, 10 },		// 75 bits per transform
			{ 13, 3, 9 },		// 75 bits per transform
			{ 13, 4, 8 },		// 75 bits per transform
			{ 13, 5, 7 },		// 75 bits per transform
			{ 13, 6, 6 },		// 75 bits per transform
			{ 13, 7, 5 },		// 75 bits per transform
			{ 13, 8, 4 },		// 75 bits per transform
			{ 13, 9, 3 },		// 75 bits per transform
			{ 13, 10, 2 },		// 75 bits per transform
			{ 13, 11, 1 },		// 75 bits per transform
			{ 13, 12, 0 },		// 75 bits per transform
			{ 14, 0, 11 },		// 75 bits per transform
			{ 14, 1, 10 },		// 75 bits per transform
			{ 14, 2, 9 },		// 75 bits per transform
			{ 14, 3, 8 },		// 75 bits per transform
			{ 14, 4, 7 },		// 75 bits per transform
			{ 14, 5, 6 },		// 75 bits per transform
			{ 14, 6, 5 },		// 75 bits per transform
			{ 14, 7, 4 },		// 75 bits per transform
			{ 14, 8, 3 },		// 75 bits per transform
			{ 14, 9, 2 },		// 75 bits per transform
			{ 14, 10, 1 },		// 75 bits per transform
			{ 14, 11, 0 },		// 75 bits per transform
			{ 15, 0, 10 },		// 75 bits per transform
			{ 15, 1, 9 },		// 75 bits per transform
			{ 15, 2, 8 },		// 75 bits per transform
			{ 15, 3, 7 },		// 75 bits per transform
			{ 15, 4, 6 },		// 75 bits per transform
			{ 15, 5, 5 },		// 75 bits per transform
			{ 15, 6, 4 },		// 75 bits per transform
			{ 15, 7, 3 },		// 75 bits per transform
			{ 15, 8, 2 },		// 75 bits per transform
			{ 15, 9, 1 },		// 75 bits per transform
			{ 15, 10, 0 },		// 75 bits per transform
			{ 16, 0, 9 },		// 75 bits per transform
			{ 16, 1, 8 },		// 75 bits per transform
			{ 16, 2, 7 },		// 75 bits per transform
			{ 16, 3, 6 },		// 75 bits per transform
			{ 16, 4, 5 },		// 75 bits per transform
			{ 16, 5, 4 },		// 75 bits per transform
			{ 16, 6, 3 },		// 75 bits per transform
			{ 16, 7, 2 },		// 75 bits per transform
			{ 16, 8, 1 },		// 75 bits per transform
			{ 16, 9, 0 },		// 75 bits per transform
			{ 17, 0, 8 },		// 75 bits per transform
			{ 17, 1, 7 },		// 75 bits per transform
			{ 17, 2, 6 },		// 75 bits per transform
			{ 17, 3, 5 },		// 75 bits per transform
			{ 17, 4, 4 },		// 75 bits per transform
			{ 17, 5, 3 },		// 75 bits per transform
			{ 17, 6, 2 },		// 75 bits per transform
			{ 17, 7, 1 },		// 75 bits per transform
			{ 17, 8, 0 },		// 75 bits per transform
			{ 18, 0, 7 },		// 75 bits per transform
			{ 18, 1, 6 },		// 75 bits per transform
			{ 18, 2, 5 },		// 75 bits per transform
			{ 18, 3, 4 },		// 75 bits per transform
			{ 18, 4, 3 },		// 75 bits per transform
			{ 18, 5, 2 },		// 75 bits per transform
			{ 18, 6, 1 },		// 75 bits per transform
			{ 18, 7, 0 },		// 75 bits per transform
			{ 19, 0, 6 },		// 75 bits per transform
			{ 19, 1, 5 },		// 75 bits per transform
			{ 19, 2, 4 },		// 75 bits per transform
			{ 19, 3, 3 },		// 75 bits per transform
			{ 19, 4, 2 },		// 75 bits per transform
			{ 19, 5, 1 },		// 75 bits per transform
			{ 19, 6, 0 },		// 75 bits per transform
			{ 20, 0, 5 },		// 75 bits per transform
			{ 20, 1, 4 },		// 75 bits per transform
			{ 20, 2, 3 },		// 75 bits per transform
			{ 20, 3, 2 },		// 75 bits per transform
			{ 20, 4, 1 },		// 75 bits per transform
			{ 20, 5, 0 },		// 75 bits per transform
			{ 21, 0, 4 },		// 75 bits per transform
			{ 21, 1, 3 },		// 75 bits per transform
			{ 21, 2, 2 },		// 75 bits per transform
			{ 21, 3, 1 },		// 75 bits per transform
			{ 21, 4, 0 },		// 75 bits per transform
			{ 22, 0, 3 },		// 75 bits per transform
			{ 22, 1, 2 },		// 75 bits per transform
			{ 22, 2, 1 },		// 75 bits per transform
			{ 22, 3, 0 },		// 75 bits per transform
			{ 23, 0, 2 },		// 75 bits per transform
			{ 23, 1, 1 },		// 75 bits per transform
			{ 23, 2, 0 },		// 75 bits per transform
			{ 0, 3, 23 },		// 78 bits per transform
			{ 0, 4, 22 },		// 78 bits per transform
			{ 0, 5, 21 },		// 78 bits per transform
			{ 0, 6, 20 },		// 78 bits per transform
			{ 0, 7, 19 },		// 78 bits per transform
			{ 0, 8, 18 },		// 78 bits per transform
			{ 0, 9, 17 },		// 78 bits per transform
			{ 0, 10, 16 },		// 78 bits per transform
			{ 0, 11, 15 },		// 78 bits per transform
			{ 0, 12, 14 },		// 78 bits per transform
			{ 0, 13, 13 },		// 78 bits per transform
			{ 0, 14, 12 },		// 78 bits per transform
			{ 0, 15, 11 },		// 78 bits per transform
			{ 0, 16, 10 },		// 78 bits per transform
			{ 0, 17, 9 },		// 78 bits per transform
			{ 0, 18, 8 },		// 78 bits per transform
			{ 0, 19, 7 },		// 78 bits per transform
			{ 0, 20, 6 },		// 78 bits per transform
			{ 0, 21, 5 },		// 78 bits per transform
			{ 0, 22, 4 },		// 78 bits per transform
			{ 0, 23, 3 },		// 78 bits per transform
			{ 1, 2, 23 },		// 78 bits per transform
			{ 1, 3, 22 },		// 78 bits per transform
			{ 1, 4, 21 },		// 78 bits per transform
			{ 1, 5, 20 },		// 78 bits per transform
			{ 1, 6, 19 },		// 78 bits per transform
			{ 1, 7, 18 },		// 78 bits per transform
			{ 1, 8, 17 },		// 78 bits per transform
			{ 1, 9, 16 },		// 78 bits per transform
			{ 1, 10, 15 },		// 78 bits per transform
			{ 1, 11, 14 },		// 78 bits per transform
			{ 1, 12, 13 },		// 78 bits per transform
			{ 1, 13, 12 },		// 78 bits per transform
			{ 1, 14, 11 },		// 78 bits per transform
			{ 1, 15, 10 },		// 78 bits per transform
			{ 1, 16, 9 },		// 78 bits per transform
			{ 1, 17, 8 },		// 78 bits per transform
			{ 1, 18, 7 },		// 78 bits per transform
			{ 1, 19, 6 },		// 78 bits per transform
			{ 1, 20, 5 },		// 78 bits per transform
			{ 1, 21, 4 },		// 78 bits per transform
			{ 1, 22, 3 },		// 78 bits per transform
			{ 1, 23, 2 },		// 78 bits per transform
			{ 2, 1, 23 },		// 78 bits per transform
			{ 2, 2, 22 },		// 78 bits per transform
			{ 2, 3, 21 },		// 78 bits per transform
			{ 2, 4, 20 },		// 78 bits per transform
			{ 2, 5, 19 },		// 78 bits per transform
			{ 2, 6, 18 },		// 78 bits per transform
			{ 2, 7, 17 },		// 78 bits per transform
			{ 2, 8, 16 },		// 78 bits per transform
			{ 2, 9, 15 },		// 78 bits per transform
			{ 2, 10, 14 },		// 78 bits per transform
			{ 2, 11, 13 },		// 78 bits per transform
			{ 2, 12, 12 },		// 78 bits per transform
			{ 2, 13, 11 },		// 78 bits per transform
			{ 2, 14, 10 },		// 78 bits per transform
			{ 2, 15, 9 },		// 78 bits per transform
			{ 2, 16, 8 },		// 78 bits per transform
			{ 2, 17, 7 },		// 78 bits per transform
			{ 2, 18, 6 },		// 78 bits per transform
			{ 2, 19, 5 },		// 78 bits per transform
			{ 2, 20, 4 },		// 78 bits per transform
			{ 2, 21, 3 },		// 78 bits per transform
			{ 2, 22, 2 },		// 78 bits per transform
			{ 2, 23, 1 },		// 78 bits per transform
			{ 3, 0, 23 },		// 78 bits per transform
			{ 3, 1, 22 },		// 78 bits per transform
			{ 3, 2, 21 },		// 78 bits per transform
			{ 3, 3, 20 },		// 78 bits per transform
			{ 3, 4, 19 },		// 78 bits per transform
			{ 3, 5, 18 },		// 78 bits per transform
			{ 3, 6, 17 },		// 78 bits per transform
			{ 3, 7, 16 },		// 78 bits per transform
			{ 3, 8, 15 },		// 78 bits per transform
			{ 3, 9, 14 },		// 78 bits per transform
			{ 3, 10, 13 },		// 78 bits per transform
			{ 3, 11, 12 },		// 78 bits per transform
			{ 3, 12, 11 },		// 78 bits per transform
			{ 3, 13, 10 },		// 78 bits per transform
			{ 3, 14, 9 },		// 78 bits per transform
			{ 3, 15, 8 },		// 78 bits per transform
			{ 3, 16, 7 },		// 78 bits per transform
			{ 3, 17, 6 },		// 78 bits per transform
			{ 3, 18, 5 },		// 78 bits per transform
			{ 3, 19, 4 },		// 78 bits per transform
			{ 3, 20, 3 },		// 78 bits per transform
			{ 3, 21, 2 },		// 78 bits per transform
			{ 3, 22, 1 },		// 78 bits per transform
			{ 3, 23, 0 },		// 78 bits per transform
			{ 4, 0, 22 },		// 78 bits per transform
			{ 4, 1, 21 },		// 78 bits per transform
			{ 4, 2, 20 },		// 78 bits per transform
			{ 4, 3, 19 },		// 78 bits per transform
			{ 4, 4, 18 },		// 78 bits per transform
			{ 4, 5, 17 },		// 78 bits per transform
			{ 4, 6, 16 },		// 78 bits per transform
			{ 4, 7, 15 },		// 78 bits per transform
			{ 4, 8, 14 },		// 78 bits per transform
			{ 4, 9, 13 },		// 78 bits per transform
			{ 4, 10, 12 },		// 78 bits per transform
			{ 4, 11, 11 },		// 78 bits per transform
			{ 4, 12, 10 },		// 78 bits per transform
			{ 4, 13, 9 },		// 78 bits per transform
			{ 4, 14, 8 },		// 78 bits per transform
			{ 4, 15, 7 },		// 78 bits per transform
			{ 4, 16, 6 },		// 78 bits per transform
			{ 4, 17, 5 },		// 78 bits per transform
			{ 4, 18, 4 },		// 78 bits per transform
			{ 4, 19, 3 },		// 78 bits per transform
			{ 4, 20, 2 },		// 78 bits per transform
			{ 4, 21, 1 },		// 78 bits per transform
			{ 4, 22, 0 },		// 78 bits per transform
			{ 5, 0, 21 },		// 78 bits per transform
			{ 5, 1, 20 },		// 78 bits per transform
			{ 5, 2, 19 },		// 78 bits per transform
			{ 5, 3, 18 },		// 78 bits per transform
			{ 5, 4, 17 },		// 78 bits per transform
			{ 5, 5, 16 },		// 78 bits per transform
			{ 5, 6, 15 },		// 78 bits per transform
			{ 5, 7, 14 },		// 78 bits per transform
			{ 5, 8, 13 },		// 78 bits per transform
			{ 5, 9, 12 },		// 78 bits per transform
			{ 5, 10, 11 },		// 78 bits per transform
			{ 5, 11, 10 },		// 78 bits per transform
			{ 5, 12, 9 },		// 78 bits per transform
			{ 5, 13, 8 },		// 78 bits per transform
			{ 5, 14, 7 },		// 78 bits per transform
			{ 5, 15, 6 },		// 78 bits per transform
			{ 5, 16, 5 },		// 78 bits per transform
			{ 5, 17, 4 },		// 78 bits per transform
			{ 5, 18, 3 },		// 78 bits per transform
			{ 5, 19, 2 },		// 78 bits per transform
			{ 5, 20, 1 },		// 78 bits per transform
			{ 5, 21, 0 },		// 78 bits per transform
			{ 6, 0, 20 },		// 78 bits per transform
			{ 6, 1, 19 },		// 78 bits per transform
			{ 6, 2, 18 },		// 78 bits per transform
			{ 6, 3, 17 },		// 78 bits per transform
			{ 6, 4, 16 },		// 78 bits per transform
			{ 6, 5, 15 },		// 78 bits per transform
			{ 6, 6, 14 },		// 78 bits per transform
			{ 6, 7, 13 },		// 78 bits per transform
			{ 6, 8, 12 },		// 78 bits per transform
			{ 6, 9, 11 },		// 78 bits per transform
			{ 6, 10, 10 },		// 78 bits per transform
			{ 6, 11, 9 },		// 78 bits per transform
			{ 6, 12, 8 },		// 78 bits per transform
			{ 6, 13, 7 },		// 78 bits per transform
			{ 6, 14, 6 },		// 78 bits per transform
			{ 6, 15, 5 },		// 78 bits per transform
			{ 6, 16, 4 },		// 78 bits per transform
			{ 6, 17, 3 },		// 78 bits per transform
			{ 6, 18, 2 },		// 78 bits per transform
			{ 6, 19, 1 },		// 78 bits per transform
			{ 6, 20, 0 },		// 78 bits per transform
			{ 7, 0, 19 },		// 78 bits per transform
			{ 7, 1, 18 },		// 78 bits per transform
			{ 7, 2, 17 },		// 78 bits per transform
			{ 7, 3, 16 },		// 78 bits per transform
			{ 7, 4, 15 },		// 78 bits per transform
			{ 7, 5, 14 },		// 78 bits per transform
			{ 7, 6, 13 },		// 78 bits per transform
			{ 7, 7, 12 },		// 78 bits per transform
			{ 7, 8, 11 },		// 78 bits per transform
			{ 7, 9, 10 },		// 78 bits per transform
			{ 7, 10, 9 },		// 78 bits per transform
			{ 7, 11, 8 },		// 78 bits per transform
			{ 7, 12, 7 },		// 78 bits per transform
			{ 7, 13, 6 },		// 78 bits per transform
			{ 7, 14, 5 },		// 78 bits per transform
			{ 7, 15, 4 },		// 78 bits per transform
			{ 7, 16, 3 },		// 78 bits per transform
			{ 7, 17, 2 },		// 78 bits per transform
			{ 7, 18, 1 },		// 78 bits per transform
			{ 7, 19, 0 },		// 78 bits per transform
			{ 8, 0, 18 },		// 78 bits per transform
			{ 8, 1, 17 },		// 78 bits per transform
			{ 8, 2, 16 },		// 78 bits per transform
			{ 8, 3, 15 },		// 78 bits per transform
			{ 8, 4, 14 },		// 78 bits per transform
			{ 8, 5, 13 },		// 78 bits per transform
			{ 8, 6, 12 },		// 78 bits per transform
			{ 8, 7, 11 },		// 78 bits per transform
			{ 8, 8, 10 },		// 78 bits per transform
			{ 8, 9, 9 },		// 78 bits per transform
			{ 8, 10, 8 },		// 78 bits per transform
			{ 8, 11, 7 },		// 78 bits per transform
			{ 8, 12, 6 },		// 78 bits per transform
			{ 8, 13, 5 },		// 78 bits per transform
			{ 8, 14, 4 },		// 78 bits per transform
			{ 8, 15, 3 },		// 78 bits per transform
			{ 8, 16, 2 },		// 78 bits per transform
			{ 8, 17, 1 },		// 78 bits per transform
			{ 8, 18, 0 },		// 78 bits per transform
			{ 9, 0, 17 },		// 78 bits per transform
			{ 9, 1, 16 },		// 78 bits per transform
			{ 9, 2, 15 },		// 78 bits per transform
			{ 9, 3, 14 },		// 78 bits per transform
			{ 9, 4, 13 },		// 78 bits per transform
			{ 9, 5, 12 },		// 78 bits per transform
			{ 9, 6, 11 },		// 78 bits per transform
			{ 9, 7, 10 },		// 78 bits per transform
			{ 9, 8, 9 },		// 78 bits per transform
			{ 9, 9, 8 },		// 78 bits per transform
			{ 9, 10, 7 },		// 78 bits per transform
			{ 9, 11, 6 },		// 78 bits per transform
			{ 9, 12, 5 },		// 78 bits per transform
			{ 9, 13, 4 },		// 78 bits per transform
			{ 9, 14, 3 },		// 78 bits per transform
			{ 9, 15, 2 },		// 78 bits per transform
			{ 9, 16, 1 },		// 78 bits per transform
			{ 9, 17, 0 },		// 78 bits per transform
			{ 10, 0, 16 },		// 78 bits per transform
			{ 10, 1, 15 },		// 78 bits per transform
			{ 10, 2, 14 },		// 78 bits per transform
			{ 10, 3, 13 },		// 78 bits per transform
			{ 10, 4, 12 },		// 78 bits per transform
			{ 10, 5, 11 },		// 78 bits per transform
			{ 10, 6, 10 },		// 78 bits per transform
			{ 10, 7, 9 },		// 78 bits per transform
			{ 10, 8, 8 },		// 78 bits per transform
			{ 10, 9, 7 },		// 78 bits per transform
			{ 10, 10, 6 },		// 78 bits per transform
			{ 10, 11, 5 },		// 78 bits per transform
			{ 10, 12, 4 },		// 78 bits per transform
			{ 10, 13, 3 },		// 78 bits per transform
			{ 10, 14, 2 },		// 78 bits per transform
			{ 10, 15, 1 },		// 78 bits per transform
			{ 10, 16, 0 },		// 78 bits per transform
			{ 11, 0, 15 },		// 78 bits per transform
			{ 11, 1, 14 },		// 78 bits per transform
			{ 11, 2, 13 },		// 78 bits per transform
			{ 11, 3, 12 },		// 78 bits per transform
			{ 11, 4, 11 },		// 78 bits per transform
			{ 11, 5, 10 },		// 78 bits per transform
			{ 11, 6, 9 },		// 78 bits per transform
			{ 11, 7, 8 },		// 78 bits per transform
			{ 11, 8, 7 },		// 78 bits per transform
			{ 11, 9, 6 },		// 78 bits per transform
			{ 11, 10, 5 },		// 78 bits per transform
			{ 11, 11, 4 },		// 78 bits per transform
			{ 11, 12, 3 },		// 78 bits per transform
			{ 11, 13, 2 },		// 78 bits per transform
			{ 11, 14, 1 },		// 78 bits per transform
			{ 11, 15, 0 },		// 78 bits per transform
			{ 12, 0, 14 },		// 78 bits per transform
			{ 12, 1, 13 },		// 78 bits per transform
			{ 12, 2, 12 },		// 78 bits per transform
			{ 12, 3, 11 },		// 78 bits per transform
			{ 12, 4, 10 },		// 78 bits per transform
			{ 12, 5, 9 },		// 78 bits per transform
			{ 12, 6, 8 },		// 78 bits per transform
			{ 12, 7, 7 },		// 78 bits per transform
			{ 12, 8, 6 },		// 78 bits per transform
			{ 12, 9, 5 },		// 78 bits per transform
			{ 12, 10, 4 },		// 78 bits per transform
			{ 12, 11, 3 },		// 78 bits per transform
			{ 12, 12, 2 },		// 78 bits per transform
			{ 12, 13, 1 },		// 78 bits per transform
			{ 12, 14, 0 },		// 78 bits per transform
			{ 13, 0, 13 },		// 78 bits per transform
			{ 13, 1, 12 },		// 78 bits per transform
			{ 13, 2, 11 },		// 78 bits per transform
			{ 13, 3, 10 },		// 78 bits per transform
			{ 13, 4, 9 },		// 78 bits per transform
			{ 13, 5, 8 },		// 78 bits per transform
			{ 13, 6, 7 },		// 78 bits per transform
			{ 13, 7, 6 },		// 78 bits per transform
			{ 13, 8, 5 },		// 78 bits per transform
			{ 13, 9, 4 },		// 78 bits per transform
			{ 13, 10, 3 },		// 78 bits per transform
			{ 13, 11, 2 },		// 78 bits per transform
			{ 13, 12, 1 },		// 78 bits per transform
			{ 13, 13, 0 },		// 78 bits per transform
			{ 14, 0, 12 },		// 78 bits per transform
			{ 14, 1, 11 },		// 78 bits per transform
			{ 14, 2, 10 },		// 78 bits per transform
			{ 14, 3, 9 },		// 78 bits per transform
			{ 14, 4, 8 },		// 78 bits per transform
			{ 14, 5, 7 },		// 78 bits per transform
			{ 14, 6, 6 },		// 78 bits per transform
			{ 14, 7, 5 },		// 78 bits per transform
			{ 14, 8, 4 },		// 78 bits per transform
			{ 14, 9, 3 },		// 78 bits per transform
			{ 14, 10, 2 },		// 78 bits per transform
			{ 14, 11, 1 },		// 78 bits per transform
			{ 14, 12, 0 },		// 78 bits per transform
			{ 15, 0, 11 },		// 78 bits per transform
			{ 15, 1, 10 },		// 78 bits per transform
			{ 15, 2, 9 },		// 78 bits per transform
			{ 15, 3, 8 },		// 78 bits per transform
			{ 15, 4, 7 },		// 78 bits per transform
			{ 15, 5, 6 },		// 78 bits per transform
			{ 15, 6, 5 },		// 78 bits per transform
			{ 15, 7, 4 },		// 78 bits per transform
			{ 15, 8, 3 },		// 78 bits per transform
			{ 15, 9, 2 },		// 78 bits per transform
			{ 15, 10, 1 },		// 78 bits per transform
			{ 15, 11, 0 },		// 78 bits per transform
			{ 16, 0, 10 },		// 78 bits per transform
			{ 16, 1, 9 },		// 78 bits per transform
			{ 16, 2, 8 },		// 78 bits per transform
			{ 16, 3, 7 },		// 78 bits per transform
			{ 16, 4, 6 },		// 78 bits per transform
			{ 16, 5, 5 },		// 78 bits per transform
			{ 16, 6, 4 },		// 78 bits per transform
			{ 16, 7, 3 },		// 78 bits per transform
			{ 16, 8, 2 },		// 78 bits per transform
			{ 16, 9, 1 },		// 78 bits per transform
			{ 16, 10, 0 },		// 78 bits per transform
			{ 17, 0, 9 },		// 78 bits per transform
			{ 17, 1, 8 },		// 78 bits per transform
			{ 17, 2, 7 },		// 78 bits per transform
			{ 17, 3, 6 },		// 78 bits per transform
			{ 17, 4, 5 },		// 78 bits per transform
			{ 17, 5, 4 },		// 78 bits per transform
			{ 17, 6, 3 },		// 78 bits per transform
			{ 17, 7, 2 },		// 78 bits per transform
			{ 17, 8, 1 },		// 78 bits per transform
			{ 17, 9, 0 },		// 78 bits per transform
			{ 18, 0, 8 },		// 78 bits per transform
			{ 18, 1, 7 },		// 78 bits per transform
			{ 18, 2, 6 },		// 78 bits per transform
			{ 18, 3, 5 },		// 78 bits per transform
			{ 18, 4, 4 },		// 78 bits per transform
			{ 18, 5, 3 },		// 78 bits per transform
			{ 18, 6, 2 },		// 78 bits per transform
			{ 18, 7, 1 },		// 78 bits per transform
			{ 18, 8, 0 },		// 78 bits per transform
			{ 19, 0, 7 },		// 78 bits per transform
			{ 19, 1, 6 },		// 78 bits per transform
			{ 19, 2, 5 },		// 78 bits per transform
			{ 19, 3, 4 },		// 78 bits per transform
			{ 19, 4, 3 },		// 78 bits per transform
			{ 19, 5, 2 },		// 78 bits per transform
			{ 19, 6, 1 },		// 78 bits per transform
			{ 19, 7, 0 },		// 78 bits per transform
			{ 20, 0, 6 },		// 78 bits per transform
			{ 20, 1, 5 },		// 78 bits per transform
			{ 20, 2, 4 },		// 78 bits per transform
			{ 20, 3, 3 },		// 78 bits per transform
			{ 20, 4, 2 },		// 78 bits per transform
			{ 20, 5, 1 },		// 78 bits per transform
			{ 20, 6, 0 },		// 78 bits per transform
			{ 21, 0, 5 },		// 78 bits per transform
			{ 21, 1, 4 },		// 78 bits per transform
			{ 21, 2, 3 },		// 78 bits per transform
			{ 21, 3, 2 },		// 78 bits per transform
			{ 21, 4, 1 },		// 78 bits per transform
			{ 21, 5, 0 },		// 78 bits per transform
			{ 22, 0, 4 },		// 78 bits per transform
			{ 22, 1, 3 },		// 78 bits per transform
			{ 22, 2, 2 },		// 78 bits per transform
			{ 22, 3, 1 },		// 78 bits per transform
			{ 22, 4, 0 },		// 78 bits per transform
			{ 23, 0, 3 },		// 78 bits per transform
			{ 23, 1, 2 },		// 78 bits per transform
			{ 23, 2, 1 },		// 78 bits per transform
			{ 23, 3, 0 },		// 78 bits per transform
			{ 0, 4, 23 },		// 81 bits per transform
			{ 0, 5, 22 },		// 81 bits per transform
			{ 0, 6, 21 },		// 81 bits per transform
			{ 0, 7, 20 },		// 81 bits per transform
			{ 0, 8, 19 },		// 81 bits per transform
			{ 0, 9, 18 },		// 81 bits per transform
			{ 0, 10, 17 },		// 81 bits per transform
			{ 0, 11, 16 },		// 81 bits per transform
			{ 0, 12, 15 },		// 81 bits per transform
			{ 0, 13, 14 },		// 81 bits per transform
			{ 0, 14, 13 },		// 81 bits per transform
			{ 0, 15, 12 },		// 81 bits per transform
			{ 0, 16, 11 },		// 81 bits per transform
			{ 0, 17, 10 },		// 81 bits per transform
			{ 0, 18, 9 },		// 81 bits per transform
			{ 0, 19, 8 },		// 81 bits per transform
			{ 0, 20, 7 },		// 81 bits per transform
			{ 0, 21, 6 },		// 81 bits per transform
			{ 0, 22, 5 },		// 81 bits per transform
			{ 0, 23, 4 },		// 81 bits per transform
			{ 1, 3, 23 },		// 81 bits per transform
			{ 1, 4, 22 },		// 81 bits per transform
			{ 1, 5, 21 },		// 81 bits per transform
			{ 1, 6, 20 },		// 81 bits per transform
			{ 1, 7, 19 },		// 81 bits per transform
			{ 1, 8, 18 },		// 81 bits per transform
			{ 1, 9, 17 },		// 81 bits per transform
			{ 1, 10, 16 },		// 81 bits per transform
			{ 1, 11, 15 },		// 81 bits per transform
			{ 1, 12, 14 },		// 81 bits per transform
			{ 1, 13, 13 },		// 81 bits per transform
			{ 1, 14, 12 },		// 81 bits per transform
			{ 1, 15, 11 },		// 81 bits per transform
			{ 1, 16, 10 },		// 81 bits per transform
			{ 1, 17, 9 },		// 81 bits per transform
			{ 1, 18, 8 },		// 81 bits per transform
			{ 1, 19, 7 },		// 81 bits per transform
			{ 1, 20, 6 },		// 81 bits per transform
			{ 1, 21, 5 },		// 81 bits per transform
			{ 1, 22, 4 },		// 81 bits per transform
			{ 1, 23, 3 },		// 81 bits per transform
			{ 2, 2, 23 },		// 81 bits per transform
			{ 2, 3, 22 },		// 81 bits per transform
			{ 2, 4, 21 },		// 81 bits per transform
			{ 2, 5, 20 },		// 81 bits per transform
			{ 2, 6, 19 },		// 81 bits per transform
			{ 2, 7, 18 },		// 81 bits per transform
			{ 2, 8, 17 },		// 81 bits per transform
			{ 2, 9, 16 },		// 81 bits per transform
			{ 2, 10, 15 },		// 81 bits per transform
			{ 2, 11, 14 },		// 81 bits per transform
			{ 2, 12, 13 },		// 81 bits per transform
			{ 2, 13, 12 },		// 81 bits per transform
			{ 2, 14, 11 },		// 81 bits per transform
			{ 2, 15, 10 },		// 81 bits per transform
			{ 2, 16, 9 },		// 81 bits per transform
			{ 2, 17, 8 },		// 81 bits per transform
			{ 2, 18, 7 },		// 81 bits per transform
			{ 2, 19, 6 },		// 81 bits per transform
			{ 2, 20, 5 },		// 81 bits per transform
			{ 2, 21, 4 },		// 81 bits per transform
			{ 2, 22, 3 },		// 81 bits per transform
			{ 2, 23, 2 },		// 81 bits per transform
			{ 3, 1, 23 },		// 81 bits per transform
			{ 3, 2, 22 },		// 81 bits per transform
			{ 3, 3, 21 },		// 81 bits per transform
			{ 3, 4, 20 },		// 81 bits per transform
			{ 3, 5, 19 },		// 81 bits per transform
			{ 3, 6, 18 },		// 81 bits per transform
			{ 3, 7, 17 },		// 81 bits per transform
			{ 3, 8, 16 },		// 81 bits per transform
			{ 3, 9, 15 },		// 81 bits per transform
			{ 3, 10, 14 },		// 81 bits per transform
			{ 3, 11, 13 },		// 81 bits per transform
			{ 3, 12, 12 },		// 81 bits per transform
			{ 3, 13, 11 },		// 81 bits per transform
			{ 3, 14, 10 },		// 81 bits per transform
			{ 3, 15, 9 },		// 81 bits per transform
			{ 3, 16, 8 },		// 81 bits per transform
			{ 3, 17, 7 },		// 81 bits per transform
			{ 3, 18, 6 },		// 81 bits per transform
			{ 3, 19, 5 },		// 81 bits per transform
			{ 3, 20, 4 },		// 81 bits per transform
			{ 3, 21, 3 },		// 81 bits per transform
			{ 3, 22, 2 },		// 81 bits per transform
			{ 3, 23, 1 },		// 81 bits per transform
			{ 4, 0, 23 },		// 81 bits per transform
			{ 4, 1, 22 },		// 81 bits per transform
			{ 4, 2, 21 },		// 81 bits per transform
			{ 4, 3, 20 },		// 81 bits per transform
			{ 4, 4, 19 },		// 81 bits per transform
			{ 4, 5, 18 },		// 81 bits per transform
			{ 4, 6, 17 },		// 81 bits per transform
			{ 4, 7, 16 },		// 81 bits per transform
			{ 4, 8, 15 },		// 81 bits per transform
			{ 4, 9, 14 },		// 81 bits per transform
			{ 4, 10, 13 },		// 81 bits per transform
			{ 4, 11, 12 },		// 81 bits per transform
			{ 4, 12, 11 },		// 81 bits per transform
			{ 4, 13, 10 },		// 81 bits per transform
			{ 4, 14, 9 },		// 81 bits per transform
			{ 4, 15, 8 },		// 81 bits per transform
			{ 4, 16, 7 },		// 81 bits per transform
			{ 4, 17, 6 },		// 81 bits per transform
			{ 4, 18, 5 },		// 81 bits per transform
			{ 4, 19, 4 },		// 81 bits per transform
			{ 4, 20, 3 },		// 81 bits per transform
			{ 4, 21, 2 },		// 81 bits per transform
			{ 4, 22, 1 },		// 81 bits per transform
			{ 4, 23, 0 },		// 81 bits per transform
			{ 5, 0, 22 },		// 81 bits per transform
			{ 5, 1, 21 },		// 81 bits per transform
			{ 5, 2, 20 },		// 81 bits per transform
			{ 5, 3, 19 },		// 81 bits per transform
			{ 5, 4, 18 },		// 81 bits per transform
			{ 5, 5, 17 },		// 81 bits per transform
			{ 5, 6, 16 },		// 81 bits per transform
			{ 5, 7, 15 },		// 81 bits per transform
			{ 5, 8, 14 },		// 81 bits per transform
			{ 5, 9, 13 },		// 81 bits per transform
			{ 5, 10, 12 },		// 81 bits per transform
			{ 5, 11, 11 },		// 81 bits per transform
			{ 5, 12, 10 },		// 81 bits per transform
			{ 5, 13, 9 },		// 81 bits per transform
			{ 5, 14, 8 },		// 81 bits per transform
			{ 5, 15, 7 },		// 81 bits per transform
			{ 5, 16, 6 },		// 81 bits per transform
			{ 5, 17, 5 },		// 81 bits per transform
			{ 5, 18, 4 },		// 81 bits per transform
			{ 5, 19, 3 },		// 81 bits per transform
			{ 5, 20, 2 },		// 81 bits per transform
			{ 5, 21, 1 },		// 81 bits per transform
			{ 5, 22, 0 },		// 81 bits per transform
			{ 6, 0, 21 },		// 81 bits per transform
			{ 6, 1, 20 },		// 81 bits per transform
			{ 6, 2, 19 },		// 81 bits per transform
			{ 6, 3, 18 },		// 81 bits per transform
			{ 6, 4, 17 },		// 81 bits per transform
			{ 6, 5, 16 },		// 81 bits per transform
			{ 6, 6, 15 },		// 81 bits per transform
			{ 6, 7, 14 },		// 81 bits per transform
			{ 6, 8, 13 },		// 81 bits per transform
			{ 6, 9, 12 },		// 81 bits per transform
			{ 6, 10, 11 },		// 81 bits per transform
			{ 6, 11, 10 },		// 81 bits per transform
			{ 6, 12, 9 },		// 81 bits per transform
			{ 6, 13, 8 },		// 81 bits per transform
			{ 6, 14, 7 },		// 81 bits per transform
			{ 6, 15, 6 },		// 81 bits per transform
			{ 6, 16, 5 },		// 81 bits per transform
			{ 6, 17, 4 },		// 81 bits per transform
			{ 6, 18, 3 },		// 81 bits per transform
			{ 6, 19, 2 },		// 81 bits per transform
			{ 6, 20, 1 },		// 81 bits per transform
			{ 6, 21, 0 },		// 81 bits per transform
			{ 7, 0, 20 },		// 81 bits per transform
			{ 7, 1, 19 },		// 81 bits per transform
			{ 7, 2, 18 },		// 81 bits per transform
			{ 7, 3, 17 },		// 81 bits per transform
			{ 7, 4, 16 },		// 81 bits per transform
			{ 7, 5, 15 },		// 81 bits per transform
			{ 7, 6, 14 },		// 81 bits per transform
			{ 7, 7, 13 },		// 81 bits per transform
			{ 7, 8, 12 },		// 81 bits per transform
			{ 7, 9, 11 },		// 81 bits per transform
			{ 7, 10, 10 },		// 81 bits per transform
			{ 7, 11, 9 },		// 81 bits per transform
			{ 7, 12, 8 },		// 81 bits per transform
			{ 7, 13, 7 },		// 81 bits per transform
			{ 7, 14, 6 },		// 81 bits per transform
			{ 7, 15, 5 },		// 81 bits per transform
			{ 7, 16, 4 },		// 81 bits per transform
			{ 7, 17, 3 },		// 81 bits per transform
			{ 7, 18, 2 },		// 81 bits per transform
			{ 7, 19, 1 },		// 81 bits per transform
			{ 7, 20, 0 },		// 81 bits per transform
			{ 8, 0, 19 },		// 81 bits per transform
			{ 8, 1, 18 },		// 81 bits per transform
			{ 8, 2, 17 },		// 81 bits per transform
			{ 8, 3, 16 },		// 81 bits per transform
			{ 8, 4, 15 },		// 81 bits per transform
			{ 8, 5, 14 },		// 81 bits per transform
			{ 8, 6, 13 },		// 81 bits per transform
			{ 8, 7, 12 },		// 81 bits per transform
			{ 8, 8, 11 },		// 81 bits per transform
			{ 8, 9, 10 },		// 81 bits per transform
			{ 8, 10, 9 },		// 81 bits per transform
			{ 8, 11, 8 },		// 81 bits per transform
			{ 8, 12, 7 },		// 81 bits per transform
			{ 8, 13, 6 },		// 81 bits per transform
			{ 8, 14, 5 },		// 81 bits per transform
			{ 8, 15, 4 },		// 81 bits per transform
			{ 8, 16, 3 },		// 81 bits per transform
			{ 8, 17, 2 },		// 81 bits per transform
			{ 8, 18, 1 },		// 81 bits per transform
			{ 8, 19, 0 },		// 81 bits per transform
			{ 9, 0, 18 },		// 81 bits per transform
			{ 9, 1, 17 },		// 81 bits per transform
			{ 9, 2, 16 },		// 81 bits per transform
			{ 9, 3, 15 },		// 81 bits per transform
			{ 9, 4, 14 },		// 81 bits per transform
			{ 9, 5, 13 },		// 81 bits per transform
			{ 9, 6, 12 },		// 81 bits per transform
			{ 9, 7, 11 },		// 81 bits per transform
			{ 9, 8, 10 },		// 81 bits per transform
			{ 9, 9, 9 },		// 81 bits per transform
			{ 9, 10, 8 },		// 81 bits per transform
			{ 9, 11, 7 },		// 81 bits per transform
			{ 9, 12, 6 },		// 81 bits per transform
			{ 9, 13, 5 },		// 81 bits per transform
			{ 9, 14, 4 },		// 81 bits per transform
			{ 9, 15, 3 },		// 81 bits per transform
			{ 9, 16, 2 },		// 81 bits per transform
			{ 9, 17, 1 },		// 81 bits per transform
			{ 9, 18, 0 },		// 81 bits per transform
			{ 10, 0, 17 },		// 81 bits per transform
			{ 10, 1, 16 },		// 81 bits per transform
			{ 10, 2, 15 },		// 81 bits per transform
			{ 10, 3, 14 },		// 81 bits per transform
			{ 10, 4, 13 },		// 81 bits per transform
			{ 10, 5, 12 },		// 81 bits per transform
			{ 10, 6, 11 },		// 81 bits per transform
			{ 10, 7, 10 },		// 81 bits per transform
			{ 10, 8, 9 },		// 81 bits per transform
			{ 10, 9, 8 },		// 81 bits per transform
			{ 10, 10, 7 },		// 81 bits per transform
			{ 10, 11, 6 },		// 81 bits per transform
			{ 10, 12, 5 },		// 81 bits per transform
			{ 10, 13, 4 },		// 81 bits per transform
			{ 10, 14, 3 },		// 81 bits per transform
			{ 10, 15, 2 },		// 81 bits per transform
			{ 10, 16, 1 },		// 81 bits per transform
			{ 10, 17, 0 },		// 81 bits per transform
			{ 11, 0, 16 },		// 81 bits per transform
			{ 11, 1, 15 },		// 81 bits per transform
			{ 11, 2, 14 },		// 81 bits per transform
			{ 11, 3, 13 },		// 81 bits per transform
			{ 11, 4, 12 },		// 81 bits per transform
			{ 11, 5, 11 },		// 81 bits per transform
			{ 11, 6, 10 },		// 81 bits per transform
			{ 11, 7, 9 },		// 81 bits per transform
			{ 11, 8, 8 },		// 81 bits per transform
			{ 11, 9, 7 },		// 81 bits per transform
			{ 11, 10, 6 },		// 81 bits per transform
			{ 11, 11, 5 },		// 81 bits per transform
			{ 11, 12, 4 },		// 81 bits per transform
			{ 11, 13, 3 },		// 81 bits per transform
			{ 11, 14, 2 },		// 81 bits per transform
			{ 11, 15, 1 },		// 81 bits per transform
			{ 11, 16, 0 },		// 81 bits per transform
			{ 12, 0, 15 },		// 81 bits per transform
			{ 12, 1, 14 },		// 81 bits per transform
			{ 12, 2, 13 },		// 81 bits per transform
			{ 12, 3, 12 },		// 81 bits per transform
			{ 12, 4, 11 },		// 81 bits per transform
			{ 12, 5, 10 },		// 81 bits per transform
			{ 12, 6, 9 },		// 81 bits per transform
			{ 12, 7, 8 },		// 81 bits per transform
			{ 12, 8, 7 },		// 81 bits per transform
			{ 12, 9, 6 },		// 81 bits per transform
			{ 12, 10, 5 },		// 81 bits per transform
			{ 12, 11, 4 },		// 81 bits per transform
			{ 12, 12, 3 },		// 81 bits per transform
			{ 12, 13, 2 },		// 81 bits per transform
			{ 12, 14, 1 },		// 81 bits per transform
			{ 12, 15, 0 },		// 81 bits per transform
			{ 13, 0, 14 },		// 81 bits per transform
			{ 13, 1, 13 },		// 81 bits per transform
			{ 13, 2, 12 },		// 81 bits per transform
			{ 13, 3, 11 },		// 81 bits per transform
			{ 13, 4, 10 },		// 81 bits per transform
			{ 13, 5, 9 },		// 81 bits per transform
			{ 13, 6, 8 },		// 81 bits per transform
			{ 13, 7, 7 },		// 81 bits per transform
			{ 13, 8, 6 },		// 81 bits per transform
			{ 13, 9, 5 },		// 81 bits per transform
			{ 13, 10, 4 },		// 81 bits per transform
			{ 13, 11, 3 },		// 81 bits per transform
			{ 13, 12, 2 },		// 81 bits per transform
			{ 13, 13, 1 },		// 81 bits per transform
			{ 13, 14, 0 },		// 81 bits per transform
			{ 14, 0, 13 },		// 81 bits per transform
			{ 14, 1, 12 },		// 81 bits per transform
			{ 14, 2, 11 },		// 81 bits per transform
			{ 14, 3, 10 },		// 81 bits per transform
			{ 14, 4, 9 },		// 81 bits per transform
			{ 14, 5, 8 },		// 81 bits per transform
			{ 14, 6, 7 },		// 81 bits per transform
			{ 14, 7, 6 },		// 81 bits per transform
			{ 14, 8, 5 },		// 81 bits per transform
			{ 14, 9, 4 },		// 81 bits per transform
			{ 14, 10, 3 },		// 81 bits per transform
			{ 14, 11, 2 },		// 81 bits per transform
			{ 14, 12, 1 },		// 81 bits per transform
			{ 14, 13, 0 },		// 81 bits per transform
			{ 15, 0, 12 },		// 81 bits per transform
			{ 15, 1, 11 },		// 81 bits per transform
			{ 15, 2, 10 },		// 81 bits per transform
			{ 15, 3, 9 },		// 81 bits per transform
			{ 15, 4, 8 },		// 81 bits per transform
			{ 15, 5, 7 },		// 81 bits per transform
			{ 15, 6, 6 },		// 81 bits per transform
			{ 15, 7, 5 },		// 81 bits per transform
			{ 15, 8, 4 },		// 81 bits per transform
			{ 15, 9, 3 },		// 81 bits per transform
			{ 15, 10, 2 },		// 81 bits per transform
			{ 15, 11, 1 },		// 81 bits per transform
			{ 15, 12, 0 },		// 81 bits per transform
			{ 16, 0, 11 },		// 81 bits per transform
			{ 16, 1, 10 },		// 81 bits per transform
			{ 16, 2, 9 },		// 81 bits per transform
			{ 16, 3, 8 },		// 81 bits per transform
			{ 16, 4, 7 },		// 81 bits per transform
			{ 16, 5, 6 },		// 81 bits per transform
			{ 16, 6, 5 },		// 81 bits per transform
			{ 16, 7, 4 },		// 81 bits per transform
			{ 16, 8, 3 },		// 81 bits per transform
			{ 16, 9, 2 },		// 81 bits per transform
			{ 16, 10, 1 },		// 81 bits per transform
			{ 16, 11, 0 },		// 81 bits per transform
			{ 17, 0, 10 },		// 81 bits per transform
			{ 17, 1, 9 },		// 81 bits per transform
			{ 17, 2, 8 },		// 81 bits per transform
			{ 17, 3, 7 },		// 81 bits per transform
			{ 17, 4, 6 },		// 81 bits per transform
			{ 17, 5, 5 },		// 81 bits per transform
			{ 17, 6, 4 },		// 81 bits per transform
			{ 17, 7, 3 },		// 81 bits per transform
			{ 17, 8, 2 },		// 81 bits per transform
			{ 17, 9, 1 },		// 81 bits per transform
			{ 17, 10, 0 },		// 81 bits per transform
			{ 18, 0, 9 },		// 81 bits per transform
			{ 18, 1, 8 },		// 81 bits per transform
			{ 18, 2, 7 },		// 81 bits per transform
			{ 18, 3, 6 },		// 81 bits per transform
			{ 18, 4, 5 },		// 81 bits per transform
			{ 18, 5, 4 },		// 81 bits per transform
			{ 18, 6, 3 },		// 81 bits per transform
			{ 18, 7, 2 },		// 81 bits per transform
			{ 18, 8, 1 },		// 81 bits per transform
			{ 18, 9, 0 },		// 81 bits per transform
			{ 19, 0, 8 },		// 81 bits per transform
			{ 19, 1, 7 },		// 81 bits per transform
			{ 19, 2, 6 },		// 81 bits per transform
			{ 19, 3, 5 },		// 81 bits per transform
			{ 19, 4, 4 },		// 81 bits per transform
			{ 19, 5, 3 },		// 81 bits per transform
			{ 19, 6, 2 },		// 81 bits per transform
			{ 19, 7, 1 },		// 81 bits per transform
			{ 19, 8, 0 },		// 81 bits per transform
			{ 20, 0, 7 },		// 81 bits per transform
			{ 20, 1, 6 },		// 81 bits per transform
			{ 20, 2, 5 },		// 81 bits per transform
			{ 20, 3, 4 },		// 81 bits per transform
			{ 20, 4, 3 },		// 81 bits per transform
			{ 20, 5, 2 },		// 81 bits per transform
			{ 20, 6, 1 },		// 81 bits per transform
			{ 20, 7, 0 },		// 81 bits per transform
			{ 21, 0, 6 },		// 81 bits per transform
			{ 21, 1, 5 },		// 81 bits per transform
			{ 21, 2, 4 },		// 81 bits per transform
			{ 21, 3, 3 },		// 81 bits per transform
			{ 21, 4, 2 },		// 81 bits per transform
			{ 21, 5, 1 },		// 81 bits per transform
			{ 21, 6, 0 },		// 81 bits per transform
			{ 22, 0, 5 },		// 81 bits per transform
			{ 22, 1, 4 },		// 81 bits per transform
			{ 22, 2, 3 },		// 81 bits per transform
			{ 22, 3, 2 },		// 81 bits per transform
			{ 22, 4, 1 },		// 81 bits per transform
			{ 22, 5, 0 },		// 81 bits per transform
			{ 23, 0, 4 },		// 81 bits per transform
			{ 23, 1, 3 },		// 81 bits per transform
			{ 23, 2, 2 },		// 81 bits per transform
			{ 23, 3, 1 },		// 81 bits per transform
			{ 23, 4, 0 },		// 81 bits per transform
			{ 0, 5, 23 },		// 84 bits per transform
			{ 0, 6, 22 },		// 84 bits per transform
			{ 0, 7, 21 },		// 84 bits per transform
			{ 0, 8, 20 },		// 84 bits per transform
			{ 0, 9, 19 },		// 84 bits per transform
			{ 0, 10, 18 },		// 84 bits per transform
			{ 0, 11, 17 },		// 84 bits per transform
			{ 0, 12, 16 },		// 84 bits per transform
			{ 0, 13, 15 },		// 84 bits per transform
			{ 0, 14, 14 },		// 84 bits per transform
			{ 0, 15, 13 },		// 84 bits per transform
			{ 0, 16, 12 },		// 84 bits per transform
			{ 0, 17, 11 },		// 84 bits per transform
			{ 0, 18, 10 },		// 84 bits per transform
			{ 0, 19, 9 },		// 84 bits per transform
			{ 0, 20, 8 },		// 84 bits per transform
			{ 0, 21, 7 },		// 84 bits per transform
			{ 0, 22, 6 },		// 84 bits per transform
			{ 0, 23, 5 },		// 84 bits per transform
			{ 1, 4, 23 },		// 84 bits per transform
			{ 1, 5, 22 },		// 84 bits per transform
			{ 1, 6, 21 },		// 84 bits per transform
			{ 1, 7, 20 },		// 84 bits per transform
			{ 1, 8, 19 },		// 84 bits per transform
			{ 1, 9, 18 },		// 84 bits per transform
			{ 1, 10, 17 },		// 84 bits per transform
			{ 1, 11, 16 },		// 84 bits per transform
			{ 1, 12, 15 },		// 84 bits per transform
			{ 1, 13, 14 },		// 84 bits per transform
			{ 1, 14, 13 },		// 84 bits per transform
			{ 1, 15, 12 },		// 84 bits per transform
			{ 1, 16, 11 },		// 84 bits per transform
			{ 1, 17, 10 },		// 84 bits per transform
			{ 1, 18, 9 },		// 84 bits per transform
			{ 1, 19, 8 },		// 84 bits per transform
			{ 1, 20, 7 },		// 84 bits per transform
			{ 1, 21, 6 },		// 84 bits per transform
			{ 1, 22, 5 },		// 84 bits per transform
			{ 1, 23, 4 },		// 84 bits per transform
			{ 2, 3, 23 },		// 84 bits per transform
			{ 2, 4, 22 },		// 84 bits per transform
			{ 2, 5, 21 },		// 84 bits per transform
			{ 2, 6, 20 },		// 84 bits per transform
			{ 2, 7, 19 },		// 84 bits per transform
			{ 2, 8, 18 },		// 84 bits per transform
			{ 2, 9, 17 },		// 84 bits per transform
			{ 2, 10, 16 },		// 84 bits per transform
			{ 2, 11, 15 },		// 84 bits per transform
			{ 2, 12, 14 },		// 84 bits per transform
			{ 2, 13, 13 },		// 84 bits per transform
			{ 2, 14, 12 },		// 84 bits per transform
			{ 2, 15, 11 },		// 84 bits per transform
			{ 2, 16, 10 },		// 84 bits per transform
			{ 2, 17, 9 },		// 84 bits per transform
			{ 2, 18, 8 },		// 84 bits per transform
			{ 2, 19, 7 },		// 84 bits per transform
			{ 2, 20, 6 },		// 84 bits per transform
			{ 2, 21, 5 },		// 84 bits per transform
			{ 2, 22, 4 },		// 84 bits per transform
			{ 2, 23, 3 },		// 84 bits per transform
			{ 3, 2, 23 },		// 84 bits per transform
			{ 3, 3, 22 },		// 84 bits per transform
			{ 3, 4, 21 },		// 84 bits per transform
			{ 3, 5, 20 },		// 84 bits per transform
			{ 3, 6, 19 },		// 84 bits per transform
			{ 3, 7, 18 },		// 84 bits per transform
			{ 3, 8, 17 },		// 84 bits per transform
			{ 3, 9, 16 },		// 84 bits per transform
			{ 3, 10, 15 },		// 84 bits per transform
			{ 3, 11, 14 },		// 84 bits per transform
			{ 3, 12, 13 },		// 84 bits per transform
			{ 3, 13, 12 },		// 84 bits per transform
			{ 3, 14, 11 },		// 84 bits per transform
			{ 3, 15, 10 },		// 84 bits per transform
			{ 3, 16, 9 },		// 84 bits per transform
			{ 3, 17, 8 },		// 84 bits per transform
			{ 3, 18, 7 },		// 84 bits per transform
			{ 3, 19, 6 },		// 84 bits per transform
			{ 3, 20, 5 },		// 84 bits per transform
			{ 3, 21, 4 },		// 84 bits per transform
			{ 3, 22, 3 },		// 84 bits per transform
			{ 3, 23, 2 },		// 84 bits per transform
			{ 4, 1, 23 },		// 84 bits per transform
			{ 4, 2, 22 },		// 84 bits per transform
			{ 4, 3, 21 },		// 84 bits per transform
			{ 4, 4, 20 },		// 84 bits per transform
			{ 4, 5, 19 },		// 84 bits per transform
			{ 4, 6, 18 },		// 84 bits per transform
			{ 4, 7, 17 },		// 84 bits per transform
			{ 4, 8, 16 },		// 84 bits per transform
			{ 4, 9, 15 },		// 84 bits per transform
			{ 4, 10, 14 },		// 84 bits per transform
			{ 4, 11, 13 },		// 84 bits per transform
			{ 4, 12, 12 },		// 84 bits per transform
			{ 4, 13, 11 },		// 84 bits per transform
			{ 4, 14, 10 },		// 84 bits per transform
			{ 4, 15, 9 },		// 84 bits per transform
			{ 4, 16, 8 },		// 84 bits per transform
			{ 4, 17, 7 },		// 84 bits per transform
			{ 4, 18, 6 },		// 84 bits per transform
			{ 4, 19, 5 },		// 84 bits per transform
			{ 4, 20, 4 },		// 84 bits per transform
			{ 4, 21, 3 },		// 84 bits per transform
			{ 4, 22, 2 },		// 84 bits per transform
			{ 4, 23, 1 },		// 84 bits per transform
			{ 5, 0, 23 },		// 84 bits per transform
			{ 5, 1, 22 },		// 84 bits per transform
			{ 5, 2, 21 },		// 84 bits per transform
			{ 5, 3, 20 },		// 84 bits per transform
			{ 5, 4, 19 },		// 84 bits per transform
			{ 5, 5, 18 },		// 84 bits per transform
			{ 5, 6, 17 },		// 84 bits per transform
			{ 5, 7, 16 },		// 84 bits per transform
			{ 5, 8, 15 },		// 84 bits per transform
			{ 5, 9, 14 },		// 84 bits per transform
			{ 5, 10, 13 },		// 84 bits per transform
			{ 5, 11, 12 },		// 84 bits per transform
			{ 5, 12, 11 },		// 84 bits per transform
			{ 5, 13, 10 },		// 84 bits per transform
			{ 5, 14, 9 },		// 84 bits per transform
			{ 5, 15, 8 },		// 84 bits per transform
			{ 5, 16, 7 },		// 84 bits per transform
			{ 5, 17, 6 },		// 84 bits per transform
			{ 5, 18, 5 },		// 84 bits per transform
			{ 5, 19, 4 },		// 84 bits per transform
			{ 5, 20, 3 },		// 84 bits per transform
			{ 5, 21, 2 },		// 84 bits per transform
			{ 5, 22, 1 },		// 84 bits per transform
			{ 5, 23, 0 },		// 84 bits per transform
			{ 6, 0, 22 },		// 84 bits per transform
			{ 6, 1, 21 },		// 84 bits per transform
			{ 6, 2, 20 },		// 84 bits per transform
			{ 6, 3, 19 },		// 84 bits per transform
			{ 6, 4, 18 },		// 84 bits per transform
			{ 6, 5, 17 },		// 84 bits per transform
			{ 6, 6, 16 },		// 84 bits per transform
			{ 6, 7, 15 },		// 84 bits per transform
			{ 6, 8, 14 },		// 84 bits per transform
			{ 6, 9, 13 },		// 84 bits per transform
			{ 6, 10, 12 },		// 84 bits per transform
			{ 6, 11, 11 },		// 84 bits per transform
			{ 6, 12, 10 },		// 84 bits per transform
			{ 6, 13, 9 },		// 84 bits per transform
			{ 6, 14, 8 },		// 84 bits per transform
			{ 6, 15, 7 },		// 84 bits per transform
			{ 6, 16, 6 },		// 84 bits per transform
			{ 6, 17, 5 },		// 84 bits per transform
			{ 6, 18, 4 },		// 84 bits per transform
			{ 6, 19, 3 },		// 84 bits per transform
			{ 6, 20, 2 },		// 84 bits per transform
			{ 6, 21, 1 },		// 84 bits per transform
			{ 6, 22, 0 },		// 84 bits per transform
			{ 7, 0, 21 },		// 84 bits per transform
			{ 7, 1, 20 },		// 84 bits per transform
			{ 7, 2, 19 },		// 84 bits per transform
			{ 7, 3, 18 },		// 84 bits per transform
			{ 7, 4, 17 },		// 84 bits per transform
			{ 7, 5, 16 },		// 84 bits per transform
			{ 7, 6, 15 },		// 84 bits per transform
			{ 7, 7, 14 },		// 84 bits per transform
			{ 7, 8, 13 },		// 84 bits per transform
			{ 7, 9, 12 },		// 84 bits per transform
			{ 7, 10, 11 },		// 84 bits per transform
			{ 7, 11, 10 },		// 84 bits per transform
			{ 7, 12, 9 },		// 84 bits per transform
			{ 7, 13, 8 },		// 84 bits per transform
			{ 7, 14, 7 },		// 84 bits per transform
			{ 7, 15, 6 },		// 84 bits per transform
			{ 7, 16, 5 },		// 84 bits per transform
			{ 7, 17, 4 },		// 84 bits per transform
			{ 7, 18, 3 },		// 84 bits per transform
			{ 7, 19, 2 },		// 84 bits per transform
			{ 7, 20, 1 },		// 84 bits per transform
			{ 7, 21, 0 },		// 84 bits per transform
			{ 8, 0, 20 },		// 84 bits per transform
			{ 8, 1, 19 },		// 84 bits per transform
			{ 8, 2, 18 },		// 84 bits per transform
			{ 8, 3, 17 },		// 84 bits per transform
			{ 8, 4, 16 },		// 84 bits per transform
			{ 8, 5, 15 },		// 84 bits per transform
			{ 8, 6, 14 },		// 84 bits per transform
			{ 8, 7, 13 },		// 84 bits per transform
			{ 8, 8, 12 },		// 84 bits per transform
			{ 8, 9, 11 },		// 84 bits per transform
			{ 8, 10, 10 },		// 84 bits per transform
			{ 8, 11, 9 },		// 84 bits per transform
			{ 8, 12, 8 },		// 84 bits per transform
			{ 8, 13, 7 },		// 84 bits per transform
			{ 8, 14, 6 },		// 84 bits per transform
			{ 8, 15, 5 },		// 84 bits per transform
			{ 8, 16, 4 },		// 84 bits per transform
			{ 8, 17, 3 },		// 84 bits per transform
			{ 8, 18, 2 },		// 84 bits per transform
			{ 8, 19, 1 },		// 84 bits per transform
			{ 8, 20, 0 },		// 84 bits per transform
			{ 9, 0, 19 },		// 84 bits per transform
			{ 9, 1, 18 },		// 84 bits per transform
			{ 9, 2, 17 },		// 84 bits per transform
			{ 9, 3, 16 },		// 84 bits per transform
			{ 9, 4, 15 },		// 84 bits per transform
			{ 9, 5, 14 },		// 84 bits per transform
			{ 9, 6, 13 },		// 84 bits per transform
			{ 9, 7, 12 },		// 84 bits per transform
			{ 9, 8, 11 },		// 84 bits per transform
			{ 9, 9, 10 },		// 84 bits per transform
			{ 9, 10, 9 },		// 84 bits per transform
			{ 9, 11, 8 },		// 84 bits per transform
			{ 9, 12, 7 },		// 84 bits per transform
			{ 9, 13, 6 },		// 84 bits per transform
			{ 9, 14, 5 },		// 84 bits per transform
			{ 9, 15, 4 },		// 84 bits per transform
			{ 9, 16, 3 },		// 84 bits per transform
			{ 9, 17, 2 },		// 84 bits per transform
			{ 9, 18, 1 },		// 84 bits per transform
			{ 9, 19, 0 },		// 84 bits per transform
			{ 10, 0, 18 },		// 84 bits per transform
			{ 10, 1, 17 },		// 84 bits per transform
			{ 10, 2, 16 },		// 84 bits per transform
			{ 10, 3, 15 },		// 84 bits per transform
			{ 10, 4, 14 },		// 84 bits per transform
			{ 10, 5, 13 },		// 84 bits per transform
			{ 10, 6, 12 },		// 84 bits per transform
			{ 10, 7, 11 },		// 84 bits per transform
			{ 10, 8, 10 },		// 84 bits per transform
			{ 10, 9, 9 },		// 84 bits per transform
			{ 10, 10, 8 },		// 84 bits per transform
			{ 10, 11, 7 },		// 84 bits per transform
			{ 10, 12, 6 },		// 84 bits per transform
			{ 10, 13, 5 },		// 84 bits per transform
			{ 10, 14, 4 },		// 84 bits per transform
			{ 10, 15, 3 },		// 84 bits per transform
			{ 10, 16, 2 },		// 84 bits per transform
			{ 10, 17, 1 },		// 84 bits per transform
			{ 10, 18, 0 },		// 84 bits per transform
			{ 11, 0, 17 },		// 84 bits per transform
			{ 11, 1, 16 },		// 84 bits per transform
			{ 11, 2, 15 },		// 84 bits per transform
			{ 11, 3, 14 },		// 84 bits per transform
			{ 11, 4, 13 },		// 84 bits per transform
			{ 11, 5, 12 },		// 84 bits per transform
			{ 11, 6, 11 },		// 84 bits per transform
			{ 11, 7, 10 },		// 84 bits per transform
			{ 11, 8, 9 },		// 84 bits per transform
			{ 11, 9, 8 },		// 84 bits per transform
			{ 11, 10, 7 },		// 84 bits per transform
			{ 11, 11, 6 },		// 84 bits per transform
			{ 11, 12, 5 },		// 84 bits per transform
			{ 11, 13, 4 },		// 84 bits per transform
			{ 11, 14, 3 },		// 84 bits per transform
			{ 11, 15, 2 },		// 84 bits per transform
			{ 11, 16, 1 },		// 84 bits per transform
			{ 11, 17, 0 },		// 84 bits per transform
			{ 12, 0, 16 },		// 84 bits per transform
			{ 12, 1, 15 },		// 84 bits per transform
			{ 12, 2, 14 },		// 84 bits per transform
			{ 12, 3, 13 },		// 84 bits per transform
			{ 12, 4, 12 },		// 84 bits per transform
			{ 12, 5, 11 },		// 84 bits per transform
			{ 12, 6, 10 },		// 84 bits per transform
			{ 12, 7, 9 },		// 84 bits per transform
			{ 12, 8, 8 },		// 84 bits per transform
			{ 12, 9, 7 },		// 84 bits per transform
			{ 12, 10, 6 },		// 84 bits per transform
			{ 12, 11, 5 },		// 84 bits per transform
			{ 12, 12, 4 },		// 84 bits per transform
			{ 12, 13, 3 },		// 84 bits per transform
			{ 12, 14, 2 },		// 84 bits per transform
			{ 12, 15, 1 },		// 84 bits per transform
			{ 12, 16, 0 },		// 84 bits per transform
			{ 13, 0, 15 },		// 84 bits per transform
			{ 13, 1, 14 },		// 84 bits per transform
			{ 13, 2, 13 },		// 84 bits per transform
			{ 13, 3, 12 },		// 84 bits per transform
			{ 13, 4, 11 },		// 84 bits per transform
			{ 13, 5, 10 },		// 84 bits per transform
			{ 13, 6, 9 },		// 84 bits per transform
			{ 13, 7, 8 },		// 84 bits per transform
			{ 13, 8, 7 },		// 84 bits per transform
			{ 13, 9, 6 },		// 84 bits per transform
			{ 13, 10, 5 },		// 84 bits per transform
			{ 13, 11, 4 },		// 84 bits per transform
			{ 13, 12, 3 },		// 84 bits per transform
			{ 13, 13, 2 },		// 84 bits per transform
			{ 13, 14, 1 },		// 84 bits per transform
			{ 13, 15, 0 },		// 84 bits per transform
			{ 14, 0, 14 },		// 84 bits per transform
			{ 14, 1, 13 },		// 84 bits per transform
			{ 14, 2, 12 },		// 84 bits per transform
			{ 14, 3, 11 },		// 84 bits per transform
			{ 14, 4, 10 },		// 84 bits per transform
			{ 14, 5, 9 },		// 84 bits per transform
			{ 14, 6, 8 },		// 84 bits per transform
			{ 14, 7, 7 },		// 84 bits per transform
			{ 14, 8, 6 },		// 84 bits per transform
			{ 14, 9, 5 },		// 84 bits per transform
			{ 14, 10, 4 },		// 84 bits per transform
			{ 14, 11, 3 },		// 84 bits per transform
			{ 14, 12, 2 },		// 84 bits per transform
			{ 14, 13, 1 },		// 84 bits per transform
			{ 14, 14, 0 },		// 84 bits per transform
			{ 15, 0, 13 },		// 84 bits per transform
			{ 15, 1, 12 },		// 84 bits per transform
			{ 15, 2, 11 },		// 84 bits per transform
			{ 15, 3, 10 },		// 84 bits per transform
			{ 15, 4, 9 },		// 84 bits per transform
			{ 15, 5, 8 },		// 84 bits per transform
			{ 15, 6, 7 },		// 84 bits per transform
			{ 15, 7, 6 },		// 84 bits per transform
			{ 15, 8, 5 },		// 84 bits per transform
			{ 15, 9, 4 },		// 84 bits per transform
			{ 15, 10, 3 },		// 84 bits per transform
			{ 15, 11, 2 },		// 84 bits per transform
			{ 15, 12, 1 },		// 84 bits per transform
			{ 15, 13, 0 },		// 84 bits per transform
			{ 16, 0, 12 },		// 84 bits per transform
			{ 16, 1, 11 },		// 84 bits per transform
			{ 16, 2, 10 },		// 84 bits per transform
			{ 16, 3, 9 },		// 84 bits per transform
			{ 16, 4, 8 },		// 84 bits per transform
			{ 16, 5, 7 },		// 84 bits per transform
			{ 16, 6, 6 },		// 84 bits per transform
			{ 16, 7, 5 },		// 84 bits per transform
			{ 16, 8, 4 },		// 84 bits per transform
			{ 16, 9, 3 },		// 84 bits per transform
			{ 16, 10, 2 },		// 84 bits per transform
			{ 16, 11, 1 },		// 84 bits per transform
			{ 16, 12, 0 },		// 84 bits per transform
			{ 17, 0, 11 },		// 84 bits per transform
			{ 17, 1, 10 },		// 84 bits per transform
			{ 17, 2, 9 },		// 84 bits per transform
			{ 17, 3, 8 },		// 84 bits per transform
			{ 17, 4, 7 },		// 84 bits per transform
			{ 17, 5, 6 },		// 84 bits per transform
			{ 17, 6, 5 },		// 84 bits per transform
			{ 17, 7, 4 },		// 84 bits per transform
			{ 17, 8, 3 },		// 84 bits per transform
			{ 17, 9, 2 },		// 84 bits per transform
			{ 17, 10, 1 },		// 84 bits per transform
			{ 17, 11, 0 },		// 84 bits per transform
			{ 18, 0, 10 },		// 84 bits per transform
			{ 18, 1, 9 },		// 84 bits per transform
			{ 18, 2, 8 },		// 84 bits per transform
			{ 18, 3, 7 },		// 84 bits per transform
			{ 18, 4, 6 },		// 84 bits per transform
			{ 18, 5, 5 },		// 84 bits per transform
			{ 18, 6, 4 },		// 84 bits per transform
			{ 18, 7, 3 },		// 84 bits per transform
			{ 18, 8, 2 },		// 84 bits per transform
			{ 18, 9, 1 },		// 84 bits per transform
			{ 18, 10, 0 },		// 84 bits per transform
			{ 19, 0, 9 },		// 84 bits per transform
			{ 19, 1, 8 },		// 84 bits per transform
			{ 19, 2, 7 },		// 84 bits per transform
			{ 19, 3, 6 },		// 84 bits per transform
			{ 19, 4, 5 },		// 84 bits per transform
			{ 19, 5, 4 },		// 84 bits per transform
			{ 19, 6, 3 },		// 84 bits per transform
			{ 19, 7, 2 },		// 84 bits per transform
			{ 19, 8, 1 },		// 84 bits per transform
			{ 19, 9, 0 },		// 84 bits per transform
			{ 20, 0, 8 },		// 84 bits per transform
			{ 20, 1, 7 },		// 84 bits per transform
			{ 20, 2, 6 },		// 84 bits per transform
			{ 20, 3, 5 },		// 84 bits per transform
			{ 20, 4, 4 },		// 84 bits per transform
			{ 20, 5, 3 },		// 84 bits per transform
			{ 20, 6, 2 },		// 84 bits per transform
			{ 20, 7, 1 },		// 84 bits per transform
			{ 20, 8, 0 },		// 84 bits per transform
			{ 21, 0, 7 },		// 84 bits per transform
			{ 21, 1, 6 },		// 84 bits per transform
			{ 21, 2, 5 },		// 84 bits per transform
			{ 21, 3, 4 },		// 84 bits per transform
			{ 21, 4, 3 },		// 84 bits per transform
			{ 21, 5, 2 },		// 84 bits per transform
			{ 21, 6, 1 },		// 84 bits per transform
			{ 21, 7, 0 },		// 84 bits per transform
			{ 22, 0, 6 },		// 84 bits per transform
			{ 22, 1, 5 },		// 84 bits per transform
			{ 22, 2, 4 },		// 84 bits per transform
			{ 22, 3, 3 },		// 84 bits per transform
			{ 22, 4, 2 },		// 84 bits per transform
			{ 22, 5, 1 },		// 84 bits per transform
			{ 22, 6, 0 },		// 84 bits per transform
			{ 23, 0, 5 },		// 84 bits per transform
			{ 23, 1, 4 },		// 84 bits per transform
			{ 23, 2, 3 },		// 84 bits per transform
			{ 23, 3, 2 },		// 84 bits per transform
			{ 23, 4, 1 },		// 84 bits per transform
			{ 23, 5, 0 },		// 84 bits per transform
			{ 0, 6, 23 },		// 87 bits per transform
			{ 0, 7, 22 },		// 87 bits per transform
			{ 0, 8, 21 },		// 87 bits per transform
			{ 0, 9, 20 },		// 87 bits per transform
			{ 0, 10, 19 },		// 87 bits per transform
			{ 0, 11, 18 },		// 87 bits per transform
			{ 0, 12, 17 },		// 87 bits per transform
			{ 0, 13, 16 },		// 87 bits per transform
			{ 0, 14, 15 },		// 87 bits per transform
			{ 0, 15, 14 },		// 87 bits per transform
			{ 0, 16, 13 },		// 87 bits per transform
			{ 0, 17, 12 },		// 87 bits per transform
			{ 0, 18, 11 },		// 87 bits per transform
			{ 0, 19, 10 },		// 87 bits per transform
			{ 0, 20, 9 },		// 87 bits per transform
			{ 0, 21, 8 },		// 87 bits per transform
			{ 0, 22, 7 },		// 87 bits per transform
			{ 0, 23, 6 },		// 87 bits per transform
			{ 1, 5, 23 },		// 87 bits per transform
			{ 1, 6, 22 },		// 87 bits per transform
			{ 1, 7, 21 },		// 87 bits per transform
			{ 1, 8, 20 },		// 87 bits per transform
			{ 1, 9, 19 },		// 87 bits per transform
			{ 1, 10, 18 },		// 87 bits per transform
			{ 1, 11, 17 },		// 87 bits per transform
			{ 1, 12, 16 },		// 87 bits per transform
			{ 1, 13, 15 },		// 87 bits per transform
			{ 1, 14, 14 },		// 87 bits per transform
			{ 1, 15, 13 },		// 87 bits per transform
			{ 1, 16, 12 },		// 87 bits per transform
			{ 1, 17, 11 },		// 87 bits per transform
			{ 1, 18, 10 },		// 87 bits per transform
			{ 1, 19, 9 },		// 87 bits per transform
			{ 1, 20, 8 },		// 87 bits per transform
			{ 1, 21, 7 },		// 87 bits per transform
			{ 1, 22, 6 },		// 87 bits per transform
			{ 1, 23, 5 },		// 87 bits per transform
			{ 2, 4, 23 },		// 87 bits per transform
			{ 2, 5, 22 },		// 87 bits per transform
			{ 2, 6, 21 },		// 87 bits per transform
			{ 2, 7, 20 },		// 87 bits per transform
			{ 2, 8, 19 },		// 87 bits per transform
			{ 2, 9, 18 },		// 87 bits per transform
			{ 2, 10, 17 },		// 87 bits per transform
			{ 2, 11, 16 },		// 87 bits per transform
			{ 2, 12, 15 },		// 87 bits per transform
			{ 2, 13, 14 },		// 87 bits per transform
			{ 2, 14, 13 },		// 87 bits per transform
			{ 2, 15, 12 },		// 87 bits per transform
			{ 2, 16, 11 },		// 87 bits per transform
			{ 2, 17, 10 },		// 87 bits per transform
			{ 2, 18, 9 },		// 87 bits per transform
			{ 2, 19, 8 },		// 87 bits per transform
			{ 2, 20, 7 },		// 87 bits per transform
			{ 2, 21, 6 },		// 87 bits per transform
			{ 2, 22, 5 },		// 87 bits per transform
			{ 2, 23, 4 },		// 87 bits per transform
			{ 3, 3, 23 },		// 87 bits per transform
			{ 3, 4, 22 },		// 87 bits per transform
			{ 3, 5, 21 },		// 87 bits per transform
			{ 3, 6, 20 },		// 87 bits per transform
			{ 3, 7, 19 },		// 87 bits per transform
			{ 3, 8, 18 },		// 87 bits per transform
			{ 3, 9, 17 },		// 87 bits per transform
			{ 3, 10, 16 },		// 87 bits per transform
			{ 3, 11, 15 },		// 87 bits per transform
			{ 3, 12, 14 },		// 87 bits per transform
			{ 3, 13, 13 },		// 87 bits per transform
			{ 3, 14, 12 },		// 87 bits per transform
			{ 3, 15, 11 },		// 87 bits per transform
			{ 3, 16, 10 },		// 87 bits per transform
			{ 3, 17, 9 },		// 87 bits per transform
			{ 3, 18, 8 },		// 87 bits per transform
			{ 3, 19, 7 },		// 87 bits per transform
			{ 3, 20, 6 },		// 87 bits per transform
			{ 3, 21, 5 },		// 87 bits per transform
			{ 3, 22, 4 },		// 87 bits per transform
			{ 3, 23, 3 },		// 87 bits per transform
			{ 4, 2, 23 },		// 87 bits per transform
			{ 4, 3, 22 },		// 87 bits per transform
			{ 4, 4, 21 },		// 87 bits per transform
			{ 4, 5, 20 },		// 87 bits per transform
			{ 4, 6, 19 },		// 87 bits per transform
			{ 4, 7, 18 },		// 87 bits per transform
			{ 4, 8, 17 },		// 87 bits per transform
			{ 4, 9, 16 },		// 87 bits per transform
			{ 4, 10, 15 },		// 87 bits per transform
			{ 4, 11, 14 },		// 87 bits per transform
			{ 4, 12, 13 },		// 87 bits per transform
			{ 4, 13, 12 },		// 87 bits per transform
			{ 4, 14, 11 },		// 87 bits per transform
			{ 4, 15, 10 },		// 87 bits per transform
			{ 4, 16, 9 },		// 87 bits per transform
			{ 4, 17, 8 },		// 87 bits per transform
			{ 4, 18, 7 },		// 87 bits per transform
			{ 4, 19, 6 },		// 87 bits per transform
			{ 4, 20, 5 },		// 87 bits per transform
			{ 4, 21, 4 },		// 87 bits per transform
			{ 4, 22, 3 },		// 87 bits per transform
			{ 4, 23, 2 },		// 87 bits per transform
			{ 5, 1, 23 },		// 87 bits per transform
			{ 5, 2, 22 },		// 87 bits per transform
			{ 5, 3, 21 },		// 87 bits per transform
			{ 5, 4, 20 },		// 87 bits per transform
			{ 5, 5, 19 },		// 87 bits per transform
			{ 5, 6, 18 },		// 87 bits per transform
			{ 5, 7, 17 },		// 87 bits per transform
			{ 5, 8, 16 },		// 87 bits per transform
			{ 5, 9, 15 },		// 87 bits per transform
			{ 5, 10, 14 },		// 87 bits per transform
			{ 5, 11, 13 },		// 87 bits per transform
			{ 5, 12, 12 },		// 87 bits per transform
			{ 5, 13, 11 },		// 87 bits per transform
			{ 5, 14, 10 },		// 87 bits per transform
			{ 5, 15, 9 },		// 87 bits per transform
			{ 5, 16, 8 },		// 87 bits per transform
			{ 5, 17, 7 },		// 87 bits per transform
			{ 5, 18, 6 },		// 87 bits per transform
			{ 5, 19, 5 },		// 87 bits per transform
			{ 5, 20, 4 },		// 87 bits per transform
			{ 5, 21, 3 },		// 87 bits per transform
			{ 5, 22, 2 },		// 87 bits per transform
			{ 5, 23, 1 },		// 87 bits per transform
			{ 6, 0, 23 },		// 87 bits per transform
			{ 6, 1, 22 },		// 87 bits per transform
			{ 6, 2, 21 },		// 87 bits per transform
			{ 6, 3, 20 },		// 87 bits per transform
			{ 6, 4, 19 },		// 87 bits per transform
			{ 6, 5, 18 },		// 87 bits per transform
			{ 6, 6, 17 },		// 87 bits per transform
			{ 6, 7, 16 },		// 87 bits per transform
			{ 6, 8, 15 },		// 87 bits per transform
			{ 6, 9, 14 },		// 87 bits per transform
			{ 6, 10, 13 },		// 87 bits per transform
			{ 6, 11, 12 },		// 87 bits per transform
			{ 6, 12, 11 },		// 87 bits per transform
			{ 6, 13, 10 },		// 87 bits per transform
			{ 6, 14, 9 },		// 87 bits per transform
			{ 6, 15, 8 },		// 87 bits per transform
			{ 6, 16, 7 },		// 87 bits per transform
			{ 6, 17, 6 },		// 87 bits per transform
			{ 6, 18, 5 },		// 87 bits per transform
			{ 6, 19, 4 },		// 87 bits per transform
			{ 6, 20, 3 },		// 87 bits per transform
			{ 6, 21, 2 },		// 87 bits per transform
			{ 6, 22, 1 },		// 87 bits per transform
			{ 6, 23, 0 },		// 87 bits per transform
			{ 7, 0, 22 },		// 87 bits per transform
			{ 7, 1, 21 },		// 87 bits per transform
			{ 7, 2, 20 },		// 87 bits per transform
			{ 7, 3, 19 },		// 87 bits per transform
			{ 7, 4, 18 },		// 87 bits per transform
			{ 7, 5, 17 },		// 87 bits per transform
			{ 7, 6, 16 },		// 87 bits per transform
			{ 7, 7, 15 },		// 87 bits per transform
			{ 7, 8, 14 },		// 87 bits per transform
			{ 7, 9, 13 },		// 87 bits per transform
			{ 7, 10, 12 },		// 87 bits per transform
			{ 7, 11, 11 },		// 87 bits per transform
			{ 7, 12, 10 },		// 87 bits per transform
			{ 7, 13, 9 },		// 87 bits per transform
			{ 7, 14, 8 },		// 87 bits per transform
			{ 7, 15, 7 },		// 87 bits per transform
			{ 7, 16, 6 },		// 87 bits per transform
			{ 7, 17, 5 },		// 87 bits per transform
			{ 7, 18, 4 },		// 87 bits per transform
			{ 7, 19, 3 },		// 87 bits per transform
			{ 7, 20, 2 },		// 87 bits per transform
			{ 7, 21, 1 },		// 87 bits per transform
			{ 7, 22, 0 },		// 87 bits per transform
			{ 8, 0, 21 },		// 87 bits per transform
			{ 8, 1, 20 },		// 87 bits per transform
			{ 8, 2, 19 },		// 87 bits per transform
			{ 8, 3, 18 },		// 87 bits per transform
			{ 8, 4, 17 },		// 87 bits per transform
			{ 8, 5, 16 },		// 87 bits per transform
			{ 8, 6, 15 },		// 87 bits per transform
			{ 8, 7, 14 },		// 87 bits per transform
			{ 8, 8, 13 },		// 87 bits per transform
			{ 8, 9, 12 },		// 87 bits per transform
			{ 8, 10, 11 },		// 87 bits per transform
			{ 8, 11, 10 },		// 87 bits per transform
			{ 8, 12, 9 },		// 87 bits per transform
			{ 8, 13, 8 },		// 87 bits per transform
			{ 8, 14, 7 },		// 87 bits per transform
			{ 8, 15, 6 },		// 87 bits per transform
			{ 8, 16, 5 },		// 87 bits per transform
			{ 8, 17, 4 },		// 87 bits per transform
			{ 8, 18, 3 },		// 87 bits per transform
			{ 8, 19, 2 },		// 87 bits per transform
			{ 8, 20, 1 },		// 87 bits per transform
			{ 8, 21, 0 },		// 87 bits per transform
			{ 9, 0, 20 },		// 87 bits per transform
			{ 9, 1, 19 },		// 87 bits per transform
			{ 9, 2, 18 },		// 87 bits per transform
			{ 9, 3, 17 },		// 87 bits per transform
			{ 9, 4, 16 },		// 87 bits per transform
			{ 9, 5, 15 },		// 87 bits per transform
			{ 9, 6, 14 },		// 87 bits per transform
			{ 9, 7, 13 },		// 87 bits per transform
			{ 9, 8, 12 },		// 87 bits per transform
			{ 9, 9, 11 },		// 87 bits per transform
			{ 9, 10, 10 },		// 87 bits per transform
			{ 9, 11, 9 },		// 87 bits per transform
			{ 9, 12, 8 },		// 87 bits per transform
			{ 9, 13, 7 },		// 87 bits per transform
			{ 9, 14, 6 },		// 87 bits per transform
			{ 9, 15, 5 },		// 87 bits per transform
			{ 9, 16, 4 },		// 87 bits per transform
			{ 9, 17, 3 },		// 87 bits per transform
			{ 9, 18, 2 },		// 87 bits per transform
			{ 9, 19, 1 },		// 87 bits per transform
			{ 9, 20, 0 },		// 87 bits per transform
			{ 10, 0, 19 },		// 87 bits per transform
			{ 10, 1, 18 },		// 87 bits per transform
			{ 10, 2, 17 },		// 87 bits per transform
			{ 10, 3, 16 },		// 87 bits per transform
			{ 10, 4, 15 },		// 87 bits per transform
			{ 10, 5, 14 },		// 87 bits per transform
			{ 10, 6, 13 },		// 87 bits per transform
			{ 10, 7, 12 },		// 87 bits per transform
			{ 10, 8, 11 },		// 87 bits per transform
			{ 10, 9, 10 },		// 87 bits per transform
			{ 10, 10, 9 },		// 87 bits per transform
			{ 10, 11, 8 },		// 87 bits per transform
			{ 10, 12, 7 },		// 87 bits per transform
			{ 10, 13, 6 },		// 87 bits per transform
			{ 10, 14, 5 },		// 87 bits per transform
			{ 10, 15, 4 },		// 87 bits per transform
			{ 10, 16, 3 },		// 87 bits per transform
			{ 10, 17, 2 },		// 87 bits per transform
			{ 10, 18, 1 },		// 87 bits per transform
			{ 10, 19, 0 },		// 87 bits per transform
			{ 11, 0, 18 },		// 87 bits per transform
			{ 11, 1, 17 },		// 87 bits per transform
			{ 11, 2, 16 },		// 87 bits per transform
			{ 11, 3, 15 },		// 87 bits per transform
			{ 11, 4, 14 },		// 87 bits per transform
			{ 11, 5, 13 },		// 87 bits per transform
			{ 11, 6, 12 },		// 87 bits per transform
			{ 11, 7, 11 },		// 87 bits per transform
			{ 11, 8, 10 },		// 87 bits per transform
			{ 11, 9, 9 },		// 87 bits per transform
			{ 11, 10, 8 },		// 87 bits per transform
			{ 11, 11, 7 },		// 87 bits per transform
			{ 11, 12, 6 },		// 87 bits per transform
			{ 11, 13, 5 },		// 87 bits per transform
			{ 11, 14, 4 },		// 87 bits per transform
			{ 11, 15, 3 },		// 87 bits per transform
			{ 11, 16, 2 },		// 87 bits per transform
			{ 11, 17, 1 },		// 87 bits per transform
			{ 11, 18, 0 },		// 87 bits per transform
			{ 12, 0, 17 },		// 87 bits per transform
			{ 12, 1, 16 },		// 87 bits per transform
			{ 12, 2, 15 },		// 87 bits per transform
			{ 12, 3, 14 },		// 87 bits per transform
			{ 12, 4, 13 },		// 87 bits per transform
			{ 12, 5, 12 },		// 87 bits per transform
			{ 12, 6, 11 },		// 87 bits per transform
			{ 12, 7, 10 },		// 87 bits per transform
			{ 12, 8, 9 },		// 87 bits per transform
			{ 12, 9, 8 },		// 87 bits per transform
			{ 12, 10, 7 },		// 87 bits per transform
			{ 12, 11, 6 },		// 87 bits per transform
			{ 12, 12, 5 },		// 87 bits per transform
			{ 12, 13, 4 },		// 87 bits per transform
			{ 12, 14, 3 },		// 87 bits per transform
			{ 12, 15, 2 },		// 87 bits per transform
			{ 12, 16, 1 },		// 87 bits per transform
			{ 12, 17, 0 },		// 87 bits per transform
			{ 13, 0, 16 },		// 87 bits per transform
			{ 13, 1, 15 },		// 87 bits per transform
			{ 13, 2, 14 },		// 87 bits per transform
			{ 13, 3, 13 },		// 87 bits per transform
			{ 13, 4, 12 },		// 87 bits per transform
			{ 13, 5, 11 },		// 87 bits per transform
			{ 13, 6, 10 },		// 87 bits per transform
			{ 13, 7, 9 },		// 87 bits per transform
			{ 13, 8, 8 },		// 87 bits per transform
			{ 13, 9, 7 },		// 87 bits per transform
			{ 13, 10, 6 },		// 87 bits per transform
			{ 13, 11, 5 },		// 87 bits per transform
			{ 13, 12, 4 },		// 87 bits per transform
			{ 13, 13, 3 },		// 87 bits per transform
			{ 13, 14, 2 },		// 87 bits per transform
			{ 13, 15, 1 },		// 87 bits per transform
			{ 13, 16, 0 },		// 87 bits per transform
			{ 14, 0, 15 },		// 87 bits per transform
			{ 14, 1, 14 },		// 87 bits per transform
			{ 14, 2, 13 },		// 87 bits per transform
			{ 14, 3, 12 },		// 87 bits per transform
			{ 14, 4, 11 },		// 87 bits per transform
			{ 14, 5, 10 },		// 87 bits per transform
			{ 14, 6, 9 },		// 87 bits per transform
			{ 14, 7, 8 },		// 87 bits per transform
			{ 14, 8, 7 },		// 87 bits per transform
			{ 14, 9, 6 },		// 87 bits per transform
			{ 14, 10, 5 },		// 87 bits per transform
			{ 14, 11, 4 },		// 87 bits per transform
			{ 14, 12, 3 },		// 87 bits per transform
			{ 14, 13, 2 },		// 87 bits per transform
			{ 14, 14, 1 },		// 87 bits per transform
			{ 14, 15, 0 },		// 87 bits per transform
			{ 15, 0, 14 },		// 87 bits per transform
			{ 15, 1, 13 },		// 87 bits per transform
			{ 15, 2, 12 },		// 87 bits per transform
			{ 15, 3, 11 },		// 87 bits per transform
			{ 15, 4, 10 },		// 87 bits per transform
			{ 15, 5, 9 },		// 87 bits per transform
			{ 15, 6, 8 },		// 87 bits per transform
			{ 15, 7, 7 },		// 87 bits per transform
			{ 15, 8, 6 },		// 87 bits per transform
			{ 15, 9, 5 },		// 87 bits per transform
			{ 15, 10, 4 },		// 87 bits per transform
			{ 15, 11, 3 },		// 87 bits per transform
			{ 15, 12, 2 },		// 87 bits per transform
			{ 15, 13, 1 },		// 87 bits per transform
			{ 15, 14, 0 },		// 87 bits per transform
			{ 16, 0, 13 },		// 87 bits per transform
			{ 16, 1, 12 },		// 87 bits per transform
			{ 16, 2, 11 },		// 87 bits per transform
			{ 16, 3, 10 },		// 87 bits per transform
			{ 16, 4, 9 },		// 87 bits per transform
			{ 16, 5, 8 },		// 87 bits per transform
			{ 16, 6, 7 },		// 87 bits per transform
			{ 16, 7, 6 },		// 87 bits per transform
			{ 16, 8, 5 },		// 87 bits per transform
			{ 16, 9, 4 },		// 87 bits per transform
			{ 16, 10, 3 },		// 87 bits per transform
			{ 16, 11, 2 },		// 87 bits per transform
			{ 16, 12, 1 },		// 87 bits per transform
			{ 16, 13, 0 },		// 87 bits per transform
			{ 17, 0, 12 },		// 87 bits per transform
			{ 17, 1, 11 },		// 87 bits per transform
			{ 17, 2, 10 },		// 87 bits per transform
			{ 17, 3, 9 },		// 87 bits per transform
			{ 17, 4, 8 },		// 87 bits per transform
			{ 17, 5, 7 },		// 87 bits per transform
			{ 17, 6, 6 },		// 87 bits per transform
			{ 17, 7, 5 },		// 87 bits per transform
			{ 17, 8, 4 },		// 87 bits per transform
			{ 17, 9, 3 },		// 87 bits per transform
			{ 17, 10, 2 },		// 87 bits per transform
			{ 17, 11, 1 },		// 87 bits per transform
			{ 17, 12, 0 },		// 87 bits per transform
			{ 18, 0, 11 },		// 87 bits per transform
			{ 18, 1, 10 },		// 87 bits per transform
			{ 18, 2, 9 },		// 87 bits per transform
			{ 18, 3, 8 },		// 87 bits per transform
			{ 18, 4, 7 },		// 87 bits per transform
			{ 18, 5, 6 },		// 87 bits per transform
			{ 18, 6, 5 },		// 87 bits per transform
			{ 18, 7, 4 },		// 87 bits per transform
			{ 18, 8, 3 },		// 87 bits per transform
			{ 18, 9, 2 },		// 87 bits per transform
			{ 18, 10, 1 },		// 87 bits per transform
			{ 18, 11, 0 },		// 87 bits per transform
			{ 19, 0, 10 },		// 87 bits per transform
			{ 19, 1, 9 },		// 87 bits per transform
			{ 19, 2, 8 },		// 87 bits per transform
			{ 19, 3, 7 },		// 87 bits per transform
			{ 19, 4, 6 },		// 87 bits per transform
			{ 19, 5, 5 },		// 87 bits per transform
			{ 19, 6, 4 },		// 87 bits per transform
			{ 19, 7, 3 },		// 87 bits per transform
			{ 19, 8, 2 },		// 87 bits per transform
			{ 19, 9, 1 },		// 87 bits per transform
			{ 19, 10, 0 },		// 87 bits per transform
			{ 20, 0, 9 },		// 87 bits per transform
			{ 20, 1, 8 },		// 87 bits per transform
			{ 20, 2, 7 },		// 87 bits per transform
			{ 20, 3, 6 },		// 87 bits per transform
			{ 20, 4, 5 },		// 87 bits per transform
			{ 20, 5, 4 },		// 87 bits per transform
			{ 20, 6, 3 },		// 87 bits per transform
			{ 20, 7, 2 },		// 87 bits per transform
			{ 20, 8, 1 },		// 87 bits per transform
			{ 20, 9, 0 },		// 87 bits per transform
			{ 21, 0, 8 },		// 87 bits per transform
			{ 21, 1, 7 },		// 87 bits per transform
			{ 21, 2, 6 },		// 87 bits per transform
			{ 21, 3, 5 },		// 87 bits per transform
			{ 21, 4, 4 },		// 87 bits per transform
			{ 21, 5, 3 },		// 87 bits per transform
			{ 21, 6, 2 },		// 87 bits per transform
			{ 21, 7, 1 },		// 87 bits per transform
			{ 21, 8, 0 },		// 87 bits per transform
			{ 22, 0, 7 },		// 87 bits per transform
			{ 22, 1, 6 },		// 87 bits per transform
			{ 22, 2, 5 },		// 87 bits per transform
			{ 22, 3, 4 },		// 87 bits per transform
			{ 22, 4, 3 },		// 87 bits per transform
			{ 22, 5, 2 },		// 87 bits per transform
			{ 22, 6, 1 },		// 87 bits per transform
			{ 22, 7, 0 },		// 87 bits per transform
			{ 23, 0, 6 },		// 87 bits per transform
			{ 23, 1, 5 },		// 87 bits per transform
			{ 23, 2, 4 },		// 87 bits per transform
			{ 23, 3, 3 },		// 87 bits per transform
			{ 23, 4, 2 },		// 87 bits per transform
			{ 23, 5, 1 },		// 87 bits per transform
			{ 23, 6, 0 },		// 87 bits per transform
			{ 0, 7, 23 },		// 90 bits per transform
			{ 0, 8, 22 },		// 90 bits per transform
			{ 0, 9, 21 },		// 90 bits per transform
			{ 0, 10, 20 },		// 90 bits per transform
			{ 0, 11, 19 },		// 90 bits per transform
			{ 0, 12, 18 },		// 90 bits per transform
			{ 0, 13, 17 },		// 90 bits per transform
			{ 0, 14, 16 },		// 90 bits per transform
			{ 0, 15, 15 },		// 90 bits per transform
			{ 0, 16, 14 },		// 90 bits per transform
			{ 0, 17, 13 },		// 90 bits per transform
			{ 0, 18, 12 },		// 90 bits per transform
			{ 0, 19, 11 },		// 90 bits per transform
			{ 0, 20, 10 },		// 90 bits per transform
			{ 0, 21, 9 },		// 90 bits per transform
			{ 0, 22, 8 },		// 90 bits per transform
			{ 0, 23, 7 },		// 90 bits per transform
			{ 1, 6, 23 },		// 90 bits per transform
			{ 1, 7, 22 },		// 90 bits per transform
			{ 1, 8, 21 },		// 90 bits per transform
			{ 1, 9, 20 },		// 90 bits per transform
			{ 1, 10, 19 },		// 90 bits per transform
			{ 1, 11, 18 },		// 90 bits per transform
			{ 1, 12, 17 },		// 90 bits per transform
			{ 1, 13, 16 },		// 90 bits per transform
			{ 1, 14, 15 },		// 90 bits per transform
			{ 1, 15, 14 },		// 90 bits per transform
			{ 1, 16, 13 },		// 90 bits per transform
			{ 1, 17, 12 },		// 90 bits per transform
			{ 1, 18, 11 },		// 90 bits per transform
			{ 1, 19, 10 },		// 90 bits per transform
			{ 1, 20, 9 },		// 90 bits per transform
			{ 1, 21, 8 },		// 90 bits per transform
			{ 1, 22, 7 },		// 90 bits per transform
			{ 1, 23, 6 },		// 90 bits per transform
			{ 2, 5, 23 },		// 90 bits per transform
			{ 2, 6, 22 },		// 90 bits per transform
			{ 2, 7, 21 },		// 90 bits per transform
			{ 2, 8, 20 },		// 90 bits per transform
			{ 2, 9, 19 },		// 90 bits per transform
			{ 2, 10, 18 },		// 90 bits per transform
			{ 2, 11, 17 },		// 90 bits per transform
			{ 2, 12, 16 },		// 90 bits per transform
			{ 2, 13, 15 },		// 90 bits per transform
			{ 2, 14, 14 },		// 90 bits per transform
			{ 2, 15, 13 },		// 90 bits per transform
			{ 2, 16, 12 },		// 90 bits per transform
			{ 2, 17, 11 },		// 90 bits per transform
			{ 2, 18, 10 },		// 90 bits per transform
			{ 2, 19, 9 },		// 90 bits per transform
			{ 2, 20, 8 },		// 90 bits per transform
			{ 2, 21, 7 },		// 90 bits per transform
			{ 2, 22, 6 },		// 90 bits per transform
			{ 2, 23, 5 },		// 90 bits per transform
			{ 3, 4, 23 },		// 90 bits per transform
			{ 3, 5, 22 },		// 90 bits per transform
			{ 3, 6, 21 },		// 90 bits per transform
			{ 3, 7, 20 },		// 90 bits per transform
			{ 3, 8, 19 },		// 90 bits per transform
			{ 3, 9, 18 },		// 90 bits per transform
			{ 3, 10, 17 },		// 90 bits per transform
			{ 3, 11, 16 },		// 90 bits per transform
			{ 3, 12, 15 },		// 90 bits per transform
			{ 3, 13, 14 },		// 90 bits per transform
			{ 3, 14, 13 },		// 90 bits per transform
			{ 3, 15, 12 },		// 90 bits per transform
			{ 3, 16, 11 },		// 90 bits per transform
			{ 3, 17, 10 },		// 90 bits per transform
			{ 3, 18, 9 },		// 90 bits per transform
			{ 3, 19, 8 },		// 90 bits per transform
			{ 3, 20, 7 },		// 90 bits per transform
			{ 3, 21, 6 },		// 90 bits per transform
			{ 3, 22, 5 },		// 90 bits per transform
			{ 3, 23, 4 },		// 90 bits per transform
			{ 4, 3, 23 },		// 90 bits per transform
			{ 4, 4, 22 },		// 90 bits per transform
			{ 4, 5, 21 },		// 90 bits per transform
			{ 4, 6, 20 },		// 90 bits per transform
			{ 4, 7, 19 },		// 90 bits per transform
			{ 4, 8, 18 },		// 90 bits per transform
			{ 4, 9, 17 },		// 90 bits per transform
			{ 4, 10, 16 },		// 90 bits per transform
			{ 4, 11, 15 },		// 90 bits per transform
			{ 4, 12, 14 },		// 90 bits per transform
			{ 4, 13, 13 },		// 90 bits per transform
			{ 4, 14, 12 },		// 90 bits per transform
			{ 4, 15, 11 },		// 90 bits per transform
			{ 4, 16, 10 },		// 90 bits per transform
			{ 4, 17, 9 },		// 90 bits per transform
			{ 4, 18, 8 },		// 90 bits per transform
			{ 4, 19, 7 },		// 90 bits per transform
			{ 4, 20, 6 },		// 90 bits per transform
			{ 4, 21, 5 },		// 90 bits per transform
			{ 4, 22, 4 },		// 90 bits per transform
			{ 4, 23, 3 },		// 90 bits per transform
			{ 5, 2, 23 },		// 90 bits per transform
			{ 5, 3, 22 },		// 90 bits per transform
			{ 5, 4, 21 },		// 90 bits per transform
			{ 5, 5, 20 },		// 90 bits per transform
			{ 5, 6, 19 },		// 90 bits per transform
			{ 5, 7, 18 },		// 90 bits per transform
			{ 5, 8, 17 },		// 90 bits per transform
			{ 5, 9, 16 },		// 90 bits per transform
			{ 5, 10, 15 },		// 90 bits per transform
			{ 5, 11, 14 },		// 90 bits per transform
			{ 5, 12, 13 },		// 90 bits per transform
			{ 5, 13, 12 },		// 90 bits per transform
			{ 5, 14, 11 },		// 90 bits per transform
			{ 5, 15, 10 },		// 90 bits per transform
			{ 5, 16, 9 },		// 90 bits per transform
			{ 5, 17, 8 },		// 90 bits per transform
			{ 5, 18, 7 },		// 90 bits per transform
			{ 5, 19, 6 },		// 90 bits per transform
			{ 5, 20, 5 },		// 90 bits per transform
			{ 5, 21, 4 },		// 90 bits per transform
			{ 5, 22, 3 },		// 90 bits per transform
			{ 5, 23, 2 },		// 90 bits per transform
			{ 6, 1, 23 },		// 90 bits per transform
			{ 6, 2, 22 },		// 90 bits per transform
			{ 6, 3, 21 },		// 90 bits per transform
			{ 6, 4, 20 },		// 90 bits per transform
			{ 6, 5, 19 },		// 90 bits per transform
			{ 6, 6, 18 },		// 90 bits per transform
			{ 6, 7, 17 },		// 90 bits per transform
			{ 6, 8, 16 },		// 90 bits per transform
			{ 6, 9, 15 },		// 90 bits per transform
			{ 6, 10, 14 },		// 90 bits per transform
			{ 6, 11, 13 },		// 90 bits per transform
			{ 6, 12, 12 },		// 90 bits per transform
			{ 6, 13, 11 },		// 90 bits per transform
			{ 6, 14, 10 },		// 90 bits per transform
			{ 6, 15, 9 },		// 90 bits per transform
			{ 6, 16, 8 },		// 90 bits per transform
			{ 6, 17, 7 },		// 90 bits per transform
			{ 6, 18, 6 },		// 90 bits per transform
			{ 6, 19, 5 },		// 90 bits per transform
			{ 6, 20, 4 },		// 90 bits per transform
			{ 6, 21, 3 },		// 90 bits per transform
			{ 6, 22, 2 },		// 90 bits per transform
			{ 6, 23, 1 },		// 90 bits per transform
			{ 7, 0, 23 },		// 90 bits per transform
			{ 7, 1, 22 },		// 90 bits per transform
			{ 7, 2, 21 },		// 90 bits per transform
			{ 7, 3, 20 },		// 90 bits per transform
			{ 7, 4, 19 },		// 90 bits per transform
			{ 7, 5, 18 },		// 90 bits per transform
			{ 7, 6, 17 },		// 90 bits per transform
			{ 7, 7, 16 },		// 90 bits per transform
			{ 7, 8, 15 },		// 90 bits per transform
			{ 7, 9, 14 },		// 90 bits per transform
			{ 7, 10, 13 },		// 90 bits per transform
			{ 7, 11, 12 },		// 90 bits per transform
			{ 7, 12, 11 },		// 90 bits per transform
			{ 7, 13, 10 },		// 90 bits per transform
			{ 7, 14, 9 },		// 90 bits per transform
			{ 7, 15, 8 },		// 90 bits per transform
			{ 7, 16, 7 },		// 90 bits per transform
			{ 7, 17, 6 },		// 90 bits per transform
			{ 7, 18, 5 },		// 90 bits per transform
			{ 7, 19, 4 },		// 90 bits per transform
			{ 7, 20, 3 },		// 90 bits per transform
			{ 7, 21, 2 },		// 90 bits per transform
			{ 7, 22, 1 },		// 90 bits per transform
			{ 7, 23, 0 },		// 90 bits per transform
			{ 8, 0, 22 },		// 90 bits per transform
			{ 8, 1, 21 },		// 90 bits per transform
			{ 8, 2, 20 },		// 90 bits per transform
			{ 8, 3, 19 },		// 90 bits per transform
			{ 8, 4, 18 },		// 90 bits per transform
			{ 8, 5, 17 },		// 90 bits per transform
			{ 8, 6, 16 },		// 90 bits per transform
			{ 8, 7, 15 },		// 90 bits per transform
			{ 8, 8, 14 },		// 90 bits per transform
			{ 8, 9, 13 },		// 90 bits per transform
			{ 8, 10, 12 },		// 90 bits per transform
			{ 8, 11, 11 },		// 90 bits per transform
			{ 8, 12, 10 },		// 90 bits per transform
			{ 8, 13, 9 },		// 90 bits per transform
			{ 8, 14, 8 },		// 90 bits per transform
			{ 8, 15, 7 },		// 90 bits per transform
			{ 8, 16, 6 },		// 90 bits per transform
			{ 8, 17, 5 },		// 90 bits per transform
			{ 8, 18, 4 },		// 90 bits per transform
			{ 8, 19, 3 },		// 90 bits per transform
			{ 8, 20, 2 },		// 90 bits per transform
			{ 8, 21, 1 },		// 90 bits per transform
			{ 8, 22, 0 },		// 90 bits per transform
			{ 9, 0, 21 },		// 90 bits per transform
			{ 9, 1, 20 },		// 90 bits per transform
			{ 9, 2, 19 },		// 90 bits per transform
			{ 9, 3, 18 },		// 90 bits per transform
			{ 9, 4, 17 },		// 90 bits per transform
			{ 9, 5, 16 },		// 90 bits per transform
			{ 9, 6, 15 },		// 90 bits per transform
			{ 9, 7, 14 },		// 90 bits per transform
			{ 9, 8, 13 },		// 90 bits per transform
			{ 9, 9, 12 },		// 90 bits per transform
			{ 9, 10, 11 },		// 90 bits per transform
			{ 9, 11, 10 },		// 90 bits per transform
			{ 9, 12, 9 },		// 90 bits per transform
			{ 9, 13, 8 },		// 90 bits per transform
			{ 9, 14, 7 },		// 90 bits per transform
			{ 9, 15, 6 },		// 90 bits per transform
			{ 9, 16, 5 },		// 90 bits per transform
			{ 9, 17, 4 },		// 90 bits per transform
			{ 9, 18, 3 },		// 90 bits per transform
			{ 9, 19, 2 },		// 90 bits per transform
			{ 9, 20, 1 },		// 90 bits per transform
			{ 9, 21, 0 },		// 90 bits per transform
			{ 10, 0, 20 },		// 90 bits per transform
			{ 10, 1, 19 },		// 90 bits per transform
			{ 10, 2, 18 },		// 90 bits per transform
			{ 10, 3, 17 },		// 90 bits per transform
			{ 10, 4, 16 },		// 90 bits per transform
			{ 10, 5, 15 },		// 90 bits per transform
			{ 10, 6, 14 },		// 90 bits per transform
			{ 10, 7, 13 },		// 90 bits per transform
			{ 10, 8, 12 },		// 90 bits per transform
			{ 10, 9, 11 },		// 90 bits per transform
			{ 10, 10, 10 },		// 90 bits per transform
			{ 10, 11, 9 },		// 90 bits per transform
			{ 10, 12, 8 },		// 90 bits per transform
			{ 10, 13, 7 },		// 90 bits per transform
			{ 10, 14, 6 },		// 90 bits per transform
			{ 10, 15, 5 },		// 90 bits per transform
			{ 10, 16, 4 },		// 90 bits per transform
			{ 10, 17, 3 },		// 90 bits per transform
			{ 10, 18, 2 },		// 90 bits per transform
			{ 10, 19, 1 },		// 90 bits per transform
			{ 10, 20, 0 },		// 90 bits per transform
			{ 11, 0, 19 },		// 90 bits per transform
			{ 11, 1, 18 },		// 90 bits per transform
			{ 11, 2, 17 },		// 90 bits per transform
			{ 11, 3, 16 },		// 90 bits per transform
			{ 11, 4, 15 },		// 90 bits per transform
			{ 11, 5, 14 },		// 90 bits per transform
			{ 11, 6, 13 },		// 90 bits per transform
			{ 11, 7, 12 },		// 90 bits per transform
			{ 11, 8, 11 },		// 90 bits per transform
			{ 11, 9, 10 },		// 90 bits per transform
			{ 11, 10, 9 },		// 90 bits per transform
			{ 11, 11, 8 },		// 90 bits per transform
			{ 11, 12, 7 },		// 90 bits per transform
			{ 11, 13, 6 },		// 90 bits per transform
			{ 11, 14, 5 },		// 90 bits per transform
			{ 11, 15, 4 },		// 90 bits per transform
			{ 11, 16, 3 },		// 90 bits per transform
			{ 11, 17, 2 },		// 90 bits per transform
			{ 11, 18, 1 },		// 90 bits per transform
			{ 11, 19, 0 },		// 90 bits per transform
			{ 12, 0, 18 },		// 90 bits per transform
			{ 12, 1, 17 },		// 90 bits per transform
			{ 12, 2, 16 },		// 90 bits per transform
			{ 12, 3, 15 },		// 90 bits per transform
			{ 12, 4, 14 },		// 90 bits per transform
			{ 12, 5, 13 },		// 90 bits per transform
			{ 12, 6, 12 },		// 90 bits per transform
			{ 12, 7, 11 },		// 90 bits per transform
			{ 12, 8, 10 },		// 90 bits per transform
			{ 12, 9, 9 },		// 90 bits per transform
			{ 12, 10, 8 },		// 90 bits per transform
			{ 12, 11, 7 },		// 90 bits per transform
			{ 12, 12, 6 },		// 90 bits per transform
			{ 12, 13, 5 },		// 90 bits per transform
			{ 12, 14, 4 },		// 90 bits per transform
			{ 12, 15, 3 },		// 90 bits per transform
			{ 12, 16, 2 },		// 90 bits per transform
			{ 12, 17, 1 },		// 90 bits per transform
			{ 12, 18, 0 },		// 90 bits per transform
			{ 13, 0, 17 },		// 90 bits per transform
			{ 13, 1, 16 },		// 90 bits per transform
			{ 13, 2, 15 },		// 90 bits per transform
			{ 13, 3, 14 },		// 90 bits per transform
			{ 13, 4, 13 },		// 90 bits per transform
			{ 13, 5, 12 },		// 90 bits per transform
			{ 13, 6, 11 },		// 90 bits per transform
			{ 13, 7, 10 },		// 90 bits per transform
			{ 13, 8, 9 },		// 90 bits per transform
			{ 13, 9, 8 },		// 90 bits per transform
			{ 13, 10, 7 },		// 90 bits per transform
			{ 13, 11, 6 },		// 90 bits per transform
			{ 13, 12, 5 },		// 90 bits per transform
			{ 13, 13, 4 },		// 90 bits per transform
			{ 13, 14, 3 },		// 90 bits per transform
			{ 13, 15, 2 },		// 90 bits per transform
			{ 13, 16, 1 },		// 90 bits per transform
			{ 13, 17, 0 },		// 90 bits per transform
			{ 14, 0, 16 },		// 90 bits per transform
			{ 14, 1, 15 },		// 90 bits per transform
			{ 14, 2, 14 },		// 90 bits per transform
			{ 14, 3, 13 },		// 90 bits per transform
			{ 14, 4, 12 },		// 90 bits per transform
			{ 14, 5, 11 },		// 90 bits per transform
			{ 14, 6, 10 },		// 90 bits per transform
			{ 14, 7, 9 },		// 90 bits per transform
			{ 14, 8, 8 },		// 90 bits per transform
			{ 14, 9, 7 },		// 90 bits per transform
			{ 14, 10, 6 },		// 90 bits per transform
			{ 14, 11, 5 },		// 90 bits per transform
			{ 14, 12, 4 },		// 90 bits per transform
			{ 14, 13, 3 },		// 90 bits per transform
			{ 14, 14, 2 },		// 90 bits per transform
			{ 14, 15, 1 },		// 90 bits per transform
			{ 14, 16, 0 },		// 90 bits per transform
			{ 15, 0, 15 },		// 90 bits per transform
			{ 15, 1, 14 },		// 90 bits per transform
			{ 15, 2, 13 },		// 90 bits per transform
			{ 15, 3, 12 },		// 90 bits per transform
			{ 15, 4, 11 },		// 90 bits per transform
			{ 15, 5, 10 },		// 90 bits per transform
			{ 15, 6, 9 },		// 90 bits per transform
			{ 15, 7, 8 },		// 90 bits per transform
			{ 15, 8, 7 },		// 90 bits per transform
			{ 15, 9, 6 },		// 90 bits per transform
			{ 15, 10, 5 },		// 90 bits per transform
			{ 15, 11, 4 },		// 90 bits per transform
			{ 15, 12, 3 },		// 90 bits per transform
			{ 15, 13, 2 },		// 90 bits per transform
			{ 15, 14, 1 },		// 90 bits per transform
			{ 15, 15, 0 },		// 90 bits per transform
			{ 16, 0, 14 },		// 90 bits per transform
			{ 16, 1, 13 },		// 90 bits per transform
			{ 16, 2, 12 },		// 90 bits per transform
			{ 16, 3, 11 },		// 90 bits per transform
			{ 16, 4, 10 },		// 90 bits per transform
			{ 16, 5, 9 },		// 90 bits per transform
			{ 16, 6, 8 },		// 90 bits per transform
			{ 16, 7, 7 },		// 90 bits per transform
			{ 16, 8, 6 },		// 90 bits per transform
			{ 16, 9, 5 },		// 90 bits per transform
			{ 16, 10, 4 },		// 90 bits per transform
			{ 16, 11, 3 },		// 90 bits per transform
			{ 16, 12, 2 },		// 90 bits per transform
			{ 16, 13, 1 },		// 90 bits per transform
			{ 16, 14, 0 },		// 90 bits per transform
			{ 17, 0, 13 },		// 90 bits per transform
			{ 17, 1, 12 },		// 90 bits per transform
			{ 17, 2, 11 },		// 90 bits per transform
			{ 17, 3, 10 },		// 90 bits per transform
			{ 17, 4, 9 },		// 90 bits per transform
			{ 17, 5, 8 },		// 90 bits per transform
			{ 17, 6, 7 },		// 90 bits per transform
			{ 17, 7, 6 },		// 90 bits per transform
			{ 17, 8, 5 },		// 90 bits per transform
			{ 17, 9, 4 },		// 90 bits per transform
			{ 17, 10, 3 },		// 90 bits per transform
			{ 17, 11, 2 },		// 90 bits per transform
			{ 17, 12, 1 },		// 90 bits per transform
			{ 17, 13, 0 },		// 90 bits per transform
			{ 18, 0, 12 },		// 90 bits per transform
			{ 18, 1, 11 },		// 90 bits per transform
			{ 18, 2, 10 },		// 90 bits per transform
			{ 18, 3, 9 },		// 90 bits per transform
			{ 18, 4, 8 },		// 90 bits per transform
			{ 18, 5, 7 },		// 90 bits per transform
			{ 18, 6, 6 },		// 90 bits per transform
			{ 18, 7, 5 },		// 90 bits per transform
			{ 18, 8, 4 },		// 90 bits per transform
			{ 18, 9, 3 },		// 90 bits per transform
			{ 18, 10, 2 },		// 90 bits per transform
			{ 18, 11, 1 },		// 90 bits per transform
			{ 18, 12, 0 },		// 90 bits per transform
			{ 19, 0, 11 },		// 90 bits per transform
			{ 19, 1, 10 },		// 90 bits per transform
			{ 19, 2, 9 },		// 90 bits per transform
			{ 19, 3, 8 },		// 90 bits per transform
			{ 19, 4, 7 },		// 90 bits per transform
			{ 19, 5, 6 },		// 90 bits per transform
			{ 19, 6, 5 },		// 90 bits per transform
			{ 19, 7, 4 },		// 90 bits per transform
			{ 19, 8, 3 },		// 90 bits per transform
			{ 19, 9, 2 },		// 90 bits per transform
			{ 19, 10, 1 },		// 90 bits per transform
			{ 19, 11, 0 },		// 90 bits per transform
			{ 20, 0, 10 },		// 90 bits per transform
			{ 20, 1, 9 },		// 90 bits per transform
			{ 20, 2, 8 },		// 90 bits per transform
			{ 20, 3, 7 },		// 90 bits per transform
			{ 20, 4, 6 },		// 90 bits per transform
			{ 20, 5, 5 },		// 90 bits per transform
			{ 20, 6, 4 },		// 90 bits per transform
			{ 20, 7, 3 },		// 90 bits per transform
			{ 20, 8, 2 },		// 90 bits per transform
			{ 20, 9, 1 },		// 90 bits per transform
			{ 20, 10, 0 },		// 90 bits per transform
			{ 21, 0, 9 },		// 90 bits per transform
			{ 21, 1, 8 },		// 90 bits per transform
			{ 21, 2, 7 },		// 90 bits per transform
			{ 21, 3, 6 },		// 90 bits per transform
			{ 21, 4, 5 },		// 90 bits per transform
			{ 21, 5, 4 },		// 90 bits per transform
			{ 21, 6, 3 },		// 90 bits per transform
			{ 21, 7, 2 },		// 90 bits per transform
			{ 21, 8, 1 },		// 90 bits per transform
			{ 21, 9, 0 },		// 90 bits per transform
			{ 22, 0, 8 },		// 90 bits per transform
			{ 22, 1, 7 },		// 90 bits per transform
			{ 22, 2, 6 },		// 90 bits per transform
			{ 22, 3, 5 },		// 90 bits per transform
			{ 22, 4, 4 },		// 90 bits per transform
			{ 22, 5, 3 },		// 90 bits per transform
			{ 22, 6, 2 },		// 90 bits per transform
			{ 22, 7, 1 },		// 90 bits per transform
			{ 22, 8, 0 },		// 90 bits per transform
			{ 23, 0, 7 },		// 90 bits per transform
			{ 23, 1, 6 },		// 90 bits per transform
			{ 23, 2, 5 },		// 90 bits per transform
			{ 23, 3, 4 },		// 90 bits per transform
			{ 23, 4, 3 },		// 90 bits per transform
			{ 23, 5, 2 },		// 90 bits per transform
			{ 23, 6, 1 },		// 90 bits per transform
			{ 23, 7, 0 },		// 90 bits per transform
			{ 0, 8, 23 },		// 93 bits per transform
			{ 0, 9, 22 },		// 93 bits per transform
			{ 0, 10, 21 },		// 93 bits per transform
			{ 0, 11, 20 },		// 93 bits per transform
			{ 0, 12, 19 },		// 93 bits per transform
			{ 0, 13, 18 },		// 93 bits per transform
			{ 0, 14, 17 },		// 93 bits per transform
			{ 0, 15, 16 },		// 93 bits per transform
			{ 0, 16, 15 },		// 93 bits per transform
			{ 0, 17, 14 },		// 93 bits per transform
			{ 0, 18, 13 },		// 93 bits per transform
			{ 0, 19, 12 },		// 93 bits per transform
			{ 0, 20, 11 },		// 93 bits per transform
			{ 0, 21, 10 },		// 93 bits per transform
			{ 0, 22, 9 },		// 93 bits per transform
			{ 0, 23, 8 },		// 93 bits per transform
			{ 1, 7, 23 },		// 93 bits per transform
			{ 1, 8, 22 },		// 93 bits per transform
			{ 1, 9, 21 },		// 93 bits per transform
			{ 1, 10, 20 },		// 93 bits per transform
			{ 1, 11, 19 },		// 93 bits per transform
			{ 1, 12, 18 },		// 93 bits per transform
			{ 1, 13, 17 },		// 93 bits per transform
			{ 1, 14, 16 },		// 93 bits per transform
			{ 1, 15, 15 },		// 93 bits per transform
			{ 1, 16, 14 },		// 93 bits per transform
			{ 1, 17, 13 },		// 93 bits per transform
			{ 1, 18, 12 },		// 93 bits per transform
			{ 1, 19, 11 },		// 93 bits per transform
			{ 1, 20, 10 },		// 93 bits per transform
			{ 1, 21, 9 },		// 93 bits per transform
			{ 1, 22, 8 },		// 93 bits per transform
			{ 1, 23, 7 },		// 93 bits per transform
			{ 2, 6, 23 },		// 93 bits per transform
			{ 2, 7, 22 },		// 93 bits per transform
			{ 2, 8, 21 },		// 93 bits per transform
			{ 2, 9, 20 },		// 93 bits per transform
			{ 2, 10, 19 },		// 93 bits per transform
			{ 2, 11, 18 },		// 93 bits per transform
			{ 2, 12, 17 },		// 93 bits per transform
			{ 2, 13, 16 },		// 93 bits per transform
			{ 2, 14, 15 },		// 93 bits per transform
			{ 2, 15, 14 },		// 93 bits per transform
			{ 2, 16, 13 },		// 93 bits per transform
			{ 2, 17, 12 },		// 93 bits per transform
			{ 2, 18, 11 },		// 93 bits per transform
			{ 2, 19, 10 },		// 93 bits per transform
			{ 2, 20, 9 },		// 93 bits per transform
			{ 2, 21, 8 },		// 93 bits per transform
			{ 2, 22, 7 },		// 93 bits per transform
			{ 2, 23, 6 },		// 93 bits per transform
			{ 3, 5, 23 },		// 93 bits per transform
			{ 3, 6, 22 },		// 93 bits per transform
			{ 3, 7, 21 },		// 93 bits per transform
			{ 3, 8, 20 },		// 93 bits per transform
			{ 3, 9, 19 },		// 93 bits per transform
			{ 3, 10, 18 },		// 93 bits per transform
			{ 3, 11, 17 },		// 93 bits per transform
			{ 3, 12, 16 },		// 93 bits per transform
			{ 3, 13, 15 },		// 93 bits per transform
			{ 3, 14, 14 },		// 93 bits per transform
			{ 3, 15, 13 },		// 93 bits per transform
			{ 3, 16, 12 },		// 93 bits per transform
			{ 3, 17, 11 },		// 93 bits per transform
			{ 3, 18, 10 },		// 93 bits per transform
			{ 3, 19, 9 },		// 93 bits per transform
			{ 3, 20, 8 },		// 93 bits per transform
			{ 3, 21, 7 },		// 93 bits per transform
			{ 3, 22, 6 },		// 93 bits per transform
			{ 3, 23, 5 },		// 93 bits per transform
			{ 4, 4, 23 },		// 93 bits per transform
			{ 4, 5, 22 },		// 93 bits per transform
			{ 4, 6, 21 },		// 93 bits per transform
			{ 4, 7, 20 },		// 93 bits per transform
			{ 4, 8, 19 },		// 93 bits per transform
			{ 4, 9, 18 },		// 93 bits per transform
			{ 4, 10, 17 },		// 93 bits per transform
			{ 4, 11, 16 },		// 93 bits per transform
			{ 4, 12, 15 },		// 93 bits per transform
			{ 4, 13, 14 },		// 93 bits per transform
			{ 4, 14, 13 },		// 93 bits per transform
			{ 4, 15, 12 },		// 93 bits per transform
			{ 4, 16, 11 },		// 93 bits per transform
			{ 4, 17, 10 },		// 93 bits per transform
			{ 4, 18, 9 },		// 93 bits per transform
			{ 4, 19, 8 },		// 93 bits per transform
			{ 4, 20, 7 },		// 93 bits per transform
			{ 4, 21, 6 },		// 93 bits per transform
			{ 4, 22, 5 },		// 93 bits per transform
			{ 4, 23, 4 },		// 93 bits per transform
			{ 5, 3, 23 },		// 93 bits per transform
			{ 5, 4, 22 },		// 93 bits per transform
			{ 5, 5, 21 },		// 93 bits per transform
			{ 5, 6, 20 },		// 93 bits per transform
			{ 5, 7, 19 },		// 93 bits per transform
			{ 5, 8, 18 },		// 93 bits per transform
			{ 5, 9, 17 },		// 93 bits per transform
			{ 5, 10, 16 },		// 93 bits per transform
			{ 5, 11, 15 },		// 93 bits per transform
			{ 5, 12, 14 },		// 93 bits per transform
			{ 5, 13, 13 },		// 93 bits per transform
			{ 5, 14, 12 },		// 93 bits per transform
			{ 5, 15, 11 },		// 93 bits per transform
			{ 5, 16, 10 },		// 93 bits per transform
			{ 5, 17, 9 },		// 93 bits per transform
			{ 5, 18, 8 },		// 93 bits per transform
			{ 5, 19, 7 },		// 93 bits per transform
			{ 5, 20, 6 },		// 93 bits per transform
			{ 5, 21, 5 },		// 93 bits per transform
			{ 5, 22, 4 },		// 93 bits per transform
			{ 5, 23, 3 },		// 93 bits per transform
			{ 6, 2, 23 },		// 93 bits per transform
			{ 6, 3, 22 },		// 93 bits per transform
			{ 6, 4, 21 },		// 93 bits per transform
			{ 6, 5, 20 },		// 93 bits per transform
			{ 6, 6, 19 },		// 93 bits per transform
			{ 6, 7, 18 },		// 93 bits per transform
			{ 6, 8, 17 },		// 93 bits per transform
			{ 6, 9, 16 },		// 93 bits per transform
			{ 6, 10, 15 },		// 93 bits per transform
			{ 6, 11, 14 },		// 93 bits per transform
			{ 6, 12, 13 },		// 93 bits per transform
			{ 6, 13, 12 },		// 93 bits per transform
			{ 6, 14, 11 },		// 93 bits per transform
			{ 6, 15, 10 },		// 93 bits per transform
			{ 6, 16, 9 },		// 93 bits per transform
			{ 6, 17, 8 },		// 93 bits per transform
			{ 6, 18, 7 },		// 93 bits per transform
			{ 6, 19, 6 },		// 93 bits per transform
			{ 6, 20, 5 },		// 93 bits per transform
			{ 6, 21, 4 },		// 93 bits per transform
			{ 6, 22, 3 },		// 93 bits per transform
			{ 6, 23, 2 },		// 93 bits per transform
			{ 7, 1, 23 },		// 93 bits per transform
			{ 7, 2, 22 },		// 93 bits per transform
			{ 7, 3, 21 },		// 93 bits per transform
			{ 7, 4, 20 },		// 93 bits per transform
			{ 7, 5, 19 },		// 93 bits per transform
			{ 7, 6, 18 },		// 93 bits per transform
			{ 7, 7, 17 },		// 93 bits per transform
			{ 7, 8, 16 },		// 93 bits per transform
			{ 7, 9, 15 },		// 93 bits per transform
			{ 7, 10, 14 },		// 93 bits per transform
			{ 7, 11, 13 },		// 93 bits per transform
			{ 7, 12, 12 },		// 93 bits per transform
			{ 7, 13, 11 },		// 93 bits per transform
			{ 7, 14, 10 },		// 93 bits per transform
			{ 7, 15, 9 },		// 93 bits per transform
			{ 7, 16, 8 },		// 93 bits per transform
			{ 7, 17, 7 },		// 93 bits per transform
			{ 7, 18, 6 },		// 93 bits per transform
			{ 7, 19, 5 },		// 93 bits per transform
			{ 7, 20, 4 },		// 93 bits per transform
			{ 7, 21, 3 },		// 93 bits per transform
			{ 7, 22, 2 },		// 93 bits per transform
			{ 7, 23, 1 },		// 93 bits per transform
			{ 8, 0, 23 },		// 93 bits per transform
			{ 8, 1, 22 },		// 93 bits per transform
			{ 8, 2, 21 },		// 93 bits per transform
			{ 8, 3, 20 },		// 93 bits per transform
			{ 8, 4, 19 },		// 93 bits per transform
			{ 8, 5, 18 },		// 93 bits per transform
			{ 8, 6, 17 },		// 93 bits per transform
			{ 8, 7, 16 },		// 93 bits per transform
			{ 8, 8, 15 },		// 93 bits per transform
			{ 8, 9, 14 },		// 93 bits per transform
			{ 8, 10, 13 },		// 93 bits per transform
			{ 8, 11, 12 },		// 93 bits per transform
			{ 8, 12, 11 },		// 93 bits per transform
			{ 8, 13, 10 },		// 93 bits per transform
			{ 8, 14, 9 },		// 93 bits per transform
			{ 8, 15, 8 },		// 93 bits per transform
			{ 8, 16, 7 },		// 93 bits per transform
			{ 8, 17, 6 },		// 93 bits per transform
			{ 8, 18, 5 },		// 93 bits per transform
			{ 8, 19, 4 },		// 93 bits per transform
			{ 8, 20, 3 },		// 93 bits per transform
			{ 8, 21, 2 },		// 93 bits per transform
			{ 8, 22, 1 },		// 93 bits per transform
			{ 8, 23, 0 },		// 93 bits per transform
			{ 9, 0, 22 },		// 93 bits per transform
			{ 9, 1, 21 },		// 93 bits per transform
			{ 9, 2, 20 },		// 93 bits per transform
			{ 9, 3, 19 },		// 93 bits per transform
			{ 9, 4, 18 },		// 93 bits per transform
			{ 9, 5, 17 },		// 93 bits per transform
			{ 9, 6, 16 },		// 93 bits per transform
			{ 9, 7, 15 },		// 93 bits per transform
			{ 9, 8, 14 },		// 93 bits per transform
			{ 9, 9, 13 },		// 93 bits per transform
			{ 9, 10, 12 },		// 93 bits per transform
			{ 9, 11, 11 },		// 93 bits per transform
			{ 9, 12, 10 },		// 93 bits per transform
			{ 9, 13, 9 },		// 93 bits per transform
			{ 9, 14, 8 },		// 93 bits per transform
			{ 9, 15, 7 },		// 93 bits per transform
			{ 9, 16, 6 },		// 93 bits per transform
			{ 9, 17, 5 },		// 93 bits per transform
			{ 9, 18, 4 },		// 93 bits per transform
			{ 9, 19, 3 },		// 93 bits per transform
			{ 9, 20, 2 },		// 93 bits per transform
			{ 9, 21, 1 },		// 93 bits per transform
			{ 9, 22, 0 },		// 93 bits per transform
			{ 10, 0, 21 },		// 93 bits per transform
			{ 10, 1, 20 },		// 93 bits per transform
			{ 10, 2, 19 },		// 93 bits per transform
			{ 10, 3, 18 },		// 93 bits per transform
			{ 10, 4, 17 },		// 93 bits per transform
			{ 10, 5, 16 },		// 93 bits per transform
			{ 10, 6, 15 },		// 93 bits per transform
			{ 10, 7, 14 },		// 93 bits per transform
			{ 10, 8, 13 },		// 93 bits per transform
			{ 10, 9, 12 },		// 93 bits per transform
			{ 10, 10, 11 },		// 93 bits per transform
			{ 10, 11, 10 },		// 93 bits per transform
			{ 10, 12, 9 },		// 93 bits per transform
			{ 10, 13, 8 },		// 93 bits per transform
			{ 10, 14, 7 },		// 93 bits per transform
			{ 10, 15, 6 },		// 93 bits per transform
			{ 10, 16, 5 },		// 93 bits per transform
			{ 10, 17, 4 },		// 93 bits per transform
			{ 10, 18, 3 },		// 93 bits per transform
			{ 10, 19, 2 },		// 93 bits per transform
			{ 10, 20, 1 },		// 93 bits per transform
			{ 10, 21, 0 },		// 93 bits per transform
			{ 11, 0, 20 },		// 93 bits per transform
			{ 11, 1, 19 },		// 93 bits per transform
			{ 11, 2, 18 },		// 93 bits per transform
			{ 11, 3, 17 },		// 93 bits per transform
			{ 11, 4, 16 },		// 93 bits per transform
			{ 11, 5, 15 },		// 93 bits per transform
			{ 11, 6, 14 },		// 93 bits per transform
			{ 11, 7, 13 },		// 93 bits per transform
			{ 11, 8, 12 },		// 93 bits per transform
			{ 11, 9, 11 },		// 93 bits per transform
			{ 11, 10, 10 },		// 93 bits per transform
			{ 11, 11, 9 },		// 93 bits per transform
			{ 11, 12, 8 },		// 93 bits per transform
			{ 11, 13, 7 },		// 93 bits per transform
			{ 11, 14, 6 },		// 93 bits per transform
			{ 11, 15, 5 },		// 93 bits per transform
			{ 11, 16, 4 },		// 93 bits per transform
			{ 11, 17, 3 },		// 93 bits per transform
			{ 11, 18, 2 },		// 93 bits per transform
			{ 11, 19, 1 },		// 93 bits per transform
			{ 11, 20, 0 },		// 93 bits per transform
			{ 12, 0, 19 },		// 93 bits per transform
			{ 12, 1, 18 },		// 93 bits per transform
			{ 12, 2, 17 },		// 93 bits per transform
			{ 12, 3, 16 },		// 93 bits per transform
			{ 12, 4, 15 },		// 93 bits per transform
			{ 12, 5, 14 },		// 93 bits per transform
			{ 12, 6, 13 },		// 93 bits per transform
			{ 12, 7, 12 },		// 93 bits per transform
			{ 12, 8, 11 },		// 93 bits per transform
			{ 12, 9, 10 },		// 93 bits per transform
			{ 12, 10, 9 },		// 93 bits per transform
			{ 12, 11, 8 },		// 93 bits per transform
			{ 12, 12, 7 },		// 93 bits per transform
			{ 12, 13, 6 },		// 93 bits per transform
			{ 12, 14, 5 },		// 93 bits per transform
			{ 12, 15, 4 },		// 93 bits per transform
			{ 12, 16, 3 },		// 93 bits per transform
			{ 12, 17, 2 },		// 93 bits per transform
			{ 12, 18, 1 },		// 93 bits per transform
			{ 12, 19, 0 },		// 93 bits per transform
			{ 13, 0, 18 },		// 93 bits per transform
			{ 13, 1, 17 },		// 93 bits per transform
			{ 13, 2, 16 },		// 93 bits per transform
			{ 13, 3, 15 },		// 93 bits per transform
			{ 13, 4, 14 },		// 93 bits per transform
			{ 13, 5, 13 },		// 93 bits per transform
			{ 13, 6, 12 },		// 93 bits per transform
			{ 13, 7, 11 },		// 93 bits per transform
			{ 13, 8, 10 },		// 93 bits per transform
			{ 13, 9, 9 },		// 93 bits per transform
			{ 13, 10, 8 },		// 93 bits per transform
			{ 13, 11, 7 },		// 93 bits per transform
			{ 13, 12, 6 },		// 93 bits per transform
			{ 13, 13, 5 },		// 93 bits per transform
			{ 13, 14, 4 },		// 93 bits per transform
			{ 13, 15, 3 },		// 93 bits per transform
			{ 13, 16, 2 },		// 93 bits per transform
			{ 13, 17, 1 },		// 93 bits per transform
			{ 13, 18, 0 },		// 93 bits per transform
			{ 14, 0, 17 },		// 93 bits per transform
			{ 14, 1, 16 },		// 93 bits per transform
			{ 14, 2, 15 },		// 93 bits per transform
			{ 14, 3, 14 },		// 93 bits per transform
			{ 14, 4, 13 },		// 93 bits per transform
			{ 14, 5, 12 },		// 93 bits per transform
			{ 14, 6, 11 },		// 93 bits per transform
			{ 14, 7, 10 },		// 93 bits per transform
			{ 14, 8, 9 },		// 93 bits per transform
			{ 14, 9, 8 },		// 93 bits per transform
			{ 14, 10, 7 },		// 93 bits per transform
			{ 14, 11, 6 },		// 93 bits per transform
			{ 14, 12, 5 },		// 93 bits per transform
			{ 14, 13, 4 },		// 93 bits per transform
			{ 14, 14, 3 },		// 93 bits per transform
			{ 14, 15, 2 },		// 93 bits per transform
			{ 14, 16, 1 },		// 93 bits per transform
			{ 14, 17, 0 },		// 93 bits per transform
			{ 15, 0, 16 },		// 93 bits per transform
			{ 15, 1, 15 },		// 93 bits per transform
			{ 15, 2, 14 },		// 93 bits per transform
			{ 15, 3, 13 },		// 93 bits per transform
			{ 15, 4, 12 },		// 93 bits per transform
			{ 15, 5, 11 },		// 93 bits per transform
			{ 15, 6, 10 },		// 93 bits per transform
			{ 15, 7, 9 },		// 93 bits per transform
			{ 15, 8, 8 },		// 93 bits per transform
			{ 15, 9, 7 },		// 93 bits per transform
			{ 15, 10, 6 },		// 93 bits per transform
			{ 15, 11, 5 },		// 93 bits per transform
			{ 15, 12, 4 },		// 93 bits per transform
			{ 15, 13, 3 },		// 93 bits per transform
			{ 15, 14, 2 },		// 93 bits per transform
			{ 15, 15, 1 },		// 93 bits per transform
			{ 15, 16, 0 },		// 93 bits per transform
			{ 16, 0, 15 },		// 93 bits per transform
			{ 16, 1, 14 },		// 93 bits per transform
			{ 16, 2, 13 },		// 93 bits per transform
			{ 16, 3, 12 },		// 93 bits per transform
			{ 16, 4, 11 },		// 93 bits per transform
			{ 16, 5, 10 },		// 93 bits per transform
			{ 16, 6, 9 },		// 93 bits per transform
			{ 16, 7, 8 },		// 93 bits per transform
			{ 16, 8, 7 },		// 93 bits per transform
			{ 16, 9, 6 },		// 93 bits per transform
			{ 16, 10, 5 },		// 93 bits per transform
			{ 16, 11, 4 },		// 93 bits per transform
			{ 16, 12, 3 },		// 93 bits per transform
			{ 16, 13, 2 },		// 93 bits per transform
			{ 16, 14, 1 },		// 93 bits per transform
			{ 16, 15, 0 },		// 93 bits per transform
			{ 17, 0, 14 },		// 93 bits per transform
			{ 17, 1, 13 },		// 93 bits per transform
			{ 17, 2, 12 },		// 93 bits per transform
			{ 17, 3, 11 },		// 93 bits per transform
			{ 17, 4, 10 },		// 93 bits per transform
			{ 17, 5, 9 },		// 93 bits per transform
			{ 17, 6, 8 },		// 93 bits per transform
			{ 17, 7, 7 },		// 93 bits per transform
			{ 17, 8, 6 },		// 93 bits per transform
			{ 17, 9, 5 },		// 93 bits per transform
			{ 17, 10, 4 },		// 93 bits per transform
			{ 17, 11, 3 },		// 93 bits per transform
			{ 17, 12, 2 },		// 93 bits per transform
			{ 17, 13, 1 },		// 93 bits per transform
			{ 17, 14, 0 },		// 93 bits per transform
			{ 18, 0, 13 },		// 93 bits per transform
			{ 18, 1, 12 },		// 93 bits per transform
			{ 18, 2, 11 },		// 93 bits per transform
			{ 18, 3, 10 },		// 93 bits per transform
			{ 18, 4, 9 },		// 93 bits per transform
			{ 18, 5, 8 },		// 93 bits per transform
			{ 18, 6, 7 },		// 93 bits per transform
			{ 18, 7, 6 },		// 93 bits per transform
			{ 18, 8, 5 },		// 93 bits per transform
			{ 18, 9, 4 },		// 93 bits per transform
			{ 18, 10, 3 },		// 93 bits per transform
			{ 18, 11, 2 },		// 93 bits per transform
			{ 18, 12, 1 },		// 93 bits per transform
			{ 18, 13, 0 },		// 93 bits per transform
			{ 19, 0, 12 },		// 93 bits per transform
			{ 19, 1, 11 },		// 93 bits per transform
			{ 19, 2, 10 },		// 93 bits per transform
			{ 19, 3, 9 },		// 93 bits per transform
			{ 19, 4, 8 },		// 93 bits per transform
			{ 19, 5, 7 },		// 93 bits per transform
			{ 19, 6, 6 },		// 93 bits per transform
			{ 19, 7, 5 },		// 93 bits per transform
			{ 19, 8, 4 },		// 93 bits per transform
			{ 19, 9, 3 },		// 93 bits per transform
			{ 19, 10, 2 },		// 93 bits per transform
			{ 19, 11, 1 },		// 93 bits per transform
			{ 19, 12, 0 },		// 93 bits per transform
			{ 20, 0, 11 },		// 93 bits per transform
			{ 20, 1, 10 },		// 93 bits per transform
			{ 20, 2, 9 },		// 93 bits per transform
			{ 20, 3, 8 },		// 93 bits per transform
			{ 20, 4, 7 },		// 93 bits per transform
			{ 20, 5, 6 },		// 93 bits per transform
			{ 20, 6, 5 },		// 93 bits per transform
			{ 20, 7, 4 },		// 93 bits per transform
			{ 20, 8, 3 },		// 93 bits per transform
			{ 20, 9, 2 },		// 93 bits per transform
			{ 20, 10, 1 },		// 93 bits per transform
			{ 20, 11, 0 },		// 93 bits per transform
			{ 21, 0, 10 },		// 93 bits per transform
			{ 21, 1, 9 },		// 93 bits per transform
			{ 21, 2, 8 },		// 93 bits per transform
			{ 21, 3, 7 },		// 93 bits per transform
			{ 21, 4, 6 },		// 93 bits per transform
			{ 21, 5, 5 },		// 93 bits per transform
			{ 21, 6, 4 },		// 93 bits per transform
			{ 21, 7, 3 },		// 93 bits per transform
			{ 21, 8, 2 },		// 93 bits per transform
			{ 21, 9, 1 },		// 93 bits per transform
			{ 21, 10, 0 },		// 93 bits per transform
			{ 22, 0, 9 },		// 93 bits per transform
			{ 22, 1, 8 },		// 93 bits per transform
			{ 22, 2, 7 },		// 93 bits per transform
			{ 22, 3, 6 },		// 93 bits per transform
			{ 22, 4, 5 },		// 93 bits per transform
			{ 22, 5, 4 },		// 93 bits per transform
			{ 22, 6, 3 },		// 93 bits per transform
			{ 22, 7, 2 },		// 93 bits per transform
			{ 22, 8, 1 },		// 93 bits per transform
			{ 22, 9, 0 },		// 93 bits per transform
			{ 23, 0, 8 },		// 93 bits per transform
			{ 23, 1, 7 },		// 93 bits per transform
			{ 23, 2, 6 },		// 93 bits per transform
			{ 23, 3, 5 },		// 93 bits per transform
			{ 23, 4, 4 },		// 93 bits per transform
			{ 23, 5, 3 },		// 93 bits per transform
			{ 23, 6, 2 },		// 93 bits per transform
			{ 23, 7, 1 },		// 93 bits per transform
			{ 23, 8, 0 },		// 93 bits per transform
			{ 0, 0, 24 },		// 96 bits per transform
			{ 0, 9, 23 },		// 96 bits per transform
			{ 0, 10, 22 },		// 96 bits per transform
			{ 0, 11, 21 },		// 96 bits per transform
			{ 0, 12, 20 },		// 96 bits per transform
			{ 0, 13, 19 },		// 96 bits per transform
			{ 0, 14, 18 },		// 96 bits per transform
			{ 0, 15, 17 },		// 96 bits per transform
			{ 0, 16, 16 },		// 96 bits per transform
			{ 0, 17, 15 },		// 96 bits per transform
			{ 0, 18, 14 },		// 96 bits per transform
			{ 0, 19, 13 },		// 96 bits per transform
			{ 0, 20, 12 },		// 96 bits per transform
			{ 0, 21, 11 },		// 96 bits per transform
			{ 0, 22, 10 },		// 96 bits per transform
			{ 0, 23, 9 },		// 96 bits per transform
			{ 0, 24, 0 },		// 96 bits per transform
			{ 1, 8, 23 },		// 96 bits per transform
			{ 1, 9, 22 },		// 96 bits per transform
			{ 1, 10, 21 },		// 96 bits per transform
			{ 1, 11, 20 },		// 96 bits per transform
			{ 1, 12, 19 },		// 96 bits per transform
			{ 1, 13, 18 },		// 96 bits per transform
			{ 1, 14, 17 },		// 96 bits per transform
			{ 1, 15, 16 },		// 96 bits per transform
			{ 1, 16, 15 },		// 96 bits per transform
			{ 1, 17, 14 },		// 96 bits per transform
			{ 1, 18, 13 },		// 96 bits per transform
			{ 1, 19, 12 },		// 96 bits per transform
			{ 1, 20, 11 },		// 96 bits per transform
			{ 1, 21, 10 },		// 96 bits per transform
			{ 1, 22, 9 },		// 96 bits per transform
			{ 1, 23, 8 },		// 96 bits per transform
			{ 2, 7, 23 },		// 96 bits per transform
			{ 2, 8, 22 },		// 96 bits per transform
			{ 2, 9, 21 },		// 96 bits per transform
			{ 2, 10, 20 },		// 96 bits per transform
			{ 2, 11, 19 },		// 96 bits per transform
			{ 2, 12, 18 },		// 96 bits per transform
			{ 2, 13, 17 },		// 96 bits per transform
			{ 2, 14, 16 },		// 96 bits per transform
			{ 2, 15, 15 },		// 96 bits per transform
			{ 2, 16, 14 },		// 96 bits per transform
			{ 2, 17, 13 },		// 96 bits per transform
			{ 2, 18, 12 },		// 96 bits per transform
			{ 2, 19, 11 },		// 96 bits per transform
			{ 2, 20, 10 },		// 96 bits per transform
			{ 2, 21, 9 },		// 96 bits per transform
			{ 2, 22, 8 },		// 96 bits per transform
			{ 2, 23, 7 },		// 96 bits per transform
			{ 3, 6, 23 },		// 96 bits per transform
			{ 3, 7, 22 },		// 96 bits per transform
			{ 3, 8, 21 },		// 96 bits per transform
			{ 3, 9, 20 },		// 96 bits per transform
			{ 3, 10, 19 },		// 96 bits per transform
			{ 3, 11, 18 },		// 96 bits per transform
			{ 3, 12, 17 },		// 96 bits per transform
			{ 3, 13, 16 },		// 96 bits per transform
			{ 3, 14, 15 },		// 96 bits per transform
			{ 3, 15, 14 },		// 96 bits per transform
			{ 3, 16, 13 },		// 96 bits per transform
			{ 3, 17, 12 },		// 96 bits per transform
			{ 3, 18, 11 },		// 96 bits per transform
			{ 3, 19, 10 },		// 96 bits per transform
			{ 3, 20, 9 },		// 96 bits per transform
			{ 3, 21, 8 },		// 96 bits per transform
			{ 3, 22, 7 },		// 96 bits per transform
			{ 3, 23, 6 },		// 96 bits per transform
			{ 4, 5, 23 },		// 96 bits per transform
			{ 4, 6, 22 },		// 96 bits per transform
			{ 4, 7, 21 },		// 96 bits per transform
			{ 4, 8, 20 },		// 96 bits per transform
			{ 4, 9, 19 },		// 96 bits per transform
			{ 4, 10, 18 },		// 96 bits per transform
			{ 4, 11, 17 },		// 96 bits per transform
			{ 4, 12, 16 },		// 96 bits per transform
			{ 4, 13, 15 },		// 96 bits per transform
			{ 4, 14, 14 },		// 96 bits per transform
			{ 4, 15, 13 },		// 96 bits per transform
			{ 4, 16, 12 },		// 96 bits per transform
			{ 4, 17, 11 },		// 96 bits per transform
			{ 4, 18, 10 },		// 96 bits per transform
			{ 4, 19, 9 },		// 96 bits per transform
			{ 4, 20, 8 },		// 96 bits per transform
			{ 4, 21, 7 },		// 96 bits per transform
			{ 4, 22, 6 },		// 96 bits per transform
			{ 4, 23, 5 },		// 96 bits per transform
			{ 5, 4, 23 },		// 96 bits per transform
			{ 5, 5, 22 },		// 96 bits per transform
			{ 5, 6, 21 },		// 96 bits per transform
			{ 5, 7, 20 },		// 96 bits per transform
			{ 5, 8, 19 },		// 96 bits per transform
			{ 5, 9, 18 },		// 96 bits per transform
			{ 5, 10, 17 },		// 96 bits per transform
			{ 5, 11, 16 },		// 96 bits per transform
			{ 5, 12, 15 },		// 96 bits per transform
			{ 5, 13, 14 },		// 96 bits per transform
			{ 5, 14, 13 },		// 96 bits per transform
			{ 5, 15, 12 },		// 96 bits per transform
			{ 5, 16, 11 },		// 96 bits per transform
			{ 5, 17, 10 },		// 96 bits per transform
			{ 5, 18, 9 },		// 96 bits per transform
			{ 5, 19, 8 },		// 96 bits per transform
			{ 5, 20, 7 },		// 96 bits per transform
			{ 5, 21, 6 },		// 96 bits per transform
			{ 5, 22, 5 },		// 96 bits per transform
			{ 5, 23, 4 },		// 96 bits per transform
			{ 6, 3, 23 },		// 96 bits per transform
			{ 6, 4, 22 },		// 96 bits per transform
			{ 6, 5, 21 },		// 96 bits per transform
			{ 6, 6, 20 },		// 96 bits per transform
			{ 6, 7, 19 },		// 96 bits per transform
			{ 6, 8, 18 },		// 96 bits per transform
			{ 6, 9, 17 },		// 96 bits per transform
			{ 6, 10, 16 },		// 96 bits per transform
			{ 6, 11, 15 },		// 96 bits per transform
			{ 6, 12, 14 },		// 96 bits per transform
			{ 6, 13, 13 },		// 96 bits per transform
			{ 6, 14, 12 },		// 96 bits per transform
			{ 6, 15, 11 },		// 96 bits per transform
			{ 6, 16, 10 },		// 96 bits per transform
			{ 6, 17, 9 },		// 96 bits per transform
			{ 6, 18, 8 },		// 96 bits per transform
			{ 6, 19, 7 },		// 96 bits per transform
			{ 6, 20, 6 },		// 96 bits per transform
			{ 6, 21, 5 },		// 96 bits per transform
			{ 6, 22, 4 },		// 96 bits per transform
			{ 6, 23, 3 },		// 96 bits per transform
			{ 7, 2, 23 },		// 96 bits per transform
			{ 7, 3, 22 },		// 96 bits per transform
			{ 7, 4, 21 },		// 96 bits per transform
			{ 7, 5, 20 },		// 96 bits per transform
			{ 7, 6, 19 },		// 96 bits per transform
			{ 7, 7, 18 },		// 96 bits per transform
			{ 7, 8, 17 },		// 96 bits per transform
			{ 7, 9, 16 },		// 96 bits per transform
			{ 7, 10, 15 },		// 96 bits per transform
			{ 7, 11, 14 },		// 96 bits per transform
			{ 7, 12, 13 },		// 96 bits per transform
			{ 7, 13, 12 },		// 96 bits per transform
			{ 7, 14, 11 },		// 96 bits per transform
			{ 7, 15, 10 },		// 96 bits per transform
			{ 7, 16, 9 },		// 96 bits per transform
			{ 7, 17, 8 },		// 96 bits per transform
			{ 7, 18, 7 },		// 96 bits per transform
			{ 7, 19, 6 },		// 96 bits per transform
			{ 7, 20, 5 },		// 96 bits per transform
			{ 7, 21, 4 },		// 96 bits per transform
			{ 7, 22, 3 },		// 96 bits per transform
			{ 7, 23, 2 },		// 96 bits per transform
			{ 8, 1, 23 },		// 96 bits per transform
			{ 8, 2, 22 },		// 96 bits per transform
			{ 8, 3, 21 },		// 96 bits per transform
			{ 8, 4, 20 },		// 96 bits per transform
			{ 8, 5, 19 },		// 96 bits per transform
			{ 8, 6, 18 },		// 96 bits per transform
			{ 8, 7, 17 },		// 96 bits per transform
			{ 8, 8, 16 },		// 96 bits per transform
			{ 8, 9, 15 },		// 96 bits per transform
			{ 8, 10, 14 },		// 96 bits per transform
			{ 8, 11, 13 },		// 96 bits per transform
			{ 8, 12, 12 },		// 96 bits per transform
			{ 8, 13, 11 },		// 96 bits per transform
			{ 8, 14, 10 },		// 96 bits per transform
			{ 8, 15, 9 },		// 96 bits per transform
			{ 8, 16, 8 },		// 96 bits per transform
			{ 8, 17, 7 },		// 96 bits per transform
			{ 8, 18, 6 },		// 96 bits per transform
			{ 8, 19, 5 },		// 96 bits per transform
			{ 8, 20, 4 },		// 96 bits per transform
			{ 8, 21, 3 },		// 96 bits per transform
			{ 8, 22, 2 },		// 96 bits per transform
			{ 8, 23, 1 },		// 96 bits per transform
			{ 9, 0, 23 },		// 96 bits per transform
			{ 9, 1, 22 },		// 96 bits per transform
			{ 9, 2, 21 },		// 96 bits per transform
			{ 9, 3, 20 },		// 96 bits per transform
			{ 9, 4, 19 },		// 96 bits per transform
			{ 9, 5, 18 },		// 96 bits per transform
			{ 9, 6, 17 },		// 96 bits per transform
			{ 9, 7, 16 },		// 96 bits per transform
			{ 9, 8, 15 },		// 96 bits per transform
			{ 9, 9, 14 },		// 96 bits per transform
			{ 9, 10, 13 },		// 96 bits per transform
			{ 9, 11, 12 },		// 96 bits per transform
			{ 9, 12, 11 },		// 96 bits per transform
			{ 9, 13, 10 },		// 96 bits per transform
			{ 9, 14, 9 },		// 96 bits per transform
			{ 9, 15, 8 },		// 96 bits per transform
			{ 9, 16, 7 },		// 96 bits per transform
			{ 9, 17, 6 },		// 96 bits per transform
			{ 9, 18, 5 },		// 96 bits per transform
			{ 9, 19, 4 },		// 96 bits per transform
			{ 9, 20, 3 },		// 96 bits per transform
			{ 9, 21, 2 },		// 96 bits per transform
			{ 9, 22, 1 },		// 96 bits per transform
			{ 9, 23, 0 },		// 96 bits per transform
			{ 10, 0, 22 },		// 96 bits per transform
			{ 10, 1, 21 },		// 96 bits per transform
			{ 10, 2, 20 },		// 96 bits per transform
			{ 10, 3, 19 },		// 96 bits per transform
			{ 10, 4, 18 },		// 96 bits per transform
			{ 10, 5, 17 },		// 96 bits per transform
			{ 10, 6, 16 },		// 96 bits per transform
			{ 10, 7, 15 },		// 96 bits per transform
			{ 10, 8, 14 },		// 96 bits per transform
			{ 10, 9, 13 },		// 96 bits per transform
			{ 10, 10, 12 },		// 96 bits per transform
			{ 10, 11, 11 },		// 96 bits per transform
			{ 10, 12, 10 },		// 96 bits per transform
			{ 10, 13, 9 },		// 96 bits per transform
			{ 10, 14, 8 },		// 96 bits per transform
			{ 10, 15, 7 },		// 96 bits per transform
			{ 10, 16, 6 },		// 96 bits per transform
			{ 10, 17, 5 },		// 96 bits per transform
			{ 10, 18, 4 },		// 96 bits per transform
			{ 10, 19, 3 },		// 96 bits per transform
			{ 10, 20, 2 },		// 96 bits per transform
			{ 10, 21, 1 },		// 96 bits per transform
			{ 10, 22, 0 },		// 96 bits per transform
			{ 11, 0, 21 },		// 96 bits per transform
			{ 11, 1, 20 },		// 96 bits per transform
			{ 11, 2, 19 },		// 96 bits per transform
			{ 11, 3, 18 },		// 96 bits per transform
			{ 11, 4, 17 },		// 96 bits per transform
			{ 11, 5, 16 },		// 96 bits per transform
			{ 11, 6, 15 },		// 96 bits per transform
			{ 11, 7, 14 },		// 96 bits per transform
			{ 11, 8, 13 },		// 96 bits per transform
			{ 11, 9, 12 },		// 96 bits per transform
			{ 11, 10, 11 },		// 96 bits per transform
			{ 11, 11, 10 },		// 96 bits per transform
			{ 11, 12, 9 },		// 96 bits per transform
			{ 11, 13, 8 },		// 96 bits per transform
			{ 11, 14, 7 },		// 96 bits per transform
			{ 11, 15, 6 },		// 96 bits per transform
			{ 11, 16, 5 },		// 96 bits per transform
			{ 11, 17, 4 },		// 96 bits per transform
			{ 11, 18, 3 },		// 96 bits per transform
			{ 11, 19, 2 },		// 96 bits per transform
			{ 11, 20, 1 },		// 96 bits per transform
			{ 11, 21, 0 },		// 96 bits per transform
			{ 12, 0, 20 },		// 96 bits per transform
			{ 12, 1, 19 },		// 96 bits per transform
			{ 12, 2, 18 },		// 96 bits per transform
			{ 12, 3, 17 },		// 96 bits per transform
			{ 12, 4, 16 },		// 96 bits per transform
			{ 12, 5, 15 },		// 96 bits per transform
			{ 12, 6, 14 },		// 96 bits per transform
			{ 12, 7, 13 },		// 96 bits per transform
			{ 12, 8, 12 },		// 96 bits per transform
			{ 12, 9, 11 },		// 96 bits per transform
			{ 12, 10, 10 },		// 96 bits per transform
			{ 12, 11, 9 },		// 96 bits per transform
			{ 12, 12, 8 },		// 96 bits per transform
			{ 12, 13, 7 },		// 96 bits per transform
			{ 12, 14, 6 },		// 96 bits per transform
			{ 12, 15, 5 },		// 96 bits per transform
			{ 12, 16, 4 },		// 96 bits per transform
			{ 12, 17, 3 },		// 96 bits per transform
			{ 12, 18, 2 },		// 96 bits per transform
			{ 12, 19, 1 },		// 96 bits per transform
			{ 12, 20, 0 },		// 96 bits per transform
			{ 13, 0, 19 },		// 96 bits per transform
			{ 13, 1, 18 },		// 96 bits per transform
			{ 13, 2, 17 },		// 96 bits per transform
			{ 13, 3, 16 },		// 96 bits per transform
			{ 13, 4, 15 },		// 96 bits per transform
			{ 13, 5, 14 },		// 96 bits per transform
			{ 13, 6, 13 },		// 96 bits per transform
			{ 13, 7, 12 },		// 96 bits per transform
			{ 13, 8, 11 },		// 96 bits per transform
			{ 13, 9, 10 },		// 96 bits per transform
			{ 13, 10, 9 },		// 96 bits per transform
			{ 13, 11, 8 },		// 96 bits per transform
			{ 13, 12, 7 },		// 96 bits per transform
			{ 13, 13, 6 },		// 96 bits per transform
			{ 13, 14, 5 },		// 96 bits per transform
			{ 13, 15, 4 },		// 96 bits per transform
			{ 13, 16, 3 },		// 96 bits per transform
			{ 13, 17, 2 },		// 96 bits per transform
			{ 13, 18, 1 },		// 96 bits per transform
			{ 13, 19, 0 },		// 96 bits per transform
			{ 14, 0, 18 },		// 96 bits per transform
			{ 14, 1, 17 },		// 96 bits per transform
			{ 14, 2, 16 },		// 96 bits per transform
			{ 14, 3, 15 },		// 96 bits per transform
			{ 14, 4, 14 },		// 96 bits per transform
			{ 14, 5, 13 },		// 96 bits per transform
			{ 14, 6, 12 },		// 96 bits per transform
			{ 14, 7, 11 },		// 96 bits per transform
			{ 14, 8, 10 },		// 96 bits per transform
			{ 14, 9, 9 },		// 96 bits per transform
			{ 14, 10, 8 },		// 96 bits per transform
			{ 14, 11, 7 },		// 96 bits per transform
			{ 14, 12, 6 },		// 96 bits per transform
			{ 14, 13, 5 },		// 96 bits per transform
			{ 14, 14, 4 },		// 96 bits per transform
			{ 14, 15, 3 },		// 96 bits per transform
			{ 14, 16, 2 },		// 96 bits per transform
			{ 14, 17, 1 },		// 96 bits per transform
			{ 14, 18, 0 },		// 96 bits per transform
			{ 15, 0, 17 },		// 96 bits per transform
			{ 15, 1, 16 },		// 96 bits per transform
			{ 15, 2, 15 },		// 96 bits per transform
			{ 15, 3, 14 },		// 96 bits per transform
			{ 15, 4, 13 },		// 96 bits per transform
			{ 15, 5, 12 },		// 96 bits per transform
			{ 15, 6, 11 },		// 96 bits per transform
			{ 15, 7, 10 },		// 96 bits per transform
			{ 15, 8, 9 },		// 96 bits per transform
			{ 15, 9, 8 },		// 96 bits per transform
			{ 15, 10, 7 },		// 96 bits per transform
			{ 15, 11, 6 },		// 96 bits per transform
			{ 15, 12, 5 },		// 96 bits per transform
			{ 15, 13, 4 },		// 96 bits per transform
			{ 15, 14, 3 },		// 96 bits per transform
			{ 15, 15, 2 },		// 96 bits per transform
			{ 15, 16, 1 },		// 96 bits per transform
			{ 15, 17, 0 },		// 96 bits per transform
			{ 16, 0, 16 },		// 96 bits per transform
			{ 16, 1, 15 },		// 96 bits per transform
			{ 16, 2, 14 },		// 96 bits per transform
			{ 16, 3, 13 },		// 96 bits per transform
			{ 16, 4, 12 },		// 96 bits per transform
			{ 16, 5, 11 },		// 96 bits per transform
			{ 16, 6, 10 },		// 96 bits per transform
			{ 16, 7, 9 },		// 96 bits per transform
			{ 16, 8, 8 },		// 96 bits per transform
			{ 16, 9, 7 },		// 96 bits per transform
			{ 16, 10, 6 },		// 96 bits per transform
			{ 16, 11, 5 },		// 96 bits per transform
			{ 16, 12, 4 },		// 96 bits per transform
			{ 16, 13, 3 },		// 96 bits per transform
			{ 16, 14, 2 },		// 96 bits per transform
			{ 16, 15, 1 },		// 96 bits per transform
			{ 16, 16, 0 },		// 96 bits per transform
			{ 17, 0, 15 },		// 96 bits per transform
			{ 17, 1, 14 },		// 96 bits per transform
			{ 17, 2, 13 },		// 96 bits per transform
			{ 17, 3, 12 },		// 96 bits per transform
			{ 17, 4, 11 },		// 96 bits per transform
			{ 17, 5, 10 },		// 96 bits per transform
			{ 17, 6, 9 },		// 96 bits per transform
			{ 17, 7, 8 },		// 96 bits per transform
			{ 17, 8, 7 },		// 96 bits per transform
			{ 17, 9, 6 },		// 96 bits per transform
			{ 17, 10, 5 },		// 96 bits per transform
			{ 17, 11, 4 },		// 96 bits per transform
			{ 17, 12, 3 },		// 96 bits per transform
			{ 17, 13, 2 },		// 96 bits per transform
			{ 17, 14, 1 },		// 96 bits per transform
			{ 17, 15, 0 },		// 96 bits per transform
			{ 18, 0, 14 },		// 96 bits per transform
			{ 18, 1, 13 },		// 96 bits per transform
			{ 18, 2, 12 },		// 96 bits per transform
			{ 18, 3, 11 },		// 96 bits per transform
			{ 18, 4, 10 },		// 96 bits per transform
			{ 18, 5, 9 },		// 96 bits per transform
			{ 18, 6, 8 },		// 96 bits per transform
			{ 18, 7, 7 },		// 96 bits per transform
			{ 18, 8, 6 },		// 96 bits per transform
			{ 18, 9, 5 },		// 96 bits per transform
			{ 18, 10, 4 },		// 96 bits per transform
			{ 18, 11, 3 },		// 96 bits per transform
			{ 18, 12, 2 },		// 96 bits per transform
			{ 18, 13, 1 },		// 96 bits per transform
			{ 18, 14, 0 },		// 96 bits per transform
			{ 19, 0, 13 },		// 96 bits per transform
			{ 19, 1, 12 },		// 96 bits per transform
			{ 19, 2, 11 },		// 96 bits per transform
			{ 19, 3, 10 },		// 96 bits per transform
			{ 19, 4, 9 },		// 96 bits per transform
			{ 19, 5, 8 },		// 96 bits per transform
			{ 19, 6, 7 },		// 96 bits per transform
			{ 19, 7, 6 },		// 96 bits per transform
			{ 19, 8, 5 },		// 96 bits per transform
			{ 19, 9, 4 },		// 96 bits per transform
			{ 19, 10, 3 },		// 96 bits per transform
			{ 19, 11, 2 },		// 96 bits per transform
			{ 19, 12, 1 },		// 96 bits per transform
			{ 19, 13, 0 },		// 96 bits per transform
			{ 20, 0, 12 },		// 96 bits per transform
			{ 20, 1, 11 },		// 96 bits per transform
			{ 20, 2, 10 },		// 96 bits per transform
			{ 20, 3, 9 },		// 96 bits per transform
			{ 20, 4, 8 },		// 96 bits per transform
			{ 20, 5, 7 },		// 96 bits per transform
			{ 20, 6, 6 },		// 96 bits per transform
			{ 20, 7, 5 },		// 96 bits per transform
			{ 20, 8, 4 },		// 96 bits per transform
			{ 20, 9, 3 },		// 96 bits per transform
			{ 20, 10, 2 },		// 96 bits per transform
			{ 20, 11, 1 },		// 96 bits per transform
			{ 20, 12, 0 },		// 96 bits per transform
			{ 21, 0, 11 },		// 96 bits per transform
			{ 21, 1, 10 },		// 96 bits per transform
			{ 21, 2, 9 },		// 96 bits per transform
			{ 21, 3, 8 },		// 96 bits per transform
			{ 21, 4, 7 },		// 96 bits per transform
			{ 21, 5, 6 },		// 96 bits per transform
			{ 21, 6, 5 },		// 96 bits per transform
			{ 21, 7, 4 },		// 96 bits per transform
			{ 21, 8, 3 },		// 96 bits per transform
			{ 21, 9, 2 },		// 96 bits per transform
			{ 21, 10, 1 },		// 96 bits per transform
			{ 21, 11, 0 },		// 96 bits per transform
			{ 22, 0, 10 },		// 96 bits per transform
			{ 22, 1, 9 },		// 96 bits per transform
			{ 22, 2, 8 },		// 96 bits per transform
			{ 22, 3, 7 },		// 96 bits per transform
			{ 22, 4, 6 },		// 96 bits per transform
			{ 22, 5, 5 },		// 96 bits per transform
			{ 22, 6, 4 },		// 96 bits per transform
			{ 22, 7, 3 },		// 96 bits per transform
			{ 22, 8, 2 },		// 96 bits per transform
			{ 22, 9, 1 },		// 96 bits per transform
			{ 22, 10, 0 },		// 96 bits per transform
			{ 23, 0, 9 },		// 96 bits per transform
			{ 23, 1, 8 },		// 96 bits per transform
			{ 23, 2, 7 },		// 96 bits per transform
			{ 23, 3, 6 },		// 96 bits per transform
			{ 23, 4, 5 },		// 96 bits per transform
			{ 23, 5, 4 },		// 96 bits per transform
			{ 23, 6, 3 },		// 96 bits per transform
			{ 23, 7, 2 },		// 96 bits per transform
			{ 23, 8, 1 },		// 96 bits per transform
			{ 23, 9, 0 },		// 96 bits per transform
			{ 24, 0, 0 },		// 96 bits per transform
			{ 0, 1, 24 },		// 99 bits per transform
			{ 0, 10, 23 },		// 99 bits per transform
			{ 0, 11, 22 },		// 99 bits per transform
			{ 0, 12, 21 },		// 99 bits per transform
			{ 0, 13, 20 },		// 99 bits per transform
			{ 0, 14, 19 },		// 99 bits per transform
			{ 0, 15, 18 },		// 99 bits per transform
			{ 0, 16, 17 },		// 99 bits per transform
			{ 0, 17, 16 },		// 99 bits per transform
			{ 0, 18, 15 },		// 99 bits per transform
			{ 0, 19, 14 },		// 99 bits per transform
			{ 0, 20, 13 },		// 99 bits per transform
			{ 0, 21, 12 },		// 99 bits per transform
			{ 0, 22, 11 },		// 99 bits per transform
			{ 0, 23, 10 },		// 99 bits per transform
			{ 0, 24, 1 },		// 99 bits per transform
			{ 1, 0, 24 },		// 99 bits per transform
			{ 1, 9, 23 },		// 99 bits per transform
			{ 1, 10, 22 },		// 99 bits per transform
			{ 1, 11, 21 },		// 99 bits per transform
			{ 1, 12, 20 },		// 99 bits per transform
			{ 1, 13, 19 },		// 99 bits per transform
			{ 1, 14, 18 },		// 99 bits per transform
			{ 1, 15, 17 },		// 99 bits per transform
			{ 1, 16, 16 },		// 99 bits per transform
			{ 1, 17, 15 },		// 99 bits per transform
			{ 1, 18, 14 },		// 99 bits per transform
			{ 1, 19, 13 },		// 99 bits per transform
			{ 1, 20, 12 },		// 99 bits per transform
			{ 1, 21, 11 },		// 99 bits per transform
			{ 1, 22, 10 },		// 99 bits per transform
			{ 1, 23, 9 },		// 99 bits per transform
			{ 1, 24, 0 },		// 99 bits per transform
			{ 2, 8, 23 },		// 99 bits per transform
			{ 2, 9, 22 },		// 99 bits per transform
			{ 2, 10, 21 },		// 99 bits per transform
			{ 2, 11, 20 },		// 99 bits per transform
			{ 2, 12, 19 },		// 99 bits per transform
			{ 2, 13, 18 },		// 99 bits per transform
			{ 2, 14, 17 },		// 99 bits per transform
			{ 2, 15, 16 },		// 99 bits per transform
			{ 2, 16, 15 },		// 99 bits per transform
			{ 2, 17, 14 },		// 99 bits per transform
			{ 2, 18, 13 },		// 99 bits per transform
			{ 2, 19, 12 },		// 99 bits per transform
			{ 2, 20, 11 },		// 99 bits per transform
			{ 2, 21, 10 },		// 99 bits per transform
			{ 2, 22, 9 },		// 99 bits per transform
			{ 2, 23, 8 },		// 99 bits per transform
			{ 3, 7, 23 },		// 99 bits per transform
			{ 3, 8, 22 },		// 99 bits per transform
			{ 3, 9, 21 },		// 99 bits per transform
			{ 3, 10, 20 },		// 99 bits per transform
			{ 3, 11, 19 },		// 99 bits per transform
			{ 3, 12, 18 },		// 99 bits per transform
			{ 3, 13, 17 },		// 99 bits per transform
			{ 3, 14, 16 },		// 99 bits per transform
			{ 3, 15, 15 },		// 99 bits per transform
			{ 3, 16, 14 },		// 99 bits per transform
			{ 3, 17, 13 },		// 99 bits per transform
			{ 3, 18, 12 },		// 99 bits per transform
			{ 3, 19, 11 },		// 99 bits per transform
			{ 3, 20, 10 },		// 99 bits per transform
			{ 3, 21, 9 },		// 99 bits per transform
			{ 3, 22, 8 },		// 99 bits per transform
			{ 3, 23, 7 },		// 99 bits per transform
			{ 4, 6, 23 },		// 99 bits per transform
			{ 4, 7, 22 },		// 99 bits per transform
			{ 4, 8, 21 },		// 99 bits per transform
			{ 4, 9, 20 },		// 99 bits per transform
			{ 4, 10, 19 },		// 99 bits per transform
			{ 4, 11, 18 },		// 99 bits per transform
			{ 4, 12, 17 },		// 99 bits per transform
			{ 4, 13, 16 },		// 99 bits per transform
			{ 4, 14, 15 },		// 99 bits per transform
			{ 4, 15, 14 },		// 99 bits per transform
			{ 4, 16, 13 },		// 99 bits per transform
			{ 4, 17, 12 },		// 99 bits per transform
			{ 4, 18, 11 },		// 99 bits per transform
			{ 4, 19, 10 },		// 99 bits per transform
			{ 4, 20, 9 },		// 99 bits per transform
			{ 4, 21, 8 },		// 99 bits per transform
			{ 4, 22, 7 },		// 99 bits per transform
			{ 4, 23, 6 },		// 99 bits per transform
			{ 5, 5, 23 },		// 99 bits per transform
			{ 5, 6, 22 },		// 99 bits per transform
			{ 5, 7, 21 },		// 99 bits per transform
			{ 5, 8, 20 },		// 99 bits per transform
			{ 5, 9, 19 },		// 99 bits per transform
			{ 5, 10, 18 },		// 99 bits per transform
			{ 5, 11, 17 },		// 99 bits per transform
			{ 5, 12, 16 },		// 99 bits per transform
			{ 5, 13, 15 },		// 99 bits per transform
			{ 5, 14, 14 },		// 99 bits per transform
			{ 5, 15, 13 },		// 99 bits per transform
			{ 5, 16, 12 },		// 99 bits per transform
			{ 5, 17, 11 },		// 99 bits per transform
			{ 5, 18, 10 },		// 99 bits per transform
			{ 5, 19, 9 },		// 99 bits per transform
			{ 5, 20, 8 },		// 99 bits per transform
			{ 5, 21, 7 },		// 99 bits per transform
			{ 5, 22, 6 },		// 99 bits per transform
			{ 5, 23, 5 },		// 99 bits per transform
			{ 6, 4, 23 },		// 99 bits per transform
			{ 6, 5, 22 },		// 99 bits per transform
			{ 6, 6, 21 },		// 99 bits per transform
			{ 6, 7, 20 },		// 99 bits per transform
			{ 6, 8, 19 },		// 99 bits per transform
			{ 6, 9, 18 },		// 99 bits per transform
			{ 6, 10, 17 },		// 99 bits per transform
			{ 6, 11, 16 },		// 99 bits per transform
			{ 6, 12, 15 },		// 99 bits per transform
			{ 6, 13, 14 },		// 99 bits per transform
			{ 6, 14, 13 },		// 99 bits per transform
			{ 6, 15, 12 },		// 99 bits per transform
			{ 6, 16, 11 },		// 99 bits per transform
			{ 6, 17, 10 },		// 99 bits per transform
			{ 6, 18, 9 },		// 99 bits per transform
			{ 6, 19, 8 },		// 99 bits per transform
			{ 6, 20, 7 },		// 99 bits per transform
			{ 6, 21, 6 },		// 99 bits per transform
			{ 6, 22, 5 },		// 99 bits per transform
			{ 6, 23, 4 },		// 99 bits per transform
			{ 7, 3, 23 },		// 99 bits per transform
			{ 7, 4, 22 },		// 99 bits per transform
			{ 7, 5, 21 },		// 99 bits per transform
			{ 7, 6, 20 },		// 99 bits per transform
			{ 7, 7, 19 },		// 99 bits per transform
			{ 7, 8, 18 },		// 99 bits per transform
			{ 7, 9, 17 },		// 99 bits per transform
			{ 7, 10, 16 },		// 99 bits per transform
			{ 7, 11, 15 },		// 99 bits per transform
			{ 7, 12, 14 },		// 99 bits per transform
			{ 7, 13, 13 },		// 99 bits per transform
			{ 7, 14, 12 },		// 99 bits per transform
			{ 7, 15, 11 },		// 99 bits per transform
			{ 7, 16, 10 },		// 99 bits per transform
			{ 7, 17, 9 },		// 99 bits per transform
			{ 7, 18, 8 },		// 99 bits per transform
			{ 7, 19, 7 },		// 99 bits per transform
			{ 7, 20, 6 },		// 99 bits per transform
			{ 7, 21, 5 },		// 99 bits per transform
			{ 7, 22, 4 },		// 99 bits per transform
			{ 7, 23, 3 },		// 99 bits per transform
			{ 8, 2, 23 },		// 99 bits per transform
			{ 8, 3, 22 },		// 99 bits per transform
			{ 8, 4, 21 },		// 99 bits per transform
			{ 8, 5, 20 },		// 99 bits per transform
			{ 8, 6, 19 },		// 99 bits per transform
			{ 8, 7, 18 },		// 99 bits per transform
			{ 8, 8, 17 },		// 99 bits per transform
			{ 8, 9, 16 },		// 99 bits per transform
			{ 8, 10, 15 },		// 99 bits per transform
			{ 8, 11, 14 },		// 99 bits per transform
			{ 8, 12, 13 },		// 99 bits per transform
			{ 8, 13, 12 },		// 99 bits per transform
			{ 8, 14, 11 },		// 99 bits per transform
			{ 8, 15, 10 },		// 99 bits per transform
			{ 8, 16, 9 },		// 99 bits per transform
			{ 8, 17, 8 },		// 99 bits per transform
			{ 8, 18, 7 },		// 99 bits per transform
			{ 8, 19, 6 },		// 99 bits per transform
			{ 8, 20, 5 },		// 99 bits per transform
			{ 8, 21, 4 },		// 99 bits per transform
			{ 8, 22, 3 },		// 99 bits per transform
			{ 8, 23, 2 },		// 99 bits per transform
			{ 9, 1, 23 },		// 99 bits per transform
			{ 9, 2, 22 },		// 99 bits per transform
			{ 9, 3, 21 },		// 99 bits per transform
			{ 9, 4, 20 },		// 99 bits per transform
			{ 9, 5, 19 },		// 99 bits per transform
			{ 9, 6, 18 },		// 99 bits per transform
			{ 9, 7, 17 },		// 99 bits per transform
			{ 9, 8, 16 },		// 99 bits per transform
			{ 9, 9, 15 },		// 99 bits per transform
			{ 9, 10, 14 },		// 99 bits per transform
			{ 9, 11, 13 },		// 99 bits per transform
			{ 9, 12, 12 },		// 99 bits per transform
			{ 9, 13, 11 },		// 99 bits per transform
			{ 9, 14, 10 },		// 99 bits per transform
			{ 9, 15, 9 },		// 99 bits per transform
			{ 9, 16, 8 },		// 99 bits per transform
			{ 9, 17, 7 },		// 99 bits per transform
			{ 9, 18, 6 },		// 99 bits per transform
			{ 9, 19, 5 },		// 99 bits per transform
			{ 9, 20, 4 },		// 99 bits per transform
			{ 9, 21, 3 },		// 99 bits per transform
			{ 9, 22, 2 },		// 99 bits per transform
			{ 9, 23, 1 },		// 99 bits per transform
			{ 10, 0, 23 },		// 99 bits per transform
			{ 10, 1, 22 },		// 99 bits per transform
			{ 10, 2, 21 },		// 99 bits per transform
			{ 10, 3, 20 },		// 99 bits per transform
			{ 10, 4, 19 },		// 99 bits per transform
			{ 10, 5, 18 },		// 99 bits per transform
			{ 10, 6, 17 },		// 99 bits per transform
			{ 10, 7, 16 },		// 99 bits per transform
			{ 10, 8, 15 },		// 99 bits per transform
			{ 10, 9, 14 },		// 99 bits per transform
			{ 10, 10, 13 },		// 99 bits per transform
			{ 10, 11, 12 },		// 99 bits per transform
			{ 10, 12, 11 },		// 99 bits per transform
			{ 10, 13, 10 },		// 99 bits per transform
			{ 10, 14, 9 },		// 99 bits per transform
			{ 10, 15, 8 },		// 99 bits per transform
			{ 10, 16, 7 },		// 99 bits per transform
			{ 10, 17, 6 },		// 99 bits per transform
			{ 10, 18, 5 },		// 99 bits per transform
			{ 10, 19, 4 },		// 99 bits per transform
			{ 10, 20, 3 },		// 99 bits per transform
			{ 10, 21, 2 },		// 99 bits per transform
			{ 10, 22, 1 },		// 99 bits per transform
			{ 10, 23, 0 },		// 99 bits per transform
			{ 11, 0, 22 },		// 99 bits per transform
			{ 11, 1, 21 },		// 99 bits per transform
			{ 11, 2, 20 },		// 99 bits per transform
			{ 11, 3, 19 },		// 99 bits per transform
			{ 11, 4, 18 },		// 99 bits per transform
			{ 11, 5, 17 },		// 99 bits per transform
			{ 11, 6, 16 },		// 99 bits per transform
			{ 11, 7, 15 },		// 99 bits per transform
			{ 11, 8, 14 },		// 99 bits per transform
			{ 11, 9, 13 },		// 99 bits per transform
			{ 11, 10, 12 },		// 99 bits per transform
			{ 11, 11, 11 },		// 99 bits per transform
			{ 11, 12, 10 },		// 99 bits per transform
			{ 11, 13, 9 },		// 99 bits per transform
			{ 11, 14, 8 },		// 99 bits per transform
			{ 11, 15, 7 },		// 99 bits per transform
			{ 11, 16, 6 },		// 99 bits per transform
			{ 11, 17, 5 },		// 99 bits per transform
			{ 11, 18, 4 },		// 99 bits per transform
			{ 11, 19, 3 },		// 99 bits per transform
			{ 11, 20, 2 },		// 99 bits per transform
			{ 11, 21, 1 },		// 99 bits per transform
			{ 11, 22, 0 },		// 99 bits per transform
			{ 12, 0, 21 },		// 99 bits per transform
			{ 12, 1, 20 },		// 99 bits per transform
			{ 12, 2, 19 },		// 99 bits per transform
			{ 12, 3, 18 },		// 99 bits per transform
			{ 12, 4, 17 },		// 99 bits per transform
			{ 12, 5, 16 },		// 99 bits per transform
			{ 12, 6, 15 },		// 99 bits per transform
			{ 12, 7, 14 },		// 99 bits per transform
			{ 12, 8, 13 },		// 99 bits per transform
			{ 12, 9, 12 },		// 99 bits per transform
			{ 12, 10, 11 },		// 99 bits per transform
			{ 12, 11, 10 },		// 99 bits per transform
			{ 12, 12, 9 },		// 99 bits per transform
			{ 12, 13, 8 },		// 99 bits per transform
			{ 12, 14, 7 },		// 99 bits per transform
			{ 12, 15, 6 },		// 99 bits per transform
			{ 12, 16, 5 },		// 99 bits per transform
			{ 12, 17, 4 },		// 99 bits per transform
			{ 12, 18, 3 },		// 99 bits per transform
			{ 12, 19, 2 },		// 99 bits per transform
			{ 12, 20, 1 },		// 99 bits per transform
			{ 12, 21, 0 },		// 99 bits per transform
			{ 13, 0, 20 },		// 99 bits per transform
			{ 13, 1, 19 },		// 99 bits per transform
			{ 13, 2, 18 },		// 99 bits per transform
			{ 13, 3, 17 },		// 99 bits per transform
			{ 13, 4, 16 },		// 99 bits per transform
			{ 13, 5, 15 },		// 99 bits per transform
			{ 13, 6, 14 },		// 99 bits per transform
			{ 13, 7, 13 },		// 99 bits per transform
			{ 13, 8, 12 },		// 99 bits per transform
			{ 13, 9, 11 },		// 99 bits per transform
			{ 13, 10, 10 },		// 99 bits per transform
			{ 13, 11, 9 },		// 99 bits per transform
			{ 13, 12, 8 },		// 99 bits per transform
			{ 13, 13, 7 },		// 99 bits per transform
			{ 13, 14, 6 },		// 99 bits per transform
			{ 13, 15, 5 },		// 99 bits per transform
			{ 13, 16, 4 },		// 99 bits per transform
			{ 13, 17, 3 },		// 99 bits per transform
			{ 13, 18, 2 },		// 99 bits per transform
			{ 13, 19, 1 },		// 99 bits per transform
			{ 13, 20, 0 },		// 99 bits per transform
			{ 14, 0, 19 },		// 99 bits per transform
			{ 14, 1, 18 },		// 99 bits per transform
			{ 14, 2, 17 },		// 99 bits per transform
			{ 14, 3, 16 },		// 99 bits per transform
			{ 14, 4, 15 },		// 99 bits per transform
			{ 14, 5, 14 },		// 99 bits per transform
			{ 14, 6, 13 },		// 99 bits per transform
			{ 14, 7, 12 },		// 99 bits per transform
			{ 14, 8, 11 },		// 99 bits per transform
			{ 14, 9, 10 },		// 99 bits per transform
			{ 14, 10, 9 },		// 99 bits per transform
			{ 14, 11, 8 },		// 99 bits per transform
			{ 14, 12, 7 },		// 99 bits per transform
			{ 14, 13, 6 },		// 99 bits per transform
			{ 14, 14, 5 },		// 99 bits per transform
			{ 14, 15, 4 },		// 99 bits per transform
			{ 14, 16, 3 },		// 99 bits per transform
			{ 14, 17, 2 },		// 99 bits per transform
			{ 14, 18, 1 },		// 99 bits per transform
			{ 14, 19, 0 },		// 99 bits per transform
			{ 15, 0, 18 },		// 99 bits per transform
			{ 15, 1, 17 },		// 99 bits per transform
			{ 15, 2, 16 },		// 99 bits per transform
			{ 15, 3, 15 },		// 99 bits per transform
			{ 15, 4, 14 },		// 99 bits per transform
			{ 15, 5, 13 },		// 99 bits per transform
			{ 15, 6, 12 },		// 99 bits per transform
			{ 15, 7, 11 },		// 99 bits per transform
			{ 15, 8, 10 },		// 99 bits per transform
			{ 15, 9, 9 },		// 99 bits per transform
			{ 15, 10, 8 },		// 99 bits per transform
			{ 15, 11, 7 },		// 99 bits per transform
			{ 15, 12, 6 },		// 99 bits per transform
			{ 15, 13, 5 },		// 99 bits per transform
			{ 15, 14, 4 },		// 99 bits per transform
			{ 15, 15, 3 },		// 99 bits per transform
			{ 15, 16, 2 },		// 99 bits per transform
			{ 15, 17, 1 },		// 99 bits per transform
			{ 15, 18, 0 },		// 99 bits per transform
			{ 16, 0, 17 },		// 99 bits per transform
			{ 16, 1, 16 },		// 99 bits per transform
			{ 16, 2, 15 },		// 99 bits per transform
			{ 16, 3, 14 },		// 99 bits per transform
			{ 16, 4, 13 },		// 99 bits per transform
			{ 16, 5, 12 },		// 99 bits per transform
			{ 16, 6, 11 },		// 99 bits per transform
			{ 16, 7, 10 },		// 99 bits per transform
			{ 16, 8, 9 },		// 99 bits per transform
			{ 16, 9, 8 },		// 99 bits per transform
			{ 16, 10, 7 },		// 99 bits per transform
			{ 16, 11, 6 },		// 99 bits per transform
			{ 16, 12, 5 },		// 99 bits per transform
			{ 16, 13, 4 },		// 99 bits per transform
			{ 16, 14, 3 },		// 99 bits per transform
			{ 16, 15, 2 },		// 99 bits per transform
			{ 16, 16, 1 },		// 99 bits per transform
			{ 16, 17, 0 },		// 99 bits per transform
			{ 17, 0, 16 },		// 99 bits per transform
			{ 17, 1, 15 },		// 99 bits per transform
			{ 17, 2, 14 },		// 99 bits per transform
			{ 17, 3, 13 },		// 99 bits per transform
			{ 17, 4, 12 },		// 99 bits per transform
			{ 17, 5, 11 },		// 99 bits per transform
			{ 17, 6, 10 },		// 99 bits per transform
			{ 17, 7, 9 },		// 99 bits per transform
			{ 17, 8, 8 },		// 99 bits per transform
			{ 17, 9, 7 },		// 99 bits per transform
			{ 17, 10, 6 },		// 99 bits per transform
			{ 17, 11, 5 },		// 99 bits per transform
			{ 17, 12, 4 },		// 99 bits per transform
			{ 17, 13, 3 },		// 99 bits per transform
			{ 17, 14, 2 },		// 99 bits per transform
			{ 17, 15, 1 },		// 99 bits per transform
			{ 17, 16, 0 },		// 99 bits per transform
			{ 18, 0, 15 },		// 99 bits per transform
			{ 18, 1, 14 },		// 99 bits per transform
			{ 18, 2, 13 },		// 99 bits per transform
			{ 18, 3, 12 },		// 99 bits per transform
			{ 18, 4, 11 },		// 99 bits per transform
			{ 18, 5, 10 },		// 99 bits per transform
			{ 18, 6, 9 },		// 99 bits per transform
			{ 18, 7, 8 },		// 99 bits per transform
			{ 18, 8, 7 },		// 99 bits per transform
			{ 18, 9, 6 },		// 99 bits per transform
			{ 18, 10, 5 },		// 99 bits per transform
			{ 18, 11, 4 },		// 99 bits per transform
			{ 18, 12, 3 },		// 99 bits per transform
			{ 18, 13, 2 },		// 99 bits per transform
			{ 18, 14, 1 },		// 99 bits per transform
			{ 18, 15, 0 },		// 99 bits per transform
			{ 19, 0, 14 },		// 99 bits per transform
			{ 19, 1, 13 },		// 99 bits per transform
			{ 19, 2, 12 },		// 99 bits per transform
			{ 19, 3, 11 },		// 99 bits per transform
			{ 19, 4, 10 },		// 99 bits per transform
			{ 19, 5, 9 },		// 99 bits per transform
			{ 19, 6, 8 },		// 99 bits per transform
			{ 19, 7, 7 },		// 99 bits per transform
			{ 19, 8, 6 },		// 99 bits per transform
			{ 19, 9, 5 },		// 99 bits per transform
			{ 19, 10, 4 },		// 99 bits per transform
			{ 19, 11, 3 },		// 99 bits per transform
			{ 19, 12, 2 },		// 99 bits per transform
			{ 19, 13, 1 },		// 99 bits per transform
			{ 19, 14, 0 },		// 99 bits per transform
			{ 20, 0, 13 },		// 99 bits per transform
			{ 20, 1, 12 },		// 99 bits per transform
			{ 20, 2, 11 },		// 99 bits per transform
			{ 20, 3, 10 },		// 99 bits per transform
			{ 20, 4, 9 },		// 99 bits per transform
			{ 20, 5, 8 },		// 99 bits per transform
			{ 20, 6, 7 },		// 99 bits per transform
			{ 20, 7, 6 },		// 99 bits per transform
			{ 20, 8, 5 },		// 99 bits per transform
			{ 20, 9, 4 },		// 99 bits per transform
			{ 20, 10, 3 },		// 99 bits per transform
			{ 20, 11, 2 },		// 99 bits per transform
			{ 20, 12, 1 },		// 99 bits per transform
			{ 20, 13, 0 },		// 99 bits per transform
			{ 21, 0, 12 },		// 99 bits per transform
			{ 21, 1, 11 },		// 99 bits per transform
			{ 21, 2, 10 },		// 99 bits per transform
			{ 21, 3, 9 },		// 99 bits per transform
			{ 21, 4, 8 },		// 99 bits per transform
			{ 21, 5, 7 },		// 99 bits per transform
			{ 21, 6, 6 },		// 99 bits per transform
			{ 21, 7, 5 },		// 99 bits per transform
			{ 21, 8, 4 },		// 99 bits per transform
			{ 21, 9, 3 },		// 99 bits per transform
			{ 21, 10, 2 },		// 99 bits per transform
			{ 21, 11, 1 },		// 99 bits per transform
			{ 21, 12, 0 },		// 99 bits per transform
			{ 22, 0, 11 },		// 99 bits per transform
			{ 22, 1, 10 },		// 99 bits per transform
			{ 22, 2, 9 },		// 99 bits per transform
			{ 22, 3, 8 },		// 99 bits per transform
			{ 22, 4, 7 },		// 99 bits per transform
			{ 22, 5, 6 },		// 99 bits per transform
			{ 22, 6, 5 },		// 99 bits per transform
			{ 22, 7, 4 },		// 99 bits per transform
			{ 22, 8, 3 },		// 99 bits per transform
			{ 22, 9, 2 },		// 99 bits per transform
			{ 22, 10, 1 },		// 99 bits per transform
			{ 22, 11, 0 },		// 99 bits per transform
			{ 23, 0, 10 },		// 99 bits per transform
			{ 23, 1, 9 },		// 99 bits per transform
			{ 23, 2, 8 },		// 99 bits per transform
			{ 23, 3, 7 },		// 99 bits per transform
			{ 23, 4, 6 },		// 99 bits per transform
			{ 23, 5, 5 },		// 99 bits per transform
			{ 23, 6, 4 },		// 99 bits per transform
			{ 23, 7, 3 },		// 99 bits per transform
			{ 23, 8, 2 },		// 99 bits per transform
			{ 23, 9, 1 },		// 99 bits per transform
			{ 23, 10, 0 },		// 99 bits per transform
			{ 24, 0, 1 },		// 99 bits per transform
			{ 24, 1, 0 },		// 99 bits per transform
			{ 0, 2, 24 },		// 102 bits per transform
			{ 0, 11, 23 },		// 102 bits per transform
			{ 0, 12, 22 },		// 102 bits per transform
			{ 0, 13, 21 },		// 102 bits per transform
			{ 0, 14, 20 },		// 102 bits per transform
			{ 0, 15, 19 },		// 102 bits per transform
			{ 0, 16, 18 },		// 102 bits per transform
			{ 0, 17, 17 },		// 102 bits per transform
			{ 0, 18, 16 },		// 102 bits per transform
			{ 0, 19, 15 },		// 102 bits per transform
			{ 0, 20, 14 },		// 102 bits per transform
			{ 0, 21, 13 },		// 102 bits per transform
			{ 0, 22, 12 },		// 102 bits per transform
			{ 0, 23, 11 },		// 102 bits per transform
			{ 0, 24, 2 },		// 102 bits per transform
			{ 1, 1, 24 },		// 102 bits per transform
			{ 1, 10, 23 },		// 102 bits per transform
			{ 1, 11, 22 },		// 102 bits per transform
			{ 1, 12, 21 },		// 102 bits per transform
			{ 1, 13, 20 },		// 102 bits per transform
			{ 1, 14, 19 },		// 102 bits per transform
			{ 1, 15, 18 },		// 102 bits per transform
			{ 1, 16, 17 },		// 102 bits per transform
			{ 1, 17, 16 },		// 102 bits per transform
			{ 1, 18, 15 },		// 102 bits per transform
			{ 1, 19, 14 },		// 102 bits per transform
			{ 1, 20, 13 },		// 102 bits per transform
			{ 1, 21, 12 },		// 102 bits per transform
			{ 1, 22, 11 },		// 102 bits per transform
			{ 1, 23, 10 },		// 102 bits per transform
			{ 1, 24, 1 },		// 102 bits per transform
			{ 2, 0, 24 },		// 102 bits per transform
			{ 2, 9, 23 },		// 102 bits per transform
			{ 2, 10, 22 },		// 102 bits per transform
			{ 2, 11, 21 },		// 102 bits per transform
			{ 2, 12, 20 },		// 102 bits per transform
			{ 2, 13, 19 },		// 102 bits per transform
			{ 2, 14, 18 },		// 102 bits per transform
			{ 2, 15, 17 },		// 102 bits per transform
			{ 2, 16, 16 },		// 102 bits per transform
			{ 2, 17, 15 },		// 102 bits per transform
			{ 2, 18, 14 },		// 102 bits per transform
			{ 2, 19, 13 },		// 102 bits per transform
			{ 2, 20, 12 },		// 102 bits per transform
			{ 2, 21, 11 },		// 102 bits per transform
			{ 2, 22, 10 },		// 102 bits per transform
			{ 2, 23, 9 },		// 102 bits per transform
			{ 2, 24, 0 },		// 102 bits per transform
			{ 3, 8, 23 },		// 102 bits per transform
			{ 3, 9, 22 },		// 102 bits per transform
			{ 3, 10, 21 },		// 102 bits per transform
			{ 3, 11, 20 },		// 102 bits per transform
			{ 3, 12, 19 },		// 102 bits per transform
			{ 3, 13, 18 },		// 102 bits per transform
			{ 3, 14, 17 },		// 102 bits per transform
			{ 3, 15, 16 },		// 102 bits per transform
			{ 3, 16, 15 },		// 102 bits per transform
			{ 3, 17, 14 },		// 102 bits per transform
			{ 3, 18, 13 },		// 102 bits per transform
			{ 3, 19, 12 },		// 102 bits per transform
			{ 3, 20, 11 },		// 102 bits per transform
			{ 3, 21, 10 },		// 102 bits per transform
			{ 3, 22, 9 },		// 102 bits per transform
			{ 3, 23, 8 },		// 102 bits per transform
			{ 4, 7, 23 },		// 102 bits per transform
			{ 4, 8, 22 },		// 102 bits per transform
			{ 4, 9, 21 },		// 102 bits per transform
			{ 4, 10, 20 },		// 102 bits per transform
			{ 4, 11, 19 },		// 102 bits per transform
			{ 4, 12, 18 },		// 102 bits per transform
			{ 4, 13, 17 },		// 102 bits per transform
			{ 4, 14, 16 },		// 102 bits per transform
			{ 4, 15, 15 },		// 102 bits per transform
			{ 4, 16, 14 },		// 102 bits per transform
			{ 4, 17, 13 },		// 102 bits per transform
			{ 4, 18, 12 },		// 102 bits per transform
			{ 4, 19, 11 },		// 102 bits per transform
			{ 4, 20, 10 },		// 102 bits per transform
			{ 4, 21, 9 },		// 102 bits per transform
			{ 4, 22, 8 },		// 102 bits per transform
			{ 4, 23, 7 },		// 102 bits per transform
			{ 5, 6, 23 },		// 102 bits per transform
			{ 5, 7, 22 },		// 102 bits per transform
			{ 5, 8, 21 },		// 102 bits per transform
			{ 5, 9, 20 },		// 102 bits per transform
			{ 5, 10, 19 },		// 102 bits per transform
			{ 5, 11, 18 },		// 102 bits per transform
			{ 5, 12, 17 },		// 102 bits per transform
			{ 5, 13, 16 },		// 102 bits per transform
			{ 5, 14, 15 },		// 102 bits per transform
			{ 5, 15, 14 },		// 102 bits per transform
			{ 5, 16, 13 },		// 102 bits per transform
			{ 5, 17, 12 },		// 102 bits per transform
			{ 5, 18, 11 },		// 102 bits per transform
			{ 5, 19, 10 },		// 102 bits per transform
			{ 5, 20, 9 },		// 102 bits per transform
			{ 5, 21, 8 },		// 102 bits per transform
			{ 5, 22, 7 },		// 102 bits per transform
			{ 5, 23, 6 },		// 102 bits per transform
			{ 6, 5, 23 },		// 102 bits per transform
			{ 6, 6, 22 },		// 102 bits per transform
			{ 6, 7, 21 },		// 102 bits per transform
			{ 6, 8, 20 },		// 102 bits per transform
			{ 6, 9, 19 },		// 102 bits per transform
			{ 6, 10, 18 },		// 102 bits per transform
			{ 6, 11, 17 },		// 102 bits per transform
			{ 6, 12, 16 },		// 102 bits per transform
			{ 6, 13, 15 },		// 102 bits per transform
			{ 6, 14, 14 },		// 102 bits per transform
			{ 6, 15, 13 },		// 102 bits per transform
			{ 6, 16, 12 },		// 102 bits per transform
			{ 6, 17, 11 },		// 102 bits per transform
			{ 6, 18, 10 },		// 102 bits per transform
			{ 6, 19, 9 },		// 102 bits per transform
			{ 6, 20, 8 },		// 102 bits per transform
			{ 6, 21, 7 },		// 102 bits per transform
			{ 6, 22, 6 },		// 102 bits per transform
			{ 6, 23, 5 },		// 102 bits per transform
			{ 7, 4, 23 },		// 102 bits per transform
			{ 7, 5, 22 },		// 102 bits per transform
			{ 7, 6, 21 },		// 102 bits per transform
			{ 7, 7, 20 },		// 102 bits per transform
			{ 7, 8, 19 },		// 102 bits per transform
			{ 7, 9, 18 },		// 102 bits per transform
			{ 7, 10, 17 },		// 102 bits per transform
			{ 7, 11, 16 },		// 102 bits per transform
			{ 7, 12, 15 },		// 102 bits per transform
			{ 7, 13, 14 },		// 102 bits per transform
			{ 7, 14, 13 },		// 102 bits per transform
			{ 7, 15, 12 },		// 102 bits per transform
			{ 7, 16, 11 },		// 102 bits per transform
			{ 7, 17, 10 },		// 102 bits per transform
			{ 7, 18, 9 },		// 102 bits per transform
			{ 7, 19, 8 },		// 102 bits per transform
			{ 7, 20, 7 },		// 102 bits per transform
			{ 7, 21, 6 },		// 102 bits per transform
			{ 7, 22, 5 },		// 102 bits per transform
			{ 7, 23, 4 },		// 102 bits per transform
			{ 8, 3, 23 },		// 102 bits per transform
			{ 8, 4, 22 },		// 102 bits per transform
			{ 8, 5, 21 },		// 102 bits per transform
			{ 8, 6, 20 },		// 102 bits per transform
			{ 8, 7, 19 },		// 102 bits per transform
			{ 8, 8, 18 },		// 102 bits per transform
			{ 8, 9, 17 },		// 102 bits per transform
			{ 8, 10, 16 },		// 102 bits per transform
			{ 8, 11, 15 },		// 102 bits per transform
			{ 8, 12, 14 },		// 102 bits per transform
			{ 8, 13, 13 },		// 102 bits per transform
			{ 8, 14, 12 },		// 102 bits per transform
			{ 8, 15, 11 },		// 102 bits per transform
			{ 8, 16, 10 },		// 102 bits per transform
			{ 8, 17, 9 },		// 102 bits per transform
			{ 8, 18, 8 },		// 102 bits per transform
			{ 8, 19, 7 },		// 102 bits per transform
			{ 8, 20, 6 },		// 102 bits per transform
			{ 8, 21, 5 },		// 102 bits per transform
			{ 8, 22, 4 },		// 102 bits per transform
			{ 8, 23, 3 },		// 102 bits per transform
			{ 9, 2, 23 },		// 102 bits per transform
			{ 9, 3, 22 },		// 102 bits per transform
			{ 9, 4, 21 },		// 102 bits per transform
			{ 9, 5, 20 },		// 102 bits per transform
			{ 9, 6, 19 },		// 102 bits per transform
			{ 9, 7, 18 },		// 102 bits per transform
			{ 9, 8, 17 },		// 102 bits per transform
			{ 9, 9, 16 },		// 102 bits per transform
			{ 9, 10, 15 },		// 102 bits per transform
			{ 9, 11, 14 },		// 102 bits per transform
			{ 9, 12, 13 },		// 102 bits per transform
			{ 9, 13, 12 },		// 102 bits per transform
			{ 9, 14, 11 },		// 102 bits per transform
			{ 9, 15, 10 },		// 102 bits per transform
			{ 9, 16, 9 },		// 102 bits per transform
			{ 9, 17, 8 },		// 102 bits per transform
			{ 9, 18, 7 },		// 102 bits per transform
			{ 9, 19, 6 },		// 102 bits per transform
			{ 9, 20, 5 },		// 102 bits per transform
			{ 9, 21, 4 },		// 102 bits per transform
			{ 9, 22, 3 },		// 102 bits per transform
			{ 9, 23, 2 },		// 102 bits per transform
			{ 10, 1, 23 },		// 102 bits per transform
			{ 10, 2, 22 },		// 102 bits per transform
			{ 10, 3, 21 },		// 102 bits per transform
			{ 10, 4, 20 },		// 102 bits per transform
			{ 10, 5, 19 },		// 102 bits per transform
			{ 10, 6, 18 },		// 102 bits per transform
			{ 10, 7, 17 },		// 102 bits per transform
			{ 10, 8, 16 },		// 102 bits per transform
			{ 10, 9, 15 },		// 102 bits per transform
			{ 10, 10, 14 },		// 102 bits per transform
			{ 10, 11, 13 },		// 102 bits per transform
			{ 10, 12, 12 },		// 102 bits per transform
			{ 10, 13, 11 },		// 102 bits per transform
			{ 10, 14, 10 },		// 102 bits per transform
			{ 10, 15, 9 },		// 102 bits per transform
			{ 10, 16, 8 },		// 102 bits per transform
			{ 10, 17, 7 },		// 102 bits per transform
			{ 10, 18, 6 },		// 102 bits per transform
			{ 10, 19, 5 },		// 102 bits per transform
			{ 10, 20, 4 },		// 102 bits per transform
			{ 10, 21, 3 },		// 102 bits per transform
			{ 10, 22, 2 },		// 102 bits per transform
			{ 10, 23, 1 },		// 102 bits per transform
			{ 11, 0, 23 },		// 102 bits per transform
			{ 11, 1, 22 },		// 102 bits per transform
			{ 11, 2, 21 },		// 102 bits per transform
			{ 11, 3, 20 },		// 102 bits per transform
			{ 11, 4, 19 },		// 102 bits per transform
			{ 11, 5, 18 },		// 102 bits per transform
			{ 11, 6, 17 },		// 102 bits per transform
			{ 11, 7, 16 },		// 102 bits per transform
			{ 11, 8, 15 },		// 102 bits per transform
			{ 11, 9, 14 },		// 102 bits per transform
			{ 11, 10, 13 },		// 102 bits per transform
			{ 11, 11, 12 },		// 102 bits per transform
			{ 11, 12, 11 },		// 102 bits per transform
			{ 11, 13, 10 },		// 102 bits per transform
			{ 11, 14, 9 },		// 102 bits per transform
			{ 11, 15, 8 },		// 102 bits per transform
			{ 11, 16, 7 },		// 102 bits per transform
			{ 11, 17, 6 },		// 102 bits per transform
			{ 11, 18, 5 },		// 102 bits per transform
			{ 11, 19, 4 },		// 102 bits per transform
			{ 11, 20, 3 },		// 102 bits per transform
			{ 11, 21, 2 },		// 102 bits per transform
			{ 11, 22, 1 },		// 102 bits per transform
			{ 11, 23, 0 },		// 102 bits per transform
			{ 12, 0, 22 },		// 102 bits per transform
			{ 12, 1, 21 },		// 102 bits per transform
			{ 12, 2, 20 },		// 102 bits per transform
			{ 12, 3, 19 },		// 102 bits per transform
			{ 12, 4, 18 },		// 102 bits per transform
			{ 12, 5, 17 },		// 102 bits per transform
			{ 12, 6, 16 },		// 102 bits per transform
			{ 12, 7, 15 },		// 102 bits per transform
			{ 12, 8, 14 },		// 102 bits per transform
			{ 12, 9, 13 },		// 102 bits per transform
			{ 12, 10, 12 },		// 102 bits per transform
			{ 12, 11, 11 },		// 102 bits per transform
			{ 12, 12, 10 },		// 102 bits per transform
			{ 12, 13, 9 },		// 102 bits per transform
			{ 12, 14, 8 },		// 102 bits per transform
			{ 12, 15, 7 },		// 102 bits per transform
			{ 12, 16, 6 },		// 102 bits per transform
			{ 12, 17, 5 },		// 102 bits per transform
			{ 12, 18, 4 },		// 102 bits per transform
			{ 12, 19, 3 },		// 102 bits per transform
			{ 12, 20, 2 },		// 102 bits per transform
			{ 12, 21, 1 },		// 102 bits per transform
			{ 12, 22, 0 },		// 102 bits per transform
			{ 13, 0, 21 },		// 102 bits per transform
			{ 13, 1, 20 },		// 102 bits per transform
			{ 13, 2, 19 },		// 102 bits per transform
			{ 13, 3, 18 },		// 102 bits per transform
			{ 13, 4, 17 },		// 102 bits per transform
			{ 13, 5, 16 },		// 102 bits per transform
			{ 13, 6, 15 },		// 102 bits per transform
			{ 13, 7, 14 },		// 102 bits per transform
			{ 13, 8, 13 },		// 102 bits per transform
			{ 13, 9, 12 },		// 102 bits per transform
			{ 13, 10, 11 },		// 102 bits per transform
			{ 13, 11, 10 },		// 102 bits per transform
			{ 13, 12, 9 },		// 102 bits per transform
			{ 13, 13, 8 },		// 102 bits per transform
			{ 13, 14, 7 },		// 102 bits per transform
			{ 13, 15, 6 },		// 102 bits per transform
			{ 13, 16, 5 },		// 102 bits per transform
			{ 13, 17, 4 },		// 102 bits per transform
			{ 13, 18, 3 },		// 102 bits per transform
			{ 13, 19, 2 },		// 102 bits per transform
			{ 13, 20, 1 },		// 102 bits per transform
			{ 13, 21, 0 },		// 102 bits per transform
			{ 14, 0, 20 },		// 102 bits per transform
			{ 14, 1, 19 },		// 102 bits per transform
			{ 14, 2, 18 },		// 102 bits per transform
			{ 14, 3, 17 },		// 102 bits per transform
			{ 14, 4, 16 },		// 102 bits per transform
			{ 14, 5, 15 },		// 102 bits per transform
			{ 14, 6, 14 },		// 102 bits per transform
			{ 14, 7, 13 },		// 102 bits per transform
			{ 14, 8, 12 },		// 102 bits per transform
			{ 14, 9, 11 },		// 102 bits per transform
			{ 14, 10, 10 },		// 102 bits per transform
			{ 14, 11, 9 },		// 102 bits per transform
			{ 14, 12, 8 },		// 102 bits per transform
			{ 14, 13, 7 },		// 102 bits per transform
			{ 14, 14, 6 },		// 102 bits per transform
			{ 14, 15, 5 },		// 102 bits per transform
			{ 14, 16, 4 },		// 102 bits per transform
			{ 14, 17, 3 },		// 102 bits per transform
			{ 14, 18, 2 },		// 102 bits per transform
			{ 14, 19, 1 },		// 102 bits per transform
			{ 14, 20, 0 },		// 102 bits per transform
			{ 15, 0, 19 },		// 102 bits per transform
			{ 15, 1, 18 },		// 102 bits per transform
			{ 15, 2, 17 },		// 102 bits per transform
			{ 15, 3, 16 },		// 102 bits per transform
			{ 15, 4, 15 },		// 102 bits per transform
			{ 15, 5, 14 },		// 102 bits per transform
			{ 15, 6, 13 },		// 102 bits per transform
			{ 15, 7, 12 },		// 102 bits per transform
			{ 15, 8, 11 },		// 102 bits per transform
			{ 15, 9, 10 },		// 102 bits per transform
			{ 15, 10, 9 },		// 102 bits per transform
			{ 15, 11, 8 },		// 102 bits per transform
			{ 15, 12, 7 },		// 102 bits per transform
			{ 15, 13, 6 },		// 102 bits per transform
			{ 15, 14, 5 },		// 102 bits per transform
			{ 15, 15, 4 },		// 102 bits per transform
			{ 15, 16, 3 },		// 102 bits per transform
			{ 15, 17, 2 },		// 102 bits per transform
			{ 15, 18, 1 },		// 102 bits per transform
			{ 15, 19, 0 },		// 102 bits per transform
			{ 16, 0, 18 },		// 102 bits per transform
			{ 16, 1, 17 },		// 102 bits per transform
			{ 16, 2, 16 },		// 102 bits per transform
			{ 16, 3, 15 },		// 102 bits per transform
			{ 16, 4, 14 },		// 102 bits per transform
			{ 16, 5, 13 },		// 102 bits per transform
			{ 16, 6, 12 },		// 102 bits per transform
			{ 16, 7, 11 },		// 102 bits per transform
			{ 16, 8, 10 },		// 102 bits per transform
			{ 16, 9, 9 },		// 102 bits per transform
			{ 16, 10, 8 },		// 102 bits per transform
			{ 16, 11, 7 },		// 102 bits per transform
			{ 16, 12, 6 },		// 102 bits per transform
			{ 16, 13, 5 },		// 102 bits per transform
			{ 16, 14, 4 },		// 102 bits per transform
			{ 16, 15, 3 },		// 102 bits per transform
			{ 16, 16, 2 },		// 102 bits per transform
			{ 16, 17, 1 },		// 102 bits per transform
			{ 16, 18, 0 },		// 102 bits per transform
			{ 17, 0, 17 },		// 102 bits per transform
			{ 17, 1, 16 },		// 102 bits per transform
			{ 17, 2, 15 },		// 102 bits per transform
			{ 17, 3, 14 },		// 102 bits per transform
			{ 17, 4, 13 },		// 102 bits per transform
			{ 17, 5, 12 },		// 102 bits per transform
			{ 17, 6, 11 },		// 102 bits per transform
			{ 17, 7, 10 },		// 102 bits per transform
			{ 17, 8, 9 },		// 102 bits per transform
			{ 17, 9, 8 },		// 102 bits per transform
			{ 17, 10, 7 },		// 102 bits per transform
			{ 17, 11, 6 },		// 102 bits per transform
			{ 17, 12, 5 },		// 102 bits per transform
			{ 17, 13, 4 },		// 102 bits per transform
			{ 17, 14, 3 },		// 102 bits per transform
			{ 17, 15, 2 },		// 102 bits per transform
			{ 17, 16, 1 },		// 102 bits per transform
			{ 17, 17, 0 },		// 102 bits per transform
			{ 18, 0, 16 },		// 102 bits per transform
			{ 18, 1, 15 },		// 102 bits per transform
			{ 18, 2, 14 },		// 102 bits per transform
			{ 18, 3, 13 },		// 102 bits per transform
			{ 18, 4, 12 },		// 102 bits per transform
			{ 18, 5, 11 },		// 102 bits per transform
			{ 18, 6, 10 },		// 102 bits per transform
			{ 18, 7, 9 },		// 102 bits per transform
			{ 18, 8, 8 },		// 102 bits per transform
			{ 18, 9, 7 },		// 102 bits per transform
			{ 18, 10, 6 },		// 102 bits per transform
			{ 18, 11, 5 },		// 102 bits per transform
			{ 18, 12, 4 },		// 102 bits per transform
			{ 18, 13, 3 },		// 102 bits per transform
			{ 18, 14, 2 },		// 102 bits per transform
			{ 18, 15, 1 },		// 102 bits per transform
			{ 18, 16, 0 },		// 102 bits per transform
			{ 19, 0, 15 },		// 102 bits per transform
			{ 19, 1, 14 },		// 102 bits per transform
			{ 19, 2, 13 },		// 102 bits per transform
			{ 19, 3, 12 },		// 102 bits per transform
			{ 19, 4, 11 },		// 102 bits per transform
			{ 19, 5, 10 },		// 102 bits per transform
			{ 19, 6, 9 },		// 102 bits per transform
			{ 19, 7, 8 },		// 102 bits per transform
			{ 19, 8, 7 },		// 102 bits per transform
			{ 19, 9, 6 },		// 102 bits per transform
			{ 19, 10, 5 },		// 102 bits per transform
			{ 19, 11, 4 },		// 102 bits per transform
			{ 19, 12, 3 },		// 102 bits per transform
			{ 19, 13, 2 },		// 102 bits per transform
			{ 19, 14, 1 },		// 102 bits per transform
			{ 19, 15, 0 },		// 102 bits per transform
			{ 20, 0, 14 },		// 102 bits per transform
			{ 20, 1, 13 },		// 102 bits per transform
			{ 20, 2, 12 },		// 102 bits per transform
			{ 20, 3, 11 },		// 102 bits per transform
			{ 20, 4, 10 },		// 102 bits per transform
			{ 20, 5, 9 },		// 102 bits per transform
			{ 20, 6, 8 },		// 102 bits per transform
			{ 20, 7, 7 },		// 102 bits per transform
			{ 20, 8, 6 },		// 102 bits per transform
			{ 20, 9, 5 },		// 102 bits per transform
			{ 20, 10, 4 },		// 102 bits per transform
			{ 20, 11, 3 },		// 102 bits per transform
			{ 20, 12, 2 },		// 102 bits per transform
			{ 20, 13, 1 },		// 102 bits per transform
			{ 20, 14, 0 },		// 102 bits per transform
			{ 21, 0, 13 },		// 102 bits per transform
			{ 21, 1, 12 },		// 102 bits per transform
			{ 21, 2, 11 },		// 102 bits per transform
			{ 21, 3, 10 },		// 102 bits per transform
			{ 21, 4, 9 },		// 102 bits per transform
			{ 21, 5, 8 },		// 102 bits per transform
			{ 21, 6, 7 },		// 102 bits per transform
			{ 21, 7, 6 },		// 102 bits per transform
			{ 21, 8, 5 },		// 102 bits per transform
			{ 21, 9, 4 },		// 102 bits per transform
			{ 21, 10, 3 },		// 102 bits per transform
			{ 21, 11, 2 },		// 102 bits per transform
			{ 21, 12, 1 },		// 102 bits per transform
			{ 21, 13, 0 },		// 102 bits per transform
			{ 22, 0, 12 },		// 102 bits per transform
			{ 22, 1, 11 },		// 102 bits per transform
			{ 22, 2, 10 },		// 102 bits per transform
			{ 22, 3, 9 },		// 102 bits per transform
			{ 22, 4, 8 },		// 102 bits per transform
			{ 22, 5, 7 },		// 102 bits per transform
			{ 22, 6, 6 },		// 102 bits per transform
			{ 22, 7, 5 },		// 102 bits per transform
			{ 22, 8, 4 },		// 102 bits per transform
			{ 22, 9, 3 },		// 102 bits per transform
			{ 22, 10, 2 },		// 102 bits per transform
			{ 22, 11, 1 },		// 102 bits per transform
			{ 22, 12, 0 },		// 102 bits per transform
			{ 23, 0, 11 },		// 102 bits per transform
			{ 23, 1, 10 },		// 102 bits per transform
			{ 23, 2, 9 },		// 102 bits per transform
			{ 23, 3, 8 },		// 102 bits per transform
			{ 23, 4, 7 },		// 102 bits per transform
			{ 23, 5, 6 },		// 102 bits per transform
			{ 23, 6, 5 },		// 102 bits per transform
			{ 23, 7, 4 },		// 102 bits per transform
			{ 23, 8, 3 },		// 102 bits per transform
			{ 23, 9, 2 },		// 102 bits per transform
			{ 23, 10, 1 },		// 102 bits per transform
			{ 23, 11, 0 },		// 102 bits per transform
			{ 24, 0, 2 },		// 102 bits per transform
			{ 24, 1, 1 },		// 102 bits per transform
			{ 24, 2, 0 },		// 102 bits per transform
			{ 0, 3, 24 },		// 105 bits per transform
			{ 0, 12, 23 },		// 105 bits per transform
			{ 0, 13, 22 },		// 105 bits per transform
			{ 0, 14, 21 },		// 105 bits per transform
			{ 0, 15, 20 },		// 105 bits per transform
			{ 0, 16, 19 },		// 105 bits per transform
			{ 0, 17, 18 },		// 105 bits per transform
			{ 0, 18, 17 },		// 105 bits per transform
			{ 0, 19, 16 },		// 105 bits per transform
			{ 0, 20, 15 },		// 105 bits per transform
			{ 0, 21, 14 },		// 105 bits per transform
			{ 0, 22, 13 },		// 105 bits per transform
			{ 0, 23, 12 },		// 105 bits per transform
			{ 0, 24, 3 },		// 105 bits per transform
			{ 1, 2, 24 },		// 105 bits per transform
			{ 1, 11, 23 },		// 105 bits per transform
			{ 1, 12, 22 },		// 105 bits per transform
			{ 1, 13, 21 },		// 105 bits per transform
			{ 1, 14, 20 },		// 105 bits per transform
			{ 1, 15, 19 },		// 105 bits per transform
			{ 1, 16, 18 },		// 105 bits per transform
			{ 1, 17, 17 },		// 105 bits per transform
			{ 1, 18, 16 },		// 105 bits per transform
			{ 1, 19, 15 },		// 105 bits per transform
			{ 1, 20, 14 },		// 105 bits per transform
			{ 1, 21, 13 },		// 105 bits per transform
			{ 1, 22, 12 },		// 105 bits per transform
			{ 1, 23, 11 },		// 105 bits per transform
			{ 1, 24, 2 },		// 105 bits per transform
			{ 2, 1, 24 },		// 105 bits per transform
			{ 2, 10, 23 },		// 105 bits per transform
			{ 2, 11, 22 },		// 105 bits per transform
			{ 2, 12, 21 },		// 105 bits per transform
			{ 2, 13, 20 },		// 105 bits per transform
			{ 2, 14, 19 },		// 105 bits per transform
			{ 2, 15, 18 },		// 105 bits per transform
			{ 2, 16, 17 },		// 105 bits per transform
			{ 2, 17, 16 },		// 105 bits per transform
			{ 2, 18, 15 },		// 105 bits per transform
			{ 2, 19, 14 },		// 105 bits per transform
			{ 2, 20, 13 },		// 105 bits per transform
			{ 2, 21, 12 },		// 105 bits per transform
			{ 2, 22, 11 },		// 105 bits per transform
			{ 2, 23, 10 },		// 105 bits per transform
			{ 2, 24, 1 },		// 105 bits per transform
			{ 3, 0, 24 },		// 105 bits per transform
			{ 3, 9, 23 },		// 105 bits per transform
			{ 3, 10, 22 },		// 105 bits per transform
			{ 3, 11, 21 },		// 105 bits per transform
			{ 3, 12, 20 },		// 105 bits per transform
			{ 3, 13, 19 },		// 105 bits per transform
			{ 3, 14, 18 },		// 105 bits per transform
			{ 3, 15, 17 },		// 105 bits per transform
			{ 3, 16, 16 },		// 105 bits per transform
			{ 3, 17, 15 },		// 105 bits per transform
			{ 3, 18, 14 },		// 105 bits per transform
			{ 3, 19, 13 },		// 105 bits per transform
			{ 3, 20, 12 },		// 105 bits per transform
			{ 3, 21, 11 },		// 105 bits per transform
			{ 3, 22, 10 },		// 105 bits per transform
			{ 3, 23, 9 },		// 105 bits per transform
			{ 3, 24, 0 },		// 105 bits per transform
			{ 4, 8, 23 },		// 105 bits per transform
			{ 4, 9, 22 },		// 105 bits per transform
			{ 4, 10, 21 },		// 105 bits per transform
			{ 4, 11, 20 },		// 105 bits per transform
			{ 4, 12, 19 },		// 105 bits per transform
			{ 4, 13, 18 },		// 105 bits per transform
			{ 4, 14, 17 },		// 105 bits per transform
			{ 4, 15, 16 },		// 105 bits per transform
			{ 4, 16, 15 },		// 105 bits per transform
			{ 4, 17, 14 },		// 105 bits per transform
			{ 4, 18, 13 },		// 105 bits per transform
			{ 4, 19, 12 },		// 105 bits per transform
			{ 4, 20, 11 },		// 105 bits per transform
			{ 4, 21, 10 },		// 105 bits per transform
			{ 4, 22, 9 },		// 105 bits per transform
			{ 4, 23, 8 },		// 105 bits per transform
			{ 5, 7, 23 },		// 105 bits per transform
			{ 5, 8, 22 },		// 105 bits per transform
			{ 5, 9, 21 },		// 105 bits per transform
			{ 5, 10, 20 },		// 105 bits per transform
			{ 5, 11, 19 },		// 105 bits per transform
			{ 5, 12, 18 },		// 105 bits per transform
			{ 5, 13, 17 },		// 105 bits per transform
			{ 5, 14, 16 },		// 105 bits per transform
			{ 5, 15, 15 },		// 105 bits per transform
			{ 5, 16, 14 },		// 105 bits per transform
			{ 5, 17, 13 },		// 105 bits per transform
			{ 5, 18, 12 },		// 105 bits per transform
			{ 5, 19, 11 },		// 105 bits per transform
			{ 5, 20, 10 },		// 105 bits per transform
			{ 5, 21, 9 },		// 105 bits per transform
			{ 5, 22, 8 },		// 105 bits per transform
			{ 5, 23, 7 },		// 105 bits per transform
			{ 6, 6, 23 },		// 105 bits per transform
			{ 6, 7, 22 },		// 105 bits per transform
			{ 6, 8, 21 },		// 105 bits per transform
			{ 6, 9, 20 },		// 105 bits per transform
			{ 6, 10, 19 },		// 105 bits per transform
			{ 6, 11, 18 },		// 105 bits per transform
			{ 6, 12, 17 },		// 105 bits per transform
			{ 6, 13, 16 },		// 105 bits per transform
			{ 6, 14, 15 },		// 105 bits per transform
			{ 6, 15, 14 },		// 105 bits per transform
			{ 6, 16, 13 },		// 105 bits per transform
			{ 6, 17, 12 },		// 105 bits per transform
			{ 6, 18, 11 },		// 105 bits per transform
			{ 6, 19, 10 },		// 105 bits per transform
			{ 6, 20, 9 },		// 105 bits per transform
			{ 6, 21, 8 },		// 105 bits per transform
			{ 6, 22, 7 },		// 105 bits per transform
			{ 6, 23, 6 },		// 105 bits per transform
			{ 7, 5, 23 },		// 105 bits per transform
			{ 7, 6, 22 },		// 105 bits per transform
			{ 7, 7, 21 },		// 105 bits per transform
			{ 7, 8, 20 },		// 105 bits per transform
			{ 7, 9, 19 },		// 105 bits per transform
			{ 7, 10, 18 },		// 105 bits per transform
			{ 7, 11, 17 },		// 105 bits per transform
			{ 7, 12, 16 },		// 105 bits per transform
			{ 7, 13, 15 },		// 105 bits per transform
			{ 7, 14, 14 },		// 105 bits per transform
			{ 7, 15, 13 },		// 105 bits per transform
			{ 7, 16, 12 },		// 105 bits per transform
			{ 7, 17, 11 },		// 105 bits per transform
			{ 7, 18, 10 },		// 105 bits per transform
			{ 7, 19, 9 },		// 105 bits per transform
			{ 7, 20, 8 },		// 105 bits per transform
			{ 7, 21, 7 },		// 105 bits per transform
			{ 7, 22, 6 },		// 105 bits per transform
			{ 7, 23, 5 },		// 105 bits per transform
			{ 8, 4, 23 },		// 105 bits per transform
			{ 8, 5, 22 },		// 105 bits per transform
			{ 8, 6, 21 },		// 105 bits per transform
			{ 8, 7, 20 },		// 105 bits per transform
			{ 8, 8, 19 },		// 105 bits per transform
			{ 8, 9, 18 },		// 105 bits per transform
			{ 8, 10, 17 },		// 105 bits per transform
			{ 8, 11, 16 },		// 105 bits per transform
			{ 8, 12, 15 },		// 105 bits per transform
			{ 8, 13, 14 },		// 105 bits per transform
			{ 8, 14, 13 },		// 105 bits per transform
			{ 8, 15, 12 },		// 105 bits per transform
			{ 8, 16, 11 },		// 105 bits per transform
			{ 8, 17, 10 },		// 105 bits per transform
			{ 8, 18, 9 },		// 105 bits per transform
			{ 8, 19, 8 },		// 105 bits per transform
			{ 8, 20, 7 },		// 105 bits per transform
			{ 8, 21, 6 },		// 105 bits per transform
			{ 8, 22, 5 },		// 105 bits per transform
			{ 8, 23, 4 },		// 105 bits per transform
			{ 9, 3, 23 },		// 105 bits per transform
			{ 9, 4, 22 },		// 105 bits per transform
			{ 9, 5, 21 },		// 105 bits per transform
			{ 9, 6, 20 },		// 105 bits per transform
			{ 9, 7, 19 },		// 105 bits per transform
			{ 9, 8, 18 },		// 105 bits per transform
			{ 9, 9, 17 },		// 105 bits per transform
			{ 9, 10, 16 },		// 105 bits per transform
			{ 9, 11, 15 },		// 105 bits per transform
			{ 9, 12, 14 },		// 105 bits per transform
			{ 9, 13, 13 },		// 105 bits per transform
			{ 9, 14, 12 },		// 105 bits per transform
			{ 9, 15, 11 },		// 105 bits per transform
			{ 9, 16, 10 },		// 105 bits per transform
			{ 9, 17, 9 },		// 105 bits per transform
			{ 9, 18, 8 },		// 105 bits per transform
			{ 9, 19, 7 },		// 105 bits per transform
			{ 9, 20, 6 },		// 105 bits per transform
			{ 9, 21, 5 },		// 105 bits per transform
			{ 9, 22, 4 },		// 105 bits per transform
			{ 9, 23, 3 },		// 105 bits per transform
			{ 10, 2, 23 },		// 105 bits per transform
			{ 10, 3, 22 },		// 105 bits per transform
			{ 10, 4, 21 },		// 105 bits per transform
			{ 10, 5, 20 },		// 105 bits per transform
			{ 10, 6, 19 },		// 105 bits per transform
			{ 10, 7, 18 },		// 105 bits per transform
			{ 10, 8, 17 },		// 105 bits per transform
			{ 10, 9, 16 },		// 105 bits per transform
			{ 10, 10, 15 },		// 105 bits per transform
			{ 10, 11, 14 },		// 105 bits per transform
			{ 10, 12, 13 },		// 105 bits per transform
			{ 10, 13, 12 },		// 105 bits per transform
			{ 10, 14, 11 },		// 105 bits per transform
			{ 10, 15, 10 },		// 105 bits per transform
			{ 10, 16, 9 },		// 105 bits per transform
			{ 10, 17, 8 },		// 105 bits per transform
			{ 10, 18, 7 },		// 105 bits per transform
			{ 10, 19, 6 },		// 105 bits per transform
			{ 10, 20, 5 },		// 105 bits per transform
			{ 10, 21, 4 },		// 105 bits per transform
			{ 10, 22, 3 },		// 105 bits per transform
			{ 10, 23, 2 },		// 105 bits per transform
			{ 11, 1, 23 },		// 105 bits per transform
			{ 11, 2, 22 },		// 105 bits per transform
			{ 11, 3, 21 },		// 105 bits per transform
			{ 11, 4, 20 },		// 105 bits per transform
			{ 11, 5, 19 },		// 105 bits per transform
			{ 11, 6, 18 },		// 105 bits per transform
			{ 11, 7, 17 },		// 105 bits per transform
			{ 11, 8, 16 },		// 105 bits per transform
			{ 11, 9, 15 },		// 105 bits per transform
			{ 11, 10, 14 },		// 105 bits per transform
			{ 11, 11, 13 },		// 105 bits per transform
			{ 11, 12, 12 },		// 105 bits per transform
			{ 11, 13, 11 },		// 105 bits per transform
			{ 11, 14, 10 },		// 105 bits per transform
			{ 11, 15, 9 },		// 105 bits per transform
			{ 11, 16, 8 },		// 105 bits per transform
			{ 11, 17, 7 },		// 105 bits per transform
			{ 11, 18, 6 },		// 105 bits per transform
			{ 11, 19, 5 },		// 105 bits per transform
			{ 11, 20, 4 },		// 105 bits per transform
			{ 11, 21, 3 },		// 105 bits per transform
			{ 11, 22, 2 },		// 105 bits per transform
			{ 11, 23, 1 },		// 105 bits per transform
			{ 12, 0, 23 },		// 105 bits per transform
			{ 12, 1, 22 },		// 105 bits per transform
			{ 12, 2, 21 },		// 105 bits per transform
			{ 12, 3, 20 },		// 105 bits per transform
			{ 12, 4, 19 },		// 105 bits per transform
			{ 12, 5, 18 },		// 105 bits per transform
			{ 12, 6, 17 },		// 105 bits per transform
			{ 12, 7, 16 },		// 105 bits per transform
			{ 12, 8, 15 },		// 105 bits per transform
			{ 12, 9, 14 },		// 105 bits per transform
			{ 12, 10, 13 },		// 105 bits per transform
			{ 12, 11, 12 },		// 105 bits per transform
			{ 12, 12, 11 },		// 105 bits per transform
			{ 12, 13, 10 },		// 105 bits per transform
			{ 12, 14, 9 },		// 105 bits per transform
			{ 12, 15, 8 },		// 105 bits per transform
			{ 12, 16, 7 },		// 105 bits per transform
			{ 12, 17, 6 },		// 105 bits per transform
			{ 12, 18, 5 },		// 105 bits per transform
			{ 12, 19, 4 },		// 105 bits per transform
			{ 12, 20, 3 },		// 105 bits per transform
			{ 12, 21, 2 },		// 105 bits per transform
			{ 12, 22, 1 },		// 105 bits per transform
			{ 12, 23, 0 },		// 105 bits per transform
			{ 13, 0, 22 },		// 105 bits per transform
			{ 13, 1, 21 },		// 105 bits per transform
			{ 13, 2, 20 },		// 105 bits per transform
			{ 13, 3, 19 },		// 105 bits per transform
			{ 13, 4, 18 },		// 105 bits per transform
			{ 13, 5, 17 },		// 105 bits per transform
			{ 13, 6, 16 },		// 105 bits per transform
			{ 13, 7, 15 },		// 105 bits per transform
			{ 13, 8, 14 },		// 105 bits per transform
			{ 13, 9, 13 },		// 105 bits per transform
			{ 13, 10, 12 },		// 105 bits per transform
			{ 13, 11, 11 },		// 105 bits per transform
			{ 13, 12, 10 },		// 105 bits per transform
			{ 13, 13, 9 },		// 105 bits per transform
			{ 13, 14, 8 },		// 105 bits per transform
			{ 13, 15, 7 },		// 105 bits per transform
			{ 13, 16, 6 },		// 105 bits per transform
			{ 13, 17, 5 },		// 105 bits per transform
			{ 13, 18, 4 },		// 105 bits per transform
			{ 13, 19, 3 },		// 105 bits per transform
			{ 13, 20, 2 },		// 105 bits per transform
			{ 13, 21, 1 },		// 105 bits per transform
			{ 13, 22, 0 },		// 105 bits per transform
			{ 14, 0, 21 },		// 105 bits per transform
			{ 14, 1, 20 },		// 105 bits per transform
			{ 14, 2, 19 },		// 105 bits per transform
			{ 14, 3, 18 },		// 105 bits per transform
			{ 14, 4, 17 },		// 105 bits per transform
			{ 14, 5, 16 },		// 105 bits per transform
			{ 14, 6, 15 },		// 105 bits per transform
			{ 14, 7, 14 },		// 105 bits per transform
			{ 14, 8, 13 },		// 105 bits per transform
			{ 14, 9, 12 },		// 105 bits per transform
			{ 14, 10, 11 },		// 105 bits per transform
			{ 14, 11, 10 },		// 105 bits per transform
			{ 14, 12, 9 },		// 105 bits per transform
			{ 14, 13, 8 },		// 105 bits per transform
			{ 14, 14, 7 },		// 105 bits per transform
			{ 14, 15, 6 },		// 105 bits per transform
			{ 14, 16, 5 },		// 105 bits per transform
			{ 14, 17, 4 },		// 105 bits per transform
			{ 14, 18, 3 },		// 105 bits per transform
			{ 14, 19, 2 },		// 105 bits per transform
			{ 14, 20, 1 },		// 105 bits per transform
			{ 14, 21, 0 },		// 105 bits per transform
			{ 15, 0, 20 },		// 105 bits per transform
			{ 15, 1, 19 },		// 105 bits per transform
			{ 15, 2, 18 },		// 105 bits per transform
			{ 15, 3, 17 },		// 105 bits per transform
			{ 15, 4, 16 },		// 105 bits per transform
			{ 15, 5, 15 },		// 105 bits per transform
			{ 15, 6, 14 },		// 105 bits per transform
			{ 15, 7, 13 },		// 105 bits per transform
			{ 15, 8, 12 },		// 105 bits per transform
			{ 15, 9, 11 },		// 105 bits per transform
			{ 15, 10, 10 },		// 105 bits per transform
			{ 15, 11, 9 },		// 105 bits per transform
			{ 15, 12, 8 },		// 105 bits per transform
			{ 15, 13, 7 },		// 105 bits per transform
			{ 15, 14, 6 },		// 105 bits per transform
			{ 15, 15, 5 },		// 105 bits per transform
			{ 15, 16, 4 },		// 105 bits per transform
			{ 15, 17, 3 },		// 105 bits per transform
			{ 15, 18, 2 },		// 105 bits per transform
			{ 15, 19, 1 },		// 105 bits per transform
			{ 15, 20, 0 },		// 105 bits per transform
			{ 16, 0, 19 },		// 105 bits per transform
			{ 16, 1, 18 },		// 105 bits per transform
			{ 16, 2, 17 },		// 105 bits per transform
			{ 16, 3, 16 },		// 105 bits per transform
			{ 16, 4, 15 },		// 105 bits per transform
			{ 16, 5, 14 },		// 105 bits per transform
			{ 16, 6, 13 },		// 105 bits per transform
			{ 16, 7, 12 },		// 105 bits per transform
			{ 16, 8, 11 },		// 105 bits per transform
			{ 16, 9, 10 },		// 105 bits per transform
			{ 16, 10, 9 },		// 105 bits per transform
			{ 16, 11, 8 },		// 105 bits per transform
			{ 16, 12, 7 },		// 105 bits per transform
			{ 16, 13, 6 },		// 105 bits per transform
			{ 16, 14, 5 },		// 105 bits per transform
			{ 16, 15, 4 },		// 105 bits per transform
			{ 16, 16, 3 },		// 105 bits per transform
			{ 16, 17, 2 },		// 105 bits per transform
			{ 16, 18, 1 },		// 105 bits per transform
			{ 16, 19, 0 },		// 105 bits per transform
			{ 17, 0, 18 },		// 105 bits per transform
			{ 17, 1, 17 },		// 105 bits per transform
			{ 17, 2, 16 },		// 105 bits per transform
			{ 17, 3, 15 },		// 105 bits per transform
			{ 17, 4, 14 },		// 105 bits per transform
			{ 17, 5, 13 },		// 105 bits per transform
			{ 17, 6, 12 },		// 105 bits per transform
			{ 17, 7, 11 },		// 105 bits per transform
			{ 17, 8, 10 },		// 105 bits per transform
			{ 17, 9, 9 },		// 105 bits per transform
			{ 17, 10, 8 },		// 105 bits per transform
			{ 17, 11, 7 },		// 105 bits per transform
			{ 17, 12, 6 },		// 105 bits per transform
			{ 17, 13, 5 },		// 105 bits per transform
			{ 17, 14, 4 },		// 105 bits per transform
			{ 17, 15, 3 },		// 105 bits per transform
			{ 17, 16, 2 },		// 105 bits per transform
			{ 17, 17, 1 },		// 105 bits per transform
			{ 17, 18, 0 },		// 105 bits per transform
			{ 18, 0, 17 },		// 105 bits per transform
			{ 18, 1, 16 },		// 105 bits per transform
			{ 18, 2, 15 },		// 105 bits per transform
			{ 18, 3, 14 },		// 105 bits per transform
			{ 18, 4, 13 },		// 105 bits per transform
			{ 18, 5, 12 },		// 105 bits per transform
			{ 18, 6, 11 },		// 105 bits per transform
			{ 18, 7, 10 },		// 105 bits per transform
			{ 18, 8, 9 },		// 105 bits per transform
			{ 18, 9, 8 },		// 105 bits per transform
			{ 18, 10, 7 },		// 105 bits per transform
			{ 18, 11, 6 },		// 105 bits per transform
			{ 18, 12, 5 },		// 105 bits per transform
			{ 18, 13, 4 },		// 105 bits per transform
			{ 18, 14, 3 },		// 105 bits per transform
			{ 18, 15, 2 },		// 105 bits per transform
			{ 18, 16, 1 },		// 105 bits per transform
			{ 18, 17, 0 },		// 105 bits per transform
			{ 19, 0, 16 },		// 105 bits per transform
			{ 19, 1, 15 },		// 105 bits per transform
			{ 19, 2, 14 },		// 105 bits per transform
			{ 19, 3, 13 },		// 105 bits per transform
			{ 19, 4, 12 },		// 105 bits per transform
			{ 19, 5, 11 },		// 105 bits per transform
			{ 19, 6, 10 },		// 105 bits per transform
			{ 19, 7, 9 },		// 105 bits per transform
			{ 19, 8, 8 },		// 105 bits per transform
			{ 19, 9, 7 },		// 105 bits per transform
			{ 19, 10, 6 },		// 105 bits per transform
			{ 19, 11, 5 },		// 105 bits per transform
			{ 19, 12, 4 },		// 105 bits per transform
			{ 19, 13, 3 },		// 105 bits per transform
			{ 19, 14, 2 },		// 105 bits per transform
			{ 19, 15, 1 },		// 105 bits per transform
			{ 19, 16, 0 },		// 105 bits per transform
			{ 20, 0, 15 },		// 105 bits per transform
			{ 20, 1, 14 },		// 105 bits per transform
			{ 20, 2, 13 },		// 105 bits per transform
			{ 20, 3, 12 },		// 105 bits per transform
			{ 20, 4, 11 },		// 105 bits per transform
			{ 20, 5, 10 },		// 105 bits per transform
			{ 20, 6, 9 },		// 105 bits per transform
			{ 20, 7, 8 },		// 105 bits per transform
			{ 20, 8, 7 },		// 105 bits per transform
			{ 20, 9, 6 },		// 105 bits per transform
			{ 20, 10, 5 },		// 105 bits per transform
			{ 20, 11, 4 },		// 105 bits per transform
			{ 20, 12, 3 },		// 105 bits per transform
			{ 20, 13, 2 },		// 105 bits per transform
			{ 20, 14, 1 },		// 105 bits per transform
			{ 20, 15, 0 },		// 105 bits per transform
			{ 21, 0, 14 },		// 105 bits per transform
			{ 21, 1, 13 },		// 105 bits per transform
			{ 21, 2, 12 },		// 105 bits per transform
			{ 21, 3, 11 },		// 105 bits per transform
			{ 21, 4, 10 },		// 105 bits per transform
			{ 21, 5, 9 },		// 105 bits per transform
			{ 21, 6, 8 },		// 105 bits per transform
			{ 21, 7, 7 },		// 105 bits per transform
			{ 21, 8, 6 },		// 105 bits per transform
			{ 21, 9, 5 },		// 105 bits per transform
			{ 21, 10, 4 },		// 105 bits per transform
			{ 21, 11, 3 },		// 105 bits per transform
			{ 21, 12, 2 },		// 105 bits per transform
			{ 21, 13, 1 },		// 105 bits per transform
			{ 21, 14, 0 },		// 105 bits per transform
			{ 22, 0, 13 },		// 105 bits per transform
			{ 22, 1, 12 },		// 105 bits per transform
			{ 22, 2, 11 },		// 105 bits per transform
			{ 22, 3, 10 },		// 105 bits per transform
			{ 22, 4, 9 },		// 105 bits per transform
			{ 22, 5, 8 },		// 105 bits per transform
			{ 22, 6, 7 },		// 105 bits per transform
			{ 22, 7, 6 },		// 105 bits per transform
			{ 22, 8, 5 },		// 105 bits per transform
			{ 22, 9, 4 },		// 105 bits per transform
			{ 22, 10, 3 },		// 105 bits per transform
			{ 22, 11, 2 },		// 105 bits per transform
			{ 22, 12, 1 },		// 105 bits per transform
			{ 22, 13, 0 },		// 105 bits per transform
			{ 23, 0, 12 },		// 105 bits per transform
			{ 23, 1, 11 },		// 105 bits per transform
			{ 23, 2, 10 },		// 105 bits per transform
			{ 23, 3, 9 },		// 105 bits per transform
			{ 23, 4, 8 },		// 105 bits per transform
			{ 23, 5, 7 },		// 105 bits per transform
			{ 23, 6, 6 },		// 105 bits per transform
			{ 23, 7, 5 },		// 105 bits per transform
			{ 23, 8, 4 },		// 105 bits per transform
			{ 23, 9, 3 },		// 105 bits per transform
			{ 23, 10, 2 },		// 105 bits per transform
			{ 23, 11, 1 },		// 105 bits per transform
			{ 23, 12, 0 },		// 105 bits per transform
			{ 24, 0, 3 },		// 105 bits per transform
			{ 24, 1, 2 },		// 105 bits per transform
			{ 24, 2, 1 },		// 105 bits per transform
			{ 24, 3, 0 },		// 105 bits per transform
			{ 0, 4, 24 },		// 108 bits per transform
			{ 0, 13, 23 },		// 108 bits per transform
			{ 0, 14, 22 },		// 108 bits per transform
			{ 0, 15, 21 },		// 108 bits per transform
			{ 0, 16, 20 },		// 108 bits per transform
			{ 0, 17, 19 },		// 108 bits per transform
			{ 0, 18, 18 },		// 108 bits per transform
			{ 0, 19, 17 },		// 108 bits per transform
			{ 0, 20, 16 },		// 108 bits per transform
			{ 0, 21, 15 },		// 108 bits per transform
			{ 0, 22, 14 },		// 108 bits per transform
			{ 0, 23, 13 },		// 108 bits per transform
			{ 0, 24, 4 },		// 108 bits per transform
			{ 1, 3, 24 },		// 108 bits per transform
			{ 1, 12, 23 },		// 108 bits per transform
			{ 1, 13, 22 },		// 108 bits per transform
			{ 1, 14, 21 },		// 108 bits per transform
			{ 1, 15, 20 },		// 108 bits per transform
			{ 1, 16, 19 },		// 108 bits per transform
			{ 1, 17, 18 },		// 108 bits per transform
			{ 1, 18, 17 },		// 108 bits per transform
			{ 1, 19, 16 },		// 108 bits per transform
			{ 1, 20, 15 },		// 108 bits per transform
			{ 1, 21, 14 },		// 108 bits per transform
			{ 1, 22, 13 },		// 108 bits per transform
			{ 1, 23, 12 },		// 108 bits per transform
			{ 1, 24, 3 },		// 108 bits per transform
			{ 2, 2, 24 },		// 108 bits per transform
			{ 2, 11, 23 },		// 108 bits per transform
			{ 2, 12, 22 },		// 108 bits per transform
			{ 2, 13, 21 },		// 108 bits per transform
			{ 2, 14, 20 },		// 108 bits per transform
			{ 2, 15, 19 },		// 108 bits per transform
			{ 2, 16, 18 },		// 108 bits per transform
			{ 2, 17, 17 },		// 108 bits per transform
			{ 2, 18, 16 },		// 108 bits per transform
			{ 2, 19, 15 },		// 108 bits per transform
			{ 2, 20, 14 },		// 108 bits per transform
			{ 2, 21, 13 },		// 108 bits per transform
			{ 2, 22, 12 },		// 108 bits per transform
			{ 2, 23, 11 },		// 108 bits per transform
			{ 2, 24, 2 },		// 108 bits per transform
			{ 3, 1, 24 },		// 108 bits per transform
			{ 3, 10, 23 },		// 108 bits per transform
			{ 3, 11, 22 },		// 108 bits per transform
			{ 3, 12, 21 },		// 108 bits per transform
			{ 3, 13, 20 },		// 108 bits per transform
			{ 3, 14, 19 },		// 108 bits per transform
			{ 3, 15, 18 },		// 108 bits per transform
			{ 3, 16, 17 },		// 108 bits per transform
			{ 3, 17, 16 },		// 108 bits per transform
			{ 3, 18, 15 },		// 108 bits per transform
			{ 3, 19, 14 },		// 108 bits per transform
			{ 3, 20, 13 },		// 108 bits per transform
			{ 3, 21, 12 },		// 108 bits per transform
			{ 3, 22, 11 },		// 108 bits per transform
			{ 3, 23, 10 },		// 108 bits per transform
			{ 3, 24, 1 },		// 108 bits per transform
			{ 4, 0, 24 },		// 108 bits per transform
			{ 4, 9, 23 },		// 108 bits per transform
			{ 4, 10, 22 },		// 108 bits per transform
			{ 4, 11, 21 },		// 108 bits per transform
			{ 4, 12, 20 },		// 108 bits per transform
			{ 4, 13, 19 },		// 108 bits per transform
			{ 4, 14, 18 },		// 108 bits per transform
			{ 4, 15, 17 },		// 108 bits per transform
			{ 4, 16, 16 },		// 108 bits per transform
			{ 4, 17, 15 },		// 108 bits per transform
			{ 4, 18, 14 },		// 108 bits per transform
			{ 4, 19, 13 },		// 108 bits per transform
			{ 4, 20, 12 },		// 108 bits per transform
			{ 4, 21, 11 },		// 108 bits per transform
			{ 4, 22, 10 },		// 108 bits per transform
			{ 4, 23, 9 },		// 108 bits per transform
			{ 4, 24, 0 },		// 108 bits per transform
			{ 5, 8, 23 },		// 108 bits per transform
			{ 5, 9, 22 },		// 108 bits per transform
			{ 5, 10, 21 },		// 108 bits per transform
			{ 5, 11, 20 },		// 108 bits per transform
			{ 5, 12, 19 },		// 108 bits per transform
			{ 5, 13, 18 },		// 108 bits per transform
			{ 5, 14, 17 },		// 108 bits per transform
			{ 5, 15, 16 },		// 108 bits per transform
			{ 5, 16, 15 },		// 108 bits per transform
			{ 5, 17, 14 },		// 108 bits per transform
			{ 5, 18, 13 },		// 108 bits per transform
			{ 5, 19, 12 },		// 108 bits per transform
			{ 5, 20, 11 },		// 108 bits per transform
			{ 5, 21, 10 },		// 108 bits per transform
			{ 5, 22, 9 },		// 108 bits per transform
			{ 5, 23, 8 },		// 108 bits per transform
			{ 6, 7, 23 },		// 108 bits per transform
			{ 6, 8, 22 },		// 108 bits per transform
			{ 6, 9, 21 },		// 108 bits per transform
			{ 6, 10, 20 },		// 108 bits per transform
			{ 6, 11, 19 },		// 108 bits per transform
			{ 6, 12, 18 },		// 108 bits per transform
			{ 6, 13, 17 },		// 108 bits per transform
			{ 6, 14, 16 },		// 108 bits per transform
			{ 6, 15, 15 },		// 108 bits per transform
			{ 6, 16, 14 },		// 108 bits per transform
			{ 6, 17, 13 },		// 108 bits per transform
			{ 6, 18, 12 },		// 108 bits per transform
			{ 6, 19, 11 },		// 108 bits per transform
			{ 6, 20, 10 },		// 108 bits per transform
			{ 6, 21, 9 },		// 108 bits per transform
			{ 6, 22, 8 },		// 108 bits per transform
			{ 6, 23, 7 },		// 108 bits per transform
			{ 7, 6, 23 },		// 108 bits per transform
			{ 7, 7, 22 },		// 108 bits per transform
			{ 7, 8, 21 },		// 108 bits per transform
			{ 7, 9, 20 },		// 108 bits per transform
			{ 7, 10, 19 },		// 108 bits per transform
			{ 7, 11, 18 },		// 108 bits per transform
			{ 7, 12, 17 },		// 108 bits per transform
			{ 7, 13, 16 },		// 108 bits per transform
			{ 7, 14, 15 },		// 108 bits per transform
			{ 7, 15, 14 },		// 108 bits per transform
			{ 7, 16, 13 },		// 108 bits per transform
			{ 7, 17, 12 },		// 108 bits per transform
			{ 7, 18, 11 },		// 108 bits per transform
			{ 7, 19, 10 },		// 108 bits per transform
			{ 7, 20, 9 },		// 108 bits per transform
			{ 7, 21, 8 },		// 108 bits per transform
			{ 7, 22, 7 },		// 108 bits per transform
			{ 7, 23, 6 },		// 108 bits per transform
			{ 8, 5, 23 },		// 108 bits per transform
			{ 8, 6, 22 },		// 108 bits per transform
			{ 8, 7, 21 },		// 108 bits per transform
			{ 8, 8, 20 },		// 108 bits per transform
			{ 8, 9, 19 },		// 108 bits per transform
			{ 8, 10, 18 },		// 108 bits per transform
			{ 8, 11, 17 },		// 108 bits per transform
			{ 8, 12, 16 },		// 108 bits per transform
			{ 8, 13, 15 },		// 108 bits per transform
			{ 8, 14, 14 },		// 108 bits per transform
			{ 8, 15, 13 },		// 108 bits per transform
			{ 8, 16, 12 },		// 108 bits per transform
			{ 8, 17, 11 },		// 108 bits per transform
			{ 8, 18, 10 },		// 108 bits per transform
			{ 8, 19, 9 },		// 108 bits per transform
			{ 8, 20, 8 },		// 108 bits per transform
			{ 8, 21, 7 },		// 108 bits per transform
			{ 8, 22, 6 },		// 108 bits per transform
			{ 8, 23, 5 },		// 108 bits per transform
			{ 9, 4, 23 },		// 108 bits per transform
			{ 9, 5, 22 },		// 108 bits per transform
			{ 9, 6, 21 },		// 108 bits per transform
			{ 9, 7, 20 },		// 108 bits per transform
			{ 9, 8, 19 },		// 108 bits per transform
			{ 9, 9, 18 },		// 108 bits per transform
			{ 9, 10, 17 },		// 108 bits per transform
			{ 9, 11, 16 },		// 108 bits per transform
			{ 9, 12, 15 },		// 108 bits per transform
			{ 9, 13, 14 },		// 108 bits per transform
			{ 9, 14, 13 },		// 108 bits per transform
			{ 9, 15, 12 },		// 108 bits per transform
			{ 9, 16, 11 },		// 108 bits per transform
			{ 9, 17, 10 },		// 108 bits per transform
			{ 9, 18, 9 },		// 108 bits per transform
			{ 9, 19, 8 },		// 108 bits per transform
			{ 9, 20, 7 },		// 108 bits per transform
			{ 9, 21, 6 },		// 108 bits per transform
			{ 9, 22, 5 },		// 108 bits per transform
			{ 9, 23, 4 },		// 108 bits per transform
			{ 10, 3, 23 },		// 108 bits per transform
			{ 10, 4, 22 },		// 108 bits per transform
			{ 10, 5, 21 },		// 108 bits per transform
			{ 10, 6, 20 },		// 108 bits per transform
			{ 10, 7, 19 },		// 108 bits per transform
			{ 10, 8, 18 },		// 108 bits per transform
			{ 10, 9, 17 },		// 108 bits per transform
			{ 10, 10, 16 },		// 108 bits per transform
			{ 10, 11, 15 },		// 108 bits per transform
			{ 10, 12, 14 },		// 108 bits per transform
			{ 10, 13, 13 },		// 108 bits per transform
			{ 10, 14, 12 },		// 108 bits per transform
			{ 10, 15, 11 },		// 108 bits per transform
			{ 10, 16, 10 },		// 108 bits per transform
			{ 10, 17, 9 },		// 108 bits per transform
			{ 10, 18, 8 },		// 108 bits per transform
			{ 10, 19, 7 },		// 108 bits per transform
			{ 10, 20, 6 },		// 108 bits per transform
			{ 10, 21, 5 },		// 108 bits per transform
			{ 10, 22, 4 },		// 108 bits per transform
			{ 10, 23, 3 },		// 108 bits per transform
			{ 11, 2, 23 },		// 108 bits per transform
			{ 11, 3, 22 },		// 108 bits per transform
			{ 11, 4, 21 },		// 108 bits per transform
			{ 11, 5, 20 },		// 108 bits per transform
			{ 11, 6, 19 },		// 108 bits per transform
			{ 11, 7, 18 },		// 108 bits per transform
			{ 11, 8, 17 },		// 108 bits per transform
			{ 11, 9, 16 },		// 108 bits per transform
			{ 11, 10, 15 },		// 108 bits per transform
			{ 11, 11, 14 },		// 108 bits per transform
			{ 11, 12, 13 },		// 108 bits per transform
			{ 11, 13, 12 },		// 108 bits per transform
			{ 11, 14, 11 },		// 108 bits per transform
			{ 11, 15, 10 },		// 108 bits per transform
			{ 11, 16, 9 },		// 108 bits per transform
			{ 11, 17, 8 },		// 108 bits per transform
			{ 11, 18, 7 },		// 108 bits per transform
			{ 11, 19, 6 },		// 108 bits per transform
			{ 11, 20, 5 },		// 108 bits per transform
			{ 11, 21, 4 },		// 108 bits per transform
			{ 11, 22, 3 },		// 108 bits per transform
			{ 11, 23, 2 },		// 108 bits per transform
			{ 12, 1, 23 },		// 108 bits per transform
			{ 12, 2, 22 },		// 108 bits per transform
			{ 12, 3, 21 },		// 108 bits per transform
			{ 12, 4, 20 },		// 108 bits per transform
			{ 12, 5, 19 },		// 108 bits per transform
			{ 12, 6, 18 },		// 108 bits per transform
			{ 12, 7, 17 },		// 108 bits per transform
			{ 12, 8, 16 },		// 108 bits per transform
			{ 12, 9, 15 },		// 108 bits per transform
			{ 12, 10, 14 },		// 108 bits per transform
			{ 12, 11, 13 },		// 108 bits per transform
			{ 12, 12, 12 },		// 108 bits per transform
			{ 12, 13, 11 },		// 108 bits per transform
			{ 12, 14, 10 },		// 108 bits per transform
			{ 12, 15, 9 },		// 108 bits per transform
			{ 12, 16, 8 },		// 108 bits per transform
			{ 12, 17, 7 },		// 108 bits per transform
			{ 12, 18, 6 },		// 108 bits per transform
			{ 12, 19, 5 },		// 108 bits per transform
			{ 12, 20, 4 },		// 108 bits per transform
			{ 12, 21, 3 },		// 108 bits per transform
			{ 12, 22, 2 },		// 108 bits per transform
			{ 12, 23, 1 },		// 108 bits per transform
			{ 13, 0, 23 },		// 108 bits per transform
			{ 13, 1, 22 },		// 108 bits per transform
			{ 13, 2, 21 },		// 108 bits per transform
			{ 13, 3, 20 },		// 108 bits per transform
			{ 13, 4, 19 },		// 108 bits per transform
			{ 13, 5, 18 },		// 108 bits per transform
			{ 13, 6, 17 },		// 108 bits per transform
			{ 13, 7, 16 },		// 108 bits per transform
			{ 13, 8, 15 },		// 108 bits per transform
			{ 13, 9, 14 },		// 108 bits per transform
			{ 13, 10, 13 },		// 108 bits per transform
			{ 13, 11, 12 },		// 108 bits per transform
			{ 13, 12, 11 },		// 108 bits per transform
			{ 13, 13, 10 },		// 108 bits per transform
			{ 13, 14, 9 },		// 108 bits per transform
			{ 13, 15, 8 },		// 108 bits per transform
			{ 13, 16, 7 },		// 108 bits per transform
			{ 13, 17, 6 },		// 108 bits per transform
			{ 13, 18, 5 },		// 108 bits per transform
			{ 13, 19, 4 },		// 108 bits per transform
			{ 13, 20, 3 },		// 108 bits per transform
			{ 13, 21, 2 },		// 108 bits per transform
			{ 13, 22, 1 },		// 108 bits per transform
			{ 13, 23, 0 },		// 108 bits per transform
			{ 14, 0, 22 },		// 108 bits per transform
			{ 14, 1, 21 },		// 108 bits per transform
			{ 14, 2, 20 },		// 108 bits per transform
			{ 14, 3, 19 },		// 108 bits per transform
			{ 14, 4, 18 },		// 108 bits per transform
			{ 14, 5, 17 },		// 108 bits per transform
			{ 14, 6, 16 },		// 108 bits per transform
			{ 14, 7, 15 },		// 108 bits per transform
			{ 14, 8, 14 },		// 108 bits per transform
			{ 14, 9, 13 },		// 108 bits per transform
			{ 14, 10, 12 },		// 108 bits per transform
			{ 14, 11, 11 },		// 108 bits per transform
			{ 14, 12, 10 },		// 108 bits per transform
			{ 14, 13, 9 },		// 108 bits per transform
			{ 14, 14, 8 },		// 108 bits per transform
			{ 14, 15, 7 },		// 108 bits per transform
			{ 14, 16, 6 },		// 108 bits per transform
			{ 14, 17, 5 },		// 108 bits per transform
			{ 14, 18, 4 },		// 108 bits per transform
			{ 14, 19, 3 },		// 108 bits per transform
			{ 14, 20, 2 },		// 108 bits per transform
			{ 14, 21, 1 },		// 108 bits per transform
			{ 14, 22, 0 },		// 108 bits per transform
			{ 15, 0, 21 },		// 108 bits per transform
			{ 15, 1, 20 },		// 108 bits per transform
			{ 15, 2, 19 },		// 108 bits per transform
			{ 15, 3, 18 },		// 108 bits per transform
			{ 15, 4, 17 },		// 108 bits per transform
			{ 15, 5, 16 },		// 108 bits per transform
			{ 15, 6, 15 },		// 108 bits per transform
			{ 15, 7, 14 },		// 108 bits per transform
			{ 15, 8, 13 },		// 108 bits per transform
			{ 15, 9, 12 },		// 108 bits per transform
			{ 15, 10, 11 },		// 108 bits per transform
			{ 15, 11, 10 },		// 108 bits per transform
			{ 15, 12, 9 },		// 108 bits per transform
			{ 15, 13, 8 },		// 108 bits per transform
			{ 15, 14, 7 },		// 108 bits per transform
			{ 15, 15, 6 },		// 108 bits per transform
			{ 15, 16, 5 },		// 108 bits per transform
			{ 15, 17, 4 },		// 108 bits per transform
			{ 15, 18, 3 },		// 108 bits per transform
			{ 15, 19, 2 },		// 108 bits per transform
			{ 15, 20, 1 },		// 108 bits per transform
			{ 15, 21, 0 },		// 108 bits per transform
			{ 16, 0, 20 },		// 108 bits per transform
			{ 16, 1, 19 },		// 108 bits per transform
			{ 16, 2, 18 },		// 108 bits per transform
			{ 16, 3, 17 },		// 108 bits per transform
			{ 16, 4, 16 },		// 108 bits per transform
			{ 16, 5, 15 },		// 108 bits per transform
			{ 16, 6, 14 },		// 108 bits per transform
			{ 16, 7, 13 },		// 108 bits per transform
			{ 16, 8, 12 },		// 108 bits per transform
			{ 16, 9, 11 },		// 108 bits per transform
			{ 16, 10, 10 },		// 108 bits per transform
			{ 16, 11, 9 },		// 108 bits per transform
			{ 16, 12, 8 },		// 108 bits per transform
			{ 16, 13, 7 },		// 108 bits per transform
			{ 16, 14, 6 },		// 108 bits per transform
			{ 16, 15, 5 },		// 108 bits per transform
			{ 16, 16, 4 },		// 108 bits per transform
			{ 16, 17, 3 },		// 108 bits per transform
			{ 16, 18, 2 },		// 108 bits per transform
			{ 16, 19, 1 },		// 108 bits per transform
			{ 16, 20, 0 },		// 108 bits per transform
			{ 17, 0, 19 },		// 108 bits per transform
			{ 17, 1, 18 },		// 108 bits per transform
			{ 17, 2, 17 },		// 108 bits per transform
			{ 17, 3, 16 },		// 108 bits per transform
			{ 17, 4, 15 },		// 108 bits per transform
			{ 17, 5, 14 },		// 108 bits per transform
			{ 17, 6, 13 },		// 108 bits per transform
			{ 17, 7, 12 },		// 108 bits per transform
			{ 17, 8, 11 },		// 108 bits per transform
			{ 17, 9, 10 },		// 108 bits per transform
			{ 17, 10, 9 },		// 108 bits per transform
			{ 17, 11, 8 },		// 108 bits per transform
			{ 17, 12, 7 },		// 108 bits per transform
			{ 17, 13, 6 },		// 108 bits per transform
			{ 17, 14, 5 },		// 108 bits per transform
			{ 17, 15, 4 },		// 108 bits per transform
			{ 17, 16, 3 },		// 108 bits per transform
			{ 17, 17, 2 },		// 108 bits per transform
			{ 17, 18, 1 },		// 108 bits per transform
			{ 17, 19, 0 },		// 108 bits per transform
			{ 18, 0, 18 },		// 108 bits per transform
			{ 18, 1, 17 },		// 108 bits per transform
			{ 18, 2, 16 },		// 108 bits per transform
			{ 18, 3, 15 },		// 108 bits per transform
			{ 18, 4, 14 },		// 108 bits per transform
			{ 18, 5, 13 },		// 108 bits per transform
			{ 18, 6, 12 },		// 108 bits per transform
			{ 18, 7, 11 },		// 108 bits per transform
			{ 18, 8, 10 },		// 108 bits per transform
			{ 18, 9, 9 },		// 108 bits per transform
			{ 18, 10, 8 },		// 108 bits per transform
			{ 18, 11, 7 },		// 108 bits per transform
			{ 18, 12, 6 },		// 108 bits per transform
			{ 18, 13, 5 },		// 108 bits per transform
			{ 18, 14, 4 },		// 108 bits per transform
			{ 18, 15, 3 },		// 108 bits per transform
			{ 18, 16, 2 },		// 108 bits per transform
			{ 18, 17, 1 },		// 108 bits per transform
			{ 18, 18, 0 },		// 108 bits per transform
			{ 19, 0, 17 },		// 108 bits per transform
			{ 19, 1, 16 },		// 108 bits per transform
			{ 19, 2, 15 },		// 108 bits per transform
			{ 19, 3, 14 },		// 108 bits per transform
			{ 19, 4, 13 },		// 108 bits per transform
			{ 19, 5, 12 },		// 108 bits per transform
			{ 19, 6, 11 },		// 108 bits per transform
			{ 19, 7, 10 },		// 108 bits per transform
			{ 19, 8, 9 },		// 108 bits per transform
			{ 19, 9, 8 },		// 108 bits per transform
			{ 19, 10, 7 },		// 108 bits per transform
			{ 19, 11, 6 },		// 108 bits per transform
			{ 19, 12, 5 },		// 108 bits per transform
			{ 19, 13, 4 },		// 108 bits per transform
			{ 19, 14, 3 },		// 108 bits per transform
			{ 19, 15, 2 },		// 108 bits per transform
			{ 19, 16, 1 },		// 108 bits per transform
			{ 19, 17, 0 },		// 108 bits per transform
			{ 20, 0, 16 },		// 108 bits per transform
			{ 20, 1, 15 },		// 108 bits per transform
			{ 20, 2, 14 },		// 108 bits per transform
			{ 20, 3, 13 },		// 108 bits per transform
			{ 20, 4, 12 },		// 108 bits per transform
			{ 20, 5, 11 },		// 108 bits per transform
			{ 20, 6, 10 },		// 108 bits per transform
			{ 20, 7, 9 },		// 108 bits per transform
			{ 20, 8, 8 },		// 108 bits per transform
			{ 20, 9, 7 },		// 108 bits per transform
			{ 20, 10, 6 },		// 108 bits per transform
			{ 20, 11, 5 },		// 108 bits per transform
			{ 20, 12, 4 },		// 108 bits per transform
			{ 20, 13, 3 },		// 108 bits per transform
			{ 20, 14, 2 },		// 108 bits per transform
			{ 20, 15, 1 },		// 108 bits per transform
			{ 20, 16, 0 },		// 108 bits per transform
			{ 21, 0, 15 },		// 108 bits per transform
			{ 21, 1, 14 },		// 108 bits per transform
			{ 21, 2, 13 },		// 108 bits per transform
			{ 21, 3, 12 },		// 108 bits per transform
			{ 21, 4, 11 },		// 108 bits per transform
			{ 21, 5, 10 },		// 108 bits per transform
			{ 21, 6, 9 },		// 108 bits per transform
			{ 21, 7, 8 },		// 108 bits per transform
			{ 21, 8, 7 },		// 108 bits per transform
			{ 21, 9, 6 },		// 108 bits per transform
			{ 21, 10, 5 },		// 108 bits per transform
			{ 21, 11, 4 },		// 108 bits per transform
			{ 21, 12, 3 },		// 108 bits per transform
			{ 21, 13, 2 },		// 108 bits per transform
			{ 21, 14, 1 },		// 108 bits per transform
			{ 21, 15, 0 },		// 108 bits per transform
			{ 22, 0, 14 },		// 108 bits per transform
			{ 22, 1, 13 },		// 108 bits per transform
			{ 22, 2, 12 },		// 108 bits per transform
			{ 22, 3, 11 },		// 108 bits per transform
			{ 22, 4, 10 },		// 108 bits per transform
			{ 22, 5, 9 },		// 108 bits per transform
			{ 22, 6, 8 },		// 108 bits per transform
			{ 22, 7, 7 },		// 108 bits per transform
			{ 22, 8, 6 },		// 108 bits per transform
			{ 22, 9, 5 },		// 108 bits per transform
			{ 22, 10, 4 },		// 108 bits per transform
			{ 22, 11, 3 },		// 108 bits per transform
			{ 22, 12, 2 },		// 108 bits per transform
			{ 22, 13, 1 },		// 108 bits per transform
			{ 22, 14, 0 },		// 108 bits per transform
			{ 23, 0, 13 },		// 108 bits per transform
			{ 23, 1, 12 },		// 108 bits per transform
			{ 23, 2, 11 },		// 108 bits per transform
			{ 23, 3, 10 },		// 108 bits per transform
			{ 23, 4, 9 },		// 108 bits per transform
			{ 23, 5, 8 },		// 108 bits per transform
			{ 23, 6, 7 },		// 108 bits per transform
			{ 23, 7, 6 },		// 108 bits per transform
			{ 23, 8, 5 },		// 108 bits per transform
			{ 23, 9, 4 },		// 108 bits per transform
			{ 23, 10, 3 },		// 108 bits per transform
			{ 23, 11, 2 },		// 108 bits per transform
			{ 23, 12, 1 },		// 108 bits per transform
			{ 23, 13, 0 },		// 108 bits per transform
			{ 24, 0, 4 },		// 108 bits per transform
			{ 24, 1, 3 },		// 108 bits per transform
			{ 24, 2, 2 },		// 108 bits per transform
			{ 24, 3, 1 },		// 108 bits per transform
			{ 24, 4, 0 },		// 108 bits per transform
			{ 0, 5, 24 },		// 111 bits per transform
			{ 0, 14, 23 },		// 111 bits per transform
			{ 0, 15, 22 },		// 111 bits per transform
			{ 0, 16, 21 },		// 111 bits per transform
			{ 0, 17, 20 },		// 111 bits per transform
			{ 0, 18, 19 },		// 111 bits per transform
			{ 0, 19, 18 },		// 111 bits per transform
			{ 0, 20, 17 },		// 111 bits per transform
			{ 0, 21, 16 },		// 111 bits per transform
			{ 0, 22, 15 },		// 111 bits per transform
			{ 0, 23, 14 },		// 111 bits per transform
			{ 0, 24, 5 },		// 111 bits per transform
			{ 1, 4, 24 },		// 111 bits per transform
			{ 1, 13, 23 },		// 111 bits per transform
			{ 1, 14, 22 },		// 111 bits per transform
			{ 1, 15, 21 },		// 111 bits per transform
			{ 1, 16, 20 },		// 111 bits per transform
			{ 1, 17, 19 },		// 111 bits per transform
			{ 1, 18, 18 },		// 111 bits per transform
			{ 1, 19, 17 },		// 111 bits per transform
			{ 1, 20, 16 },		// 111 bits per transform
			{ 1, 21, 15 },		// 111 bits per transform
			{ 1, 22, 14 },		// 111 bits per transform
			{ 1, 23, 13 },		// 111 bits per transform
			{ 1, 24, 4 },		// 111 bits per transform
			{ 2, 3, 24 },		// 111 bits per transform
			{ 2, 12, 23 },		// 111 bits per transform
			{ 2, 13, 22 },		// 111 bits per transform
			{ 2, 14, 21 },		// 111 bits per transform
			{ 2, 15, 20 },		// 111 bits per transform
			{ 2, 16, 19 },		// 111 bits per transform
			{ 2, 17, 18 },		// 111 bits per transform
			{ 2, 18, 17 },		// 111 bits per transform
			{ 2, 19, 16 },		// 111 bits per transform
			{ 2, 20, 15 },		// 111 bits per transform
			{ 2, 21, 14 },		// 111 bits per transform
			{ 2, 22, 13 },		// 111 bits per transform
			{ 2, 23, 12 },		// 111 bits per transform
			{ 2, 24, 3 },		// 111 bits per transform
			{ 3, 2, 24 },		// 111 bits per transform
			{ 3, 11, 23 },		// 111 bits per transform
			{ 3, 12, 22 },		// 111 bits per transform
			{ 3, 13, 21 },		// 111 bits per transform
			{ 3, 14, 20 },		// 111 bits per transform
			{ 3, 15, 19 },		// 111 bits per transform
			{ 3, 16, 18 },		// 111 bits per transform
			{ 3, 17, 17 },		// 111 bits per transform
			{ 3, 18, 16 },		// 111 bits per transform
			{ 3, 19, 15 },		// 111 bits per transform
			{ 3, 20, 14 },		// 111 bits per transform
			{ 3, 21, 13 },		// 111 bits per transform
			{ 3, 22, 12 },		// 111 bits per transform
			{ 3, 23, 11 },		// 111 bits per transform
			{ 3, 24, 2 },		// 111 bits per transform
			{ 4, 1, 24 },		// 111 bits per transform
			{ 4, 10, 23 },		// 111 bits per transform
			{ 4, 11, 22 },		// 111 bits per transform
			{ 4, 12, 21 },		// 111 bits per transform
			{ 4, 13, 20 },		// 111 bits per transform
			{ 4, 14, 19 },		// 111 bits per transform
			{ 4, 15, 18 },		// 111 bits per transform
			{ 4, 16, 17 },		// 111 bits per transform
			{ 4, 17, 16 },		// 111 bits per transform
			{ 4, 18, 15 },		// 111 bits per transform
			{ 4, 19, 14 },		// 111 bits per transform
			{ 4, 20, 13 },		// 111 bits per transform
			{ 4, 21, 12 },		// 111 bits per transform
			{ 4, 22, 11 },		// 111 bits per transform
			{ 4, 23, 10 },		// 111 bits per transform
			{ 4, 24, 1 },		// 111 bits per transform
			{ 5, 0, 24 },		// 111 bits per transform
			{ 5, 9, 23 },		// 111 bits per transform
			{ 5, 10, 22 },		// 111 bits per transform
			{ 5, 11, 21 },		// 111 bits per transform
			{ 5, 12, 20 },		// 111 bits per transform
			{ 5, 13, 19 },		// 111 bits per transform
			{ 5, 14, 18 },		// 111 bits per transform
			{ 5, 15, 17 },		// 111 bits per transform
			{ 5, 16, 16 },		// 111 bits per transform
			{ 5, 17, 15 },		// 111 bits per transform
			{ 5, 18, 14 },		// 111 bits per transform
			{ 5, 19, 13 },		// 111 bits per transform
			{ 5, 20, 12 },		// 111 bits per transform
			{ 5, 21, 11 },		// 111 bits per transform
			{ 5, 22, 10 },		// 111 bits per transform
			{ 5, 23, 9 },		// 111 bits per transform
			{ 5, 24, 0 },		// 111 bits per transform
			{ 6, 8, 23 },		// 111 bits per transform
			{ 6, 9, 22 },		// 111 bits per transform
			{ 6, 10, 21 },		// 111 bits per transform
			{ 6, 11, 20 },		// 111 bits per transform
			{ 6, 12, 19 },		// 111 bits per transform
			{ 6, 13, 18 },		// 111 bits per transform
			{ 6, 14, 17 },		// 111 bits per transform
			{ 6, 15, 16 },		// 111 bits per transform
			{ 6, 16, 15 },		// 111 bits per transform
			{ 6, 17, 14 },		// 111 bits per transform
			{ 6, 18, 13 },		// 111 bits per transform
			{ 6, 19, 12 },		// 111 bits per transform
			{ 6, 20, 11 },		// 111 bits per transform
			{ 6, 21, 10 },		// 111 bits per transform
			{ 6, 22, 9 },		// 111 bits per transform
			{ 6, 23, 8 },		// 111 bits per transform
			{ 7, 7, 23 },		// 111 bits per transform
			{ 7, 8, 22 },		// 111 bits per transform
			{ 7, 9, 21 },		// 111 bits per transform
			{ 7, 10, 20 },		// 111 bits per transform
			{ 7, 11, 19 },		// 111 bits per transform
			{ 7, 12, 18 },		// 111 bits per transform
			{ 7, 13, 17 },		// 111 bits per transform
			{ 7, 14, 16 },		// 111 bits per transform
			{ 7, 15, 15 },		// 111 bits per transform
			{ 7, 16, 14 },		// 111 bits per transform
			{ 7, 17, 13 },		// 111 bits per transform
			{ 7, 18, 12 },		// 111 bits per transform
			{ 7, 19, 11 },		// 111 bits per transform
			{ 7, 20, 10 },		// 111 bits per transform
			{ 7, 21, 9 },		// 111 bits per transform
			{ 7, 22, 8 },		// 111 bits per transform
			{ 7, 23, 7 },		// 111 bits per transform
			{ 8, 6, 23 },		// 111 bits per transform
			{ 8, 7, 22 },		// 111 bits per transform
			{ 8, 8, 21 },		// 111 bits per transform
			{ 8, 9, 20 },		// 111 bits per transform
			{ 8, 10, 19 },		// 111 bits per transform
			{ 8, 11, 18 },		// 111 bits per transform
			{ 8, 12, 17 },		// 111 bits per transform
			{ 8, 13, 16 },		// 111 bits per transform
			{ 8, 14, 15 },		// 111 bits per transform
			{ 8, 15, 14 },		// 111 bits per transform
			{ 8, 16, 13 },		// 111 bits per transform
			{ 8, 17, 12 },		// 111 bits per transform
			{ 8, 18, 11 },		// 111 bits per transform
			{ 8, 19, 10 },		// 111 bits per transform
			{ 8, 20, 9 },		// 111 bits per transform
			{ 8, 21, 8 },		// 111 bits per transform
			{ 8, 22, 7 },		// 111 bits per transform
			{ 8, 23, 6 },		// 111 bits per transform
			{ 9, 5, 23 },		// 111 bits per transform
			{ 9, 6, 22 },		// 111 bits per transform
			{ 9, 7, 21 },		// 111 bits per transform
			{ 9, 8, 20 },		// 111 bits per transform
			{ 9, 9, 19 },		// 111 bits per transform
			{ 9, 10, 18 },		// 111 bits per transform
			{ 9, 11, 17 },		// 111 bits per transform
			{ 9, 12, 16 },		// 111 bits per transform
			{ 9, 13, 15 },		// 111 bits per transform
			{ 9, 14, 14 },		// 111 bits per transform
			{ 9, 15, 13 },		// 111 bits per transform
			{ 9, 16, 12 },		// 111 bits per transform
			{ 9, 17, 11 },		// 111 bits per transform
			{ 9, 18, 10 },		// 111 bits per transform
			{ 9, 19, 9 },		// 111 bits per transform
			{ 9, 20, 8 },		// 111 bits per transform
			{ 9, 21, 7 },		// 111 bits per transform
			{ 9, 22, 6 },		// 111 bits per transform
			{ 9, 23, 5 },		// 111 bits per transform
			{ 10, 4, 23 },		// 111 bits per transform
			{ 10, 5, 22 },		// 111 bits per transform
			{ 10, 6, 21 },		// 111 bits per transform
			{ 10, 7, 20 },		// 111 bits per transform
			{ 10, 8, 19 },		// 111 bits per transform
			{ 10, 9, 18 },		// 111 bits per transform
			{ 10, 10, 17 },		// 111 bits per transform
			{ 10, 11, 16 },		// 111 bits per transform
			{ 10, 12, 15 },		// 111 bits per transform
			{ 10, 13, 14 },		// 111 bits per transform
			{ 10, 14, 13 },		// 111 bits per transform
			{ 10, 15, 12 },		// 111 bits per transform
			{ 10, 16, 11 },		// 111 bits per transform
			{ 10, 17, 10 },		// 111 bits per transform
			{ 10, 18, 9 },		// 111 bits per transform
			{ 10, 19, 8 },		// 111 bits per transform
			{ 10, 20, 7 },		// 111 bits per transform
			{ 10, 21, 6 },		// 111 bits per transform
			{ 10, 22, 5 },		// 111 bits per transform
			{ 10, 23, 4 },		// 111 bits per transform
			{ 11, 3, 23 },		// 111 bits per transform
			{ 11, 4, 22 },		// 111 bits per transform
			{ 11, 5, 21 },		// 111 bits per transform
			{ 11, 6, 20 },		// 111 bits per transform
			{ 11, 7, 19 },		// 111 bits per transform
			{ 11, 8, 18 },		// 111 bits per transform
			{ 11, 9, 17 },		// 111 bits per transform
			{ 11, 10, 16 },		// 111 bits per transform
			{ 11, 11, 15 },		// 111 bits per transform
			{ 11, 12, 14 },		// 111 bits per transform
			{ 11, 13, 13 },		// 111 bits per transform
			{ 11, 14, 12 },		// 111 bits per transform
			{ 11, 15, 11 },		// 111 bits per transform
			{ 11, 16, 10 },		// 111 bits per transform
			{ 11, 17, 9 },		// 111 bits per transform
			{ 11, 18, 8 },		// 111 bits per transform
			{ 11, 19, 7 },		// 111 bits per transform
			{ 11, 20, 6 },		// 111 bits per transform
			{ 11, 21, 5 },		// 111 bits per transform
			{ 11, 22, 4 },		// 111 bits per transform
			{ 11, 23, 3 },		// 111 bits per transform
			{ 12, 2, 23 },		// 111 bits per transform
			{ 12, 3, 22 },		// 111 bits per transform
			{ 12, 4, 21 },		// 111 bits per transform
			{ 12, 5, 20 },		// 111 bits per transform
			{ 12, 6, 19 },		// 111 bits per transform
			{ 12, 7, 18 },		// 111 bits per transform
			{ 12, 8, 17 },		// 111 bits per transform
			{ 12, 9, 16 },		// 111 bits per transform
			{ 12, 10, 15 },		// 111 bits per transform
			{ 12, 11, 14 },		// 111 bits per transform
			{ 12, 12, 13 },		// 111 bits per transform
			{ 12, 13, 12 },		// 111 bits per transform
			{ 12, 14, 11 },		// 111 bits per transform
			{ 12, 15, 10 },		// 111 bits per transform
			{ 12, 16, 9 },		// 111 bits per transform
			{ 12, 17, 8 },		// 111 bits per transform
			{ 12, 18, 7 },		// 111 bits per transform
			{ 12, 19, 6 },		// 111 bits per transform
			{ 12, 20, 5 },		// 111 bits per transform
			{ 12, 21, 4 },		// 111 bits per transform
			{ 12, 22, 3 },		// 111 bits per transform
			{ 12, 23, 2 },		// 111 bits per transform
			{ 13, 1, 23 },		// 111 bits per transform
			{ 13, 2, 22 },		// 111 bits per transform
			{ 13, 3, 21 },		// 111 bits per transform
			{ 13, 4, 20 },		// 111 bits per transform
			{ 13, 5, 19 },		// 111 bits per transform
			{ 13, 6, 18 },		// 111 bits per transform
			{ 13, 7, 17 },		// 111 bits per transform
			{ 13, 8, 16 },		// 111 bits per transform
			{ 13, 9, 15 },		// 111 bits per transform
			{ 13, 10, 14 },		// 111 bits per transform
			{ 13, 11, 13 },		// 111 bits per transform
			{ 13, 12, 12 },		// 111 bits per transform
			{ 13, 13, 11 },		// 111 bits per transform
			{ 13, 14, 10 },		// 111 bits per transform
			{ 13, 15, 9 },		// 111 bits per transform
			{ 13, 16, 8 },		// 111 bits per transform
			{ 13, 17, 7 },		// 111 bits per transform
			{ 13, 18, 6 },		// 111 bits per transform
			{ 13, 19, 5 },		// 111 bits per transform
			{ 13, 20, 4 },		// 111 bits per transform
			{ 13, 21, 3 },		// 111 bits per transform
			{ 13, 22, 2 },		// 111 bits per transform
			{ 13, 23, 1 },		// 111 bits per transform
			{ 14, 0, 23 },		// 111 bits per transform
			{ 14, 1, 22 },		// 111 bits per transform
			{ 14, 2, 21 },		// 111 bits per transform
			{ 14, 3, 20 },		// 111 bits per transform
			{ 14, 4, 19 },		// 111 bits per transform
			{ 14, 5, 18 },		// 111 bits per transform
			{ 14, 6, 17 },		// 111 bits per transform
			{ 14, 7, 16 },		// 111 bits per transform
			{ 14, 8, 15 },		// 111 bits per transform
			{ 14, 9, 14 },		// 111 bits per transform
			{ 14, 10, 13 },		// 111 bits per transform
			{ 14, 11, 12 },		// 111 bits per transform
			{ 14, 12, 11 },		// 111 bits per transform
			{ 14, 13, 10 },		// 111 bits per transform
			{ 14, 14, 9 },		// 111 bits per transform
			{ 14, 15, 8 },		// 111 bits per transform
			{ 14, 16, 7 },		// 111 bits per transform
			{ 14, 17, 6 },		// 111 bits per transform
			{ 14, 18, 5 },		// 111 bits per transform
			{ 14, 19, 4 },		// 111 bits per transform
			{ 14, 20, 3 },		// 111 bits per transform
			{ 14, 21, 2 },		// 111 bits per transform
			{ 14, 22, 1 },		// 111 bits per transform
			{ 14, 23, 0 },		// 111 bits per transform
			{ 15, 0, 22 },		// 111 bits per transform
			{ 15, 1, 21 },		// 111 bits per transform
			{ 15, 2, 20 },		// 111 bits per transform
			{ 15, 3, 19 },		// 111 bits per transform
			{ 15, 4, 18 },		// 111 bits per transform
			{ 15, 5, 17 },		// 111 bits per transform
			{ 15, 6, 16 },		// 111 bits per transform
			{ 15, 7, 15 },		// 111 bits per transform
			{ 15, 8, 14 },		// 111 bits per transform
			{ 15, 9, 13 },		// 111 bits per transform
			{ 15, 10, 12 },		// 111 bits per transform
			{ 15, 11, 11 },		// 111 bits per transform
			{ 15, 12, 10 },		// 111 bits per transform
			{ 15, 13, 9 },		// 111 bits per transform
			{ 15, 14, 8 },		// 111 bits per transform
			{ 15, 15, 7 },		// 111 bits per transform
			{ 15, 16, 6 },		// 111 bits per transform
			{ 15, 17, 5 },		// 111 bits per transform
			{ 15, 18, 4 },		// 111 bits per transform
			{ 15, 19, 3 },		// 111 bits per transform
			{ 15, 20, 2 },		// 111 bits per transform
			{ 15, 21, 1 },		// 111 bits per transform
			{ 15, 22, 0 },		// 111 bits per transform
			{ 16, 0, 21 },		// 111 bits per transform
			{ 16, 1, 20 },		// 111 bits per transform
			{ 16, 2, 19 },		// 111 bits per transform
			{ 16, 3, 18 },		// 111 bits per transform
			{ 16, 4, 17 },		// 111 bits per transform
			{ 16, 5, 16 },		// 111 bits per transform
			{ 16, 6, 15 },		// 111 bits per transform
			{ 16, 7, 14 },		// 111 bits per transform
			{ 16, 8, 13 },		// 111 bits per transform
			{ 16, 9, 12 },		// 111 bits per transform
			{ 16, 10, 11 },		// 111 bits per transform
			{ 16, 11, 10 },		// 111 bits per transform
			{ 16, 12, 9 },		// 111 bits per transform
			{ 16, 13, 8 },		// 111 bits per transform
			{ 16, 14, 7 },		// 111 bits per transform
			{ 16, 15, 6 },		// 111 bits per transform
			{ 16, 16, 5 },		// 111 bits per transform
			{ 16, 17, 4 },		// 111 bits per transform
			{ 16, 18, 3 },		// 111 bits per transform
			{ 16, 19, 2 },		// 111 bits per transform
			{ 16, 20, 1 },		// 111 bits per transform
			{ 16, 21, 0 },		// 111 bits per transform
			{ 17, 0, 20 },		// 111 bits per transform
			{ 17, 1, 19 },		// 111 bits per transform
			{ 17, 2, 18 },		// 111 bits per transform
			{ 17, 3, 17 },		// 111 bits per transform
			{ 17, 4, 16 },		// 111 bits per transform
			{ 17, 5, 15 },		// 111 bits per transform
			{ 17, 6, 14 },		// 111 bits per transform
			{ 17, 7, 13 },		// 111 bits per transform
			{ 17, 8, 12 },		// 111 bits per transform
			{ 17, 9, 11 },		// 111 bits per transform
			{ 17, 10, 10 },		// 111 bits per transform
			{ 17, 11, 9 },		// 111 bits per transform
			{ 17, 12, 8 },		// 111 bits per transform
			{ 17, 13, 7 },		// 111 bits per transform
			{ 17, 14, 6 },		// 111 bits per transform
			{ 17, 15, 5 },		// 111 bits per transform
			{ 17, 16, 4 },		// 111 bits per transform
			{ 17, 17, 3 },		// 111 bits per transform
			{ 17, 18, 2 },		// 111 bits per transform
			{ 17, 19, 1 },		// 111 bits per transform
			{ 17, 20, 0 },		// 111 bits per transform
			{ 18, 0, 19 },		// 111 bits per transform
			{ 18, 1, 18 },		// 111 bits per transform
			{ 18, 2, 17 },		// 111 bits per transform
			{ 18, 3, 16 },		// 111 bits per transform
			{ 18, 4, 15 },		// 111 bits per transform
			{ 18, 5, 14 },		// 111 bits per transform
			{ 18, 6, 13 },		// 111 bits per transform
			{ 18, 7, 12 },		// 111 bits per transform
			{ 18, 8, 11 },		// 111 bits per transform
			{ 18, 9, 10 },		// 111 bits per transform
			{ 18, 10, 9 },		// 111 bits per transform
			{ 18, 11, 8 },		// 111 bits per transform
			{ 18, 12, 7 },		// 111 bits per transform
			{ 18, 13, 6 },		// 111 bits per transform
			{ 18, 14, 5 },		// 111 bits per transform
			{ 18, 15, 4 },		// 111 bits per transform
			{ 18, 16, 3 },		// 111 bits per transform
			{ 18, 17, 2 },		// 111 bits per transform
			{ 18, 18, 1 },		// 111 bits per transform
			{ 18, 19, 0 },		// 111 bits per transform
			{ 19, 0, 18 },		// 111 bits per transform
			{ 19, 1, 17 },		// 111 bits per transform
			{ 19, 2, 16 },		// 111 bits per transform
			{ 19, 3, 15 },		// 111 bits per transform
			{ 19, 4, 14 },		// 111 bits per transform
			{ 19, 5, 13 },		// 111 bits per transform
			{ 19, 6, 12 },		// 111 bits per transform
			{ 19, 7, 11 },		// 111 bits per transform
			{ 19, 8, 10 },		// 111 bits per transform
			{ 19, 9, 9 },		// 111 bits per transform
			{ 19, 10, 8 },		// 111 bits per transform
			{ 19, 11, 7 },		// 111 bits per transform
			{ 19, 12, 6 },		// 111 bits per transform
			{ 19, 13, 5 },		// 111 bits per transform
			{ 19, 14, 4 },		// 111 bits per transform
			{ 19, 15, 3 },		// 111 bits per transform
			{ 19, 16, 2 },		// 111 bits per transform
			{ 19, 17, 1 },		// 111 bits per transform
			{ 19, 18, 0 },		// 111 bits per transform
			{ 20, 0, 17 },		// 111 bits per transform
			{ 20, 1, 16 },		// 111 bits per transform
			{ 20, 2, 15 },		// 111 bits per transform
			{ 20, 3, 14 },		// 111 bits per transform
			{ 20, 4, 13 },		// 111 bits per transform
			{ 20, 5, 12 },		// 111 bits per transform
			{ 20, 6, 11 },		// 111 bits per transform
			{ 20, 7, 10 },		// 111 bits per transform
			{ 20, 8, 9 },		// 111 bits per transform
			{ 20, 9, 8 },		// 111 bits per transform
			{ 20, 10, 7 },		// 111 bits per transform
			{ 20, 11, 6 },		// 111 bits per transform
			{ 20, 12, 5 },		// 111 bits per transform
			{ 20, 13, 4 },		// 111 bits per transform
			{ 20, 14, 3 },		// 111 bits per transform
			{ 20, 15, 2 },		// 111 bits per transform
			{ 20, 16, 1 },		// 111 bits per transform
			{ 20, 17, 0 },		// 111 bits per transform
			{ 21, 0, 16 },		// 111 bits per transform
			{ 21, 1, 15 },		// 111 bits per transform
			{ 21, 2, 14 },		// 111 bits per transform
			{ 21, 3, 13 },		// 111 bits per transform
			{ 21, 4, 12 },		// 111 bits per transform
			{ 21, 5, 11 },		// 111 bits per transform
			{ 21, 6, 10 },		// 111 bits per transform
			{ 21, 7, 9 },		// 111 bits per transform
			{ 21, 8, 8 },		// 111 bits per transform
			{ 21, 9, 7 },		// 111 bits per transform
			{ 21, 10, 6 },		// 111 bits per transform
			{ 21, 11, 5 },		// 111 bits per transform
			{ 21, 12, 4 },		// 111 bits per transform
			{ 21, 13, 3 },		// 111 bits per transform
			{ 21, 14, 2 },		// 111 bits per transform
			{ 21, 15, 1 },		// 111 bits per transform
			{ 21, 16, 0 },		// 111 bits per transform
			{ 22, 0, 15 },		// 111 bits per transform
			{ 22, 1, 14 },		// 111 bits per transform
			{ 22, 2, 13 },		// 111 bits per transform
			{ 22, 3, 12 },		// 111 bits per transform
			{ 22, 4, 11 },		// 111 bits per transform
			{ 22, 5, 10 },		// 111 bits per transform
			{ 22, 6, 9 },		// 111 bits per transform
			{ 22, 7, 8 },		// 111 bits per transform
			{ 22, 8, 7 },		// 111 bits per transform
			{ 22, 9, 6 },		// 111 bits per transform
			{ 22, 10, 5 },		// 111 bits per transform
			{ 22, 11, 4 },		// 111 bits per transform
			{ 22, 12, 3 },		// 111 bits per transform
			{ 22, 13, 2 },		// 111 bits per transform
			{ 22, 14, 1 },		// 111 bits per transform
			{ 22, 15, 0 },		// 111 bits per transform
			{ 23, 0, 14 },		// 111 bits per transform
			{ 23, 1, 13 },		// 111 bits per transform
			{ 23, 2, 12 },		// 111 bits per transform
			{ 23, 3, 11 },		// 111 bits per transform
			{ 23, 4, 10 },		// 111 bits per transform
			{ 23, 5, 9 },		// 111 bits per transform
			{ 23, 6, 8 },		// 111 bits per transform
			{ 23, 7, 7 },		// 111 bits per transform
			{ 23, 8, 6 },		// 111 bits per transform
			{ 23, 9, 5 },		// 111 bits per transform
			{ 23, 10, 4 },		// 111 bits per transform
			{ 23, 11, 3 },		// 111 bits per transform
			{ 23, 12, 2 },		// 111 bits per transform
			{ 23, 13, 1 },		// 111 bits per transform
			{ 23, 14, 0 },		// 111 bits per transform
			{ 24, 0, 5 },		// 111 bits per transform
			{ 24, 1, 4 },		// 111 bits per transform
			{ 24, 2, 3 },		// 111 bits per transform
			{ 24, 3, 2 },		// 111 bits per transform
			{ 24, 4, 1 },		// 111 bits per transform
			{ 24, 5, 0 },		// 111 bits per transform
			{ 0, 6, 24 },		// 114 bits per transform
			{ 0, 15, 23 },		// 114 bits per transform
			{ 0, 16, 22 },		// 114 bits per transform
			{ 0, 17, 21 },		// 114 bits per transform
			{ 0, 18, 20 },		// 114 bits per transform
			{ 0, 19, 19 },		// 114 bits per transform
			{ 0, 20, 18 },		// 114 bits per transform
			{ 0, 21, 17 },		// 114 bits per transform
			{ 0, 22, 16 },		// 114 bits per transform
			{ 0, 23, 15 },		// 114 bits per transform
			{ 0, 24, 6 },		// 114 bits per transform
			{ 1, 5, 24 },		// 114 bits per transform
			{ 1, 14, 23 },		// 114 bits per transform
			{ 1, 15, 22 },		// 114 bits per transform
			{ 1, 16, 21 },		// 114 bits per transform
			{ 1, 17, 20 },		// 114 bits per transform
			{ 1, 18, 19 },		// 114 bits per transform
			{ 1, 19, 18 },		// 114 bits per transform
			{ 1, 20, 17 },		// 114 bits per transform
			{ 1, 21, 16 },		// 114 bits per transform
			{ 1, 22, 15 },		// 114 bits per transform
			{ 1, 23, 14 },		// 114 bits per transform
			{ 1, 24, 5 },		// 114 bits per transform
			{ 2, 4, 24 },		// 114 bits per transform
			{ 2, 13, 23 },		// 114 bits per transform
			{ 2, 14, 22 },		// 114 bits per transform
			{ 2, 15, 21 },		// 114 bits per transform
			{ 2, 16, 20 },		// 114 bits per transform
			{ 2, 17, 19 },		// 114 bits per transform
			{ 2, 18, 18 },		// 114 bits per transform
			{ 2, 19, 17 },		// 114 bits per transform
			{ 2, 20, 16 },		// 114 bits per transform
			{ 2, 21, 15 },		// 114 bits per transform
			{ 2, 22, 14 },		// 114 bits per transform
			{ 2, 23, 13 },		// 114 bits per transform
			{ 2, 24, 4 },		// 114 bits per transform
			{ 3, 3, 24 },		// 114 bits per transform
			{ 3, 12, 23 },		// 114 bits per transform
			{ 3, 13, 22 },		// 114 bits per transform
			{ 3, 14, 21 },		// 114 bits per transform
			{ 3, 15, 20 },		// 114 bits per transform
			{ 3, 16, 19 },		// 114 bits per transform
			{ 3, 17, 18 },		// 114 bits per transform
			{ 3, 18, 17 },		// 114 bits per transform
			{ 3, 19, 16 },		// 114 bits per transform
			{ 3, 20, 15 },		// 114 bits per transform
			{ 3, 21, 14 },		// 114 bits per transform
			{ 3, 22, 13 },		// 114 bits per transform
			{ 3, 23, 12 },		// 114 bits per transform
			{ 3, 24, 3 },		// 114 bits per transform
			{ 4, 2, 24 },		// 114 bits per transform
			{ 4, 11, 23 },		// 114 bits per transform
			{ 4, 12, 22 },		// 114 bits per transform
			{ 4, 13, 21 },		// 114 bits per transform
			{ 4, 14, 20 },		// 114 bits per transform
			{ 4, 15, 19 },		// 114 bits per transform
			{ 4, 16, 18 },		// 114 bits per transform
			{ 4, 17, 17 },		// 114 bits per transform
			{ 4, 18, 16 },		// 114 bits per transform
			{ 4, 19, 15 },		// 114 bits per transform
			{ 4, 20, 14 },		// 114 bits per transform
			{ 4, 21, 13 },		// 114 bits per transform
			{ 4, 22, 12 },		// 114 bits per transform
			{ 4, 23, 11 },		// 114 bits per transform
			{ 4, 24, 2 },		// 114 bits per transform
			{ 5, 1, 24 },		// 114 bits per transform
			{ 5, 10, 23 },		// 114 bits per transform
			{ 5, 11, 22 },		// 114 bits per transform
			{ 5, 12, 21 },		// 114 bits per transform
			{ 5, 13, 20 },		// 114 bits per transform
			{ 5, 14, 19 },		// 114 bits per transform
			{ 5, 15, 18 },		// 114 bits per transform
			{ 5, 16, 17 },		// 114 bits per transform
			{ 5, 17, 16 },		// 114 bits per transform
			{ 5, 18, 15 },		// 114 bits per transform
			{ 5, 19, 14 },		// 114 bits per transform
			{ 5, 20, 13 },		// 114 bits per transform
			{ 5, 21, 12 },		// 114 bits per transform
			{ 5, 22, 11 },		// 114 bits per transform
			{ 5, 23, 10 },		// 114 bits per transform
			{ 5, 24, 1 },		// 114 bits per transform
			{ 6, 0, 24 },		// 114 bits per transform
			{ 6, 9, 23 },		// 114 bits per transform
			{ 6, 10, 22 },		// 114 bits per transform
			{ 6, 11, 21 },		// 114 bits per transform
			{ 6, 12, 20 },		// 114 bits per transform
			{ 6, 13, 19 },		// 114 bits per transform
			{ 6, 14, 18 },		// 114 bits per transform
			{ 6, 15, 17 },		// 114 bits per transform
			{ 6, 16, 16 },		// 114 bits per transform
			{ 6, 17, 15 },		// 114 bits per transform
			{ 6, 18, 14 },		// 114 bits per transform
			{ 6, 19, 13 },		// 114 bits per transform
			{ 6, 20, 12 },		// 114 bits per transform
			{ 6, 21, 11 },		// 114 bits per transform
			{ 6, 22, 10 },		// 114 bits per transform
			{ 6, 23, 9 },		// 114 bits per transform
			{ 6, 24, 0 },		// 114 bits per transform
			{ 7, 8, 23 },		// 114 bits per transform
			{ 7, 9, 22 },		// 114 bits per transform
			{ 7, 10, 21 },		// 114 bits per transform
			{ 7, 11, 20 },		// 114 bits per transform
			{ 7, 12, 19 },		// 114 bits per transform
			{ 7, 13, 18 },		// 114 bits per transform
			{ 7, 14, 17 },		// 114 bits per transform
			{ 7, 15, 16 },		// 114 bits per transform
			{ 7, 16, 15 },		// 114 bits per transform
			{ 7, 17, 14 },		// 114 bits per transform
			{ 7, 18, 13 },		// 114 bits per transform
			{ 7, 19, 12 },		// 114 bits per transform
			{ 7, 20, 11 },		// 114 bits per transform
			{ 7, 21, 10 },		// 114 bits per transform
			{ 7, 22, 9 },		// 114 bits per transform
			{ 7, 23, 8 },		// 114 bits per transform
			{ 8, 7, 23 },		// 114 bits per transform
			{ 8, 8, 22 },		// 114 bits per transform
			{ 8, 9, 21 },		// 114 bits per transform
			{ 8, 10, 20 },		// 114 bits per transform
			{ 8, 11, 19 },		// 114 bits per transform
			{ 8, 12, 18 },		// 114 bits per transform
			{ 8, 13, 17 },		// 114 bits per transform
			{ 8, 14, 16 },		// 114 bits per transform
			{ 8, 15, 15 },		// 114 bits per transform
			{ 8, 16, 14 },		// 114 bits per transform
			{ 8, 17, 13 },		// 114 bits per transform
			{ 8, 18, 12 },		// 114 bits per transform
			{ 8, 19, 11 },		// 114 bits per transform
			{ 8, 20, 10 },		// 114 bits per transform
			{ 8, 21, 9 },		// 114 bits per transform
			{ 8, 22, 8 },		// 114 bits per transform
			{ 8, 23, 7 },		// 114 bits per transform
			{ 9, 6, 23 },		// 114 bits per transform
			{ 9, 7, 22 },		// 114 bits per transform
			{ 9, 8, 21 },		// 114 bits per transform
			{ 9, 9, 20 },		// 114 bits per transform
			{ 9, 10, 19 },		// 114 bits per transform
			{ 9, 11, 18 },		// 114 bits per transform
			{ 9, 12, 17 },		// 114 bits per transform
			{ 9, 13, 16 },		// 114 bits per transform
			{ 9, 14, 15 },		// 114 bits per transform
			{ 9, 15, 14 },		// 114 bits per transform
			{ 9, 16, 13 },		// 114 bits per transform
			{ 9, 17, 12 },		// 114 bits per transform
			{ 9, 18, 11 },		// 114 bits per transform
			{ 9, 19, 10 },		// 114 bits per transform
			{ 9, 20, 9 },		// 114 bits per transform
			{ 9, 21, 8 },		// 114 bits per transform
			{ 9, 22, 7 },		// 114 bits per transform
			{ 9, 23, 6 },		// 114 bits per transform
			{ 10, 5, 23 },		// 114 bits per transform
			{ 10, 6, 22 },		// 114 bits per transform
			{ 10, 7, 21 },		// 114 bits per transform
			{ 10, 8, 20 },		// 114 bits per transform
			{ 10, 9, 19 },		// 114 bits per transform
			{ 10, 10, 18 },		// 114 bits per transform
			{ 10, 11, 17 },		// 114 bits per transform
			{ 10, 12, 16 },		// 114 bits per transform
			{ 10, 13, 15 },		// 114 bits per transform
			{ 10, 14, 14 },		// 114 bits per transform
			{ 10, 15, 13 },		// 114 bits per transform
			{ 10, 16, 12 },		// 114 bits per transform
			{ 10, 17, 11 },		// 114 bits per transform
			{ 10, 18, 10 },		// 114 bits per transform
			{ 10, 19, 9 },		// 114 bits per transform
			{ 10, 20, 8 },		// 114 bits per transform
			{ 10, 21, 7 },		// 114 bits per transform
			{ 10, 22, 6 },		// 114 bits per transform
			{ 10, 23, 5 },		// 114 bits per transform
			{ 11, 4, 23 },		// 114 bits per transform
			{ 11, 5, 22 },		// 114 bits per transform
			{ 11, 6, 21 },		// 114 bits per transform
			{ 11, 7, 20 },		// 114 bits per transform
			{ 11, 8, 19 },		// 114 bits per transform
			{ 11, 9, 18 },		// 114 bits per transform
			{ 11, 10, 17 },		// 114 bits per transform
			{ 11, 11, 16 },		// 114 bits per transform
			{ 11, 12, 15 },		// 114 bits per transform
			{ 11, 13, 14 },		// 114 bits per transform
			{ 11, 14, 13 },		// 114 bits per transform
			{ 11, 15, 12 },		// 114 bits per transform
			{ 11, 16, 11 },		// 114 bits per transform
			{ 11, 17, 10 },		// 114 bits per transform
			{ 11, 18, 9 },		// 114 bits per transform
			{ 11, 19, 8 },		// 114 bits per transform
			{ 11, 20, 7 },		// 114 bits per transform
			{ 11, 21, 6 },		// 114 bits per transform
			{ 11, 22, 5 },		// 114 bits per transform
			{ 11, 23, 4 },		// 114 bits per transform
			{ 12, 3, 23 },		// 114 bits per transform
			{ 12, 4, 22 },		// 114 bits per transform
			{ 12, 5, 21 },		// 114 bits per transform
			{ 12, 6, 20 },		// 114 bits per transform
			{ 12, 7, 19 },		// 114 bits per transform
			{ 12, 8, 18 },		// 114 bits per transform
			{ 12, 9, 17 },		// 114 bits per transform
			{ 12, 10, 16 },		// 114 bits per transform
			{ 12, 11, 15 },		// 114 bits per transform
			{ 12, 12, 14 },		// 114 bits per transform
			{ 12, 13, 13 },		// 114 bits per transform
			{ 12, 14, 12 },		// 114 bits per transform
			{ 12, 15, 11 },		// 114 bits per transform
			{ 12, 16, 10 },		// 114 bits per transform
			{ 12, 17, 9 },		// 114 bits per transform
			{ 12, 18, 8 },		// 114 bits per transform
			{ 12, 19, 7 },		// 114 bits per transform
			{ 12, 20, 6 },		// 114 bits per transform
			{ 12, 21, 5 },		// 114 bits per transform
			{ 12, 22, 4 },		// 114 bits per transform
			{ 12, 23, 3 },		// 114 bits per transform
			{ 13, 2, 23 },		// 114 bits per transform
			{ 13, 3, 22 },		// 114 bits per transform
			{ 13, 4, 21 },		// 114 bits per transform
			{ 13, 5, 20 },		// 114 bits per transform
			{ 13, 6, 19 },		// 114 bits per transform
			{ 13, 7, 18 },		// 114 bits per transform
			{ 13, 8, 17 },		// 114 bits per transform
			{ 13, 9, 16 },		// 114 bits per transform
			{ 13, 10, 15 },		// 114 bits per transform
			{ 13, 11, 14 },		// 114 bits per transform
			{ 13, 12, 13 },		// 114 bits per transform
			{ 13, 13, 12 },		// 114 bits per transform
			{ 13, 14, 11 },		// 114 bits per transform
			{ 13, 15, 10 },		// 114 bits per transform
			{ 13, 16, 9 },		// 114 bits per transform
			{ 13, 17, 8 },		// 114 bits per transform
			{ 13, 18, 7 },		// 114 bits per transform
			{ 13, 19, 6 },		// 114 bits per transform
			{ 13, 20, 5 },		// 114 bits per transform
			{ 13, 21, 4 },		// 114 bits per transform
			{ 13, 22, 3 },		// 114 bits per transform
			{ 13, 23, 2 },		// 114 bits per transform
			{ 14, 1, 23 },		// 114 bits per transform
			{ 14, 2, 22 },		// 114 bits per transform
			{ 14, 3, 21 },		// 114 bits per transform
			{ 14, 4, 20 },		// 114 bits per transform
			{ 14, 5, 19 },		// 114 bits per transform
			{ 14, 6, 18 },		// 114 bits per transform
			{ 14, 7, 17 },		// 114 bits per transform
			{ 14, 8, 16 },		// 114 bits per transform
			{ 14, 9, 15 },		// 114 bits per transform
			{ 14, 10, 14 },		// 114 bits per transform
			{ 14, 11, 13 },		// 114 bits per transform
			{ 14, 12, 12 },		// 114 bits per transform
			{ 14, 13, 11 },		// 114 bits per transform
			{ 14, 14, 10 },		// 114 bits per transform
			{ 14, 15, 9 },		// 114 bits per transform
			{ 14, 16, 8 },		// 114 bits per transform
			{ 14, 17, 7 },		// 114 bits per transform
			{ 14, 18, 6 },		// 114 bits per transform
			{ 14, 19, 5 },		// 114 bits per transform
			{ 14, 20, 4 },		// 114 bits per transform
			{ 14, 21, 3 },		// 114 bits per transform
			{ 14, 22, 2 },		// 114 bits per transform
			{ 14, 23, 1 },		// 114 bits per transform
			{ 15, 0, 23 },		// 114 bits per transform
			{ 15, 1, 22 },		// 114 bits per transform
			{ 15, 2, 21 },		// 114 bits per transform
			{ 15, 3, 20 },		// 114 bits per transform
			{ 15, 4, 19 },		// 114 bits per transform
			{ 15, 5, 18 },		// 114 bits per transform
			{ 15, 6, 17 },		// 114 bits per transform
			{ 15, 7, 16 },		// 114 bits per transform
			{ 15, 8, 15 },		// 114 bits per transform
			{ 15, 9, 14 },		// 114 bits per transform
			{ 15, 10, 13 },		// 114 bits per transform
			{ 15, 11, 12 },		// 114 bits per transform
			{ 15, 12, 11 },		// 114 bits per transform
			{ 15, 13, 10 },		// 114 bits per transform
			{ 15, 14, 9 },		// 114 bits per transform
			{ 15, 15, 8 },		// 114 bits per transform
			{ 15, 16, 7 },		// 114 bits per transform
			{ 15, 17, 6 },		// 114 bits per transform
			{ 15, 18, 5 },		// 114 bits per transform
			{ 15, 19, 4 },		// 114 bits per transform
			{ 15, 20, 3 },		// 114 bits per transform
			{ 15, 21, 2 },		// 114 bits per transform
			{ 15, 22, 1 },		// 114 bits per transform
			{ 15, 23, 0 },		// 114 bits per transform
			{ 16, 0, 22 },		// 114 bits per transform
			{ 16, 1, 21 },		// 114 bits per transform
			{ 16, 2, 20 },		// 114 bits per transform
			{ 16, 3, 19 },		// 114 bits per transform
			{ 16, 4, 18 },		// 114 bits per transform
			{ 16, 5, 17 },		// 114 bits per transform
			{ 16, 6, 16 },		// 114 bits per transform
			{ 16, 7, 15 },		// 114 bits per transform
			{ 16, 8, 14 },		// 114 bits per transform
			{ 16, 9, 13 },		// 114 bits per transform
			{ 16, 10, 12 },		// 114 bits per transform
			{ 16, 11, 11 },		// 114 bits per transform
			{ 16, 12, 10 },		// 114 bits per transform
			{ 16, 13, 9 },		// 114 bits per transform
			{ 16, 14, 8 },		// 114 bits per transform
			{ 16, 15, 7 },		// 114 bits per transform
			{ 16, 16, 6 },		// 114 bits per transform
			{ 16, 17, 5 },		// 114 bits per transform
			{ 16, 18, 4 },		// 114 bits per transform
			{ 16, 19, 3 },		// 114 bits per transform
			{ 16, 20, 2 },		// 114 bits per transform
			{ 16, 21, 1 },		// 114 bits per transform
			{ 16, 22, 0 },		// 114 bits per transform
			{ 17, 0, 21 },		// 114 bits per transform
			{ 17, 1, 20 },		// 114 bits per transform
			{ 17, 2, 19 },		// 114 bits per transform
			{ 17, 3, 18 },		// 114 bits per transform
			{ 17, 4, 17 },		// 114 bits per transform
			{ 17, 5, 16 },		// 114 bits per transform
			{ 17, 6, 15 },		// 114 bits per transform
			{ 17, 7, 14 },		// 114 bits per transform
			{ 17, 8, 13 },		// 114 bits per transform
			{ 17, 9, 12 },		// 114 bits per transform
			{ 17, 10, 11 },		// 114 bits per transform
			{ 17, 11, 10 },		// 114 bits per transform
			{ 17, 12, 9 },		// 114 bits per transform
			{ 17, 13, 8 },		// 114 bits per transform
			{ 17, 14, 7 },		// 114 bits per transform
			{ 17, 15, 6 },		// 114 bits per transform
			{ 17, 16, 5 },		// 114 bits per transform
			{ 17, 17, 4 },		// 114 bits per transform
			{ 17, 18, 3 },		// 114 bits per transform
			{ 17, 19, 2 },		// 114 bits per transform
			{ 17, 20, 1 },		// 114 bits per transform
			{ 17, 21, 0 },		// 114 bits per transform
			{ 18, 0, 20 },		// 114 bits per transform
			{ 18, 1, 19 },		// 114 bits per transform
			{ 18, 2, 18 },		// 114 bits per transform
			{ 18, 3, 17 },		// 114 bits per transform
			{ 18, 4, 16 },		// 114 bits per transform
			{ 18, 5, 15 },		// 114 bits per transform
			{ 18, 6, 14 },		// 114 bits per transform
			{ 18, 7, 13 },		// 114 bits per transform
			{ 18, 8, 12 },		// 114 bits per transform
			{ 18, 9, 11 },		// 114 bits per transform
			{ 18, 10, 10 },		// 114 bits per transform
			{ 18, 11, 9 },		// 114 bits per transform
			{ 18, 12, 8 },		// 114 bits per transform
			{ 18, 13, 7 },		// 114 bits per transform
			{ 18, 14, 6 },		// 114 bits per transform
			{ 18, 15, 5 },		// 114 bits per transform
			{ 18, 16, 4 },		// 114 bits per transform
			{ 18, 17, 3 },		// 114 bits per transform
			{ 18, 18, 2 },		// 114 bits per transform
			{ 18, 19, 1 },		// 114 bits per transform
			{ 18, 20, 0 },		// 114 bits per transform
			{ 19, 0, 19 },		// 114 bits per transform
			{ 19, 1, 18 },		// 114 bits per transform
			{ 19, 2, 17 },		// 114 bits per transform
			{ 19, 3, 16 },		// 114 bits per transform
			{ 19, 4, 15 },		// 114 bits per transform
			{ 19, 5, 14 },		// 114 bits per transform
			{ 19, 6, 13 },		// 114 bits per transform
			{ 19, 7, 12 },		// 114 bits per transform
			{ 19, 8, 11 },		// 114 bits per transform
			{ 19, 9, 10 },		// 114 bits per transform
			{ 19, 10, 9 },		// 114 bits per transform
			{ 19, 11, 8 },		// 114 bits per transform
			{ 19, 12, 7 },		// 114 bits per transform
			{ 19, 13, 6 },		// 114 bits per transform
			{ 19, 14, 5 },		// 114 bits per transform
			{ 19, 15, 4 },		// 114 bits per transform
			{ 19, 16, 3 },		// 114 bits per transform
			{ 19, 17, 2 },		// 114 bits per transform
			{ 19, 18, 1 },		// 114 bits per transform
			{ 19, 19, 0 },		// 114 bits per transform
			{ 20, 0, 18 },		// 114 bits per transform
			{ 20, 1, 17 },		// 114 bits per transform
			{ 20, 2, 16 },		// 114 bits per transform
			{ 20, 3, 15 },		// 114 bits per transform
			{ 20, 4, 14 },		// 114 bits per transform
			{ 20, 5, 13 },		// 114 bits per transform
			{ 20, 6, 12 },		// 114 bits per transform
			{ 20, 7, 11 },		// 114 bits per transform
			{ 20, 8, 10 },		// 114 bits per transform
			{ 20, 9, 9 },		// 114 bits per transform
			{ 20, 10, 8 },		// 114 bits per transform
			{ 20, 11, 7 },		// 114 bits per transform
			{ 20, 12, 6 },		// 114 bits per transform
			{ 20, 13, 5 },		// 114 bits per transform
			{ 20, 14, 4 },		// 114 bits per transform
			{ 20, 15, 3 },		// 114 bits per transform
			{ 20, 16, 2 },		// 114 bits per transform
			{ 20, 17, 1 },		// 114 bits per transform
			{ 20, 18, 0 },		// 114 bits per transform
			{ 21, 0, 17 },		// 114 bits per transform
			{ 21, 1, 16 },		// 114 bits per transform
			{ 21, 2, 15 },		// 114 bits per transform
			{ 21, 3, 14 },		// 114 bits per transform
			{ 21, 4, 13 },		// 114 bits per transform
			{ 21, 5, 12 },		// 114 bits per transform
			{ 21, 6, 11 },		// 114 bits per transform
			{ 21, 7, 10 },		// 114 bits per transform
			{ 21, 8, 9 },		// 114 bits per transform
			{ 21, 9, 8 },		// 114 bits per transform
			{ 21, 10, 7 },		// 114 bits per transform
			{ 21, 11, 6 },		// 114 bits per transform
			{ 21, 12, 5 },		// 114 bits per transform
			{ 21, 13, 4 },		// 114 bits per transform
			{ 21, 14, 3 },		// 114 bits per transform
			{ 21, 15, 2 },		// 114 bits per transform
			{ 21, 16, 1 },		// 114 bits per transform
			{ 21, 17, 0 },		// 114 bits per transform
			{ 22, 0, 16 },		// 114 bits per transform
			{ 22, 1, 15 },		// 114 bits per transform
			{ 22, 2, 14 },		// 114 bits per transform
			{ 22, 3, 13 },		// 114 bits per transform
			{ 22, 4, 12 },		// 114 bits per transform
			{ 22, 5, 11 },		// 114 bits per transform
			{ 22, 6, 10 },		// 114 bits per transform
			{ 22, 7, 9 },		// 114 bits per transform
			{ 22, 8, 8 },		// 114 bits per transform
			{ 22, 9, 7 },		// 114 bits per transform
			{ 22, 10, 6 },		// 114 bits per transform
			{ 22, 11, 5 },		// 114 bits per transform
			{ 22, 12, 4 },		// 114 bits per transform
			{ 22, 13, 3 },		// 114 bits per transform
			{ 22, 14, 2 },		// 114 bits per transform
			{ 22, 15, 1 },		// 114 bits per transform
			{ 22, 16, 0 },		// 114 bits per transform
			{ 23, 0, 15 },		// 114 bits per transform
			{ 23, 1, 14 },		// 114 bits per transform
			{ 23, 2, 13 },		// 114 bits per transform
			{ 23, 3, 12 },		// 114 bits per transform
			{ 23, 4, 11 },		// 114 bits per transform
			{ 23, 5, 10 },		// 114 bits per transform
			{ 23, 6, 9 },		// 114 bits per transform
			{ 23, 7, 8 },		// 114 bits per transform
			{ 23, 8, 7 },		// 114 bits per transform
			{ 23, 9, 6 },		// 114 bits per transform
			{ 23, 10, 5 },		// 114 bits per transform
			{ 23, 11, 4 },		// 114 bits per transform
			{ 23, 12, 3 },		// 114 bits per transform
			{ 23, 13, 2 },		// 114 bits per transform
			{ 23, 14, 1 },		// 114 bits per transform
			{ 23, 15, 0 },		// 114 bits per transform
			{ 24, 0, 6 },		// 114 bits per transform
			{ 24, 1, 5 },		// 114 bits per transform
			{ 24, 2, 4 },		// 114 bits per transform
			{ 24, 3, 3 },		// 114 bits per transform
			{ 24, 4, 2 },		// 114 bits per transform
			{ 24, 5, 1 },		// 114 bits per transform
			{ 24, 6, 0 },		// 114 bits per transform
			{ 0, 7, 24 },		// 117 bits per transform
			{ 0, 16, 23 },		// 117 bits per transform
			{ 0, 17, 22 },		// 117 bits per transform
			{ 0, 18, 21 },		// 117 bits per transform
			{ 0, 19, 20 },		// 117 bits per transform
			{ 0, 20, 19 },		// 117 bits per transform
			{ 0, 21, 18 },		// 117 bits per transform
			{ 0, 22, 17 },		// 117 bits per transform
			{ 0, 23, 16 },		// 117 bits per transform
			{ 0, 24, 7 },		// 117 bits per transform
			{ 1, 6, 24 },		// 117 bits per transform
			{ 1, 15, 23 },		// 117 bits per transform
			{ 1, 16, 22 },		// 117 bits per transform
			{ 1, 17, 21 },		// 117 bits per transform
			{ 1, 18, 20 },		// 117 bits per transform
			{ 1, 19, 19 },		// 117 bits per transform
			{ 1, 20, 18 },		// 117 bits per transform
			{ 1, 21, 17 },		// 117 bits per transform
			{ 1, 22, 16 },		// 117 bits per transform
			{ 1, 23, 15 },		// 117 bits per transform
			{ 1, 24, 6 },		// 117 bits per transform
			{ 2, 5, 24 },		// 117 bits per transform
			{ 2, 14, 23 },		// 117 bits per transform
			{ 2, 15, 22 },		// 117 bits per transform
			{ 2, 16, 21 },		// 117 bits per transform
			{ 2, 17, 20 },		// 117 bits per transform
			{ 2, 18, 19 },		// 117 bits per transform
			{ 2, 19, 18 },		// 117 bits per transform
			{ 2, 20, 17 },		// 117 bits per transform
			{ 2, 21, 16 },		// 117 bits per transform
			{ 2, 22, 15 },		// 117 bits per transform
			{ 2, 23, 14 },		// 117 bits per transform
			{ 2, 24, 5 },		// 117 bits per transform
			{ 3, 4, 24 },		// 117 bits per transform
			{ 3, 13, 23 },		// 117 bits per transform
			{ 3, 14, 22 },		// 117 bits per transform
			{ 3, 15, 21 },		// 117 bits per transform
			{ 3, 16, 20 },		// 117 bits per transform
			{ 3, 17, 19 },		// 117 bits per transform
			{ 3, 18, 18 },		// 117 bits per transform
			{ 3, 19, 17 },		// 117 bits per transform
			{ 3, 20, 16 },		// 117 bits per transform
			{ 3, 21, 15 },		// 117 bits per transform
			{ 3, 22, 14 },		// 117 bits per transform
			{ 3, 23, 13 },		// 117 bits per transform
			{ 3, 24, 4 },		// 117 bits per transform
			{ 4, 3, 24 },		// 117 bits per transform
			{ 4, 12, 23 },		// 117 bits per transform
			{ 4, 13, 22 },		// 117 bits per transform
			{ 4, 14, 21 },		// 117 bits per transform
			{ 4, 15, 20 },		// 117 bits per transform
			{ 4, 16, 19 },		// 117 bits per transform
			{ 4, 17, 18 },		// 117 bits per transform
			{ 4, 18, 17 },		// 117 bits per transform
			{ 4, 19, 16 },		// 117 bits per transform
			{ 4, 20, 15 },		// 117 bits per transform
			{ 4, 21, 14 },		// 117 bits per transform
			{ 4, 22, 13 },		// 117 bits per transform
			{ 4, 23, 12 },		// 117 bits per transform
			{ 4, 24, 3 },		// 117 bits per transform
			{ 5, 2, 24 },		// 117 bits per transform
			{ 5, 11, 23 },		// 117 bits per transform
			{ 5, 12, 22 },		// 117 bits per transform
			{ 5, 13, 21 },		// 117 bits per transform
			{ 5, 14, 20 },		// 117 bits per transform
			{ 5, 15, 19 },		// 117 bits per transform
			{ 5, 16, 18 },		// 117 bits per transform
			{ 5, 17, 17 },		// 117 bits per transform
			{ 5, 18, 16 },		// 117 bits per transform
			{ 5, 19, 15 },		// 117 bits per transform
			{ 5, 20, 14 },		// 117 bits per transform
			{ 5, 21, 13 },		// 117 bits per transform
			{ 5, 22, 12 },		// 117 bits per transform
			{ 5, 23, 11 },		// 117 bits per transform
			{ 5, 24, 2 },		// 117 bits per transform
			{ 6, 1, 24 },		// 117 bits per transform
			{ 6, 10, 23 },		// 117 bits per transform
			{ 6, 11, 22 },		// 117 bits per transform
			{ 6, 12, 21 },		// 117 bits per transform
			{ 6, 13, 20 },		// 117 bits per transform
			{ 6, 14, 19 },		// 117 bits per transform
			{ 6, 15, 18 },		// 117 bits per transform
			{ 6, 16, 17 },		// 117 bits per transform
			{ 6, 17, 16 },		// 117 bits per transform
			{ 6, 18, 15 },		// 117 bits per transform
			{ 6, 19, 14 },		// 117 bits per transform
			{ 6, 20, 13 },		// 117 bits per transform
			{ 6, 21, 12 },		// 117 bits per transform
			{ 6, 22, 11 },		// 117 bits per transform
			{ 6, 23, 10 },		// 117 bits per transform
			{ 6, 24, 1 },		// 117 bits per transform
			{ 7, 0, 24 },		// 117 bits per transform
			{ 7, 9, 23 },		// 117 bits per transform
			{ 7, 10, 22 },		// 117 bits per transform
			{ 7, 11, 21 },		// 117 bits per transform
			{ 7, 12, 20 },		// 117 bits per transform
			{ 7, 13, 19 },		// 117 bits per transform
			{ 7, 14, 18 },		// 117 bits per transform
			{ 7, 15, 17 },		// 117 bits per transform
			{ 7, 16, 16 },		// 117 bits per transform
			{ 7, 17, 15 },		// 117 bits per transform
			{ 7, 18, 14 },		// 117 bits per transform
			{ 7, 19, 13 },		// 117 bits per transform
			{ 7, 20, 12 },		// 117 bits per transform
			{ 7, 21, 11 },		// 117 bits per transform
			{ 7, 22, 10 },		// 117 bits per transform
			{ 7, 23, 9 },		// 117 bits per transform
			{ 7, 24, 0 },		// 117 bits per transform
			{ 8, 8, 23 },		// 117 bits per transform
			{ 8, 9, 22 },		// 117 bits per transform
			{ 8, 10, 21 },		// 117 bits per transform
			{ 8, 11, 20 },		// 117 bits per transform
			{ 8, 12, 19 },		// 117 bits per transform
			{ 8, 13, 18 },		// 117 bits per transform
			{ 8, 14, 17 },		// 117 bits per transform
			{ 8, 15, 16 },		// 117 bits per transform
			{ 8, 16, 15 },		// 117 bits per transform
			{ 8, 17, 14 },		// 117 bits per transform
			{ 8, 18, 13 },		// 117 bits per transform
			{ 8, 19, 12 },		// 117 bits per transform
			{ 8, 20, 11 },		// 117 bits per transform
			{ 8, 21, 10 },		// 117 bits per transform
			{ 8, 22, 9 },		// 117 bits per transform
			{ 8, 23, 8 },		// 117 bits per transform
			{ 9, 7, 23 },		// 117 bits per transform
			{ 9, 8, 22 },		// 117 bits per transform
			{ 9, 9, 21 },		// 117 bits per transform
			{ 9, 10, 20 },		// 117 bits per transform
			{ 9, 11, 19 },		// 117 bits per transform
			{ 9, 12, 18 },		// 117 bits per transform
			{ 9, 13, 17 },		// 117 bits per transform
			{ 9, 14, 16 },		// 117 bits per transform
			{ 9, 15, 15 },		// 117 bits per transform
			{ 9, 16, 14 },		// 117 bits per transform
			{ 9, 17, 13 },		// 117 bits per transform
			{ 9, 18, 12 },		// 117 bits per transform
			{ 9, 19, 11 },		// 117 bits per transform
			{ 9, 20, 10 },		// 117 bits per transform
			{ 9, 21, 9 },		// 117 bits per transform
			{ 9, 22, 8 },		// 117 bits per transform
			{ 9, 23, 7 },		// 117 bits per transform
			{ 10, 6, 23 },		// 117 bits per transform
			{ 10, 7, 22 },		// 117 bits per transform
			{ 10, 8, 21 },		// 117 bits per transform
			{ 10, 9, 20 },		// 117 bits per transform
			{ 10, 10, 19 },		// 117 bits per transform
			{ 10, 11, 18 },		// 117 bits per transform
			{ 10, 12, 17 },		// 117 bits per transform
			{ 10, 13, 16 },		// 117 bits per transform
			{ 10, 14, 15 },		// 117 bits per transform
			{ 10, 15, 14 },		// 117 bits per transform
			{ 10, 16, 13 },		// 117 bits per transform
			{ 10, 17, 12 },		// 117 bits per transform
			{ 10, 18, 11 },		// 117 bits per transform
			{ 10, 19, 10 },		// 117 bits per transform
			{ 10, 20, 9 },		// 117 bits per transform
			{ 10, 21, 8 },		// 117 bits per transform
			{ 10, 22, 7 },		// 117 bits per transform
			{ 10, 23, 6 },		// 117 bits per transform
			{ 11, 5, 23 },		// 117 bits per transform
			{ 11, 6, 22 },		// 117 bits per transform
			{ 11, 7, 21 },		// 117 bits per transform
			{ 11, 8, 20 },		// 117 bits per transform
			{ 11, 9, 19 },		// 117 bits per transform
			{ 11, 10, 18 },		// 117 bits per transform
			{ 11, 11, 17 },		// 117 bits per transform
			{ 11, 12, 16 },		// 117 bits per transform
			{ 11, 13, 15 },		// 117 bits per transform
			{ 11, 14, 14 },		// 117 bits per transform
			{ 11, 15, 13 },		// 117 bits per transform
			{ 11, 16, 12 },		// 117 bits per transform
			{ 11, 17, 11 },		// 117 bits per transform
			{ 11, 18, 10 },		// 117 bits per transform
			{ 11, 19, 9 },		// 117 bits per transform
			{ 11, 20, 8 },		// 117 bits per transform
			{ 11, 21, 7 },		// 117 bits per transform
			{ 11, 22, 6 },		// 117 bits per transform
			{ 11, 23, 5 },		// 117 bits per transform
			{ 12, 4, 23 },		// 117 bits per transform
			{ 12, 5, 22 },		// 117 bits per transform
			{ 12, 6, 21 },		// 117 bits per transform
			{ 12, 7, 20 },		// 117 bits per transform
			{ 12, 8, 19 },		// 117 bits per transform
			{ 12, 9, 18 },		// 117 bits per transform
			{ 12, 10, 17 },		// 117 bits per transform
			{ 12, 11, 16 },		// 117 bits per transform
			{ 12, 12, 15 },		// 117 bits per transform
			{ 12, 13, 14 },		// 117 bits per transform
			{ 12, 14, 13 },		// 117 bits per transform
			{ 12, 15, 12 },		// 117 bits per transform
			{ 12, 16, 11 },		// 117 bits per transform
			{ 12, 17, 10 },		// 117 bits per transform
			{ 12, 18, 9 },		// 117 bits per transform
			{ 12, 19, 8 },		// 117 bits per transform
			{ 12, 20, 7 },		// 117 bits per transform
			{ 12, 21, 6 },		// 117 bits per transform
			{ 12, 22, 5 },		// 117 bits per transform
			{ 12, 23, 4 },		// 117 bits per transform
			{ 13, 3, 23 },		// 117 bits per transform
			{ 13, 4, 22 },		// 117 bits per transform
			{ 13, 5, 21 },		// 117 bits per transform
			{ 13, 6, 20 },		// 117 bits per transform
			{ 13, 7, 19 },		// 117 bits per transform
			{ 13, 8, 18 },		// 117 bits per transform
			{ 13, 9, 17 },		// 117 bits per transform
			{ 13, 10, 16 },		// 117 bits per transform
			{ 13, 11, 15 },		// 117 bits per transform
			{ 13, 12, 14 },		// 117 bits per transform
			{ 13, 13, 13 },		// 117 bits per transform
			{ 13, 14, 12 },		// 117 bits per transform
			{ 13, 15, 11 },		// 117 bits per transform
			{ 13, 16, 10 },		// 117 bits per transform
			{ 13, 17, 9 },		// 117 bits per transform
			{ 13, 18, 8 },		// 117 bits per transform
			{ 13, 19, 7 },		// 117 bits per transform
			{ 13, 20, 6 },		// 117 bits per transform
			{ 13, 21, 5 },		// 117 bits per transform
			{ 13, 22, 4 },		// 117 bits per transform
			{ 13, 23, 3 },		// 117 bits per transform
			{ 14, 2, 23 },		// 117 bits per transform
			{ 14, 3, 22 },		// 117 bits per transform
			{ 14, 4, 21 },		// 117 bits per transform
			{ 14, 5, 20 },		// 117 bits per transform
			{ 14, 6, 19 },		// 117 bits per transform
			{ 14, 7, 18 },		// 117 bits per transform
			{ 14, 8, 17 },		// 117 bits per transform
			{ 14, 9, 16 },		// 117 bits per transform
			{ 14, 10, 15 },		// 117 bits per transform
			{ 14, 11, 14 },		// 117 bits per transform
			{ 14, 12, 13 },		// 117 bits per transform
			{ 14, 13, 12 },		// 117 bits per transform
			{ 14, 14, 11 },		// 117 bits per transform
			{ 14, 15, 10 },		// 117 bits per transform
			{ 14, 16, 9 },		// 117 bits per transform
			{ 14, 17, 8 },		// 117 bits per transform
			{ 14, 18, 7 },		// 117 bits per transform
			{ 14, 19, 6 },		// 117 bits per transform
			{ 14, 20, 5 },		// 117 bits per transform
			{ 14, 21, 4 },		// 117 bits per transform
			{ 14, 22, 3 },		// 117 bits per transform
			{ 14, 23, 2 },		// 117 bits per transform
			{ 15, 1, 23 },		// 117 bits per transform
			{ 15, 2, 22 },		// 117 bits per transform
			{ 15, 3, 21 },		// 117 bits per transform
			{ 15, 4, 20 },		// 117 bits per transform
			{ 15, 5, 19 },		// 117 bits per transform
			{ 15, 6, 18 },		// 117 bits per transform
			{ 15, 7, 17 },		// 117 bits per transform
			{ 15, 8, 16 },		// 117 bits per transform
			{ 15, 9, 15 },		// 117 bits per transform
			{ 15, 10, 14 },		// 117 bits per transform
			{ 15, 11, 13 },		// 117 bits per transform
			{ 15, 12, 12 },		// 117 bits per transform
			{ 15, 13, 11 },		// 117 bits per transform
			{ 15, 14, 10 },		// 117 bits per transform
			{ 15, 15, 9 },		// 117 bits per transform
			{ 15, 16, 8 },		// 117 bits per transform
			{ 15, 17, 7 },		// 117 bits per transform
			{ 15, 18, 6 },		// 117 bits per transform
			{ 15, 19, 5 },		// 117 bits per transform
			{ 15, 20, 4 },		// 117 bits per transform
			{ 15, 21, 3 },		// 117 bits per transform
			{ 15, 22, 2 },		// 117 bits per transform
			{ 15, 23, 1 },		// 117 bits per transform
			{ 16, 0, 23 },		// 117 bits per transform
			{ 16, 1, 22 },		// 117 bits per transform
			{ 16, 2, 21 },		// 117 bits per transform
			{ 16, 3, 20 },		// 117 bits per transform
			{ 16, 4, 19 },		// 117 bits per transform
			{ 16, 5, 18 },		// 117 bits per transform
			{ 16, 6, 17 },		// 117 bits per transform
			{ 16, 7, 16 },		// 117 bits per transform
			{ 16, 8, 15 },		// 117 bits per transform
			{ 16, 9, 14 },		// 117 bits per transform
			{ 16, 10, 13 },		// 117 bits per transform
			{ 16, 11, 12 },		// 117 bits per transform
			{ 16, 12, 11 },		// 117 bits per transform
			{ 16, 13, 10 },		// 117 bits per transform
			{ 16, 14, 9 },		// 117 bits per transform
			{ 16, 15, 8 },		// 117 bits per transform
			{ 16, 16, 7 },		// 117 bits per transform
			{ 16, 17, 6 },		// 117 bits per transform
			{ 16, 18, 5 },		// 117 bits per transform
			{ 16, 19, 4 },		// 117 bits per transform
			{ 16, 20, 3 },		// 117 bits per transform
			{ 16, 21, 2 },		// 117 bits per transform
			{ 16, 22, 1 },		// 117 bits per transform
			{ 16, 23, 0 },		// 117 bits per transform
			{ 17, 0, 22 },		// 117 bits per transform
			{ 17, 1, 21 },		// 117 bits per transform
			{ 17, 2, 20 },		// 117 bits per transform
			{ 17, 3, 19 },		// 117 bits per transform
			{ 17, 4, 18 },		// 117 bits per transform
			{ 17, 5, 17 },		// 117 bits per transform
			{ 17, 6, 16 },		// 117 bits per transform
			{ 17, 7, 15 },		// 117 bits per transform
			{ 17, 8, 14 },		// 117 bits per transform
			{ 17, 9, 13 },		// 117 bits per transform
			{ 17, 10, 12 },		// 117 bits per transform
			{ 17, 11, 11 },		// 117 bits per transform
			{ 17, 12, 10 },		// 117 bits per transform
			{ 17, 13, 9 },		// 117 bits per transform
			{ 17, 14, 8 },		// 117 bits per transform
			{ 17, 15, 7 },		// 117 bits per transform
			{ 17, 16, 6 },		// 117 bits per transform
			{ 17, 17, 5 },		// 117 bits per transform
			{ 17, 18, 4 },		// 117 bits per transform
			{ 17, 19, 3 },		// 117 bits per transform
			{ 17, 20, 2 },		// 117 bits per transform
			{ 17, 21, 1 },		// 117 bits per transform
			{ 17, 22, 0 },		// 117 bits per transform
			{ 18, 0, 21 },		// 117 bits per transform
			{ 18, 1, 20 },		// 117 bits per transform
			{ 18, 2, 19 },		// 117 bits per transform
			{ 18, 3, 18 },		// 117 bits per transform
			{ 18, 4, 17 },		// 117 bits per transform
			{ 18, 5, 16 },		// 117 bits per transform
			{ 18, 6, 15 },		// 117 bits per transform
			{ 18, 7, 14 },		// 117 bits per transform
			{ 18, 8, 13 },		// 117 bits per transform
			{ 18, 9, 12 },		// 117 bits per transform
			{ 18, 10, 11 },		// 117 bits per transform
			{ 18, 11, 10 },		// 117 bits per transform
			{ 18, 12, 9 },		// 117 bits per transform
			{ 18, 13, 8 },		// 117 bits per transform
			{ 18, 14, 7 },		// 117 bits per transform
			{ 18, 15, 6 },		// 117 bits per transform
			{ 18, 16, 5 },		// 117 bits per transform
			{ 18, 17, 4 },		// 117 bits per transform
			{ 18, 18, 3 },		// 117 bits per transform
			{ 18, 19, 2 },		// 117 bits per transform
			{ 18, 20, 1 },		// 117 bits per transform
			{ 18, 21, 0 },		// 117 bits per transform
			{ 19, 0, 20 },		// 117 bits per transform
			{ 19, 1, 19 },		// 117 bits per transform
			{ 19, 2, 18 },		// 117 bits per transform
			{ 19, 3, 17 },		// 117 bits per transform
			{ 19, 4, 16 },		// 117 bits per transform
			{ 19, 5, 15 },		// 117 bits per transform
			{ 19, 6, 14 },		// 117 bits per transform
			{ 19, 7, 13 },		// 117 bits per transform
			{ 19, 8, 12 },		// 117 bits per transform
			{ 19, 9, 11 },		// 117 bits per transform
			{ 19, 10, 10 },		// 117 bits per transform
			{ 19, 11, 9 },		// 117 bits per transform
			{ 19, 12, 8 },		// 117 bits per transform
			{ 19, 13, 7 },		// 117 bits per transform
			{ 19, 14, 6 },		// 117 bits per transform
			{ 19, 15, 5 },		// 117 bits per transform
			{ 19, 16, 4 },		// 117 bits per transform
			{ 19, 17, 3 },		// 117 bits per transform
			{ 19, 18, 2 },		// 117 bits per transform
			{ 19, 19, 1 },		// 117 bits per transform
			{ 19, 20, 0 },		// 117 bits per transform
			{ 20, 0, 19 },		// 117 bits per transform
			{ 20, 1, 18 },		// 117 bits per transform
			{ 20, 2, 17 },		// 117 bits per transform
			{ 20, 3, 16 },		// 117 bits per transform
			{ 20, 4, 15 },		// 117 bits per transform
			{ 20, 5, 14 },		// 117 bits per transform
			{ 20, 6, 13 },		// 117 bits per transform
			{ 20, 7, 12 },		// 117 bits per transform
			{ 20, 8, 11 },		// 117 bits per transform
			{ 20, 9, 10 },		// 117 bits per transform
			{ 20, 10, 9 },		// 117 bits per transform
			{ 20, 11, 8 },		// 117 bits per transform
			{ 20, 12, 7 },		// 117 bits per transform
			{ 20, 13, 6 },		// 117 bits per transform
			{ 20, 14, 5 },		// 117 bits per transform
			{ 20, 15, 4 },		// 117 bits per transform
			{ 20, 16, 3 },		// 117 bits per transform
			{ 20, 17, 2 },		// 117 bits per transform
			{ 20, 18, 1 },		// 117 bits per transform
			{ 20, 19, 0 },		// 117 bits per transform
			{ 21, 0, 18 },		// 117 bits per transform
			{ 21, 1, 17 },		// 117 bits per transform
			{ 21, 2, 16 },		// 117 bits per transform
			{ 21, 3, 15 },		// 117 bits per transform
			{ 21, 4, 14 },		// 117 bits per transform
			{ 21, 5, 13 },		// 117 bits per transform
			{ 21, 6, 12 },		// 117 bits per transform
			{ 21, 7, 11 },		// 117 bits per transform
			{ 21, 8, 10 },		// 117 bits per transform
			{ 21, 9, 9 },		// 117 bits per transform
			{ 21, 10, 8 },		// 117 bits per transform
			{ 21, 11, 7 },		// 117 bits per transform
			{ 21, 12, 6 },		// 117 bits per transform
			{ 21, 13, 5 },		// 117 bits per transform
			{ 21, 14, 4 },		// 117 bits per transform
			{ 21, 15, 3 },		// 117 bits per transform
			{ 21, 16, 2 },		// 117 bits per transform
			{ 21, 17, 1 },		// 117 bits per transform
			{ 21, 18, 0 },		// 117 bits per transform
			{ 22, 0, 17 },		// 117 bits per transform
			{ 22, 1, 16 },		// 117 bits per transform
			{ 22, 2, 15 },		// 117 bits per transform
			{ 22, 3, 14 },		// 117 bits per transform
			{ 22, 4, 13 },		// 117 bits per transform
			{ 22, 5, 12 },		// 117 bits per transform
			{ 22, 6, 11 },		// 117 bits per transform
			{ 22, 7, 10 },		// 117 bits per transform
			{ 22, 8, 9 },		// 117 bits per transform
			{ 22, 9, 8 },		// 117 bits per transform
			{ 22, 10, 7 },		// 117 bits per transform
			{ 22, 11, 6 },		// 117 bits per transform
			{ 22, 12, 5 },		// 117 bits per transform
			{ 22, 13, 4 },		// 117 bits per transform
			{ 22, 14, 3 },		// 117 bits per transform
			{ 22, 15, 2 },		// 117 bits per transform
			{ 22, 16, 1 },		// 117 bits per transform
			{ 22, 17, 0 },		// 117 bits per transform
			{ 23, 0, 16 },		// 117 bits per transform
			{ 23, 1, 15 },		// 117 bits per transform
			{ 23, 2, 14 },		// 117 bits per transform
			{ 23, 3, 13 },		// 117 bits per transform
			{ 23, 4, 12 },		// 117 bits per transform
			{ 23, 5, 11 },		// 117 bits per transform
			{ 23, 6, 10 },		// 117 bits per transform
			{ 23, 7, 9 },		// 117 bits per transform
			{ 23, 8, 8 },		// 117 bits per transform
			{ 23, 9, 7 },		// 117 bits per transform
			{ 23, 10, 6 },		// 117 bits per transform
			{ 23, 11, 5 },		// 117 bits per transform
			{ 23, 12, 4 },		// 117 bits per transform
			{ 23, 13, 3 },		// 117 bits per transform
			{ 23, 14, 2 },		// 117 bits per transform
			{ 23, 15, 1 },		// 117 bits per transform
			{ 23, 16, 0 },		// 117 bits per transform
			{ 24, 0, 7 },		// 117 bits per transform
			{ 24, 1, 6 },		// 117 bits per transform
			{ 24, 2, 5 },		// 117 bits per transform
			{ 24, 3, 4 },		// 117 bits per transform
			{ 24, 4, 3 },		// 117 bits per transform
			{ 24, 5, 2 },		// 117 bits per transform
			{ 24, 6, 1 },		// 117 bits per transform
			{ 24, 7, 0 },		// 117 bits per transform
			{ 0, 8, 24 },		// 120 bits per transform
			{ 0, 17, 23 },		// 120 bits per transform
			{ 0, 18, 22 },		// 120 bits per transform
			{ 0, 19, 21 },		// 120 bits per transform
			{ 0, 20, 20 },		// 120 bits per transform
			{ 0, 21, 19 },		// 120 bits per transform
			{ 0, 22, 18 },		// 120 bits per transform
			{ 0, 23, 17 },		// 120 bits per transform
			{ 0, 24, 8 },		// 120 bits per transform
			{ 1, 7, 24 },		// 120 bits per transform
			{ 1, 16, 23 },		// 120 bits per transform
			{ 1, 17, 22 },		// 120 bits per transform
			{ 1, 18, 21 },		// 120 bits per transform
			{ 1, 19, 20 },		// 120 bits per transform
			{ 1, 20, 19 },		// 120 bits per transform
			{ 1, 21, 18 },		// 120 bits per transform
			{ 1, 22, 17 },		// 120 bits per transform
			{ 1, 23, 16 },		// 120 bits per transform
			{ 1, 24, 7 },		// 120 bits per transform
			{ 2, 6, 24 },		// 120 bits per transform
			{ 2, 15, 23 },		// 120 bits per transform
			{ 2, 16, 22 },		// 120 bits per transform
			{ 2, 17, 21 },		// 120 bits per transform
			{ 2, 18, 20 },		// 120 bits per transform
			{ 2, 19, 19 },		// 120 bits per transform
			{ 2, 20, 18 },		// 120 bits per transform
			{ 2, 21, 17 },		// 120 bits per transform
			{ 2, 22, 16 },		// 120 bits per transform
			{ 2, 23, 15 },		// 120 bits per transform
			{ 2, 24, 6 },		// 120 bits per transform
			{ 3, 5, 24 },		// 120 bits per transform
			{ 3, 14, 23 },		// 120 bits per transform
			{ 3, 15, 22 },		// 120 bits per transform
			{ 3, 16, 21 },		// 120 bits per transform
			{ 3, 17, 20 },		// 120 bits per transform
			{ 3, 18, 19 },		// 120 bits per transform
			{ 3, 19, 18 },		// 120 bits per transform
			{ 3, 20, 17 },		// 120 bits per transform
			{ 3, 21, 16 },		// 120 bits per transform
			{ 3, 22, 15 },		// 120 bits per transform
			{ 3, 23, 14 },		// 120 bits per transform
			{ 3, 24, 5 },		// 120 bits per transform
			{ 4, 4, 24 },		// 120 bits per transform
			{ 4, 13, 23 },		// 120 bits per transform
			{ 4, 14, 22 },		// 120 bits per transform
			{ 4, 15, 21 },		// 120 bits per transform
			{ 4, 16, 20 },		// 120 bits per transform
			{ 4, 17, 19 },		// 120 bits per transform
			{ 4, 18, 18 },		// 120 bits per transform
			{ 4, 19, 17 },		// 120 bits per transform
			{ 4, 20, 16 },		// 120 bits per transform
			{ 4, 21, 15 },		// 120 bits per transform
			{ 4, 22, 14 },		// 120 bits per transform
			{ 4, 23, 13 },		// 120 bits per transform
			{ 4, 24, 4 },		// 120 bits per transform
			{ 5, 3, 24 },		// 120 bits per transform
			{ 5, 12, 23 },		// 120 bits per transform
			{ 5, 13, 22 },		// 120 bits per transform
			{ 5, 14, 21 },		// 120 bits per transform
			{ 5, 15, 20 },		// 120 bits per transform
			{ 5, 16, 19 },		// 120 bits per transform
			{ 5, 17, 18 },		// 120 bits per transform
			{ 5, 18, 17 },		// 120 bits per transform
			{ 5, 19, 16 },		// 120 bits per transform
			{ 5, 20, 15 },		// 120 bits per transform
			{ 5, 21, 14 },		// 120 bits per transform
			{ 5, 22, 13 },		// 120 bits per transform
			{ 5, 23, 12 },		// 120 bits per transform
			{ 5, 24, 3 },		// 120 bits per transform
			{ 6, 2, 24 },		// 120 bits per transform
			{ 6, 11, 23 },		// 120 bits per transform
			{ 6, 12, 22 },		// 120 bits per transform
			{ 6, 13, 21 },		// 120 bits per transform
			{ 6, 14, 20 },		// 120 bits per transform
			{ 6, 15, 19 },		// 120 bits per transform
			{ 6, 16, 18 },		// 120 bits per transform
			{ 6, 17, 17 },		// 120 bits per transform
			{ 6, 18, 16 },		// 120 bits per transform
			{ 6, 19, 15 },		// 120 bits per transform
			{ 6, 20, 14 },		// 120 bits per transform
			{ 6, 21, 13 },		// 120 bits per transform
			{ 6, 22, 12 },		// 120 bits per transform
			{ 6, 23, 11 },		// 120 bits per transform
			{ 6, 24, 2 },		// 120 bits per transform
			{ 7, 1, 24 },		// 120 bits per transform
			{ 7, 10, 23 },		// 120 bits per transform
			{ 7, 11, 22 },		// 120 bits per transform
			{ 7, 12, 21 },		// 120 bits per transform
			{ 7, 13, 20 },		// 120 bits per transform
			{ 7, 14, 19 },		// 120 bits per transform
			{ 7, 15, 18 },		// 120 bits per transform
			{ 7, 16, 17 },		// 120 bits per transform
			{ 7, 17, 16 },		// 120 bits per transform
			{ 7, 18, 15 },		// 120 bits per transform
			{ 7, 19, 14 },		// 120 bits per transform
			{ 7, 20, 13 },		// 120 bits per transform
			{ 7, 21, 12 },		// 120 bits per transform
			{ 7, 22, 11 },		// 120 bits per transform
			{ 7, 23, 10 },		// 120 bits per transform
			{ 7, 24, 1 },		// 120 bits per transform
			{ 8, 0, 24 },		// 120 bits per transform
			{ 8, 9, 23 },		// 120 bits per transform
			{ 8, 10, 22 },		// 120 bits per transform
			{ 8, 11, 21 },		// 120 bits per transform
			{ 8, 12, 20 },		// 120 bits per transform
			{ 8, 13, 19 },		// 120 bits per transform
			{ 8, 14, 18 },		// 120 bits per transform
			{ 8, 15, 17 },		// 120 bits per transform
			{ 8, 16, 16 },		// 120 bits per transform
			{ 8, 17, 15 },		// 120 bits per transform
			{ 8, 18, 14 },		// 120 bits per transform
			{ 8, 19, 13 },		// 120 bits per transform
			{ 8, 20, 12 },		// 120 bits per transform
			{ 8, 21, 11 },		// 120 bits per transform
			{ 8, 22, 10 },		// 120 bits per transform
			{ 8, 23, 9 },		// 120 bits per transform
			{ 8, 24, 0 },		// 120 bits per transform
			{ 9, 8, 23 },		// 120 bits per transform
			{ 9, 9, 22 },		// 120 bits per transform
			{ 9, 10, 21 },		// 120 bits per transform
			{ 9, 11, 20 },		// 120 bits per transform
			{ 9, 12, 19 },		// 120 bits per transform
			{ 9, 13, 18 },		// 120 bits per transform
			{ 9, 14, 17 },		// 120 bits per transform
			{ 9, 15, 16 },		// 120 bits per transform
			{ 9, 16, 15 },		// 120 bits per transform
			{ 9, 17, 14 },		// 120 bits per transform
			{ 9, 18, 13 },		// 120 bits per transform
			{ 9, 19, 12 },		// 120 bits per transform
			{ 9, 20, 11 },		// 120 bits per transform
			{ 9, 21, 10 },		// 120 bits per transform
			{ 9, 22, 9 },		// 120 bits per transform
			{ 9, 23, 8 },		// 120 bits per transform
			{ 10, 7, 23 },		// 120 bits per transform
			{ 10, 8, 22 },		// 120 bits per transform
			{ 10, 9, 21 },		// 120 bits per transform
			{ 10, 10, 20 },		// 120 bits per transform
			{ 10, 11, 19 },		// 120 bits per transform
			{ 10, 12, 18 },		// 120 bits per transform
			{ 10, 13, 17 },		// 120 bits per transform
			{ 10, 14, 16 },		// 120 bits per transform
			{ 10, 15, 15 },		// 120 bits per transform
			{ 10, 16, 14 },		// 120 bits per transform
			{ 10, 17, 13 },		// 120 bits per transform
			{ 10, 18, 12 },		// 120 bits per transform
			{ 10, 19, 11 },		// 120 bits per transform
			{ 10, 20, 10 },		// 120 bits per transform
			{ 10, 21, 9 },		// 120 bits per transform
			{ 10, 22, 8 },		// 120 bits per transform
			{ 10, 23, 7 },		// 120 bits per transform
			{ 11, 6, 23 },		// 120 bits per transform
			{ 11, 7, 22 },		// 120 bits per transform
			{ 11, 8, 21 },		// 120 bits per transform
			{ 11, 9, 20 },		// 120 bits per transform
			{ 11, 10, 19 },		// 120 bits per transform
			{ 11, 11, 18 },		// 120 bits per transform
			{ 11, 12, 17 },		// 120 bits per transform
			{ 11, 13, 16 },		// 120 bits per transform
			{ 11, 14, 15 },		// 120 bits per transform
			{ 11, 15, 14 },		// 120 bits per transform
			{ 11, 16, 13 },		// 120 bits per transform
			{ 11, 17, 12 },		// 120 bits per transform
			{ 11, 18, 11 },		// 120 bits per transform
			{ 11, 19, 10 },		// 120 bits per transform
			{ 11, 20, 9 },		// 120 bits per transform
			{ 11, 21, 8 },		// 120 bits per transform
			{ 11, 22, 7 },		// 120 bits per transform
			{ 11, 23, 6 },		// 120 bits per transform
			{ 12, 5, 23 },		// 120 bits per transform
			{ 12, 6, 22 },		// 120 bits per transform
			{ 12, 7, 21 },		// 120 bits per transform
			{ 12, 8, 20 },		// 120 bits per transform
			{ 12, 9, 19 },		// 120 bits per transform
			{ 12, 10, 18 },		// 120 bits per transform
			{ 12, 11, 17 },		// 120 bits per transform
			{ 12, 12, 16 },		// 120 bits per transform
			{ 12, 13, 15 },		// 120 bits per transform
			{ 12, 14, 14 },		// 120 bits per transform
			{ 12, 15, 13 },		// 120 bits per transform
			{ 12, 16, 12 },		// 120 bits per transform
			{ 12, 17, 11 },		// 120 bits per transform
			{ 12, 18, 10 },		// 120 bits per transform
			{ 12, 19, 9 },		// 120 bits per transform
			{ 12, 20, 8 },		// 120 bits per transform
			{ 12, 21, 7 },		// 120 bits per transform
			{ 12, 22, 6 },		// 120 bits per transform
			{ 12, 23, 5 },		// 120 bits per transform
			{ 13, 4, 23 },		// 120 bits per transform
			{ 13, 5, 22 },		// 120 bits per transform
			{ 13, 6, 21 },		// 120 bits per transform
			{ 13, 7, 20 },		// 120 bits per transform
			{ 13, 8, 19 },		// 120 bits per transform
			{ 13, 9, 18 },		// 120 bits per transform
			{ 13, 10, 17 },		// 120 bits per transform
			{ 13, 11, 16 },		// 120 bits per transform
			{ 13, 12, 15 },		// 120 bits per transform
			{ 13, 13, 14 },		// 120 bits per transform
			{ 13, 14, 13 },		// 120 bits per transform
			{ 13, 15, 12 },		// 120 bits per transform
			{ 13, 16, 11 },		// 120 bits per transform
			{ 13, 17, 10 },		// 120 bits per transform
			{ 13, 18, 9 },		// 120 bits per transform
			{ 13, 19, 8 },		// 120 bits per transform
			{ 13, 20, 7 },		// 120 bits per transform
			{ 13, 21, 6 },		// 120 bits per transform
			{ 13, 22, 5 },		// 120 bits per transform
			{ 13, 23, 4 },		// 120 bits per transform
			{ 14, 3, 23 },		// 120 bits per transform
			{ 14, 4, 22 },		// 120 bits per transform
			{ 14, 5, 21 },		// 120 bits per transform
			{ 14, 6, 20 },		// 120 bits per transform
			{ 14, 7, 19 },		// 120 bits per transform
			{ 14, 8, 18 },		// 120 bits per transform
			{ 14, 9, 17 },		// 120 bits per transform
			{ 14, 10, 16 },		// 120 bits per transform
			{ 14, 11, 15 },		// 120 bits per transform
			{ 14, 12, 14 },		// 120 bits per transform
			{ 14, 13, 13 },		// 120 bits per transform
			{ 14, 14, 12 },		// 120 bits per transform
			{ 14, 15, 11 },		// 120 bits per transform
			{ 14, 16, 10 },		// 120 bits per transform
			{ 14, 17, 9 },		// 120 bits per transform
			{ 14, 18, 8 },		// 120 bits per transform
			{ 14, 19, 7 },		// 120 bits per transform
			{ 14, 20, 6 },		// 120 bits per transform
			{ 14, 21, 5 },		// 120 bits per transform
			{ 14, 22, 4 },		// 120 bits per transform
			{ 14, 23, 3 },		// 120 bits per transform
			{ 15, 2, 23 },		// 120 bits per transform
			{ 15, 3, 22 },		// 120 bits per transform
			{ 15, 4, 21 },		// 120 bits per transform
			{ 15, 5, 20 },		// 120 bits per transform
			{ 15, 6, 19 },		// 120 bits per transform
			{ 15, 7, 18 },		// 120 bits per transform
			{ 15, 8, 17 },		// 120 bits per transform
			{ 15, 9, 16 },		// 120 bits per transform
			{ 15, 10, 15 },		// 120 bits per transform
			{ 15, 11, 14 },		// 120 bits per transform
			{ 15, 12, 13 },		// 120 bits per transform
			{ 15, 13, 12 },		// 120 bits per transform
			{ 15, 14, 11 },		// 120 bits per transform
			{ 15, 15, 10 },		// 120 bits per transform
			{ 15, 16, 9 },		// 120 bits per transform
			{ 15, 17, 8 },		// 120 bits per transform
			{ 15, 18, 7 },		// 120 bits per transform
			{ 15, 19, 6 },		// 120 bits per transform
			{ 15, 20, 5 },		// 120 bits per transform
			{ 15, 21, 4 },		// 120 bits per transform
			{ 15, 22, 3 },		// 120 bits per transform
			{ 15, 23, 2 },		// 120 bits per transform
			{ 16, 1, 23 },		// 120 bits per transform
			{ 16, 2, 22 },		// 120 bits per transform
			{ 16, 3, 21 },		// 120 bits per transform
			{ 16, 4, 20 },		// 120 bits per transform
			{ 16, 5, 19 },		// 120 bits per transform
			{ 16, 6, 18 },		// 120 bits per transform
			{ 16, 7, 17 },		// 120 bits per transform
			{ 16, 8, 16 },		// 120 bits per transform
			{ 16, 9, 15 },		// 120 bits per transform
			{ 16, 10, 14 },		// 120 bits per transform
			{ 16, 11, 13 },		// 120 bits per transform
			{ 16, 12, 12 },		// 120 bits per transform
			{ 16, 13, 11 },		// 120 bits per transform
			{ 16, 14, 10 },		// 120 bits per transform
			{ 16, 15, 9 },		// 120 bits per transform
			{ 16, 16, 8 },		// 120 bits per transform
			{ 16, 17, 7 },		// 120 bits per transform
			{ 16, 18, 6 },		// 120 bits per transform
			{ 16, 19, 5 },		// 120 bits per transform
			{ 16, 20, 4 },		// 120 bits per transform
			{ 16, 21, 3 },		// 120 bits per transform
			{ 16, 22, 2 },		// 120 bits per transform
			{ 16, 23, 1 },		// 120 bits per transform
			{ 17, 0, 23 },		// 120 bits per transform
			{ 17, 1, 22 },		// 120 bits per transform
			{ 17, 2, 21 },		// 120 bits per transform
			{ 17, 3, 20 },		// 120 bits per transform
			{ 17, 4, 19 },		// 120 bits per transform
			{ 17, 5, 18 },		// 120 bits per transform
			{ 17, 6, 17 },		// 120 bits per transform
			{ 17, 7, 16 },		// 120 bits per transform
			{ 17, 8, 15 },		// 120 bits per transform
			{ 17, 9, 14 },		// 120 bits per transform
			{ 17, 10, 13 },		// 120 bits per transform
			{ 17, 11, 12 },		// 120 bits per transform
			{ 17, 12, 11 },		// 120 bits per transform
			{ 17, 13, 10 },		// 120 bits per transform
			{ 17, 14, 9 },		// 120 bits per transform
			{ 17, 15, 8 },		// 120 bits per transform
			{ 17, 16, 7 },		// 120 bits per transform
			{ 17, 17, 6 },		// 120 bits per transform
			{ 17, 18, 5 },		// 120 bits per transform
			{ 17, 19, 4 },		// 120 bits per transform
			{ 17, 20, 3 },		// 120 bits per transform
			{ 17, 21, 2 },		// 120 bits per transform
			{ 17, 22, 1 },		// 120 bits per transform
			{ 17, 23, 0 },		// 120 bits per transform
			{ 18, 0, 22 },		// 120 bits per transform
			{ 18, 1, 21 },		// 120 bits per transform
			{ 18, 2, 20 },		// 120 bits per transform
			{ 18, 3, 19 },		// 120 bits per transform
			{ 18, 4, 18 },		// 120 bits per transform
			{ 18, 5, 17 },		// 120 bits per transform
			{ 18, 6, 16 },		// 120 bits per transform
			{ 18, 7, 15 },		// 120 bits per transform
			{ 18, 8, 14 },		// 120 bits per transform
			{ 18, 9, 13 },		// 120 bits per transform
			{ 18, 10, 12 },		// 120 bits per transform
			{ 18, 11, 11 },		// 120 bits per transform
			{ 18, 12, 10 },		// 120 bits per transform
			{ 18, 13, 9 },		// 120 bits per transform
			{ 18, 14, 8 },		// 120 bits per transform
			{ 18, 15, 7 },		// 120 bits per transform
			{ 18, 16, 6 },		// 120 bits per transform
			{ 18, 17, 5 },		// 120 bits per transform
			{ 18, 18, 4 },		// 120 bits per transform
			{ 18, 19, 3 },		// 120 bits per transform
			{ 18, 20, 2 },		// 120 bits per transform
			{ 18, 21, 1 },		// 120 bits per transform
			{ 18, 22, 0 },		// 120 bits per transform
			{ 19, 0, 21 },		// 120 bits per transform
			{ 19, 1, 20 },		// 120 bits per transform
			{ 19, 2, 19 },		// 120 bits per transform
			{ 19, 3, 18 },		// 120 bits per transform
			{ 19, 4, 17 },		// 120 bits per transform
			{ 19, 5, 16 },		// 120 bits per transform
			{ 19, 6, 15 },		// 120 bits per transform
			{ 19, 7, 14 },		// 120 bits per transform
			{ 19, 8, 13 },		// 120 bits per transform
			{ 19, 9, 12 },		// 120 bits per transform
			{ 19, 10, 11 },		// 120 bits per transform
			{ 19, 11, 10 },		// 120 bits per transform
			{ 19, 12, 9 },		// 120 bits per transform
			{ 19, 13, 8 },		// 120 bits per transform
			{ 19, 14, 7 },		// 120 bits per transform
			{ 19, 15, 6 },		// 120 bits per transform
			{ 19, 16, 5 },		// 120 bits per transform
			{ 19, 17, 4 },		// 120 bits per transform
			{ 19, 18, 3 },		// 120 bits per transform
			{ 19, 19, 2 },		// 120 bits per transform
			{ 19, 20, 1 },		// 120 bits per transform
			{ 19, 21, 0 },		// 120 bits per transform
			{ 20, 0, 20 },		// 120 bits per transform
			{ 20, 1, 19 },		// 120 bits per transform
			{ 20, 2, 18 },		// 120 bits per transform
			{ 20, 3, 17 },		// 120 bits per transform
			{ 20, 4, 16 },		// 120 bits per transform
			{ 20, 5, 15 },		// 120 bits per transform
			{ 20, 6, 14 },		// 120 bits per transform
			{ 20, 7, 13 },		// 120 bits per transform
			{ 20, 8, 12 },		// 120 bits per transform
			{ 20, 9, 11 },		// 120 bits per transform
			{ 20, 10, 10 },		// 120 bits per transform
			{ 20, 11, 9 },		// 120 bits per transform
			{ 20, 12, 8 },		// 120 bits per transform
			{ 20, 13, 7 },		// 120 bits per transform
			{ 20, 14, 6 },		// 120 bits per transform
			{ 20, 15, 5 },		// 120 bits per transform
			{ 20, 16, 4 },		// 120 bits per transform
			{ 20, 17, 3 },		// 120 bits per transform
			{ 20, 18, 2 },		// 120 bits per transform
			{ 20, 19, 1 },		// 120 bits per transform
			{ 20, 20, 0 },		// 120 bits per transform
			{ 21, 0, 19 },		// 120 bits per transform
			{ 21, 1, 18 },		// 120 bits per transform
			{ 21, 2, 17 },		// 120 bits per transform
			{ 21, 3, 16 },		// 120 bits per transform
			{ 21, 4, 15 },		// 120 bits per transform
			{ 21, 5, 14 },		// 120 bits per transform
			{ 21, 6, 13 },		// 120 bits per transform
			{ 21, 7, 12 },		// 120 bits per transform
			{ 21, 8, 11 },		// 120 bits per transform
			{ 21, 9, 10 },		// 120 bits per transform
			{ 21, 10, 9 },		// 120 bits per transform
			{ 21, 11, 8 },		// 120 bits per transform
			{ 21, 12, 7 },		// 120 bits per transform
			{ 21, 13, 6 },		// 120 bits per transform
			{ 21, 14, 5 },		// 120 bits per transform
			{ 21, 15, 4 },		// 120 bits per transform
			{ 21, 16, 3 },		// 120 bits per transform
			{ 21, 17, 2 },		// 120 bits per transform
			{ 21, 18, 1 },		// 120 bits per transform
			{ 21, 19, 0 },		// 120 bits per transform
			{ 22, 0, 18 },		// 120 bits per transform
			{ 22, 1, 17 },		// 120 bits per transform
			{ 22, 2, 16 },		// 120 bits per transform
			{ 22, 3, 15 },		// 120 bits per transform
			{ 22, 4, 14 },		// 120 bits per transform
			{ 22, 5, 13 },		// 120 bits per transform
			{ 22, 6, 12 },		// 120 bits per transform
			{ 22, 7, 11 },		// 120 bits per transform
			{ 22, 8, 10 },		// 120 bits per transform
			{ 22, 9, 9 },		// 120 bits per transform
			{ 22, 10, 8 },		// 120 bits per transform
			{ 22, 11, 7 },		// 120 bits per transform
			{ 22, 12, 6 },		// 120 bits per transform
			{ 22, 13, 5 },		// 120 bits per transform
			{ 22, 14, 4 },		// 120 bits per transform
			{ 22, 15, 3 },		// 120 bits per transform
			{ 22, 16, 2 },		// 120 bits per transform
			{ 22, 17, 1 },		// 120 bits per transform
			{ 22, 18, 0 },		// 120 bits per transform
			{ 23, 0, 17 },		// 120 bits per transform
			{ 23, 1, 16 },		// 120 bits per transform
			{ 23, 2, 15 },		// 120 bits per transform
			{ 23, 3, 14 },		// 120 bits per transform
			{ 23, 4, 13 },		// 120 bits per transform
			{ 23, 5, 12 },		// 120 bits per transform
			{ 23, 6, 11 },		// 120 bits per transform
			{ 23, 7, 10 },		// 120 bits per transform
			{ 23, 8, 9 },		// 120 bits per transform
			{ 23, 9, 8 },		// 120 bits per transform
			{ 23, 10, 7 },		// 120 bits per transform
			{ 23, 11, 6 },		// 120 bits per transform
			{ 23, 12, 5 },		// 120 bits per transform
			{ 23, 13, 4 },		// 120 bits per transform
			{ 23, 14, 3 },		// 120 bits per transform
			{ 23, 15, 2 },		// 120 bits per transform
			{ 23, 16, 1 },		// 120 bits per transform
			{ 23, 17, 0 },		// 120 bits per transform
			{ 24, 0, 8 },		// 120 bits per transform
			{ 24, 1, 7 },		// 120 bits per transform
			{ 24, 2, 6 },		// 120 bits per transform
			{ 24, 3, 5 },		// 120 bits per transform
			{ 24, 4, 4 },		// 120 bits per transform
			{ 24, 5, 3 },		// 120 bits per transform
			{ 24, 6, 2 },		// 120 bits per transform
			{ 24, 7, 1 },		// 120 bits per transform
			{ 24, 8, 0 },		// 120 bits per transform
			{ 0, 9, 24 },		// 123 bits per transform
			{ 0, 18, 23 },		// 123 bits per transform
			{ 0, 19, 22 },		// 123 bits per transform
			{ 0, 20, 21 },		// 123 bits per transform
			{ 0, 21, 20 },		// 123 bits per transform
			{ 0, 22, 19 },		// 123 bits per transform
			{ 0, 23, 18 },		// 123 bits per transform
			{ 0, 24, 9 },		// 123 bits per transform
			{ 1, 8, 24 },		// 123 bits per transform
			{ 1, 17, 23 },		// 123 bits per transform
			{ 1, 18, 22 },		// 123 bits per transform
			{ 1, 19, 21 },		// 123 bits per transform
			{ 1, 20, 20 },		// 123 bits per transform
			{ 1, 21, 19 },		// 123 bits per transform
			{ 1, 22, 18 },		// 123 bits per transform
			{ 1, 23, 17 },		// 123 bits per transform
			{ 1, 24, 8 },		// 123 bits per transform
			{ 2, 7, 24 },		// 123 bits per transform
			{ 2, 16, 23 },		// 123 bits per transform
			{ 2, 17, 22 },		// 123 bits per transform
			{ 2, 18, 21 },		// 123 bits per transform
			{ 2, 19, 20 },		// 123 bits per transform
			{ 2, 20, 19 },		// 123 bits per transform
			{ 2, 21, 18 },		// 123 bits per transform
			{ 2, 22, 17 },		// 123 bits per transform
			{ 2, 23, 16 },		// 123 bits per transform
			{ 2, 24, 7 },		// 123 bits per transform
			{ 3, 6, 24 },		// 123 bits per transform
			{ 3, 15, 23 },		// 123 bits per transform
			{ 3, 16, 22 },		// 123 bits per transform
			{ 3, 17, 21 },		// 123 bits per transform
			{ 3, 18, 20 },		// 123 bits per transform
			{ 3, 19, 19 },		// 123 bits per transform
			{ 3, 20, 18 },		// 123 bits per transform
			{ 3, 21, 17 },		// 123 bits per transform
			{ 3, 22, 16 },		// 123 bits per transform
			{ 3, 23, 15 },		// 123 bits per transform
			{ 3, 24, 6 },		// 123 bits per transform
			{ 4, 5, 24 },		// 123 bits per transform
			{ 4, 14, 23 },		// 123 bits per transform
			{ 4, 15, 22 },		// 123 bits per transform
			{ 4, 16, 21 },		// 123 bits per transform
			{ 4, 17, 20 },		// 123 bits per transform
			{ 4, 18, 19 },		// 123 bits per transform
			{ 4, 19, 18 },		// 123 bits per transform
			{ 4, 20, 17 },		// 123 bits per transform
			{ 4, 21, 16 },		// 123 bits per transform
			{ 4, 22, 15 },		// 123 bits per transform
			{ 4, 23, 14 },		// 123 bits per transform
			{ 4, 24, 5 },		// 123 bits per transform
			{ 5, 4, 24 },		// 123 bits per transform
			{ 5, 13, 23 },		// 123 bits per transform
			{ 5, 14, 22 },		// 123 bits per transform
			{ 5, 15, 21 },		// 123 bits per transform
			{ 5, 16, 20 },		// 123 bits per transform
			{ 5, 17, 19 },		// 123 bits per transform
			{ 5, 18, 18 },		// 123 bits per transform
			{ 5, 19, 17 },		// 123 bits per transform
			{ 5, 20, 16 },		// 123 bits per transform
			{ 5, 21, 15 },		// 123 bits per transform
			{ 5, 22, 14 },		// 123 bits per transform
			{ 5, 23, 13 },		// 123 bits per transform
			{ 5, 24, 4 },		// 123 bits per transform
			{ 6, 3, 24 },		// 123 bits per transform
			{ 6, 12, 23 },		// 123 bits per transform
			{ 6, 13, 22 },		// 123 bits per transform
			{ 6, 14, 21 },		// 123 bits per transform
			{ 6, 15, 20 },		// 123 bits per transform
			{ 6, 16, 19 },		// 123 bits per transform
			{ 6, 17, 18 },		// 123 bits per transform
			{ 6, 18, 17 },		// 123 bits per transform
			{ 6, 19, 16 },		// 123 bits per transform
			{ 6, 20, 15 },		// 123 bits per transform
			{ 6, 21, 14 },		// 123 bits per transform
			{ 6, 22, 13 },		// 123 bits per transform
			{ 6, 23, 12 },		// 123 bits per transform
			{ 6, 24, 3 },		// 123 bits per transform
			{ 7, 2, 24 },		// 123 bits per transform
			{ 7, 11, 23 },		// 123 bits per transform
			{ 7, 12, 22 },		// 123 bits per transform
			{ 7, 13, 21 },		// 123 bits per transform
			{ 7, 14, 20 },		// 123 bits per transform
			{ 7, 15, 19 },		// 123 bits per transform
			{ 7, 16, 18 },		// 123 bits per transform
			{ 7, 17, 17 },		// 123 bits per transform
			{ 7, 18, 16 },		// 123 bits per transform
			{ 7, 19, 15 },		// 123 bits per transform
			{ 7, 20, 14 },		// 123 bits per transform
			{ 7, 21, 13 },		// 123 bits per transform
			{ 7, 22, 12 },		// 123 bits per transform
			{ 7, 23, 11 },		// 123 bits per transform
			{ 7, 24, 2 },		// 123 bits per transform
			{ 8, 1, 24 },		// 123 bits per transform
			{ 8, 10, 23 },		// 123 bits per transform
			{ 8, 11, 22 },		// 123 bits per transform
			{ 8, 12, 21 },		// 123 bits per transform
			{ 8, 13, 20 },		// 123 bits per transform
			{ 8, 14, 19 },		// 123 bits per transform
			{ 8, 15, 18 },		// 123 bits per transform
			{ 8, 16, 17 },		// 123 bits per transform
			{ 8, 17, 16 },		// 123 bits per transform
			{ 8, 18, 15 },		// 123 bits per transform
			{ 8, 19, 14 },		// 123 bits per transform
			{ 8, 20, 13 },		// 123 bits per transform
			{ 8, 21, 12 },		// 123 bits per transform
			{ 8, 22, 11 },		// 123 bits per transform
			{ 8, 23, 10 },		// 123 bits per transform
			{ 8, 24, 1 },		// 123 bits per transform
			{ 9, 0, 24 },		// 123 bits per transform
			{ 9, 9, 23 },		// 123 bits per transform
			{ 9, 10, 22 },		// 123 bits per transform
			{ 9, 11, 21 },		// 123 bits per transform
			{ 9, 12, 20 },		// 123 bits per transform
			{ 9, 13, 19 },		// 123 bits per transform
			{ 9, 14, 18 },		// 123 bits per transform
			{ 9, 15, 17 },		// 123 bits per transform
			{ 9, 16, 16 },		// 123 bits per transform
			{ 9, 17, 15 },		// 123 bits per transform
			{ 9, 18, 14 },		// 123 bits per transform
			{ 9, 19, 13 },		// 123 bits per transform
			{ 9, 20, 12 },		// 123 bits per transform
			{ 9, 21, 11 },		// 123 bits per transform
			{ 9, 22, 10 },		// 123 bits per transform
			{ 9, 23, 9 },		// 123 bits per transform
			{ 9, 24, 0 },		// 123 bits per transform
			{ 10, 8, 23 },		// 123 bits per transform
			{ 10, 9, 22 },		// 123 bits per transform
			{ 10, 10, 21 },		// 123 bits per transform
			{ 10, 11, 20 },		// 123 bits per transform
			{ 10, 12, 19 },		// 123 bits per transform
			{ 10, 13, 18 },		// 123 bits per transform
			{ 10, 14, 17 },		// 123 bits per transform
			{ 10, 15, 16 },		// 123 bits per transform
			{ 10, 16, 15 },		// 123 bits per transform
			{ 10, 17, 14 },		// 123 bits per transform
			{ 10, 18, 13 },		// 123 bits per transform
			{ 10, 19, 12 },		// 123 bits per transform
			{ 10, 20, 11 },		// 123 bits per transform
			{ 10, 21, 10 },		// 123 bits per transform
			{ 10, 22, 9 },		// 123 bits per transform
			{ 10, 23, 8 },		// 123 bits per transform
			{ 11, 7, 23 },		// 123 bits per transform
			{ 11, 8, 22 },		// 123 bits per transform
			{ 11, 9, 21 },		// 123 bits per transform
			{ 11, 10, 20 },		// 123 bits per transform
			{ 11, 11, 19 },		// 123 bits per transform
			{ 11, 12, 18 },		// 123 bits per transform
			{ 11, 13, 17 },		// 123 bits per transform
			{ 11, 14, 16 },		// 123 bits per transform
			{ 11, 15, 15 },		// 123 bits per transform
			{ 11, 16, 14 },		// 123 bits per transform
			{ 11, 17, 13 },		// 123 bits per transform
			{ 11, 18, 12 },		// 123 bits per transform
			{ 11, 19, 11 },		// 123 bits per transform
			{ 11, 20, 10 },		// 123 bits per transform
			{ 11, 21, 9 },		// 123 bits per transform
			{ 11, 22, 8 },		// 123 bits per transform
			{ 11, 23, 7 },		// 123 bits per transform
			{ 12, 6, 23 },		// 123 bits per transform
			{ 12, 7, 22 },		// 123 bits per transform
			{ 12, 8, 21 },		// 123 bits per transform
			{ 12, 9, 20 },		// 123 bits per transform
			{ 12, 10, 19 },		// 123 bits per transform
			{ 12, 11, 18 },		// 123 bits per transform
			{ 12, 12, 17 },		// 123 bits per transform
			{ 12, 13, 16 },		// 123 bits per transform
			{ 12, 14, 15 },		// 123 bits per transform
			{ 12, 15, 14 },		// 123 bits per transform
			{ 12, 16, 13 },		// 123 bits per transform
			{ 12, 17, 12 },		// 123 bits per transform
			{ 12, 18, 11 },		// 123 bits per transform
			{ 12, 19, 10 },		// 123 bits per transform
			{ 12, 20, 9 },		// 123 bits per transform
			{ 12, 21, 8 },		// 123 bits per transform
			{ 12, 22, 7 },		// 123 bits per transform
			{ 12, 23, 6 },		// 123 bits per transform
			{ 13, 5, 23 },		// 123 bits per transform
			{ 13, 6, 22 },		// 123 bits per transform
			{ 13, 7, 21 },		// 123 bits per transform
			{ 13, 8, 20 },		// 123 bits per transform
			{ 13, 9, 19 },		// 123 bits per transform
			{ 13, 10, 18 },		// 123 bits per transform
			{ 13, 11, 17 },		// 123 bits per transform
			{ 13, 12, 16 },		// 123 bits per transform
			{ 13, 13, 15 },		// 123 bits per transform
			{ 13, 14, 14 },		// 123 bits per transform
			{ 13, 15, 13 },		// 123 bits per transform
			{ 13, 16, 12 },		// 123 bits per transform
			{ 13, 17, 11 },		// 123 bits per transform
			{ 13, 18, 10 },		// 123 bits per transform
			{ 13, 19, 9 },		// 123 bits per transform
			{ 13, 20, 8 },		// 123 bits per transform
			{ 13, 21, 7 },		// 123 bits per transform
			{ 13, 22, 6 },		// 123 bits per transform
			{ 13, 23, 5 },		// 123 bits per transform
			{ 14, 4, 23 },		// 123 bits per transform
			{ 14, 5, 22 },		// 123 bits per transform
			{ 14, 6, 21 },		// 123 bits per transform
			{ 14, 7, 20 },		// 123 bits per transform
			{ 14, 8, 19 },		// 123 bits per transform
			{ 14, 9, 18 },		// 123 bits per transform
			{ 14, 10, 17 },		// 123 bits per transform
			{ 14, 11, 16 },		// 123 bits per transform
			{ 14, 12, 15 },		// 123 bits per transform
			{ 14, 13, 14 },		// 123 bits per transform
			{ 14, 14, 13 },		// 123 bits per transform
			{ 14, 15, 12 },		// 123 bits per transform
			{ 14, 16, 11 },		// 123 bits per transform
			{ 14, 17, 10 },		// 123 bits per transform
			{ 14, 18, 9 },		// 123 bits per transform
			{ 14, 19, 8 },		// 123 bits per transform
			{ 14, 20, 7 },		// 123 bits per transform
			{ 14, 21, 6 },		// 123 bits per transform
			{ 14, 22, 5 },		// 123 bits per transform
			{ 14, 23, 4 },		// 123 bits per transform
			{ 15, 3, 23 },		// 123 bits per transform
			{ 15, 4, 22 },		// 123 bits per transform
			{ 15, 5, 21 },		// 123 bits per transform
			{ 15, 6, 20 },		// 123 bits per transform
			{ 15, 7, 19 },		// 123 bits per transform
			{ 15, 8, 18 },		// 123 bits per transform
			{ 15, 9, 17 },		// 123 bits per transform
			{ 15, 10, 16 },		// 123 bits per transform
			{ 15, 11, 15 },		// 123 bits per transform
			{ 15, 12, 14 },		// 123 bits per transform
			{ 15, 13, 13 },		// 123 bits per transform
			{ 15, 14, 12 },		// 123 bits per transform
			{ 15, 15, 11 },		// 123 bits per transform
			{ 15, 16, 10 },		// 123 bits per transform
			{ 15, 17, 9 },		// 123 bits per transform
			{ 15, 18, 8 },		// 123 bits per transform
			{ 15, 19, 7 },		// 123 bits per transform
			{ 15, 20, 6 },		// 123 bits per transform
			{ 15, 21, 5 },		// 123 bits per transform
			{ 15, 22, 4 },		// 123 bits per transform
			{ 15, 23, 3 },		// 123 bits per transform
			{ 16, 2, 23 },		// 123 bits per transform
			{ 16, 3, 22 },		// 123 bits per transform
			{ 16, 4, 21 },		// 123 bits per transform
			{ 16, 5, 20 },		// 123 bits per transform
			{ 16, 6, 19 },		// 123 bits per transform
			{ 16, 7, 18 },		// 123 bits per transform
			{ 16, 8, 17 },		// 123 bits per transform
			{ 16, 9, 16 },		// 123 bits per transform
			{ 16, 10, 15 },		// 123 bits per transform
			{ 16, 11, 14 },		// 123 bits per transform
			{ 16, 12, 13 },		// 123 bits per transform
			{ 16, 13, 12 },		// 123 bits per transform
			{ 16, 14, 11 },		// 123 bits per transform
			{ 16, 15, 10 },		// 123 bits per transform
			{ 16, 16, 9 },		// 123 bits per transform
			{ 16, 17, 8 },		// 123 bits per transform
			{ 16, 18, 7 },		// 123 bits per transform
			{ 16, 19, 6 },		// 123 bits per transform
			{ 16, 20, 5 },		// 123 bits per transform
			{ 16, 21, 4 },		// 123 bits per transform
			{ 16, 22, 3 },		// 123 bits per transform
			{ 16, 23, 2 },		// 123 bits per transform
			{ 17, 1, 23 },		// 123 bits per transform
			{ 17, 2, 22 },		// 123 bits per transform
			{ 17, 3, 21 },		// 123 bits per transform
			{ 17, 4, 20 },		// 123 bits per transform
			{ 17, 5, 19 },		// 123 bits per transform
			{ 17, 6, 18 },		// 123 bits per transform
			{ 17, 7, 17 },		// 123 bits per transform
			{ 17, 8, 16 },		// 123 bits per transform
			{ 17, 9, 15 },		// 123 bits per transform
			{ 17, 10, 14 },		// 123 bits per transform
			{ 17, 11, 13 },		// 123 bits per transform
			{ 17, 12, 12 },		// 123 bits per transform
			{ 17, 13, 11 },		// 123 bits per transform
			{ 17, 14, 10 },		// 123 bits per transform
			{ 17, 15, 9 },		// 123 bits per transform
			{ 17, 16, 8 },		// 123 bits per transform
			{ 17, 17, 7 },		// 123 bits per transform
			{ 17, 18, 6 },		// 123 bits per transform
			{ 17, 19, 5 },		// 123 bits per transform
			{ 17, 20, 4 },		// 123 bits per transform
			{ 17, 21, 3 },		// 123 bits per transform
			{ 17, 22, 2 },		// 123 bits per transform
			{ 17, 23, 1 },		// 123 bits per transform
			{ 18, 0, 23 },		// 123 bits per transform
			{ 18, 1, 22 },		// 123 bits per transform
			{ 18, 2, 21 },		// 123 bits per transform
			{ 18, 3, 20 },		// 123 bits per transform
			{ 18, 4, 19 },		// 123 bits per transform
			{ 18, 5, 18 },		// 123 bits per transform
			{ 18, 6, 17 },		// 123 bits per transform
			{ 18, 7, 16 },		// 123 bits per transform
			{ 18, 8, 15 },		// 123 bits per transform
			{ 18, 9, 14 },		// 123 bits per transform
			{ 18, 10, 13 },		// 123 bits per transform
			{ 18, 11, 12 },		// 123 bits per transform
			{ 18, 12, 11 },		// 123 bits per transform
			{ 18, 13, 10 },		// 123 bits per transform
			{ 18, 14, 9 },		// 123 bits per transform
			{ 18, 15, 8 },		// 123 bits per transform
			{ 18, 16, 7 },		// 123 bits per transform
			{ 18, 17, 6 },		// 123 bits per transform
			{ 18, 18, 5 },		// 123 bits per transform
			{ 18, 19, 4 },		// 123 bits per transform
			{ 18, 20, 3 },		// 123 bits per transform
			{ 18, 21, 2 },		// 123 bits per transform
			{ 18, 22, 1 },		// 123 bits per transform
			{ 18, 23, 0 },		// 123 bits per transform
			{ 19, 0, 22 },		// 123 bits per transform
			{ 19, 1, 21 },		// 123 bits per transform
			{ 19, 2, 20 },		// 123 bits per transform
			{ 19, 3, 19 },		// 123 bits per transform
			{ 19, 4, 18 },		// 123 bits per transform
			{ 19, 5, 17 },		// 123 bits per transform
			{ 19, 6, 16 },		// 123 bits per transform
			{ 19, 7, 15 },		// 123 bits per transform
			{ 19, 8, 14 },		// 123 bits per transform
			{ 19, 9, 13 },		// 123 bits per transform
			{ 19, 10, 12 },		// 123 bits per transform
			{ 19, 11, 11 },		// 123 bits per transform
			{ 19, 12, 10 },		// 123 bits per transform
			{ 19, 13, 9 },		// 123 bits per transform
			{ 19, 14, 8 },		// 123 bits per transform
			{ 19, 15, 7 },		// 123 bits per transform
			{ 19, 16, 6 },		// 123 bits per transform
			{ 19, 17, 5 },		// 123 bits per transform
			{ 19, 18, 4 },		// 123 bits per transform
			{ 19, 19, 3 },		// 123 bits per transform
			{ 19, 20, 2 },		// 123 bits per transform
			{ 19, 21, 1 },		// 123 bits per transform
			{ 19, 22, 0 },		// 123 bits per transform
			{ 20, 0, 21 },		// 123 bits per transform
			{ 20, 1, 20 },		// 123 bits per transform
			{ 20, 2, 19 },		// 123 bits per transform
			{ 20, 3, 18 },		// 123 bits per transform
			{ 20, 4, 17 },		// 123 bits per transform
			{ 20, 5, 16 },		// 123 bits per transform
			{ 20, 6, 15 },		// 123 bits per transform
			{ 20, 7, 14 },		// 123 bits per transform
			{ 20, 8, 13 },		// 123 bits per transform
			{ 20, 9, 12 },		// 123 bits per transform
			{ 20, 10, 11 },		// 123 bits per transform
			{ 20, 11, 10 },		// 123 bits per transform
			{ 20, 12, 9 },		// 123 bits per transform
			{ 20, 13, 8 },		// 123 bits per transform
			{ 20, 14, 7 },		// 123 bits per transform
			{ 20, 15, 6 },		// 123 bits per transform
			{ 20, 16, 5 },		// 123 bits per transform
			{ 20, 17, 4 },		// 123 bits per transform
			{ 20, 18, 3 },		// 123 bits per transform
			{ 20, 19, 2 },		// 123 bits per transform
			{ 20, 20, 1 },		// 123 bits per transform
			{ 20, 21, 0 },		// 123 bits per transform
			{ 21, 0, 20 },		// 123 bits per transform
			{ 21, 1, 19 },		// 123 bits per transform
			{ 21, 2, 18 },		// 123 bits per transform
			{ 21, 3, 17 },		// 123 bits per transform
			{ 21, 4, 16 },		// 123 bits per transform
			{ 21, 5, 15 },		// 123 bits per transform
			{ 21, 6, 14 },		// 123 bits per transform
			{ 21, 7, 13 },		// 123 bits per transform
			{ 21, 8, 12 },		// 123 bits per transform
			{ 21, 9, 11 },		// 123 bits per transform
			{ 21, 10, 10 },		// 123 bits per transform
			{ 21, 11, 9 },		// 123 bits per transform
			{ 21, 12, 8 },		// 123 bits per transform
			{ 21, 13, 7 },		// 123 bits per transform
			{ 21, 14, 6 },		// 123 bits per transform
			{ 21, 15, 5 },		// 123 bits per transform
			{ 21, 16, 4 },		// 123 bits per transform
			{ 21, 17, 3 },		// 123 bits per transform
			{ 21, 18, 2 },		// 123 bits per transform
			{ 21, 19, 1 },		// 123 bits per transform
			{ 21, 20, 0 },		// 123 bits per transform
			{ 22, 0, 19 },		// 123 bits per transform
			{ 22, 1, 18 },		// 123 bits per transform
			{ 22, 2, 17 },		// 123 bits per transform
			{ 22, 3, 16 },		// 123 bits per transform
			{ 22, 4, 15 },		// 123 bits per transform
			{ 22, 5, 14 },		// 123 bits per transform
			{ 22, 6, 13 },		// 123 bits per transform
			{ 22, 7, 12 },		// 123 bits per transform
			{ 22, 8, 11 },		// 123 bits per transform
			{ 22, 9, 10 },		// 123 bits per transform
			{ 22, 10, 9 },		// 123 bits per transform
			{ 22, 11, 8 },		// 123 bits per transform
			{ 22, 12, 7 },		// 123 bits per transform
			{ 22, 13, 6 },		// 123 bits per transform
			{ 22, 14, 5 },		// 123 bits per transform
			{ 22, 15, 4 },		// 123 bits per transform
			{ 22, 16, 3 },		// 123 bits per transform
			{ 22, 17, 2 },		// 123 bits per transform
			{ 22, 18, 1 },		// 123 bits per transform
			{ 22, 19, 0 },		// 123 bits per transform
			{ 23, 0, 18 },		// 123 bits per transform
			{ 23, 1, 17 },		// 123 bits per transform
			{ 23, 2, 16 },		// 123 bits per transform
			{ 23, 3, 15 },		// 123 bits per transform
			{ 23, 4, 14 },		// 123 bits per transform
			{ 23, 5, 13 },		// 123 bits per transform
			{ 23, 6, 12 },		// 123 bits per transform
			{ 23, 7, 11 },		// 123 bits per transform
			{ 23, 8, 10 },		// 123 bits per transform
			{ 23, 9, 9 },		// 123 bits per transform
			{ 23, 10, 8 },		// 123 bits per transform
			{ 23, 11, 7 },		// 123 bits per transform
			{ 23, 12, 6 },		// 123 bits per transform
			{ 23, 13, 5 },		// 123 bits per transform
			{ 23, 14, 4 },		// 123 bits per transform
			{ 23, 15, 3 },		// 123 bits per transform
			{ 23, 16, 2 },		// 123 bits per transform
			{ 23, 17, 1 },		// 123 bits per transform
			{ 23, 18, 0 },		// 123 bits per transform
			{ 24, 0, 9 },		// 123 bits per transform
			{ 24, 1, 8 },		// 123 bits per transform
			{ 24, 2, 7 },		// 123 bits per transform
			{ 24, 3, 6 },		// 123 bits per transform
			{ 24, 4, 5 },		// 123 bits per transform
			{ 24, 5, 4 },		// 123 bits per transform
			{ 24, 6, 3 },		// 123 bits per transform
			{ 24, 7, 2 },		// 123 bits per transform
			{ 24, 8, 1 },		// 123 bits per transform
			{ 24, 9, 0 },		// 123 bits per transform
			{ 0, 10, 24 },		// 126 bits per transform
			{ 0, 19, 23 },		// 126 bits per transform
			{ 0, 20, 22 },		// 126 bits per transform
			{ 0, 21, 21 },		// 126 bits per transform
			{ 0, 22, 20 },		// 126 bits per transform
			{ 0, 23, 19 },		// 126 bits per transform
			{ 0, 24, 10 },		// 126 bits per transform
			{ 1, 9, 24 },		// 126 bits per transform
			{ 1, 18, 23 },		// 126 bits per transform
			{ 1, 19, 22 },		// 126 bits per transform
			{ 1, 20, 21 },		// 126 bits per transform
			{ 1, 21, 20 },		// 126 bits per transform
			{ 1, 22, 19 },		// 126 bits per transform
			{ 1, 23, 18 },		// 126 bits per transform
			{ 1, 24, 9 },		// 126 bits per transform
			{ 2, 8, 24 },		// 126 bits per transform
			{ 2, 17, 23 },		// 126 bits per transform
			{ 2, 18, 22 },		// 126 bits per transform
			{ 2, 19, 21 },		// 126 bits per transform
			{ 2, 20, 20 },		// 126 bits per transform
			{ 2, 21, 19 },		// 126 bits per transform
			{ 2, 22, 18 },		// 126 bits per transform
			{ 2, 23, 17 },		// 126 bits per transform
			{ 2, 24, 8 },		// 126 bits per transform
			{ 3, 7, 24 },		// 126 bits per transform
			{ 3, 16, 23 },		// 126 bits per transform
			{ 3, 17, 22 },		// 126 bits per transform
			{ 3, 18, 21 },		// 126 bits per transform
			{ 3, 19, 20 },		// 126 bits per transform
			{ 3, 20, 19 },		// 126 bits per transform
			{ 3, 21, 18 },		// 126 bits per transform
			{ 3, 22, 17 },		// 126 bits per transform
			{ 3, 23, 16 },		// 126 bits per transform
			{ 3, 24, 7 },		// 126 bits per transform
			{ 4, 6, 24 },		// 126 bits per transform
			{ 4, 15, 23 },		// 126 bits per transform
			{ 4, 16, 22 },		// 126 bits per transform
			{ 4, 17, 21 },		// 126 bits per transform
			{ 4, 18, 20 },		// 126 bits per transform
			{ 4, 19, 19 },		// 126 bits per transform
			{ 4, 20, 18 },		// 126 bits per transform
			{ 4, 21, 17 },		// 126 bits per transform
			{ 4, 22, 16 },		// 126 bits per transform
			{ 4, 23, 15 },		// 126 bits per transform
			{ 4, 24, 6 },		// 126 bits per transform
			{ 5, 5, 24 },		// 126 bits per transform
			{ 5, 14, 23 },		// 126 bits per transform
			{ 5, 15, 22 },		// 126 bits per transform
			{ 5, 16, 21 },		// 126 bits per transform
			{ 5, 17, 20 },		// 126 bits per transform
			{ 5, 18, 19 },		// 126 bits per transform
			{ 5, 19, 18 },		// 126 bits per transform
			{ 5, 20, 17 },		// 126 bits per transform
			{ 5, 21, 16 },		// 126 bits per transform
			{ 5, 22, 15 },		// 126 bits per transform
			{ 5, 23, 14 },		// 126 bits per transform
			{ 5, 24, 5 },		// 126 bits per transform
			{ 6, 4, 24 },		// 126 bits per transform
			{ 6, 13, 23 },		// 126 bits per transform
			{ 6, 14, 22 },		// 126 bits per transform
			{ 6, 15, 21 },		// 126 bits per transform
			{ 6, 16, 20 },		// 126 bits per transform
			{ 6, 17, 19 },		// 126 bits per transform
			{ 6, 18, 18 },		// 126 bits per transform
			{ 6, 19, 17 },		// 126 bits per transform
			{ 6, 20, 16 },		// 126 bits per transform
			{ 6, 21, 15 },		// 126 bits per transform
			{ 6, 22, 14 },		// 126 bits per transform
			{ 6, 23, 13 },		// 126 bits per transform
			{ 6, 24, 4 },		// 126 bits per transform
			{ 7, 3, 24 },		// 126 bits per transform
			{ 7, 12, 23 },		// 126 bits per transform
			{ 7, 13, 22 },		// 126 bits per transform
			{ 7, 14, 21 },		// 126 bits per transform
			{ 7, 15, 20 },		// 126 bits per transform
			{ 7, 16, 19 },		// 126 bits per transform
			{ 7, 17, 18 },		// 126 bits per transform
			{ 7, 18, 17 },		// 126 bits per transform
			{ 7, 19, 16 },		// 126 bits per transform
			{ 7, 20, 15 },		// 126 bits per transform
			{ 7, 21, 14 },		// 126 bits per transform
			{ 7, 22, 13 },		// 126 bits per transform
			{ 7, 23, 12 },		// 126 bits per transform
			{ 7, 24, 3 },		// 126 bits per transform
			{ 8, 2, 24 },		// 126 bits per transform
			{ 8, 11, 23 },		// 126 bits per transform
			{ 8, 12, 22 },		// 126 bits per transform
			{ 8, 13, 21 },		// 126 bits per transform
			{ 8, 14, 20 },		// 126 bits per transform
			{ 8, 15, 19 },		// 126 bits per transform
			{ 8, 16, 18 },		// 126 bits per transform
			{ 8, 17, 17 },		// 126 bits per transform
			{ 8, 18, 16 },		// 126 bits per transform
			{ 8, 19, 15 },		// 126 bits per transform
			{ 8, 20, 14 },		// 126 bits per transform
			{ 8, 21, 13 },		// 126 bits per transform
			{ 8, 22, 12 },		// 126 bits per transform
			{ 8, 23, 11 },		// 126 bits per transform
			{ 8, 24, 2 },		// 126 bits per transform
			{ 9, 1, 24 },		// 126 bits per transform
			{ 9, 10, 23 },		// 126 bits per transform
			{ 9, 11, 22 },		// 126 bits per transform
			{ 9, 12, 21 },		// 126 bits per transform
			{ 9, 13, 20 },		// 126 bits per transform
			{ 9, 14, 19 },		// 126 bits per transform
			{ 9, 15, 18 },		// 126 bits per transform
			{ 9, 16, 17 },		// 126 bits per transform
			{ 9, 17, 16 },		// 126 bits per transform
			{ 9, 18, 15 },		// 126 bits per transform
			{ 9, 19, 14 },		// 126 bits per transform
			{ 9, 20, 13 },		// 126 bits per transform
			{ 9, 21, 12 },		// 126 bits per transform
			{ 9, 22, 11 },		// 126 bits per transform
			{ 9, 23, 10 },		// 126 bits per transform
			{ 9, 24, 1 },		// 126 bits per transform
			{ 10, 0, 24 },		// 126 bits per transform
			{ 10, 9, 23 },		// 126 bits per transform
			{ 10, 10, 22 },		// 126 bits per transform
			{ 10, 11, 21 },		// 126 bits per transform
			{ 10, 12, 20 },		// 126 bits per transform
			{ 10, 13, 19 },		// 126 bits per transform
			{ 10, 14, 18 },		// 126 bits per transform
			{ 10, 15, 17 },		// 126 bits per transform
			{ 10, 16, 16 },		// 126 bits per transform
			{ 10, 17, 15 },		// 126 bits per transform
			{ 10, 18, 14 },		// 126 bits per transform
			{ 10, 19, 13 },		// 126 bits per transform
			{ 10, 20, 12 },		// 126 bits per transform
			{ 10, 21, 11 },		// 126 bits per transform
			{ 10, 22, 10 },		// 126 bits per transform
			{ 10, 23, 9 },		// 126 bits per transform
			{ 10, 24, 0 },		// 126 bits per transform
			{ 11, 8, 23 },		// 126 bits per transform
			{ 11, 9, 22 },		// 126 bits per transform
			{ 11, 10, 21 },		// 126 bits per transform
			{ 11, 11, 20 },		// 126 bits per transform
			{ 11, 12, 19 },		// 126 bits per transform
			{ 11, 13, 18 },		// 126 bits per transform
			{ 11, 14, 17 },		// 126 bits per transform
			{ 11, 15, 16 },		// 126 bits per transform
			{ 11, 16, 15 },		// 126 bits per transform
			{ 11, 17, 14 },		// 126 bits per transform
			{ 11, 18, 13 },		// 126 bits per transform
			{ 11, 19, 12 },		// 126 bits per transform
			{ 11, 20, 11 },		// 126 bits per transform
			{ 11, 21, 10 },		// 126 bits per transform
			{ 11, 22, 9 },		// 126 bits per transform
			{ 11, 23, 8 },		// 126 bits per transform
			{ 12, 7, 23 },		// 126 bits per transform
			{ 12, 8, 22 },		// 126 bits per transform
			{ 12, 9, 21 },		// 126 bits per transform
			{ 12, 10, 20 },		// 126 bits per transform
			{ 12, 11, 19 },		// 126 bits per transform
			{ 12, 12, 18 },		// 126 bits per transform
			{ 12, 13, 17 },		// 126 bits per transform
			{ 12, 14, 16 },		// 126 bits per transform
			{ 12, 15, 15 },		// 126 bits per transform
			{ 12, 16, 14 },		// 126 bits per transform
			{ 12, 17, 13 },		// 126 bits per transform
			{ 12, 18, 12 },		// 126 bits per transform
			{ 12, 19, 11 },		// 126 bits per transform
			{ 12, 20, 10 },		// 126 bits per transform
			{ 12, 21, 9 },		// 126 bits per transform
			{ 12, 22, 8 },		// 126 bits per transform
			{ 12, 23, 7 },		// 126 bits per transform
			{ 13, 6, 23 },		// 126 bits per transform
			{ 13, 7, 22 },		// 126 bits per transform
			{ 13, 8, 21 },		// 126 bits per transform
			{ 13, 9, 20 },		// 126 bits per transform
			{ 13, 10, 19 },		// 126 bits per transform
			{ 13, 11, 18 },		// 126 bits per transform
			{ 13, 12, 17 },		// 126 bits per transform
			{ 13, 13, 16 },		// 126 bits per transform
			{ 13, 14, 15 },		// 126 bits per transform
			{ 13, 15, 14 },		// 126 bits per transform
			{ 13, 16, 13 },		// 126 bits per transform
			{ 13, 17, 12 },		// 126 bits per transform
			{ 13, 18, 11 },		// 126 bits per transform
			{ 13, 19, 10 },		// 126 bits per transform
			{ 13, 20, 9 },		// 126 bits per transform
			{ 13, 21, 8 },		// 126 bits per transform
			{ 13, 22, 7 },		// 126 bits per transform
			{ 13, 23, 6 },		// 126 bits per transform
			{ 14, 5, 23 },		// 126 bits per transform
			{ 14, 6, 22 },		// 126 bits per transform
			{ 14, 7, 21 },		// 126 bits per transform
			{ 14, 8, 20 },		// 126 bits per transform
			{ 14, 9, 19 },		// 126 bits per transform
			{ 14, 10, 18 },		// 126 bits per transform
			{ 14, 11, 17 },		// 126 bits per transform
			{ 14, 12, 16 },		// 126 bits per transform
			{ 14, 13, 15 },		// 126 bits per transform
			{ 14, 14, 14 },		// 126 bits per transform
			{ 14, 15, 13 },		// 126 bits per transform
			{ 14, 16, 12 },		// 126 bits per transform
			{ 14, 17, 11 },		// 126 bits per transform
			{ 14, 18, 10 },		// 126 bits per transform
			{ 14, 19, 9 },		// 126 bits per transform
			{ 14, 20, 8 },		// 126 bits per transform
			{ 14, 21, 7 },		// 126 bits per transform
			{ 14, 22, 6 },		// 126 bits per transform
			{ 14, 23, 5 },		// 126 bits per transform
			{ 15, 4, 23 },		// 126 bits per transform
			{ 15, 5, 22 },		// 126 bits per transform
			{ 15, 6, 21 },		// 126 bits per transform
			{ 15, 7, 20 },		// 126 bits per transform
			{ 15, 8, 19 },		// 126 bits per transform
			{ 15, 9, 18 },		// 126 bits per transform
			{ 15, 10, 17 },		// 126 bits per transform
			{ 15, 11, 16 },		// 126 bits per transform
			{ 15, 12, 15 },		// 126 bits per transform
			{ 15, 13, 14 },		// 126 bits per transform
			{ 15, 14, 13 },		// 126 bits per transform
			{ 15, 15, 12 },		// 126 bits per transform
			{ 15, 16, 11 },		// 126 bits per transform
			{ 15, 17, 10 },		// 126 bits per transform
			{ 15, 18, 9 },		// 126 bits per transform
			{ 15, 19, 8 },		// 126 bits per transform
			{ 15, 20, 7 },		// 126 bits per transform
			{ 15, 21, 6 },		// 126 bits per transform
			{ 15, 22, 5 },		// 126 bits per transform
			{ 15, 23, 4 },		// 126 bits per transform
			{ 16, 3, 23 },		// 126 bits per transform
			{ 16, 4, 22 },		// 126 bits per transform
			{ 16, 5, 21 },		// 126 bits per transform
			{ 16, 6, 20 },		// 126 bits per transform
			{ 16, 7, 19 },		// 126 bits per transform
			{ 16, 8, 18 },		// 126 bits per transform
			{ 16, 9, 17 },		// 126 bits per transform
			{ 16, 10, 16 },		// 126 bits per transform
			{ 16, 11, 15 },		// 126 bits per transform
			{ 16, 12, 14 },		// 126 bits per transform
			{ 16, 13, 13 },		// 126 bits per transform
			{ 16, 14, 12 },		// 126 bits per transform
			{ 16, 15, 11 },		// 126 bits per transform
			{ 16, 16, 10 },		// 126 bits per transform
			{ 16, 17, 9 },		// 126 bits per transform
			{ 16, 18, 8 },		// 126 bits per transform
			{ 16, 19, 7 },		// 126 bits per transform
			{ 16, 20, 6 },		// 126 bits per transform
			{ 16, 21, 5 },		// 126 bits per transform
			{ 16, 22, 4 },		// 126 bits per transform
			{ 16, 23, 3 },		// 126 bits per transform
			{ 17, 2, 23 },		// 126 bits per transform
			{ 17, 3, 22 },		// 126 bits per transform
			{ 17, 4, 21 },		// 126 bits per transform
			{ 17, 5, 20 },		// 126 bits per transform
			{ 17, 6, 19 },		// 126 bits per transform
			{ 17, 7, 18 },		// 126 bits per transform
			{ 17, 8, 17 },		// 126 bits per transform
			{ 17, 9, 16 },		// 126 bits per transform
			{ 17, 10, 15 },		// 126 bits per transform
			{ 17, 11, 14 },		// 126 bits per transform
			{ 17, 12, 13 },		// 126 bits per transform
			{ 17, 13, 12 },		// 126 bits per transform
			{ 17, 14, 11 },		// 126 bits per transform
			{ 17, 15, 10 },		// 126 bits per transform
			{ 17, 16, 9 },		// 126 bits per transform
			{ 17, 17, 8 },		// 126 bits per transform
			{ 17, 18, 7 },		// 126 bits per transform
			{ 17, 19, 6 },		// 126 bits per transform
			{ 17, 20, 5 },		// 126 bits per transform
			{ 17, 21, 4 },		// 126 bits per transform
			{ 17, 22, 3 },		// 126 bits per transform
			{ 17, 23, 2 },		// 126 bits per transform
			{ 18, 1, 23 },		// 126 bits per transform
			{ 18, 2, 22 },		// 126 bits per transform
			{ 18, 3, 21 },		// 126 bits per transform
			{ 18, 4, 20 },		// 126 bits per transform
			{ 18, 5, 19 },		// 126 bits per transform
			{ 18, 6, 18 },		// 126 bits per transform
			{ 18, 7, 17 },		// 126 bits per transform
			{ 18, 8, 16 },		// 126 bits per transform
			{ 18, 9, 15 },		// 126 bits per transform
			{ 18, 10, 14 },		// 126 bits per transform
			{ 18, 11, 13 },		// 126 bits per transform
			{ 18, 12, 12 },		// 126 bits per transform
			{ 18, 13, 11 },		// 126 bits per transform
			{ 18, 14, 10 },		// 126 bits per transform
			{ 18, 15, 9 },		// 126 bits per transform
			{ 18, 16, 8 },		// 126 bits per transform
			{ 18, 17, 7 },		// 126 bits per transform
			{ 18, 18, 6 },		// 126 bits per transform
			{ 18, 19, 5 },		// 126 bits per transform
			{ 18, 20, 4 },		// 126 bits per transform
			{ 18, 21, 3 },		// 126 bits per transform
			{ 18, 22, 2 },		// 126 bits per transform
			{ 18, 23, 1 },		// 126 bits per transform
			{ 19, 0, 23 },		// 126 bits per transform
			{ 19, 1, 22 },		// 126 bits per transform
			{ 19, 2, 21 },		// 126 bits per transform
			{ 19, 3, 20 },		// 126 bits per transform
			{ 19, 4, 19 },		// 126 bits per transform
			{ 19, 5, 18 },		// 126 bits per transform
			{ 19, 6, 17 },		// 126 bits per transform
			{ 19, 7, 16 },		// 126 bits per transform
			{ 19, 8, 15 },		// 126 bits per transform
			{ 19, 9, 14 },		// 126 bits per transform
			{ 19, 10, 13 },		// 126 bits per transform
			{ 19, 11, 12 },		// 126 bits per transform
			{ 19, 12, 11 },		// 126 bits per transform
			{ 19, 13, 10 },		// 126 bits per transform
			{ 19, 14, 9 },		// 126 bits per transform
			{ 19, 15, 8 },		// 126 bits per transform
			{ 19, 16, 7 },		// 126 bits per transform
			{ 19, 17, 6 },		// 126 bits per transform
			{ 19, 18, 5 },		// 126 bits per transform
			{ 19, 19, 4 },		// 126 bits per transform
			{ 19, 20, 3 },		// 126 bits per transform
			{ 19, 21, 2 },		// 126 bits per transform
			{ 19, 22, 1 },		// 126 bits per transform
			{ 19, 23, 0 },		// 126 bits per transform
			{ 20, 0, 22 },		// 126 bits per transform
			{ 20, 1, 21 },		// 126 bits per transform
			{ 20, 2, 20 },		// 126 bits per transform
			{ 20, 3, 19 },		// 126 bits per transform
			{ 20, 4, 18 },		// 126 bits per transform
			{ 20, 5, 17 },		// 126 bits per transform
			{ 20, 6, 16 },		// 126 bits per transform
			{ 20, 7, 15 },		// 126 bits per transform
			{ 20, 8, 14 },		// 126 bits per transform
			{ 20, 9, 13 },		// 126 bits per transform
			{ 20, 10, 12 },		// 126 bits per transform
			{ 20, 11, 11 },		// 126 bits per transform
			{ 20, 12, 10 },		// 126 bits per transform
			{ 20, 13, 9 },		// 126 bits per transform
			{ 20, 14, 8 },		// 126 bits per transform
			{ 20, 15, 7 },		// 126 bits per transform
			{ 20, 16, 6 },		// 126 bits per transform
			{ 20, 17, 5 },		// 126 bits per transform
			{ 20, 18, 4 },		// 126 bits per transform
			{ 20, 19, 3 },		// 126 bits per transform
			{ 20, 20, 2 },		// 126 bits per transform
			{ 20, 21, 1 },		// 126 bits per transform
			{ 20, 22, 0 },		// 126 bits per transform
			{ 21, 0, 21 },		// 126 bits per transform
			{ 21, 1, 20 },		// 126 bits per transform
			{ 21, 2, 19 },		// 126 bits per transform
			{ 21, 3, 18 },		// 126 bits per transform
			{ 21, 4, 17 },		// 126 bits per transform
			{ 21, 5, 16 },		// 126 bits per transform
			{ 21, 6, 15 },		// 126 bits per transform
			{ 21, 7, 14 },		// 126 bits per transform
			{ 21, 8, 13 },		// 126 bits per transform
			{ 21, 9, 12 },		// 126 bits per transform
			{ 21, 10, 11 },		// 126 bits per transform
			{ 21, 11, 10 },		// 126 bits per transform
			{ 21, 12, 9 },		// 126 bits per transform
			{ 21, 13, 8 },		// 126 bits per transform
			{ 21, 14, 7 },		// 126 bits per transform
			{ 21, 15, 6 },		// 126 bits per transform
			{ 21, 16, 5 },		// 126 bits per transform
			{ 21, 17, 4 },		// 126 bits per transform
			{ 21, 18, 3 },		// 126 bits per transform
			{ 21, 19, 2 },		// 126 bits per transform
			{ 21, 20, 1 },		// 126 bits per transform
			{ 21, 21, 0 },		// 126 bits per transform
			{ 22, 0, 20 },		// 126 bits per transform
			{ 22, 1, 19 },		// 126 bits per transform
			{ 22, 2, 18 },		// 126 bits per transform
			{ 22, 3, 17 },		// 126 bits per transform
			{ 22, 4, 16 },		// 126 bits per transform
			{ 22, 5, 15 },		// 126 bits per transform
			{ 22, 6, 14 },		// 126 bits per transform
			{ 22, 7, 13 },		// 126 bits per transform
			{ 22, 8, 12 },		// 126 bits per transform
			{ 22, 9, 11 },		// 126 bits per transform
			{ 22, 10, 10 },		// 126 bits per transform
			{ 22, 11, 9 },		// 126 bits per transform
			{ 22, 12, 8 },		// 126 bits per transform
			{ 22, 13, 7 },		// 126 bits per transform
			{ 22, 14, 6 },		// 126 bits per transform
			{ 22, 15, 5 },		// 126 bits per transform
			{ 22, 16, 4 },		// 126 bits per transform
			{ 22, 17, 3 },		// 126 bits per transform
			{ 22, 18, 2 },		// 126 bits per transform
			{ 22, 19, 1 },		// 126 bits per transform
			{ 22, 20, 0 },		// 126 bits per transform
			{ 23, 0, 19 },		// 126 bits per transform
			{ 23, 1, 18 },		// 126 bits per transform
			{ 23, 2, 17 },		// 126 bits per transform
			{ 23, 3, 16 },		// 126 bits per transform
			{ 23, 4, 15 },		// 126 bits per transform
			{ 23, 5, 14 },		// 126 bits per transform
			{ 23, 6, 13 },		// 126 bits per transform
			{ 23, 7, 12 },		// 126 bits per transform
			{ 23, 8, 11 },		// 126 bits per transform
			{ 23, 9, 10 },		// 126 bits per transform
			{ 23, 10, 9 },		// 126 bits per transform
			{ 23, 11, 8 },		// 126 bits per transform
			{ 23, 12, 7 },		// 126 bits per transform
			{ 23, 13, 6 },		// 126 bits per transform
			{ 23, 14, 5 },		// 126 bits per transform
			{ 23, 15, 4 },		// 126 bits per transform
			{ 23, 16, 3 },		// 126 bits per transform
			{ 23, 17, 2 },		// 126 bits per transform
			{ 23, 18, 1 },		// 126 bits per transform
			{ 23, 19, 0 },		// 126 bits per transform
			{ 24, 0, 10 },		// 126 bits per transform
			{ 24, 1, 9 },		// 126 bits per transform
			{ 24, 2, 8 },		// 126 bits per transform
			{ 24, 3, 7 },		// 126 bits per transform
			{ 24, 4, 6 },		// 126 bits per transform
			{ 24, 5, 5 },		// 126 bits per transform
			{ 24, 6, 4 },		// 126 bits per transform
			{ 24, 7, 3 },		// 126 bits per transform
			{ 24, 8, 2 },		// 126 bits per transform
			{ 24, 9, 1 },		// 126 bits per transform
			{ 24, 10, 0 },		// 126 bits per transform
			{ 0, 11, 24 },		// 129 bits per transform
			{ 0, 20, 23 },		// 129 bits per transform
			{ 0, 21, 22 },		// 129 bits per transform
			{ 0, 22, 21 },		// 129 bits per transform
			{ 0, 23, 20 },		// 129 bits per transform
			{ 0, 24, 11 },		// 129 bits per transform
			{ 1, 10, 24 },		// 129 bits per transform
			{ 1, 19, 23 },		// 129 bits per transform
			{ 1, 20, 22 },		// 129 bits per transform
			{ 1, 21, 21 },		// 129 bits per transform
			{ 1, 22, 20 },		// 129 bits per transform
			{ 1, 23, 19 },		// 129 bits per transform
			{ 1, 24, 10 },		// 129 bits per transform
			{ 2, 9, 24 },		// 129 bits per transform
			{ 2, 18, 23 },		// 129 bits per transform
			{ 2, 19, 22 },		// 129 bits per transform
			{ 2, 20, 21 },		// 129 bits per transform
			{ 2, 21, 20 },		// 129 bits per transform
			{ 2, 22, 19 },		// 129 bits per transform
			{ 2, 23, 18 },		// 129 bits per transform
			{ 2, 24, 9 },		// 129 bits per transform
			{ 3, 8, 24 },		// 129 bits per transform
			{ 3, 17, 23 },		// 129 bits per transform
			{ 3, 18, 22 },		// 129 bits per transform
			{ 3, 19, 21 },		// 129 bits per transform
			{ 3, 20, 20 },		// 129 bits per transform
			{ 3, 21, 19 },		// 129 bits per transform
			{ 3, 22, 18 },		// 129 bits per transform
			{ 3, 23, 17 },		// 129 bits per transform
			{ 3, 24, 8 },		// 129 bits per transform
			{ 4, 7, 24 },		// 129 bits per transform
			{ 4, 16, 23 },		// 129 bits per transform
			{ 4, 17, 22 },		// 129 bits per transform
			{ 4, 18, 21 },		// 129 bits per transform
			{ 4, 19, 20 },		// 129 bits per transform
			{ 4, 20, 19 },		// 129 bits per transform
			{ 4, 21, 18 },		// 129 bits per transform
			{ 4, 22, 17 },		// 129 bits per transform
			{ 4, 23, 16 },		// 129 bits per transform
			{ 4, 24, 7 },		// 129 bits per transform
			{ 5, 6, 24 },		// 129 bits per transform
			{ 5, 15, 23 },		// 129 bits per transform
			{ 5, 16, 22 },		// 129 bits per transform
			{ 5, 17, 21 },		// 129 bits per transform
			{ 5, 18, 20 },		// 129 bits per transform
			{ 5, 19, 19 },		// 129 bits per transform
			{ 5, 20, 18 },		// 129 bits per transform
			{ 5, 21, 17 },		// 129 bits per transform
			{ 5, 22, 16 },		// 129 bits per transform
			{ 5, 23, 15 },		// 129 bits per transform
			{ 5, 24, 6 },		// 129 bits per transform
			{ 6, 5, 24 },		// 129 bits per transform
			{ 6, 14, 23 },		// 129 bits per transform
			{ 6, 15, 22 },		// 129 bits per transform
			{ 6, 16, 21 },		// 129 bits per transform
			{ 6, 17, 20 },		// 129 bits per transform
			{ 6, 18, 19 },		// 129 bits per transform
			{ 6, 19, 18 },		// 129 bits per transform
			{ 6, 20, 17 },		// 129 bits per transform
			{ 6, 21, 16 },		// 129 bits per transform
			{ 6, 22, 15 },		// 129 bits per transform
			{ 6, 23, 14 },		// 129 bits per transform
			{ 6, 24, 5 },		// 129 bits per transform
			{ 7, 4, 24 },		// 129 bits per transform
			{ 7, 13, 23 },		// 129 bits per transform
			{ 7, 14, 22 },		// 129 bits per transform
			{ 7, 15, 21 },		// 129 bits per transform
			{ 7, 16, 20 },		// 129 bits per transform
			{ 7, 17, 19 },		// 129 bits per transform
			{ 7, 18, 18 },		// 129 bits per transform
			{ 7, 19, 17 },		// 129 bits per transform
			{ 7, 20, 16 },		// 129 bits per transform
			{ 7, 21, 15 },		// 129 bits per transform
			{ 7, 22, 14 },		// 129 bits per transform
			{ 7, 23, 13 },		// 129 bits per transform
			{ 7, 24, 4 },		// 129 bits per transform
			{ 8, 3, 24 },		// 129 bits per transform
			{ 8, 12, 23 },		// 129 bits per transform
			{ 8, 13, 22 },		// 129 bits per transform
			{ 8, 14, 21 },		// 129 bits per transform
			{ 8, 15, 20 },		// 129 bits per transform
			{ 8, 16, 19 },		// 129 bits per transform
			{ 8, 17, 18 },		// 129 bits per transform
			{ 8, 18, 17 },		// 129 bits per transform
			{ 8, 19, 16 },		// 129 bits per transform
			{ 8, 20, 15 },		// 129 bits per transform
			{ 8, 21, 14 },		// 129 bits per transform
			{ 8, 22, 13 },		// 129 bits per transform
			{ 8, 23, 12 },		// 129 bits per transform
			{ 8, 24, 3 },		// 129 bits per transform
			{ 9, 2, 24 },		// 129 bits per transform
			{ 9, 11, 23 },		// 129 bits per transform
			{ 9, 12, 22 },		// 129 bits per transform
			{ 9, 13, 21 },		// 129 bits per transform
			{ 9, 14, 20 },		// 129 bits per transform
			{ 9, 15, 19 },		// 129 bits per transform
			{ 9, 16, 18 },		// 129 bits per transform
			{ 9, 17, 17 },		// 129 bits per transform
			{ 9, 18, 16 },		// 129 bits per transform
			{ 9, 19, 15 },		// 129 bits per transform
			{ 9, 20, 14 },		// 129 bits per transform
			{ 9, 21, 13 },		// 129 bits per transform
			{ 9, 22, 12 },		// 129 bits per transform
			{ 9, 23, 11 },		// 129 bits per transform
			{ 9, 24, 2 },		// 129 bits per transform
			{ 10, 1, 24 },		// 129 bits per transform
			{ 10, 10, 23 },		// 129 bits per transform
			{ 10, 11, 22 },		// 129 bits per transform
			{ 10, 12, 21 },		// 129 bits per transform
			{ 10, 13, 20 },		// 129 bits per transform
			{ 10, 14, 19 },		// 129 bits per transform
			{ 10, 15, 18 },		// 129 bits per transform
			{ 10, 16, 17 },		// 129 bits per transform
			{ 10, 17, 16 },		// 129 bits per transform
			{ 10, 18, 15 },		// 129 bits per transform
			{ 10, 19, 14 },		// 129 bits per transform
			{ 10, 20, 13 },		// 129 bits per transform
			{ 10, 21, 12 },		// 129 bits per transform
			{ 10, 22, 11 },		// 129 bits per transform
			{ 10, 23, 10 },		// 129 bits per transform
			{ 10, 24, 1 },		// 129 bits per transform
			{ 11, 0, 24 },		// 129 bits per transform
			{ 11, 9, 23 },		// 129 bits per transform
			{ 11, 10, 22 },		// 129 bits per transform
			{ 11, 11, 21 },		// 129 bits per transform
			{ 11, 12, 20 },		// 129 bits per transform
			{ 11, 13, 19 },		// 129 bits per transform
			{ 11, 14, 18 },		// 129 bits per transform
			{ 11, 15, 17 },		// 129 bits per transform
			{ 11, 16, 16 },		// 129 bits per transform
			{ 11, 17, 15 },		// 129 bits per transform
			{ 11, 18, 14 },		// 129 bits per transform
			{ 11, 19, 13 },		// 129 bits per transform
			{ 11, 20, 12 },		// 129 bits per transform
			{ 11, 21, 11 },		// 129 bits per transform
			{ 11, 22, 10 },		// 129 bits per transform
			{ 11, 23, 9 },		// 129 bits per transform
			{ 11, 24, 0 },		// 129 bits per transform
			{ 12, 8, 23 },		// 129 bits per transform
			{ 12, 9, 22 },		// 129 bits per transform
			{ 12, 10, 21 },		// 129 bits per transform
			{ 12, 11, 20 },		// 129 bits per transform
			{ 12, 12, 19 },		// 129 bits per transform
			{ 12, 13, 18 },		// 129 bits per transform
			{ 12, 14, 17 },		// 129 bits per transform
			{ 12, 15, 16 },		// 129 bits per transform
			{ 12, 16, 15 },		// 129 bits per transform
			{ 12, 17, 14 },		// 129 bits per transform
			{ 12, 18, 13 },		// 129 bits per transform
			{ 12, 19, 12 },		// 129 bits per transform
			{ 12, 20, 11 },		// 129 bits per transform
			{ 12, 21, 10 },		// 129 bits per transform
			{ 12, 22, 9 },		// 129 bits per transform
			{ 12, 23, 8 },		// 129 bits per transform
			{ 13, 7, 23 },		// 129 bits per transform
			{ 13, 8, 22 },		// 129 bits per transform
			{ 13, 9, 21 },		// 129 bits per transform
			{ 13, 10, 20 },		// 129 bits per transform
			{ 13, 11, 19 },		// 129 bits per transform
			{ 13, 12, 18 },		// 129 bits per transform
			{ 13, 13, 17 },		// 129 bits per transform
			{ 13, 14, 16 },		// 129 bits per transform
			{ 13, 15, 15 },		// 129 bits per transform
			{ 13, 16, 14 },		// 129 bits per transform
			{ 13, 17, 13 },		// 129 bits per transform
			{ 13, 18, 12 },		// 129 bits per transform
			{ 13, 19, 11 },		// 129 bits per transform
			{ 13, 20, 10 },		// 129 bits per transform
			{ 13, 21, 9 },		// 129 bits per transform
			{ 13, 22, 8 },		// 129 bits per transform
			{ 13, 23, 7 },		// 129 bits per transform
			{ 14, 6, 23 },		// 129 bits per transform
			{ 14, 7, 22 },		// 129 bits per transform
			{ 14, 8, 21 },		// 129 bits per transform
			{ 14, 9, 20 },		// 129 bits per transform
			{ 14, 10, 19 },		// 129 bits per transform
			{ 14, 11, 18 },		// 129 bits per transform
			{ 14, 12, 17 },		// 129 bits per transform
			{ 14, 13, 16 },		// 129 bits per transform
			{ 14, 14, 15 },		// 129 bits per transform
			{ 14, 15, 14 },		// 129 bits per transform
			{ 14, 16, 13 },		// 129 bits per transform
			{ 14, 17, 12 },		// 129 bits per transform
			{ 14, 18, 11 },		// 129 bits per transform
			{ 14, 19, 10 },		// 129 bits per transform
			{ 14, 20, 9 },		// 129 bits per transform
			{ 14, 21, 8 },		// 129 bits per transform
			{ 14, 22, 7 },		// 129 bits per transform
			{ 14, 23, 6 },		// 129 bits per transform
			{ 15, 5, 23 },		// 129 bits per transform
			{ 15, 6, 22 },		// 129 bits per transform
			{ 15, 7, 21 },		// 129 bits per transform
			{ 15, 8, 20 },		// 129 bits per transform
			{ 15, 9, 19 },		// 129 bits per transform
			{ 15, 10, 18 },		// 129 bits per transform
			{ 15, 11, 17 },		// 129 bits per transform
			{ 15, 12, 16 },		// 129 bits per transform
			{ 15, 13, 15 },		// 129 bits per transform
			{ 15, 14, 14 },		// 129 bits per transform
			{ 15, 15, 13 },		// 129 bits per transform
			{ 15, 16, 12 },		// 129 bits per transform
			{ 15, 17, 11 },		// 129 bits per transform
			{ 15, 18, 10 },		// 129 bits per transform
			{ 15, 19, 9 },		// 129 bits per transform
			{ 15, 20, 8 },		// 129 bits per transform
			{ 15, 21, 7 },		// 129 bits per transform
			{ 15, 22, 6 },		// 129 bits per transform
			{ 15, 23, 5 },		// 129 bits per transform
			{ 16, 4, 23 },		// 129 bits per transform
			{ 16, 5, 22 },		// 129 bits per transform
			{ 16, 6, 21 },		// 129 bits per transform
			{ 16, 7, 20 },		// 129 bits per transform
			{ 16, 8, 19 },		// 129 bits per transform
			{ 16, 9, 18 },		// 129 bits per transform
			{ 16, 10, 17 },		// 129 bits per transform
			{ 16, 11, 16 },		// 129 bits per transform
			{ 16, 12, 15 },		// 129 bits per transform
			{ 16, 13, 14 },		// 129 bits per transform
			{ 16, 14, 13 },		// 129 bits per transform
			{ 16, 15, 12 },		// 129 bits per transform
			{ 16, 16, 11 },		// 129 bits per transform
			{ 16, 17, 10 },		// 129 bits per transform
			{ 16, 18, 9 },		// 129 bits per transform
			{ 16, 19, 8 },		// 129 bits per transform
			{ 16, 20, 7 },		// 129 bits per transform
			{ 16, 21, 6 },		// 129 bits per transform
			{ 16, 22, 5 },		// 129 bits per transform
			{ 16, 23, 4 },		// 129 bits per transform
			{ 17, 3, 23 },		// 129 bits per transform
			{ 17, 4, 22 },		// 129 bits per transform
			{ 17, 5, 21 },		// 129 bits per transform
			{ 17, 6, 20 },		// 129 bits per transform
			{ 17, 7, 19 },		// 129 bits per transform
			{ 17, 8, 18 },		// 129 bits per transform
			{ 17, 9, 17 },		// 129 bits per transform
			{ 17, 10, 16 },		// 129 bits per transform
			{ 17, 11, 15 },		// 129 bits per transform
			{ 17, 12, 14 },		// 129 bits per transform
			{ 17, 13, 13 },		// 129 bits per transform
			{ 17, 14, 12 },		// 129 bits per transform
			{ 17, 15, 11 },		// 129 bits per transform
			{ 17, 16, 10 },		// 129 bits per transform
			{ 17, 17, 9 },		// 129 bits per transform
			{ 17, 18, 8 },		// 129 bits per transform
			{ 17, 19, 7 },		// 129 bits per transform
			{ 17, 20, 6 },		// 129 bits per transform
			{ 17, 21, 5 },		// 129 bits per transform
			{ 17, 22, 4 },		// 129 bits per transform
			{ 17, 23, 3 },		// 129 bits per transform
			{ 18, 2, 23 },		// 129 bits per transform
			{ 18, 3, 22 },		// 129 bits per transform
			{ 18, 4, 21 },		// 129 bits per transform
			{ 18, 5, 20 },		// 129 bits per transform
			{ 18, 6, 19 },		// 129 bits per transform
			{ 18, 7, 18 },		// 129 bits per transform
			{ 18, 8, 17 },		// 129 bits per transform
			{ 18, 9, 16 },		// 129 bits per transform
			{ 18, 10, 15 },		// 129 bits per transform
			{ 18, 11, 14 },		// 129 bits per transform
			{ 18, 12, 13 },		// 129 bits per transform
			{ 18, 13, 12 },		// 129 bits per transform
			{ 18, 14, 11 },		// 129 bits per transform
			{ 18, 15, 10 },		// 129 bits per transform
			{ 18, 16, 9 },		// 129 bits per transform
			{ 18, 17, 8 },		// 129 bits per transform
			{ 18, 18, 7 },		// 129 bits per transform
			{ 18, 19, 6 },		// 129 bits per transform
			{ 18, 20, 5 },		// 129 bits per transform
			{ 18, 21, 4 },		// 129 bits per transform
			{ 18, 22, 3 },		// 129 bits per transform
			{ 18, 23, 2 },		// 129 bits per transform
			{ 19, 1, 23 },		// 129 bits per transform
			{ 19, 2, 22 },		// 129 bits per transform
			{ 19, 3, 21 },		// 129 bits per transform
			{ 19, 4, 20 },		// 129 bits per transform
			{ 19, 5, 19 },		// 129 bits per transform
			{ 19, 6, 18 },		// 129 bits per transform
			{ 19, 7, 17 },		// 129 bits per transform
			{ 19, 8, 16 },		// 129 bits per transform
			{ 19, 9, 15 },		// 129 bits per transform
			{ 19, 10, 14 },		// 129 bits per transform
			{ 19, 11, 13 },		// 129 bits per transform
			{ 19, 12, 12 },		// 129 bits per transform
			{ 19, 13, 11 },		// 129 bits per transform
			{ 19, 14, 10 },		// 129 bits per transform
			{ 19, 15, 9 },		// 129 bits per transform
			{ 19, 16, 8 },		// 129 bits per transform
			{ 19, 17, 7 },		// 129 bits per transform
			{ 19, 18, 6 },		// 129 bits per transform
			{ 19, 19, 5 },		// 129 bits per transform
			{ 19, 20, 4 },		// 129 bits per transform
			{ 19, 21, 3 },		// 129 bits per transform
			{ 19, 22, 2 },		// 129 bits per transform
			{ 19, 23, 1 },		// 129 bits per transform
			{ 20, 0, 23 },		// 129 bits per transform
			{ 20, 1, 22 },		// 129 bits per transform
			{ 20, 2, 21 },		// 129 bits per transform
			{ 20, 3, 20 },		// 129 bits per transform
			{ 20, 4, 19 },		// 129 bits per transform
			{ 20, 5, 18 },		// 129 bits per transform
			{ 20, 6, 17 },		// 129 bits per transform
			{ 20, 7, 16 },		// 129 bits per transform
			{ 20, 8, 15 },		// 129 bits per transform
			{ 20, 9, 14 },		// 129 bits per transform
			{ 20, 10, 13 },		// 129 bits per transform
			{ 20, 11, 12 },		// 129 bits per transform
			{ 20, 12, 11 },		// 129 bits per transform
			{ 20, 13, 10 },		// 129 bits per transform
			{ 20, 14, 9 },		// 129 bits per transform
			{ 20, 15, 8 },		// 129 bits per transform
			{ 20, 16, 7 },		// 129 bits per transform
			{ 20, 17, 6 },		// 129 bits per transform
			{ 20, 18, 5 },		// 129 bits per transform
			{ 20, 19, 4 },		// 129 bits per transform
			{ 20, 20, 3 },		// 129 bits per transform
			{ 20, 21, 2 },		// 129 bits per transform
			{ 20, 22, 1 },		// 129 bits per transform
			{ 20, 23, 0 },		// 129 bits per transform
			{ 21, 0, 22 },		// 129 bits per transform
			{ 21, 1, 21 },		// 129 bits per transform
			{ 21, 2, 20 },		// 129 bits per transform
			{ 21, 3, 19 },		// 129 bits per transform
			{ 21, 4, 18 },		// 129 bits per transform
			{ 21, 5, 17 },		// 129 bits per transform
			{ 21, 6, 16 },		// 129 bits per transform
			{ 21, 7, 15 },		// 129 bits per transform
			{ 21, 8, 14 },		// 129 bits per transform
			{ 21, 9, 13 },		// 129 bits per transform
			{ 21, 10, 12 },		// 129 bits per transform
			{ 21, 11, 11 },		// 129 bits per transform
			{ 21, 12, 10 },		// 129 bits per transform
			{ 21, 13, 9 },		// 129 bits per transform
			{ 21, 14, 8 },		// 129 bits per transform
			{ 21, 15, 7 },		// 129 bits per transform
			{ 21, 16, 6 },		// 129 bits per transform
			{ 21, 17, 5 },		// 129 bits per transform
			{ 21, 18, 4 },		// 129 bits per transform
			{ 21, 19, 3 },		// 129 bits per transform
			{ 21, 20, 2 },		// 129 bits per transform
			{ 21, 21, 1 },		// 129 bits per transform
			{ 21, 22, 0 },		// 129 bits per transform
			{ 22, 0, 21 },		// 129 bits per transform
			{ 22, 1, 20 },		// 129 bits per transform
			{ 22, 2, 19 },		// 129 bits per transform
			{ 22, 3, 18 },		// 129 bits per transform
			{ 22, 4, 17 },		// 129 bits per transform
			{ 22, 5, 16 },		// 129 bits per transform
			{ 22, 6, 15 },		// 129 bits per transform
			{ 22, 7, 14 },		// 129 bits per transform
			{ 22, 8, 13 },		// 129 bits per transform
			{ 22, 9, 12 },		// 129 bits per transform
			{ 22, 10, 11 },		// 129 bits per transform
			{ 22, 11, 10 },		// 129 bits per transform
			{ 22, 12, 9 },		// 129 bits per transform
			{ 22, 13, 8 },		// 129 bits per transform
			{ 22, 14, 7 },		// 129 bits per transform
			{ 22, 15, 6 },		// 129 bits per transform
			{ 22, 16, 5 },		// 129 bits per transform
			{ 22, 17, 4 },		// 129 bits per transform
			{ 22, 18, 3 },		// 129 bits per transform
			{ 22, 19, 2 },		// 129 bits per transform
			{ 22, 20, 1 },		// 129 bits per transform
			{ 22, 21, 0 },		// 129 bits per transform
			{ 23, 0, 20 },		// 129 bits per transform
			{ 23, 1, 19 },		// 129 bits per transform
			{ 23, 2, 18 },		// 129 bits per transform
			{ 23, 3, 17 },		// 129 bits per transform
			{ 23, 4, 16 },		// 129 bits per transform
			{ 23, 5, 15 },		// 129 bits per transform
			{ 23, 6, 14 },		// 129 bits per transform
			{ 23, 7, 13 },		// 129 bits per transform
			{ 23, 8, 12 },		// 129 bits per transform
			{ 23, 9, 11 },		// 129 bits per transform
			{ 23, 10, 10 },		// 129 bits per transform
			{ 23, 11, 9 },		// 129 bits per transform
			{ 23, 12, 8 },		// 129 bits per transform
			{ 23, 13, 7 },		// 129 bits per transform
			{ 23, 14, 6 },		// 129 bits per transform
			{ 23, 15, 5 },		// 129 bits per transform
			{ 23, 16, 4 },		// 129 bits per transform
			{ 23, 17, 3 },		// 129 bits per transform
			{ 23, 18, 2 },		// 129 bits per transform
			{ 23, 19, 1 },		// 129 bits per transform
			{ 23, 20, 0 },		// 129 bits per transform
			{ 24, 0, 11 },		// 129 bits per transform
			{ 24, 1, 10 },		// 129 bits per transform
			{ 24, 2, 9 },		// 129 bits per transform
			{ 24, 3, 8 },		// 129 bits per transform
			{ 24, 4, 7 },		// 129 bits per transform
			{ 24, 5, 6 },		// 129 bits per transform
			{ 24, 6, 5 },		// 129 bits per transform
			{ 24, 7, 4 },		// 129 bits per transform
			{ 24, 8, 3 },		// 129 bits per transform
			{ 24, 9, 2 },		// 129 bits per transform
			{ 24, 10, 1 },		// 129 bits per transform
			{ 24, 11, 0 },		// 129 bits per transform
			{ 0, 12, 24 },		// 132 bits per transform
			{ 0, 21, 23 },		// 132 bits per transform
			{ 0, 22, 22 },		// 132 bits per transform
			{ 0, 23, 21 },		// 132 bits per transform
			{ 0, 24, 12 },		// 132 bits per transform
			{ 1, 11, 24 },		// 132 bits per transform
			{ 1, 20, 23 },		// 132 bits per transform
			{ 1, 21, 22 },		// 132 bits per transform
			{ 1, 22, 21 },		// 132 bits per transform
			{ 1, 23, 20 },		// 132 bits per transform
			{ 1, 24, 11 },		// 132 bits per transform
			{ 2, 10, 24 },		// 132 bits per transform
			{ 2, 19, 23 },		// 132 bits per transform
			{ 2, 20, 22 },		// 132 bits per transform
			{ 2, 21, 21 },		// 132 bits per transform
			{ 2, 22, 20 },		// 132 bits per transform
			{ 2, 23, 19 },		// 132 bits per transform
			{ 2, 24, 10 },		// 132 bits per transform
			{ 3, 9, 24 },		// 132 bits per transform
			{ 3, 18, 23 },		// 132 bits per transform
			{ 3, 19, 22 },		// 132 bits per transform
			{ 3, 20, 21 },		// 132 bits per transform
			{ 3, 21, 20 },		// 132 bits per transform
			{ 3, 22, 19 },		// 132 bits per transform
			{ 3, 23, 18 },		// 132 bits per transform
			{ 3, 24, 9 },		// 132 bits per transform
			{ 4, 8, 24 },		// 132 bits per transform
			{ 4, 17, 23 },		// 132 bits per transform
			{ 4, 18, 22 },		// 132 bits per transform
			{ 4, 19, 21 },		// 132 bits per transform
			{ 4, 20, 20 },		// 132 bits per transform
			{ 4, 21, 19 },		// 132 bits per transform
			{ 4, 22, 18 },		// 132 bits per transform
			{ 4, 23, 17 },		// 132 bits per transform
			{ 4, 24, 8 },		// 132 bits per transform
			{ 5, 7, 24 },		// 132 bits per transform
			{ 5, 16, 23 },		// 132 bits per transform
			{ 5, 17, 22 },		// 132 bits per transform
			{ 5, 18, 21 },		// 132 bits per transform
			{ 5, 19, 20 },		// 132 bits per transform
			{ 5, 20, 19 },		// 132 bits per transform
			{ 5, 21, 18 },		// 132 bits per transform
			{ 5, 22, 17 },		// 132 bits per transform
			{ 5, 23, 16 },		// 132 bits per transform
			{ 5, 24, 7 },		// 132 bits per transform
			{ 6, 6, 24 },		// 132 bits per transform
			{ 6, 15, 23 },		// 132 bits per transform
			{ 6, 16, 22 },		// 132 bits per transform
			{ 6, 17, 21 },		// 132 bits per transform
			{ 6, 18, 20 },		// 132 bits per transform
			{ 6, 19, 19 },		// 132 bits per transform
			{ 6, 20, 18 },		// 132 bits per transform
			{ 6, 21, 17 },		// 132 bits per transform
			{ 6, 22, 16 },		// 132 bits per transform
			{ 6, 23, 15 },		// 132 bits per transform
			{ 6, 24, 6 },		// 132 bits per transform
			{ 7, 5, 24 },		// 132 bits per transform
			{ 7, 14, 23 },		// 132 bits per transform
			{ 7, 15, 22 },		// 132 bits per transform
			{ 7, 16, 21 },		// 132 bits per transform
			{ 7, 17, 20 },		// 132 bits per transform
			{ 7, 18, 19 },		// 132 bits per transform
			{ 7, 19, 18 },		// 132 bits per transform
			{ 7, 20, 17 },		// 132 bits per transform
			{ 7, 21, 16 },		// 132 bits per transform
			{ 7, 22, 15 },		// 132 bits per transform
			{ 7, 23, 14 },		// 132 bits per transform
			{ 7, 24, 5 },		// 132 bits per transform
			{ 8, 4, 24 },		// 132 bits per transform
			{ 8, 13, 23 },		// 132 bits per transform
			{ 8, 14, 22 },		// 132 bits per transform
			{ 8, 15, 21 },		// 132 bits per transform
			{ 8, 16, 20 },		// 132 bits per transform
			{ 8, 17, 19 },		// 132 bits per transform
			{ 8, 18, 18 },		// 132 bits per transform
			{ 8, 19, 17 },		// 132 bits per transform
			{ 8, 20, 16 },		// 132 bits per transform
			{ 8, 21, 15 },		// 132 bits per transform
			{ 8, 22, 14 },		// 132 bits per transform
			{ 8, 23, 13 },		// 132 bits per transform
			{ 8, 24, 4 },		// 132 bits per transform
			{ 9, 3, 24 },		// 132 bits per transform
			{ 9, 12, 23 },		// 132 bits per transform
			{ 9, 13, 22 },		// 132 bits per transform
			{ 9, 14, 21 },		// 132 bits per transform
			{ 9, 15, 20 },		// 132 bits per transform
			{ 9, 16, 19 },		// 132 bits per transform
			{ 9, 17, 18 },		// 132 bits per transform
			{ 9, 18, 17 },		// 132 bits per transform
			{ 9, 19, 16 },		// 132 bits per transform
			{ 9, 20, 15 },		// 132 bits per transform
			{ 9, 21, 14 },		// 132 bits per transform
			{ 9, 22, 13 },		// 132 bits per transform
			{ 9, 23, 12 },		// 132 bits per transform
			{ 9, 24, 3 },		// 132 bits per transform
			{ 10, 2, 24 },		// 132 bits per transform
			{ 10, 11, 23 },		// 132 bits per transform
			{ 10, 12, 22 },		// 132 bits per transform
			{ 10, 13, 21 },		// 132 bits per transform
			{ 10, 14, 20 },		// 132 bits per transform
			{ 10, 15, 19 },		// 132 bits per transform
			{ 10, 16, 18 },		// 132 bits per transform
			{ 10, 17, 17 },		// 132 bits per transform
			{ 10, 18, 16 },		// 132 bits per transform
			{ 10, 19, 15 },		// 132 bits per transform
			{ 10, 20, 14 },		// 132 bits per transform
			{ 10, 21, 13 },		// 132 bits per transform
			{ 10, 22, 12 },		// 132 bits per transform
			{ 10, 23, 11 },		// 132 bits per transform
			{ 10, 24, 2 },		// 132 bits per transform
			{ 11, 1, 24 },		// 132 bits per transform
			{ 11, 10, 23 },		// 132 bits per transform
			{ 11, 11, 22 },		// 132 bits per transform
			{ 11, 12, 21 },		// 132 bits per transform
			{ 11, 13, 20 },		// 132 bits per transform
			{ 11, 14, 19 },		// 132 bits per transform
			{ 11, 15, 18 },		// 132 bits per transform
			{ 11, 16, 17 },		// 132 bits per transform
			{ 11, 17, 16 },		// 132 bits per transform
			{ 11, 18, 15 },		// 132 bits per transform
			{ 11, 19, 14 },		// 132 bits per transform
			{ 11, 20, 13 },		// 132 bits per transform
			{ 11, 21, 12 },		// 132 bits per transform
			{ 11, 22, 11 },		// 132 bits per transform
			{ 11, 23, 10 },		// 132 bits per transform
			{ 11, 24, 1 },		// 132 bits per transform
			{ 12, 0, 24 },		// 132 bits per transform
			{ 12, 9, 23 },		// 132 bits per transform
			{ 12, 10, 22 },		// 132 bits per transform
			{ 12, 11, 21 },		// 132 bits per transform
			{ 12, 12, 20 },		// 132 bits per transform
			{ 12, 13, 19 },		// 132 bits per transform
			{ 12, 14, 18 },		// 132 bits per transform
			{ 12, 15, 17 },		// 132 bits per transform
			{ 12, 16, 16 },		// 132 bits per transform
			{ 12, 17, 15 },		// 132 bits per transform
			{ 12, 18, 14 },		// 132 bits per transform
			{ 12, 19, 13 },		// 132 bits per transform
			{ 12, 20, 12 },		// 132 bits per transform
			{ 12, 21, 11 },		// 132 bits per transform
			{ 12, 22, 10 },		// 132 bits per transform
			{ 12, 23, 9 },		// 132 bits per transform
			{ 12, 24, 0 },		// 132 bits per transform
			{ 13, 8, 23 },		// 132 bits per transform
			{ 13, 9, 22 },		// 132 bits per transform
			{ 13, 10, 21 },		// 132 bits per transform
			{ 13, 11, 20 },		// 132 bits per transform
			{ 13, 12, 19 },		// 132 bits per transform
			{ 13, 13, 18 },		// 132 bits per transform
			{ 13, 14, 17 },		// 132 bits per transform
			{ 13, 15, 16 },		// 132 bits per transform
			{ 13, 16, 15 },		// 132 bits per transform
			{ 13, 17, 14 },		// 132 bits per transform
			{ 13, 18, 13 },		// 132 bits per transform
			{ 13, 19, 12 },		// 132 bits per transform
			{ 13, 20, 11 },		// 132 bits per transform
			{ 13, 21, 10 },		// 132 bits per transform
			{ 13, 22, 9 },		// 132 bits per transform
			{ 13, 23, 8 },		// 132 bits per transform
			{ 14, 7, 23 },		// 132 bits per transform
			{ 14, 8, 22 },		// 132 bits per transform
			{ 14, 9, 21 },		// 132 bits per transform
			{ 14, 10, 20 },		// 132 bits per transform
			{ 14, 11, 19 },		// 132 bits per transform
			{ 14, 12, 18 },		// 132 bits per transform
			{ 14, 13, 17 },		// 132 bits per transform
			{ 14, 14, 16 },		// 132 bits per transform
			{ 14, 15, 15 },		// 132 bits per transform
			{ 14, 16, 14 },		// 132 bits per transform
			{ 14, 17, 13 },		// 132 bits per transform
			{ 14, 18, 12 },		// 132 bits per transform
			{ 14, 19, 11 },		// 132 bits per transform
			{ 14, 20, 10 },		// 132 bits per transform
			{ 14, 21, 9 },		// 132 bits per transform
			{ 14, 22, 8 },		// 132 bits per transform
			{ 14, 23, 7 },		// 132 bits per transform
			{ 15, 6, 23 },		// 132 bits per transform
			{ 15, 7, 22 },		// 132 bits per transform
			{ 15, 8, 21 },		// 132 bits per transform
			{ 15, 9, 20 },		// 132 bits per transform
			{ 15, 10, 19 },		// 132 bits per transform
			{ 15, 11, 18 },		// 132 bits per transform
			{ 15, 12, 17 },		// 132 bits per transform
			{ 15, 13, 16 },		// 132 bits per transform
			{ 15, 14, 15 },		// 132 bits per transform
			{ 15, 15, 14 },		// 132 bits per transform
			{ 15, 16, 13 },		// 132 bits per transform
			{ 15, 17, 12 },		// 132 bits per transform
			{ 15, 18, 11 },		// 132 bits per transform
			{ 15, 19, 10 },		// 132 bits per transform
			{ 15, 20, 9 },		// 132 bits per transform
			{ 15, 21, 8 },		// 132 bits per transform
			{ 15, 22, 7 },		// 132 bits per transform
			{ 15, 23, 6 },		// 132 bits per transform
			{ 16, 5, 23 },		// 132 bits per transform
			{ 16, 6, 22 },		// 132 bits per transform
			{ 16, 7, 21 },		// 132 bits per transform
			{ 16, 8, 20 },		// 132 bits per transform
			{ 16, 9, 19 },		// 132 bits per transform
			{ 16, 10, 18 },		// 132 bits per transform
			{ 16, 11, 17 },		// 132 bits per transform
			{ 16, 12, 16 },		// 132 bits per transform
			{ 16, 13, 15 },		// 132 bits per transform
			{ 16, 14, 14 },		// 132 bits per transform
			{ 16, 15, 13 },		// 132 bits per transform
			{ 16, 16, 12 },		// 132 bits per transform
			{ 16, 17, 11 },		// 132 bits per transform
			{ 16, 18, 10 },		// 132 bits per transform
			{ 16, 19, 9 },		// 132 bits per transform
			{ 16, 20, 8 },		// 132 bits per transform
			{ 16, 21, 7 },		// 132 bits per transform
			{ 16, 22, 6 },		// 132 bits per transform
			{ 16, 23, 5 },		// 132 bits per transform
			{ 17, 4, 23 },		// 132 bits per transform
			{ 17, 5, 22 },		// 132 bits per transform
			{ 17, 6, 21 },		// 132 bits per transform
			{ 17, 7, 20 },		// 132 bits per transform
			{ 17, 8, 19 },		// 132 bits per transform
			{ 17, 9, 18 },		// 132 bits per transform
			{ 17, 10, 17 },		// 132 bits per transform
			{ 17, 11, 16 },		// 132 bits per transform
			{ 17, 12, 15 },		// 132 bits per transform
			{ 17, 13, 14 },		// 132 bits per transform
			{ 17, 14, 13 },		// 132 bits per transform
			{ 17, 15, 12 },		// 132 bits per transform
			{ 17, 16, 11 },		// 132 bits per transform
			{ 17, 17, 10 },		// 132 bits per transform
			{ 17, 18, 9 },		// 132 bits per transform
			{ 17, 19, 8 },		// 132 bits per transform
			{ 17, 20, 7 },		// 132 bits per transform
			{ 17, 21, 6 },		// 132 bits per transform
			{ 17, 22, 5 },		// 132 bits per transform
			{ 17, 23, 4 },		// 132 bits per transform
			{ 18, 3, 23 },		// 132 bits per transform
			{ 18, 4, 22 },		// 132 bits per transform
			{ 18, 5, 21 },		// 132 bits per transform
			{ 18, 6, 20 },		// 132 bits per transform
			{ 18, 7, 19 },		// 132 bits per transform
			{ 18, 8, 18 },		// 132 bits per transform
			{ 18, 9, 17 },		// 132 bits per transform
			{ 18, 10, 16 },		// 132 bits per transform
			{ 18, 11, 15 },		// 132 bits per transform
			{ 18, 12, 14 },		// 132 bits per transform
			{ 18, 13, 13 },		// 132 bits per transform
			{ 18, 14, 12 },		// 132 bits per transform
			{ 18, 15, 11 },		// 132 bits per transform
			{ 18, 16, 10 },		// 132 bits per transform
			{ 18, 17, 9 },		// 132 bits per transform
			{ 18, 18, 8 },		// 132 bits per transform
			{ 18, 19, 7 },		// 132 bits per transform
			{ 18, 20, 6 },		// 132 bits per transform
			{ 18, 21, 5 },		// 132 bits per transform
			{ 18, 22, 4 },		// 132 bits per transform
			{ 18, 23, 3 },		// 132 bits per transform
			{ 19, 2, 23 },		// 132 bits per transform
			{ 19, 3, 22 },		// 132 bits per transform
			{ 19, 4, 21 },		// 132 bits per transform
			{ 19, 5, 20 },		// 132 bits per transform
			{ 19, 6, 19 },		// 132 bits per transform
			{ 19, 7, 18 },		// 132 bits per transform
			{ 19, 8, 17 },		// 132 bits per transform
			{ 19, 9, 16 },		// 132 bits per transform
			{ 19, 10, 15 },		// 132 bits per transform
			{ 19, 11, 14 },		// 132 bits per transform
			{ 19, 12, 13 },		// 132 bits per transform
			{ 19, 13, 12 },		// 132 bits per transform
			{ 19, 14, 11 },		// 132 bits per transform
			{ 19, 15, 10 },		// 132 bits per transform
			{ 19, 16, 9 },		// 132 bits per transform
			{ 19, 17, 8 },		// 132 bits per transform
			{ 19, 18, 7 },		// 132 bits per transform
			{ 19, 19, 6 },		// 132 bits per transform
			{ 19, 20, 5 },		// 132 bits per transform
			{ 19, 21, 4 },		// 132 bits per transform
			{ 19, 22, 3 },		// 132 bits per transform
			{ 19, 23, 2 },		// 132 bits per transform
			{ 20, 1, 23 },		// 132 bits per transform
			{ 20, 2, 22 },		// 132 bits per transform
			{ 20, 3, 21 },		// 132 bits per transform
			{ 20, 4, 20 },		// 132 bits per transform
			{ 20, 5, 19 },		// 132 bits per transform
			{ 20, 6, 18 },		// 132 bits per transform
			{ 20, 7, 17 },		// 132 bits per transform
			{ 20, 8, 16 },		// 132 bits per transform
			{ 20, 9, 15 },		// 132 bits per transform
			{ 20, 10, 14 },		// 132 bits per transform
			{ 20, 11, 13 },		// 132 bits per transform
			{ 20, 12, 12 },		// 132 bits per transform
			{ 20, 13, 11 },		// 132 bits per transform
			{ 20, 14, 10 },		// 132 bits per transform
			{ 20, 15, 9 },		// 132 bits per transform
			{ 20, 16, 8 },		// 132 bits per transform
			{ 20, 17, 7 },		// 132 bits per transform
			{ 20, 18, 6 },		// 132 bits per transform
			{ 20, 19, 5 },		// 132 bits per transform
			{ 20, 20, 4 },		// 132 bits per transform
			{ 20, 21, 3 },		// 132 bits per transform
			{ 20, 22, 2 },		// 132 bits per transform
			{ 20, 23, 1 },		// 132 bits per transform
			{ 21, 0, 23 },		// 132 bits per transform
			{ 21, 1, 22 },		// 132 bits per transform
			{ 21, 2, 21 },		// 132 bits per transform
			{ 21, 3, 20 },		// 132 bits per transform
			{ 21, 4, 19 },		// 132 bits per transform
			{ 21, 5, 18 },		// 132 bits per transform
			{ 21, 6, 17 },		// 132 bits per transform
			{ 21, 7, 16 },		// 132 bits per transform
			{ 21, 8, 15 },		// 132 bits per transform
			{ 21, 9, 14 },		// 132 bits per transform
			{ 21, 10, 13 },		// 132 bits per transform
			{ 21, 11, 12 },		// 132 bits per transform
			{ 21, 12, 11 },		// 132 bits per transform
			{ 21, 13, 10 },		// 132 bits per transform
			{ 21, 14, 9 },		// 132 bits per transform
			{ 21, 15, 8 },		// 132 bits per transform
			{ 21, 16, 7 },		// 132 bits per transform
			{ 21, 17, 6 },		// 132 bits per transform
			{ 21, 18, 5 },		// 132 bits per transform
			{ 21, 19, 4 },		// 132 bits per transform
			{ 21, 20, 3 },		// 132 bits per transform
			{ 21, 21, 2 },		// 132 bits per transform
			{ 21, 22, 1 },		// 132 bits per transform
			{ 21, 23, 0 },		// 132 bits per transform
			{ 22, 0, 22 },		// 132 bits per transform
			{ 22, 1, 21 },		// 132 bits per transform
			{ 22, 2, 20 },		// 132 bits per transform
			{ 22, 3, 19 },		// 132 bits per transform
			{ 22, 4, 18 },		// 132 bits per transform
			{ 22, 5, 17 },		// 132 bits per transform
			{ 22, 6, 16 },		// 132 bits per transform
			{ 22, 7, 15 },		// 132 bits per transform
			{ 22, 8, 14 },		// 132 bits per transform
			{ 22, 9, 13 },		// 132 bits per transform
			{ 22, 10, 12 },		// 132 bits per transform
			{ 22, 11, 11 },		// 132 bits per transform
			{ 22, 12, 10 },		// 132 bits per transform
			{ 22, 13, 9 },		// 132 bits per transform
			{ 22, 14, 8 },		// 132 bits per transform
			{ 22, 15, 7 },		// 132 bits per transform
			{ 22, 16, 6 },		// 132 bits per transform
			{ 22, 17, 5 },		// 132 bits per transform
			{ 22, 18, 4 },		// 132 bits per transform
			{ 22, 19, 3 },		// 132 bits per transform
			{ 22, 20, 2 },		// 132 bits per transform
			{ 22, 21, 1 },		// 132 bits per transform
			{ 22, 22, 0 },		// 132 bits per transform
			{ 23, 0, 21 },		// 132 bits per transform
			{ 23, 1, 20 },		// 132 bits per transform
			{ 23, 2, 19 },		// 132 bits per transform
			{ 23, 3, 18 },		// 132 bits per transform
			{ 23, 4, 17 },		// 132 bits per transform
			{ 23, 5, 16 },		// 132 bits per transform
			{ 23, 6, 15 },		// 132 bits per transform
			{ 23, 7, 14 },		// 132 bits per transform
			{ 23, 8, 13 },		// 132 bits per transform
			{ 23, 9, 12 },		// 132 bits per transform
			{ 23, 10, 11 },		// 132 bits per transform
			{ 23, 11, 10 },		// 132 bits per transform
			{ 23, 12, 9 },		// 132 bits per transform
			{ 23, 13, 8 },		// 132 bits per transform
			{ 23, 14, 7 },		// 132 bits per transform
			{ 23, 15, 6 },		// 132 bits per transform
			{ 23, 16, 5 },		// 132 bits per transform
			{ 23, 17, 4 },		// 132 bits per transform
			{ 23, 18, 3 },		// 132 bits per transform
			{ 23, 19, 2 },		// 132 bits per transform
			{ 23, 20, 1 },		// 132 bits per transform
			{ 23, 21, 0 },		// 132 bits per transform
			{ 24, 0, 12 },		// 132 bits per transform
			{ 24, 1, 11 },		// 132 bits per transform
			{ 24, 2, 10 },		// 132 bits per transform
			{ 24, 3, 9 },		// 132 bits per transform
			{ 24, 4, 8 },		// 132 bits per transform
			{ 24, 5, 7 },		// 132 bits per transform
			{ 24, 6, 6 },		// 132 bits per transform
			{ 24, 7, 5 },		// 132 bits per transform
			{ 24, 8, 4 },		// 132 bits per transform
			{ 24, 9, 3 },		// 132 bits per transform
			{ 24, 10, 2 },		// 132 bits per transform
			{ 24, 11, 1 },		// 132 bits per transform
			{ 24, 12, 0 },		// 132 bits per transform
			{ 0, 13, 24 },		// 135 bits per transform
			{ 0, 22, 23 },		// 135 bits per transform
			{ 0, 23, 22 },		// 135 bits per transform
			{ 0, 24, 13 },		// 135 bits per transform
			{ 1, 12, 24 },		// 135 bits per transform
			{ 1, 21, 23 },		// 135 bits per transform
			{ 1, 22, 22 },		// 135 bits per transform
			{ 1, 23, 21 },		// 135 bits per transform
			{ 1, 24, 12 },		// 135 bits per transform
			{ 2, 11, 24 },		// 135 bits per transform
			{ 2, 20, 23 },		// 135 bits per transform
			{ 2, 21, 22 },		// 135 bits per transform
			{ 2, 22, 21 },		// 135 bits per transform
			{ 2, 23, 20 },		// 135 bits per transform
			{ 2, 24, 11 },		// 135 bits per transform
			{ 3, 10, 24 },		// 135 bits per transform
			{ 3, 19, 23 },		// 135 bits per transform
			{ 3, 20, 22 },		// 135 bits per transform
			{ 3, 21, 21 },		// 135 bits per transform
			{ 3, 22, 20 },		// 135 bits per transform
			{ 3, 23, 19 },		// 135 bits per transform
			{ 3, 24, 10 },		// 135 bits per transform
			{ 4, 9, 24 },		// 135 bits per transform
			{ 4, 18, 23 },		// 135 bits per transform
			{ 4, 19, 22 },		// 135 bits per transform
			{ 4, 20, 21 },		// 135 bits per transform
			{ 4, 21, 20 },		// 135 bits per transform
			{ 4, 22, 19 },		// 135 bits per transform
			{ 4, 23, 18 },		// 135 bits per transform
			{ 4, 24, 9 },		// 135 bits per transform
			{ 5, 8, 24 },		// 135 bits per transform
			{ 5, 17, 23 },		// 135 bits per transform
			{ 5, 18, 22 },		// 135 bits per transform
			{ 5, 19, 21 },		// 135 bits per transform
			{ 5, 20, 20 },		// 135 bits per transform
			{ 5, 21, 19 },		// 135 bits per transform
			{ 5, 22, 18 },		// 135 bits per transform
			{ 5, 23, 17 },		// 135 bits per transform
			{ 5, 24, 8 },		// 135 bits per transform
			{ 6, 7, 24 },		// 135 bits per transform
			{ 6, 16, 23 },		// 135 bits per transform
			{ 6, 17, 22 },		// 135 bits per transform
			{ 6, 18, 21 },		// 135 bits per transform
			{ 6, 19, 20 },		// 135 bits per transform
			{ 6, 20, 19 },		// 135 bits per transform
			{ 6, 21, 18 },		// 135 bits per transform
			{ 6, 22, 17 },		// 135 bits per transform
			{ 6, 23, 16 },		// 135 bits per transform
			{ 6, 24, 7 },		// 135 bits per transform
			{ 7, 6, 24 },		// 135 bits per transform
			{ 7, 15, 23 },		// 135 bits per transform
			{ 7, 16, 22 },		// 135 bits per transform
			{ 7, 17, 21 },		// 135 bits per transform
			{ 7, 18, 20 },		// 135 bits per transform
			{ 7, 19, 19 },		// 135 bits per transform
			{ 7, 20, 18 },		// 135 bits per transform
			{ 7, 21, 17 },		// 135 bits per transform
			{ 7, 22, 16 },		// 135 bits per transform
			{ 7, 23, 15 },		// 135 bits per transform
			{ 7, 24, 6 },		// 135 bits per transform
			{ 8, 5, 24 },		// 135 bits per transform
			{ 8, 14, 23 },		// 135 bits per transform
			{ 8, 15, 22 },		// 135 bits per transform
			{ 8, 16, 21 },		// 135 bits per transform
			{ 8, 17, 20 },		// 135 bits per transform
			{ 8, 18, 19 },		// 135 bits per transform
			{ 8, 19, 18 },		// 135 bits per transform
			{ 8, 20, 17 },		// 135 bits per transform
			{ 8, 21, 16 },		// 135 bits per transform
			{ 8, 22, 15 },		// 135 bits per transform
			{ 8, 23, 14 },		// 135 bits per transform
			{ 8, 24, 5 },		// 135 bits per transform
			{ 9, 4, 24 },		// 135 bits per transform
			{ 9, 13, 23 },		// 135 bits per transform
			{ 9, 14, 22 },		// 135 bits per transform
			{ 9, 15, 21 },		// 135 bits per transform
			{ 9, 16, 20 },		// 135 bits per transform
			{ 9, 17, 19 },		// 135 bits per transform
			{ 9, 18, 18 },		// 135 bits per transform
			{ 9, 19, 17 },		// 135 bits per transform
			{ 9, 20, 16 },		// 135 bits per transform
			{ 9, 21, 15 },		// 135 bits per transform
			{ 9, 22, 14 },		// 135 bits per transform
			{ 9, 23, 13 },		// 135 bits per transform
			{ 9, 24, 4 },		// 135 bits per transform
			{ 10, 3, 24 },		// 135 bits per transform
			{ 10, 12, 23 },		// 135 bits per transform
			{ 10, 13, 22 },		// 135 bits per transform
			{ 10, 14, 21 },		// 135 bits per transform
			{ 10, 15, 20 },		// 135 bits per transform
			{ 10, 16, 19 },		// 135 bits per transform
			{ 10, 17, 18 },		// 135 bits per transform
			{ 10, 18, 17 },		// 135 bits per transform
			{ 10, 19, 16 },		// 135 bits per transform
			{ 10, 20, 15 },		// 135 bits per transform
			{ 10, 21, 14 },		// 135 bits per transform
			{ 10, 22, 13 },		// 135 bits per transform
			{ 10, 23, 12 },		// 135 bits per transform
			{ 10, 24, 3 },		// 135 bits per transform
			{ 11, 2, 24 },		// 135 bits per transform
			{ 11, 11, 23 },		// 135 bits per transform
			{ 11, 12, 22 },		// 135 bits per transform
			{ 11, 13, 21 },		// 135 bits per transform
			{ 11, 14, 20 },		// 135 bits per transform
			{ 11, 15, 19 },		// 135 bits per transform
			{ 11, 16, 18 },		// 135 bits per transform
			{ 11, 17, 17 },		// 135 bits per transform
			{ 11, 18, 16 },		// 135 bits per transform
			{ 11, 19, 15 },		// 135 bits per transform
			{ 11, 20, 14 },		// 135 bits per transform
			{ 11, 21, 13 },		// 135 bits per transform
			{ 11, 22, 12 },		// 135 bits per transform
			{ 11, 23, 11 },		// 135 bits per transform
			{ 11, 24, 2 },		// 135 bits per transform
			{ 12, 1, 24 },		// 135 bits per transform
			{ 12, 10, 23 },		// 135 bits per transform
			{ 12, 11, 22 },		// 135 bits per transform
			{ 12, 12, 21 },		// 135 bits per transform
			{ 12, 13, 20 },		// 135 bits per transform
			{ 12, 14, 19 },		// 135 bits per transform
			{ 12, 15, 18 },		// 135 bits per transform
			{ 12, 16, 17 },		// 135 bits per transform
			{ 12, 17, 16 },		// 135 bits per transform
			{ 12, 18, 15 },		// 135 bits per transform
			{ 12, 19, 14 },		// 135 bits per transform
			{ 12, 20, 13 },		// 135 bits per transform
			{ 12, 21, 12 },		// 135 bits per transform
			{ 12, 22, 11 },		// 135 bits per transform
			{ 12, 23, 10 },		// 135 bits per transform
			{ 12, 24, 1 },		// 135 bits per transform
			{ 13, 0, 24 },		// 135 bits per transform
			{ 13, 9, 23 },		// 135 bits per transform
			{ 13, 10, 22 },		// 135 bits per transform
			{ 13, 11, 21 },		// 135 bits per transform
			{ 13, 12, 20 },		// 135 bits per transform
			{ 13, 13, 19 },		// 135 bits per transform
			{ 13, 14, 18 },		// 135 bits per transform
			{ 13, 15, 17 },		// 135 bits per transform
			{ 13, 16, 16 },		// 135 bits per transform
			{ 13, 17, 15 },		// 135 bits per transform
			{ 13, 18, 14 },		// 135 bits per transform
			{ 13, 19, 13 },		// 135 bits per transform
			{ 13, 20, 12 },		// 135 bits per transform
			{ 13, 21, 11 },		// 135 bits per transform
			{ 13, 22, 10 },		// 135 bits per transform
			{ 13, 23, 9 },		// 135 bits per transform
			{ 13, 24, 0 },		// 135 bits per transform
			{ 14, 8, 23 },		// 135 bits per transform
			{ 14, 9, 22 },		// 135 bits per transform
			{ 14, 10, 21 },		// 135 bits per transform
			{ 14, 11, 20 },		// 135 bits per transform
			{ 14, 12, 19 },		// 135 bits per transform
			{ 14, 13, 18 },		// 135 bits per transform
			{ 14, 14, 17 },		// 135 bits per transform
			{ 14, 15, 16 },		// 135 bits per transform
			{ 14, 16, 15 },		// 135 bits per transform
			{ 14, 17, 14 },		// 135 bits per transform
			{ 14, 18, 13 },		// 135 bits per transform
			{ 14, 19, 12 },		// 135 bits per transform
			{ 14, 20, 11 },		// 135 bits per transform
			{ 14, 21, 10 },		// 135 bits per transform
			{ 14, 22, 9 },		// 135 bits per transform
			{ 14, 23, 8 },		// 135 bits per transform
			{ 15, 7, 23 },		// 135 bits per transform
			{ 15, 8, 22 },		// 135 bits per transform
			{ 15, 9, 21 },		// 135 bits per transform
			{ 15, 10, 20 },		// 135 bits per transform
			{ 15, 11, 19 },		// 135 bits per transform
			{ 15, 12, 18 },		// 135 bits per transform
			{ 15, 13, 17 },		// 135 bits per transform
			{ 15, 14, 16 },		// 135 bits per transform
			{ 15, 15, 15 },		// 135 bits per transform
			{ 15, 16, 14 },		// 135 bits per transform
			{ 15, 17, 13 },		// 135 bits per transform
			{ 15, 18, 12 },		// 135 bits per transform
			{ 15, 19, 11 },		// 135 bits per transform
			{ 15, 20, 10 },		// 135 bits per transform
			{ 15, 21, 9 },		// 135 bits per transform
			{ 15, 22, 8 },		// 135 bits per transform
			{ 15, 23, 7 },		// 135 bits per transform
			{ 16, 6, 23 },		// 135 bits per transform
			{ 16, 7, 22 },		// 135 bits per transform
			{ 16, 8, 21 },		// 135 bits per transform
			{ 16, 9, 20 },		// 135 bits per transform
			{ 16, 10, 19 },		// 135 bits per transform
			{ 16, 11, 18 },		// 135 bits per transform
			{ 16, 12, 17 },		// 135 bits per transform
			{ 16, 13, 16 },		// 135 bits per transform
			{ 16, 14, 15 },		// 135 bits per transform
			{ 16, 15, 14 },		// 135 bits per transform
			{ 16, 16, 13 },		// 135 bits per transform
			{ 16, 17, 12 },		// 135 bits per transform
			{ 16, 18, 11 },		// 135 bits per transform
			{ 16, 19, 10 },		// 135 bits per transform
			{ 16, 20, 9 },		// 135 bits per transform
			{ 16, 21, 8 },		// 135 bits per transform
			{ 16, 22, 7 },		// 135 bits per transform
			{ 16, 23, 6 },		// 135 bits per transform
			{ 17, 5, 23 },		// 135 bits per transform
			{ 17, 6, 22 },		// 135 bits per transform
			{ 17, 7, 21 },		// 135 bits per transform
			{ 17, 8, 20 },		// 135 bits per transform
			{ 17, 9, 19 },		// 135 bits per transform
			{ 17, 10, 18 },		// 135 bits per transform
			{ 17, 11, 17 },		// 135 bits per transform
			{ 17, 12, 16 },		// 135 bits per transform
			{ 17, 13, 15 },		// 135 bits per transform
			{ 17, 14, 14 },		// 135 bits per transform
			{ 17, 15, 13 },		// 135 bits per transform
			{ 17, 16, 12 },		// 135 bits per transform
			{ 17, 17, 11 },		// 135 bits per transform
			{ 17, 18, 10 },		// 135 bits per transform
			{ 17, 19, 9 },		// 135 bits per transform
			{ 17, 20, 8 },		// 135 bits per transform
			{ 17, 21, 7 },		// 135 bits per transform
			{ 17, 22, 6 },		// 135 bits per transform
			{ 17, 23, 5 },		// 135 bits per transform
			{ 18, 4, 23 },		// 135 bits per transform
			{ 18, 5, 22 },		// 135 bits per transform
			{ 18, 6, 21 },		// 135 bits per transform
			{ 18, 7, 20 },		// 135 bits per transform
			{ 18, 8, 19 },		// 135 bits per transform
			{ 18, 9, 18 },		// 135 bits per transform
			{ 18, 10, 17 },		// 135 bits per transform
			{ 18, 11, 16 },		// 135 bits per transform
			{ 18, 12, 15 },		// 135 bits per transform
			{ 18, 13, 14 },		// 135 bits per transform
			{ 18, 14, 13 },		// 135 bits per transform
			{ 18, 15, 12 },		// 135 bits per transform
			{ 18, 16, 11 },		// 135 bits per transform
			{ 18, 17, 10 },		// 135 bits per transform
			{ 18, 18, 9 },		// 135 bits per transform
			{ 18, 19, 8 },		// 135 bits per transform
			{ 18, 20, 7 },		// 135 bits per transform
			{ 18, 21, 6 },		// 135 bits per transform
			{ 18, 22, 5 },		// 135 bits per transform
			{ 18, 23, 4 },		// 135 bits per transform
			{ 19, 3, 23 },		// 135 bits per transform
			{ 19, 4, 22 },		// 135 bits per transform
			{ 19, 5, 21 },		// 135 bits per transform
			{ 19, 6, 20 },		// 135 bits per transform
			{ 19, 7, 19 },		// 135 bits per transform
			{ 19, 8, 18 },		// 135 bits per transform
			{ 19, 9, 17 },		// 135 bits per transform
			{ 19, 10, 16 },		// 135 bits per transform
			{ 19, 11, 15 },		// 135 bits per transform
			{ 19, 12, 14 },		// 135 bits per transform
			{ 19, 13, 13 },		// 135 bits per transform
			{ 19, 14, 12 },		// 135 bits per transform
			{ 19, 15, 11 },		// 135 bits per transform
			{ 19, 16, 10 },		// 135 bits per transform
			{ 19, 17, 9 },		// 135 bits per transform
			{ 19, 18, 8 },		// 135 bits per transform
			{ 19, 19, 7 },		// 135 bits per transform
			{ 19, 20, 6 },		// 135 bits per transform
			{ 19, 21, 5 },		// 135 bits per transform
			{ 19, 22, 4 },		// 135 bits per transform
			{ 19, 23, 3 },		// 135 bits per transform
			{ 20, 2, 23 },		// 135 bits per transform
			{ 20, 3, 22 },		// 135 bits per transform
			{ 20, 4, 21 },		// 135 bits per transform
			{ 20, 5, 20 },		// 135 bits per transform
			{ 20, 6, 19 },		// 135 bits per transform
			{ 20, 7, 18 },		// 135 bits per transform
			{ 20, 8, 17 },		// 135 bits per transform
			{ 20, 9, 16 },		// 135 bits per transform
			{ 20, 10, 15 },		// 135 bits per transform
			{ 20, 11, 14 },		// 135 bits per transform
			{ 20, 12, 13 },		// 135 bits per transform
			{ 20, 13, 12 },		// 135 bits per transform
			{ 20, 14, 11 },		// 135 bits per transform
			{ 20, 15, 10 },		// 135 bits per transform
			{ 20, 16, 9 },		// 135 bits per transform
			{ 20, 17, 8 },		// 135 bits per transform
			{ 20, 18, 7 },		// 135 bits per transform
			{ 20, 19, 6 },		// 135 bits per transform
			{ 20, 20, 5 },		// 135 bits per transform
			{ 20, 21, 4 },		// 135 bits per transform
			{ 20, 22, 3 },		// 135 bits per transform
			{ 20, 23, 2 },		// 135 bits per transform
			{ 21, 1, 23 },		// 135 bits per transform
			{ 21, 2, 22 },		// 135 bits per transform
			{ 21, 3, 21 },		// 135 bits per transform
			{ 21, 4, 20 },		// 135 bits per transform
			{ 21, 5, 19 },		// 135 bits per transform
			{ 21, 6, 18 },		// 135 bits per transform
			{ 21, 7, 17 },		// 135 bits per transform
			{ 21, 8, 16 },		// 135 bits per transform
			{ 21, 9, 15 },		// 135 bits per transform
			{ 21, 10, 14 },		// 135 bits per transform
			{ 21, 11, 13 },		// 135 bits per transform
			{ 21, 12, 12 },		// 135 bits per transform
			{ 21, 13, 11 },		// 135 bits per transform
			{ 21, 14, 10 },		// 135 bits per transform
			{ 21, 15, 9 },		// 135 bits per transform
			{ 21, 16, 8 },		// 135 bits per transform
			{ 21, 17, 7 },		// 135 bits per transform
			{ 21, 18, 6 },		// 135 bits per transform
			{ 21, 19, 5 },		// 135 bits per transform
			{ 21, 20, 4 },		// 135 bits per transform
			{ 21, 21, 3 },		// 135 bits per transform
			{ 21, 22, 2 },		// 135 bits per transform
			{ 21, 23, 1 },		// 135 bits per transform
			{ 22, 0, 23 },		// 135 bits per transform
			{ 22, 1, 22 },		// 135 bits per transform
			{ 22, 2, 21 },		// 135 bits per transform
			{ 22, 3, 20 },		// 135 bits per transform
			{ 22, 4, 19 },		// 135 bits per transform
			{ 22, 5, 18 },		// 135 bits per transform
			{ 22, 6, 17 },		// 135 bits per transform
			{ 22, 7, 16 },		// 135 bits per transform
			{ 22, 8, 15 },		// 135 bits per transform
			{ 22, 9, 14 },		// 135 bits per transform
			{ 22, 10, 13 },		// 135 bits per transform
			{ 22, 11, 12 },		// 135 bits per transform
			{ 22, 12, 11 },		// 135 bits per transform
			{ 22, 13, 10 },		// 135 bits per transform
			{ 22, 14, 9 },		// 135 bits per transform
			{ 22, 15, 8 },		// 135 bits per transform
			{ 22, 16, 7 },		// 135 bits per transform
			{ 22, 17, 6 },		// 135 bits per transform
			{ 22, 18, 5 },		// 135 bits per transform
			{ 22, 19, 4 },		// 135 bits per transform
			{ 22, 20, 3 },		// 135 bits per transform
			{ 22, 21, 2 },		// 135 bits per transform
			{ 22, 22, 1 },		// 135 bits per transform
			{ 22, 23, 0 },		// 135 bits per transform
			{ 23, 0, 22 },		// 135 bits per transform
			{ 23, 1, 21 },		// 135 bits per transform
			{ 23, 2, 20 },		// 135 bits per transform
			{ 23, 3, 19 },		// 135 bits per transform
			{ 23, 4, 18 },		// 135 bits per transform
			{ 23, 5, 17 },		// 135 bits per transform
			{ 23, 6, 16 },		// 135 bits per transform
			{ 23, 7, 15 },		// 135 bits per transform
			{ 23, 8, 14 },		// 135 bits per transform
			{ 23, 9, 13 },		// 135 bits per transform
			{ 23, 10, 12 },		// 135 bits per transform
			{ 23, 11, 11 },		// 135 bits per transform
			{ 23, 12, 10 },		// 135 bits per transform
			{ 23, 13, 9 },		// 135 bits per transform
			{ 23, 14, 8 },		// 135 bits per transform
			{ 23, 15, 7 },		// 135 bits per transform
			{ 23, 16, 6 },		// 135 bits per transform
			{ 23, 17, 5 },		// 135 bits per transform
			{ 23, 18, 4 },		// 135 bits per transform
			{ 23, 19, 3 },		// 135 bits per transform
			{ 23, 20, 2 },		// 135 bits per transform
			{ 23, 21, 1 },		// 135 bits per transform
			{ 23, 22, 0 },		// 135 bits per transform
			{ 24, 0, 13 },		// 135 bits per transform
			{ 24, 1, 12 },		// 135 bits per transform
			{ 24, 2, 11 },		// 135 bits per transform
			{ 24, 3, 10 },		// 135 bits per transform
			{ 24, 4, 9 },		// 135 bits per transform
			{ 24, 5, 8 },		// 135 bits per transform
			{ 24, 6, 7 },		// 135 bits per transform
			{ 24, 7, 6 },		// 135 bits per transform
			{ 24, 8, 5 },		// 135 bits per transform
			{ 24, 9, 4 },		// 135 bits per transform
			{ 24, 10, 3 },		// 135 bits per transform
			{ 24, 11, 2 },		// 135 bits per transform
			{ 24, 12, 1 },		// 135 bits per transform
			{ 24, 13, 0 },		// 135 bits per transform
			{ 0, 14, 24 },		// 138 bits per transform
			{ 0, 23, 23 },		// 138 bits per transform
			{ 0, 24, 14 },		// 138 bits per transform
			{ 1, 13, 24 },		// 138 bits per transform
			{ 1, 22, 23 },		// 138 bits per transform
			{ 1, 23, 22 },		// 138 bits per transform
			{ 1, 24, 13 },		// 138 bits per transform
			{ 2, 12, 24 },		// 138 bits per transform
			{ 2, 21, 23 },		// 138 bits per transform
			{ 2, 22, 22 },		// 138 bits per transform
			{ 2, 23, 21 },		// 138 bits per transform
			{ 2, 24, 12 },		// 138 bits per transform
			{ 3, 11, 24 },		// 138 bits per transform
			{ 3, 20, 23 },		// 138 bits per transform
			{ 3, 21, 22 },		// 138 bits per transform
			{ 3, 22, 21 },		// 138 bits per transform
			{ 3, 23, 20 },		// 138 bits per transform
			{ 3, 24, 11 },		// 138 bits per transform
			{ 4, 10, 24 },		// 138 bits per transform
			{ 4, 19, 23 },		// 138 bits per transform
			{ 4, 20, 22 },		// 138 bits per transform
			{ 4, 21, 21 },		// 138 bits per transform
			{ 4, 22, 20 },		// 138 bits per transform
			{ 4, 23, 19 },		// 138 bits per transform
			{ 4, 24, 10 },		// 138 bits per transform
			{ 5, 9, 24 },		// 138 bits per transform
			{ 5, 18, 23 },		// 138 bits per transform
			{ 5, 19, 22 },		// 138 bits per transform
			{ 5, 20, 21 },		// 138 bits per transform
			{ 5, 21, 20 },		// 138 bits per transform
			{ 5, 22, 19 },		// 138 bits per transform
			{ 5, 23, 18 },		// 138 bits per transform
			{ 5, 24, 9 },		// 138 bits per transform
			{ 6, 8, 24 },		// 138 bits per transform
			{ 6, 17, 23 },		// 138 bits per transform
			{ 6, 18, 22 },		// 138 bits per transform
			{ 6, 19, 21 },		// 138 bits per transform
			{ 6, 20, 20 },		// 138 bits per transform
			{ 6, 21, 19 },		// 138 bits per transform
			{ 6, 22, 18 },		// 138 bits per transform
			{ 6, 23, 17 },		// 138 bits per transform
			{ 6, 24, 8 },		// 138 bits per transform
			{ 7, 7, 24 },		// 138 bits per transform
			{ 7, 16, 23 },		// 138 bits per transform
			{ 7, 17, 22 },		// 138 bits per transform
			{ 7, 18, 21 },		// 138 bits per transform
			{ 7, 19, 20 },		// 138 bits per transform
			{ 7, 20, 19 },		// 138 bits per transform
			{ 7, 21, 18 },		// 138 bits per transform
			{ 7, 22, 17 },		// 138 bits per transform
			{ 7, 23, 16 },		// 138 bits per transform
			{ 7, 24, 7 },		// 138 bits per transform
			{ 8, 6, 24 },		// 138 bits per transform
			{ 8, 15, 23 },		// 138 bits per transform
			{ 8, 16, 22 },		// 138 bits per transform
			{ 8, 17, 21 },		// 138 bits per transform
			{ 8, 18, 20 },		// 138 bits per transform
			{ 8, 19, 19 },		// 138 bits per transform
			{ 8, 20, 18 },		// 138 bits per transform
			{ 8, 21, 17 },		// 138 bits per transform
			{ 8, 22, 16 },		// 138 bits per transform
			{ 8, 23, 15 },		// 138 bits per transform
			{ 8, 24, 6 },		// 138 bits per transform
			{ 9, 5, 24 },		// 138 bits per transform
			{ 9, 14, 23 },		// 138 bits per transform
			{ 9, 15, 22 },		// 138 bits per transform
			{ 9, 16, 21 },		// 138 bits per transform
			{ 9, 17, 20 },		// 138 bits per transform
			{ 9, 18, 19 },		// 138 bits per transform
			{ 9, 19, 18 },		// 138 bits per transform
			{ 9, 20, 17 },		// 138 bits per transform
			{ 9, 21, 16 },		// 138 bits per transform
			{ 9, 22, 15 },		// 138 bits per transform
			{ 9, 23, 14 },		// 138 bits per transform
			{ 9, 24, 5 },		// 138 bits per transform
			{ 10, 4, 24 },		// 138 bits per transform
			{ 10, 13, 23 },		// 138 bits per transform
			{ 10, 14, 22 },		// 138 bits per transform
			{ 10, 15, 21 },		// 138 bits per transform
			{ 10, 16, 20 },		// 138 bits per transform
			{ 10, 17, 19 },		// 138 bits per transform
			{ 10, 18, 18 },		// 138 bits per transform
			{ 10, 19, 17 },		// 138 bits per transform
			{ 10, 20, 16 },		// 138 bits per transform
			{ 10, 21, 15 },		// 138 bits per transform
			{ 10, 22, 14 },		// 138 bits per transform
			{ 10, 23, 13 },		// 138 bits per transform
			{ 10, 24, 4 },		// 138 bits per transform
			{ 11, 3, 24 },		// 138 bits per transform
			{ 11, 12, 23 },		// 138 bits per transform
			{ 11, 13, 22 },		// 138 bits per transform
			{ 11, 14, 21 },		// 138 bits per transform
			{ 11, 15, 20 },		// 138 bits per transform
			{ 11, 16, 19 },		// 138 bits per transform
			{ 11, 17, 18 },		// 138 bits per transform
			{ 11, 18, 17 },		// 138 bits per transform
			{ 11, 19, 16 },		// 138 bits per transform
			{ 11, 20, 15 },		// 138 bits per transform
			{ 11, 21, 14 },		// 138 bits per transform
			{ 11, 22, 13 },		// 138 bits per transform
			{ 11, 23, 12 },		// 138 bits per transform
			{ 11, 24, 3 },		// 138 bits per transform
			{ 12, 2, 24 },		// 138 bits per transform
			{ 12, 11, 23 },		// 138 bits per transform
			{ 12, 12, 22 },		// 138 bits per transform
			{ 12, 13, 21 },		// 138 bits per transform
			{ 12, 14, 20 },		// 138 bits per transform
			{ 12, 15, 19 },		// 138 bits per transform
			{ 12, 16, 18 },		// 138 bits per transform
			{ 12, 17, 17 },		// 138 bits per transform
			{ 12, 18, 16 },		// 138 bits per transform
			{ 12, 19, 15 },		// 138 bits per transform
			{ 12, 20, 14 },		// 138 bits per transform
			{ 12, 21, 13 },		// 138 bits per transform
			{ 12, 22, 12 },		// 138 bits per transform
			{ 12, 23, 11 },		// 138 bits per transform
			{ 12, 24, 2 },		// 138 bits per transform
			{ 13, 1, 24 },		// 138 bits per transform
			{ 13, 10, 23 },		// 138 bits per transform
			{ 13, 11, 22 },		// 138 bits per transform
			{ 13, 12, 21 },		// 138 bits per transform
			{ 13, 13, 20 },		// 138 bits per transform
			{ 13, 14, 19 },		// 138 bits per transform
			{ 13, 15, 18 },		// 138 bits per transform
			{ 13, 16, 17 },		// 138 bits per transform
			{ 13, 17, 16 },		// 138 bits per transform
			{ 13, 18, 15 },		// 138 bits per transform
			{ 13, 19, 14 },		// 138 bits per transform
			{ 13, 20, 13 },		// 138 bits per transform
			{ 13, 21, 12 },		// 138 bits per transform
			{ 13, 22, 11 },		// 138 bits per transform
			{ 13, 23, 10 },		// 138 bits per transform
			{ 13, 24, 1 },		// 138 bits per transform
			{ 14, 0, 24 },		// 138 bits per transform
			{ 14, 9, 23 },		// 138 bits per transform
			{ 14, 10, 22 },		// 138 bits per transform
			{ 14, 11, 21 },		// 138 bits per transform
			{ 14, 12, 20 },		// 138 bits per transform
			{ 14, 13, 19 },		// 138 bits per transform
			{ 14, 14, 18 },		// 138 bits per transform
			{ 14, 15, 17 },		// 138 bits per transform
			{ 14, 16, 16 },		// 138 bits per transform
			{ 14, 17, 15 },		// 138 bits per transform
			{ 14, 18, 14 },		// 138 bits per transform
			{ 14, 19, 13 },		// 138 bits per transform
			{ 14, 20, 12 },		// 138 bits per transform
			{ 14, 21, 11 },		// 138 bits per transform
			{ 14, 22, 10 },		// 138 bits per transform
			{ 14, 23, 9 },		// 138 bits per transform
			{ 14, 24, 0 },		// 138 bits per transform
			{ 15, 8, 23 },		// 138 bits per transform
			{ 15, 9, 22 },		// 138 bits per transform
			{ 15, 10, 21 },		// 138 bits per transform
			{ 15, 11, 20 },		// 138 bits per transform
			{ 15, 12, 19 },		// 138 bits per transform
			{ 15, 13, 18 },		// 138 bits per transform
			{ 15, 14, 17 },		// 138 bits per transform
			{ 15, 15, 16 },		// 138 bits per transform
			{ 15, 16, 15 },		// 138 bits per transform
			{ 15, 17, 14 },		// 138 bits per transform
			{ 15, 18, 13 },		// 138 bits per transform
			{ 15, 19, 12 },		// 138 bits per transform
			{ 15, 20, 11 },		// 138 bits per transform
			{ 15, 21, 10 },		// 138 bits per transform
			{ 15, 22, 9 },		// 138 bits per transform
			{ 15, 23, 8 },		// 138 bits per transform
			{ 16, 7, 23 },		// 138 bits per transform
			{ 16, 8, 22 },		// 138 bits per transform
			{ 16, 9, 21 },		// 138 bits per transform
			{ 16, 10, 20 },		// 138 bits per transform
			{ 16, 11, 19 },		// 138 bits per transform
			{ 16, 12, 18 },		// 138 bits per transform
			{ 16, 13, 17 },		// 138 bits per transform
			{ 16, 14, 16 },		// 138 bits per transform
			{ 16, 15, 15 },		// 138 bits per transform
			{ 16, 16, 14 },		// 138 bits per transform
			{ 16, 17, 13 },		// 138 bits per transform
			{ 16, 18, 12 },		// 138 bits per transform
			{ 16, 19, 11 },		// 138 bits per transform
			{ 16, 20, 10 },		// 138 bits per transform
			{ 16, 21, 9 },		// 138 bits per transform
			{ 16, 22, 8 },		// 138 bits per transform
			{ 16, 23, 7 },		// 138 bits per transform
			{ 17, 6, 23 },		// 138 bits per transform
			{ 17, 7, 22 },		// 138 bits per transform
			{ 17, 8, 21 },		// 138 bits per transform
			{ 17, 9, 20 },		// 138 bits per transform
			{ 17, 10, 19 },		// 138 bits per transform
			{ 17, 11, 18 },		// 138 bits per transform
			{ 17, 12, 17 },		// 138 bits per transform
			{ 17, 13, 16 },		// 138 bits per transform
			{ 17, 14, 15 },		// 138 bits per transform
			{ 17, 15, 14 },		// 138 bits per transform
			{ 17, 16, 13 },		// 138 bits per transform
			{ 17, 17, 12 },		// 138 bits per transform
			{ 17, 18, 11 },		// 138 bits per transform
			{ 17, 19, 10 },		// 138 bits per transform
			{ 17, 20, 9 },		// 138 bits per transform
			{ 17, 21, 8 },		// 138 bits per transform
			{ 17, 22, 7 },		// 138 bits per transform
			{ 17, 23, 6 },		// 138 bits per transform
			{ 18, 5, 23 },		// 138 bits per transform
			{ 18, 6, 22 },		// 138 bits per transform
			{ 18, 7, 21 },		// 138 bits per transform
			{ 18, 8, 20 },		// 138 bits per transform
			{ 18, 9, 19 },		// 138 bits per transform
			{ 18, 10, 18 },		// 138 bits per transform
			{ 18, 11, 17 },		// 138 bits per transform
			{ 18, 12, 16 },		// 138 bits per transform
			{ 18, 13, 15 },		// 138 bits per transform
			{ 18, 14, 14 },		// 138 bits per transform
			{ 18, 15, 13 },		// 138 bits per transform
			{ 18, 16, 12 },		// 138 bits per transform
			{ 18, 17, 11 },		// 138 bits per transform
			{ 18, 18, 10 },		// 138 bits per transform
			{ 18, 19, 9 },		// 138 bits per transform
			{ 18, 20, 8 },		// 138 bits per transform
			{ 18, 21, 7 },		// 138 bits per transform
			{ 18, 22, 6 },		// 138 bits per transform
			{ 18, 23, 5 },		// 138 bits per transform
			{ 19, 4, 23 },		// 138 bits per transform
			{ 19, 5, 22 },		// 138 bits per transform
			{ 19, 6, 21 },		// 138 bits per transform
			{ 19, 7, 20 },		// 138 bits per transform
			{ 19, 8, 19 },		// 138 bits per transform
			{ 19, 9, 18 },		// 138 bits per transform
			{ 19, 10, 17 },		// 138 bits per transform
			{ 19, 11, 16 },		// 138 bits per transform
			{ 19, 12, 15 },		// 138 bits per transform
			{ 19, 13, 14 },		// 138 bits per transform
			{ 19, 14, 13 },		// 138 bits per transform
			{ 19, 15, 12 },		// 138 bits per transform
			{ 19, 16, 11 },		// 138 bits per transform
			{ 19, 17, 10 },		// 138 bits per transform
			{ 19, 18, 9 },		// 138 bits per transform
			{ 19, 19, 8 },		// 138 bits per transform
			{ 19, 20, 7 },		// 138 bits per transform
			{ 19, 21, 6 },		// 138 bits per transform
			{ 19, 22, 5 },		// 138 bits per transform
			{ 19, 23, 4 },		// 138 bits per transform
			{ 20, 3, 23 },		// 138 bits per transform
			{ 20, 4, 22 },		// 138 bits per transform
			{ 20, 5, 21 },		// 138 bits per transform
			{ 20, 6, 20 },		// 138 bits per transform
			{ 20, 7, 19 },		// 138 bits per transform
			{ 20, 8, 18 },		// 138 bits per transform
			{ 20, 9, 17 },		// 138 bits per transform
			{ 20, 10, 16 },		// 138 bits per transform
			{ 20, 11, 15 },		// 138 bits per transform
			{ 20, 12, 14 },		// 138 bits per transform
			{ 20, 13, 13 },		// 138 bits per transform
			{ 20, 14, 12 },		// 138 bits per transform
			{ 20, 15, 11 },		// 138 bits per transform
			{ 20, 16, 10 },		// 138 bits per transform
			{ 20, 17, 9 },		// 138 bits per transform
			{ 20, 18, 8 },		// 138 bits per transform
			{ 20, 19, 7 },		// 138 bits per transform
			{ 20, 20, 6 },		// 138 bits per transform
			{ 20, 21, 5 },		// 138 bits per transform
			{ 20, 22, 4 },		// 138 bits per transform
			{ 20, 23, 3 },		// 138 bits per transform
			{ 21, 2, 23 },		// 138 bits per transform
			{ 21, 3, 22 },		// 138 bits per transform
			{ 21, 4, 21 },		// 138 bits per transform
			{ 21, 5, 20 },		// 138 bits per transform
			{ 21, 6, 19 },		// 138 bits per transform
			{ 21, 7, 18 },		// 138 bits per transform
			{ 21, 8, 17 },		// 138 bits per transform
			{ 21, 9, 16 },		// 138 bits per transform
			{ 21, 10, 15 },		// 138 bits per transform
			{ 21, 11, 14 },		// 138 bits per transform
			{ 21, 12, 13 },		// 138 bits per transform
			{ 21, 13, 12 },		// 138 bits per transform
			{ 21, 14, 11 },		// 138 bits per transform
			{ 21, 15, 10 },		// 138 bits per transform
			{ 21, 16, 9 },		// 138 bits per transform
			{ 21, 17, 8 },		// 138 bits per transform
			{ 21, 18, 7 },		// 138 bits per transform
			{ 21, 19, 6 },		// 138 bits per transform
			{ 21, 20, 5 },		// 138 bits per transform
			{ 21, 21, 4 },		// 138 bits per transform
			{ 21, 22, 3 },		// 138 bits per transform
			{ 21, 23, 2 },		// 138 bits per transform
			{ 22, 1, 23 },		// 138 bits per transform
			{ 22, 2, 22 },		// 138 bits per transform
			{ 22, 3, 21 },		// 138 bits per transform
			{ 22, 4, 20 },		// 138 bits per transform
			{ 22, 5, 19 },		// 138 bits per transform
			{ 22, 6, 18 },		// 138 bits per transform
			{ 22, 7, 17 },		// 138 bits per transform
			{ 22, 8, 16 },		// 138 bits per transform
			{ 22, 9, 15 },		// 138 bits per transform
			{ 22, 10, 14 },		// 138 bits per transform
			{ 22, 11, 13 },		// 138 bits per transform
			{ 22, 12, 12 },		// 138 bits per transform
			{ 22, 13, 11 },		// 138 bits per transform
			{ 22, 14, 10 },		// 138 bits per transform
			{ 22, 15, 9 },		// 138 bits per transform
			{ 22, 16, 8 },		// 138 bits per transform
			{ 22, 17, 7 },		// 138 bits per transform
			{ 22, 18, 6 },		// 138 bits per transform
			{ 22, 19, 5 },		// 138 bits per transform
			{ 22, 20, 4 },		// 138 bits per transform
			{ 22, 21, 3 },		// 138 bits per transform
			{ 22, 22, 2 },		// 138 bits per transform
			{ 22, 23, 1 },		// 138 bits per transform
			{ 23, 0, 23 },		// 138 bits per transform
			{ 23, 1, 22 },		// 138 bits per transform
			{ 23, 2, 21 },		// 138 bits per transform
			{ 23, 3, 20 },		// 138 bits per transform
			{ 23, 4, 19 },		// 138 bits per transform
			{ 23, 5, 18 },		// 138 bits per transform
			{ 23, 6, 17 },		// 138 bits per transform
			{ 23, 7, 16 },		// 138 bits per transform
			{ 23, 8, 15 },		// 138 bits per transform
			{ 23, 9, 14 },		// 138 bits per transform
			{ 23, 10, 13 },		// 138 bits per transform
			{ 23, 11, 12 },		// 138 bits per transform
			{ 23, 12, 11 },		// 138 bits per transform
			{ 23, 13, 10 },		// 138 bits per transform
			{ 23, 14, 9 },		// 138 bits per transform
			{ 23, 15, 8 },		// 138 bits per transform
			{ 23, 16, 7 },		// 138 bits per transform
			{ 23, 17, 6 },		// 138 bits per transform
			{ 23, 18, 5 },		// 138 bits per transform
			{ 23, 19, 4 },		// 138 bits per transform
			{ 23, 20, 3 },		// 138 bits per transform
			{ 23, 21, 2 },		// 138 bits per transform
			{ 23, 22, 1 },		// 138 bits per transform
			{ 23, 23, 0 },		// 138 bits per transform
			{ 24, 0, 14 },		// 138 bits per transform
			{ 24, 1, 13 },		// 138 bits per transform
			{ 24, 2, 12 },		// 138 bits per transform
			{ 24, 3, 11 },		// 138 bits per transform
			{ 24, 4, 10 },		// 138 bits per transform
			{ 24, 5, 9 },		// 138 bits per transform
			{ 24, 6, 8 },		// 138 bits per transform
			{ 24, 7, 7 },		// 138 bits per transform
			{ 24, 8, 6 },		// 138 bits per transform
			{ 24, 9, 5 },		// 138 bits per transform
			{ 24, 10, 4 },		// 138 bits per transform
			{ 24, 11, 3 },		// 138 bits per transform
			{ 24, 12, 2 },		// 138 bits per transform
			{ 24, 13, 1 },		// 138 bits per transform
			{ 24, 14, 0 },		// 138 bits per transform
			{ 0, 15, 24 },		// 141 bits per transform
			{ 0, 24, 15 },		// 141 bits per transform
			{ 1, 14, 24 },		// 141 bits per transform
			{ 1, 23, 23 },		// 141 bits per transform
			{ 1, 24, 14 },		// 141 bits per transform
			{ 2, 13, 24 },		// 141 bits per transform
			{ 2, 22, 23 },		// 141 bits per transform
			{ 2, 23, 22 },		// 141 bits per transform
			{ 2, 24, 13 },		// 141 bits per transform
			{ 3, 12, 24 },		// 141 bits per transform
			{ 3, 21, 23 },		// 141 bits per transform
			{ 3, 22, 22 },		// 141 bits per transform
			{ 3, 23, 21 },		// 141 bits per transform
			{ 3, 24, 12 },		// 141 bits per transform
			{ 4, 11, 24 },		// 141 bits per transform
			{ 4, 20, 23 },		// 141 bits per transform
			{ 4, 21, 22 },		// 141 bits per transform
			{ 4, 22, 21 },		// 141 bits per transform
			{ 4, 23, 20 },		// 141 bits per transform
			{ 4, 24, 11 },		// 141 bits per transform
			{ 5, 10, 24 },		// 141 bits per transform
			{ 5, 19, 23 },		// 141 bits per transform
			{ 5, 20, 22 },		// 141 bits per transform
			{ 5, 21, 21 },		// 141 bits per transform
			{ 5, 22, 20 },		// 141 bits per transform
			{ 5, 23, 19 },		// 141 bits per transform
			{ 5, 24, 10 },		// 141 bits per transform
			{ 6, 9, 24 },		// 141 bits per transform
			{ 6, 18, 23 },		// 141 bits per transform
			{ 6, 19, 22 },		// 141 bits per transform
			{ 6, 20, 21 },		// 141 bits per transform
			{ 6, 21, 20 },		// 141 bits per transform
			{ 6, 22, 19 },		// 141 bits per transform
			{ 6, 23, 18 },		// 141 bits per transform
			{ 6, 24, 9 },		// 141 bits per transform
			{ 7, 8, 24 },		// 141 bits per transform
			{ 7, 17, 23 },		// 141 bits per transform
			{ 7, 18, 22 },		// 141 bits per transform
			{ 7, 19, 21 },		// 141 bits per transform
			{ 7, 20, 20 },		// 141 bits per transform
			{ 7, 21, 19 },		// 141 bits per transform
			{ 7, 22, 18 },		// 141 bits per transform
			{ 7, 23, 17 },		// 141 bits per transform
			{ 7, 24, 8 },		// 141 bits per transform
			{ 8, 7, 24 },		// 141 bits per transform
			{ 8, 16, 23 },		// 141 bits per transform
			{ 8, 17, 22 },		// 141 bits per transform
			{ 8, 18, 21 },		// 141 bits per transform
			{ 8, 19, 20 },		// 141 bits per transform
			{ 8, 20, 19 },		// 141 bits per transform
			{ 8, 21, 18 },		// 141 bits per transform
			{ 8, 22, 17 },		// 141 bits per transform
			{ 8, 23, 16 },		// 141 bits per transform
			{ 8, 24, 7 },		// 141 bits per transform
			{ 9, 6, 24 },		// 141 bits per transform
			{ 9, 15, 23 },		// 141 bits per transform
			{ 9, 16, 22 },		// 141 bits per transform
			{ 9, 17, 21 },		// 141 bits per transform
			{ 9, 18, 20 },		// 141 bits per transform
			{ 9, 19, 19 },		// 141 bits per transform
			{ 9, 20, 18 },		// 141 bits per transform
			{ 9, 21, 17 },		// 141 bits per transform
			{ 9, 22, 16 },		// 141 bits per transform
			{ 9, 23, 15 },		// 141 bits per transform
			{ 9, 24, 6 },		// 141 bits per transform
			{ 10, 5, 24 },		// 141 bits per transform
			{ 10, 14, 23 },		// 141 bits per transform
			{ 10, 15, 22 },		// 141 bits per transform
			{ 10, 16, 21 },		// 141 bits per transform
			{ 10, 17, 20 },		// 141 bits per transform
			{ 10, 18, 19 },		// 141 bits per transform
			{ 10, 19, 18 },		// 141 bits per transform
			{ 10, 20, 17 },		// 141 bits per transform
			{ 10, 21, 16 },		// 141 bits per transform
			{ 10, 22, 15 },		// 141 bits per transform
			{ 10, 23, 14 },		// 141 bits per transform
			{ 10, 24, 5 },		// 141 bits per transform
			{ 11, 4, 24 },		// 141 bits per transform
			{ 11, 13, 23 },		// 141 bits per transform
			{ 11, 14, 22 },		// 141 bits per transform
			{ 11, 15, 21 },		// 141 bits per transform
			{ 11, 16, 20 },		// 141 bits per transform
			{ 11, 17, 19 },		// 141 bits per transform
			{ 11, 18, 18 },		// 141 bits per transform
			{ 11, 19, 17 },		// 141 bits per transform
			{ 11, 20, 16 },		// 141 bits per transform
			{ 11, 21, 15 },		// 141 bits per transform
			{ 11, 22, 14 },		// 141 bits per transform
			{ 11, 23, 13 },		// 141 bits per transform
			{ 11, 24, 4 },		// 141 bits per transform
			{ 12, 3, 24 },		// 141 bits per transform
			{ 12, 12, 23 },		// 141 bits per transform
			{ 12, 13, 22 },		// 141 bits per transform
			{ 12, 14, 21 },		// 141 bits per transform
			{ 12, 15, 20 },		// 141 bits per transform
			{ 12, 16, 19 },		// 141 bits per transform
			{ 12, 17, 18 },		// 141 bits per transform
			{ 12, 18, 17 },		// 141 bits per transform
			{ 12, 19, 16 },		// 141 bits per transform
			{ 12, 20, 15 },		// 141 bits per transform
			{ 12, 21, 14 },		// 141 bits per transform
			{ 12, 22, 13 },		// 141 bits per transform
			{ 12, 23, 12 },		// 141 bits per transform
			{ 12, 24, 3 },		// 141 bits per transform
			{ 13, 2, 24 },		// 141 bits per transform
			{ 13, 11, 23 },		// 141 bits per transform
			{ 13, 12, 22 },		// 141 bits per transform
			{ 13, 13, 21 },		// 141 bits per transform
			{ 13, 14, 20 },		// 141 bits per transform
			{ 13, 15, 19 },		// 141 bits per transform
			{ 13, 16, 18 },		// 141 bits per transform
			{ 13, 17, 17 },		// 141 bits per transform
			{ 13, 18, 16 },		// 141 bits per transform
			{ 13, 19, 15 },		// 141 bits per transform
			{ 13, 20, 14 },		// 141 bits per transform
			{ 13, 21, 13 },		// 141 bits per transform
			{ 13, 22, 12 },		// 141 bits per transform
			{ 13, 23, 11 },		// 141 bits per transform
			{ 13, 24, 2 },		// 141 bits per transform
			{ 14, 1, 24 },		// 141 bits per transform
			{ 14, 10, 23 },		// 141 bits per transform
			{ 14, 11, 22 },		// 141 bits per transform
			{ 14, 12, 21 },		// 141 bits per transform
			{ 14, 13, 20 },		// 141 bits per transform
			{ 14, 14, 19 },		// 141 bits per transform
			{ 14, 15, 18 },		// 141 bits per transform
			{ 14, 16, 17 },		// 141 bits per transform
			{ 14, 17, 16 },		// 141 bits per transform
			{ 14, 18, 15 },		// 141 bits per transform
			{ 14, 19, 14 },		// 141 bits per transform
			{ 14, 20, 13 },		// 141 bits per transform
			{ 14, 21, 12 },		// 141 bits per transform
			{ 14, 22, 11 },		// 141 bits per transform
			{ 14, 23, 10 },		// 141 bits per transform
			{ 14, 24, 1 },		// 141 bits per transform
			{ 15, 0, 24 },		// 141 bits per transform
			{ 15, 9, 23 },		// 141 bits per transform
			{ 15, 10, 22 },		// 141 bits per transform
			{ 15, 11, 21 },		// 141 bits per transform
			{ 15, 12, 20 },		// 141 bits per transform
			{ 15, 13, 19 },		// 141 bits per transform
			{ 15, 14, 18 },		// 141 bits per transform
			{ 15, 15, 17 },		// 141 bits per transform
			{ 15, 16, 16 },		// 141 bits per transform
			{ 15, 17, 15 },		// 141 bits per transform
			{ 15, 18, 14 },		// 141 bits per transform
			{ 15, 19, 13 },		// 141 bits per transform
			{ 15, 20, 12 },		// 141 bits per transform
			{ 15, 21, 11 },		// 141 bits per transform
			{ 15, 22, 10 },		// 141 bits per transform
			{ 15, 23, 9 },		// 141 bits per transform
			{ 15, 24, 0 },		// 141 bits per transform
			{ 16, 8, 23 },		// 141 bits per transform
			{ 16, 9, 22 },		// 141 bits per transform
			{ 16, 10, 21 },		// 141 bits per transform
			{ 16, 11, 20 },		// 141 bits per transform
			{ 16, 12, 19 },		// 141 bits per transform
			{ 16, 13, 18 },		// 141 bits per transform
			{ 16, 14, 17 },		// 141 bits per transform
			{ 16, 15, 16 },		// 141 bits per transform
			{ 16, 16, 15 },		// 141 bits per transform
			{ 16, 17, 14 },		// 141 bits per transform
			{ 16, 18, 13 },		// 141 bits per transform
			{ 16, 19, 12 },		// 141 bits per transform
			{ 16, 20, 11 },		// 141 bits per transform
			{ 16, 21, 10 },		// 141 bits per transform
			{ 16, 22, 9 },		// 141 bits per transform
			{ 16, 23, 8 },		// 141 bits per transform
			{ 17, 7, 23 },		// 141 bits per transform
			{ 17, 8, 22 },		// 141 bits per transform
			{ 17, 9, 21 },		// 141 bits per transform
			{ 17, 10, 20 },		// 141 bits per transform
			{ 17, 11, 19 },		// 141 bits per transform
			{ 17, 12, 18 },		// 141 bits per transform
			{ 17, 13, 17 },		// 141 bits per transform
			{ 17, 14, 16 },		// 141 bits per transform
			{ 17, 15, 15 },		// 141 bits per transform
			{ 17, 16, 14 },		// 141 bits per transform
			{ 17, 17, 13 },		// 141 bits per transform
			{ 17, 18, 12 },		// 141 bits per transform
			{ 17, 19, 11 },		// 141 bits per transform
			{ 17, 20, 10 },		// 141 bits per transform
			{ 17, 21, 9 },		// 141 bits per transform
			{ 17, 22, 8 },		// 141 bits per transform
			{ 17, 23, 7 },		// 141 bits per transform
			{ 18, 6, 23 },		// 141 bits per transform
			{ 18, 7, 22 },		// 141 bits per transform
			{ 18, 8, 21 },		// 141 bits per transform
			{ 18, 9, 20 },		// 141 bits per transform
			{ 18, 10, 19 },		// 141 bits per transform
			{ 18, 11, 18 },		// 141 bits per transform
			{ 18, 12, 17 },		// 141 bits per transform
			{ 18, 13, 16 },		// 141 bits per transform
			{ 18, 14, 15 },		// 141 bits per transform
			{ 18, 15, 14 },		// 141 bits per transform
			{ 18, 16, 13 },		// 141 bits per transform
			{ 18, 17, 12 },		// 141 bits per transform
			{ 18, 18, 11 },		// 141 bits per transform
			{ 18, 19, 10 },		// 141 bits per transform
			{ 18, 20, 9 },		// 141 bits per transform
			{ 18, 21, 8 },		// 141 bits per transform
			{ 18, 22, 7 },		// 141 bits per transform
			{ 18, 23, 6 },		// 141 bits per transform
			{ 19, 5, 23 },		// 141 bits per transform
			{ 19, 6, 22 },		// 141 bits per transform
			{ 19, 7, 21 },		// 141 bits per transform
			{ 19, 8, 20 },		// 141 bits per transform
			{ 19, 9, 19 },		// 141 bits per transform
			{ 19, 10, 18 },		// 141 bits per transform
			{ 19, 11, 17 },		// 141 bits per transform
			{ 19, 12, 16 },		// 141 bits per transform
			{ 19, 13, 15 },		// 141 bits per transform
			{ 19, 14, 14 },		// 141 bits per transform
			{ 19, 15, 13 },		// 141 bits per transform
			{ 19, 16, 12 },		// 141 bits per transform
			{ 19, 17, 11 },		// 141 bits per transform
			{ 19, 18, 10 },		// 141 bits per transform
			{ 19, 19, 9 },		// 141 bits per transform
			{ 19, 20, 8 },		// 141 bits per transform
			{ 19, 21, 7 },		// 141 bits per transform
			{ 19, 22, 6 },		// 141 bits per transform
			{ 19, 23, 5 },		// 141 bits per transform
			{ 20, 4, 23 },		// 141 bits per transform
			{ 20, 5, 22 },		// 141 bits per transform
			{ 20, 6, 21 },		// 141 bits per transform
			{ 20, 7, 20 },		// 141 bits per transform
			{ 20, 8, 19 },		// 141 bits per transform
			{ 20, 9, 18 },		// 141 bits per transform
			{ 20, 10, 17 },		// 141 bits per transform
			{ 20, 11, 16 },		// 141 bits per transform
			{ 20, 12, 15 },		// 141 bits per transform
			{ 20, 13, 14 },		// 141 bits per transform
			{ 20, 14, 13 },		// 141 bits per transform
			{ 20, 15, 12 },		// 141 bits per transform
			{ 20, 16, 11 },		// 141 bits per transform
			{ 20, 17, 10 },		// 141 bits per transform
			{ 20, 18, 9 },		// 141 bits per transform
			{ 20, 19, 8 },		// 141 bits per transform
			{ 20, 20, 7 },		// 141 bits per transform
			{ 20, 21, 6 },		// 141 bits per transform
			{ 20, 22, 5 },		// 141 bits per transform
			{ 20, 23, 4 },		// 141 bits per transform
			{ 21, 3, 23 },		// 141 bits per transform
			{ 21, 4, 22 },		// 141 bits per transform
			{ 21, 5, 21 },		// 141 bits per transform
			{ 21, 6, 20 },		// 141 bits per transform
			{ 21, 7, 19 },		// 141 bits per transform
			{ 21, 8, 18 },		// 141 bits per transform
			{ 21, 9, 17 },		// 141 bits per transform
			{ 21, 10, 16 },		// 141 bits per transform
			{ 21, 11, 15 },		// 141 bits per transform
			{ 21, 12, 14 },		// 141 bits per transform
			{ 21, 13, 13 },		// 141 bits per transform
			{ 21, 14, 12 },		// 141 bits per transform
			{ 21, 15, 11 },		// 141 bits per transform
			{ 21, 16, 10 },		// 141 bits per transform
			{ 21, 17, 9 },		// 141 bits per transform
			{ 21, 18, 8 },		// 141 bits per transform
			{ 21, 19, 7 },		// 141 bits per transform
			{ 21, 20, 6 },		// 141 bits per transform
			{ 21, 21, 5 },		// 141 bits per transform
			{ 21, 22, 4 },		// 141 bits per transform
			{ 21, 23, 3 },		// 141 bits per transform
			{ 22, 2, 23 },		// 141 bits per transform
			{ 22, 3, 22 },		// 141 bits per transform
			{ 22, 4, 21 },		// 141 bits per transform
			{ 22, 5, 20 },		// 141 bits per transform
			{ 22, 6, 19 },		// 141 bits per transform
			{ 22, 7, 18 },		// 141 bits per transform
			{ 22, 8, 17 },		// 141 bits per transform
			{ 22, 9, 16 },		// 141 bits per transform
			{ 22, 10, 15 },		// 141 bits per transform
			{ 22, 11, 14 },		// 141 bits per transform
			{ 22, 12, 13 },		// 141 bits per transform
			{ 22, 13, 12 },		// 141 bits per transform
			{ 22, 14, 11 },		// 141 bits per transform
			{ 22, 15, 10 },		// 141 bits per transform
			{ 22, 16, 9 },		// 141 bits per transform
			{ 22, 17, 8 },		// 141 bits per transform
			{ 22, 18, 7 },		// 141 bits per transform
			{ 22, 19, 6 },		// 141 bits per transform
			{ 22, 20, 5 },		// 141 bits per transform
			{ 22, 21, 4 },		// 141 bits per transform
			{ 22, 22, 3 },		// 141 bits per transform
			{ 22, 23, 2 },		// 141 bits per transform
			{ 23, 1, 23 },		// 141 bits per transform
			{ 23, 2, 22 },		// 141 bits per transform
			{ 23, 3, 21 },		// 141 bits per transform
			{ 23, 4, 20 },		// 141 bits per transform
			{ 23, 5, 19 },		// 141 bits per transform
			{ 23, 6, 18 },		// 141 bits per transform
			{ 23, 7, 17 },		// 141 bits per transform
			{ 23, 8, 16 },		// 141 bits per transform
			{ 23, 9, 15 },		// 141 bits per transform
			{ 23, 10, 14 },		// 141 bits per transform
			{ 23, 11, 13 },		// 141 bits per transform
			{ 23, 12, 12 },		// 141 bits per transform
			{ 23, 13, 11 },		// 141 bits per transform
			{ 23, 14, 10 },		// 141 bits per transform
			{ 23, 15, 9 },		// 141 bits per transform
			{ 23, 16, 8 },		// 141 bits per transform
			{ 23, 17, 7 },		// 141 bits per transform
			{ 23, 18, 6 },		// 141 bits per transform
			{ 23, 19, 5 },		// 141 bits per transform
			{ 23, 20, 4 },		// 141 bits per transform
			{ 23, 21, 3 },		// 141 bits per transform
			{ 23, 22, 2 },		// 141 bits per transform
			{ 23, 23, 1 },		// 141 bits per transform
			{ 24, 0, 15 },		// 141 bits per transform
			{ 24, 1, 14 },		// 141 bits per transform
			{ 24, 2, 13 },		// 141 bits per transform
			{ 24, 3, 12 },		// 141 bits per transform
			{ 24, 4, 11 },		// 141 bits per transform
			{ 24, 5, 10 },		// 141 bits per transform
			{ 24, 6, 9 },		// 141 bits per transform
			{ 24, 7, 8 },		// 141 bits per transform
			{ 24, 8, 7 },		// 141 bits per transform
			{ 24, 9, 6 },		// 141 bits per transform
			{ 24, 10, 5 },		// 141 bits per transform
			{ 24, 11, 4 },		// 141 bits per transform
			{ 24, 12, 3 },		// 141 bits per transform
			{ 24, 13, 2 },		// 141 bits per transform
			{ 24, 14, 1 },		// 141 bits per transform
			{ 24, 15, 0 },		// 141 bits per transform
			{ 0, 16, 24 },		// 144 bits per transform
			{ 0, 24, 16 },		// 144 bits per transform
			{ 1, 15, 24 },		// 144 bits per transform
			{ 1, 24, 15 },		// 144 bits per transform
			{ 2, 14, 24 },		// 144 bits per transform
			{ 2, 23, 23 },		// 144 bits per transform
			{ 2, 24, 14 },		// 144 bits per transform
			{ 3, 13, 24 },		// 144 bits per transform
			{ 3, 22, 23 },		// 144 bits per transform
			{ 3, 23, 22 },		// 144 bits per transform
			{ 3, 24, 13 },		// 144 bits per transform
			{ 4, 12, 24 },		// 144 bits per transform
			{ 4, 21, 23 },		// 144 bits per transform
			{ 4, 22, 22 },		// 144 bits per transform
			{ 4, 23, 21 },		// 144 bits per transform
			{ 4, 24, 12 },		// 144 bits per transform
			{ 5, 11, 24 },		// 144 bits per transform
			{ 5, 20, 23 },		// 144 bits per transform
			{ 5, 21, 22 },		// 144 bits per transform
			{ 5, 22, 21 },		// 144 bits per transform
			{ 5, 23, 20 },		// 144 bits per transform
			{ 5, 24, 11 },		// 144 bits per transform
			{ 6, 10, 24 },		// 144 bits per transform
			{ 6, 19, 23 },		// 144 bits per transform
			{ 6, 20, 22 },		// 144 bits per transform
			{ 6, 21, 21 },		// 144 bits per transform
			{ 6, 22, 20 },		// 144 bits per transform
			{ 6, 23, 19 },		// 144 bits per transform
			{ 6, 24, 10 },		// 144 bits per transform
			{ 7, 9, 24 },		// 144 bits per transform
			{ 7, 18, 23 },		// 144 bits per transform
			{ 7, 19, 22 },		// 144 bits per transform
			{ 7, 20, 21 },		// 144 bits per transform
			{ 7, 21, 20 },		// 144 bits per transform
			{ 7, 22, 19 },		// 144 bits per transform
			{ 7, 23, 18 },		// 144 bits per transform
			{ 7, 24, 9 },		// 144 bits per transform
			{ 8, 8, 24 },		// 144 bits per transform
			{ 8, 17, 23 },		// 144 bits per transform
			{ 8, 18, 22 },		// 144 bits per transform
			{ 8, 19, 21 },		// 144 bits per transform
			{ 8, 20, 20 },		// 144 bits per transform
			{ 8, 21, 19 },		// 144 bits per transform
			{ 8, 22, 18 },		// 144 bits per transform
			{ 8, 23, 17 },		// 144 bits per transform
			{ 8, 24, 8 },		// 144 bits per transform
			{ 9, 7, 24 },		// 144 bits per transform
			{ 9, 16, 23 },		// 144 bits per transform
			{ 9, 17, 22 },		// 144 bits per transform
			{ 9, 18, 21 },		// 144 bits per transform
			{ 9, 19, 20 },		// 144 bits per transform
			{ 9, 20, 19 },		// 144 bits per transform
			{ 9, 21, 18 },		// 144 bits per transform
			{ 9, 22, 17 },		// 144 bits per transform
			{ 9, 23, 16 },		// 144 bits per transform
			{ 9, 24, 7 },		// 144 bits per transform
			{ 10, 6, 24 },		// 144 bits per transform
			{ 10, 15, 23 },		// 144 bits per transform
			{ 10, 16, 22 },		// 144 bits per transform
			{ 10, 17, 21 },		// 144 bits per transform
			{ 10, 18, 20 },		// 144 bits per transform
			{ 10, 19, 19 },		// 144 bits per transform
			{ 10, 20, 18 },		// 144 bits per transform
			{ 10, 21, 17 },		// 144 bits per transform
			{ 10, 22, 16 },		// 144 bits per transform
			{ 10, 23, 15 },		// 144 bits per transform
			{ 10, 24, 6 },		// 144 bits per transform
			{ 11, 5, 24 },		// 144 bits per transform
			{ 11, 14, 23 },		// 144 bits per transform
			{ 11, 15, 22 },		// 144 bits per transform
			{ 11, 16, 21 },		// 144 bits per transform
			{ 11, 17, 20 },		// 144 bits per transform
			{ 11, 18, 19 },		// 144 bits per transform
			{ 11, 19, 18 },		// 144 bits per transform
			{ 11, 20, 17 },		// 144 bits per transform
			{ 11, 21, 16 },		// 144 bits per transform
			{ 11, 22, 15 },		// 144 bits per transform
			{ 11, 23, 14 },		// 144 bits per transform
			{ 11, 24, 5 },		// 144 bits per transform
			{ 12, 4, 24 },		// 144 bits per transform
			{ 12, 13, 23 },		// 144 bits per transform
			{ 12, 14, 22 },		// 144 bits per transform
			{ 12, 15, 21 },		// 144 bits per transform
			{ 12, 16, 20 },		// 144 bits per transform
			{ 12, 17, 19 },		// 144 bits per transform
			{ 12, 18, 18 },		// 144 bits per transform
			{ 12, 19, 17 },		// 144 bits per transform
			{ 12, 20, 16 },		// 144 bits per transform
			{ 12, 21, 15 },		// 144 bits per transform
			{ 12, 22, 14 },		// 144 bits per transform
			{ 12, 23, 13 },		// 144 bits per transform
			{ 12, 24, 4 },		// 144 bits per transform
			{ 13, 3, 24 },		// 144 bits per transform
			{ 13, 12, 23 },		// 144 bits per transform
			{ 13, 13, 22 },		// 144 bits per transform
			{ 13, 14, 21 },		// 144 bits per transform
			{ 13, 15, 20 },		// 144 bits per transform
			{ 13, 16, 19 },		// 144 bits per transform
			{ 13, 17, 18 },		// 144 bits per transform
			{ 13, 18, 17 },		// 144 bits per transform
			{ 13, 19, 16 },		// 144 bits per transform
			{ 13, 20, 15 },		// 144 bits per transform
			{ 13, 21, 14 },		// 144 bits per transform
			{ 13, 22, 13 },		// 144 bits per transform
			{ 13, 23, 12 },		// 144 bits per transform
			{ 13, 24, 3 },		// 144 bits per transform
			{ 14, 2, 24 },		// 144 bits per transform
			{ 14, 11, 23 },		// 144 bits per transform
			{ 14, 12, 22 },		// 144 bits per transform
			{ 14, 13, 21 },		// 144 bits per transform
			{ 14, 14, 20 },		// 144 bits per transform
			{ 14, 15, 19 },		// 144 bits per transform
			{ 14, 16, 18 },		// 144 bits per transform
			{ 14, 17, 17 },		// 144 bits per transform
			{ 14, 18, 16 },		// 144 bits per transform
			{ 14, 19, 15 },		// 144 bits per transform
			{ 14, 20, 14 },		// 144 bits per transform
			{ 14, 21, 13 },		// 144 bits per transform
			{ 14, 22, 12 },		// 144 bits per transform
			{ 14, 23, 11 },		// 144 bits per transform
			{ 14, 24, 2 },		// 144 bits per transform
			{ 15, 1, 24 },		// 144 bits per transform
			{ 15, 10, 23 },		// 144 bits per transform
			{ 15, 11, 22 },		// 144 bits per transform
			{ 15, 12, 21 },		// 144 bits per transform
			{ 15, 13, 20 },		// 144 bits per transform
			{ 15, 14, 19 },		// 144 bits per transform
			{ 15, 15, 18 },		// 144 bits per transform
			{ 15, 16, 17 },		// 144 bits per transform
			{ 15, 17, 16 },		// 144 bits per transform
			{ 15, 18, 15 },		// 144 bits per transform
			{ 15, 19, 14 },		// 144 bits per transform
			{ 15, 20, 13 },		// 144 bits per transform
			{ 15, 21, 12 },		// 144 bits per transform
			{ 15, 22, 11 },		// 144 bits per transform
			{ 15, 23, 10 },		// 144 bits per transform
			{ 15, 24, 1 },		// 144 bits per transform
			{ 16, 0, 24 },		// 144 bits per transform
			{ 16, 9, 23 },		// 144 bits per transform
			{ 16, 10, 22 },		// 144 bits per transform
			{ 16, 11, 21 },		// 144 bits per transform
			{ 16, 12, 20 },		// 144 bits per transform
			{ 16, 13, 19 },		// 144 bits per transform
			{ 16, 14, 18 },		// 144 bits per transform
			{ 16, 15, 17 },		// 144 bits per transform
			{ 16, 16, 16 },		// 144 bits per transform
			{ 16, 17, 15 },		// 144 bits per transform
			{ 16, 18, 14 },		// 144 bits per transform
			{ 16, 19, 13 },		// 144 bits per transform
			{ 16, 20, 12 },		// 144 bits per transform
			{ 16, 21, 11 },		// 144 bits per transform
			{ 16, 22, 10 },		// 144 bits per transform
			{ 16, 23, 9 },		// 144 bits per transform
			{ 16, 24, 0 },		// 144 bits per transform
			{ 17, 8, 23 },		// 144 bits per transform
			{ 17, 9, 22 },		// 144 bits per transform
			{ 17, 10, 21 },		// 144 bits per transform
			{ 17, 11, 20 },		// 144 bits per transform
			{ 17, 12, 19 },		// 144 bits per transform
			{ 17, 13, 18 },		// 144 bits per transform
			{ 17, 14, 17 },		// 144 bits per transform
			{ 17, 15, 16 },		// 144 bits per transform
			{ 17, 16, 15 },		// 144 bits per transform
			{ 17, 17, 14 },		// 144 bits per transform
			{ 17, 18, 13 },		// 144 bits per transform
			{ 17, 19, 12 },		// 144 bits per transform
			{ 17, 20, 11 },		// 144 bits per transform
			{ 17, 21, 10 },		// 144 bits per transform
			{ 17, 22, 9 },		// 144 bits per transform
			{ 17, 23, 8 },		// 144 bits per transform
			{ 18, 7, 23 },		// 144 bits per transform
			{ 18, 8, 22 },		// 144 bits per transform
			{ 18, 9, 21 },		// 144 bits per transform
			{ 18, 10, 20 },		// 144 bits per transform
			{ 18, 11, 19 },		// 144 bits per transform
			{ 18, 12, 18 },		// 144 bits per transform
			{ 18, 13, 17 },		// 144 bits per transform
			{ 18, 14, 16 },		// 144 bits per transform
			{ 18, 15, 15 },		// 144 bits per transform
			{ 18, 16, 14 },		// 144 bits per transform
			{ 18, 17, 13 },		// 144 bits per transform
			{ 18, 18, 12 },		// 144 bits per transform
			{ 18, 19, 11 },		// 144 bits per transform
			{ 18, 20, 10 },		// 144 bits per transform
			{ 18, 21, 9 },		// 144 bits per transform
			{ 18, 22, 8 },		// 144 bits per transform
			{ 18, 23, 7 },		// 144 bits per transform
			{ 19, 6, 23 },		// 144 bits per transform
			{ 19, 7, 22 },		// 144 bits per transform
			{ 19, 8, 21 },		// 144 bits per transform
			{ 19, 9, 20 },		// 144 bits per transform
			{ 19, 10, 19 },		// 144 bits per transform
			{ 19, 11, 18 },		// 144 bits per transform
			{ 19, 12, 17 },		// 144 bits per transform
			{ 19, 13, 16 },		// 144 bits per transform
			{ 19, 14, 15 },		// 144 bits per transform
			{ 19, 15, 14 },		// 144 bits per transform
			{ 19, 16, 13 },		// 144 bits per transform
			{ 19, 17, 12 },		// 144 bits per transform
			{ 19, 18, 11 },		// 144 bits per transform
			{ 19, 19, 10 },		// 144 bits per transform
			{ 19, 20, 9 },		// 144 bits per transform
			{ 19, 21, 8 },		// 144 bits per transform
			{ 19, 22, 7 },		// 144 bits per transform
			{ 19, 23, 6 },		// 144 bits per transform
			{ 20, 5, 23 },		// 144 bits per transform
			{ 20, 6, 22 },		// 144 bits per transform
			{ 20, 7, 21 },		// 144 bits per transform
			{ 20, 8, 20 },		// 144 bits per transform
			{ 20, 9, 19 },		// 144 bits per transform
			{ 20, 10, 18 },		// 144 bits per transform
			{ 20, 11, 17 },		// 144 bits per transform
			{ 20, 12, 16 },		// 144 bits per transform
			{ 20, 13, 15 },		// 144 bits per transform
			{ 20, 14, 14 },		// 144 bits per transform
			{ 20, 15, 13 },		// 144 bits per transform
			{ 20, 16, 12 },		// 144 bits per transform
			{ 20, 17, 11 },		// 144 bits per transform
			{ 20, 18, 10 },		// 144 bits per transform
			{ 20, 19, 9 },		// 144 bits per transform
			{ 20, 20, 8 },		// 144 bits per transform
			{ 20, 21, 7 },		// 144 bits per transform
			{ 20, 22, 6 },		// 144 bits per transform
			{ 20, 23, 5 },		// 144 bits per transform
			{ 21, 4, 23 },		// 144 bits per transform
			{ 21, 5, 22 },		// 144 bits per transform
			{ 21, 6, 21 },		// 144 bits per transform
			{ 21, 7, 20 },		// 144 bits per transform
			{ 21, 8, 19 },		// 144 bits per transform
			{ 21, 9, 18 },		// 144 bits per transform
			{ 21, 10, 17 },		// 144 bits per transform
			{ 21, 11, 16 },		// 144 bits per transform
			{ 21, 12, 15 },		// 144 bits per transform
			{ 21, 13, 14 },		// 144 bits per transform
			{ 21, 14, 13 },		// 144 bits per transform
			{ 21, 15, 12 },		// 144 bits per transform
			{ 21, 16, 11 },		// 144 bits per transform
			{ 21, 17, 10 },		// 144 bits per transform
			{ 21, 18, 9 },		// 144 bits per transform
			{ 21, 19, 8 },		// 144 bits per transform
			{ 21, 20, 7 },		// 144 bits per transform
			{ 21, 21, 6 },		// 144 bits per transform
			{ 21, 22, 5 },		// 144 bits per transform
			{ 21, 23, 4 },		// 144 bits per transform
			{ 22, 3, 23 },		// 144 bits per transform
			{ 22, 4, 22 },		// 144 bits per transform
			{ 22, 5, 21 },		// 144 bits per transform
			{ 22, 6, 20 },		// 144 bits per transform
			{ 22, 7, 19 },		// 144 bits per transform
			{ 22, 8, 18 },		// 144 bits per transform
			{ 22, 9, 17 },		// 144 bits per transform
			{ 22, 10, 16 },		// 144 bits per transform
			{ 22, 11, 15 },		// 144 bits per transform
			{ 22, 12, 14 },		// 144 bits per transform
			{ 22, 13, 13 },		// 144 bits per transform
			{ 22, 14, 12 },		// 144 bits per transform
			{ 22, 15, 11 },		// 144 bits per transform
			{ 22, 16, 10 },		// 144 bits per transform
			{ 22, 17, 9 },		// 144 bits per transform
			{ 22, 18, 8 },		// 144 bits per transform
			{ 22, 19, 7 },		// 144 bits per transform
			{ 22, 20, 6 },		// 144 bits per transform
			{ 22, 21, 5 },		// 144 bits per transform
			{ 22, 22, 4 },		// 144 bits per transform
			{ 22, 23, 3 },		// 144 bits per transform
			{ 23, 2, 23 },		// 144 bits per transform
			{ 23, 3, 22 },		// 144 bits per transform
			{ 23, 4, 21 },		// 144 bits per transform
			{ 23, 5, 20 },		// 144 bits per transform
			{ 23, 6, 19 },		// 144 bits per transform
			{ 23, 7, 18 },		// 144 bits per transform
			{ 23, 8, 17 },		// 144 bits per transform
			{ 23, 9, 16 },		// 144 bits per transform
			{ 23, 10, 15 },		// 144 bits per transform
			{ 23, 11, 14 },		// 144 bits per transform
			{ 23, 12, 13 },		// 144 bits per transform
			{ 23, 13, 12 },		// 144 bits per transform
			{ 23, 14, 11 },		// 144 bits per transform
			{ 23, 15, 10 },		// 144 bits per transform
			{ 23, 16, 9 },		// 144 bits per transform
			{ 23, 17, 8 },		// 144 bits per transform
			{ 23, 18, 7 },		// 144 bits per transform
			{ 23, 19, 6 },		// 144 bits per transform
			{ 23, 20, 5 },		// 144 bits per transform
			{ 23, 21, 4 },		// 144 bits per transform
			{ 23, 22, 3 },		// 144 bits per transform
			{ 23, 23, 2 },		// 144 bits per transform
			{ 24, 0, 16 },		// 144 bits per transform
			{ 24, 1, 15 },		// 144 bits per transform
			{ 24, 2, 14 },		// 144 bits per transform
			{ 24, 3, 13 },		// 144 bits per transform
			{ 24, 4, 12 },		// 144 bits per transform
			{ 24, 5, 11 },		// 144 bits per transform
			{ 24, 6, 10 },		// 144 bits per transform
			{ 24, 7, 9 },		// 144 bits per transform
			{ 24, 8, 8 },		// 144 bits per transform
			{ 24, 9, 7 },		// 144 bits per transform
			{ 24, 10, 6 },		// 144 bits per transform
			{ 24, 11, 5 },		// 144 bits per transform
			{ 24, 12, 4 },		// 144 bits per transform
			{ 24, 13, 3 },		// 144 bits per transform
			{ 24, 14, 2 },		// 144 bits per transform
			{ 24, 15, 1 },		// 144 bits per transform
			{ 24, 16, 0 },		// 144 bits per transform
			{ 0, 17, 24 },		// 147 bits per transform
			{ 0, 24, 17 },		// 147 bits per transform
			{ 1, 16, 24 },		// 147 bits per transform
			{ 1, 24, 16 },		// 147 bits per transform
			{ 2, 15, 24 },		// 147 bits per transform
			{ 2, 24, 15 },		// 147 bits per transform
			{ 3, 14, 24 },		// 147 bits per transform
			{ 3, 23, 23 },		// 147 bits per transform
			{ 3, 24, 14 },		// 147 bits per transform
			{ 4, 13, 24 },		// 147 bits per transform
			{ 4, 22, 23 },		// 147 bits per transform
			{ 4, 23, 22 },		// 147 bits per transform
			{ 4, 24, 13 },		// 147 bits per transform
			{ 5, 12, 24 },		// 147 bits per transform
			{ 5, 21, 23 },		// 147 bits per transform
			{ 5, 22, 22 },		// 147 bits per transform
			{ 5, 23, 21 },		// 147 bits per transform
			{ 5, 24, 12 },		// 147 bits per transform
			{ 6, 11, 24 },		// 147 bits per transform
			{ 6, 20, 23 },		// 147 bits per transform
			{ 6, 21, 22 },		// 147 bits per transform
			{ 6, 22, 21 },		// 147 bits per transform
			{ 6, 23, 20 },		// 147 bits per transform
			{ 6, 24, 11 },		// 147 bits per transform
			{ 7, 10, 24 },		// 147 bits per transform
			{ 7, 19, 23 },		// 147 bits per transform
			{ 7, 20, 22 },		// 147 bits per transform
			{ 7, 21, 21 },		// 147 bits per transform
			{ 7, 22, 20 },		// 147 bits per transform
			{ 7, 23, 19 },		// 147 bits per transform
			{ 7, 24, 10 },		// 147 bits per transform
			{ 8, 9, 24 },		// 147 bits per transform
			{ 8, 18, 23 },		// 147 bits per transform
			{ 8, 19, 22 },		// 147 bits per transform
			{ 8, 20, 21 },		// 147 bits per transform
			{ 8, 21, 20 },		// 147 bits per transform
			{ 8, 22, 19 },		// 147 bits per transform
			{ 8, 23, 18 },		// 147 bits per transform
			{ 8, 24, 9 },		// 147 bits per transform
			{ 9, 8, 24 },		// 147 bits per transform
			{ 9, 17, 23 },		// 147 bits per transform
			{ 9, 18, 22 },		// 147 bits per transform
			{ 9, 19, 21 },		// 147 bits per transform
			{ 9, 20, 20 },		// 147 bits per transform
			{ 9, 21, 19 },		// 147 bits per transform
			{ 9, 22, 18 },		// 147 bits per transform
			{ 9, 23, 17 },		// 147 bits per transform
			{ 9, 24, 8 },		// 147 bits per transform
			{ 10, 7, 24 },		// 147 bits per transform
			{ 10, 16, 23 },		// 147 bits per transform
			{ 10, 17, 22 },		// 147 bits per transform
			{ 10, 18, 21 },		// 147 bits per transform
			{ 10, 19, 20 },		// 147 bits per transform
			{ 10, 20, 19 },		// 147 bits per transform
			{ 10, 21, 18 },		// 147 bits per transform
			{ 10, 22, 17 },		// 147 bits per transform
			{ 10, 23, 16 },		// 147 bits per transform
			{ 10, 24, 7 },		// 147 bits per transform
			{ 11, 6, 24 },		// 147 bits per transform
			{ 11, 15, 23 },		// 147 bits per transform
			{ 11, 16, 22 },		// 147 bits per transform
			{ 11, 17, 21 },		// 147 bits per transform
			{ 11, 18, 20 },		// 147 bits per transform
			{ 11, 19, 19 },		// 147 bits per transform
			{ 11, 20, 18 },		// 147 bits per transform
			{ 11, 21, 17 },		// 147 bits per transform
			{ 11, 22, 16 },		// 147 bits per transform
			{ 11, 23, 15 },		// 147 bits per transform
			{ 11, 24, 6 },		// 147 bits per transform
			{ 12, 5, 24 },		// 147 bits per transform
			{ 12, 14, 23 },		// 147 bits per transform
			{ 12, 15, 22 },		// 147 bits per transform
			{ 12, 16, 21 },		// 147 bits per transform
			{ 12, 17, 20 },		// 147 bits per transform
			{ 12, 18, 19 },		// 147 bits per transform
			{ 12, 19, 18 },		// 147 bits per transform
			{ 12, 20, 17 },		// 147 bits per transform
			{ 12, 21, 16 },		// 147 bits per transform
			{ 12, 22, 15 },		// 147 bits per transform
			{ 12, 23, 14 },		// 147 bits per transform
			{ 12, 24, 5 },		// 147 bits per transform
			{ 13, 4, 24 },		// 147 bits per transform
			{ 13, 13, 23 },		// 147 bits per transform
			{ 13, 14, 22 },		// 147 bits per transform
			{ 13, 15, 21 },		// 147 bits per transform
			{ 13, 16, 20 },		// 147 bits per transform
			{ 13, 17, 19 },		// 147 bits per transform
			{ 13, 18, 18 },		// 147 bits per transform
			{ 13, 19, 17 },		// 147 bits per transform
			{ 13, 20, 16 },		// 147 bits per transform
			{ 13, 21, 15 },		// 147 bits per transform
			{ 13, 22, 14 },		// 147 bits per transform
			{ 13, 23, 13 },		// 147 bits per transform
			{ 13, 24, 4 },		// 147 bits per transform
			{ 14, 3, 24 },		// 147 bits per transform
			{ 14, 12, 23 },		// 147 bits per transform
			{ 14, 13, 22 },		// 147 bits per transform
			{ 14, 14, 21 },		// 147 bits per transform
			{ 14, 15, 20 },		// 147 bits per transform
			{ 14, 16, 19 },		// 147 bits per transform
			{ 14, 17, 18 },		// 147 bits per transform
			{ 14, 18, 17 },		// 147 bits per transform
			{ 14, 19, 16 },		// 147 bits per transform
			{ 14, 20, 15 },		// 147 bits per transform
			{ 14, 21, 14 },		// 147 bits per transform
			{ 14, 22, 13 },		// 147 bits per transform
			{ 14, 23, 12 },		// 147 bits per transform
			{ 14, 24, 3 },		// 147 bits per transform
			{ 15, 2, 24 },		// 147 bits per transform
			{ 15, 11, 23 },		// 147 bits per transform
			{ 15, 12, 22 },		// 147 bits per transform
			{ 15, 13, 21 },		// 147 bits per transform
			{ 15, 14, 20 },		// 147 bits per transform
			{ 15, 15, 19 },		// 147 bits per transform
			{ 15, 16, 18 },		// 147 bits per transform
			{ 15, 17, 17 },		// 147 bits per transform
			{ 15, 18, 16 },		// 147 bits per transform
			{ 15, 19, 15 },		// 147 bits per transform
			{ 15, 20, 14 },		// 147 bits per transform
			{ 15, 21, 13 },		// 147 bits per transform
			{ 15, 22, 12 },		// 147 bits per transform
			{ 15, 23, 11 },		// 147 bits per transform
			{ 15, 24, 2 },		// 147 bits per transform
			{ 16, 1, 24 },		// 147 bits per transform
			{ 16, 10, 23 },		// 147 bits per transform
			{ 16, 11, 22 },		// 147 bits per transform
			{ 16, 12, 21 },		// 147 bits per transform
			{ 16, 13, 20 },		// 147 bits per transform
			{ 16, 14, 19 },		// 147 bits per transform
			{ 16, 15, 18 },		// 147 bits per transform
			{ 16, 16, 17 },		// 147 bits per transform
			{ 16, 17, 16 },		// 147 bits per transform
			{ 16, 18, 15 },		// 147 bits per transform
			{ 16, 19, 14 },		// 147 bits per transform
			{ 16, 20, 13 },		// 147 bits per transform
			{ 16, 21, 12 },		// 147 bits per transform
			{ 16, 22, 11 },		// 147 bits per transform
			{ 16, 23, 10 },		// 147 bits per transform
			{ 16, 24, 1 },		// 147 bits per transform
			{ 17, 0, 24 },		// 147 bits per transform
			{ 17, 9, 23 },		// 147 bits per transform
			{ 17, 10, 22 },		// 147 bits per transform
			{ 17, 11, 21 },		// 147 bits per transform
			{ 17, 12, 20 },		// 147 bits per transform
			{ 17, 13, 19 },		// 147 bits per transform
			{ 17, 14, 18 },		// 147 bits per transform
			{ 17, 15, 17 },		// 147 bits per transform
			{ 17, 16, 16 },		// 147 bits per transform
			{ 17, 17, 15 },		// 147 bits per transform
			{ 17, 18, 14 },		// 147 bits per transform
			{ 17, 19, 13 },		// 147 bits per transform
			{ 17, 20, 12 },		// 147 bits per transform
			{ 17, 21, 11 },		// 147 bits per transform
			{ 17, 22, 10 },		// 147 bits per transform
			{ 17, 23, 9 },		// 147 bits per transform
			{ 17, 24, 0 },		// 147 bits per transform
			{ 18, 8, 23 },		// 147 bits per transform
			{ 18, 9, 22 },		// 147 bits per transform
			{ 18, 10, 21 },		// 147 bits per transform
			{ 18, 11, 20 },		// 147 bits per transform
			{ 18, 12, 19 },		// 147 bits per transform
			{ 18, 13, 18 },		// 147 bits per transform
			{ 18, 14, 17 },		// 147 bits per transform
			{ 18, 15, 16 },		// 147 bits per transform
			{ 18, 16, 15 },		// 147 bits per transform
			{ 18, 17, 14 },		// 147 bits per transform
			{ 18, 18, 13 },		// 147 bits per transform
			{ 18, 19, 12 },		// 147 bits per transform
			{ 18, 20, 11 },		// 147 bits per transform
			{ 18, 21, 10 },		// 147 bits per transform
			{ 18, 22, 9 },		// 147 bits per transform
			{ 18, 23, 8 },		// 147 bits per transform
			{ 19, 7, 23 },		// 147 bits per transform
			{ 19, 8, 22 },		// 147 bits per transform
			{ 19, 9, 21 },		// 147 bits per transform
			{ 19, 10, 20 },		// 147 bits per transform
			{ 19, 11, 19 },		// 147 bits per transform
			{ 19, 12, 18 },		// 147 bits per transform
			{ 19, 13, 17 },		// 147 bits per transform
			{ 19, 14, 16 },		// 147 bits per transform
			{ 19, 15, 15 },		// 147 bits per transform
			{ 19, 16, 14 },		// 147 bits per transform
			{ 19, 17, 13 },		// 147 bits per transform
			{ 19, 18, 12 },		// 147 bits per transform
			{ 19, 19, 11 },		// 147 bits per transform
			{ 19, 20, 10 },		// 147 bits per transform
			{ 19, 21, 9 },		// 147 bits per transform
			{ 19, 22, 8 },		// 147 bits per transform
			{ 19, 23, 7 },		// 147 bits per transform
			{ 20, 6, 23 },		// 147 bits per transform
			{ 20, 7, 22 },		// 147 bits per transform
			{ 20, 8, 21 },		// 147 bits per transform
			{ 20, 9, 20 },		// 147 bits per transform
			{ 20, 10, 19 },		// 147 bits per transform
			{ 20, 11, 18 },		// 147 bits per transform
			{ 20, 12, 17 },		// 147 bits per transform
			{ 20, 13, 16 },		// 147 bits per transform
			{ 20, 14, 15 },		// 147 bits per transform
			{ 20, 15, 14 },		// 147 bits per transform
			{ 20, 16, 13 },		// 147 bits per transform
			{ 20, 17, 12 },		// 147 bits per transform
			{ 20, 18, 11 },		// 147 bits per transform
			{ 20, 19, 10 },		// 147 bits per transform
			{ 20, 20, 9 },		// 147 bits per transform
			{ 20, 21, 8 },		// 147 bits per transform
			{ 20, 22, 7 },		// 147 bits per transform
			{ 20, 23, 6 },		// 147 bits per transform
			{ 21, 5, 23 },		// 147 bits per transform
			{ 21, 6, 22 },		// 147 bits per transform
			{ 21, 7, 21 },		// 147 bits per transform
			{ 21, 8, 20 },		// 147 bits per transform
			{ 21, 9, 19 },		// 147 bits per transform
			{ 21, 10, 18 },		// 147 bits per transform
			{ 21, 11, 17 },		// 147 bits per transform
			{ 21, 12, 16 },		// 147 bits per transform
			{ 21, 13, 15 },		// 147 bits per transform
			{ 21, 14, 14 },		// 147 bits per transform
			{ 21, 15, 13 },		// 147 bits per transform
			{ 21, 16, 12 },		// 147 bits per transform
			{ 21, 17, 11 },		// 147 bits per transform
			{ 21, 18, 10 },		// 147 bits per transform
			{ 21, 19, 9 },		// 147 bits per transform
			{ 21, 20, 8 },		// 147 bits per transform
			{ 21, 21, 7 },		// 147 bits per transform
			{ 21, 22, 6 },		// 147 bits per transform
			{ 21, 23, 5 },		// 147 bits per transform
			{ 22, 4, 23 },		// 147 bits per transform
			{ 22, 5, 22 },		// 147 bits per transform
			{ 22, 6, 21 },		// 147 bits per transform
			{ 22, 7, 20 },		// 147 bits per transform
			{ 22, 8, 19 },		// 147 bits per transform
			{ 22, 9, 18 },		// 147 bits per transform
			{ 22, 10, 17 },		// 147 bits per transform
			{ 22, 11, 16 },		// 147 bits per transform
			{ 22, 12, 15 },		// 147 bits per transform
			{ 22, 13, 14 },		// 147 bits per transform
			{ 22, 14, 13 },		// 147 bits per transform
			{ 22, 15, 12 },		// 147 bits per transform
			{ 22, 16, 11 },		// 147 bits per transform
			{ 22, 17, 10 },		// 147 bits per transform
			{ 22, 18, 9 },		// 147 bits per transform
			{ 22, 19, 8 },		// 147 bits per transform
			{ 22, 20, 7 },		// 147 bits per transform
			{ 22, 21, 6 },		// 147 bits per transform
			{ 22, 22, 5 },		// 147 bits per transform
			{ 22, 23, 4 },		// 147 bits per transform
			{ 23, 3, 23 },		// 147 bits per transform
			{ 23, 4, 22 },		// 147 bits per transform
			{ 23, 5, 21 },		// 147 bits per transform
			{ 23, 6, 20 },		// 147 bits per transform
			{ 23, 7, 19 },		// 147 bits per transform
			{ 23, 8, 18 },		// 147 bits per transform
			{ 23, 9, 17 },		// 147 bits per transform
			{ 23, 10, 16 },		// 147 bits per transform
			{ 23, 11, 15 },		// 147 bits per transform
			{ 23, 12, 14 },		// 147 bits per transform
			{ 23, 13, 13 },		// 147 bits per transform
			{ 23, 14, 12 },		// 147 bits per transform
			{ 23, 15, 11 },		// 147 bits per transform
			{ 23, 16, 10 },		// 147 bits per transform
			{ 23, 17, 9 },		// 147 bits per transform
			{ 23, 18, 8 },		// 147 bits per transform
			{ 23, 19, 7 },		// 147 bits per transform
			{ 23, 20, 6 },		// 147 bits per transform
			{ 23, 21, 5 },		// 147 bits per transform
			{ 23, 22, 4 },		// 147 bits per transform
			{ 23, 23, 3 },		// 147 bits per transform
			{ 24, 0, 17 },		// 147 bits per transform
			{ 24, 1, 16 },		// 147 bits per transform
			{ 24, 2, 15 },		// 147 bits per transform
			{ 24, 3, 14 },		// 147 bits per transform
			{ 24, 4, 13 },		// 147 bits per transform
			{ 24, 5, 12 },		// 147 bits per transform
			{ 24, 6, 11 },		// 147 bits per transform
			{ 24, 7, 10 },		// 147 bits per transform
			{ 24, 8, 9 },		// 147 bits per transform
			{ 24, 9, 8 },		// 147 bits per transform
			{ 24, 10, 7 },		// 147 bits per transform
			{ 24, 11, 6 },		// 147 bits per transform
			{ 24, 12, 5 },		// 147 bits per transform
			{ 24, 13, 4 },		// 147 bits per transform
			{ 24, 14, 3 },		// 147 bits per transform
			{ 24, 15, 2 },		// 147 bits per transform
			{ 24, 16, 1 },		// 147 bits per transform
			{ 24, 17, 0 },		// 147 bits per transform
			{ 0, 18, 24 },		// 150 bits per transform
			{ 0, 24, 18 },		// 150 bits per transform
			{ 1, 17, 24 },		// 150 bits per transform
			{ 1, 24, 17 },		// 150 bits per transform
			{ 2, 16, 24 },		// 150 bits per transform
			{ 2, 24, 16 },		// 150 bits per transform
			{ 3, 15, 24 },		// 150 bits per transform
			{ 3, 24, 15 },		// 150 bits per transform
			{ 4, 14, 24 },		// 150 bits per transform
			{ 4, 23, 23 },		// 150 bits per transform
			{ 4, 24, 14 },		// 150 bits per transform
			{ 5, 13, 24 },		// 150 bits per transform
			{ 5, 22, 23 },		// 150 bits per transform
			{ 5, 23, 22 },		// 150 bits per transform
			{ 5, 24, 13 },		// 150 bits per transform
			{ 6, 12, 24 },		// 150 bits per transform
			{ 6, 21, 23 },		// 150 bits per transform
			{ 6, 22, 22 },		// 150 bits per transform
			{ 6, 23, 21 },		// 150 bits per transform
			{ 6, 24, 12 },		// 150 bits per transform
			{ 7, 11, 24 },		// 150 bits per transform
			{ 7, 20, 23 },		// 150 bits per transform
			{ 7, 21, 22 },		// 150 bits per transform
			{ 7, 22, 21 },		// 150 bits per transform
			{ 7, 23, 20 },		// 150 bits per transform
			{ 7, 24, 11 },		// 150 bits per transform
			{ 8, 10, 24 },		// 150 bits per transform
			{ 8, 19, 23 },		// 150 bits per transform
			{ 8, 20, 22 },		// 150 bits per transform
			{ 8, 21, 21 },		// 150 bits per transform
			{ 8, 22, 20 },		// 150 bits per transform
			{ 8, 23, 19 },		// 150 bits per transform
			{ 8, 24, 10 },		// 150 bits per transform
			{ 9, 9, 24 },		// 150 bits per transform
			{ 9, 18, 23 },		// 150 bits per transform
			{ 9, 19, 22 },		// 150 bits per transform
			{ 9, 20, 21 },		// 150 bits per transform
			{ 9, 21, 20 },		// 150 bits per transform
			{ 9, 22, 19 },		// 150 bits per transform
			{ 9, 23, 18 },		// 150 bits per transform
			{ 9, 24, 9 },		// 150 bits per transform
			{ 10, 8, 24 },		// 150 bits per transform
			{ 10, 17, 23 },		// 150 bits per transform
			{ 10, 18, 22 },		// 150 bits per transform
			{ 10, 19, 21 },		// 150 bits per transform
			{ 10, 20, 20 },		// 150 bits per transform
			{ 10, 21, 19 },		// 150 bits per transform
			{ 10, 22, 18 },		// 150 bits per transform
			{ 10, 23, 17 },		// 150 bits per transform
			{ 10, 24, 8 },		// 150 bits per transform
			{ 11, 7, 24 },		// 150 bits per transform
			{ 11, 16, 23 },		// 150 bits per transform
			{ 11, 17, 22 },		// 150 bits per transform
			{ 11, 18, 21 },		// 150 bits per transform
			{ 11, 19, 20 },		// 150 bits per transform
			{ 11, 20, 19 },		// 150 bits per transform
			{ 11, 21, 18 },		// 150 bits per transform
			{ 11, 22, 17 },		// 150 bits per transform
			{ 11, 23, 16 },		// 150 bits per transform
			{ 11, 24, 7 },		// 150 bits per transform
			{ 12, 6, 24 },		// 150 bits per transform
			{ 12, 15, 23 },		// 150 bits per transform
			{ 12, 16, 22 },		// 150 bits per transform
			{ 12, 17, 21 },		// 150 bits per transform
			{ 12, 18, 20 },		// 150 bits per transform
			{ 12, 19, 19 },		// 150 bits per transform
			{ 12, 20, 18 },		// 150 bits per transform
			{ 12, 21, 17 },		// 150 bits per transform
			{ 12, 22, 16 },		// 150 bits per transform
			{ 12, 23, 15 },		// 150 bits per transform
			{ 12, 24, 6 },		// 150 bits per transform
			{ 13, 5, 24 },		// 150 bits per transform
			{ 13, 14, 23 },		// 150 bits per transform
			{ 13, 15, 22 },		// 150 bits per transform
			{ 13, 16, 21 },		// 150 bits per transform
			{ 13, 17, 20 },		// 150 bits per transform
			{ 13, 18, 19 },		// 150 bits per transform
			{ 13, 19, 18 },		// 150 bits per transform
			{ 13, 20, 17 },		// 150 bits per transform
			{ 13, 21, 16 },		// 150 bits per transform
			{ 13, 22, 15 },		// 150 bits per transform
			{ 13, 23, 14 },		// 150 bits per transform
			{ 13, 24, 5 },		// 150 bits per transform
			{ 14, 4, 24 },		// 150 bits per transform
			{ 14, 13, 23 },		// 150 bits per transform
			{ 14, 14, 22 },		// 150 bits per transform
			{ 14, 15, 21 },		// 150 bits per transform
			{ 14, 16, 20 },		// 150 bits per transform
			{ 14, 17, 19 },		// 150 bits per transform
			{ 14, 18, 18 },		// 150 bits per transform
			{ 14, 19, 17 },		// 150 bits per transform
			{ 14, 20, 16 },		// 150 bits per transform
			{ 14, 21, 15 },		// 150 bits per transform
			{ 14, 22, 14 },		// 150 bits per transform
			{ 14, 23, 13 },		// 150 bits per transform
			{ 14, 24, 4 },		// 150 bits per transform
			{ 15, 3, 24 },		// 150 bits per transform
			{ 15, 12, 23 },		// 150 bits per transform
			{ 15, 13, 22 },		// 150 bits per transform
			{ 15, 14, 21 },		// 150 bits per transform
			{ 15, 15, 20 },		// 150 bits per transform
			{ 15, 16, 19 },		// 150 bits per transform
			{ 15, 17, 18 },		// 150 bits per transform
			{ 15, 18, 17 },		// 150 bits per transform
			{ 15, 19, 16 },		// 150 bits per transform
			{ 15, 20, 15 },		// 150 bits per transform
			{ 15, 21, 14 },		// 150 bits per transform
			{ 15, 22, 13 },		// 150 bits per transform
			{ 15, 23, 12 },		// 150 bits per transform
			{ 15, 24, 3 },		// 150 bits per transform
			{ 16, 2, 24 },		// 150 bits per transform
			{ 16, 11, 23 },		// 150 bits per transform
			{ 16, 12, 22 },		// 150 bits per transform
			{ 16, 13, 21 },		// 150 bits per transform
			{ 16, 14, 20 },		// 150 bits per transform
			{ 16, 15, 19 },		// 150 bits per transform
			{ 16, 16, 18 },		// 150 bits per transform
			{ 16, 17, 17 },		// 150 bits per transform
			{ 16, 18, 16 },		// 150 bits per transform
			{ 16, 19, 15 },		// 150 bits per transform
			{ 16, 20, 14 },		// 150 bits per transform
			{ 16, 21, 13 },		// 150 bits per transform
			{ 16, 22, 12 },		// 150 bits per transform
			{ 16, 23, 11 },		// 150 bits per transform
			{ 16, 24, 2 },		// 150 bits per transform
			{ 17, 1, 24 },		// 150 bits per transform
			{ 17, 10, 23 },		// 150 bits per transform
			{ 17, 11, 22 },		// 150 bits per transform
			{ 17, 12, 21 },		// 150 bits per transform
			{ 17, 13, 20 },		// 150 bits per transform
			{ 17, 14, 19 },		// 150 bits per transform
			{ 17, 15, 18 },		// 150 bits per transform
			{ 17, 16, 17 },		// 150 bits per transform
			{ 17, 17, 16 },		// 150 bits per transform
			{ 17, 18, 15 },		// 150 bits per transform
			{ 17, 19, 14 },		// 150 bits per transform
			{ 17, 20, 13 },		// 150 bits per transform
			{ 17, 21, 12 },		// 150 bits per transform
			{ 17, 22, 11 },		// 150 bits per transform
			{ 17, 23, 10 },		// 150 bits per transform
			{ 17, 24, 1 },		// 150 bits per transform
			{ 18, 0, 24 },		// 150 bits per transform
			{ 18, 9, 23 },		// 150 bits per transform
			{ 18, 10, 22 },		// 150 bits per transform
			{ 18, 11, 21 },		// 150 bits per transform
			{ 18, 12, 20 },		// 150 bits per transform
			{ 18, 13, 19 },		// 150 bits per transform
			{ 18, 14, 18 },		// 150 bits per transform
			{ 18, 15, 17 },		// 150 bits per transform
			{ 18, 16, 16 },		// 150 bits per transform
			{ 18, 17, 15 },		// 150 bits per transform
			{ 18, 18, 14 },		// 150 bits per transform
			{ 18, 19, 13 },		// 150 bits per transform
			{ 18, 20, 12 },		// 150 bits per transform
			{ 18, 21, 11 },		// 150 bits per transform
			{ 18, 22, 10 },		// 150 bits per transform
			{ 18, 23, 9 },		// 150 bits per transform
			{ 18, 24, 0 },		// 150 bits per transform
			{ 19, 8, 23 },		// 150 bits per transform
			{ 19, 9, 22 },		// 150 bits per transform
			{ 19, 10, 21 },		// 150 bits per transform
			{ 19, 11, 20 },		// 150 bits per transform
			{ 19, 12, 19 },		// 150 bits per transform
			{ 19, 13, 18 },		// 150 bits per transform
			{ 19, 14, 17 },		// 150 bits per transform
			{ 19, 15, 16 },		// 150 bits per transform
			{ 19, 16, 15 },		// 150 bits per transform
			{ 19, 17, 14 },		// 150 bits per transform
			{ 19, 18, 13 },		// 150 bits per transform
			{ 19, 19, 12 },		// 150 bits per transform
			{ 19, 20, 11 },		// 150 bits per transform
			{ 19, 21, 10 },		// 150 bits per transform
			{ 19, 22, 9 },		// 150 bits per transform
			{ 19, 23, 8 },		// 150 bits per transform
			{ 20, 7, 23 },		// 150 bits per transform
			{ 20, 8, 22 },		// 150 bits per transform
			{ 20, 9, 21 },		// 150 bits per transform
			{ 20, 10, 20 },		// 150 bits per transform
			{ 20, 11, 19 },		// 150 bits per transform
			{ 20, 12, 18 },		// 150 bits per transform
			{ 20, 13, 17 },		// 150 bits per transform
			{ 20, 14, 16 },		// 150 bits per transform
			{ 20, 15, 15 },		// 150 bits per transform
			{ 20, 16, 14 },		// 150 bits per transform
			{ 20, 17, 13 },		// 150 bits per transform
			{ 20, 18, 12 },		// 150 bits per transform
			{ 20, 19, 11 },		// 150 bits per transform
			{ 20, 20, 10 },		// 150 bits per transform
			{ 20, 21, 9 },		// 150 bits per transform
			{ 20, 22, 8 },		// 150 bits per transform
			{ 20, 23, 7 },		// 150 bits per transform
			{ 21, 6, 23 },		// 150 bits per transform
			{ 21, 7, 22 },		// 150 bits per transform
			{ 21, 8, 21 },		// 150 bits per transform
			{ 21, 9, 20 },		// 150 bits per transform
			{ 21, 10, 19 },		// 150 bits per transform
			{ 21, 11, 18 },		// 150 bits per transform
			{ 21, 12, 17 },		// 150 bits per transform
			{ 21, 13, 16 },		// 150 bits per transform
			{ 21, 14, 15 },		// 150 bits per transform
			{ 21, 15, 14 },		// 150 bits per transform
			{ 21, 16, 13 },		// 150 bits per transform
			{ 21, 17, 12 },		// 150 bits per transform
			{ 21, 18, 11 },		// 150 bits per transform
			{ 21, 19, 10 },		// 150 bits per transform
			{ 21, 20, 9 },		// 150 bits per transform
			{ 21, 21, 8 },		// 150 bits per transform
			{ 21, 22, 7 },		// 150 bits per transform
			{ 21, 23, 6 },		// 150 bits per transform
			{ 22, 5, 23 },		// 150 bits per transform
			{ 22, 6, 22 },		// 150 bits per transform
			{ 22, 7, 21 },		// 150 bits per transform
			{ 22, 8, 20 },		// 150 bits per transform
			{ 22, 9, 19 },		// 150 bits per transform
			{ 22, 10, 18 },		// 150 bits per transform
			{ 22, 11, 17 },		// 150 bits per transform
			{ 22, 12, 16 },		// 150 bits per transform
			{ 22, 13, 15 },		// 150 bits per transform
			{ 22, 14, 14 },		// 150 bits per transform
			{ 22, 15, 13 },		// 150 bits per transform
			{ 22, 16, 12 },		// 150 bits per transform
			{ 22, 17, 11 },		// 150 bits per transform
			{ 22, 18, 10 },		// 150 bits per transform
			{ 22, 19, 9 },		// 150 bits per transform
			{ 22, 20, 8 },		// 150 bits per transform
			{ 22, 21, 7 },		// 150 bits per transform
			{ 22, 22, 6 },		// 150 bits per transform
			{ 22, 23, 5 },		// 150 bits per transform
			{ 23, 4, 23 },		// 150 bits per transform
			{ 23, 5, 22 },		// 150 bits per transform
			{ 23, 6, 21 },		// 150 bits per transform
			{ 23, 7, 20 },		// 150 bits per transform
			{ 23, 8, 19 },		// 150 bits per transform
			{ 23, 9, 18 },		// 150 bits per transform
			{ 23, 10, 17 },		// 150 bits per transform
			{ 23, 11, 16 },		// 150 bits per transform
			{ 23, 12, 15 },		// 150 bits per transform
			{ 23, 13, 14 },		// 150 bits per transform
			{ 23, 14, 13 },		// 150 bits per transform
			{ 23, 15, 12 },		// 150 bits per transform
			{ 23, 16, 11 },		// 150 bits per transform
			{ 23, 17, 10 },		// 150 bits per transform
			{ 23, 18, 9 },		// 150 bits per transform
			{ 23, 19, 8 },		// 150 bits per transform
			{ 23, 20, 7 },		// 150 bits per transform
			{ 23, 21, 6 },		// 150 bits per transform
			{ 23, 22, 5 },		// 150 bits per transform
			{ 23, 23, 4 },		// 150 bits per transform
			{ 24, 0, 18 },		// 150 bits per transform
			{ 24, 1, 17 },		// 150 bits per transform
			{ 24, 2, 16 },		// 150 bits per transform
			{ 24, 3, 15 },		// 150 bits per transform
			{ 24, 4, 14 },		// 150 bits per transform
			{ 24, 5, 13 },		// 150 bits per transform
			{ 24, 6, 12 },		// 150 bits per transform
			{ 24, 7, 11 },		// 150 bits per transform
			{ 24, 8, 10 },		// 150 bits per transform
			{ 24, 9, 9 },		// 150 bits per transform
			{ 24, 10, 8 },		// 150 bits per transform
			{ 24, 11, 7 },		// 150 bits per transform
			{ 24, 12, 6 },		// 150 bits per transform
			{ 24, 13, 5 },		// 150 bits per transform
			{ 24, 14, 4 },		// 150 bits per transform
			{ 24, 15, 3 },		// 150 bits per transform
			{ 24, 16, 2 },		// 150 bits per transform
			{ 24, 17, 1 },		// 150 bits per transform
			{ 24, 18, 0 },		// 150 bits per transform
			{ 0, 19, 24 },		// 153 bits per transform
			{ 0, 24, 19 },		// 153 bits per transform
			{ 1, 18, 24 },		// 153 bits per transform
			{ 1, 24, 18 },		// 153 bits per transform
			{ 2, 17, 24 },		// 153 bits per transform
			{ 2, 24, 17 },		// 153 bits per transform
			{ 3, 16, 24 },		// 153 bits per transform
			{ 3, 24, 16 },		// 153 bits per transform
			{ 4, 15, 24 },		// 153 bits per transform
			{ 4, 24, 15 },		// 153 bits per transform
			{ 5, 14, 24 },		// 153 bits per transform
			{ 5, 23, 23 },		// 153 bits per transform
			{ 5, 24, 14 },		// 153 bits per transform
			{ 6, 13, 24 },		// 153 bits per transform
			{ 6, 22, 23 },		// 153 bits per transform
			{ 6, 23, 22 },		// 153 bits per transform
			{ 6, 24, 13 },		// 153 bits per transform
			{ 7, 12, 24 },		// 153 bits per transform
			{ 7, 21, 23 },		// 153 bits per transform
			{ 7, 22, 22 },		// 153 bits per transform
			{ 7, 23, 21 },		// 153 bits per transform
			{ 7, 24, 12 },		// 153 bits per transform
			{ 8, 11, 24 },		// 153 bits per transform
			{ 8, 20, 23 },		// 153 bits per transform
			{ 8, 21, 22 },		// 153 bits per transform
			{ 8, 22, 21 },		// 153 bits per transform
			{ 8, 23, 20 },		// 153 bits per transform
			{ 8, 24, 11 },		// 153 bits per transform
			{ 9, 10, 24 },		// 153 bits per transform
			{ 9, 19, 23 },		// 153 bits per transform
			{ 9, 20, 22 },		// 153 bits per transform
			{ 9, 21, 21 },		// 153 bits per transform
			{ 9, 22, 20 },		// 153 bits per transform
			{ 9, 23, 19 },		// 153 bits per transform
			{ 9, 24, 10 },		// 153 bits per transform
			{ 10, 9, 24 },		// 153 bits per transform
			{ 10, 18, 23 },		// 153 bits per transform
			{ 10, 19, 22 },		// 153 bits per transform
			{ 10, 20, 21 },		// 153 bits per transform
			{ 10, 21, 20 },		// 153 bits per transform
			{ 10, 22, 19 },		// 153 bits per transform
			{ 10, 23, 18 },		// 153 bits per transform
			{ 10, 24, 9 },		// 153 bits per transform
			{ 11, 8, 24 },		// 153 bits per transform
			{ 11, 17, 23 },		// 153 bits per transform
			{ 11, 18, 22 },		// 153 bits per transform
			{ 11, 19, 21 },		// 153 bits per transform
			{ 11, 20, 20 },		// 153 bits per transform
			{ 11, 21, 19 },		// 153 bits per transform
			{ 11, 22, 18 },		// 153 bits per transform
			{ 11, 23, 17 },		// 153 bits per transform
			{ 11, 24, 8 },		// 153 bits per transform
			{ 12, 7, 24 },		// 153 bits per transform
			{ 12, 16, 23 },		// 153 bits per transform
			{ 12, 17, 22 },		// 153 bits per transform
			{ 12, 18, 21 },		// 153 bits per transform
			{ 12, 19, 20 },		// 153 bits per transform
			{ 12, 20, 19 },		// 153 bits per transform
			{ 12, 21, 18 },		// 153 bits per transform
			{ 12, 22, 17 },		// 153 bits per transform
			{ 12, 23, 16 },		// 153 bits per transform
			{ 12, 24, 7 },		// 153 bits per transform
			{ 13, 6, 24 },		// 153 bits per transform
			{ 13, 15, 23 },		// 153 bits per transform
			{ 13, 16, 22 },		// 153 bits per transform
			{ 13, 17, 21 },		// 153 bits per transform
			{ 13, 18, 20 },		// 153 bits per transform
			{ 13, 19, 19 },		// 153 bits per transform
			{ 13, 20, 18 },		// 153 bits per transform
			{ 13, 21, 17 },		// 153 bits per transform
			{ 13, 22, 16 },		// 153 bits per transform
			{ 13, 23, 15 },		// 153 bits per transform
			{ 13, 24, 6 },		// 153 bits per transform
			{ 14, 5, 24 },		// 153 bits per transform
			{ 14, 14, 23 },		// 153 bits per transform
			{ 14, 15, 22 },		// 153 bits per transform
			{ 14, 16, 21 },		// 153 bits per transform
			{ 14, 17, 20 },		// 153 bits per transform
			{ 14, 18, 19 },		// 153 bits per transform
			{ 14, 19, 18 },		// 153 bits per transform
			{ 14, 20, 17 },		// 153 bits per transform
			{ 14, 21, 16 },		// 153 bits per transform
			{ 14, 22, 15 },		// 153 bits per transform
			{ 14, 23, 14 },		// 153 bits per transform
			{ 14, 24, 5 },		// 153 bits per transform
			{ 15, 4, 24 },		// 153 bits per transform
			{ 15, 13, 23 },		// 153 bits per transform
			{ 15, 14, 22 },		// 153 bits per transform
			{ 15, 15, 21 },		// 153 bits per transform
			{ 15, 16, 20 },		// 153 bits per transform
			{ 15, 17, 19 },		// 153 bits per transform
			{ 15, 18, 18 },		// 153 bits per transform
			{ 15, 19, 17 },		// 153 bits per transform
			{ 15, 20, 16 },		// 153 bits per transform
			{ 15, 21, 15 },		// 153 bits per transform
			{ 15, 22, 14 },		// 153 bits per transform
			{ 15, 23, 13 },		// 153 bits per transform
			{ 15, 24, 4 },		// 153 bits per transform
			{ 16, 3, 24 },		// 153 bits per transform
			{ 16, 12, 23 },		// 153 bits per transform
			{ 16, 13, 22 },		// 153 bits per transform
			{ 16, 14, 21 },		// 153 bits per transform
			{ 16, 15, 20 },		// 153 bits per transform
			{ 16, 16, 19 },		// 153 bits per transform
			{ 16, 17, 18 },		// 153 bits per transform
			{ 16, 18, 17 },		// 153 bits per transform
			{ 16, 19, 16 },		// 153 bits per transform
			{ 16, 20, 15 },		// 153 bits per transform
			{ 16, 21, 14 },		// 153 bits per transform
			{ 16, 22, 13 },		// 153 bits per transform
			{ 16, 23, 12 },		// 153 bits per transform
			{ 16, 24, 3 },		// 153 bits per transform
			{ 17, 2, 24 },		// 153 bits per transform
			{ 17, 11, 23 },		// 153 bits per transform
			{ 17, 12, 22 },		// 153 bits per transform
			{ 17, 13, 21 },		// 153 bits per transform
			{ 17, 14, 20 },		// 153 bits per transform
			{ 17, 15, 19 },		// 153 bits per transform
			{ 17, 16, 18 },		// 153 bits per transform
			{ 17, 17, 17 },		// 153 bits per transform
			{ 17, 18, 16 },		// 153 bits per transform
			{ 17, 19, 15 },		// 153 bits per transform
			{ 17, 20, 14 },		// 153 bits per transform
			{ 17, 21, 13 },		// 153 bits per transform
			{ 17, 22, 12 },		// 153 bits per transform
			{ 17, 23, 11 },		// 153 bits per transform
			{ 17, 24, 2 },		// 153 bits per transform
			{ 18, 1, 24 },		// 153 bits per transform
			{ 18, 10, 23 },		// 153 bits per transform
			{ 18, 11, 22 },		// 153 bits per transform
			{ 18, 12, 21 },		// 153 bits per transform
			{ 18, 13, 20 },		// 153 bits per transform
			{ 18, 14, 19 },		// 153 bits per transform
			{ 18, 15, 18 },		// 153 bits per transform
			{ 18, 16, 17 },		// 153 bits per transform
			{ 18, 17, 16 },		// 153 bits per transform
			{ 18, 18, 15 },		// 153 bits per transform
			{ 18, 19, 14 },		// 153 bits per transform
			{ 18, 20, 13 },		// 153 bits per transform
			{ 18, 21, 12 },		// 153 bits per transform
			{ 18, 22, 11 },		// 153 bits per transform
			{ 18, 23, 10 },		// 153 bits per transform
			{ 18, 24, 1 },		// 153 bits per transform
			{ 19, 0, 24 },		// 153 bits per transform
			{ 19, 9, 23 },		// 153 bits per transform
			{ 19, 10, 22 },		// 153 bits per transform
			{ 19, 11, 21 },		// 153 bits per transform
			{ 19, 12, 20 },		// 153 bits per transform
			{ 19, 13, 19 },		// 153 bits per transform
			{ 19, 14, 18 },		// 153 bits per transform
			{ 19, 15, 17 },		// 153 bits per transform
			{ 19, 16, 16 },		// 153 bits per transform
			{ 19, 17, 15 },		// 153 bits per transform
			{ 19, 18, 14 },		// 153 bits per transform
			{ 19, 19, 13 },		// 153 bits per transform
			{ 19, 20, 12 },		// 153 bits per transform
			{ 19, 21, 11 },		// 153 bits per transform
			{ 19, 22, 10 },		// 153 bits per transform
			{ 19, 23, 9 },		// 153 bits per transform
			{ 19, 24, 0 },		// 153 bits per transform
			{ 20, 8, 23 },		// 153 bits per transform
			{ 20, 9, 22 },		// 153 bits per transform
			{ 20, 10, 21 },		// 153 bits per transform
			{ 20, 11, 20 },		// 153 bits per transform
			{ 20, 12, 19 },		// 153 bits per transform
			{ 20, 13, 18 },		// 153 bits per transform
			{ 20, 14, 17 },		// 153 bits per transform
			{ 20, 15, 16 },		// 153 bits per transform
			{ 20, 16, 15 },		// 153 bits per transform
			{ 20, 17, 14 },		// 153 bits per transform
			{ 20, 18, 13 },		// 153 bits per transform
			{ 20, 19, 12 },		// 153 bits per transform
			{ 20, 20, 11 },		// 153 bits per transform
			{ 20, 21, 10 },		// 153 bits per transform
			{ 20, 22, 9 },		// 153 bits per transform
			{ 20, 23, 8 },		// 153 bits per transform
			{ 21, 7, 23 },		// 153 bits per transform
			{ 21, 8, 22 },		// 153 bits per transform
			{ 21, 9, 21 },		// 153 bits per transform
			{ 21, 10, 20 },		// 153 bits per transform
			{ 21, 11, 19 },		// 153 bits per transform
			{ 21, 12, 18 },		// 153 bits per transform
			{ 21, 13, 17 },		// 153 bits per transform
			{ 21, 14, 16 },		// 153 bits per transform
			{ 21, 15, 15 },		// 153 bits per transform
			{ 21, 16, 14 },		// 153 bits per transform
			{ 21, 17, 13 },		// 153 bits per transform
			{ 21, 18, 12 },		// 153 bits per transform
			{ 21, 19, 11 },		// 153 bits per transform
			{ 21, 20, 10 },		// 153 bits per transform
			{ 21, 21, 9 },		// 153 bits per transform
			{ 21, 22, 8 },		// 153 bits per transform
			{ 21, 23, 7 },		// 153 bits per transform
			{ 22, 6, 23 },		// 153 bits per transform
			{ 22, 7, 22 },		// 153 bits per transform
			{ 22, 8, 21 },		// 153 bits per transform
			{ 22, 9, 20 },		// 153 bits per transform
			{ 22, 10, 19 },		// 153 bits per transform
			{ 22, 11, 18 },		// 153 bits per transform
			{ 22, 12, 17 },		// 153 bits per transform
			{ 22, 13, 16 },		// 153 bits per transform
			{ 22, 14, 15 },		// 153 bits per transform
			{ 22, 15, 14 },		// 153 bits per transform
			{ 22, 16, 13 },		// 153 bits per transform
			{ 22, 17, 12 },		// 153 bits per transform
			{ 22, 18, 11 },		// 153 bits per transform
			{ 22, 19, 10 },		// 153 bits per transform
			{ 22, 20, 9 },		// 153 bits per transform
			{ 22, 21, 8 },		// 153 bits per transform
			{ 22, 22, 7 },		// 153 bits per transform
			{ 22, 23, 6 },		// 153 bits per transform
			{ 23, 5, 23 },		// 153 bits per transform
			{ 23, 6, 22 },		// 153 bits per transform
			{ 23, 7, 21 },		// 153 bits per transform
			{ 23, 8, 20 },		// 153 bits per transform
			{ 23, 9, 19 },		// 153 bits per transform
			{ 23, 10, 18 },		// 153 bits per transform
			{ 23, 11, 17 },		// 153 bits per transform
			{ 23, 12, 16 },		// 153 bits per transform
			{ 23, 13, 15 },		// 153 bits per transform
			{ 23, 14, 14 },		// 153 bits per transform
			{ 23, 15, 13 },		// 153 bits per transform
			{ 23, 16, 12 },		// 153 bits per transform
			{ 23, 17, 11 },		// 153 bits per transform
			{ 23, 18, 10 },		// 153 bits per transform
			{ 23, 19, 9 },		// 153 bits per transform
			{ 23, 20, 8 },		// 153 bits per transform
			{ 23, 21, 7 },		// 153 bits per transform
			{ 23, 22, 6 },		// 153 bits per transform
			{ 23, 23, 5 },		// 153 bits per transform
			{ 24, 0, 19 },		// 153 bits per transform
			{ 24, 1, 18 },		// 153 bits per transform
			{ 24, 2, 17 },		// 153 bits per transform
			{ 24, 3, 16 },		// 153 bits per transform
			{ 24, 4, 15 },		// 153 bits per transform
			{ 24, 5, 14 },		// 153 bits per transform
			{ 24, 6, 13 },		// 153 bits per transform
			{ 24, 7, 12 },		// 153 bits per transform
			{ 24, 8, 11 },		// 153 bits per transform
			{ 24, 9, 10 },		// 153 bits per transform
			{ 24, 10, 9 },		// 153 bits per transform
			{ 24, 11, 8 },		// 153 bits per transform
			{ 24, 12, 7 },		// 153 bits per transform
			{ 24, 13, 6 },		// 153 bits per transform
			{ 24, 14, 5 },		// 153 bits per transform
			{ 24, 15, 4 },		// 153 bits per transform
			{ 24, 16, 3 },		// 153 bits per transform
			{ 24, 17, 2 },		// 153 bits per transform
			{ 24, 18, 1 },		// 153 bits per transform
			{ 24, 19, 0 },		// 153 bits per transform
			{ 0, 20, 24 },		// 156 bits per transform
			{ 0, 24, 20 },		// 156 bits per transform
			{ 1, 19, 24 },		// 156 bits per transform
			{ 1, 24, 19 },		// 156 bits per transform
			{ 2, 18, 24 },		// 156 bits per transform
			{ 2, 24, 18 },		// 156 bits per transform
			{ 3, 17, 24 },		// 156 bits per transform
			{ 3, 24, 17 },		// 156 bits per transform
			{ 4, 16, 24 },		// 156 bits per transform
			{ 4, 24, 16 },		// 156 bits per transform
			{ 5, 15, 24 },		// 156 bits per transform
			{ 5, 24, 15 },		// 156 bits per transform
			{ 6, 14, 24 },		// 156 bits per transform
			{ 6, 23, 23 },		// 156 bits per transform
			{ 6, 24, 14 },		// 156 bits per transform
			{ 7, 13, 24 },		// 156 bits per transform
			{ 7, 22, 23 },		// 156 bits per transform
			{ 7, 23, 22 },		// 156 bits per transform
			{ 7, 24, 13 },		// 156 bits per transform
			{ 8, 12, 24 },		// 156 bits per transform
			{ 8, 21, 23 },		// 156 bits per transform
			{ 8, 22, 22 },		// 156 bits per transform
			{ 8, 23, 21 },		// 156 bits per transform
			{ 8, 24, 12 },		// 156 bits per transform
			{ 9, 11, 24 },		// 156 bits per transform
			{ 9, 20, 23 },		// 156 bits per transform
			{ 9, 21, 22 },		// 156 bits per transform
			{ 9, 22, 21 },		// 156 bits per transform
			{ 9, 23, 20 },		// 156 bits per transform
			{ 9, 24, 11 },		// 156 bits per transform
			{ 10, 10, 24 },		// 156 bits per transform
			{ 10, 19, 23 },		// 156 bits per transform
			{ 10, 20, 22 },		// 156 bits per transform
			{ 10, 21, 21 },		// 156 bits per transform
			{ 10, 22, 20 },		// 156 bits per transform
			{ 10, 23, 19 },		// 156 bits per transform
			{ 10, 24, 10 },		// 156 bits per transform
			{ 11, 9, 24 },		// 156 bits per transform
			{ 11, 18, 23 },		// 156 bits per transform
			{ 11, 19, 22 },		// 156 bits per transform
			{ 11, 20, 21 },		// 156 bits per transform
			{ 11, 21, 20 },		// 156 bits per transform
			{ 11, 22, 19 },		// 156 bits per transform
			{ 11, 23, 18 },		// 156 bits per transform
			{ 11, 24, 9 },		// 156 bits per transform
			{ 12, 8, 24 },		// 156 bits per transform
			{ 12, 17, 23 },		// 156 bits per transform
			{ 12, 18, 22 },		// 156 bits per transform
			{ 12, 19, 21 },		// 156 bits per transform
			{ 12, 20, 20 },		// 156 bits per transform
			{ 12, 21, 19 },		// 156 bits per transform
			{ 12, 22, 18 },		// 156 bits per transform
			{ 12, 23, 17 },		// 156 bits per transform
			{ 12, 24, 8 },		// 156 bits per transform
			{ 13, 7, 24 },		// 156 bits per transform
			{ 13, 16, 23 },		// 156 bits per transform
			{ 13, 17, 22 },		// 156 bits per transform
			{ 13, 18, 21 },		// 156 bits per transform
			{ 13, 19, 20 },		// 156 bits per transform
			{ 13, 20, 19 },		// 156 bits per transform
			{ 13, 21, 18 },		// 156 bits per transform
			{ 13, 22, 17 },		// 156 bits per transform
			{ 13, 23, 16 },		// 156 bits per transform
			{ 13, 24, 7 },		// 156 bits per transform
			{ 14, 6, 24 },		// 156 bits per transform
			{ 14, 15, 23 },		// 156 bits per transform
			{ 14, 16, 22 },		// 156 bits per transform
			{ 14, 17, 21 },		// 156 bits per transform
			{ 14, 18, 20 },		// 156 bits per transform
			{ 14, 19, 19 },		// 156 bits per transform
			{ 14, 20, 18 },		// 156 bits per transform
			{ 14, 21, 17 },		// 156 bits per transform
			{ 14, 22, 16 },		// 156 bits per transform
			{ 14, 23, 15 },		// 156 bits per transform
			{ 14, 24, 6 },		// 156 bits per transform
			{ 15, 5, 24 },		// 156 bits per transform
			{ 15, 14, 23 },		// 156 bits per transform
			{ 15, 15, 22 },		// 156 bits per transform
			{ 15, 16, 21 },		// 156 bits per transform
			{ 15, 17, 20 },		// 156 bits per transform
			{ 15, 18, 19 },		// 156 bits per transform
			{ 15, 19, 18 },		// 156 bits per transform
			{ 15, 20, 17 },		// 156 bits per transform
			{ 15, 21, 16 },		// 156 bits per transform
			{ 15, 22, 15 },		// 156 bits per transform
			{ 15, 23, 14 },		// 156 bits per transform
			{ 15, 24, 5 },		// 156 bits per transform
			{ 16, 4, 24 },		// 156 bits per transform
			{ 16, 13, 23 },		// 156 bits per transform
			{ 16, 14, 22 },		// 156 bits per transform
			{ 16, 15, 21 },		// 156 bits per transform
			{ 16, 16, 20 },		// 156 bits per transform
			{ 16, 17, 19 },		// 156 bits per transform
			{ 16, 18, 18 },		// 156 bits per transform
			{ 16, 19, 17 },		// 156 bits per transform
			{ 16, 20, 16 },		// 156 bits per transform
			{ 16, 21, 15 },		// 156 bits per transform
			{ 16, 22, 14 },		// 156 bits per transform
			{ 16, 23, 13 },		// 156 bits per transform
			{ 16, 24, 4 },		// 156 bits per transform
			{ 17, 3, 24 },		// 156 bits per transform
			{ 17, 12, 23 },		// 156 bits per transform
			{ 17, 13, 22 },		// 156 bits per transform
			{ 17, 14, 21 },		// 156 bits per transform
			{ 17, 15, 20 },		// 156 bits per transform
			{ 17, 16, 19 },		// 156 bits per transform
			{ 17, 17, 18 },		// 156 bits per transform
			{ 17, 18, 17 },		// 156 bits per transform
			{ 17, 19, 16 },		// 156 bits per transform
			{ 17, 20, 15 },		// 156 bits per transform
			{ 17, 21, 14 },		// 156 bits per transform
			{ 17, 22, 13 },		// 156 bits per transform
			{ 17, 23, 12 },		// 156 bits per transform
			{ 17, 24, 3 },		// 156 bits per transform
			{ 18, 2, 24 },		// 156 bits per transform
			{ 18, 11, 23 },		// 156 bits per transform
			{ 18, 12, 22 },		// 156 bits per transform
			{ 18, 13, 21 },		// 156 bits per transform
			{ 18, 14, 20 },		// 156 bits per transform
			{ 18, 15, 19 },		// 156 bits per transform
			{ 18, 16, 18 },		// 156 bits per transform
			{ 18, 17, 17 },		// 156 bits per transform
			{ 18, 18, 16 },		// 156 bits per transform
			{ 18, 19, 15 },		// 156 bits per transform
			{ 18, 20, 14 },		// 156 bits per transform
			{ 18, 21, 13 },		// 156 bits per transform
			{ 18, 22, 12 },		// 156 bits per transform
			{ 18, 23, 11 },		// 156 bits per transform
			{ 18, 24, 2 },		// 156 bits per transform
			{ 19, 1, 24 },		// 156 bits per transform
			{ 19, 10, 23 },		// 156 bits per transform
			{ 19, 11, 22 },		// 156 bits per transform
			{ 19, 12, 21 },		// 156 bits per transform
			{ 19, 13, 20 },		// 156 bits per transform
			{ 19, 14, 19 },		// 156 bits per transform
			{ 19, 15, 18 },		// 156 bits per transform
			{ 19, 16, 17 },		// 156 bits per transform
			{ 19, 17, 16 },		// 156 bits per transform
			{ 19, 18, 15 },		// 156 bits per transform
			{ 19, 19, 14 },		// 156 bits per transform
			{ 19, 20, 13 },		// 156 bits per transform
			{ 19, 21, 12 },		// 156 bits per transform
			{ 19, 22, 11 },		// 156 bits per transform
			{ 19, 23, 10 },		// 156 bits per transform
			{ 19, 24, 1 },		// 156 bits per transform
			{ 20, 0, 24 },		// 156 bits per transform
			{ 20, 9, 23 },		// 156 bits per transform
			{ 20, 10, 22 },		// 156 bits per transform
			{ 20, 11, 21 },		// 156 bits per transform
			{ 20, 12, 20 },		// 156 bits per transform
			{ 20, 13, 19 },		// 156 bits per transform
			{ 20, 14, 18 },		// 156 bits per transform
			{ 20, 15, 17 },		// 156 bits per transform
			{ 20, 16, 16 },		// 156 bits per transform
			{ 20, 17, 15 },		// 156 bits per transform
			{ 20, 18, 14 },		// 156 bits per transform
			{ 20, 19, 13 },		// 156 bits per transform
			{ 20, 20, 12 },		// 156 bits per transform
			{ 20, 21, 11 },		// 156 bits per transform
			{ 20, 22, 10 },		// 156 bits per transform
			{ 20, 23, 9 },		// 156 bits per transform
			{ 20, 24, 0 },		// 156 bits per transform
			{ 21, 8, 23 },		// 156 bits per transform
			{ 21, 9, 22 },		// 156 bits per transform
			{ 21, 10, 21 },		// 156 bits per transform
			{ 21, 11, 20 },		// 156 bits per transform
			{ 21, 12, 19 },		// 156 bits per transform
			{ 21, 13, 18 },		// 156 bits per transform
			{ 21, 14, 17 },		// 156 bits per transform
			{ 21, 15, 16 },		// 156 bits per transform
			{ 21, 16, 15 },		// 156 bits per transform
			{ 21, 17, 14 },		// 156 bits per transform
			{ 21, 18, 13 },		// 156 bits per transform
			{ 21, 19, 12 },		// 156 bits per transform
			{ 21, 20, 11 },		// 156 bits per transform
			{ 21, 21, 10 },		// 156 bits per transform
			{ 21, 22, 9 },		// 156 bits per transform
			{ 21, 23, 8 },		// 156 bits per transform
			{ 22, 7, 23 },		// 156 bits per transform
			{ 22, 8, 22 },		// 156 bits per transform
			{ 22, 9, 21 },		// 156 bits per transform
			{ 22, 10, 20 },		// 156 bits per transform
			{ 22, 11, 19 },		// 156 bits per transform
			{ 22, 12, 18 },		// 156 bits per transform
			{ 22, 13, 17 },		// 156 bits per transform
			{ 22, 14, 16 },		// 156 bits per transform
			{ 22, 15, 15 },		// 156 bits per transform
			{ 22, 16, 14 },		// 156 bits per transform
			{ 22, 17, 13 },		// 156 bits per transform
			{ 22, 18, 12 },		// 156 bits per transform
			{ 22, 19, 11 },		// 156 bits per transform
			{ 22, 20, 10 },		// 156 bits per transform
			{ 22, 21, 9 },		// 156 bits per transform
			{ 22, 22, 8 },		// 156 bits per transform
			{ 22, 23, 7 },		// 156 bits per transform
			{ 23, 6, 23 },		// 156 bits per transform
			{ 23, 7, 22 },		// 156 bits per transform
			{ 23, 8, 21 },		// 156 bits per transform
			{ 23, 9, 20 },		// 156 bits per transform
			{ 23, 10, 19 },		// 156 bits per transform
			{ 23, 11, 18 },		// 156 bits per transform
			{ 23, 12, 17 },		// 156 bits per transform
			{ 23, 13, 16 },		// 156 bits per transform
			{ 23, 14, 15 },		// 156 bits per transform
			{ 23, 15, 14 },		// 156 bits per transform
			{ 23, 16, 13 },		// 156 bits per transform
			{ 23, 17, 12 },		// 156 bits per transform
			{ 23, 18, 11 },		// 156 bits per transform
			{ 23, 19, 10 },		// 156 bits per transform
			{ 23, 20, 9 },		// 156 bits per transform
			{ 23, 21, 8 },		// 156 bits per transform
			{ 23, 22, 7 },		// 156 bits per transform
			{ 23, 23, 6 },		// 156 bits per transform
			{ 24, 0, 20 },		// 156 bits per transform
			{ 24, 1, 19 },		// 156 bits per transform
			{ 24, 2, 18 },		// 156 bits per transform
			{ 24, 3, 17 },		// 156 bits per transform
			{ 24, 4, 16 },		// 156 bits per transform
			{ 24, 5, 15 },		// 156 bits per transform
			{ 24, 6, 14 },		// 156 bits per transform
			{ 24, 7, 13 },		// 156 bits per transform
			{ 24, 8, 12 },		// 156 bits per transform
			{ 24, 9, 11 },		// 156 bits per transform
			{ 24, 10, 10 },		// 156 bits per transform
			{ 24, 11, 9 },		// 156 bits per transform
			{ 24, 12, 8 },		// 156 bits per transform
			{ 24, 13, 7 },		// 156 bits per transform
			{ 24, 14, 6 },		// 156 bits per transform
			{ 24, 15, 5 },		// 156 bits per transform
			{ 24, 16, 4 },		// 156 bits per transform
			{ 24, 17, 3 },		// 156 bits per transform
			{ 24, 18, 2 },		// 156 bits per transform
			{ 24, 19, 1 },		// 156 bits per transform
			{ 24, 20, 0 },		// 156 bits per transform
			{ 0, 21, 24 },		// 159 bits per transform
			{ 0, 24, 21 },		// 159 bits per transform
			{ 1, 20, 24 },		// 159 bits per transform
			{ 1, 24, 20 },		// 159 bits per transform
			{ 2, 19, 24 },		// 159 bits per transform
			{ 2, 24, 19 },		// 159 bits per transform
			{ 3, 18, 24 },		// 159 bits per transform
			{ 3, 24, 18 },		// 159 bits per transform
			{ 4, 17, 24 },		// 159 bits per transform
			{ 4, 24, 17 },		// 159 bits per transform
			{ 5, 16, 24 },		// 159 bits per transform
			{ 5, 24, 16 },		// 159 bits per transform
			{ 6, 15, 24 },		// 159 bits per transform
			{ 6, 24, 15 },		// 159 bits per transform
			{ 7, 14, 24 },		// 159 bits per transform
			{ 7, 23, 23 },		// 159 bits per transform
			{ 7, 24, 14 },		// 159 bits per transform
			{ 8, 13, 24 },		// 159 bits per transform
			{ 8, 22, 23 },		// 159 bits per transform
			{ 8, 23, 22 },		// 159 bits per transform
			{ 8, 24, 13 },		// 159 bits per transform
			{ 9, 12, 24 },		// 159 bits per transform
			{ 9, 21, 23 },		// 159 bits per transform
			{ 9, 22, 22 },		// 159 bits per transform
			{ 9, 23, 21 },		// 159 bits per transform
			{ 9, 24, 12 },		// 159 bits per transform
			{ 10, 11, 24 },		// 159 bits per transform
			{ 10, 20, 23 },		// 159 bits per transform
			{ 10, 21, 22 },		// 159 bits per transform
			{ 10, 22, 21 },		// 159 bits per transform
			{ 10, 23, 20 },		// 159 bits per transform
			{ 10, 24, 11 },		// 159 bits per transform
			{ 11, 10, 24 },		// 159 bits per transform
			{ 11, 19, 23 },		// 159 bits per transform
			{ 11, 20, 22 },		// 159 bits per transform
			{ 11, 21, 21 },		// 159 bits per transform
			{ 11, 22, 20 },		// 159 bits per transform
			{ 11, 23, 19 },		// 159 bits per transform
			{ 11, 24, 10 },		// 159 bits per transform
			{ 12, 9, 24 },		// 159 bits per transform
			{ 12, 18, 23 },		// 159 bits per transform
			{ 12, 19, 22 },		// 159 bits per transform
			{ 12, 20, 21 },		// 159 bits per transform
			{ 12, 21, 20 },		// 159 bits per transform
			{ 12, 22, 19 },		// 159 bits per transform
			{ 12, 23, 18 },		// 159 bits per transform
			{ 12, 24, 9 },		// 159 bits per transform
			{ 13, 8, 24 },		// 159 bits per transform
			{ 13, 17, 23 },		// 159 bits per transform
			{ 13, 18, 22 },		// 159 bits per transform
			{ 13, 19, 21 },		// 159 bits per transform
			{ 13, 20, 20 },		// 159 bits per transform
			{ 13, 21, 19 },		// 159 bits per transform
			{ 13, 22, 18 },		// 159 bits per transform
			{ 13, 23, 17 },		// 159 bits per transform
			{ 13, 24, 8 },		// 159 bits per transform
			{ 14, 7, 24 },		// 159 bits per transform
			{ 14, 16, 23 },		// 159 bits per transform
			{ 14, 17, 22 },		// 159 bits per transform
			{ 14, 18, 21 },		// 159 bits per transform
			{ 14, 19, 20 },		// 159 bits per transform
			{ 14, 20, 19 },		// 159 bits per transform
			{ 14, 21, 18 },		// 159 bits per transform
			{ 14, 22, 17 },		// 159 bits per transform
			{ 14, 23, 16 },		// 159 bits per transform
			{ 14, 24, 7 },		// 159 bits per transform
			{ 15, 6, 24 },		// 159 bits per transform
			{ 15, 15, 23 },		// 159 bits per transform
			{ 15, 16, 22 },		// 159 bits per transform
			{ 15, 17, 21 },		// 159 bits per transform
			{ 15, 18, 20 },		// 159 bits per transform
			{ 15, 19, 19 },		// 159 bits per transform
			{ 15, 20, 18 },		// 159 bits per transform
			{ 15, 21, 17 },		// 159 bits per transform
			{ 15, 22, 16 },		// 159 bits per transform
			{ 15, 23, 15 },		// 159 bits per transform
			{ 15, 24, 6 },		// 159 bits per transform
			{ 16, 5, 24 },		// 159 bits per transform
			{ 16, 14, 23 },		// 159 bits per transform
			{ 16, 15, 22 },		// 159 bits per transform
			{ 16, 16, 21 },		// 159 bits per transform
			{ 16, 17, 20 },		// 159 bits per transform
			{ 16, 18, 19 },		// 159 bits per transform
			{ 16, 19, 18 },		// 159 bits per transform
			{ 16, 20, 17 },		// 159 bits per transform
			{ 16, 21, 16 },		// 159 bits per transform
			{ 16, 22, 15 },		// 159 bits per transform
			{ 16, 23, 14 },		// 159 bits per transform
			{ 16, 24, 5 },		// 159 bits per transform
			{ 17, 4, 24 },		// 159 bits per transform
			{ 17, 13, 23 },		// 159 bits per transform
			{ 17, 14, 22 },		// 159 bits per transform
			{ 17, 15, 21 },		// 159 bits per transform
			{ 17, 16, 20 },		// 159 bits per transform
			{ 17, 17, 19 },		// 159 bits per transform
			{ 17, 18, 18 },		// 159 bits per transform
			{ 17, 19, 17 },		// 159 bits per transform
			{ 17, 20, 16 },		// 159 bits per transform
			{ 17, 21, 15 },		// 159 bits per transform
			{ 17, 22, 14 },		// 159 bits per transform
			{ 17, 23, 13 },		// 159 bits per transform
			{ 17, 24, 4 },		// 159 bits per transform
			{ 18, 3, 24 },		// 159 bits per transform
			{ 18, 12, 23 },		// 159 bits per transform
			{ 18, 13, 22 },		// 159 bits per transform
			{ 18, 14, 21 },		// 159 bits per transform
			{ 18, 15, 20 },		// 159 bits per transform
			{ 18, 16, 19 },		// 159 bits per transform
			{ 18, 17, 18 },		// 159 bits per transform
			{ 18, 18, 17 },		// 159 bits per transform
			{ 18, 19, 16 },		// 159 bits per transform
			{ 18, 20, 15 },		// 159 bits per transform
			{ 18, 21, 14 },		// 159 bits per transform
			{ 18, 22, 13 },		// 159 bits per transform
			{ 18, 23, 12 },		// 159 bits per transform
			{ 18, 24, 3 },		// 159 bits per transform
			{ 19, 2, 24 },		// 159 bits per transform
			{ 19, 11, 23 },		// 159 bits per transform
			{ 19, 12, 22 },		// 159 bits per transform
			{ 19, 13, 21 },		// 159 bits per transform
			{ 19, 14, 20 },		// 159 bits per transform
			{ 19, 15, 19 },		// 159 bits per transform
			{ 19, 16, 18 },		// 159 bits per transform
			{ 19, 17, 17 },		// 159 bits per transform
			{ 19, 18, 16 },		// 159 bits per transform
			{ 19, 19, 15 },		// 159 bits per transform
			{ 19, 20, 14 },		// 159 bits per transform
			{ 19, 21, 13 },		// 159 bits per transform
			{ 19, 22, 12 },		// 159 bits per transform
			{ 19, 23, 11 },		// 159 bits per transform
			{ 19, 24, 2 },		// 159 bits per transform
			{ 20, 1, 24 },		// 159 bits per transform
			{ 20, 10, 23 },		// 159 bits per transform
			{ 20, 11, 22 },		// 159 bits per transform
			{ 20, 12, 21 },		// 159 bits per transform
			{ 20, 13, 20 },		// 159 bits per transform
			{ 20, 14, 19 },		// 159 bits per transform
			{ 20, 15, 18 },		// 159 bits per transform
			{ 20, 16, 17 },		// 159 bits per transform
			{ 20, 17, 16 },		// 159 bits per transform
			{ 20, 18, 15 },		// 159 bits per transform
			{ 20, 19, 14 },		// 159 bits per transform
			{ 20, 20, 13 },		// 159 bits per transform
			{ 20, 21, 12 },		// 159 bits per transform
			{ 20, 22, 11 },		// 159 bits per transform
			{ 20, 23, 10 },		// 159 bits per transform
			{ 20, 24, 1 },		// 159 bits per transform
			{ 21, 0, 24 },		// 159 bits per transform
			{ 21, 9, 23 },		// 159 bits per transform
			{ 21, 10, 22 },		// 159 bits per transform
			{ 21, 11, 21 },		// 159 bits per transform
			{ 21, 12, 20 },		// 159 bits per transform
			{ 21, 13, 19 },		// 159 bits per transform
			{ 21, 14, 18 },		// 159 bits per transform
			{ 21, 15, 17 },		// 159 bits per transform
			{ 21, 16, 16 },		// 159 bits per transform
			{ 21, 17, 15 },		// 159 bits per transform
			{ 21, 18, 14 },		// 159 bits per transform
			{ 21, 19, 13 },		// 159 bits per transform
			{ 21, 20, 12 },		// 159 bits per transform
			{ 21, 21, 11 },		// 159 bits per transform
			{ 21, 22, 10 },		// 159 bits per transform
			{ 21, 23, 9 },		// 159 bits per transform
			{ 21, 24, 0 },		// 159 bits per transform
			{ 22, 8, 23 },		// 159 bits per transform
			{ 22, 9, 22 },		// 159 bits per transform
			{ 22, 10, 21 },		// 159 bits per transform
			{ 22, 11, 20 },		// 159 bits per transform
			{ 22, 12, 19 },		// 159 bits per transform
			{ 22, 13, 18 },		// 159 bits per transform
			{ 22, 14, 17 },		// 159 bits per transform
			{ 22, 15, 16 },		// 159 bits per transform
			{ 22, 16, 15 },		// 159 bits per transform
			{ 22, 17, 14 },		// 159 bits per transform
			{ 22, 18, 13 },		// 159 bits per transform
			{ 22, 19, 12 },		// 159 bits per transform
			{ 22, 20, 11 },		// 159 bits per transform
			{ 22, 21, 10 },		// 159 bits per transform
			{ 22, 22, 9 },		// 159 bits per transform
			{ 22, 23, 8 },		// 159 bits per transform
			{ 23, 7, 23 },		// 159 bits per transform
			{ 23, 8, 22 },		// 159 bits per transform
			{ 23, 9, 21 },		// 159 bits per transform
			{ 23, 10, 20 },		// 159 bits per transform
			{ 23, 11, 19 },		// 159 bits per transform
			{ 23, 12, 18 },		// 159 bits per transform
			{ 23, 13, 17 },		// 159 bits per transform
			{ 23, 14, 16 },		// 159 bits per transform
			{ 23, 15, 15 },		// 159 bits per transform
			{ 23, 16, 14 },		// 159 bits per transform
			{ 23, 17, 13 },		// 159 bits per transform
			{ 23, 18, 12 },		// 159 bits per transform
			{ 23, 19, 11 },		// 159 bits per transform
			{ 23, 20, 10 },		// 159 bits per transform
			{ 23, 21, 9 },		// 159 bits per transform
			{ 23, 22, 8 },		// 159 bits per transform
			{ 23, 23, 7 },		// 159 bits per transform
			{ 24, 0, 21 },		// 159 bits per transform
			{ 24, 1, 20 },		// 159 bits per transform
			{ 24, 2, 19 },		// 159 bits per transform
			{ 24, 3, 18 },		// 159 bits per transform
			{ 24, 4, 17 },		// 159 bits per transform
			{ 24, 5, 16 },		// 159 bits per transform
			{ 24, 6, 15 },		// 159 bits per transform
			{ 24, 7, 14 },		// 159 bits per transform
			{ 24, 8, 13 },		// 159 bits per transform
			{ 24, 9, 12 },		// 159 bits per transform
			{ 24, 10, 11 },		// 159 bits per transform
			{ 24, 11, 10 },		// 159 bits per transform
			{ 24, 12, 9 },		// 159 bits per transform
			{ 24, 13, 8 },		// 159 bits per transform
			{ 24, 14, 7 },		// 159 bits per transform
			{ 24, 15, 6 },		// 159 bits per transform
			{ 24, 16, 5 },		// 159 bits per transform
			{ 24, 17, 4 },		// 159 bits per transform
			{ 24, 18, 3 },		// 159 bits per transform
			{ 24, 19, 2 },		// 159 bits per transform
			{ 24, 20, 1 },		// 159 bits per transform
			{ 24, 21, 0 },		// 159 bits per transform
			{ 0, 22, 24 },		// 162 bits per transform
			{ 0, 24, 22 },		// 162 bits per transform
			{ 1, 21, 24 },		// 162 bits per transform
			{ 1, 24, 21 },		// 162 bits per transform
			{ 2, 20, 24 },		// 162 bits per transform
			{ 2, 24, 20 },		// 162 bits per transform
			{ 3, 19, 24 },		// 162 bits per transform
			{ 3, 24, 19 },		// 162 bits per transform
			{ 4, 18, 24 },		// 162 bits per transform
			{ 4, 24, 18 },		// 162 bits per transform
			{ 5, 17, 24 },		// 162 bits per transform
			{ 5, 24, 17 },		// 162 bits per transform
			{ 6, 16, 24 },		// 162 bits per transform
			{ 6, 24, 16 },		// 162 bits per transform
			{ 7, 15, 24 },		// 162 bits per transform
			{ 7, 24, 15 },		// 162 bits per transform
			{ 8, 14, 24 },		// 162 bits per transform
			{ 8, 23, 23 },		// 162 bits per transform
			{ 8, 24, 14 },		// 162 bits per transform
			{ 9, 13, 24 },		// 162 bits per transform
			{ 9, 22, 23 },		// 162 bits per transform
			{ 9, 23, 22 },		// 162 bits per transform
			{ 9, 24, 13 },		// 162 bits per transform
			{ 10, 12, 24 },		// 162 bits per transform
			{ 10, 21, 23 },		// 162 bits per transform
			{ 10, 22, 22 },		// 162 bits per transform
			{ 10, 23, 21 },		// 162 bits per transform
			{ 10, 24, 12 },		// 162 bits per transform
			{ 11, 11, 24 },		// 162 bits per transform
			{ 11, 20, 23 },		// 162 bits per transform
			{ 11, 21, 22 },		// 162 bits per transform
			{ 11, 22, 21 },		// 162 bits per transform
			{ 11, 23, 20 },		// 162 bits per transform
			{ 11, 24, 11 },		// 162 bits per transform
			{ 12, 10, 24 },		// 162 bits per transform
			{ 12, 19, 23 },		// 162 bits per transform
			{ 12, 20, 22 },		// 162 bits per transform
			{ 12, 21, 21 },		// 162 bits per transform
			{ 12, 22, 20 },		// 162 bits per transform
			{ 12, 23, 19 },		// 162 bits per transform
			{ 12, 24, 10 },		// 162 bits per transform
			{ 13, 9, 24 },		// 162 bits per transform
			{ 13, 18, 23 },		// 162 bits per transform
			{ 13, 19, 22 },		// 162 bits per transform
			{ 13, 20, 21 },		// 162 bits per transform
			{ 13, 21, 20 },		// 162 bits per transform
			{ 13, 22, 19 },		// 162 bits per transform
			{ 13, 23, 18 },		// 162 bits per transform
			{ 13, 24, 9 },		// 162 bits per transform
			{ 14, 8, 24 },		// 162 bits per transform
			{ 14, 17, 23 },		// 162 bits per transform
			{ 14, 18, 22 },		// 162 bits per transform
			{ 14, 19, 21 },		// 162 bits per transform
			{ 14, 20, 20 },		// 162 bits per transform
			{ 14, 21, 19 },		// 162 bits per transform
			{ 14, 22, 18 },		// 162 bits per transform
			{ 14, 23, 17 },		// 162 bits per transform
			{ 14, 24, 8 },		// 162 bits per transform
			{ 15, 7, 24 },		// 162 bits per transform
			{ 15, 16, 23 },		// 162 bits per transform
			{ 15, 17, 22 },		// 162 bits per transform
			{ 15, 18, 21 },		// 162 bits per transform
			{ 15, 19, 20 },		// 162 bits per transform
			{ 15, 20, 19 },		// 162 bits per transform
			{ 15, 21, 18 },		// 162 bits per transform
			{ 15, 22, 17 },		// 162 bits per transform
			{ 15, 23, 16 },		// 162 bits per transform
			{ 15, 24, 7 },		// 162 bits per transform
			{ 16, 6, 24 },		// 162 bits per transform
			{ 16, 15, 23 },		// 162 bits per transform
			{ 16, 16, 22 },		// 162 bits per transform
			{ 16, 17, 21 },		// 162 bits per transform
			{ 16, 18, 20 },		// 162 bits per transform
			{ 16, 19, 19 },		// 162 bits per transform
			{ 16, 20, 18 },		// 162 bits per transform
			{ 16, 21, 17 },		// 162 bits per transform
			{ 16, 22, 16 },		// 162 bits per transform
			{ 16, 23, 15 },		// 162 bits per transform
			{ 16, 24, 6 },		// 162 bits per transform
			{ 17, 5, 24 },		// 162 bits per transform
			{ 17, 14, 23 },		// 162 bits per transform
			{ 17, 15, 22 },		// 162 bits per transform
			{ 17, 16, 21 },		// 162 bits per transform
			{ 17, 17, 20 },		// 162 bits per transform
			{ 17, 18, 19 },		// 162 bits per transform
			{ 17, 19, 18 },		// 162 bits per transform
			{ 17, 20, 17 },		// 162 bits per transform
			{ 17, 21, 16 },		// 162 bits per transform
			{ 17, 22, 15 },		// 162 bits per transform
			{ 17, 23, 14 },		// 162 bits per transform
			{ 17, 24, 5 },		// 162 bits per transform
			{ 18, 4, 24 },		// 162 bits per transform
			{ 18, 13, 23 },		// 162 bits per transform
			{ 18, 14, 22 },		// 162 bits per transform
			{ 18, 15, 21 },		// 162 bits per transform
			{ 18, 16, 20 },		// 162 bits per transform
			{ 18, 17, 19 },		// 162 bits per transform
			{ 18, 18, 18 },		// 162 bits per transform
			{ 18, 19, 17 },		// 162 bits per transform
			{ 18, 20, 16 },		// 162 bits per transform
			{ 18, 21, 15 },		// 162 bits per transform
			{ 18, 22, 14 },		// 162 bits per transform
			{ 18, 23, 13 },		// 162 bits per transform
			{ 18, 24, 4 },		// 162 bits per transform
			{ 19, 3, 24 },		// 162 bits per transform
			{ 19, 12, 23 },		// 162 bits per transform
			{ 19, 13, 22 },		// 162 bits per transform
			{ 19, 14, 21 },		// 162 bits per transform
			{ 19, 15, 20 },		// 162 bits per transform
			{ 19, 16, 19 },		// 162 bits per transform
			{ 19, 17, 18 },		// 162 bits per transform
			{ 19, 18, 17 },		// 162 bits per transform
			{ 19, 19, 16 },		// 162 bits per transform
			{ 19, 20, 15 },		// 162 bits per transform
			{ 19, 21, 14 },		// 162 bits per transform
			{ 19, 22, 13 },		// 162 bits per transform
			{ 19, 23, 12 },		// 162 bits per transform
			{ 19, 24, 3 },		// 162 bits per transform
			{ 20, 2, 24 },		// 162 bits per transform
			{ 20, 11, 23 },		// 162 bits per transform
			{ 20, 12, 22 },		// 162 bits per transform
			{ 20, 13, 21 },		// 162 bits per transform
			{ 20, 14, 20 },		// 162 bits per transform
			{ 20, 15, 19 },		// 162 bits per transform
			{ 20, 16, 18 },		// 162 bits per transform
			{ 20, 17, 17 },		// 162 bits per transform
			{ 20, 18, 16 },		// 162 bits per transform
			{ 20, 19, 15 },		// 162 bits per transform
			{ 20, 20, 14 },		// 162 bits per transform
			{ 20, 21, 13 },		// 162 bits per transform
			{ 20, 22, 12 },		// 162 bits per transform
			{ 20, 23, 11 },		// 162 bits per transform
			{ 20, 24, 2 },		// 162 bits per transform
			{ 21, 1, 24 },		// 162 bits per transform
			{ 21, 10, 23 },		// 162 bits per transform
			{ 21, 11, 22 },		// 162 bits per transform
			{ 21, 12, 21 },		// 162 bits per transform
			{ 21, 13, 20 },		// 162 bits per transform
			{ 21, 14, 19 },		// 162 bits per transform
			{ 21, 15, 18 },		// 162 bits per transform
			{ 21, 16, 17 },		// 162 bits per transform
			{ 21, 17, 16 },		// 162 bits per transform
			{ 21, 18, 15 },		// 162 bits per transform
			{ 21, 19, 14 },		// 162 bits per transform
			{ 21, 20, 13 },		// 162 bits per transform
			{ 21, 21, 12 },		// 162 bits per transform
			{ 21, 22, 11 },		// 162 bits per transform
			{ 21, 23, 10 },		// 162 bits per transform
			{ 21, 24, 1 },		// 162 bits per transform
			{ 22, 0, 24 },		// 162 bits per transform
			{ 22, 9, 23 },		// 162 bits per transform
			{ 22, 10, 22 },		// 162 bits per transform
			{ 22, 11, 21 },		// 162 bits per transform
			{ 22, 12, 20 },		// 162 bits per transform
			{ 22, 13, 19 },		// 162 bits per transform
			{ 22, 14, 18 },		// 162 bits per transform
			{ 22, 15, 17 },		// 162 bits per transform
			{ 22, 16, 16 },		// 162 bits per transform
			{ 22, 17, 15 },		// 162 bits per transform
			{ 22, 18, 14 },		// 162 bits per transform
			{ 22, 19, 13 },		// 162 bits per transform
			{ 22, 20, 12 },		// 162 bits per transform
			{ 22, 21, 11 },		// 162 bits per transform
			{ 22, 22, 10 },		// 162 bits per transform
			{ 22, 23, 9 },		// 162 bits per transform
			{ 22, 24, 0 },		// 162 bits per transform
			{ 23, 8, 23 },		// 162 bits per transform
			{ 23, 9, 22 },		// 162 bits per transform
			{ 23, 10, 21 },		// 162 bits per transform
			{ 23, 11, 20 },		// 162 bits per transform
			{ 23, 12, 19 },		// 162 bits per transform
			{ 23, 13, 18 },		// 162 bits per transform
			{ 23, 14, 17 },		// 162 bits per transform
			{ 23, 15, 16 },		// 162 bits per transform
			{ 23, 16, 15 },		// 162 bits per transform
			{ 23, 17, 14 },		// 162 bits per transform
			{ 23, 18, 13 },		// 162 bits per transform
			{ 23, 19, 12 },		// 162 bits per transform
			{ 23, 20, 11 },		// 162 bits per transform
			{ 23, 21, 10 },		// 162 bits per transform
			{ 23, 22, 9 },		// 162 bits per transform
			{ 23, 23, 8 },		// 162 bits per transform
			{ 24, 0, 22 },		// 162 bits per transform
			{ 24, 1, 21 },		// 162 bits per transform
			{ 24, 2, 20 },		// 162 bits per transform
			{ 24, 3, 19 },		// 162 bits per transform
			{ 24, 4, 18 },		// 162 bits per transform
			{ 24, 5, 17 },		// 162 bits per transform
			{ 24, 6, 16 },		// 162 bits per transform
			{ 24, 7, 15 },		// 162 bits per transform
			{ 24, 8, 14 },		// 162 bits per transform
			{ 24, 9, 13 },		// 162 bits per transform
			{ 24, 10, 12 },		// 162 bits per transform
			{ 24, 11, 11 },		// 162 bits per transform
			{ 24, 12, 10 },		// 162 bits per transform
			{ 24, 13, 9 },		// 162 bits per transform
			{ 24, 14, 8 },		// 162 bits per transform
			{ 24, 15, 7 },		// 162 bits per transform
			{ 24, 16, 6 },		// 162 bits per transform
			{ 24, 17, 5 },		// 162 bits per transform
			{ 24, 18, 4 },		// 162 bits per transform
			{ 24, 19, 3 },		// 162 bits per transform
			{ 24, 20, 2 },		// 162 bits per transform
			{ 24, 21, 1 },		// 162 bits per transform
			{ 24, 22, 0 },		// 162 bits per transform
			{ 0, 23, 24 },		// 165 bits per transform
			{ 0, 24, 23 },		// 165 bits per transform
			{ 1, 22, 24 },		// 165 bits per transform
			{ 1, 24, 22 },		// 165 bits per transform
			{ 2, 21, 24 },		// 165 bits per transform
			{ 2, 24, 21 },		// 165 bits per transform
			{ 3, 20, 24 },		// 165 bits per transform
			{ 3, 24, 20 },		// 165 bits per transform
			{ 4, 19, 24 },		// 165 bits per transform
			{ 4, 24, 19 },		// 165 bits per transform
			{ 5, 18, 24 },		// 165 bits per transform
			{ 5, 24, 18 },		// 165 bits per transform
			{ 6, 17, 24 },		// 165 bits per transform
			{ 6, 24, 17 },		// 165 bits per transform
			{ 7, 16, 24 },		// 165 bits per transform
			{ 7, 24, 16 },		// 165 bits per transform
			{ 8, 15, 24 },		// 165 bits per transform
			{ 8, 24, 15 },		// 165 bits per transform
			{ 9, 14, 24 },		// 165 bits per transform
			{ 9, 23, 23 },		// 165 bits per transform
			{ 9, 24, 14 },		// 165 bits per transform
			{ 10, 13, 24 },		// 165 bits per transform
			{ 10, 22, 23 },		// 165 bits per transform
			{ 10, 23, 22 },		// 165 bits per transform
			{ 10, 24, 13 },		// 165 bits per transform
			{ 11, 12, 24 },		// 165 bits per transform
			{ 11, 21, 23 },		// 165 bits per transform
			{ 11, 22, 22 },		// 165 bits per transform
			{ 11, 23, 21 },		// 165 bits per transform
			{ 11, 24, 12 },		// 165 bits per transform
			{ 12, 11, 24 },		// 165 bits per transform
			{ 12, 20, 23 },		// 165 bits per transform
			{ 12, 21, 22 },		// 165 bits per transform
			{ 12, 22, 21 },		// 165 bits per transform
			{ 12, 23, 20 },		// 165 bits per transform
			{ 12, 24, 11 },		// 165 bits per transform
			{ 13, 10, 24 },		// 165 bits per transform
			{ 13, 19, 23 },		// 165 bits per transform
			{ 13, 20, 22 },		// 165 bits per transform
			{ 13, 21, 21 },		// 165 bits per transform
			{ 13, 22, 20 },		// 165 bits per transform
			{ 13, 23, 19 },		// 165 bits per transform
			{ 13, 24, 10 },		// 165 bits per transform
			{ 14, 9, 24 },		// 165 bits per transform
			{ 14, 18, 23 },		// 165 bits per transform
			{ 14, 19, 22 },		// 165 bits per transform
			{ 14, 20, 21 },		// 165 bits per transform
			{ 14, 21, 20 },		// 165 bits per transform
			{ 14, 22, 19 },		// 165 bits per transform
			{ 14, 23, 18 },		// 165 bits per transform
			{ 14, 24, 9 },		// 165 bits per transform
			{ 15, 8, 24 },		// 165 bits per transform
			{ 15, 17, 23 },		// 165 bits per transform
			{ 15, 18, 22 },		// 165 bits per transform
			{ 15, 19, 21 },		// 165 bits per transform
			{ 15, 20, 20 },		// 165 bits per transform
			{ 15, 21, 19 },		// 165 bits per transform
			{ 15, 22, 18 },		// 165 bits per transform
			{ 15, 23, 17 },		// 165 bits per transform
			{ 15, 24, 8 },		// 165 bits per transform
			{ 16, 7, 24 },		// 165 bits per transform
			{ 16, 16, 23 },		// 165 bits per transform
			{ 16, 17, 22 },		// 165 bits per transform
			{ 16, 18, 21 },		// 165 bits per transform
			{ 16, 19, 20 },		// 165 bits per transform
			{ 16, 20, 19 },		// 165 bits per transform
			{ 16, 21, 18 },		// 165 bits per transform
			{ 16, 22, 17 },		// 165 bits per transform
			{ 16, 23, 16 },		// 165 bits per transform
			{ 16, 24, 7 },		// 165 bits per transform
			{ 17, 6, 24 },		// 165 bits per transform
			{ 17, 15, 23 },		// 165 bits per transform
			{ 17, 16, 22 },		// 165 bits per transform
			{ 17, 17, 21 },		// 165 bits per transform
			{ 17, 18, 20 },		// 165 bits per transform
			{ 17, 19, 19 },		// 165 bits per transform
			{ 17, 20, 18 },		// 165 bits per transform
			{ 17, 21, 17 },		// 165 bits per transform
			{ 17, 22, 16 },		// 165 bits per transform
			{ 17, 23, 15 },		// 165 bits per transform
			{ 17, 24, 6 },		// 165 bits per transform
			{ 18, 5, 24 },		// 165 bits per transform
			{ 18, 14, 23 },		// 165 bits per transform
			{ 18, 15, 22 },		// 165 bits per transform
			{ 18, 16, 21 },		// 165 bits per transform
			{ 18, 17, 20 },		// 165 bits per transform
			{ 18, 18, 19 },		// 165 bits per transform
			{ 18, 19, 18 },		// 165 bits per transform
			{ 18, 20, 17 },		// 165 bits per transform
			{ 18, 21, 16 },		// 165 bits per transform
			{ 18, 22, 15 },		// 165 bits per transform
			{ 18, 23, 14 },		// 165 bits per transform
			{ 18, 24, 5 },		// 165 bits per transform
			{ 19, 4, 24 },		// 165 bits per transform
			{ 19, 13, 23 },		// 165 bits per transform
			{ 19, 14, 22 },		// 165 bits per transform
			{ 19, 15, 21 },		// 165 bits per transform
			{ 19, 16, 20 },		// 165 bits per transform
			{ 19, 17, 19 },		// 165 bits per transform
			{ 19, 18, 18 },		// 165 bits per transform
			{ 19, 19, 17 },		// 165 bits per transform
			{ 19, 20, 16 },		// 165 bits per transform
			{ 19, 21, 15 },		// 165 bits per transform
			{ 19, 22, 14 },		// 165 bits per transform
			{ 19, 23, 13 },		// 165 bits per transform
			{ 19, 24, 4 },		// 165 bits per transform
			{ 20, 3, 24 },		// 165 bits per transform
			{ 20, 12, 23 },		// 165 bits per transform
			{ 20, 13, 22 },		// 165 bits per transform
			{ 20, 14, 21 },		// 165 bits per transform
			{ 20, 15, 20 },		// 165 bits per transform
			{ 20, 16, 19 },		// 165 bits per transform
			{ 20, 17, 18 },		// 165 bits per transform
			{ 20, 18, 17 },		// 165 bits per transform
			{ 20, 19, 16 },		// 165 bits per transform
			{ 20, 20, 15 },		// 165 bits per transform
			{ 20, 21, 14 },		// 165 bits per transform
			{ 20, 22, 13 },		// 165 bits per transform
			{ 20, 23, 12 },		// 165 bits per transform
			{ 20, 24, 3 },		// 165 bits per transform
			{ 21, 2, 24 },		// 165 bits per transform
			{ 21, 11, 23 },		// 165 bits per transform
			{ 21, 12, 22 },		// 165 bits per transform
			{ 21, 13, 21 },		// 165 bits per transform
			{ 21, 14, 20 },		// 165 bits per transform
			{ 21, 15, 19 },		// 165 bits per transform
			{ 21, 16, 18 },		// 165 bits per transform
			{ 21, 17, 17 },		// 165 bits per transform
			{ 21, 18, 16 },		// 165 bits per transform
			{ 21, 19, 15 },		// 165 bits per transform
			{ 21, 20, 14 },		// 165 bits per transform
			{ 21, 21, 13 },		// 165 bits per transform
			{ 21, 22, 12 },		// 165 bits per transform
			{ 21, 23, 11 },		// 165 bits per transform
			{ 21, 24, 2 },		// 165 bits per transform
			{ 22, 1, 24 },		// 165 bits per transform
			{ 22, 10, 23 },		// 165 bits per transform
			{ 22, 11, 22 },		// 165 bits per transform
			{ 22, 12, 21 },		// 165 bits per transform
			{ 22, 13, 20 },		// 165 bits per transform
			{ 22, 14, 19 },		// 165 bits per transform
			{ 22, 15, 18 },		// 165 bits per transform
			{ 22, 16, 17 },		// 165 bits per transform
			{ 22, 17, 16 },		// 165 bits per transform
			{ 22, 18, 15 },		// 165 bits per transform
			{ 22, 19, 14 },		// 165 bits per transform
			{ 22, 20, 13 },		// 165 bits per transform
			{ 22, 21, 12 },		// 165 bits per transform
			{ 22, 22, 11 },		// 165 bits per transform
			{ 22, 23, 10 },		// 165 bits per transform
			{ 22, 24, 1 },		// 165 bits per transform
			{ 23, 0, 24 },		// 165 bits per transform
			{ 23, 9, 23 },		// 165 bits per transform
			{ 23, 10, 22 },		// 165 bits per transform
			{ 23, 11, 21 },		// 165 bits per transform
			{ 23, 12, 20 },		// 165 bits per transform
			{ 23, 13, 19 },		// 165 bits per transform
			{ 23, 14, 18 },		// 165 bits per transform
			{ 23, 15, 17 },		// 165 bits per transform
			{ 23, 16, 16 },		// 165 bits per transform
			{ 23, 17, 15 },		// 165 bits per transform
			{ 23, 18, 14 },		// 165 bits per transform
			{ 23, 19, 13 },		// 165 bits per transform
			{ 23, 20, 12 },		// 165 bits per transform
			{ 23, 21, 11 },		// 165 bits per transform
			{ 23, 22, 10 },		// 165 bits per transform
			{ 23, 23, 9 },		// 165 bits per transform
			{ 23, 24, 0 },		// 165 bits per transform
			{ 24, 0, 23 },		// 165 bits per transform
			{ 24, 1, 22 },		// 165 bits per transform
			{ 24, 2, 21 },		// 165 bits per transform
			{ 24, 3, 20 },		// 165 bits per transform
			{ 24, 4, 19 },		// 165 bits per transform
			{ 24, 5, 18 },		// 165 bits per transform
			{ 24, 6, 17 },		// 165 bits per transform
			{ 24, 7, 16 },		// 165 bits per transform
			{ 24, 8, 15 },		// 165 bits per transform
			{ 24, 9, 14 },		// 165 bits per transform
			{ 24, 10, 13 },		// 165 bits per transform
			{ 24, 11, 12 },		// 165 bits per transform
			{ 24, 12, 11 },		// 165 bits per transform
			{ 24, 13, 10 },		// 165 bits per transform
			{ 24, 14, 9 },		// 165 bits per transform
			{ 24, 15, 8 },		// 165 bits per transform
			{ 24, 16, 7 },		// 165 bits per transform
			{ 24, 17, 6 },		// 165 bits per transform
			{ 24, 18, 5 },		// 165 bits per transform
			{ 24, 19, 4 },		// 165 bits per transform
			{ 24, 20, 3 },		// 165 bits per transform
			{ 24, 21, 2 },		// 165 bits per transform
			{ 24, 22, 1 },		// 165 bits per transform
			{ 24, 23, 0 },		// 165 bits per transform
			{ 1, 23, 24 },		// 168 bits per transform
			{ 1, 24, 23 },		// 168 bits per transform
			{ 2, 22, 24 },		// 168 bits per transform
			{ 2, 24, 22 },		// 168 bits per transform
			{ 3, 21, 24 },		// 168 bits per transform
			{ 3, 24, 21 },		// 168 bits per transform
			{ 4, 20, 24 },		// 168 bits per transform
			{ 4, 24, 20 },		// 168 bits per transform
			{ 5, 19, 24 },		// 168 bits per transform
			{ 5, 24, 19 },		// 168 bits per transform
			{ 6, 18, 24 },		// 168 bits per transform
			{ 6, 24, 18 },		// 168 bits per transform
			{ 7, 17, 24 },		// 168 bits per transform
			{ 7, 24, 17 },		// 168 bits per transform
			{ 8, 16, 24 },		// 168 bits per transform
			{ 8, 24, 16 },		// 168 bits per transform
			{ 9, 15, 24 },		// 168 bits per transform
			{ 9, 24, 15 },		// 168 bits per transform
			{ 10, 14, 24 },		// 168 bits per transform
			{ 10, 23, 23 },		// 168 bits per transform
			{ 10, 24, 14 },		// 168 bits per transform
			{ 11, 13, 24 },		// 168 bits per transform
			{ 11, 22, 23 },		// 168 bits per transform
			{ 11, 23, 22 },		// 168 bits per transform
			{ 11, 24, 13 },		// 168 bits per transform
			{ 12, 12, 24 },		// 168 bits per transform
			{ 12, 21, 23 },		// 168 bits per transform
			{ 12, 22, 22 },		// 168 bits per transform
			{ 12, 23, 21 },		// 168 bits per transform
			{ 12, 24, 12 },		// 168 bits per transform
			{ 13, 11, 24 },		// 168 bits per transform
			{ 13, 20, 23 },		// 168 bits per transform
			{ 13, 21, 22 },		// 168 bits per transform
			{ 13, 22, 21 },		// 168 bits per transform
			{ 13, 23, 20 },		// 168 bits per transform
			{ 13, 24, 11 },		// 168 bits per transform
			{ 14, 10, 24 },		// 168 bits per transform
			{ 14, 19, 23 },		// 168 bits per transform
			{ 14, 20, 22 },		// 168 bits per transform
			{ 14, 21, 21 },		// 168 bits per transform
			{ 14, 22, 20 },		// 168 bits per transform
			{ 14, 23, 19 },		// 168 bits per transform
			{ 14, 24, 10 },		// 168 bits per transform
			{ 15, 9, 24 },		// 168 bits per transform
			{ 15, 18, 23 },		// 168 bits per transform
			{ 15, 19, 22 },		// 168 bits per transform
			{ 15, 20, 21 },		// 168 bits per transform
			{ 15, 21, 20 },		// 168 bits per transform
			{ 15, 22, 19 },		// 168 bits per transform
			{ 15, 23, 18 },		// 168 bits per transform
			{ 15, 24, 9 },		// 168 bits per transform
			{ 16, 8, 24 },		// 168 bits per transform
			{ 16, 17, 23 },		// 168 bits per transform
			{ 16, 18, 22 },		// 168 bits per transform
			{ 16, 19, 21 },		// 168 bits per transform
			{ 16, 20, 20 },		// 168 bits per transform
			{ 16, 21, 19 },		// 168 bits per transform
			{ 16, 22, 18 },		// 168 bits per transform
			{ 16, 23, 17 },		// 168 bits per transform
			{ 16, 24, 8 },		// 168 bits per transform
			{ 17, 7, 24 },		// 168 bits per transform
			{ 17, 16, 23 },		// 168 bits per transform
			{ 17, 17, 22 },		// 168 bits per transform
			{ 17, 18, 21 },		// 168 bits per transform
			{ 17, 19, 20 },		// 168 bits per transform
			{ 17, 20, 19 },		// 168 bits per transform
			{ 17, 21, 18 },		// 168 bits per transform
			{ 17, 22, 17 },		// 168 bits per transform
			{ 17, 23, 16 },		// 168 bits per transform
			{ 17, 24, 7 },		// 168 bits per transform
			{ 18, 6, 24 },		// 168 bits per transform
			{ 18, 15, 23 },		// 168 bits per transform
			{ 18, 16, 22 },		// 168 bits per transform
			{ 18, 17, 21 },		// 168 bits per transform
			{ 18, 18, 20 },		// 168 bits per transform
			{ 18, 19, 19 },		// 168 bits per transform
			{ 18, 20, 18 },		// 168 bits per transform
			{ 18, 21, 17 },		// 168 bits per transform
			{ 18, 22, 16 },		// 168 bits per transform
			{ 18, 23, 15 },		// 168 bits per transform
			{ 18, 24, 6 },		// 168 bits per transform
			{ 19, 5, 24 },		// 168 bits per transform
			{ 19, 14, 23 },		// 168 bits per transform
			{ 19, 15, 22 },		// 168 bits per transform
			{ 19, 16, 21 },		// 168 bits per transform
			{ 19, 17, 20 },		// 168 bits per transform
			{ 19, 18, 19 },		// 168 bits per transform
			{ 19, 19, 18 },		// 168 bits per transform
			{ 19, 20, 17 },		// 168 bits per transform
			{ 19, 21, 16 },		// 168 bits per transform
			{ 19, 22, 15 },		// 168 bits per transform
			{ 19, 23, 14 },		// 168 bits per transform
			{ 19, 24, 5 },		// 168 bits per transform
			{ 20, 4, 24 },		// 168 bits per transform
			{ 20, 13, 23 },		// 168 bits per transform
			{ 20, 14, 22 },		// 168 bits per transform
			{ 20, 15, 21 },		// 168 bits per transform
			{ 20, 16, 20 },		// 168 bits per transform
			{ 20, 17, 19 },		// 168 bits per transform
			{ 20, 18, 18 },		// 168 bits per transform
			{ 20, 19, 17 },		// 168 bits per transform
			{ 20, 20, 16 },		// 168 bits per transform
			{ 20, 21, 15 },		// 168 bits per transform
			{ 20, 22, 14 },		// 168 bits per transform
			{ 20, 23, 13 },		// 168 bits per transform
			{ 20, 24, 4 },		// 168 bits per transform
			{ 21, 3, 24 },		// 168 bits per transform
			{ 21, 12, 23 },		// 168 bits per transform
			{ 21, 13, 22 },		// 168 bits per transform
			{ 21, 14, 21 },		// 168 bits per transform
			{ 21, 15, 20 },		// 168 bits per transform
			{ 21, 16, 19 },		// 168 bits per transform
			{ 21, 17, 18 },		// 168 bits per transform
			{ 21, 18, 17 },		// 168 bits per transform
			{ 21, 19, 16 },		// 168 bits per transform
			{ 21, 20, 15 },		// 168 bits per transform
			{ 21, 21, 14 },		// 168 bits per transform
			{ 21, 22, 13 },		// 168 bits per transform
			{ 21, 23, 12 },		// 168 bits per transform
			{ 21, 24, 3 },		// 168 bits per transform
			{ 22, 2, 24 },		// 168 bits per transform
			{ 22, 11, 23 },		// 168 bits per transform
			{ 22, 12, 22 },		// 168 bits per transform
			{ 22, 13, 21 },		// 168 bits per transform
			{ 22, 14, 20 },		// 168 bits per transform
			{ 22, 15, 19 },		// 168 bits per transform
			{ 22, 16, 18 },		// 168 bits per transform
			{ 22, 17, 17 },		// 168 bits per transform
			{ 22, 18, 16 },		// 168 bits per transform
			{ 22, 19, 15 },		// 168 bits per transform
			{ 22, 20, 14 },		// 168 bits per transform
			{ 22, 21, 13 },		// 168 bits per transform
			{ 22, 22, 12 },		// 168 bits per transform
			{ 22, 23, 11 },		// 168 bits per transform
			{ 22, 24, 2 },		// 168 bits per transform
			{ 23, 1, 24 },		// 168 bits per transform
			{ 23, 10, 23 },		// 168 bits per transform
			{ 23, 11, 22 },		// 168 bits per transform
			{ 23, 12, 21 },		// 168 bits per transform
			{ 23, 13, 20 },		// 168 bits per transform
			{ 23, 14, 19 },		// 168 bits per transform
			{ 23, 15, 18 },		// 168 bits per transform
			{ 23, 16, 17 },		// 168 bits per transform
			{ 23, 17, 16 },		// 168 bits per transform
			{ 23, 18, 15 },		// 168 bits per transform
			{ 23, 19, 14 },		// 168 bits per transform
			{ 23, 20, 13 },		// 168 bits per transform
			{ 23, 21, 12 },		// 168 bits per transform
			{ 23, 22, 11 },		// 168 bits per transform
			{ 23, 23, 10 },		// 168 bits per transform
			{ 23, 24, 1 },		// 168 bits per transform
			{ 24, 1, 23 },		// 168 bits per transform
			{ 24, 2, 22 },		// 168 bits per transform
			{ 24, 3, 21 },		// 168 bits per transform
			{ 24, 4, 20 },		// 168 bits per transform
			{ 24, 5, 19 },		// 168 bits per transform
			{ 24, 6, 18 },		// 168 bits per transform
			{ 24, 7, 17 },		// 168 bits per transform
			{ 24, 8, 16 },		// 168 bits per transform
			{ 24, 9, 15 },		// 168 bits per transform
			{ 24, 10, 14 },		// 168 bits per transform
			{ 24, 11, 13 },		// 168 bits per transform
			{ 24, 12, 12 },		// 168 bits per transform
			{ 24, 13, 11 },		// 168 bits per transform
			{ 24, 14, 10 },		// 168 bits per transform
			{ 24, 15, 9 },		// 168 bits per transform
			{ 24, 16, 8 },		// 168 bits per transform
			{ 24, 17, 7 },		// 168 bits per transform
			{ 24, 18, 6 },		// 168 bits per transform
			{ 24, 19, 5 },		// 168 bits per transform
			{ 24, 20, 4 },		// 168 bits per transform
			{ 24, 21, 3 },		// 168 bits per transform
			{ 24, 22, 2 },		// 168 bits per transform
			{ 24, 23, 1 },		// 168 bits per transform
			{ 2, 23, 24 },		// 171 bits per transform
			{ 2, 24, 23 },		// 171 bits per transform
			{ 3, 22, 24 },		// 171 bits per transform
			{ 3, 24, 22 },		// 171 bits per transform
			{ 4, 21, 24 },		// 171 bits per transform
			{ 4, 24, 21 },		// 171 bits per transform
			{ 5, 20, 24 },		// 171 bits per transform
			{ 5, 24, 20 },		// 171 bits per transform
			{ 6, 19, 24 },		// 171 bits per transform
			{ 6, 24, 19 },		// 171 bits per transform
			{ 7, 18, 24 },		// 171 bits per transform
			{ 7, 24, 18 },		// 171 bits per transform
			{ 8, 17, 24 },		// 171 bits per transform
			{ 8, 24, 17 },		// 171 bits per transform
			{ 9, 16, 24 },		// 171 bits per transform
			{ 9, 24, 16 },		// 171 bits per transform
			{ 10, 15, 24 },		// 171 bits per transform
			{ 10, 24, 15 },		// 171 bits per transform
			{ 11, 14, 24 },		// 171 bits per transform
			{ 11, 23, 23 },		// 171 bits per transform
			{ 11, 24, 14 },		// 171 bits per transform
			{ 12, 13, 24 },		// 171 bits per transform
			{ 12, 22, 23 },		// 171 bits per transform
			{ 12, 23, 22 },		// 171 bits per transform
			{ 12, 24, 13 },		// 171 bits per transform
			{ 13, 12, 24 },		// 171 bits per transform
			{ 13, 21, 23 },		// 171 bits per transform
			{ 13, 22, 22 },		// 171 bits per transform
			{ 13, 23, 21 },		// 171 bits per transform
			{ 13, 24, 12 },		// 171 bits per transform
			{ 14, 11, 24 },		// 171 bits per transform
			{ 14, 20, 23 },		// 171 bits per transform
			{ 14, 21, 22 },		// 171 bits per transform
			{ 14, 22, 21 },		// 171 bits per transform
			{ 14, 23, 20 },		// 171 bits per transform
			{ 14, 24, 11 },		// 171 bits per transform
			{ 15, 10, 24 },		// 171 bits per transform
			{ 15, 19, 23 },		// 171 bits per transform
			{ 15, 20, 22 },		// 171 bits per transform
			{ 15, 21, 21 },		// 171 bits per transform
			{ 15, 22, 20 },		// 171 bits per transform
			{ 15, 23, 19 },		// 171 bits per transform
			{ 15, 24, 10 },		// 171 bits per transform
			{ 16, 9, 24 },		// 171 bits per transform
			{ 16, 18, 23 },		// 171 bits per transform
			{ 16, 19, 22 },		// 171 bits per transform
			{ 16, 20, 21 },		// 171 bits per transform
			{ 16, 21, 20 },		// 171 bits per transform
			{ 16, 22, 19 },		// 171 bits per transform
			{ 16, 23, 18 },		// 171 bits per transform
			{ 16, 24, 9 },		// 171 bits per transform
			{ 17, 8, 24 },		// 171 bits per transform
			{ 17, 17, 23 },		// 171 bits per transform
			{ 17, 18, 22 },		// 171 bits per transform
			{ 17, 19, 21 },		// 171 bits per transform
			{ 17, 20, 20 },		// 171 bits per transform
			{ 17, 21, 19 },		// 171 bits per transform
			{ 17, 22, 18 },		// 171 bits per transform
			{ 17, 23, 17 },		// 171 bits per transform
			{ 17, 24, 8 },		// 171 bits per transform
			{ 18, 7, 24 },		// 171 bits per transform
			{ 18, 16, 23 },		// 171 bits per transform
			{ 18, 17, 22 },		// 171 bits per transform
			{ 18, 18, 21 },		// 171 bits per transform
			{ 18, 19, 20 },		// 171 bits per transform
			{ 18, 20, 19 },		// 171 bits per transform
			{ 18, 21, 18 },		// 171 bits per transform
			{ 18, 22, 17 },		// 171 bits per transform
			{ 18, 23, 16 },		// 171 bits per transform
			{ 18, 24, 7 },		// 171 bits per transform
			{ 19, 6, 24 },		// 171 bits per transform
			{ 19, 15, 23 },		// 171 bits per transform
			{ 19, 16, 22 },		// 171 bits per transform
			{ 19, 17, 21 },		// 171 bits per transform
			{ 19, 18, 20 },		// 171 bits per transform
			{ 19, 19, 19 },		// 171 bits per transform
			{ 19, 20, 18 },		// 171 bits per transform
			{ 19, 21, 17 },		// 171 bits per transform
			{ 19, 22, 16 },		// 171 bits per transform
			{ 19, 23, 15 },		// 171 bits per transform
			{ 19, 24, 6 },		// 171 bits per transform
			{ 20, 5, 24 },		// 171 bits per transform
			{ 20, 14, 23 },		// 171 bits per transform
			{ 20, 15, 22 },		// 171 bits per transform
			{ 20, 16, 21 },		// 171 bits per transform
			{ 20, 17, 20 },		// 171 bits per transform
			{ 20, 18, 19 },		// 171 bits per transform
			{ 20, 19, 18 },		// 171 bits per transform
			{ 20, 20, 17 },		// 171 bits per transform
			{ 20, 21, 16 },		// 171 bits per transform
			{ 20, 22, 15 },		// 171 bits per transform
			{ 20, 23, 14 },		// 171 bits per transform
			{ 20, 24, 5 },		// 171 bits per transform
			{ 21, 4, 24 },		// 171 bits per transform
			{ 21, 13, 23 },		// 171 bits per transform
			{ 21, 14, 22 },		// 171 bits per transform
			{ 21, 15, 21 },		// 171 bits per transform
			{ 21, 16, 20 },		// 171 bits per transform
			{ 21, 17, 19 },		// 171 bits per transform
			{ 21, 18, 18 },		// 171 bits per transform
			{ 21, 19, 17 },		// 171 bits per transform
			{ 21, 20, 16 },		// 171 bits per transform
			{ 21, 21, 15 },		// 171 bits per transform
			{ 21, 22, 14 },		// 171 bits per transform
			{ 21, 23, 13 },		// 171 bits per transform
			{ 21, 24, 4 },		// 171 bits per transform
			{ 22, 3, 24 },		// 171 bits per transform
			{ 22, 12, 23 },		// 171 bits per transform
			{ 22, 13, 22 },		// 171 bits per transform
			{ 22, 14, 21 },		// 171 bits per transform
			{ 22, 15, 20 },		// 171 bits per transform
			{ 22, 16, 19 },		// 171 bits per transform
			{ 22, 17, 18 },		// 171 bits per transform
			{ 22, 18, 17 },		// 171 bits per transform
			{ 22, 19, 16 },		// 171 bits per transform
			{ 22, 20, 15 },		// 171 bits per transform
			{ 22, 21, 14 },		// 171 bits per transform
			{ 22, 22, 13 },		// 171 bits per transform
			{ 22, 23, 12 },		// 171 bits per transform
			{ 22, 24, 3 },		// 171 bits per transform
			{ 23, 2, 24 },		// 171 bits per transform
			{ 23, 11, 23 },		// 171 bits per transform
			{ 23, 12, 22 },		// 171 bits per transform
			{ 23, 13, 21 },		// 171 bits per transform
			{ 23, 14, 20 },		// 171 bits per transform
			{ 23, 15, 19 },		// 171 bits per transform
			{ 23, 16, 18 },		// 171 bits per transform
			{ 23, 17, 17 },		// 171 bits per transform
			{ 23, 18, 16 },		// 171 bits per transform
			{ 23, 19, 15 },		// 171 bits per transform
			{ 23, 20, 14 },		// 171 bits per transform
			{ 23, 21, 13 },		// 171 bits per transform
			{ 23, 22, 12 },		// 171 bits per transform
			{ 23, 23, 11 },		// 171 bits per transform
			{ 23, 24, 2 },		// 171 bits per transform
			{ 24, 2, 23 },		// 171 bits per transform
			{ 24, 3, 22 },		// 171 bits per transform
			{ 24, 4, 21 },		// 171 bits per transform
			{ 24, 5, 20 },		// 171 bits per transform
			{ 24, 6, 19 },		// 171 bits per transform
			{ 24, 7, 18 },		// 171 bits per transform
			{ 24, 8, 17 },		// 171 bits per transform
			{ 24, 9, 16 },		// 171 bits per transform
			{ 24, 10, 15 },		// 171 bits per transform
			{ 24, 11, 14 },		// 171 bits per transform
			{ 24, 12, 13 },		// 171 bits per transform
			{ 24, 13, 12 },		// 171 bits per transform
			{ 24, 14, 11 },		// 171 bits per transform
			{ 24, 15, 10 },		// 171 bits per transform
			{ 24, 16, 9 },		// 171 bits per transform
			{ 24, 17, 8 },		// 171 bits per transform
			{ 24, 18, 7 },		// 171 bits per transform
			{ 24, 19, 6 },		// 171 bits per transform
			{ 24, 20, 5 },		// 171 bits per transform
			{ 24, 21, 4 },		// 171 bits per transform
			{ 24, 22, 3 },		// 171 bits per transform
			{ 24, 23, 2 },		// 171 bits per transform
			{ 3, 23, 24 },		// 174 bits per transform
			{ 3, 24, 23 },		// 174 bits per transform
			{ 4, 22, 24 },		// 174 bits per transform
			{ 4, 24, 22 },		// 174 bits per transform
			{ 5, 21, 24 },		// 174 bits per transform
			{ 5, 24, 21 },		// 174 bits per transform
			{ 6, 20, 24 },		// 174 bits per transform
			{ 6, 24, 20 },		// 174 bits per transform
			{ 7, 19, 24 },		// 174 bits per transform
			{ 7, 24, 19 },		// 174 bits per transform
			{ 8, 18, 24 },		// 174 bits per transform
			{ 8, 24, 18 },		// 174 bits per transform
			{ 9, 17, 24 },		// 174 bits per transform
			{ 9, 24, 17 },		// 174 bits per transform
			{ 10, 16, 24 },		// 174 bits per transform
			{ 10, 24, 16 },		// 174 bits per transform
			{ 11, 15, 24 },		// 174 bits per transform
			{ 11, 24, 15 },		// 174 bits per transform
			{ 12, 14, 24 },		// 174 bits per transform
			{ 12, 23, 23 },		// 174 bits per transform
			{ 12, 24, 14 },		// 174 bits per transform
			{ 13, 13, 24 },		// 174 bits per transform
			{ 13, 22, 23 },		// 174 bits per transform
			{ 13, 23, 22 },		// 174 bits per transform
			{ 13, 24, 13 },		// 174 bits per transform
			{ 14, 12, 24 },		// 174 bits per transform
			{ 14, 21, 23 },		// 174 bits per transform
			{ 14, 22, 22 },		// 174 bits per transform
			{ 14, 23, 21 },		// 174 bits per transform
			{ 14, 24, 12 },		// 174 bits per transform
			{ 15, 11, 24 },		// 174 bits per transform
			{ 15, 20, 23 },		// 174 bits per transform
			{ 15, 21, 22 },		// 174 bits per transform
			{ 15, 22, 21 },		// 174 bits per transform
			{ 15, 23, 20 },		// 174 bits per transform
			{ 15, 24, 11 },		// 174 bits per transform
			{ 16, 10, 24 },		// 174 bits per transform
			{ 16, 19, 23 },		// 174 bits per transform
			{ 16, 20, 22 },		// 174 bits per transform
			{ 16, 21, 21 },		// 174 bits per transform
			{ 16, 22, 20 },		// 174 bits per transform
			{ 16, 23, 19 },		// 174 bits per transform
			{ 16, 24, 10 },		// 174 bits per transform
			{ 17, 9, 24 },		// 174 bits per transform
			{ 17, 18, 23 },		// 174 bits per transform
			{ 17, 19, 22 },		// 174 bits per transform
			{ 17, 20, 21 },		// 174 bits per transform
			{ 17, 21, 20 },		// 174 bits per transform
			{ 17, 22, 19 },		// 174 bits per transform
			{ 17, 23, 18 },		// 174 bits per transform
			{ 17, 24, 9 },		// 174 bits per transform
			{ 18, 8, 24 },		// 174 bits per transform
			{ 18, 17, 23 },		// 174 bits per transform
			{ 18, 18, 22 },		// 174 bits per transform
			{ 18, 19, 21 },		// 174 bits per transform
			{ 18, 20, 20 },		// 174 bits per transform
			{ 18, 21, 19 },		// 174 bits per transform
			{ 18, 22, 18 },		// 174 bits per transform
			{ 18, 23, 17 },		// 174 bits per transform
			{ 18, 24, 8 },		// 174 bits per transform
			{ 19, 7, 24 },		// 174 bits per transform
			{ 19, 16, 23 },		// 174 bits per transform
			{ 19, 17, 22 },		// 174 bits per transform
			{ 19, 18, 21 },		// 174 bits per transform
			{ 19, 19, 20 },		// 174 bits per transform
			{ 19, 20, 19 },		// 174 bits per transform
			{ 19, 21, 18 },		// 174 bits per transform
			{ 19, 22, 17 },		// 174 bits per transform
			{ 19, 23, 16 },		// 174 bits per transform
			{ 19, 24, 7 },		// 174 bits per transform
			{ 20, 6, 24 },		// 174 bits per transform
			{ 20, 15, 23 },		// 174 bits per transform
			{ 20, 16, 22 },		// 174 bits per transform
			{ 20, 17, 21 },		// 174 bits per transform
			{ 20, 18, 20 },		// 174 bits per transform
			{ 20, 19, 19 },		// 174 bits per transform
			{ 20, 20, 18 },		// 174 bits per transform
			{ 20, 21, 17 },		// 174 bits per transform
			{ 20, 22, 16 },		// 174 bits per transform
			{ 20, 23, 15 },		// 174 bits per transform
			{ 20, 24, 6 },		// 174 bits per transform
			{ 21, 5, 24 },		// 174 bits per transform
			{ 21, 14, 23 },		// 174 bits per transform
			{ 21, 15, 22 },		// 174 bits per transform
			{ 21, 16, 21 },		// 174 bits per transform
			{ 21, 17, 20 },		// 174 bits per transform
			{ 21, 18, 19 },		// 174 bits per transform
			{ 21, 19, 18 },		// 174 bits per transform
			{ 21, 20, 17 },		// 174 bits per transform
			{ 21, 21, 16 },		// 174 bits per transform
			{ 21, 22, 15 },		// 174 bits per transform
			{ 21, 23, 14 },		// 174 bits per transform
			{ 21, 24, 5 },		// 174 bits per transform
			{ 22, 4, 24 },		// 174 bits per transform
			{ 22, 13, 23 },		// 174 bits per transform
			{ 22, 14, 22 },		// 174 bits per transform
			{ 22, 15, 21 },		// 174 bits per transform
			{ 22, 16, 20 },		// 174 bits per transform
			{ 22, 17, 19 },		// 174 bits per transform
			{ 22, 18, 18 },		// 174 bits per transform
			{ 22, 19, 17 },		// 174 bits per transform
			{ 22, 20, 16 },		// 174 bits per transform
			{ 22, 21, 15 },		// 174 bits per transform
			{ 22, 22, 14 },		// 174 bits per transform
			{ 22, 23, 13 },		// 174 bits per transform
			{ 22, 24, 4 },		// 174 bits per transform
			{ 23, 3, 24 },		// 174 bits per transform
			{ 23, 12, 23 },		// 174 bits per transform
			{ 23, 13, 22 },		// 174 bits per transform
			{ 23, 14, 21 },		// 174 bits per transform
			{ 23, 15, 20 },		// 174 bits per transform
			{ 23, 16, 19 },		// 174 bits per transform
			{ 23, 17, 18 },		// 174 bits per transform
			{ 23, 18, 17 },		// 174 bits per transform
			{ 23, 19, 16 },		// 174 bits per transform
			{ 23, 20, 15 },		// 174 bits per transform
			{ 23, 21, 14 },		// 174 bits per transform
			{ 23, 22, 13 },		// 174 bits per transform
			{ 23, 23, 12 },		// 174 bits per transform
			{ 23, 24, 3 },		// 174 bits per transform
			{ 24, 3, 23 },		// 174 bits per transform
			{ 24, 4, 22 },		// 174 bits per transform
			{ 24, 5, 21 },		// 174 bits per transform
			{ 24, 6, 20 },		// 174 bits per transform
			{ 24, 7, 19 },		// 174 bits per transform
			{ 24, 8, 18 },		// 174 bits per transform
			{ 24, 9, 17 },		// 174 bits per transform
			{ 24, 10, 16 },		// 174 bits per transform
			{ 24, 11, 15 },		// 174 bits per transform
			{ 24, 12, 14 },		// 174 bits per transform
			{ 24, 13, 13 },		// 174 bits per transform
			{ 24, 14, 12 },		// 174 bits per transform
			{ 24, 15, 11 },		// 174 bits per transform
			{ 24, 16, 10 },		// 174 bits per transform
			{ 24, 17, 9 },		// 174 bits per transform
			{ 24, 18, 8 },		// 174 bits per transform
			{ 24, 19, 7 },		// 174 bits per transform
			{ 24, 20, 6 },		// 174 bits per transform
			{ 24, 21, 5 },		// 174 bits per transform
			{ 24, 22, 4 },		// 174 bits per transform
			{ 24, 23, 3 },		// 174 bits per transform
			{ 4, 23, 24 },		// 177 bits per transform
			{ 4, 24, 23 },		// 177 bits per transform
			{ 5, 22, 24 },		// 177 bits per transform
			{ 5, 24, 22 },		// 177 bits per transform
			{ 6, 21, 24 },		// 177 bits per transform
			{ 6, 24, 21 },		// 177 bits per transform
			{ 7, 20, 24 },		// 177 bits per transform
			{ 7, 24, 20 },		// 177 bits per transform
			{ 8, 19, 24 },		// 177 bits per transform
			{ 8, 24, 19 },		// 177 bits per transform
			{ 9, 18, 24 },		// 177 bits per transform
			{ 9, 24, 18 },		// 177 bits per transform
			{ 10, 17, 24 },		// 177 bits per transform
			{ 10, 24, 17 },		// 177 bits per transform
			{ 11, 16, 24 },		// 177 bits per transform
			{ 11, 24, 16 },		// 177 bits per transform
			{ 12, 15, 24 },		// 177 bits per transform
			{ 12, 24, 15 },		// 177 bits per transform
			{ 13, 14, 24 },		// 177 bits per transform
			{ 13, 23, 23 },		// 177 bits per transform
			{ 13, 24, 14 },		// 177 bits per transform
			{ 14, 13, 24 },		// 177 bits per transform
			{ 14, 22, 23 },		// 177 bits per transform
			{ 14, 23, 22 },		// 177 bits per transform
			{ 14, 24, 13 },		// 177 bits per transform
			{ 15, 12, 24 },		// 177 bits per transform
			{ 15, 21, 23 },		// 177 bits per transform
			{ 15, 22, 22 },		// 177 bits per transform
			{ 15, 23, 21 },		// 177 bits per transform
			{ 15, 24, 12 },		// 177 bits per transform
			{ 16, 11, 24 },		// 177 bits per transform
			{ 16, 20, 23 },		// 177 bits per transform
			{ 16, 21, 22 },		// 177 bits per transform
			{ 16, 22, 21 },		// 177 bits per transform
			{ 16, 23, 20 },		// 177 bits per transform
			{ 16, 24, 11 },		// 177 bits per transform
			{ 17, 10, 24 },		// 177 bits per transform
			{ 17, 19, 23 },		// 177 bits per transform
			{ 17, 20, 22 },		// 177 bits per transform
			{ 17, 21, 21 },		// 177 bits per transform
			{ 17, 22, 20 },		// 177 bits per transform
			{ 17, 23, 19 },		// 177 bits per transform
			{ 17, 24, 10 },		// 177 bits per transform
			{ 18, 9, 24 },		// 177 bits per transform
			{ 18, 18, 23 },		// 177 bits per transform
			{ 18, 19, 22 },		// 177 bits per transform
			{ 18, 20, 21 },		// 177 bits per transform
			{ 18, 21, 20 },		// 177 bits per transform
			{ 18, 22, 19 },		// 177 bits per transform
			{ 18, 23, 18 },		// 177 bits per transform
			{ 18, 24, 9 },		// 177 bits per transform
			{ 19, 8, 24 },		// 177 bits per transform
			{ 19, 17, 23 },		// 177 bits per transform
			{ 19, 18, 22 },		// 177 bits per transform
			{ 19, 19, 21 },		// 177 bits per transform
			{ 19, 20, 20 },		// 177 bits per transform
			{ 19, 21, 19 },		// 177 bits per transform
			{ 19, 22, 18 },		// 177 bits per transform
			{ 19, 23, 17 },		// 177 bits per transform
			{ 19, 24, 8 },		// 177 bits per transform
			{ 20, 7, 24 },		// 177 bits per transform
			{ 20, 16, 23 },		// 177 bits per transform
			{ 20, 17, 22 },		// 177 bits per transform
			{ 20, 18, 21 },		// 177 bits per transform
			{ 20, 19, 20 },		// 177 bits per transform
			{ 20, 20, 19 },		// 177 bits per transform
			{ 20, 21, 18 },		// 177 bits per transform
			{ 20, 22, 17 },		// 177 bits per transform
			{ 20, 23, 16 },		// 177 bits per transform
			{ 20, 24, 7 },		// 177 bits per transform
			{ 21, 6, 24 },		// 177 bits per transform
			{ 21, 15, 23 },		// 177 bits per transform
			{ 21, 16, 22 },		// 177 bits per transform
			{ 21, 17, 21 },		// 177 bits per transform
			{ 21, 18, 20 },		// 177 bits per transform
			{ 21, 19, 19 },		// 177 bits per transform
			{ 21, 20, 18 },		// 177 bits per transform
			{ 21, 21, 17 },		// 177 bits per transform
			{ 21, 22, 16 },		// 177 bits per transform
			{ 21, 23, 15 },		// 177 bits per transform
			{ 21, 24, 6 },		// 177 bits per transform
			{ 22, 5, 24 },		// 177 bits per transform
			{ 22, 14, 23 },		// 177 bits per transform
			{ 22, 15, 22 },		// 177 bits per transform
			{ 22, 16, 21 },		// 177 bits per transform
			{ 22, 17, 20 },		// 177 bits per transform
			{ 22, 18, 19 },		// 177 bits per transform
			{ 22, 19, 18 },		// 177 bits per transform
			{ 22, 20, 17 },		// 177 bits per transform
			{ 22, 21, 16 },		// 177 bits per transform
			{ 22, 22, 15 },		// 177 bits per transform
			{ 22, 23, 14 },		// 177 bits per transform
			{ 22, 24, 5 },		// 177 bits per transform
			{ 23, 4, 24 },		// 177 bits per transform
			{ 23, 13, 23 },		// 177 bits per transform
			{ 23, 14, 22 },		// 177 bits per transform
			{ 23, 15, 21 },		// 177 bits per transform
			{ 23, 16, 20 },		// 177 bits per transform
			{ 23, 17, 19 },		// 177 bits per transform
			{ 23, 18, 18 },		// 177 bits per transform
			{ 23, 19, 17 },		// 177 bits per transform
			{ 23, 20, 16 },		// 177 bits per transform
			{ 23, 21, 15 },		// 177 bits per transform
			{ 23, 22, 14 },		// 177 bits per transform
			{ 23, 23, 13 },		// 177 bits per transform
			{ 23, 24, 4 },		// 177 bits per transform
			{ 24, 4, 23 },		// 177 bits per transform
			{ 24, 5, 22 },		// 177 bits per transform
			{ 24, 6, 21 },		// 177 bits per transform
			{ 24, 7, 20 },		// 177 bits per transform
			{ 24, 8, 19 },		// 177 bits per transform
			{ 24, 9, 18 },		// 177 bits per transform
			{ 24, 10, 17 },		// 177 bits per transform
			{ 24, 11, 16 },		// 177 bits per transform
			{ 24, 12, 15 },		// 177 bits per transform
			{ 24, 13, 14 },		// 177 bits per transform
			{ 24, 14, 13 },		// 177 bits per transform
			{ 24, 15, 12 },		// 177 bits per transform
			{ 24, 16, 11 },		// 177 bits per transform
			{ 24, 17, 10 },		// 177 bits per transform
			{ 24, 18, 9 },		// 177 bits per transform
			{ 24, 19, 8 },		// 177 bits per transform
			{ 24, 20, 7 },		// 177 bits per transform
			{ 24, 21, 6 },		// 177 bits per transform
			{ 24, 22, 5 },		// 177 bits per transform
			{ 24, 23, 4 },		// 177 bits per transform
			{ 5, 23, 24 },		// 180 bits per transform
			{ 5, 24, 23 },		// 180 bits per transform
			{ 6, 22, 24 },		// 180 bits per transform
			{ 6, 24, 22 },		// 180 bits per transform
			{ 7, 21, 24 },		// 180 bits per transform
			{ 7, 24, 21 },		// 180 bits per transform
			{ 8, 20, 24 },		// 180 bits per transform
			{ 8, 24, 20 },		// 180 bits per transform
			{ 9, 19, 24 },		// 180 bits per transform
			{ 9, 24, 19 },		// 180 bits per transform
			{ 10, 18, 24 },		// 180 bits per transform
			{ 10, 24, 18 },		// 180 bits per transform
			{ 11, 17, 24 },		// 180 bits per transform
			{ 11, 24, 17 },		// 180 bits per transform
			{ 12, 16, 24 },		// 180 bits per transform
			{ 12, 24, 16 },		// 180 bits per transform
			{ 13, 15, 24 },		// 180 bits per transform
			{ 13, 24, 15 },		// 180 bits per transform
			{ 14, 14, 24 },		// 180 bits per transform
			{ 14, 23, 23 },		// 180 bits per transform
			{ 14, 24, 14 },		// 180 bits per transform
			{ 15, 13, 24 },		// 180 bits per transform
			{ 15, 22, 23 },		// 180 bits per transform
			{ 15, 23, 22 },		// 180 bits per transform
			{ 15, 24, 13 },		// 180 bits per transform
			{ 16, 12, 24 },		// 180 bits per transform
			{ 16, 21, 23 },		// 180 bits per transform
			{ 16, 22, 22 },		// 180 bits per transform
			{ 16, 23, 21 },		// 180 bits per transform
			{ 16, 24, 12 },		// 180 bits per transform
			{ 17, 11, 24 },		// 180 bits per transform
			{ 17, 20, 23 },		// 180 bits per transform
			{ 17, 21, 22 },		// 180 bits per transform
			{ 17, 22, 21 },		// 180 bits per transform
			{ 17, 23, 20 },		// 180 bits per transform
			{ 17, 24, 11 },		// 180 bits per transform
			{ 18, 10, 24 },		// 180 bits per transform
			{ 18, 19, 23 },		// 180 bits per transform
			{ 18, 20, 22 },		// 180 bits per transform
			{ 18, 21, 21 },		// 180 bits per transform
			{ 18, 22, 20 },		// 180 bits per transform
			{ 18, 23, 19 },		// 180 bits per transform
			{ 18, 24, 10 },		// 180 bits per transform
			{ 19, 9, 24 },		// 180 bits per transform
			{ 19, 18, 23 },		// 180 bits per transform
			{ 19, 19, 22 },		// 180 bits per transform
			{ 19, 20, 21 },		// 180 bits per transform
			{ 19, 21, 20 },		// 180 bits per transform
			{ 19, 22, 19 },		// 180 bits per transform
			{ 19, 23, 18 },		// 180 bits per transform
			{ 19, 24, 9 },		// 180 bits per transform
			{ 20, 8, 24 },		// 180 bits per transform
			{ 20, 17, 23 },		// 180 bits per transform
			{ 20, 18, 22 },		// 180 bits per transform
			{ 20, 19, 21 },		// 180 bits per transform
			{ 20, 20, 20 },		// 180 bits per transform
			{ 20, 21, 19 },		// 180 bits per transform
			{ 20, 22, 18 },		// 180 bits per transform
			{ 20, 23, 17 },		// 180 bits per transform
			{ 20, 24, 8 },		// 180 bits per transform
			{ 21, 7, 24 },		// 180 bits per transform
			{ 21, 16, 23 },		// 180 bits per transform
			{ 21, 17, 22 },		// 180 bits per transform
			{ 21, 18, 21 },		// 180 bits per transform
			{ 21, 19, 20 },		// 180 bits per transform
			{ 21, 20, 19 },		// 180 bits per transform
			{ 21, 21, 18 },		// 180 bits per transform
			{ 21, 22, 17 },		// 180 bits per transform
			{ 21, 23, 16 },		// 180 bits per transform
			{ 21, 24, 7 },		// 180 bits per transform
			{ 22, 6, 24 },		// 180 bits per transform
			{ 22, 15, 23 },		// 180 bits per transform
			{ 22, 16, 22 },		// 180 bits per transform
			{ 22, 17, 21 },		// 180 bits per transform
			{ 22, 18, 20 },		// 180 bits per transform
			{ 22, 19, 19 },		// 180 bits per transform
			{ 22, 20, 18 },		// 180 bits per transform
			{ 22, 21, 17 },		// 180 bits per transform
			{ 22, 22, 16 },		// 180 bits per transform
			{ 22, 23, 15 },		// 180 bits per transform
			{ 22, 24, 6 },		// 180 bits per transform
			{ 23, 5, 24 },		// 180 bits per transform
			{ 23, 14, 23 },		// 180 bits per transform
			{ 23, 15, 22 },		// 180 bits per transform
			{ 23, 16, 21 },		// 180 bits per transform
			{ 23, 17, 20 },		// 180 bits per transform
			{ 23, 18, 19 },		// 180 bits per transform
			{ 23, 19, 18 },		// 180 bits per transform
			{ 23, 20, 17 },		// 180 bits per transform
			{ 23, 21, 16 },		// 180 bits per transform
			{ 23, 22, 15 },		// 180 bits per transform
			{ 23, 23, 14 },		// 180 bits per transform
			{ 23, 24, 5 },		// 180 bits per transform
			{ 24, 5, 23 },		// 180 bits per transform
			{ 24, 6, 22 },		// 180 bits per transform
			{ 24, 7, 21 },		// 180 bits per transform
			{ 24, 8, 20 },		// 180 bits per transform
			{ 24, 9, 19 },		// 180 bits per transform
			{ 24, 10, 18 },		// 180 bits per transform
			{ 24, 11, 17 },		// 180 bits per transform
			{ 24, 12, 16 },		// 180 bits per transform
			{ 24, 13, 15 },		// 180 bits per transform
			{ 24, 14, 14 },		// 180 bits per transform
			{ 24, 15, 13 },		// 180 bits per transform
			{ 24, 16, 12 },		// 180 bits per transform
			{ 24, 17, 11 },		// 180 bits per transform
			{ 24, 18, 10 },		// 180 bits per transform
			{ 24, 19, 9 },		// 180 bits per transform
			{ 24, 20, 8 },		// 180 bits per transform
			{ 24, 21, 7 },		// 180 bits per transform
			{ 24, 22, 6 },		// 180 bits per transform
			{ 24, 23, 5 },		// 180 bits per transform
			{ 6, 23, 24 },		// 183 bits per transform
			{ 6, 24, 23 },		// 183 bits per transform
			{ 7, 22, 24 },		// 183 bits per transform
			{ 7, 24, 22 },		// 183 bits per transform
			{ 8, 21, 24 },		// 183 bits per transform
			{ 8, 24, 21 },		// 183 bits per transform
			{ 9, 20, 24 },		// 183 bits per transform
			{ 9, 24, 20 },		// 183 bits per transform
			{ 10, 19, 24 },		// 183 bits per transform
			{ 10, 24, 19 },		// 183 bits per transform
			{ 11, 18, 24 },		// 183 bits per transform
			{ 11, 24, 18 },		// 183 bits per transform
			{ 12, 17, 24 },		// 183 bits per transform
			{ 12, 24, 17 },		// 183 bits per transform
			{ 13, 16, 24 },		// 183 bits per transform
			{ 13, 24, 16 },		// 183 bits per transform
			{ 14, 15, 24 },		// 183 bits per transform
			{ 14, 24, 15 },		// 183 bits per transform
			{ 15, 14, 24 },		// 183 bits per transform
			{ 15, 23, 23 },		// 183 bits per transform
			{ 15, 24, 14 },		// 183 bits per transform
			{ 16, 13, 24 },		// 183 bits per transform
			{ 16, 22, 23 },		// 183 bits per transform
			{ 16, 23, 22 },		// 183 bits per transform
			{ 16, 24, 13 },		// 183 bits per transform
			{ 17, 12, 24 },		// 183 bits per transform
			{ 17, 21, 23 },		// 183 bits per transform
			{ 17, 22, 22 },		// 183 bits per transform
			{ 17, 23, 21 },		// 183 bits per transform
			{ 17, 24, 12 },		// 183 bits per transform
			{ 18, 11, 24 },		// 183 bits per transform
			{ 18, 20, 23 },		// 183 bits per transform
			{ 18, 21, 22 },		// 183 bits per transform
			{ 18, 22, 21 },		// 183 bits per transform
			{ 18, 23, 20 },		// 183 bits per transform
			{ 18, 24, 11 },		// 183 bits per transform
			{ 19, 10, 24 },		// 183 bits per transform
			{ 19, 19, 23 },		// 183 bits per transform
			{ 19, 20, 22 },		// 183 bits per transform
			{ 19, 21, 21 },		// 183 bits per transform
			{ 19, 22, 20 },		// 183 bits per transform
			{ 19, 23, 19 },		// 183 bits per transform
			{ 19, 24, 10 },		// 183 bits per transform
			{ 20, 9, 24 },		// 183 bits per transform
			{ 20, 18, 23 },		// 183 bits per transform
			{ 20, 19, 22 },		// 183 bits per transform
			{ 20, 20, 21 },		// 183 bits per transform
			{ 20, 21, 20 },		// 183 bits per transform
			{ 20, 22, 19 },		// 183 bits per transform
			{ 20, 23, 18 },		// 183 bits per transform
			{ 20, 24, 9 },		// 183 bits per transform
			{ 21, 8, 24 },		// 183 bits per transform
			{ 21, 17, 23 },		// 183 bits per transform
			{ 21, 18, 22 },		// 183 bits per transform
			{ 21, 19, 21 },		// 183 bits per transform
			{ 21, 20, 20 },		// 183 bits per transform
			{ 21, 21, 19 },		// 183 bits per transform
			{ 21, 22, 18 },		// 183 bits per transform
			{ 21, 23, 17 },		// 183 bits per transform
			{ 21, 24, 8 },		// 183 bits per transform
			{ 22, 7, 24 },		// 183 bits per transform
			{ 22, 16, 23 },		// 183 bits per transform
			{ 22, 17, 22 },		// 183 bits per transform
			{ 22, 18, 21 },		// 183 bits per transform
			{ 22, 19, 20 },		// 183 bits per transform
			{ 22, 20, 19 },		// 183 bits per transform
			{ 22, 21, 18 },		// 183 bits per transform
			{ 22, 22, 17 },		// 183 bits per transform
			{ 22, 23, 16 },		// 183 bits per transform
			{ 22, 24, 7 },		// 183 bits per transform
			{ 23, 6, 24 },		// 183 bits per transform
			{ 23, 15, 23 },		// 183 bits per transform
			{ 23, 16, 22 },		// 183 bits per transform
			{ 23, 17, 21 },		// 183 bits per transform
			{ 23, 18, 20 },		// 183 bits per transform
			{ 23, 19, 19 },		// 183 bits per transform
			{ 23, 20, 18 },		// 183 bits per transform
			{ 23, 21, 17 },		// 183 bits per transform
			{ 23, 22, 16 },		// 183 bits per transform
			{ 23, 23, 15 },		// 183 bits per transform
			{ 23, 24, 6 },		// 183 bits per transform
			{ 24, 6, 23 },		// 183 bits per transform
			{ 24, 7, 22 },		// 183 bits per transform
			{ 24, 8, 21 },		// 183 bits per transform
			{ 24, 9, 20 },		// 183 bits per transform
			{ 24, 10, 19 },		// 183 bits per transform
			{ 24, 11, 18 },		// 183 bits per transform
			{ 24, 12, 17 },		// 183 bits per transform
			{ 24, 13, 16 },		// 183 bits per transform
			{ 24, 14, 15 },		// 183 bits per transform
			{ 24, 15, 14 },		// 183 bits per transform
			{ 24, 16, 13 },		// 183 bits per transform
			{ 24, 17, 12 },		// 183 bits per transform
			{ 24, 18, 11 },		// 183 bits per transform
			{ 24, 19, 10 },		// 183 bits per transform
			{ 24, 20, 9 },		// 183 bits per transform
			{ 24, 21, 8 },		// 183 bits per transform
			{ 24, 22, 7 },		// 183 bits per transform
			{ 24, 23, 6 },		// 183 bits per transform
			{ 7, 23, 24 },		// 186 bits per transform
			{ 7, 24, 23 },		// 186 bits per transform
			{ 8, 22, 24 },		// 186 bits per transform
			{ 8, 24, 22 },		// 186 bits per transform
			{ 9, 21, 24 },		// 186 bits per transform
			{ 9, 24, 21 },		// 186 bits per transform
			{ 10, 20, 24 },		// 186 bits per transform
			{ 10, 24, 20 },		// 186 bits per transform
			{ 11, 19, 24 },		// 186 bits per transform
			{ 11, 24, 19 },		// 186 bits per transform
			{ 12, 18, 24 },		// 186 bits per transform
			{ 12, 24, 18 },		// 186 bits per transform
			{ 13, 17, 24 },		// 186 bits per transform
			{ 13, 24, 17 },		// 186 bits per transform
			{ 14, 16, 24 },		// 186 bits per transform
			{ 14, 24, 16 },		// 186 bits per transform
			{ 15, 15, 24 },		// 186 bits per transform
			{ 15, 24, 15 },		// 186 bits per transform
			{ 16, 14, 24 },		// 186 bits per transform
			{ 16, 23, 23 },		// 186 bits per transform
			{ 16, 24, 14 },		// 186 bits per transform
			{ 17, 13, 24 },		// 186 bits per transform
			{ 17, 22, 23 },		// 186 bits per transform
			{ 17, 23, 22 },		// 186 bits per transform
			{ 17, 24, 13 },		// 186 bits per transform
			{ 18, 12, 24 },		// 186 bits per transform
			{ 18, 21, 23 },		// 186 bits per transform
			{ 18, 22, 22 },		// 186 bits per transform
			{ 18, 23, 21 },		// 186 bits per transform
			{ 18, 24, 12 },		// 186 bits per transform
			{ 19, 11, 24 },		// 186 bits per transform
			{ 19, 20, 23 },		// 186 bits per transform
			{ 19, 21, 22 },		// 186 bits per transform
			{ 19, 22, 21 },		// 186 bits per transform
			{ 19, 23, 20 },		// 186 bits per transform
			{ 19, 24, 11 },		// 186 bits per transform
			{ 20, 10, 24 },		// 186 bits per transform
			{ 20, 19, 23 },		// 186 bits per transform
			{ 20, 20, 22 },		// 186 bits per transform
			{ 20, 21, 21 },		// 186 bits per transform
			{ 20, 22, 20 },		// 186 bits per transform
			{ 20, 23, 19 },		// 186 bits per transform
			{ 20, 24, 10 },		// 186 bits per transform
			{ 21, 9, 24 },		// 186 bits per transform
			{ 21, 18, 23 },		// 186 bits per transform
			{ 21, 19, 22 },		// 186 bits per transform
			{ 21, 20, 21 },		// 186 bits per transform
			{ 21, 21, 20 },		// 186 bits per transform
			{ 21, 22, 19 },		// 186 bits per transform
			{ 21, 23, 18 },		// 186 bits per transform
			{ 21, 24, 9 },		// 186 bits per transform
			{ 22, 8, 24 },		// 186 bits per transform
			{ 22, 17, 23 },		// 186 bits per transform
			{ 22, 18, 22 },		// 186 bits per transform
			{ 22, 19, 21 },		// 186 bits per transform
			{ 22, 20, 20 },		// 186 bits per transform
			{ 22, 21, 19 },		// 186 bits per transform
			{ 22, 22, 18 },		// 186 bits per transform
			{ 22, 23, 17 },		// 186 bits per transform
			{ 22, 24, 8 },		// 186 bits per transform
			{ 23, 7, 24 },		// 186 bits per transform
			{ 23, 16, 23 },		// 186 bits per transform
			{ 23, 17, 22 },		// 186 bits per transform
			{ 23, 18, 21 },		// 186 bits per transform
			{ 23, 19, 20 },		// 186 bits per transform
			{ 23, 20, 19 },		// 186 bits per transform
			{ 23, 21, 18 },		// 186 bits per transform
			{ 23, 22, 17 },		// 186 bits per transform
			{ 23, 23, 16 },		// 186 bits per transform
			{ 23, 24, 7 },		// 186 bits per transform
			{ 24, 7, 23 },		// 186 bits per transform
			{ 24, 8, 22 },		// 186 bits per transform
			{ 24, 9, 21 },		// 186 bits per transform
			{ 24, 10, 20 },		// 186 bits per transform
			{ 24, 11, 19 },		// 186 bits per transform
			{ 24, 12, 18 },		// 186 bits per transform
			{ 24, 13, 17 },		// 186 bits per transform
			{ 24, 14, 16 },		// 186 bits per transform
			{ 24, 15, 15 },		// 186 bits per transform
			{ 24, 16, 14 },		// 186 bits per transform
			{ 24, 17, 13 },		// 186 bits per transform
			{ 24, 18, 12 },		// 186 bits per transform
			{ 24, 19, 11 },		// 186 bits per transform
			{ 24, 20, 10 },		// 186 bits per transform
			{ 24, 21, 9 },		// 186 bits per transform
			{ 24, 22, 8 },		// 186 bits per transform
			{ 24, 23, 7 },		// 186 bits per transform
			{ 8, 23, 24 },		// 189 bits per transform
			{ 8, 24, 23 },		// 189 bits per transform
			{ 9, 22, 24 },		// 189 bits per transform
			{ 9, 24, 22 },		// 189 bits per transform
			{ 10, 21, 24 },		// 189 bits per transform
			{ 10, 24, 21 },		// 189 bits per transform
			{ 11, 20, 24 },		// 189 bits per transform
			{ 11, 24, 20 },		// 189 bits per transform
			{ 12, 19, 24 },		// 189 bits per transform
			{ 12, 24, 19 },		// 189 bits per transform
			{ 13, 18, 24 },		// 189 bits per transform
			{ 13, 24, 18 },		// 189 bits per transform
			{ 14, 17, 24 },		// 189 bits per transform
			{ 14, 24, 17 },		// 189 bits per transform
			{ 15, 16, 24 },		// 189 bits per transform
			{ 15, 24, 16 },		// 189 bits per transform
			{ 16, 15, 24 },		// 189 bits per transform
			{ 16, 24, 15 },		// 189 bits per transform
			{ 17, 14, 24 },		// 189 bits per transform
			{ 17, 23, 23 },		// 189 bits per transform
			{ 17, 24, 14 },		// 189 bits per transform
			{ 18, 13, 24 },		// 189 bits per transform
			{ 18, 22, 23 },		// 189 bits per transform
			{ 18, 23, 22 },		// 189 bits per transform
			{ 18, 24, 13 },		// 189 bits per transform
			{ 19, 12, 24 },		// 189 bits per transform
			{ 19, 21, 23 },		// 189 bits per transform
			{ 19, 22, 22 },		// 189 bits per transform
			{ 19, 23, 21 },		// 189 bits per transform
			{ 19, 24, 12 },		// 189 bits per transform
			{ 20, 11, 24 },		// 189 bits per transform
			{ 20, 20, 23 },		// 189 bits per transform
			{ 20, 21, 22 },		// 189 bits per transform
			{ 20, 22, 21 },		// 189 bits per transform
			{ 20, 23, 20 },		// 189 bits per transform
			{ 20, 24, 11 },		// 189 bits per transform
			{ 21, 10, 24 },		// 189 bits per transform
			{ 21, 19, 23 },		// 189 bits per transform
			{ 21, 20, 22 },		// 189 bits per transform
			{ 21, 21, 21 },		// 189 bits per transform
			{ 21, 22, 20 },		// 189 bits per transform
			{ 21, 23, 19 },		// 189 bits per transform
			{ 21, 24, 10 },		// 189 bits per transform
			{ 22, 9, 24 },		// 189 bits per transform
			{ 22, 18, 23 },		// 189 bits per transform
			{ 22, 19, 22 },		// 189 bits per transform
			{ 22, 20, 21 },		// 189 bits per transform
			{ 22, 21, 20 },		// 189 bits per transform
			{ 22, 22, 19 },		// 189 bits per transform
			{ 22, 23, 18 },		// 189 bits per transform
			{ 22, 24, 9 },		// 189 bits per transform
			{ 23, 8, 24 },		// 189 bits per transform
			{ 23, 17, 23 },		// 189 bits per transform
			{ 23, 18, 22 },		// 189 bits per transform
			{ 23, 19, 21 },		// 189 bits per transform
			{ 23, 20, 20 },		// 189 bits per transform
			{ 23, 21, 19 },		// 189 bits per transform
			{ 23, 22, 18 },		// 189 bits per transform
			{ 23, 23, 17 },		// 189 bits per transform
			{ 23, 24, 8 },		// 189 bits per transform
			{ 24, 8, 23 },		// 189 bits per transform
			{ 24, 9, 22 },		// 189 bits per transform
			{ 24, 10, 21 },		// 189 bits per transform
			{ 24, 11, 20 },		// 189 bits per transform
			{ 24, 12, 19 },		// 189 bits per transform
			{ 24, 13, 18 },		// 189 bits per transform
			{ 24, 14, 17 },		// 189 bits per transform
			{ 24, 15, 16 },		// 189 bits per transform
			{ 24, 16, 15 },		// 189 bits per transform
			{ 24, 17, 14 },		// 189 bits per transform
			{ 24, 18, 13 },		// 189 bits per transform
			{ 24, 19, 12 },		// 189 bits per transform
			{ 24, 20, 11 },		// 189 bits per transform
			{ 24, 21, 10 },		// 189 bits per transform
			{ 24, 22, 9 },		// 189 bits per transform
			{ 24, 23, 8 },		// 189 bits per transform
			{ 0, 24, 24 },		// 192 bits per transform
			{ 9, 23, 24 },		// 192 bits per transform
			{ 9, 24, 23 },		// 192 bits per transform
			{ 10, 22, 24 },		// 192 bits per transform
			{ 10, 24, 22 },		// 192 bits per transform
			{ 11, 21, 24 },		// 192 bits per transform
			{ 11, 24, 21 },		// 192 bits per transform
			{ 12, 20, 24 },		// 192 bits per transform
			{ 12, 24, 20 },		// 192 bits per transform
			{ 13, 19, 24 },		// 192 bits per transform
			{ 13, 24, 19 },		// 192 bits per transform
			{ 14, 18, 24 },		// 192 bits per transform
			{ 14, 24, 18 },		// 192 bits per transform
			{ 15, 17, 24 },		// 192 bits per transform
			{ 15, 24, 17 },		// 192 bits per transform
			{ 16, 16, 24 },		// 192 bits per transform
			{ 16, 24, 16 },		// 192 bits per transform
			{ 17, 15, 24 },		// 192 bits per transform
			{ 17, 24, 15 },		// 192 bits per transform
			{ 18, 14, 24 },		// 192 bits per transform
			{ 18, 23, 23 },		// 192 bits per transform
			{ 18, 24, 14 },		// 192 bits per transform
			{ 19, 13, 24 },		// 192 bits per transform
			{ 19, 22, 23 },		// 192 bits per transform
			{ 19, 23, 22 },		// 192 bits per transform
			{ 19, 24, 13 },		// 192 bits per transform
			{ 20, 12, 24 },		// 192 bits per transform
			{ 20, 21, 23 },		// 192 bits per transform
			{ 20, 22, 22 },		// 192 bits per transform
			{ 20, 23, 21 },		// 192 bits per transform
			{ 20, 24, 12 },		// 192 bits per transform
			{ 21, 11, 24 },		// 192 bits per transform
			{ 21, 20, 23 },		// 192 bits per transform
			{ 21, 21, 22 },		// 192 bits per transform
			{ 21, 22, 21 },		// 192 bits per transform
			{ 21, 23, 20 },		// 192 bits per transform
			{ 21, 24, 11 },		// 192 bits per transform
			{ 22, 10, 24 },		// 192 bits per transform
			{ 22, 19, 23 },		// 192 bits per transform
			{ 22, 20, 22 },		// 192 bits per transform
			{ 22, 21, 21 },		// 192 bits per transform
			{ 22, 22, 20 },		// 192 bits per transform
			{ 22, 23, 19 },		// 192 bits per transform
			{ 22, 24, 10 },		// 192 bits per transform
			{ 23, 9, 24 },		// 192 bits per transform
			{ 23, 18, 23 },		// 192 bits per transform
			{ 23, 19, 22 },		// 192 bits per transform
			{ 23, 20, 21 },		// 192 bits per transform
			{ 23, 21, 20 },		// 192 bits per transform
			{ 23, 22, 19 },		// 192 bits per transform
			{ 23, 23, 18 },		// 192 bits per transform
			{ 23, 24, 9 },		// 192 bits per transform
			{ 24, 0, 24 },		// 192 bits per transform
			{ 24, 9, 23 },		// 192 bits per transform
			{ 24, 10, 22 },		// 192 bits per transform
			{ 24, 11, 21 },		// 192 bits per transform
			{ 24, 12, 20 },		// 192 bits per transform
			{ 24, 13, 19 },		// 192 bits per transform
			{ 24, 14, 18 },		// 192 bits per transform
			{ 24, 15, 17 },		// 192 bits per transform
			{ 24, 16, 16 },		// 192 bits per transform
			{ 24, 17, 15 },		// 192 bits per transform
			{ 24, 18, 14 },		// 192 bits per transform
			{ 24, 19, 13 },		// 192 bits per transform
			{ 24, 20, 12 },		// 192 bits per transform
			{ 24, 21, 11 },		// 192 bits per transform
			{ 24, 22, 10 },		// 192 bits per transform
			{ 24, 23, 9 },		// 192 bits per transform
			{ 24, 24, 0 },		// 192 bits per transform
			{ 1, 24, 24 },		// 195 bits per transform
			{ 10, 23, 24 },		// 195 bits per transform
			{ 10, 24, 23 },		// 195 bits per transform
			{ 11, 22, 24 },		// 195 bits per transform
			{ 11, 24, 22 },		// 195 bits per transform
			{ 12, 21, 24 },		// 195 bits per transform
			{ 12, 24, 21 },		// 195 bits per transform
			{ 13, 20, 24 },		// 195 bits per transform
			{ 13, 24, 20 },		// 195 bits per transform
			{ 14, 19, 24 },		// 195 bits per transform
			{ 14, 24, 19 },		// 195 bits per transform
			{ 15, 18, 24 },		// 195 bits per transform
			{ 15, 24, 18 },		// 195 bits per transform
			{ 16, 17, 24 },		// 195 bits per transform
			{ 16, 24, 17 },		// 195 bits per transform
			{ 17, 16, 24 },		// 195 bits per transform
			{ 17, 24, 16 },		// 195 bits per transform
			{ 18, 15, 24 },		// 195 bits per transform
			{ 18, 24, 15 },		// 195 bits per transform
			{ 19, 14, 24 },		// 195 bits per transform
			{ 19, 23, 23 },		// 195 bits per transform
			{ 19, 24, 14 },		// 195 bits per transform
			{ 20, 13, 24 },		// 195 bits per transform
			{ 20, 22, 23 },		// 195 bits per transform
			{ 20, 23, 22 },		// 195 bits per transform
			{ 20, 24, 13 },		// 195 bits per transform
			{ 21, 12, 24 },		// 195 bits per transform
			{ 21, 21, 23 },		// 195 bits per transform
			{ 21, 22, 22 },		// 195 bits per transform
			{ 21, 23, 21 },		// 195 bits per transform
			{ 21, 24, 12 },		// 195 bits per transform
			{ 22, 11, 24 },		// 195 bits per transform
			{ 22, 20, 23 },		// 195 bits per transform
			{ 22, 21, 22 },		// 195 bits per transform
			{ 22, 22, 21 },		// 195 bits per transform
			{ 22, 23, 20 },		// 195 bits per transform
			{ 22, 24, 11 },		// 195 bits per transform
			{ 23, 10, 24 },		// 195 bits per transform
			{ 23, 19, 23 },		// 195 bits per transform
			{ 23, 20, 22 },		// 195 bits per transform
			{ 23, 21, 21 },		// 195 bits per transform
			{ 23, 22, 20 },		// 195 bits per transform
			{ 23, 23, 19 },		// 195 bits per transform
			{ 23, 24, 10 },		// 195 bits per transform
			{ 24, 1, 24 },		// 195 bits per transform
			{ 24, 10, 23 },		// 195 bits per transform
			{ 24, 11, 22 },		// 195 bits per transform
			{ 24, 12, 21 },		// 195 bits per transform
			{ 24, 13, 20 },		// 195 bits per transform
			{ 24, 14, 19 },		// 195 bits per transform
			{ 24, 15, 18 },		// 195 bits per transform
			{ 24, 16, 17 },		// 195 bits per transform
			{ 24, 17, 16 },		// 195 bits per transform
			{ 24, 18, 15 },		// 195 bits per transform
			{ 24, 19, 14 },		// 195 bits per transform
			{ 24, 20, 13 },		// 195 bits per transform
			{ 24, 21, 12 },		// 195 bits per transform
			{ 24, 22, 11 },		// 195 bits per transform
			{ 24, 23, 10 },		// 195 bits per transform
			{ 24, 24, 1 },		// 195 bits per transform
			{ 2, 24, 24 },		// 198 bits per transform
			{ 11, 23, 24 },		// 198 bits per transform
			{ 11, 24, 23 },		// 198 bits per transform
			{ 12, 22, 24 },		// 198 bits per transform
			{ 12, 24, 22 },		// 198 bits per transform
			{ 13, 21, 24 },		// 198 bits per transform
			{ 13, 24, 21 },		// 198 bits per transform
			{ 14, 20, 24 },		// 198 bits per transform
			{ 14, 24, 20 },		// 198 bits per transform
			{ 15, 19, 24 },		// 198 bits per transform
			{ 15, 24, 19 },		// 198 bits per transform
			{ 16, 18, 24 },		// 198 bits per transform
			{ 16, 24, 18 },		// 198 bits per transform
			{ 17, 17, 24 },		// 198 bits per transform
			{ 17, 24, 17 },		// 198 bits per transform
			{ 18, 16, 24 },		// 198 bits per transform
			{ 18, 24, 16 },		// 198 bits per transform
			{ 19, 15, 24 },		// 198 bits per transform
			{ 19, 24, 15 },		// 198 bits per transform
			{ 20, 14, 24 },		// 198 bits per transform
			{ 20, 23, 23 },		// 198 bits per transform
			{ 20, 24, 14 },		// 198 bits per transform
			{ 21, 13, 24 },		// 198 bits per transform
			{ 21, 22, 23 },		// 198 bits per transform
			{ 21, 23, 22 },		// 198 bits per transform
			{ 21, 24, 13 },		// 198 bits per transform
			{ 22, 12, 24 },		// 198 bits per transform
			{ 22, 21, 23 },		// 198 bits per transform
			{ 22, 22, 22 },		// 198 bits per transform
			{ 22, 23, 21 },		// 198 bits per transform
			{ 22, 24, 12 },		// 198 bits per transform
			{ 23, 11, 24 },		// 198 bits per transform
			{ 23, 20, 23 },		// 198 bits per transform
			{ 23, 21, 22 },		// 198 bits per transform
			{ 23, 22, 21 },		// 198 bits per transform
			{ 23, 23, 20 },		// 198 bits per transform
			{ 23, 24, 11 },		// 198 bits per transform
			{ 24, 2, 24 },		// 198 bits per transform
			{ 24, 11, 23 },		// 198 bits per transform
			{ 24, 12, 22 },		// 198 bits per transform
			{ 24, 13, 21 },		// 198 bits per transform
			{ 24, 14, 20 },		// 198 bits per transform
			{ 24, 15, 19 },		// 198 bits per transform
			{ 24, 16, 18 },		// 198 bits per transform
			{ 24, 17, 17 },		// 198 bits per transform
			{ 24, 18, 16 },		// 198 bits per transform
			{ 24, 19, 15 },		// 198 bits per transform
			{ 24, 20, 14 },		// 198 bits per transform
			{ 24, 21, 13 },		// 198 bits per transform
			{ 24, 22, 12 },		// 198 bits per transform
			{ 24, 23, 11 },		// 198 bits per transform
			{ 24, 24, 2 },		// 198 bits per transform
			{ 3, 24, 24 },		// 201 bits per transform
			{ 12, 23, 24 },		// 201 bits per transform
			{ 12, 24, 23 },		// 201 bits per transform
			{ 13, 22, 24 },		// 201 bits per transform
			{ 13, 24, 22 },		// 201 bits per transform
			{ 14, 21, 24 },		// 201 bits per transform
			{ 14, 24, 21 },		// 201 bits per transform
			{ 15, 20, 24 },		// 201 bits per transform
			{ 15, 24, 20 },		// 201 bits per transform
			{ 16, 19, 24 },		// 201 bits per transform
			{ 16, 24, 19 },		// 201 bits per transform
			{ 17, 18, 24 },		// 201 bits per transform
			{ 17, 24, 18 },		// 201 bits per transform
			{ 18, 17, 24 },		// 201 bits per transform
			{ 18, 24, 17 },		// 201 bits per transform
			{ 19, 16, 24 },		// 201 bits per transform
			{ 19, 24, 16 },		// 201 bits per transform
			{ 20, 15, 24 },		// 201 bits per transform
			{ 20, 24, 15 },		// 201 bits per transform
			{ 21, 14, 24 },		// 201 bits per transform
			{ 21, 23, 23 },		// 201 bits per transform
			{ 21, 24, 14 },		// 201 bits per transform
			{ 22, 13, 24 },		// 201 bits per transform
			{ 22, 22, 23 },		// 201 bits per transform
			{ 22, 23, 22 },		// 201 bits per transform
			{ 22, 24, 13 },		// 201 bits per transform
			{ 23, 12, 24 },		// 201 bits per transform
			{ 23, 21, 23 },		// 201 bits per transform
			{ 23, 22, 22 },		// 201 bits per transform
			{ 23, 23, 21 },		// 201 bits per transform
			{ 23, 24, 12 },		// 201 bits per transform
			{ 24, 3, 24 },		// 201 bits per transform
			{ 24, 12, 23 },		// 201 bits per transform
			{ 24, 13, 22 },		// 201 bits per transform
			{ 24, 14, 21 },		// 201 bits per transform
			{ 24, 15, 20 },		// 201 bits per transform
			{ 24, 16, 19 },		// 201 bits per transform
			{ 24, 17, 18 },		// 201 bits per transform
			{ 24, 18, 17 },		// 201 bits per transform
			{ 24, 19, 16 },		// 201 bits per transform
			{ 24, 20, 15 },		// 201 bits per transform
			{ 24, 21, 14 },		// 201 bits per transform
			{ 24, 22, 13 },		// 201 bits per transform
			{ 24, 23, 12 },		// 201 bits per transform
			{ 24, 24, 3 },		// 201 bits per transform
			{ 4, 24, 24 },		// 204 bits per transform
			{ 13, 23, 24 },		// 204 bits per transform
			{ 13, 24, 23 },		// 204 bits per transform
			{ 14, 22, 24 },		// 204 bits per transform
			{ 14, 24, 22 },		// 204 bits per transform
			{ 15, 21, 24 },		// 204 bits per transform
			{ 15, 24, 21 },		// 204 bits per transform
			{ 16, 20, 24 },		// 204 bits per transform
			{ 16, 24, 20 },		// 204 bits per transform
			{ 17, 19, 24 },		// 204 bits per transform
			{ 17, 24, 19 },		// 204 bits per transform
			{ 18, 18, 24 },		// 204 bits per transform
			{ 18, 24, 18 },		// 204 bits per transform
			{ 19, 17, 24 },		// 204 bits per transform
			{ 19, 24, 17 },		// 204 bits per transform
			{ 20, 16, 24 },		// 204 bits per transform
			{ 20, 24, 16 },		// 204 bits per transform
			{ 21, 15, 24 },		// 204 bits per transform
			{ 21, 24, 15 },		// 204 bits per transform
			{ 22, 14, 24 },		// 204 bits per transform
			{ 22, 23, 23 },		// 204 bits per transform
			{ 22, 24, 14 },		// 204 bits per transform
			{ 23, 13, 24 },		// 204 bits per transform
			{ 23, 22, 23 },		// 204 bits per transform
			{ 23, 23, 22 },		// 204 bits per transform
			{ 23, 24, 13 },		// 204 bits per transform
			{ 24, 4, 24 },		// 204 bits per transform
			{ 24, 13, 23 },		// 204 bits per transform
			{ 24, 14, 22 },		// 204 bits per transform
			{ 24, 15, 21 },		// 204 bits per transform
			{ 24, 16, 20 },		// 204 bits per transform
			{ 24, 17, 19 },		// 204 bits per transform
			{ 24, 18, 18 },		// 204 bits per transform
			{ 24, 19, 17 },		// 204 bits per transform
			{ 24, 20, 16 },		// 204 bits per transform
			{ 24, 21, 15 },		// 204 bits per transform
			{ 24, 22, 14 },		// 204 bits per transform
			{ 24, 23, 13 },		// 204 bits per transform
			{ 24, 24, 4 },		// 204 bits per transform
			{ 5, 24, 24 },		// 207 bits per transform
			{ 14, 23, 24 },		// 207 bits per transform
			{ 14, 24, 23 },		// 207 bits per transform
			{ 15, 22, 24 },		// 207 bits per transform
			{ 15, 24, 22 },		// 207 bits per transform
			{ 16, 21, 24 },		// 207 bits per transform
			{ 16, 24, 21 },		// 207 bits per transform
			{ 17, 20, 24 },		// 207 bits per transform
			{ 17, 24, 20 },		// 207 bits per transform
			{ 18, 19, 24 },		// 207 bits per transform
			{ 18, 24, 19 },		// 207 bits per transform
			{ 19, 18, 24 },		// 207 bits per transform
			{ 19, 24, 18 },		// 207 bits per transform
			{ 20, 17, 24 },		// 207 bits per transform
			{ 20, 24, 17 },		// 207 bits per transform
			{ 21, 16, 24 },		// 207 bits per transform
			{ 21, 24, 16 },		// 207 bits per transform
			{ 22, 15, 24 },		// 207 bits per transform
			{ 22, 24, 15 },		// 207 bits per transform
			{ 23, 14, 24 },		// 207 bits per transform
			{ 23, 23, 23 },		// 207 bits per transform
			{ 23, 24, 14 },		// 207 bits per transform
			{ 24, 5, 24 },		// 207 bits per transform
			{ 24, 14, 23 },		// 207 bits per transform
			{ 24, 15, 22 },		// 207 bits per transform
			{ 24, 16, 21 },		// 207 bits per transform
			{ 24, 17, 20 },		// 207 bits per transform
			{ 24, 18, 19 },		// 207 bits per transform
			{ 24, 19, 18 },		// 207 bits per transform
			{ 24, 20, 17 },		// 207 bits per transform
			{ 24, 21, 16 },		// 207 bits per transform
			{ 24, 22, 15 },		// 207 bits per transform
			{ 24, 23, 14 },		// 207 bits per transform
			{ 24, 24, 5 },		// 207 bits per transform
			{ 6, 24, 24 },		// 210 bits per transform
			{ 15, 23, 24 },		// 210 bits per transform
			{ 15, 24, 23 },		// 210 bits per transform
			{ 16, 22, 24 },		// 210 bits per transform
			{ 16, 24, 22 },		// 210 bits per transform
			{ 17, 21, 24 },		// 210 bits per transform
			{ 17, 24, 21 },		// 210 bits per transform
			{ 18, 20, 24 },		// 210 bits per transform
			{ 18, 24, 20 },		// 210 bits per transform
			{ 19, 19, 24 },		// 210 bits per transform
			{ 19, 24, 19 },		// 210 bits per transform
			{ 20, 18, 24 },		// 210 bits per transform
			{ 20, 24, 18 },		// 210 bits per transform
			{ 21, 17, 24 },		// 210 bits per transform
			{ 21, 24, 17 },		// 210 bits per transform
			{ 22, 16, 24 },		// 210 bits per transform
			{ 22, 24, 16 },		// 210 bits per transform
			{ 23, 15, 24 },		// 210 bits per transform
			{ 23, 24, 15 },		// 210 bits per transform
			{ 24, 6, 24 },		// 210 bits per transform
			{ 24, 15, 23 },		// 210 bits per transform
			{ 24, 16, 22 },		// 210 bits per transform
			{ 24, 17, 21 },		// 210 bits per transform
			{ 24, 18, 20 },		// 210 bits per transform
			{ 24, 19, 19 },		// 210 bits per transform
			{ 24, 20, 18 },		// 210 bits per transform
			{ 24, 21, 17 },		// 210 bits per transform
			{ 24, 22, 16 },		// 210 bits per transform
			{ 24, 23, 15 },		// 210 bits per transform
			{ 24, 24, 6 },		// 210 bits per transform
			{ 7, 24, 24 },		// 213 bits per transform
			{ 16, 23, 24 },		// 213 bits per transform
			{ 16, 24, 23 },		// 213 bits per transform
			{ 17, 22, 24 },		// 213 bits per transform
			{ 17, 24, 22 },		// 213 bits per transform
			{ 18, 21, 24 },		// 213 bits per transform
			{ 18, 24, 21 },		// 213 bits per transform
			{ 19, 20, 24 },		// 213 bits per transform
			{ 19, 24, 20 },		// 213 bits per transform
			{ 20, 19, 24 },		// 213 bits per transform
			{ 20, 24, 19 },		// 213 bits per transform
			{ 21, 18, 24 },		// 213 bits per transform
			{ 21, 24, 18 },		// 213 bits per transform
			{ 22, 17, 24 },		// 213 bits per transform
			{ 22, 24, 17 },		// 213 bits per transform
			{ 23, 16, 24 },		// 213 bits per transform
			{ 23, 24, 16 },		// 213 bits per transform
			{ 24, 7, 24 },		// 213 bits per transform
			{ 24, 16, 23 },		// 213 bits per transform
			{ 24, 17, 22 },		// 213 bits per transform
			{ 24, 18, 21 },		// 213 bits per transform
			{ 24, 19, 20 },		// 213 bits per transform
			{ 24, 20, 19 },		// 213 bits per transform
			{ 24, 21, 18 },		// 213 bits per transform
			{ 24, 22, 17 },		// 213 bits per transform
			{ 24, 23, 16 },		// 213 bits per transform
			{ 24, 24, 7 },		// 213 bits per transform
			{ 8, 24, 24 },		// 216 bits per transform
			{ 17, 23, 24 },		// 216 bits per transform
			{ 17, 24, 23 },		// 216 bits per transform
			{ 18, 22, 24 },		// 216 bits per transform
			{ 18, 24, 22 },		// 216 bits per transform
			{ 19, 21, 24 },		// 216 bits per transform
			{ 19, 24, 21 },		// 216 bits per transform
			{ 20, 20, 24 },		// 216 bits per transform
			{ 20, 24, 20 },		// 216 bits per transform
			{ 21, 19, 24 },		// 216 bits per transform
			{ 21, 24, 19 },		// 216 bits per transform
			{ 22, 18, 24 },		// 216 bits per transform
			{ 22, 24, 18 },		// 216 bits per transform
			{ 23, 17, 24 },		// 216 bits per transform
			{ 23, 24, 17 },		// 216 bits per transform
			{ 24, 8, 24 },		// 216 bits per transform
			{ 24, 17, 23 },		// 216 bits per transform
			{ 24, 18, 22 },		// 216 bits per transform
			{ 24, 19, 21 },		// 216 bits per transform
			{ 24, 20, 20 },		// 216 bits per transform
			{ 24, 21, 19 },		// 216 bits per transform
			{ 24, 22, 18 },		// 216 bits per transform
			{ 24, 23, 17 },		// 216 bits per transform
			{ 24, 24, 8 },		// 216 bits per transform
			{ 9, 24, 24 },		// 219 bits per transform
			{ 18, 23, 24 },		// 219 bits per transform
			{ 18, 24, 23 },		// 219 bits per transform
			{ 19, 22, 24 },		// 219 bits per transform
			{ 19, 24, 22 },		// 219 bits per transform
			{ 20, 21, 24 },		// 219 bits per transform
			{ 20, 24, 21 },		// 219 bits per transform
			{ 21, 20, 24 },		// 219 bits per transform
			{ 21, 24, 20 },		// 219 bits per transform
			{ 22, 19, 24 },		// 219 bits per transform
			{ 22, 24, 19 },		// 219 bits per transform
			{ 23, 18, 24 },		// 219 bits per transform
			{ 23, 24, 18 },		// 219 bits per transform
			{ 24, 9, 24 },		// 219 bits per transform
			{ 24, 18, 23 },		// 219 bits per transform
			{ 24, 19, 22 },		// 219 bits per transform
			{ 24, 20, 21 },		// 219 bits per transform
			{ 24, 21, 20 },		// 219 bits per transform
			{ 24, 22, 19 },		// 219 bits per transform
			{ 24, 23, 18 },		// 219 bits per transform
			{ 24, 24, 9 },		// 219 bits per transform
			{ 10, 24, 24 },		// 222 bits per transform
			{ 19, 23, 24 },		// 222 bits per transform
			{ 19, 24, 23 },		// 222 bits per transform
			{ 20, 22, 24 },		// 222 bits per transform
			{ 20, 24, 22 },		// 222 bits per transform
			{ 21, 21, 24 },		// 222 bits per transform
			{ 21, 24, 21 },		// 222 bits per transform
			{ 22, 20, 24 },		// 222 bits per transform
			{ 22, 24, 20 },		// 222 bits per transform
			{ 23, 19, 24 },		// 222 bits per transform
			{ 23, 24, 19 },		// 222 bits per transform
			{ 24, 10, 24 },		// 222 bits per transform
			{ 24, 19, 23 },		// 222 bits per transform
			{ 24, 20, 22 },		// 222 bits per transform
			{ 24, 21, 21 },		// 222 bits per transform
			{ 24, 22, 20 },		// 222 bits per transform
			{ 24, 23, 19 },		// 222 bits per transform
			{ 24, 24, 10 },		// 222 bits per transform
			{ 11, 24, 24 },		// 225 bits per transform
			{ 20, 23, 24 },		// 225 bits per transform
			{ 20, 24, 23 },		// 225 bits per transform
			{ 21, 22, 24 },		// 225 bits per transform
			{ 21, 24, 22 },		// 225 bits per transform
			{ 22, 21, 24 },		// 225 bits per transform
			{ 22, 24, 21 },		// 225 bits per transform
			{ 23, 20, 24 },		// 225 bits per transform
			{ 23, 24, 20 },		// 225 bits per transform
			{ 24, 11, 24 },		// 225 bits per transform
			{ 24, 20, 23 },		// 225 bits per transform
			{ 24, 21, 22 },		// 225 bits per transform
			{ 24, 22, 21 },		// 225 bits per transform
			{ 24, 23, 20 },		// 225 bits per transform
			{ 24, 24, 11 },		// 225 bits per transform
			{ 12, 24, 24 },		// 228 bits per transform
			{ 21, 23, 24 },		// 228 bits per transform
			{ 21, 24, 23 },		// 228 bits per transform
			{ 22, 22, 24 },		// 228 bits per transform
			{ 22, 24, 22 },		// 228 bits per transform
			{ 23, 21, 24 },		// 228 bits per transform
			{ 23, 24, 21 },		// 228 bits per transform
			{ 24, 12, 24 },		// 228 bits per transform
			{ 24, 21, 23 },		// 228 bits per transform
			{ 24, 22, 22 },		// 228 bits per transform
			{ 24, 23, 21 },		// 228 bits per transform
			{ 24, 24, 12 },		// 228 bits per transform
			{ 13, 24, 24 },		// 231 bits per transform
			{ 22, 23, 24 },		// 231 bits per transform
			{ 22, 24, 23 },		// 231 bits per transform
			{ 23, 22, 24 },		// 231 bits per transform
			{ 23, 24, 22 },		// 231 bits per transform
			{ 24, 13, 24 },		// 231 bits per transform
			{ 24, 22, 23 },		// 231 bits per transform
			{ 24, 23, 22 },		// 231 bits per transform
			{ 24, 24, 13 },		// 231 bits per transform
			{ 14, 24, 24 },		// 234 bits per transform
			{ 23, 23, 24 },		// 234 bits per transform
			{ 23, 24, 23 },		// 234 bits per transform
			{ 24, 14, 24 },		// 234 bits per transform
			{ 24, 23, 23 },		// 234 bits per transform
			{ 24, 24, 14 },		// 234 bits per transform
			{ 15, 24, 24 },		// 237 bits per transform
			{ 24, 15, 24 },		// 237 bits per transform
			{ 24, 24, 15 },		// 237 bits per transform
			{ 16, 24, 24 },		// 240 bits per transform
			{ 24, 16, 24 },		// 240 bits per transform
			{ 24, 24, 16 },		// 240 bits per transform
			{ 17, 24, 24 },		// 243 bits per transform
			{ 24, 17, 24 },		// 243 bits per transform
			{ 24, 24, 17 },		// 243 bits per transform
			{ 18, 24, 24 },		// 246 bits per transform
			{ 24, 18, 24 },		// 246 bits per transform
			{ 24, 24, 18 },		// 246 bits per transform
			{ 19, 24, 24 },		// 249 bits per transform
			{ 24, 19, 24 },		// 249 bits per transform
			{ 24, 24, 19 },		// 249 bits per transform
			{ 20, 24, 24 },		// 252 bits per transform
			{ 24, 20, 24 },		// 252 bits per transform
			{ 24, 24, 20 },		// 252 bits per transform
			{ 21, 24, 24 },		// 255 bits per transform
			{ 24, 21, 24 },		// 255 bits per transform
			{ 24, 24, 21 },		// 255 bits per transform
			{ 22, 24, 24 },		// 258 bits per transform
			{ 24, 22, 24 },		// 258 bits per transform
			{ 24, 24, 22 },		// 258 bits per transform
			{ 23, 24, 24 },		// 261 bits per transform
			{ 24, 23, 24 },		// 261 bits per transform
			{ 24, 24, 23 },		// 261 bits per transform
			{ 24, 24, 24 },		// 288 bits per transform
		};
	}

	ACL_IMPL_VERSION_NAMESPACE_END
}

ACL_IMPL_FILE_PRAGMA_POP
